/**
 * The `IssuesManager` is the central storage for issues. It collects issues from all the
 * `IssuesModel` instances in the page, and deduplicates them wrt their primary key.
 * It also takes care of clearing the issues when it sees a main-frame navigated event.
 * Any client can subscribe to the events provided, and/or query the issues via the public
 * interface.
 *
 * Additionally, the `IssuesManager` can filter Issues. All Issues are stored, but only
 * Issues that are accepted by the filter cause events to be fired or are returned by
 * `IssuesManager#issues()`.
 *
 * @implements {SDK.SDKModel.SDKModelObserver<!SDK.IssuesModel.IssuesModel>}
 */
export class IssuesManager extends Common.ObjectWrapper.ObjectWrapper implements SDK.SDKModel.SDKModelObserver<SDK.IssuesModel.IssuesModel> {
    /**
     * @param {{forceNew: boolean}} opts
     * @return {!IssuesManager}
     */
    static instance({ forceNew }?: {
        forceNew: boolean;
    }): IssuesManager;
    /** @type {!WeakMap<!SDK.IssuesModel.IssuesModel, !Common.EventTarget.EventDescriptor>} */
    _eventListeners: WeakMap<SDK.IssuesModel.IssuesModel, Common.EventTarget.EventDescriptor>;
    /** @type {!Map<string, !SDK.Issue.Issue>} */
    _issues: Map<string, SDK.Issue.Issue>;
    /** @type {!Map<string, !SDK.Issue.Issue>} */
    _filteredIssues: Map<string, SDK.Issue.Issue>;
    _hasSeenTopFrameNavigated: boolean;
    /** @type {?Common.EventTarget.EventDescriptor} */
    _showThirdPartySettingsChangeListener: Common.EventTarget.EventDescriptor | null;
    /**
     * Once we have seen at least one `TopFrameNavigated` event, we can be reasonably sure
     * that we also collected issues that were reported during the navigation to the current
     * page. If we haven't seen a main frame navigated, we might have missed issues that arose
     * during navigation.
     *
     * @return {boolean}
     */
    reloadForAccurateInformationRequired(): boolean;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onTopFrameNavigated(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onFrameAddedToTarget(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @override
     * @param {!SDK.IssuesModel.IssuesModel} issuesModel
     */
    modelAdded(issuesModel: SDK.IssuesModel.IssuesModel): void;
    /**
     * @override
     * @param {!SDK.IssuesModel.IssuesModel} issuesModel
     */
    modelRemoved(issuesModel: SDK.IssuesModel.IssuesModel): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _issueAdded(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @return {!Iterable<!SDK.Issue.Issue>}
     */
    issues(): Iterable<SDK.Issue.Issue>;
    /**
     * @return {number}
     */
    numberOfIssues(): number;
    /**
     * @return {number}
     */
    numberOfAllStoredIssues(): number;
    /**
     * @param {!SDK.Issue.Issue} issue
     * @return {boolean}
     */
    _issueFilter(issue: SDK.Issue.Issue): boolean;
    _updateFilteredIssues(): void;
}
export type Events = symbol;
export namespace Events {
    const IssuesCountUpdated: symbol;
    const IssueAdded: symbol;
    const FullUpdateRequired: symbol;
}
import * as SDK from "../sdk/sdk.js";
import * as Common from "../common/common.js";
