import { ObjectWrapper, EventTarget, Revealer } from '../common/common.js';
import { SDKModel, IssuesModel, FrameManager, Issue, LogModel, ConsoleModel, RuntimeModel, NetworkLog, NetworkRequest, Cookie } from '../sdk/sdk.js';

// Copyright 2020 The Chromium Authors. All rights reserved.
/** @type {?IssuesManager} */
let issuesManagerInstance = null;
/**
 * The `IssuesManager` is the central storage for issues. It collects issues from all the
 * `IssuesModel` instances in the page, and deduplicates them wrt their primary key.
 * It also takes care of clearing the issues when it sees a main-frame navigated event.
 * Any client can subscribe to the events provided, and/or query the issues via the public
 * interface.
 *
 * Additionally, the `IssuesManager` can filter Issues. All Issues are stored, but only
 * Issues that are accepted by the filter cause events to be fired or are returned by
 * `IssuesManager#issues()`.
 *
 * @implements {SDK.SDKModel.SDKModelObserver<!SDK.IssuesModel.IssuesModel>}
 */
class IssuesManager extends ObjectWrapper.ObjectWrapper {
    constructor() {
        super();
        /** @type {!WeakMap<!SDK.IssuesModel.IssuesModel, !Common.EventTarget.EventDescriptor>} */
        this._eventListeners = new WeakMap();
        SDKModel.TargetManager.instance().observeModels(IssuesModel.IssuesModel, this);
        /** @type {!Map<string, !SDK.Issue.Issue>} */
        this._issues = new Map();
        /** @type {!Map<string, !SDK.Issue.Issue>} */
        this._filteredIssues = new Map();
        this._hasSeenTopFrameNavigated = false;
        FrameManager.FrameManager.instance().addEventListener(FrameManager.Events.TopFrameNavigated, this._onTopFrameNavigated, this);
        FrameManager.FrameManager.instance().addEventListener(FrameManager.Events.FrameAddedToTarget, this._onFrameAddedToTarget, this);
        /** @type {?Common.EventTarget.EventDescriptor} */
        this._showThirdPartySettingsChangeListener = null;
    }
    /**
     * @param {{forceNew: boolean}} opts
     * @return {!IssuesManager}
     */
    static instance({ forceNew } = { forceNew: false }) {
        if (!issuesManagerInstance || forceNew) {
            issuesManagerInstance = new IssuesManager();
        }
        return issuesManagerInstance;
    }
    /**
     * Once we have seen at least one `TopFrameNavigated` event, we can be reasonably sure
     * that we also collected issues that were reported during the navigation to the current
     * page. If we haven't seen a main frame navigated, we might have missed issues that arose
     * during navigation.
     *
     * @return {boolean}
     */
    reloadForAccurateInformationRequired() {
        return !this._hasSeenTopFrameNavigated;
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onTopFrameNavigated(event) {
        const { frame } = /** @type {!{frame:!SDK.ResourceTreeModel.ResourceTreeFrame}} */ (event.data);
        const keptIssues = new Map();
        for (const [key, issue] of this._issues.entries()) {
            if (issue.isAssociatedWithRequestId(frame.loaderId)) {
                keptIssues.set(key, issue);
            }
        }
        this._issues = keptIssues;
        this._hasSeenTopFrameNavigated = true;
        this._updateFilteredIssues();
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onFrameAddedToTarget(event) {
        const { frame } = /** @type {!{frame:!SDK.ResourceTreeModel.ResourceTreeFrame}} */ (event.data);
        // Determining third-party status usually requires the registered domain of the top frame.
        // When DevTools is opened after navigation has completed, issues may be received
        // before the top frame is available. Thus, we trigger a recalcuation of third-party-ness
        // when we attach to the top frame.
        if (frame.isTopFrame()) {
            this._updateFilteredIssues();
        }
    }
    /**
     * @override
     * @param {!SDK.IssuesModel.IssuesModel} issuesModel
     */
    modelAdded(issuesModel) {
        const listener = issuesModel.addEventListener(IssuesModel.Events.IssueAdded, this._issueAdded, this);
        this._eventListeners.set(issuesModel, listener);
    }
    /**
     * @override
     * @param {!SDK.IssuesModel.IssuesModel} issuesModel
     */
    modelRemoved(issuesModel) {
        const listener = this._eventListeners.get(issuesModel);
        if (listener) {
            EventTarget.EventTarget.removeEventListeners([listener]);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _issueAdded(event) {
        const { issuesModel, issue } = 
        /** @type {{issuesModel: !SDK.IssuesModel.IssuesModel, issue: !SDK.Issue.Issue}} */ (event.data);
        // Ignore issues without proper description; they are invisible to the user and only cause confusion.
        if (!issue.getDescription()) {
            return;
        }
        const primaryKey = issue.primaryKey();
        if (this._issues.has(primaryKey)) {
            return;
        }
        this._issues.set(primaryKey, issue);
        if (this._issueFilter(issue)) {
            this._filteredIssues.set(primaryKey, issue);
            this.dispatchEventToListeners(Events.IssueAdded, { issuesModel, issue });
        }
        // Always fire the "count" event even if the issue was filtered out.
        // The result of `hasOnlyThirdPartyIssues` could still change.
        this.dispatchEventToListeners(Events.IssuesCountUpdated);
    }
    /**
     * @return {!Iterable<!SDK.Issue.Issue>}
     */
    issues() {
        return this._filteredIssues.values();
    }
    /**
     * @return {number}
     */
    numberOfIssues() {
        return this._filteredIssues.size;
    }
    /**
     * @return {number}
     */
    numberOfAllStoredIssues() {
        return this._issues.size;
    }
    /**
     * @param {!SDK.Issue.Issue} issue
     * @return {boolean}
     */
    _issueFilter(issue) {
        if (!this._showThirdPartySettingsChangeListener) {
            // _issueFilter uses the 'showThirdPartyIssues' setting. Clients of IssuesManager need
            // a full update when the setting changes to get an up-to-date issues list.
            //
            // The settings change listener can't be set up in IssuesManager's constructor. At that
            // time, the settings storage is not initialized yet, so the setting can't be created.
            const showThirdPartyIssuesSetting = Issue.getShowThirdPartyIssuesSetting();
            this._showThirdPartySettingsChangeListener = showThirdPartyIssuesSetting.addChangeListener(() => {
                this._updateFilteredIssues();
            });
        }
        const showThirdPartyIssuesSetting = Issue.getShowThirdPartyIssuesSetting();
        return showThirdPartyIssuesSetting.get() || !issue.isCausedByThirdParty();
    }
    _updateFilteredIssues() {
        this._filteredIssues.clear();
        // TODO(crbug.com/1011811): Replace with for .. of loop once Closure is gone.
        this._issues.forEach((issue, key) => {
            if (this._issueFilter(issue)) {
                this._filteredIssues.set(key, issue);
            }
        });
        this.dispatchEventToListeners(Events.FullUpdateRequired);
        this.dispatchEventToListeners(Events.IssuesCountUpdated);
    }
}
/** @enum {symbol} */
const Events = {
    IssuesCountUpdated: Symbol('IssuesCountUpdated'),
    IssueAdded: Symbol('IssueAdded'),
    FullUpdateRequired: Symbol('FullUpdateRequired'),
};

var IssuesManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IssuesManager: IssuesManager,
    Events: Events
});

// Copyright 2018 The Chromium Authors. All rights reserved.
/** @type {!WeakMap<!SDK.LogModel.LogModel, !Array<!Common.EventTarget.EventDescriptor>>} */
const modelToEventListeners = new WeakMap();
/**
 * @implements {SDK.SDKModel.SDKModelObserver<!SDK.LogModel.LogModel>}
 */
class LogManager {
    constructor() {
        SDKModel.TargetManager.instance().observeModels(LogModel.LogModel, this);
    }
    /**
     * @override
     * @param {!SDK.LogModel.LogModel} logModel
     */
    modelAdded(logModel) {
        const eventListeners = [];
        eventListeners.push(logModel.addEventListener(LogModel.Events.EntryAdded, this._logEntryAdded, this));
        modelToEventListeners.set(logModel, eventListeners);
    }
    /**
     * @override
     * @param {!SDK.LogModel.LogModel} logModel
     */
    modelRemoved(logModel) {
        const eventListeners = modelToEventListeners.get(logModel);
        if (eventListeners) {
            EventTarget.EventTarget.removeEventListeners(eventListeners);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _logEntryAdded(event) {
        const data = /** @type {{logModel: !SDK.LogModel.LogModel, entry: !Protocol.Log.LogEntry}} */ (event.data);
        const target = data.logModel.target();
        const consoleMessage = new ConsoleModel.ConsoleMessage(target.model(RuntimeModel.RuntimeModel), data.entry.source, data.entry.level, data.entry.text, undefined, data.entry.url, data.entry.lineNumber, undefined, [data.entry.text, ...(data.entry.args || [])], data.entry.stackTrace, data.entry.timestamp, undefined, undefined, data.entry.workerId);
        if (data.entry.networkRequestId) {
            NetworkLog.NetworkLog.instance().associateConsoleMessageWithRequest(consoleMessage, data.entry.networkRequestId);
        }
        if (consoleMessage.source === ConsoleModel.MessageSource.Worker) {
            const workerId = consoleMessage.workerId || '';
            // We have a copy of worker messages reported through the page, so that
            // user can see messages from the worker which has been already destroyed.
            // When opening DevTools, give us some time to connect to the worker and
            // not report the message twice if the worker is still alive.
            if (SDKModel.TargetManager.instance().targetById(workerId)) {
                return;
            }
            setTimeout(() => {
                if (!SDKModel.TargetManager.instance().targetById(workerId)) {
                    ConsoleModel.ConsoleModel.instance().addMessage(consoleMessage);
                }
            }, 1000);
        }
        else {
            ConsoleModel.ConsoleModel.instance().addMessage(consoleMessage);
        }
    }
}

var LogManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    LogManager: LogManager
});

// Copyright 2020 The Chromium Authors. All rights reserved.
/**
 * @typedef {!SDK.NetworkRequest.NetworkRequest | !SDK.Cookie.Cookie}
 */
// @ts-ignore typedef
let IssuesAssociatable;
/**
 * @param {!Array<!SDK.Issue.Issue>} issues
 * @param {!SDK.NetworkRequest.NetworkRequest} request
 * @return {!Array<!SDK.Issue.Issue>}
 */
function issuesAssociatedWithNetworkRequest(issues, request) {
    return issues.filter(issue => {
        for (const affectedRequest of issue.requests()) {
            if (affectedRequest.requestId === request.requestId()) {
                return true;
            }
        }
        return false;
    });
}
/**
 * @param {!Array<!SDK.Issue.Issue>} issues
 * @param {string} domain
 * @param {string} name
 * @param {string} path
 * @return {!Array<!SDK.Issue.Issue>}
 */
function issuesAssociatedWithCookie(issues, domain, name, path) {
    return issues.filter(issue => {
        for (const cookie of issue.cookies()) {
            if (cookie.domain === domain && cookie.name === name && cookie.path === path) {
                return true;
            }
        }
        return false;
    });
}
/**
 * @param {!Array<!SDK.Issue.Issue>} issues
 * @param {!IssuesAssociatable} obj
 * @return {!Array<!SDK.Issue.Issue>}
 * @throws In case obj has an unsupported type (i.e. not part of the IssuesAssociatble union).
 */
function issuesAssociatedWith(issues, obj) {
    if (obj instanceof NetworkRequest.NetworkRequest) {
        return issuesAssociatedWithNetworkRequest(issues, obj);
    }
    if (obj instanceof Cookie.Cookie) {
        return issuesAssociatedWithCookie(issues, obj.domain(), obj.name(), obj.path());
    }
    throw new Error(`issues can not be associated with ${JSON.stringify(obj)}`);
}
/**
 * @param {!IssuesAssociatable} obj
 * @return {boolean}
 */
function hasIssues(obj) {
    const issues = Array.from(IssuesManager.instance().issues());
    return issuesAssociatedWith(issues, obj).length > 0;
}
/**
 * @param {!IssuesAssociatable} obj
 * @param {!SDK.Issue.IssueCategory} category
 * @return {boolean}
 */
function hasIssueOfCategory(obj, category) {
    const issues = Array.from(IssuesManager.instance().issues());
    return issuesAssociatedWith(issues, obj).some(issue => issue.getCategory() === category);
}
/**
 * @param {!IssuesAssociatable} obj
 * @param {!SDK.Issue.IssueCategory=} category
 * @return {!Promise<undefined | void>}
 */
async function reveal(obj, category) {
    const issues = Array.from(IssuesManager.instance().issues());
    const candidates = issuesAssociatedWith(issues, obj).filter(issue => !category || issue.getCategory() === category);
    if (candidates.length > 0) {
        return Revealer.reveal(candidates[0]);
    }
}

var RelatedIssue = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IssuesAssociatable: IssuesAssociatable,
    issuesAssociatedWith: issuesAssociatedWith,
    hasIssues: hasIssues,
    hasIssueOfCategory: hasIssueOfCategory,
    reveal: reveal
});

// Copyright 2019 The Chromium Authors. All rights reserved.
const logManager = new LogManager();
// We need to force creation of the IssueManager early to make sure no issues are missed.
IssuesManager.instance();

export { IssuesManager$1 as IssuesManager, LogManager$1 as LogManager, RelatedIssue, logManager };
