// Copyright (c) 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 * @param {string} inputString
 * @param {string} charsToEscape
 * @return {string} the string with any matching chars escaped
 */
const escapeCharacters = (inputString, charsToEscape) => {
    let foundChar = false;
    for (let i = 0; i < charsToEscape.length; ++i) {
        if (inputString.indexOf(charsToEscape.charAt(i)) !== -1) {
            foundChar = true;
            break;
        }
    }
    if (!foundChar) {
        return String(inputString);
    }
    let result = '';
    for (let i = 0; i < inputString.length; ++i) {
        if (charsToEscape.indexOf(inputString.charAt(i)) !== -1) {
            result += '\\';
        }
        result += inputString.charAt(i);
    }
    return result;
};
/**
 * @enum {string}
 */
const FORMATTER_TYPES = {
    STRING: 'string',
    SPECIFIER: 'specifier',
};
/** @typedef {{type: !FORMATTER_TYPES, value: (string|{description: string}|undefined), specifier: (string|undefined), precision: (number|undefined), substitutionIndex: (number|undefined)}} */
// @ts-ignore typedef
let FORMATTER_TOKEN;
/**
 * @param {string} formatString
 * @param {!Object.<string, function(string, ...*):*>} formatters
 * @return {!Array.<!FORMATTER_TOKEN>}
 */
const tokenizeFormatString = function (formatString, formatters) {
    /** @type {!Array<!FORMATTER_TOKEN>} */
    const tokens = [];
    /**
     * @param {string} str
     */
    function addStringToken(str) {
        if (!str) {
            return;
        }
        if (tokens.length && tokens[tokens.length - 1].type === FORMATTER_TYPES.STRING) {
            tokens[tokens.length - 1].value += str;
        }
        else {
            tokens.push({
                type: FORMATTER_TYPES.STRING,
                value: str,
                specifier: undefined,
                precision: undefined,
                substitutionIndex: undefined
            });
        }
    }
    /**
     * @param {string} specifier
     * @param {number} precision
     * @param {number} substitutionIndex
     */
    function addSpecifierToken(specifier, precision, substitutionIndex) {
        tokens.push({ type: FORMATTER_TYPES.SPECIFIER, specifier, precision, substitutionIndex, value: undefined });
    }
    /**
     * @param {number} code
     */
    function addAnsiColor(code) {
        /**
         * @type {!Object<number, string>}
         */
        const types = { 3: 'color', 9: 'colorLight', 4: 'bgColor', 10: 'bgColorLight' };
        const colorCodes = ['black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'lightGray', '', 'default'];
        const colorCodesLight = ['darkGray', 'lightRed', 'lightGreen', 'lightYellow', 'lightBlue', 'lightMagenta', 'lightCyan', 'white', ''];
        /** @type {!Object<string, !Array<string>>} */
        const colors = { color: colorCodes, colorLight: colorCodesLight, bgColor: colorCodes, bgColorLight: colorCodesLight };
        const type = types[Math.floor(code / 10)];
        if (!type) {
            return;
        }
        const color = colors[type][code % 10];
        if (!color) {
            return;
        }
        tokens.push({
            type: FORMATTER_TYPES.SPECIFIER,
            specifier: 'c',
            value: { description: (type.startsWith('bg') ? 'background : ' : 'color: ') + color },
            precision: undefined,
            substitutionIndex: undefined,
        });
    }
    let textStart = 0;
    let substitutionIndex = 0;
    const re = new RegExp(`%%|%(?:(\\d+)\\$)?(?:\\.(\\d*))?([${Object.keys(formatters).join('')}])|\\u001b\\[(\\d+)m`, 'g');
    for (let match = re.exec(formatString); !!match; match = re.exec(formatString)) {
        const matchStart = match.index;
        if (matchStart > textStart) {
            addStringToken(formatString.substring(textStart, matchStart));
        }
        if (match[0] === '%%') {
            addStringToken('%');
        }
        else if (match[0].startsWith('%')) {
            // eslint-disable-next-line no-unused-vars
            const [_, substitionString, precisionString, specifierString] = match;
            if (substitionString && Number(substitionString) > 0) {
                substitutionIndex = Number(substitionString) - 1;
            }
            const precision = precisionString ? Number(precisionString) : -1;
            addSpecifierToken(specifierString, precision, substitutionIndex);
            ++substitutionIndex;
        }
        else {
            const code = Number(match[4]);
            addAnsiColor(code);
        }
        textStart = matchStart + match[0].length;
    }
    addStringToken(formatString.substring(textStart));
    return tokens;
};
/**
 * @param {string} formatString
 * @param {?ArrayLike<*>} substitutions
 * @param {!Object.<string, function(string, ...*):*>} formatters
 * @param {!T} initialValue
 * @param {function(T, *): T} append
 * @param {!Array.<!FORMATTER_TOKEN>=} tokenizedFormat
 * @return {!{formattedResult: T, unusedSubstitutions: ?ArrayLike<*>}};
 * @template T
 */
const format = function (formatString, substitutions, formatters, initialValue, append, tokenizedFormat) {
    if (!formatString || ((!substitutions || !substitutions.length) && formatString.search(/\u001b\[(\d+)m/) === -1)) {
        return { formattedResult: append(initialValue, formatString), unusedSubstitutions: substitutions };
    }
    function prettyFunctionName() {
        return 'String.format("' + formatString + '", "' + Array.prototype.join.call(substitutions, '", "') + '")';
    }
    /**
     * @param {string} msg
     */
    function warn(msg) {
        console.warn(prettyFunctionName() + ': ' + msg);
    }
    /**
     * @param {string} msg
     */
    function error(msg) {
        console.error(prettyFunctionName() + ': ' + msg);
    }
    let result = initialValue;
    const tokens = tokenizedFormat || tokenizeFormatString(formatString, formatters);
    /** @type {!Object<number, boolean>} */
    const usedSubstitutionIndexes = {};
    /** @type {!ArrayLike<*>} */
    const actualSubstitutions = substitutions || [];
    for (let i = 0; i < tokens.length; ++i) {
        const token = tokens[i];
        if (token.type === FORMATTER_TYPES.STRING) {
            result = append(result, token.value);
            continue;
        }
        if (token.type !== FORMATTER_TYPES.SPECIFIER) {
            error('Unknown token type "' + token.type + '" found.');
            continue;
        }
        if (!token.value && token.substitutionIndex !== undefined &&
            token.substitutionIndex >= actualSubstitutions.length) {
            // If there are not enough substitutions for the current substitutionIndex
            // just output the format specifier literally and move on.
            error('not enough substitution arguments. Had ' + actualSubstitutions.length + ' but needed ' +
                (token.substitutionIndex + 1) + ', so substitution was skipped.');
            result = append(result, '%' + ((token.precision !== undefined && token.precision > -1) ? token.precision : '') + token.specifier);
            continue;
        }
        if (!token.value && token.substitutionIndex !== undefined) {
            usedSubstitutionIndexes[token.substitutionIndex] = true;
        }
        if (token.specifier === undefined || !(token.specifier in formatters)) {
            // Encountered an unsupported format character, treat as a string.
            warn('unsupported format character \u201C' + token.specifier + '\u201D. Treating as a string.');
            result = append(result, (token.value || token.substitutionIndex === undefined) ? '' : actualSubstitutions[token.substitutionIndex]);
            continue;
        }
        result = append(result, formatters[token.specifier](token.value || (token.substitutionIndex !== undefined && actualSubstitutions[token.substitutionIndex]), token));
    }
    const unusedSubstitutions = [];
    for (let i = 0; i < actualSubstitutions.length; ++i) {
        if (i in usedSubstitutionIndexes) {
            continue;
        }
        unusedSubstitutions.push(actualSubstitutions[i]);
    }
    return { formattedResult: result, unusedSubstitutions: unusedSubstitutions };
};
const standardFormatters = {
    /**
     * @param {*} substitution
     * @return {number}
     */
    d: function (substitution) {
        return /** @type {number} */ (!isNaN(substitution) ? substitution : 0);
    },
    /**
     * @param {*} substitution
     * @param {!FORMATTER_TOKEN} token
     * @return {number}
     */
    f: function (substitution, token) {
        if (substitution && token.precision !== undefined && token.precision > -1) {
            substitution = substitution.toFixed(token.precision);
        }
        const precision = (token.precision !== undefined && token.precision > -1) ? Number(0).toFixed(token.precision) : 0;
        return /** @type number} */ (!isNaN(substitution) ? substitution : precision);
    },
    /**
     * @param {*} substitution
     * @return {string}
     */
    s: function (substitution) {
        return /** @type {string} */ (substitution);
    }
};
/**
 * @param {string} formatString
 * @param {!Array.<*>} substitutions
 * @return {string}
 */
const vsprintf = function (formatString, substitutions) {
    // @ts-ignore
    return format(formatString, substitutions, standardFormatters, '', function (a, b) {
        return a + b;
    }).formattedResult;
};
/**
 * @param {string} format
 * @param {...*} var_arg
 * @return {string}
 */
const sprintf = function (format, var_arg) {
    return vsprintf(format, Array.prototype.slice.call(arguments, 1));
};
/**
* @param {string} inputString
* @return {string}
*/
const toBase64 = inputString => {
    /* note to the reader: we can't use btoa here because we need to
     * support Unicode correctly. See the test cases for this function and
     * also
     * https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding#The_Unicode_Problem
     */
    /**
     * @param {number} b
     * @return {number}
     */
    function encodeBits(b) {
        return b < 26 ? b + 65 : b < 52 ? b + 71 : b < 62 ? b - 4 : b === 62 ? 43 : b === 63 ? 47 : 65;
    }
    const encoder = new TextEncoder();
    const data = encoder.encode(inputString.toString());
    const n = data.length;
    let encoded = '';
    if (n === 0) {
        return encoded;
    }
    let shift;
    let v = 0;
    for (let i = 0; i < n; i++) {
        shift = i % 3;
        v |= data[i] << (16 >>> shift & 24);
        if (shift === 2) {
            encoded += String.fromCharCode(encodeBits(v >>> 18 & 63), encodeBits(v >>> 12 & 63), encodeBits(v >>> 6 & 63), encodeBits(v & 63));
            v = 0;
        }
    }
    if (shift === 0) {
        encoded += String.fromCharCode(encodeBits(v >>> 18 & 63), encodeBits(v >>> 12 & 63), 61, 61);
    }
    else if (shift === 1) {
        encoded += String.fromCharCode(encodeBits(v >>> 18 & 63), encodeBits(v >>> 12 & 63), encodeBits(v >>> 6 & 63), 61);
    }
    return encoded;
};
/**
 *
 * @param {string} inputString
 * @param {string} searchString
 * @return {!Array.<number>}
 */
const findIndexesOfSubString = (inputString, searchString) => {
    const matches = [];
    let i = inputString.indexOf(searchString);
    while (i !== -1) {
        matches.push(i);
        i = inputString.indexOf(searchString, i + searchString.length);
    }
    return matches;
};
/**
 *
 * @param {string} inputString
 * @return {!Array.<number>}
 */
const findLineEndingIndexes = inputString => {
    const endings = findIndexesOfSubString(inputString, '\n');
    endings.push(inputString.length);
    return endings;
};
/**
 * @param {string} inputString
 * @return {boolean}
 */
const isWhitespace = inputString => {
    return /^\s*$/.test(inputString);
};
/**
 * @param {string} url
 * @param {?string=} baseURLDomain
 * @return {string}
 */
const trimURL = (url, baseURLDomain) => {
    let result = url.replace(/^(https|http|file):\/\//i, '');
    if (baseURLDomain) {
        if (result.toLowerCase().startsWith(baseURLDomain.toLowerCase())) {
            result = result.substr(baseURLDomain.length);
        }
    }
    return result;
};
/**
 * @param {string} inputString
 * @return {string}
 */
const collapseWhitespace = inputString => {
    return inputString.replace(/[\s\xA0]+/g, ' ');
};
/**
 *
 * @param {string} inputString
 * @return {string}
 */
const reverse = inputString => {
    return inputString.split('').reverse().join('');
};
/**
 * @param {string} inputString
 * @return {string}
 */
const replaceControlCharacters = inputString => {
    // Replace C0 and C1 control character sets with replacement character.
    // Do not replace '\t', \n' and '\r'.
    return inputString.replace(/[\0-\x08\x0B\f\x0E-\x1F\x80-\x9F]/g, '\uFFFD');
};
/**
 * @param {string} inputString
 * @return {number}
 */
const countWtf8Bytes = inputString => {
    let count = 0;
    for (let i = 0; i < inputString.length; i++) {
        const c = inputString.charCodeAt(i);
        if (c <= 0x7F) {
            count++;
        }
        else if (c <= 0x07FF) {
            count += 2;
        }
        else if (c < 0xD800 || 0xDFFF < c) {
            count += 3;
        }
        else {
            if (c <= 0xDBFF && i + 1 < inputString.length) {
                // The current character is a leading surrogate, and there is a
                // next character.
                const next = inputString.charCodeAt(i + 1);
                if (0xDC00 <= next && next <= 0xDFFF) {
                    // The next character is a trailing surrogate, meaning this
                    // is a surrogate pair.
                    count += 4;
                    i++;
                    continue;
                }
            }
            count += 3;
        }
    }
    return count;
};
/**
 * @param {string} inputStr
 */
const stripLineBreaks = inputStr => {
    return inputStr.replace(/(\r)?\n/g, '');
};
/**
 * @param {string} inputStr
 * @return {string}
 */
const toTitleCase = inputStr => {
    return inputStr.substring(0, 1).toUpperCase() + inputStr.substring(1);
};
/**
 * @param {string} inputStr
 * @return {string}
 */
const removeURLFragment = inputStr => {
    const url = new URL(inputStr);
    url.hash = '';
    return url.toString();
};
/**
 * @return {string}
 */
const regexSpecialCharacters = function () {
    return '^[]{}()\\.^$*+?|-,';
};
/**
 * @param {string} query
 * @return {!RegExp}
 */
const filterRegex = function (query) {
    const toEscape = regexSpecialCharacters();
    let regexString = '';
    for (let i = 0; i < query.length; ++i) {
        let c = query.charAt(i);
        if (toEscape.indexOf(c) !== -1) {
            c = '\\' + c;
        }
        if (i) {
            regexString += '[^\\0' + c + ']*';
        }
        regexString += c;
    }
    return new RegExp(regexString, 'i');
};
/**
 * @param {string} query
 * @param {boolean} caseSensitive
 * @param {boolean} isRegex
 * @return {!RegExp}
 */
const createSearchRegex = function (query, caseSensitive, isRegex) {
    const regexFlags = caseSensitive ? 'g' : 'gi';
    let regexObject;
    if (isRegex) {
        try {
            regexObject = new RegExp(query, regexFlags);
        }
        catch (e) {
            // Silent catch.
        }
    }
    if (!regexObject) {
        regexObject = self.createPlainTextSearchRegex(query, regexFlags);
    }
    return regexObject;
};

var stringUtilities = /*#__PURE__*/Object.freeze({
    __proto__: null,
    escapeCharacters: escapeCharacters,
    FORMATTER_TOKEN: FORMATTER_TOKEN,
    tokenizeFormatString: tokenizeFormatString,
    format: format,
    standardFormatters: standardFormatters,
    vsprintf: vsprintf,
    sprintf: sprintf,
    toBase64: toBase64,
    findIndexesOfSubString: findIndexesOfSubString,
    findLineEndingIndexes: findLineEndingIndexes,
    isWhitespace: isWhitespace,
    trimURL: trimURL,
    collapseWhitespace: collapseWhitespace,
    reverse: reverse,
    replaceControlCharacters: replaceControlCharacters,
    countWtf8Bytes: countWtf8Bytes,
    stripLineBreaks: stripLineBreaks,
    toTitleCase: toTitleCase,
    removeURLFragment: removeURLFragment,
    regexSpecialCharacters: regexSpecialCharacters,
    filterRegex: filterRegex,
    createSearchRegex: createSearchRegex
});

/*
 * Copyright (C) 2007 Apple Inc.  All rights reserved.
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
// Still used in the test runners that can't use ES modules :(
String.sprintf = sprintf;
String.regexSpecialCharacters = regexSpecialCharacters;
/**
 * @this {string}
 * @return {string}
 */
String.prototype.escapeForRegExp = function () {
    return escapeCharacters(this, String.regexSpecialCharacters());
};
/**
 * @param {number} maxLength
 * @return {string}
 */
String.prototype.trimMiddle = function (maxLength) {
    if (this.length <= maxLength) {
        return String(this);
    }
    let leftHalf = maxLength >> 1;
    let rightHalf = maxLength - leftHalf - 1;
    if (this.codePointAt(this.length - rightHalf - 1) >= 0x10000) {
        --rightHalf;
        ++leftHalf;
    }
    if (leftHalf > 0 && this.codePointAt(leftHalf - 1) >= 0x10000) {
        --leftHalf;
    }
    return this.substr(0, leftHalf) + '…' + this.substr(this.length - rightHalf, rightHalf);
};
/**
 * @param {number} maxLength
 * @return {string}
 */
String.prototype.trimEndWithMaxLength = function (maxLength) {
    if (this.length <= maxLength) {
        return String(this);
    }
    return this.substr(0, maxLength - 1) + '…';
};
/**
 * @param {string} other
 * @return {number}
 */
String.prototype.compareTo = function (other) {
    if (this > other) {
        return 1;
    }
    if (this < other) {
        return -1;
    }
    return 0;
};
/**
 * @param {string|undefined} string
 * @return {number}
 */
String.hashCode = function (string) {
    if (!string) {
        return 0;
    }
    // Hash algorithm for substrings is described in "Über die Komplexität der Multiplikation in
    // eingeschränkten Branchingprogrammmodellen" by Woelfe.
    // http://opendatastructures.org/versions/edition-0.1d/ods-java/node33.html#SECTION00832000000000000000
    const p = ((1 << 30) * 4 - 5); // prime: 2^32 - 5
    const z = 0x5033d967; // 32 bits from random.org
    const z2 = 0x59d2f15d; // random odd 32 bit number
    let s = 0;
    let zi = 1;
    for (let i = 0; i < string.length; i++) {
        const xi = string.charCodeAt(i) * z2;
        s = (s + zi * xi) % p;
        zi = (zi * z) % p;
    }
    s = (s + zi * (p - 1)) % p;
    return Math.abs(s | 0);
};
/**
 * @param {string} a
 * @param {string} b
 * @return {number}
 */
String.naturalOrderComparator = function (a, b) {
    const chunk = /^\d+|^\D+/;
    let chunka, chunkb, anum, bnum;
    while (1) {
        if (a) {
            if (!b) {
                return 1;
            }
        }
        else {
            if (b) {
                return -1;
            }
            return 0;
        }
        chunka = a.match(chunk)[0];
        chunkb = b.match(chunk)[0];
        anum = !isNaN(chunka);
        bnum = !isNaN(chunkb);
        if (anum && !bnum) {
            return -1;
        }
        if (bnum && !anum) {
            return 1;
        }
        if (anum && bnum) {
            const diff = chunka - chunkb;
            if (diff) {
                return diff;
            }
            if (chunka.length !== chunkb.length) {
                if (!+chunka && !+chunkb) { // chunks are strings of all 0s (special case)
                    return chunka.length - chunkb.length;
                }
                return chunkb.length - chunka.length;
            }
        }
        else if (chunka !== chunkb) {
            return (chunka < chunkb) ? -1 : 1;
        }
        a = a.substring(chunka.length);
        b = b.substring(chunkb.length);
    }
};
/**
 * @param {string} a
 * @param {string} b
 * @return {number}
 */
String.caseInsensetiveComparator = function (a, b) {
    a = a.toUpperCase();
    b = b.toUpperCase();
    if (a === b) {
        return 0;
    }
    return a > b ? 1 : -1;
};
/**
 * @param {string} value
 * @return {string}
 */
Number.toFixedIfFloating = function (value) {
    if (!value || isNaN(value)) {
        return value;
    }
    const number = Number(value);
    return number % 1 ? number.toFixed(3) : String(number);
};
(function () {
    const partition = {
        /**
           * @this {Array.<number>}
           * @param {function(number, number): number} comparator
           * @param {number} left
           * @param {number} right
           * @param {number} pivotIndex
           */
        value: function (comparator, left, right, pivotIndex) {
            function swap(array, i1, i2) {
                const temp = array[i1];
                array[i1] = array[i2];
                array[i2] = temp;
            }
            const pivotValue = this[pivotIndex];
            swap(this, right, pivotIndex);
            let storeIndex = left;
            for (let i = left; i < right; ++i) {
                if (comparator(this[i], pivotValue) < 0) {
                    swap(this, storeIndex, i);
                    ++storeIndex;
                }
            }
            swap(this, right, storeIndex);
            return storeIndex;
        },
        configurable: true
    };
    Object.defineProperty(Array.prototype, 'partition', partition);
    Object.defineProperty(Uint32Array.prototype, 'partition', partition);
    const sortRange = {
        /**
           * @param {function(number, number): number} comparator
           * @param {number} leftBound
           * @param {number} rightBound
           * @param {number} sortWindowLeft
           * @param {number} sortWindowRight
           * @return {!Array.<number>}
           * @this {Array.<number>}
           */
        value: function (comparator, leftBound, rightBound, sortWindowLeft, sortWindowRight) {
            function quickSortRange(array, comparator, left, right, sortWindowLeft, sortWindowRight) {
                if (right <= left) {
                    return;
                }
                const pivotIndex = Math.floor(Math.random() * (right - left)) + left;
                const pivotNewIndex = array.partition(comparator, left, right, pivotIndex);
                if (sortWindowLeft < pivotNewIndex) {
                    quickSortRange(array, comparator, left, pivotNewIndex - 1, sortWindowLeft, sortWindowRight);
                }
                if (pivotNewIndex < sortWindowRight) {
                    quickSortRange(array, comparator, pivotNewIndex + 1, right, sortWindowLeft, sortWindowRight);
                }
            }
            if (leftBound === 0 && rightBound === (this.length - 1) && sortWindowLeft === 0 && sortWindowRight >= rightBound) {
                this.sort(comparator);
            }
            else {
                quickSortRange(this, comparator, leftBound, rightBound, sortWindowLeft, sortWindowRight);
            }
            return this;
        },
        configurable: true
    };
    Object.defineProperty(Array.prototype, 'sortRange', sortRange);
    Object.defineProperty(Uint32Array.prototype, 'sortRange', sortRange);
})();
Object.defineProperty(Array.prototype, 'lowerBound', {
    /**
     * Return index of the leftmost element that is equal or greater
     * than the specimen object. If there's no such element (i.e. all
     * elements are smaller than the specimen) returns right bound.
     * The function works for sorted array.
     * When specified, |left| (inclusive) and |right| (exclusive) indices
     * define the search window.
     *
     * @param {!T} object
     * @param {function(!T,!S):number=} comparator
     * @param {number=} left
     * @param {number=} right
     * @return {number}
     * @this {Array.<!S>}
     * @template T,S
     */
    value: function (object, comparator, left, right) {
        function defaultComparator(a, b) {
            return a < b ? -1 : (a > b ? 1 : 0);
        }
        comparator = comparator || defaultComparator;
        let l = left || 0;
        let r = right !== undefined ? right : this.length;
        while (l < r) {
            const m = (l + r) >> 1;
            if (comparator(object, this[m]) > 0) {
                l = m + 1;
            }
            else {
                r = m;
            }
        }
        return r;
    },
    configurable: true
});
Object.defineProperty(Array.prototype, 'upperBound', {
    /**
     * Return index of the leftmost element that is greater
     * than the specimen object. If there's no such element (i.e. all
     * elements are smaller or equal to the specimen) returns right bound.
     * The function works for sorted array.
     * When specified, |left| (inclusive) and |right| (exclusive) indices
     * define the search window.
     *
     * @param {!T} object
     * @param {function(!T,!S):number=} comparator
     * @param {number=} left
     * @param {number=} right
     * @return {number}
     * @this {Array.<!S>}
     * @template T,S
     */
    value: function (object, comparator, left, right) {
        function defaultComparator(a, b) {
            return a < b ? -1 : (a > b ? 1 : 0);
        }
        comparator = comparator || defaultComparator;
        let l = left || 0;
        let r = right !== undefined ? right : this.length;
        while (l < r) {
            const m = (l + r) >> 1;
            if (comparator(object, this[m]) >= 0) {
                l = m + 1;
            }
            else {
                r = m;
            }
        }
        return r;
    },
    configurable: true
});
Object.defineProperty(Uint32Array.prototype, 'lowerBound', { value: Array.prototype.lowerBound, configurable: true });
Object.defineProperty(Uint32Array.prototype, 'upperBound', { value: Array.prototype.upperBound, configurable: true });
Object.defineProperty(Int32Array.prototype, 'lowerBound', { value: Array.prototype.lowerBound, configurable: true });
Object.defineProperty(Int32Array.prototype, 'upperBound', { value: Array.prototype.upperBound, configurable: true });
Object.defineProperty(Float64Array.prototype, 'lowerBound', { value: Array.prototype.lowerBound, configurable: true });
Object.defineProperty(Array.prototype, 'binaryIndexOf', {
    /**
     * @param {!T} value
     * @param {function(!T,!S):number} comparator
     * @return {number}
     * @this {Array.<!S>}
     * @template T,S
     */
    value: function (value, comparator) {
        const index = this.lowerBound(value, comparator);
        return index < this.length && comparator(value, this[index]) === 0 ? index : -1;
    },
    configurable: true
});
Object.defineProperty(Array.prototype, 'peekLast', {
    /**
     * @return {!T|undefined}
     * @this {Array.<!T>}
     * @template T
     */
    value: function () {
        return this[this.length - 1];
    },
    configurable: true
});
(function () {
    /**
     * @param {!Array.<T>} array1
     * @param {!Array.<T>} array2
     * @param {function(T,T):number} comparator
     * @param {boolean} mergeNotIntersect
     * @return {!Array.<T>}
     * @template T
     */
    function mergeOrIntersect(array1, array2, comparator, mergeNotIntersect) {
        const result = [];
        let i = 0;
        let j = 0;
        while (i < array1.length && j < array2.length) {
            const compareValue = comparator(array1[i], array2[j]);
            if (mergeNotIntersect || !compareValue) {
                result.push(compareValue <= 0 ? array1[i] : array2[j]);
            }
            if (compareValue <= 0) {
                i++;
            }
            if (compareValue >= 0) {
                j++;
            }
        }
        if (mergeNotIntersect) {
            while (i < array1.length) {
                result.push(array1[i++]);
            }
            while (j < array2.length) {
                result.push(array2[j++]);
            }
        }
        return result;
    }
    Object.defineProperty(Array.prototype, 'intersectOrdered', {
        /**
         * @param {!Array.<T>} array
         * @param {function(T,T):number} comparator
         * @return {!Array.<T>}
         * @this {!Array.<T>}
         * @template T
         */
        value: function (array, comparator) {
            return mergeOrIntersect(this, array, comparator, false);
        },
        configurable: true
    });
    Object.defineProperty(Array.prototype, 'mergeOrdered', {
        /**
         * @param {!Array.<T>} array
         * @param {function(T,T):number} comparator
         * @return {!Array.<T>}
         * @this {!Array.<T>}
         * @template T
         */
        value: function (array, comparator) {
            return mergeOrIntersect(this, array, comparator, true);
        },
        configurable: true
    });
})();
/**
 * @param {string} query
 * @param {string=} flags
 * @return {!RegExp}
 */
self.createPlainTextSearchRegex = function (query, flags) {
    // This should be kept the same as the one in StringUtil.cpp.
    const regexSpecialCharacters = String.regexSpecialCharacters();
    let regex = '';
    for (let i = 0; i < query.length; ++i) {
        const c = query.charAt(i);
        if (regexSpecialCharacters.indexOf(c) !== -1) {
            regex += '\\';
        }
        regex += c;
    }
    return new RegExp(regex, flags || '');
};
/**
 * @return {?T}
 * @template T
 */
Set.prototype.firstValue = function () {
    if (!this.size) {
        return null;
    }
    return this.values().next().value;
};
/**
 * @return {!Platform.Multimap<!KEY, !VALUE>}
 */
Map.prototype.inverse = function () {
    const result = new Platform.Multimap();
    for (const key of this.keys()) {
        const value = this.get(key);
        result.set(value, key);
    }
    return result;
};
/**
 * @template K, V
 */
class Multimap {
    constructor() {
        /** @type {!Map.<K, !Set.<!V>>} */
        this._map = new Map();
    }
    /**
     * @param {K} key
     * @param {V} value
     */
    set(key, value) {
        let set = this._map.get(key);
        if (!set) {
            set = new Set();
            this._map.set(key, set);
        }
        set.add(value);
    }
    /**
     * @param {K} key
     * @return {!Set<!V>}
     */
    get(key) {
        return this._map.get(key) || new Set();
    }
    /**
     * @param {K} key
     * @return {boolean}
     */
    has(key) {
        return this._map.has(key);
    }
    /**
     * @param {K} key
     * @param {V} value
     * @return {boolean}
     */
    hasValue(key, value) {
        const set = this._map.get(key);
        if (!set) {
            return false;
        }
        return set.has(value);
    }
    /**
     * @return {number}
     */
    get size() {
        return this._map.size;
    }
    /**
     * @param {K} key
     * @param {V} value
     * @return {boolean}
     */
    delete(key, value) {
        const values = this.get(key);
        if (!values) {
            return false;
        }
        const result = values.delete(value);
        if (!values.size) {
            this._map.delete(key);
        }
        return result;
    }
    /**
     * @param {K} key
     */
    deleteAll(key) {
        this._map.delete(key);
    }
    /**
     * @return {!Array.<K>}
     */
    keysArray() {
        return [...this._map.keys()];
    }
    /**
     * @return {!Array.<!V>}
     */
    valuesArray() {
        const result = [];
        for (const set of this._map.values()) {
            result.push(...set.values());
        }
        return result;
    }
    clear() {
        this._map.clear();
    }
}
/**
 * @param {*} value
 */
self.suppressUnused = function (value) { };
/**
 * @param {function()} callback
 * @return {number}
 */
self.setImmediate = function (callback) {
    const args = [...arguments].slice(1);
    Promise.resolve().then(() => callback(...args));
    return 0;
};
/**
 * TODO: move into its own module
 * @param {function()} callback
 * @suppressGlobalPropertiesCheck
 */
self.runOnWindowLoad = function (callback) {
    /**
     * @suppressGlobalPropertiesCheck
     */
    function windowLoaded() {
        self.removeEventListener('DOMContentLoaded', windowLoaded, false);
        callback();
    }
    if (document.readyState === 'complete' || document.readyState === 'interactive') {
        callback();
    }
    else {
        self.addEventListener('DOMContentLoaded', windowLoaded, false);
    }
};
const _singletonSymbol = Symbol('singleton');
/**
 * @template T
 * @param {function(new:T, ...)} constructorFunction
 * @return {!T}
 */
self.singleton = function (constructorFunction) {
    if (_singletonSymbol in constructorFunction) {
        return constructorFunction[_singletonSymbol];
    }
    const instance = new constructorFunction();
    constructorFunction[_singletonSymbol] = instance;
    return instance;
};
/**
 * @param {?string} content
 * @return {number}
 */
self.base64ToSize = function (content) {
    if (!content) {
        return 0;
    }
    let size = content.length * 3 / 4;
    if (content[content.length - 1] === '=') {
        size--;
    }
    if (content.length > 1 && content[content.length - 2] === '=') {
        size--;
    }
    return size;
};
/**
 * @param {?string} input
 * @return {string}
 */
self.unescapeCssString = function (input) {
    // https://drafts.csswg.org/css-syntax/#consume-escaped-code-point
    const reCssEscapeSequence = /(?<!\\)\\(?:([a-fA-F0-9]{1,6})|(.))[\n\t\x20]?/gs;
    return input.replace(reCssEscapeSequence, (_, $1, $2) => {
        if ($2) { // Handle the single-character escape sequence.
            return $2;
        }
        // Otherwise, handle the code point escape sequence.
        const codePoint = parseInt($1, 16);
        const isSurrogate = 0xD800 <= codePoint && codePoint <= 0xDFFF;
        if (isSurrogate || codePoint === 0x0000 || codePoint > 0x10FFFF) {
            return '\uFFFD';
        }
        return String.fromCodePoint(codePoint);
    });
};
self.Platform = self.Platform || {};
Platform = Platform || {};
/** @constructor */
Platform.Multimap = Multimap;

// Copyright (c) 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 * @param {!Array<T>} array
 * @param {!T} element
 * @param {boolean=} firstOnly Only remove the first occurrence of `element` from the array.
 * @return {boolean} True, if any element got removed.
 * @template T
 */
const removeElement = (array, element, firstOnly) => {
    let index = array.indexOf(element);
    if (index === -1) {
        return false;
    }
    if (firstOnly) {
        array.splice(index, 1);
        return true;
    }
    for (let i = index + 1, n = array.length; i < n; ++i) {
        if (array[i] !== element) {
            array[index++] = array[i];
        }
    }
    array.length = index;
    return true;
};

var arrayUtilities = /*#__PURE__*/Object.freeze({
    __proto__: null,
    removeElement: removeElement
});

// Copyright (c) 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 * @param {!Date} date
 * @return {boolean}
 */
const isValid = date => {
    return !isNaN(date.getTime());
};
/**
 * @param {!Date} date
 * @return {string}
 */
const toISO8601Compact = date => {
    /**
     * @param {number} x
     * @return {string}
     */
    function leadZero(x) {
        return (x > 9 ? '' : '0') + x;
    }
    return date.getFullYear() + leadZero(date.getMonth() + 1) + leadZero(date.getDate()) + 'T' +
        leadZero(date.getHours()) + leadZero(date.getMinutes()) + leadZero(date.getSeconds());
};

var dateUtilities = /*#__PURE__*/Object.freeze({
    __proto__: null,
    isValid: isValid,
    toISO8601Compact: toISO8601Compact
});

/*
 * Copyright (C) 2011 Google Inc.  All rights reserved.
 * Copyright (C) 2006, 2007, 2008 Apple Inc.  All rights reserved.
 * Copyright (C) 2007 Matt Lilek (pewtermoose@gmail.com).
 * Copyright (C) 2009 Joseph Pecoraro
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @param {string} string
 * @param {...*} vararg
 * @return {string}
 */
function UIString(string, ...vararg) {
    return vsprintf(localize(string), Array.prototype.slice.call(arguments, 1));
}
/**
 * @param {string} string
 * @param {?ArrayLike<*>} values
 * @return {string}
 */
function serializeUIString(string, values = []) {
    const messageParts = [string];
    const serializedMessage = { messageParts, values };
    return JSON.stringify(serializedMessage);
}
/**
 * @param {string=} serializedMessage
 * @return {*}
 */
function deserializeUIString(serializedMessage) {
    if (!serializedMessage) {
        return {};
    }
    return JSON.parse(serializedMessage);
}
/**
 * @param {string} string
 * @return {string}
 */
function localize(string) {
    return string;
}
/**
 * @unrestricted
 */
class UIStringFormat {
    /**
     * @param {string} format
     */
    constructor(format) {
        /** @type {string} */
        this._localizedFormat = localize(format);
        /** @type {!Array.<!StringUtilities.FORMATTER_TOKEN>} */
        this._tokenizedFormat =
            tokenizeFormatString(this._localizedFormat, standardFormatters);
    }
    /**
     * @param {string} a
     * @param {*} b
     * @return {string}
     */
    static _append(a, b) {
        return a + b;
    }
    /**
     * @param {...*} vararg
     * @return {string}
     */
    format(vararg) {
        // the code here uses odd generics that Closure likes but TS doesn't
        // so rather than fight to typecheck this in a dodgy way we just let TS ignore it
        // @ts-ignore
        return format(this._localizedFormat, arguments, standardFormatters, '', UIStringFormat._append, this._tokenizedFormat)
            .formattedResult;
    }
}
const _substitutionStrings = new WeakMap();
/**
 * @param {!ITemplateArray|string} strings
 * @param {...*} vararg
 * @return {string}
 */
function ls(strings, ...vararg) {
    if (typeof strings === 'string') {
        return strings;
    }
    let substitutionString = _substitutionStrings.get(strings);
    if (!substitutionString) {
        substitutionString = strings.join('%s');
        _substitutionStrings.set(strings, substitutionString);
    }
    // @ts-ignore TS gets confused with the arguments slicing
    return UIString(substitutionString, ...vararg);
}

var UIString$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    UIString: UIString,
    serializeUIString: serializeUIString,
    deserializeUIString: deserializeUIString,
    localize: localize,
    UIStringFormat: UIStringFormat,
    ls: ls
});

// Copyright (c) 2020 The Chromium Authors. All rights reserved.
/**
 * @param {number} num
 * @param {number} min
 * @param {number} max
 * @return {number}
 */
const clamp = (num, min, max) => {
    let clampedNumber = num;
    if (num < min) {
        clampedNumber = min;
    }
    else if (num > max) {
        clampedNumber = max;
    }
    return clampedNumber;
};
/**
 * @param {number} m
 * @param {number} n
 * @return {number}
 */
const mod = (m, n) => {
    return ((m % n) + n) % n;
};
/**
 * @param {number} bytes
 * @return {string}
 */
const bytesToString = bytes => {
    if (bytes < 1000) {
        return UIString('%.0f\xA0B', bytes);
    }
    const kilobytes = bytes / 1000;
    if (kilobytes < 100) {
        return UIString('%.1f\xA0kB', kilobytes);
    }
    if (kilobytes < 1000) {
        return UIString('%.0f\xA0kB', kilobytes);
    }
    const megabytes = kilobytes / 1000;
    if (megabytes < 100) {
        return UIString('%.1f\xA0MB', megabytes);
    }
    return UIString('%.0f\xA0MB', megabytes);
};

var numberUtilities = /*#__PURE__*/Object.freeze({
    __proto__: null,
    clamp: clamp,
    mod: mod,
    bytesToString: bytesToString
});

// Copyright (c) 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 * @param {!Set<T>} set
 * @param {!Iterable<!T>} iterable
 * @template T
 */
const addAll = function (set, iterable) {
    for (const item of iterable) {
        set.add(item);
    }
};

var setUtilities = /*#__PURE__*/Object.freeze({
    __proto__: null,
    addAll: addAll
});

/*
 * Copyright (C) 2019 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* We expose `ls` directly to make importing + referring to it easy
 * as it's such a commonly referenced and used utility.
 */
const { ls: ls$1 } = UIString$1;

export { arrayUtilities as ArrayUtilities, dateUtilities as DateUtilities, Multimap, numberUtilities as NumberUtilities, setUtilities as SetUtilities, stringUtilities as StringUtilities, UIString$1 as UIString, ls$1 as ls };
