/**
 * The FrameManager is a central storage for all frames. It collects frames from all
 * ResourceTreeModel-instances (one per target), so that frames can be found by id
 * without needing to know their target.
 * @implements {SDKModelObserver<!ResourceTreeModel>}
 */
export class FrameManager extends Common.ObjectWrapper.ObjectWrapper implements SDKModelObserver<ResourceTreeModel> {
    /**
     * @param {{forceNew: boolean}} opts
     * @return {!FrameManager}
     */
    static instance({ forceNew }?: {
        forceNew: boolean;
    }): FrameManager;
    /** @type {!WeakMap<!ResourceTreeModel, !Array<!Common.EventTarget.EventDescriptor>>} */
    _eventListeners: WeakMap<ResourceTreeModel, Array<Common.EventTarget.EventDescriptor>>;
    /** @type {!Map<string, {frame: !ResourceTreeFrame, count: number}>} */
    _frames: Map<string, {
        frame: ResourceTreeFrame;
        count: number;
    }>;
    /** @type {!Map<string, !Set<string>>} */
    _framesForTarget: Map<string, Set<string>>;
    /** @type {?ResourceTreeFrame} */
    _topFrame: ResourceTreeFrame | null;
    /**
     * @override
     * @param {!ResourceTreeModel} resourceTreeModel
     */
    modelAdded(resourceTreeModel: ResourceTreeModel): void;
    /**
     * @override
     * @param {!ResourceTreeModel} resourceTreeModel
     */
    modelRemoved(resourceTreeModel: ResourceTreeModel): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameAdded(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameDetached(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameNavigated(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _resourceAdded(event: Common.EventTarget.EventTargetEvent): void;
    /**
     * @param {string} frameId
     */
    _decreaseOrRemoveFrame(frameId: string): void;
    /**
     * Looks for the top frame in `_frames` and sets `_topFrame` accordingly.
     *
     * Important: This method needs to be called everytime `_frames` is updated.
     */
    _resetTopFrame(): void;
    /**
     * Returns the ResourceTreeFrame with a given frameId.
     * When a frame is being detached a new ResourceTreeFrame but with the same
     * frameId is created. Consequently getFrame() will return a different
     * ResourceTreeFrame after detachment. Callers of getFrame() should therefore
     * immediately use the function return value and not store it for later use.
     * @param {string} frameId
     * @return {?ResourceTreeFrame}
     */
    getFrame(frameId: string): ResourceTreeFrame | null;
    /**
     * @return {!Array<!ResourceTreeFrame>}
     */
    getAllFrames(): Array<ResourceTreeFrame>;
    /**
     * @return {?ResourceTreeFrame}
     */
    getTopFrame(): ResourceTreeFrame | null;
}
export type Events = symbol;
export namespace Events {
    const FrameAddedToTarget: symbol;
    const FrameNavigated: symbol;
    const FrameRemoved: symbol;
    const ResourceAdded: symbol;
    const TopFrameNavigated: symbol;
}
import { ResourceTreeModel } from "./ResourceTreeModel.js";
import { SDKModelObserver } from "./SDKModel.js";
import * as Common from "../common/common.js";
import { ResourceTreeFrame } from "./ResourceTreeModel.js";
