/** @return {!Common.Settings.Setting<boolean>} */
export function getLoadThroughTargetSetting(): Common.Settings.Setting<boolean>;
export type PageResourceLoadInitiator = {
    target: null;
    frameId: Protocol.Page.FrameId;
    initiatorUrl: string | null;
} | {
    target: Target;
    frameId: Protocol.Page.FrameId | null;
    initiatorUrl: string | null;
};
/** @typedef {{target: null, frameId: Protocol.Page.FrameId, initiatorUrl: ?string}|{target: Target, frameId: ?Protocol.Page.FrameId, initiatorUrl: ?string}} */
export let PageResourceLoadInitiator: any;
export type PageResource = {
    success: boolean | null;
    errorMessage: (undefined | string);
    initiator: PageResourceLoadInitiator;
    url: string;
    size: number | null;
};
/** @typedef {{success: ?boolean, errorMessage: (undefined|string), initiator: !PageResourceLoadInitiator, url: string, size: ?number}} */
export let PageResource: any;
/**
 * The page resource loader is a bottleneck for all DevTools-initiated resource loads. For each such load, it keeps a
 * `PageResource` object around that holds meta information. This can be as the basis for reporting to the user which
 * resources were loaded, and whether there was a load error.
 */
export class PageResourceLoader extends Common.ObjectWrapper.ObjectWrapper {
    /**
     * @param {{forceNew: boolean, loadOverride:(null|function(string):!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>), maxConcurrentLoads: number, loadTimeout: number}} _
     */
    static instance({ forceNew, loadOverride, maxConcurrentLoads, loadTimeout }?: {
        forceNew: boolean;
        loadOverride: (null | ((arg0: string) => Promise<{
            success: boolean;
            content: string;
            errorDescription: Host.ResourceLoader.LoadErrorDescription;
        }>));
        maxConcurrentLoads: number;
        loadTimeout: number;
    }): PageResourceLoader;
    /**
     * @template T
     * @param {!Promise<T>} promise
     * @param {number} timeout
     */
    static _withTimeout<T>(promise: Promise<T>, timeout: number): Promise<T>;
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @returns {string}
     */
    static makeKey(url: string, initiator: PageResourceLoadInitiator): string;
    /**
     * @param {null|function(string):!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>} loadOverride
     * @param {number} maxConcurrentLoads - Maximal number of concurrently dispatched loads. If this limit is reached, additional loads are queued up.
     * @param {number} loadTimeout - Timeout for the back-end of this loader. If the timeout is reached, the load promise is rejected and the result from the back-end gets discarded.
     */
    constructor(loadOverride: null | ((arg0: string) => Promise<{
        success: boolean;
        content: string;
        errorDescription: Host.ResourceLoader.LoadErrorDescription;
    }>), maxConcurrentLoads: number, loadTimeout: number);
    _currentlyLoading: number;
    _maxConcurrentLoads: number;
    /** @type{!Map<string, !PageResource>} */
    _pageResources: Map<string, PageResource>;
    /** @type {!Array<!{resolve:function(*):void, reject:function(*):void}>} */
    _queuedLoads: {
        resolve: (arg0: any) => void;
        reject: (arg0: any) => void;
    }[];
    _loadOverride: ((arg0: string) => Promise<{
        success: boolean;
        content: string;
        errorDescription: Host.ResourceLoader.LoadErrorDescription;
    }>) | null;
    _loadTimeout: number;
    /**
     *
     * @param {*} event
     */
    _onMainFrameNavigated(event: any): void;
    getResourcesLoaded(): Map<string, {
        success: boolean | null;
        errorMessage: (undefined | string);
        initiator: PageResourceLoadInitiator;
        url: string;
        size: number | null;
    }>;
    /**
     * Loading is the number of currently loading and queued items. Resources is the total number of resources,
     * including loading and queued resources, but not including resources that are still loading but scheduled
     * for cancelation.
     * @returns {!{loading: number, queued: number, resources: number}};
     */
    getNumberOfResources(): {
        loading: number;
        queued: number;
        resources: number;
    };
    _acquireLoadSlot(): Promise<void>;
    _releaseLoadSlot(): void;
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @return {!Promise<!{content: string}>}
     */
    loadResource(url: string, initiator: PageResourceLoadInitiator): Promise<{
        content: string;
    }>;
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @return {!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>}
     */
    _dispatchLoad(url: string, initiator: PageResourceLoadInitiator): Promise<{
        success: boolean;
        content: string;
        errorDescription: Host.ResourceLoader.LoadErrorDescription;
    }>;
    /**
     * @param {!Target} target
     * @param {?Protocol.Page.FrameId} frameId
     * @param {string} url
     */
    _loadFromTarget(target: Target, frameId: Protocol.Page.FrameId | null, url: string): Promise<{
        success: boolean;
        content: string;
        errorDescription: {
            statusCode: number;
            netError: number | undefined;
            netErrorName: string | undefined;
            message: string;
            urlValid: undefined;
        };
    }>;
}
export type Events = symbol;
export namespace Events {
    const Update: symbol;
}
import * as Common from "../common/common.js";
import { Target } from "./SDKModel.js";
import * as Host from "../host/host.js";
