import { Settings, Console, SegmentedRange, UIString } from '../common/common.js';
import { TracingModel, SDKModel, CPUProfileDataModel, LayerTreeBase, PaintProfiler } from '../sdk/sdk.js';

// Copyright 2014 The Chromium Authors. All rights reserved.
class TimelineJSProfileProcessor {
    /**
     * @param {!SDK.CPUProfileDataModel.CPUProfileDataModel} jsProfileModel
     * @param {!SDK.TracingModel.Thread} thread
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    static generateTracingEventsFromCpuProfile(jsProfileModel, thread) {
        const idleNode = jsProfileModel.idleNode;
        const programNode = jsProfileModel.programNode;
        const gcNode = jsProfileModel.gcNode;
        const samples = jsProfileModel.samples;
        const timestamps = jsProfileModel.timestamps;
        const jsEvents = [];
        /** @type {!Map<!Object, !Array<!Protocol.Runtime.CallFrame>>} */
        const nodeToStackMap = new Map();
        nodeToStackMap.set(programNode, []);
        for (let i = 0; i < samples.length; ++i) {
            let node = jsProfileModel.nodeByIndex(i);
            if (!node) {
                console.error(`Node with unknown id ${samples[i]} at index ${i}`);
                continue;
            }
            if (node === gcNode || node === idleNode) {
                continue;
            }
            let callFrames = nodeToStackMap.get(node);
            if (!callFrames) {
                callFrames = /** @type {!Array<!Protocol.Runtime.CallFrame>} */ (new Array(node.depth + 1));
                nodeToStackMap.set(node, callFrames);
                for (let j = 0; node.parent; node = node.parent) {
                    callFrames[j++] = /** @type {!Protocol.Runtime.CallFrame} */ (node);
                }
            }
            const jsSampleEvent = new TracingModel.Event(TracingModel.DevToolsTimelineEventCategory, RecordType.JSSample, TracingModel.Phase.Instant, timestamps[i], thread);
            jsSampleEvent.args['data'] = { stackTrace: callFrames };
            jsEvents.push(jsSampleEvent);
        }
        return jsEvents;
    }
    /**
     * @param {!Array<!SDK.TracingModel.Event>} events
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    static generateJSFrameEvents(events) {
        /**
         * @param {!Protocol.Runtime.CallFrame} frame1
         * @param {!Protocol.Runtime.CallFrame} frame2
         * @return {boolean}
         */
        function equalFrames(frame1, frame2) {
            return frame1.scriptId === frame2.scriptId &&
                frame1.functionName === frame2.functionName &&
                frame1.lineNumber === frame2.lineNumber;
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         * @return {boolean}
         */
        function isJSInvocationEvent(e) {
            switch (e.name) {
                case RecordType.RunMicrotasks:
                case RecordType.FunctionCall:
                case RecordType.EvaluateScript:
                case RecordType.EvaluateModule:
                case RecordType.EventDispatch:
                case RecordType.V8Execute:
                    return true;
            }
            return false;
        }
        const jsFrameEvents = [];
        const jsFramesStack = [];
        const lockedJsStackDepth = [];
        let ordinal = 0;
        const showAllEvents = Root.Runtime.experiments.isEnabled('timelineShowAllEvents');
        const showRuntimeCallStats = Root.Runtime.experiments.isEnabled('timelineV8RuntimeCallStats');
        const showNativeFunctions = Settings.Settings.instance().moduleSetting('showNativeFunctionsInJSProfile').get();
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onStartEvent(e) {
            e.ordinal = ++ordinal;
            extractStackTrace(e);
            // For the duration of the event we cannot go beyond the stack associated with it.
            lockedJsStackDepth.push(jsFramesStack.length);
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         * @param {?SDK.TracingModel.Event} parent
         */
        function onInstantEvent(e, parent) {
            e.ordinal = ++ordinal;
            if (parent && isJSInvocationEvent(parent)) {
                extractStackTrace(e);
            }
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onEndEvent(e) {
            truncateJSStack(lockedJsStackDepth.pop(), e.endTime);
        }
        /**
         * @param {number} depth
         * @param {number} time
         */
        function truncateJSStack(depth, time) {
            if (lockedJsStackDepth.length) {
                const lockedDepth = lockedJsStackDepth.peekLast();
                if (depth < lockedDepth) {
                    console.error(`Child stack is shallower (${depth}) than the parent stack (${lockedDepth}) at ${time}`);
                    depth = lockedDepth;
                }
            }
            if (jsFramesStack.length < depth) {
                console.error(`Trying to truncate higher than the current stack size at ${time}`);
                depth = jsFramesStack.length;
            }
            for (let k = 0; k < jsFramesStack.length; ++k) {
                jsFramesStack[k].setEndTime(time);
            }
            jsFramesStack.length = depth;
        }
        /**
         * @param {string} name
         * @return {boolean}
         */
        function showNativeName(name) {
            return showRuntimeCallStats && !!TimelineJSProfileProcessor.nativeGroup(name);
        }
        /**
         * @param {!Array<!Protocol.Runtime.CallFrame>} stack
         */
        function filterStackFrames(stack) {
            if (showAllEvents) {
                return;
            }
            let previousNativeFrameName = null;
            let j = 0;
            for (let i = 0; i < stack.length; ++i) {
                const frame = stack[i];
                const url = frame.url;
                const isNativeFrame = url && url.startsWith('native ');
                if (!showNativeFunctions && isNativeFrame) {
                    continue;
                }
                const isNativeRuntimeFrame = TimelineJSProfileProcessor.isNativeRuntimeFrame(frame);
                if (isNativeRuntimeFrame && !showNativeName(frame.functionName)) {
                    continue;
                }
                const nativeFrameName = isNativeRuntimeFrame ? TimelineJSProfileProcessor.nativeGroup(frame.functionName) : null;
                if (previousNativeFrameName && previousNativeFrameName === nativeFrameName) {
                    continue;
                }
                previousNativeFrameName = nativeFrameName;
                stack[j++] = frame;
            }
            stack.length = j;
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function extractStackTrace(e) {
            const recordTypes = RecordType;
            /** @type {!Array<!Protocol.Runtime.CallFrame>} */
            const callFrames = e.name === recordTypes.JSSample ? e.args['data']['stackTrace'].slice().reverse() :
                jsFramesStack.map(frameEvent => frameEvent.args['data']);
            filterStackFrames(callFrames);
            const endTime = e.endTime || e.startTime;
            const minFrames = Math.min(callFrames.length, jsFramesStack.length);
            let i;
            for (i = lockedJsStackDepth.peekLast() || 0; i < minFrames; ++i) {
                const newFrame = callFrames[i];
                const oldFrame = jsFramesStack[i].args['data'];
                if (!equalFrames(newFrame, oldFrame)) {
                    break;
                }
                jsFramesStack[i].setEndTime(Math.max(jsFramesStack[i].endTime, endTime));
            }
            truncateJSStack(i, e.startTime);
            for (; i < callFrames.length; ++i) {
                const frame = callFrames[i];
                const jsFrameEvent = new TracingModel.Event(TracingModel.DevToolsTimelineEventCategory, recordTypes.JSFrame, TracingModel.Phase.Complete, e.startTime, e.thread);
                jsFrameEvent.ordinal = e.ordinal;
                jsFrameEvent.addArgs({ data: frame });
                jsFrameEvent.setEndTime(endTime);
                jsFramesStack.push(jsFrameEvent);
                jsFrameEvents.push(jsFrameEvent);
            }
        }
        const firstTopLevelEvent = events.find(TracingModel.TracingModel.isTopLevelEvent);
        const startTime = firstTopLevelEvent ? firstTopLevelEvent.startTime : 0;
        TimelineModelImpl.forEachEvent(events, onStartEvent, onEndEvent, onInstantEvent, startTime);
        return jsFrameEvents;
    }
    /**
     * @param {!Protocol.Runtime.CallFrame} frame
     * @return {boolean}
     */
    static isNativeRuntimeFrame(frame) {
        return frame.url === 'native V8Runtime';
    }
    /**
     * @param {string} nativeName
     * @return {?TimelineJSProfileProcessor.NativeGroups}
     */
    static nativeGroup(nativeName) {
        if (nativeName.startsWith('Parse')) {
            return TimelineJSProfileProcessor.NativeGroups.Parse;
        }
        if (nativeName.startsWith('Compile') || nativeName.startsWith('Recompile')) {
            return TimelineJSProfileProcessor.NativeGroups.Compile;
        }
        return null;
    }
    /**
     * @param {*} profile
     * @param {number} tid
     * @param {boolean} injectPageEvent
     * @param {?string=} name
     * @return {!Array<!SDK.TracingManager.EventPayload>}
     */
    static buildTraceProfileFromCpuProfile(profile, tid, injectPageEvent, name) {
        const events = [];
        if (injectPageEvent) {
            appendEvent('TracingStartedInPage', { data: { 'sessionId': '1' } }, 0, 0, 'M');
        }
        if (!name) {
            name = ls `Thread ${tid}`;
        }
        appendEvent(TracingModel.MetadataEvent.ThreadName, { name }, 0, 0, 'M', '__metadata');
        if (!profile) {
            return events;
        }
        const idToNode = new Map();
        const nodes = profile['nodes'];
        for (let i = 0; i < nodes.length; ++i) {
            idToNode.set(nodes[i].id, nodes[i]);
        }
        let programEvent = null;
        let functionEvent = null;
        let nextTime = profile.startTime;
        let currentTime;
        const samples = profile['samples'];
        const timeDeltas = profile['timeDeltas'];
        for (let i = 0; i < samples.length; ++i) {
            currentTime = nextTime;
            nextTime += timeDeltas[i];
            const node = idToNode.get(samples[i]);
            const name = node.callFrame.functionName;
            if (name === '(idle)') {
                closeEvents();
                continue;
            }
            if (!programEvent) {
                programEvent = appendEvent('MessageLoop::RunTask', {}, currentTime, 0, 'X', 'toplevel');
            }
            if (name === '(program)') {
                if (functionEvent) {
                    functionEvent.dur = currentTime - functionEvent.ts;
                    functionEvent = null;
                }
            }
            else {
                // A JS function.
                if (!functionEvent) {
                    functionEvent = appendEvent('FunctionCall', { data: { 'sessionId': '1' } }, currentTime);
                }
            }
        }
        closeEvents();
        appendEvent('CpuProfile', { data: { 'cpuProfile': profile } }, profile.endTime, 0, 'I');
        return events;
        function closeEvents() {
            if (programEvent) {
                programEvent.dur = currentTime - programEvent.ts;
            }
            if (functionEvent) {
                functionEvent.dur = currentTime - functionEvent.ts;
            }
            programEvent = null;
            functionEvent = null;
        }
        /**
         * @param {string} name
         * @param {*} args
         * @param {number} ts
         * @param {number=} dur
         * @param {string=} ph
         * @param {string=} cat
         * @return {!SDK.TracingManager.EventPayload}
         */
        function appendEvent(name, args, ts, dur, ph, cat) {
            const event = /** @type {!SDK.TracingManager.EventPayload} */ ({ cat: cat || 'disabled-by-default-devtools.timeline', name, ph: ph || 'X', pid: 1, tid, ts, args });
            if (dur) {
                event.dur = dur;
            }
            events.push(event);
            return event;
        }
    }
}
/** @enum {string} */
TimelineJSProfileProcessor.NativeGroups = {
    'Compile': 'Compile',
    'Parse': 'Parse'
};

var TimelineJSProfile = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TimelineJSProfileProcessor: TimelineJSProfileProcessor
});

/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @unrestricted
 */
class TimelineModelImpl {
    constructor() {
        this._estimatedTotalBlockingTime = 0;
        this._reset();
    }
    /**
     * @param {!Array<!SDK.TracingModel.Event>} events
     * @param {function(!SDK.TracingModel.Event)} onStartEvent
     * @param {function(!SDK.TracingModel.Event)} onEndEvent
     * @param {function(!SDK.TracingModel.Event,?SDK.TracingModel.Event)|undefined=} onInstantEvent
     * @param {number=} startTime
     * @param {number=} endTime
     * @param {function(!SDK.TracingModel.Event):boolean=} filter
     */
    static forEachEvent(events, onStartEvent, onEndEvent, onInstantEvent, startTime, endTime, filter) {
        startTime = startTime || 0;
        endTime = endTime || Infinity;
        const stack = [];
        const startEvent = TimelineModelImpl._topLevelEventEndingAfter(events, startTime);
        for (let i = startEvent; i < events.length; ++i) {
            const e = events[i];
            if ((e.endTime || e.startTime) < startTime) {
                continue;
            }
            if (e.startTime >= endTime) {
                break;
            }
            if (TracingModel.TracingModel.isAsyncPhase(e.phase) || TracingModel.TracingModel.isFlowPhase(e.phase)) {
                continue;
            }
            while (stack.length && stack.peekLast().endTime <= e.startTime) {
                onEndEvent(stack.pop());
            }
            if (filter && !filter(e)) {
                continue;
            }
            if (e.duration) {
                onStartEvent(e);
                stack.push(e);
            }
            else {
                onInstantEvent && onInstantEvent(e, stack.peekLast() || null);
            }
        }
        while (stack.length) {
            onEndEvent(stack.pop());
        }
    }
    /**
     * @param {!Array<!SDK.TracingModel.Event>} events
     * @param {number} time
     */
    static _topLevelEventEndingAfter(events, time) {
        let index = events.upperBound(time, (time, event) => time - event.startTime) - 1;
        while (index > 0 && !TracingModel.TracingModel.isTopLevelEvent(events[index])) {
            index--;
        }
        return Math.max(index, 0);
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isMarkerEvent(event) {
        const recordTypes = RecordType;
        switch (event.name) {
            case recordTypes.TimeStamp:
                return true;
            case recordTypes.MarkFirstPaint:
            case recordTypes.MarkFCP:
                return this._mainFrame && event.args.frame === this._mainFrame.frameId && !!event.args.data;
            case recordTypes.MarkDOMContent:
            case recordTypes.MarkLoad:
            case recordTypes.MarkLCPCandidate:
            case recordTypes.MarkLCPInvalidate:
                return !!event.args['data']['isMainFrame'];
            default:
                return false;
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isInteractiveTimeEvent(event) {
        return event.name === RecordType.InteractiveTime;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isLayoutShiftEvent(event) {
        return event.name === RecordType.LayoutShift;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isUserTimingEvent(event) {
        return event.categoriesString === TimelineModelImpl.Category.UserTiming;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isParseHTMLEvent(event) {
        return event.name === RecordType.ParseHTML;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isLCPCandidateEvent(event) {
        return event.name === RecordType.MarkLCPCandidate && !!event.args['data']['isMainFrame'];
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    isLCPInvalidateEvent(event) {
        return event.name === RecordType.MarkLCPInvalidate && !!event.args['data']['isMainFrame'];
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {string} field
     * @return {string}
     */
    static globalEventId(event, field) {
        const data = event.args['data'] || event.args['beginData'];
        const id = data && data[field];
        if (!id) {
            return '';
        }
        return `${event.thread.process().id()}.${id}`;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {string}
     */
    static eventFrameId(event) {
        const data = event.args['data'] || event.args['beginData'];
        return data && data['frame'] || '';
    }
    /**
     * @return {!Array<!SDK.CPUProfileDataModel.CPUProfileDataModel>}
     */
    cpuProfiles() {
        return this._cpuProfiles;
    }
    /**
     * @return {{time: number, estimated: boolean}}
     */
    totalBlockingTime() {
        if (this._totalBlockingTime === -1) {
            return { time: this._estimatedTotalBlockingTime, estimated: true };
        }
        return { time: this._totalBlockingTime, estimated: false };
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {?SDK.SDKModel.Target}
     */
    targetByEvent(event) {
        // FIXME: Consider returning null for loaded traces.
        const workerId = this._workerIdByThread.get(event.thread);
        const mainTarget = SDKModel.TargetManager.instance().mainTarget();
        return workerId ? SDKModel.TargetManager.instance().targetById(workerId) : mainTarget;
    }
    /**
     * @return {!Map<string, !SDK.TracingModel.Event>}
     */
    navStartTimes() {
        if (!this._tracingModel) {
            return new Map();
        }
        return this._tracingModel.navStartTimes();
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    setEvents(tracingModel) {
        this._reset();
        this._resetProcessingState();
        this._tracingModel = tracingModel;
        this._minimumRecordTime = tracingModel.minimumRecordTime();
        this._maximumRecordTime = tracingModel.maximumRecordTime();
        // Remove LayoutShift events from the main thread list of events because they are
        // represented in the experience track. This is done prior to the main thread being processed for its own events.
        const layoutShiftEvents = [];
        for (const process of tracingModel.sortedProcesses()) {
            if (process.name() !== 'Renderer') {
                continue;
            }
            for (const thread of process.sortedThreads()) {
                const shifts = thread.removeEventsByName(RecordType.LayoutShift);
                layoutShiftEvents.push(...shifts);
            }
        }
        this._processSyncBrowserEvents(tracingModel);
        if (this._browserFrameTracking) {
            this._processThreadsForBrowserFrames(tracingModel);
        }
        else {
            // The next line is for loading legacy traces recorded before M67.
            // TODO(alph): Drop the support at some point.
            const metadataEvents = this._processMetadataEvents(tracingModel);
            this._isGenericTrace = !metadataEvents;
            if (metadataEvents) {
                this._processMetadataAndThreads(tracingModel, metadataEvents);
            }
            else {
                this._processGenericTrace(tracingModel);
            }
        }
        this._inspectedTargetEvents.sort(TracingModel.Event.compareStartTime);
        this._processAsyncBrowserEvents(tracingModel);
        this._buildGPUEvents(tracingModel);
        this._buildLoadingEvents(tracingModel, layoutShiftEvents);
        this._resetProcessingState();
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    _processGenericTrace(tracingModel) {
        let browserMainThread = TracingModel.TracingModel.browserMainThread(tracingModel);
        if (!browserMainThread && tracingModel.sortedProcesses().length) {
            browserMainThread = tracingModel.sortedProcesses()[0].sortedThreads()[0];
        }
        for (const process of tracingModel.sortedProcesses()) {
            for (const thread of process.sortedThreads()) {
                this._processThreadEvents(tracingModel, [{ from: 0, to: Infinity }], thread, thread === browserMainThread, false, true, null);
            }
        }
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @param {!MetadataEvents} metadataEvents
     */
    _processMetadataAndThreads(tracingModel, metadataEvents) {
        let startTime = 0;
        for (let i = 0, length = metadataEvents.page.length; i < length; i++) {
            const metaEvent = metadataEvents.page[i];
            const process = metaEvent.thread.process();
            const endTime = i + 1 < length ? metadataEvents.page[i + 1].startTime : Infinity;
            if (startTime === endTime) {
                continue;
            }
            this._legacyCurrentPage = metaEvent.args['data'] && metaEvent.args['data']['page'];
            for (const thread of process.sortedThreads()) {
                let workerUrl = null;
                if (thread.name() === TimelineModelImpl.WorkerThreadName ||
                    thread.name() === TimelineModelImpl.WorkerThreadNameLegacy) {
                    const workerMetaEvent = metadataEvents.workers.find(e => {
                        if (e.args['data']['workerThreadId'] !== thread.id()) {
                            return false;
                        }
                        // This is to support old traces.
                        if (e.args['data']['sessionId'] === this._sessionId) {
                            return true;
                        }
                        return !!this._pageFrames.get(TimelineModelImpl.eventFrameId(e));
                    });
                    if (!workerMetaEvent) {
                        continue;
                    }
                    const workerId = workerMetaEvent.args['data']['workerId'];
                    if (workerId) {
                        this._workerIdByThread.set(thread, workerId);
                    }
                    workerUrl = workerMetaEvent.args['data']['url'] || '';
                }
                this._processThreadEvents(tracingModel, [{ from: startTime, to: endTime }], thread, thread === metaEvent.thread, !!workerUrl, true, workerUrl);
            }
            startTime = endTime;
        }
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    _processThreadsForBrowserFrames(tracingModel) {
        const processData = new Map();
        for (const frame of this._pageFrames.values()) {
            for (let i = 0; i < frame.processes.length; i++) {
                const pid = frame.processes[i].processId;
                let data = processData.get(pid);
                if (!data) {
                    data = [];
                    processData.set(pid, data);
                }
                const to = i === frame.processes.length - 1 ? (frame.deletedTime || Infinity) : frame.processes[i + 1].time;
                data.push({ from: frame.processes[i].time, to: to, main: !frame.parent, url: frame.processes[i].url });
            }
        }
        const allMetadataEvents = tracingModel.devToolsMetadataEvents();
        for (const process of tracingModel.sortedProcesses()) {
            const data = processData.get(process.id());
            if (!data) {
                continue;
            }
            data.sort((a, b) => a.from - b.from || a.to - b.to);
            const ranges = [];
            let lastUrl = null;
            let lastMainUrl = null;
            let hasMain = false;
            for (const item of data) {
                if (!ranges.length || item.from > ranges.peekLast().to) {
                    ranges.push({ from: item.from, to: item.to });
                }
                else {
                    ranges.peekLast().to = item.to;
                }
                if (item.main) {
                    hasMain = true;
                }
                if (item.url) {
                    if (item.main) {
                        lastMainUrl = item.url;
                    }
                    lastUrl = item.url;
                }
            }
            for (const thread of process.sortedThreads()) {
                if (thread.name() === TimelineModelImpl.RendererMainThreadName) {
                    this._processThreadEvents(tracingModel, ranges, thread, true /* isMainThread */, false /* isWorker */, hasMain, hasMain ? lastMainUrl : lastUrl);
                }
                else if (thread.name() === TimelineModelImpl.WorkerThreadName ||
                    thread.name() === TimelineModelImpl.WorkerThreadNameLegacy) {
                    const workerMetaEvent = allMetadataEvents.find(e => {
                        if (e.name !== TimelineModelImpl.DevToolsMetadataEvent.TracingSessionIdForWorker) {
                            return false;
                        }
                        if (e.thread.process() !== process) {
                            return false;
                        }
                        if (e.args['data']['workerThreadId'] !== thread.id()) {
                            return false;
                        }
                        return !!this._pageFrames.get(TimelineModelImpl.eventFrameId(e));
                    });
                    if (!workerMetaEvent) {
                        continue;
                    }
                    this._workerIdByThread.set(thread, workerMetaEvent.args['data']['workerId'] || '');
                    this._processThreadEvents(tracingModel, ranges, thread, false /* isMainThread */, true /* isWorker */, false /* forMainFrame */, workerMetaEvent.args['data']['url'] || '');
                }
                else {
                    this._processThreadEvents(tracingModel, ranges, thread, false /* isMainThread */, false /* isWorker */, false /* forMainFrame */, null);
                }
            }
        }
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @return {?MetadataEvents}
     */
    _processMetadataEvents(tracingModel) {
        const metadataEvents = tracingModel.devToolsMetadataEvents();
        const pageDevToolsMetadataEvents = [];
        const workersDevToolsMetadataEvents = [];
        for (const event of metadataEvents) {
            if (event.name === TimelineModelImpl.DevToolsMetadataEvent.TracingStartedInPage) {
                pageDevToolsMetadataEvents.push(event);
                if (event.args['data'] && event.args['data']['persistentIds']) {
                    this._persistentIds = true;
                }
                const frames = ((event.args['data'] && event.args['data']['frames']) || []);
                frames.forEach(payload => this._addPageFrame(event, payload));
                this._mainFrame = this.rootFrames()[0];
            }
            else if (event.name === TimelineModelImpl.DevToolsMetadataEvent.TracingSessionIdForWorker) {
                workersDevToolsMetadataEvents.push(event);
            }
            else if (event.name === TimelineModelImpl.DevToolsMetadataEvent.TracingStartedInBrowser) {
                console.assert(!this._mainFrameNodeId, 'Multiple sessions in trace');
                this._mainFrameNodeId = event.args['frameTreeNodeId'];
            }
        }
        if (!pageDevToolsMetadataEvents.length) {
            return null;
        }
        const sessionId = pageDevToolsMetadataEvents[0].args['sessionId'] || pageDevToolsMetadataEvents[0].args['data']['sessionId'];
        this._sessionId = sessionId;
        const mismatchingIds = new Set();
        /**
         * @param {!SDK.TracingModel.Event} event
         * @return {boolean}
         */
        function checkSessionId(event) {
            let args = event.args;
            // FIXME: put sessionId into args["data"] for TracingStartedInPage event.
            if (args['data']) {
                args = args['data'];
            }
            const id = args['sessionId'];
            if (id === sessionId) {
                return true;
            }
            mismatchingIds.add(id);
            return false;
        }
        const result = {
            page: pageDevToolsMetadataEvents.filter(checkSessionId).sort(TracingModel.Event.compareStartTime),
            workers: workersDevToolsMetadataEvents.sort(TracingModel.Event.compareStartTime)
        };
        if (mismatchingIds.size) {
            Console.Console.instance().error('Timeline recording was started in more than one page simultaneously. Session id mismatch: ' +
                this._sessionId + ' and ' + [...mismatchingIds] + '.');
        }
        return result;
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    _processSyncBrowserEvents(tracingModel) {
        const browserMain = TracingModel.TracingModel.browserMainThread(tracingModel);
        if (browserMain) {
            browserMain.events().forEach(this._processBrowserEvent, this);
        }
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    _processAsyncBrowserEvents(tracingModel) {
        const browserMain = TracingModel.TracingModel.browserMainThread(tracingModel);
        if (browserMain) {
            this._processAsyncEvents(browserMain, [{ from: 0, to: Infinity }]);
        }
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     */
    _buildGPUEvents(tracingModel) {
        const thread = tracingModel.threadByName('GPU Process', 'CrGpuMain');
        if (!thread) {
            return;
        }
        const gpuEventName = RecordType.GPUTask;
        const track = this._ensureNamedTrack(TrackType.GPU);
        track.thread = thread;
        track.events = thread.events().filter(event => event.name === gpuEventName);
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @param {!Array<!SDK.TracingModel.Event>} events
     */
    _buildLoadingEvents(tracingModel, events) {
        const thread = tracingModel.threadByName('Renderer', 'CrRendererMain');
        if (!thread) {
            return;
        }
        const experienceCategory = 'experience';
        const track = this._ensureNamedTrack(TrackType.Experience);
        track.thread = thread;
        track.events = events;
        // Even though the event comes from 'loading', in order to color it differently we
        // rename its category.
        for (const trackEvent of track.events) {
            trackEvent.categoriesString = experienceCategory;
            if (trackEvent.name === RecordType.LayoutShift) {
                const eventData = trackEvent.args['data'] || trackEvent.args['beginData'] || {};
                const timelineData = TimelineData.forEvent(trackEvent);
                timelineData.backendNodeId = eventData['impacted_nodes'][0]['node_id'];
            }
        }
    }
    _resetProcessingState() {
        this._asyncEventTracker = new TimelineAsyncEventTracker();
        this._invalidationTracker = new InvalidationTracker();
        this._layoutInvalidate = {};
        this._lastScheduleStyleRecalculation = {};
        this._paintImageEventByPixelRefId = {};
        this._lastPaintForLayer = {};
        this._lastRecalculateStylesEvent = null;
        this._currentScriptEvent = null;
        this._eventStack = [];
        /** @type {!Set<string>} */
        this._knownInputEvents = new Set();
        this._browserFrameTracking = false;
        this._persistentIds = false;
        this._legacyCurrentPage = null;
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @param {!SDK.TracingModel.Thread} thread
     * @return {?SDK.CPUProfileDataModel.CPUProfileDataModel}
     */
    _extractCpuProfile(tracingModel, thread) {
        const events = thread.events();
        let cpuProfile;
        let target = null;
        // Check for legacy CpuProfile event format first.
        let cpuProfileEvent = events.peekLast();
        if (cpuProfileEvent && cpuProfileEvent.name === RecordType.CpuProfile) {
            const eventData = cpuProfileEvent.args['data'];
            cpuProfile = /** @type {?Protocol.Profiler.Profile} */ (eventData && eventData['cpuProfile']);
            target = this.targetByEvent(cpuProfileEvent);
        }
        if (!cpuProfile) {
            cpuProfileEvent = events.find(e => e.name === RecordType.Profile);
            if (!cpuProfileEvent) {
                return null;
            }
            target = this.targetByEvent(cpuProfileEvent);
            const profileGroup = tracingModel.profileGroup(cpuProfileEvent);
            if (!profileGroup) {
                Console.Console.instance().error('Invalid CPU profile format.');
                return null;
            }
            cpuProfile = /** @type {!Protocol.Profiler.Profile} */ ({
                // Do not use |cpuProfileEvent.args['data']['startTime']| as it is in
                // CLOCK_MONOTONIC domain, but use |profileEvent.startTime|
                // (|ts| in the trace event) which has been translated to
                // Perfetto's clock domain.
                //
                // |cpuProfileEvent.startTime| has been converted to milliseconds
                // when the Event was loaded but |cpuProfile.timeDeltas| are
                // expressed in microseconds.
                startTime: cpuProfileEvent.startTime * 1000,
                endTime: 0,
                nodes: [],
                samples: [],
                timeDeltas: [],
                lines: []
            });
            for (const profileEvent of profileGroup.children) {
                const eventData = profileEvent.args['data'];
                if ('startTime' in eventData) {
                    // Do not use |eventData['startTime']| as it is in CLOCK_MONOTONIC domain,
                    // but use |profileEvent.startTime| (|ts| in the trace event) which has
                    // been translated to Perfetto's clock domain.
                    //
                    // Also convert from ms to us.
                    cpuProfile.startTime = profileEvent.startTime * 1000;
                }
                if ('endTime' in eventData) {
                    // Do not use |eventData['endTime']| as it is in CLOCK_MONOTONIC domain,
                    // but use |profileEvent.startTime| (|ts| in the trace event) which has
                    // been translated to Perfetto's clock domain.
                    //
                    // Despite its name, |profileEvent.startTime| was recorded right after
                    // |eventData['endTime']| within v8 and is a reasonable substitute.
                    //
                    // Also convert from ms to us.
                    cpuProfile.endTime = profileEvent.startTime * 1000;
                }
                const nodesAndSamples = eventData['cpuProfile'] || {};
                const samples = nodesAndSamples['samples'] || [];
                const lines = eventData['lines'] || Array(samples.length).fill(0);
                cpuProfile.nodes.push(...(nodesAndSamples['nodes'] || []));
                cpuProfile.lines.push(...lines);
                cpuProfile.samples.push(...samples);
                cpuProfile.timeDeltas.push(...(eventData['timeDeltas'] || []));
                if (cpuProfile.samples.length !== cpuProfile.timeDeltas.length) {
                    Console.Console.instance().error('Failed to parse CPU profile.');
                    return null;
                }
            }
            if (!cpuProfile.endTime) {
                cpuProfile.endTime = cpuProfile.timeDeltas.reduce((x, y) => x + y, cpuProfile.startTime);
            }
        }
        try {
            const jsProfileModel = new CPUProfileDataModel.CPUProfileDataModel(cpuProfile, target);
            this._cpuProfiles.push(jsProfileModel);
            return jsProfileModel;
        }
        catch (e) {
            Console.Console.instance().error('Failed to parse CPU profile.');
        }
        return null;
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @param {!SDK.TracingModel.Thread} thread
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    _injectJSFrameEvents(tracingModel, thread) {
        const jsProfileModel = this._extractCpuProfile(tracingModel, thread);
        let events = thread.events();
        const jsSamples = jsProfileModel ? TimelineJSProfileProcessor.generateTracingEventsFromCpuProfile(jsProfileModel, thread) : null;
        if (jsSamples && jsSamples.length) {
            events = events.mergeOrdered(jsSamples, TracingModel.Event.orderedCompareStartTime);
        }
        if (jsSamples || events.some(e => e.name === RecordType.JSSample)) {
            const jsFrameEvents = TimelineJSProfileProcessor.generateJSFrameEvents(events);
            if (jsFrameEvents && jsFrameEvents.length) {
                events = jsFrameEvents.mergeOrdered(events, TracingModel.Event.orderedCompareStartTime);
            }
        }
        return events;
    }
    /**
     * @param {!SDK.TracingModel.TracingModel} tracingModel
     * @param {!Array<!{from: number, to: number}>} ranges
     * @param {!SDK.TracingModel.Thread} thread
     * @param {boolean} isMainThread
     * @param {boolean} isWorker
     * @param {boolean} forMainFrame
     * @param {?string} url
     */
    _processThreadEvents(tracingModel, ranges, thread, isMainThread, isWorker, forMainFrame, url) {
        const track = new Track();
        track.name = thread.name() || ls `Thread ${thread.id()}`;
        track.type = TrackType.Other;
        track.thread = thread;
        if (isMainThread) {
            track.type = TrackType.MainThread;
            track.url = url || null;
            track.forMainFrame = forMainFrame;
        }
        else if (isWorker) {
            track.type = TrackType.Worker;
            track.url = url;
            track.name = track.url ? ls `Worker — ${track.url}` : ls `Dedicated Worker`;
        }
        else if (thread.name().startsWith('CompositorTileWorker')) {
            track.type = TrackType.Raster;
        }
        this._tracks.push(track);
        const events = this._injectJSFrameEvents(tracingModel, thread);
        this._eventStack = [];
        const eventStack = this._eventStack;
        // Get the worker name from the target.
        if (isWorker) {
            const cpuProfileEvent = events.find(event => event.name === RecordType.Profile);
            if (cpuProfileEvent) {
                const target = this.targetByEvent(cpuProfileEvent);
                if (target) {
                    track.name = ls `Worker: ${target.name()} — ${track.url}`;
                }
            }
        }
        for (const range of ranges) {
            let i = events.lowerBound(range.from, (time, event) => time - event.startTime);
            for (; i < events.length; i++) {
                const event = events[i];
                if (event.startTime >= range.to) {
                    break;
                }
                // There may be several TTI events, only take the first one.
                if (this.isInteractiveTimeEvent(event) && this._totalBlockingTime === -1) {
                    this._totalBlockingTime = event.args['args']['total_blocking_time_ms'];
                }
                const isLongRunningTask = event.name === RecordType.Task && event.duration && event.duration > 50;
                if (isMainThread && isLongRunningTask && event.duration) {
                    // We only track main thread events that are over 50ms, and the amount of time in the
                    // event (over 50ms) is what constitutes the blocking time. An event of 70ms, therefore,
                    // contributes 20ms to TBT.
                    this._estimatedTotalBlockingTime += event.duration - 50;
                }
                while (eventStack.length && eventStack.peekLast().endTime <= event.startTime) {
                    eventStack.pop();
                }
                if (!this._processEvent(event)) {
                    continue;
                }
                if (!TracingModel.TracingModel.isAsyncPhase(event.phase) && event.duration) {
                    if (eventStack.length) {
                        const parent = eventStack.peekLast();
                        parent.selfTime -= event.duration;
                        if (parent.selfTime < 0) {
                            this._fixNegativeDuration(parent, event);
                        }
                    }
                    event.selfTime = event.duration;
                    if (!eventStack.length) {
                        track.tasks.push(event);
                    }
                    eventStack.push(event);
                }
                if (this.isMarkerEvent(event)) {
                    this._timeMarkerEvents.push(event);
                }
                track.events.push(event);
                this._inspectedTargetEvents.push(event);
            }
        }
        this._processAsyncEvents(thread, ranges);
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {!SDK.TracingModel.Event} child
     */
    _fixNegativeDuration(event, child) {
        const epsilon = 1e-3;
        if (event.selfTime < -epsilon) {
            console.error(`Children are longer than parent at ${event.startTime} ` +
                `(${(child.startTime - this.minimumRecordTime()).toFixed(3)} by ${(-event.selfTime).toFixed(3)}`);
        }
        event.selfTime = 0;
    }
    /**
     * @param {!SDK.TracingModel.Thread} thread
     * @param {!Array<!{from: number, to: number}>} ranges
     */
    _processAsyncEvents(thread, ranges) {
        const asyncEvents = thread.asyncEvents();
        const groups = new Map();
        /**
         * @param {!TrackType} type
         * @return {!Array<!SDK.TracingModel.AsyncEvent>}
         */
        function group(type) {
            if (!groups.has(type)) {
                groups.set(type, []);
            }
            return groups.get(type);
        }
        for (const range of ranges) {
            let i = asyncEvents.lowerBound(range.from, function (time, asyncEvent) {
                return time - asyncEvent.startTime;
            });
            for (; i < asyncEvents.length; ++i) {
                const asyncEvent = asyncEvents[i];
                if (asyncEvent.startTime >= range.to) {
                    break;
                }
                if (asyncEvent.hasCategory(TimelineModelImpl.Category.Console)) {
                    group(TrackType.Console).push(asyncEvent);
                    continue;
                }
                if (asyncEvent.hasCategory(TimelineModelImpl.Category.UserTiming)) {
                    group(TrackType.Timings).push(asyncEvent);
                    continue;
                }
                if (asyncEvent.name === RecordType.Animation) {
                    group(TrackType.Animation).push(asyncEvent);
                    continue;
                }
                if (asyncEvent.hasCategory(TimelineModelImpl.Category.LatencyInfo) ||
                    asyncEvent.name === RecordType.ImplSideFling) {
                    const lastStep = asyncEvent.steps.peekLast();
                    // FIXME: fix event termination on the back-end instead.
                    if (lastStep.phase !== TracingModel.Phase.AsyncEnd) {
                        continue;
                    }
                    const data = lastStep.args['data'];
                    asyncEvent.causedFrame = !!(data && data['INPUT_EVENT_LATENCY_RENDERER_SWAP_COMPONENT']);
                    if (asyncEvent.hasCategory(TimelineModelImpl.Category.LatencyInfo)) {
                        if (!this._knownInputEvents.has(lastStep.id)) {
                            continue;
                        }
                        if (asyncEvent.name === RecordType.InputLatencyMouseMove && !asyncEvent.causedFrame) {
                            continue;
                        }
                        // Coalesced events are not really been processed, no need to track them.
                        if (data['is_coalesced']) {
                            continue;
                        }
                        const rendererMain = data['INPUT_EVENT_LATENCY_RENDERER_MAIN_COMPONENT'];
                        if (rendererMain) {
                            const time = rendererMain['time'] / 1000;
                            TimelineData.forEvent(asyncEvent.steps[0]).timeWaitingForMainThread =
                                time - asyncEvent.steps[0].startTime;
                        }
                    }
                    group(TrackType.Input).push(asyncEvent);
                    continue;
                }
            }
        }
        for (const [type, events] of groups) {
            const track = this._ensureNamedTrack(type);
            track.thread = thread;
            track.asyncEvents = track.asyncEvents.mergeOrdered(events, TracingModel.Event.compareStartTime);
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    _processEvent(event) {
        const recordTypes = RecordType;
        const eventStack = this._eventStack;
        if (!eventStack.length) {
            if (this._currentTaskLayoutAndRecalcEvents && this._currentTaskLayoutAndRecalcEvents.length) {
                const totalTime = this._currentTaskLayoutAndRecalcEvents.reduce((time, event) => time + event.duration, 0);
                if (totalTime > TimelineModelImpl.Thresholds.ForcedLayout) {
                    for (const e of this._currentTaskLayoutAndRecalcEvents) {
                        const timelineData = TimelineData.forEvent(e);
                        timelineData.warning = e.name === recordTypes.Layout ? TimelineModelImpl.WarningType.ForcedLayout :
                            TimelineModelImpl.WarningType.ForcedStyle;
                    }
                }
            }
            this._currentTaskLayoutAndRecalcEvents = [];
        }
        if (this._currentScriptEvent && event.startTime > this._currentScriptEvent.endTime) {
            this._currentScriptEvent = null;
        }
        const eventData = event.args['data'] || event.args['beginData'] || {};
        const timelineData = TimelineData.forEvent(event);
        if (eventData['stackTrace']) {
            timelineData.stackTrace = eventData['stackTrace'];
        }
        if (timelineData.stackTrace && event.name !== recordTypes.JSSample) {
            // TraceEvents come with 1-based line & column numbers. The frontend code
            // requires 0-based ones. Adjust the values.
            for (let i = 0; i < timelineData.stackTrace.length; ++i) {
                --timelineData.stackTrace[i].lineNumber;
                --timelineData.stackTrace[i].columnNumber;
            }
        }
        let pageFrameId = TimelineModelImpl.eventFrameId(event);
        if (!pageFrameId && eventStack.length) {
            pageFrameId = TimelineData.forEvent(eventStack.peekLast()).frameId;
        }
        timelineData.frameId = pageFrameId || (this._mainFrame && this._mainFrame.frameId) || '';
        this._asyncEventTracker.processEvent(event);
        if (this.isMarkerEvent(event)) {
            this._ensureNamedTrack(TrackType.Timings);
        }
        switch (event.name) {
            case recordTypes.ResourceSendRequest:
            case recordTypes.WebSocketCreate: {
                timelineData.setInitiator(eventStack.peekLast() || null);
                timelineData.url = eventData['url'];
                break;
            }
            case recordTypes.ScheduleStyleRecalculation: {
                this._lastScheduleStyleRecalculation[eventData['frame']] = event;
                break;
            }
            case recordTypes.UpdateLayoutTree:
            case recordTypes.RecalculateStyles: {
                this._invalidationTracker.didRecalcStyle(event);
                if (event.args['beginData']) {
                    timelineData.setInitiator(this._lastScheduleStyleRecalculation[event.args['beginData']['frame']]);
                }
                this._lastRecalculateStylesEvent = event;
                if (this._currentScriptEvent) {
                    this._currentTaskLayoutAndRecalcEvents.push(event);
                }
                break;
            }
            case recordTypes.ScheduleStyleInvalidationTracking:
            case recordTypes.StyleRecalcInvalidationTracking:
            case recordTypes.StyleInvalidatorInvalidationTracking:
            case recordTypes.LayoutInvalidationTracking: {
                this._invalidationTracker.addInvalidation(new InvalidationTrackingEvent(event));
                break;
            }
            case recordTypes.InvalidateLayout: {
                // Consider style recalculation as a reason for layout invalidation,
                // but only if we had no earlier layout invalidation records.
                let layoutInitator = event;
                const frameId = eventData['frame'];
                if (!this._layoutInvalidate[frameId] && this._lastRecalculateStylesEvent &&
                    this._lastRecalculateStylesEvent.endTime > event.startTime) {
                    layoutInitator = TimelineData.forEvent(this._lastRecalculateStylesEvent).initiator();
                }
                this._layoutInvalidate[frameId] = layoutInitator;
                break;
            }
            case recordTypes.Layout: {
                this._invalidationTracker.didLayout(event);
                const frameId = event.args['beginData']['frame'];
                timelineData.setInitiator(this._layoutInvalidate[frameId]);
                // In case we have no closing Layout event, endData is not available.
                if (event.args['endData']) {
                    timelineData.backendNodeId = event.args['endData']['rootNode'];
                }
                this._layoutInvalidate[frameId] = null;
                if (this._currentScriptEvent) {
                    this._currentTaskLayoutAndRecalcEvents.push(event);
                }
                break;
            }
            case recordTypes.Task: {
                if (event.duration > TimelineModelImpl.Thresholds.LongTask) {
                    timelineData.warning = TimelineModelImpl.WarningType.LongTask;
                }
                break;
            }
            case recordTypes.EventDispatch: {
                if (event.duration > TimelineModelImpl.Thresholds.RecurringHandler) {
                    timelineData.warning = TimelineModelImpl.WarningType.LongHandler;
                }
                break;
            }
            case recordTypes.TimerFire:
            case recordTypes.FireAnimationFrame: {
                if (event.duration > TimelineModelImpl.Thresholds.RecurringHandler) {
                    timelineData.warning = TimelineModelImpl.WarningType.LongRecurringHandler;
                }
                break;
            }
            case recordTypes.FunctionCall: {
                // Compatibility with old format.
                if (typeof eventData['scriptName'] === 'string') {
                    eventData['url'] = eventData['scriptName'];
                }
                if (typeof eventData['scriptLine'] === 'number') {
                    eventData['lineNumber'] = eventData['scriptLine'];
                }
                // Fallthrough intended.
            }
            case recordTypes.EvaluateScript:
            case recordTypes.CompileScript: {
                if (typeof eventData['lineNumber'] === 'number') {
                    --eventData['lineNumber'];
                }
                if (typeof eventData['columnNumber'] === 'number') {
                    --eventData['columnNumber'];
                }
                // Fallthrough intended.
            }
            case recordTypes.RunMicrotasks: {
                // Microtasks technically are not necessarily scripts, but for purpose of
                // forced sync style recalc or layout detection they are.
                if (!this._currentScriptEvent) {
                    this._currentScriptEvent = event;
                }
                break;
            }
            case recordTypes.SetLayerTreeId: {
                // This is to support old traces.
                if (this._sessionId && eventData['sessionId'] && this._sessionId === eventData['sessionId']) {
                    this._mainFrameLayerTreeId = eventData['layerTreeId'];
                    break;
                }
                // We currently only show layer tree for the main frame.
                const frameId = TimelineModelImpl.eventFrameId(event);
                const pageFrame = this._pageFrames.get(frameId);
                if (!pageFrame || pageFrame.parent) {
                    return false;
                }
                this._mainFrameLayerTreeId = eventData['layerTreeId'];
                break;
            }
            case recordTypes.Paint: {
                this._invalidationTracker.didPaint(event);
                timelineData.backendNodeId = eventData['nodeId'];
                // Only keep layer paint events, skip paints for subframes that get painted to the same layer as parent.
                if (!eventData['layerId']) {
                    break;
                }
                const layerId = eventData['layerId'];
                this._lastPaintForLayer[layerId] = event;
                break;
            }
            case recordTypes.DisplayItemListSnapshot:
            case recordTypes.PictureSnapshot: {
                const layerUpdateEvent = this._findAncestorEvent(recordTypes.UpdateLayer);
                if (!layerUpdateEvent || layerUpdateEvent.args['layerTreeId'] !== this._mainFrameLayerTreeId) {
                    break;
                }
                const paintEvent = this._lastPaintForLayer[layerUpdateEvent.args['layerId']];
                if (paintEvent) {
                    TimelineData.forEvent(paintEvent).picture =
                        /** @type {!SDK.TracingModel.ObjectSnapshot} */ (event);
                }
                break;
            }
            case recordTypes.ScrollLayer: {
                timelineData.backendNodeId = eventData['nodeId'];
                break;
            }
            case recordTypes.PaintImage: {
                timelineData.backendNodeId = eventData['nodeId'];
                timelineData.url = eventData['url'];
                break;
            }
            case recordTypes.DecodeImage:
            case recordTypes.ResizeImage: {
                let paintImageEvent = this._findAncestorEvent(recordTypes.PaintImage);
                if (!paintImageEvent) {
                    const decodeLazyPixelRefEvent = this._findAncestorEvent(recordTypes.DecodeLazyPixelRef);
                    paintImageEvent = decodeLazyPixelRefEvent &&
                        this._paintImageEventByPixelRefId[decodeLazyPixelRefEvent.args['LazyPixelRef']];
                }
                if (!paintImageEvent) {
                    break;
                }
                const paintImageData = TimelineData.forEvent(paintImageEvent);
                timelineData.backendNodeId = paintImageData.backendNodeId;
                timelineData.url = paintImageData.url;
                break;
            }
            case recordTypes.DrawLazyPixelRef: {
                const paintImageEvent = this._findAncestorEvent(recordTypes.PaintImage);
                if (!paintImageEvent) {
                    break;
                }
                this._paintImageEventByPixelRefId[event.args['LazyPixelRef']] = paintImageEvent;
                const paintImageData = TimelineData.forEvent(paintImageEvent);
                timelineData.backendNodeId = paintImageData.backendNodeId;
                timelineData.url = paintImageData.url;
                break;
            }
            case recordTypes.FrameStartedLoading: {
                if (timelineData.frameId !== event.args['frame']) {
                    return false;
                }
                break;
            }
            case recordTypes.MarkLCPCandidate: {
                timelineData.backendNodeId = eventData['nodeId'];
                break;
            }
            case recordTypes.MarkDOMContent:
            case recordTypes.MarkLoad: {
                const frameId = TimelineModelImpl.eventFrameId(event);
                if (!this._pageFrames.has(frameId)) {
                    return false;
                }
                break;
            }
            case recordTypes.CommitLoad: {
                if (this._browserFrameTracking) {
                    break;
                }
                const frameId = TimelineModelImpl.eventFrameId(event);
                const isMainFrame = !!eventData['isMainFrame'];
                const pageFrame = this._pageFrames.get(frameId);
                if (pageFrame) {
                    pageFrame.update(event.startTime, eventData);
                }
                else {
                    // We should only have one main frame which has persistent id,
                    // unless it's an old trace without 'persistentIds' flag.
                    if (!this._persistentIds) {
                        if (eventData['page'] && eventData['page'] !== this._legacyCurrentPage) {
                            return false;
                        }
                    }
                    else if (isMainFrame) {
                        return false;
                    }
                    else if (!this._addPageFrame(event, eventData)) {
                        return false;
                    }
                }
                if (isMainFrame) {
                    this._mainFrame = this._pageFrames.get(frameId);
                }
                break;
            }
            case recordTypes.FireIdleCallback: {
                if (event.duration > eventData['allottedMilliseconds'] + TimelineModelImpl.Thresholds.IdleCallbackAddon) {
                    timelineData.warning = TimelineModelImpl.WarningType.IdleDeadlineExceeded;
                }
                break;
            }
        }
        return true;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    _processBrowserEvent(event) {
        if (event.name === RecordType.LatencyInfoFlow) {
            const frameId = event.args['frameTreeNodeId'];
            if (typeof frameId === 'number' && frameId === this._mainFrameNodeId && event.bind_id) {
                this._knownInputEvents.add(event.bind_id);
            }
            return;
        }
        if (event.name === RecordType.ResourceWillSendRequest) {
            const requestId = event.args['data']['requestId'];
            if (typeof requestId === 'string') {
                this._requestsFromBrowser.set(requestId, event);
            }
            return;
        }
        if (event.hasCategory(TracingModel.DevToolsMetadataEventCategory) && event.args['data']) {
            const data = event.args['data'];
            if (event.name === TimelineModelImpl.DevToolsMetadataEvent.TracingStartedInBrowser) {
                if (!data['persistentIds']) {
                    return;
                }
                this._browserFrameTracking = true;
                this._mainFrameNodeId = data['frameTreeNodeId'];
                const frames = data['frames'] || [];
                frames.forEach(payload => {
                    const parent = payload['parent'] && this._pageFrames.get(payload['parent']);
                    if (payload['parent'] && !parent) {
                        return;
                    }
                    let frame = this._pageFrames.get(payload['frame']);
                    if (!frame) {
                        frame = new PageFrame(payload);
                        this._pageFrames.set(frame.frameId, frame);
                        if (parent) {
                            parent.addChild(frame);
                        }
                        else {
                            this._mainFrame = frame;
                        }
                    }
                    // TODO(dgozman): this should use event.startTime, but due to races between tracing start
                    // in different processes we cannot do this yet.
                    frame.update(this._minimumRecordTime, payload);
                });
                return;
            }
            if (event.name === TimelineModelImpl.DevToolsMetadataEvent.FrameCommittedInBrowser &&
                this._browserFrameTracking) {
                let frame = this._pageFrames.get(data['frame']);
                if (!frame) {
                    const parent = data['parent'] && this._pageFrames.get(data['parent']);
                    if (!parent) {
                        return;
                    }
                    frame = new PageFrame(data);
                    this._pageFrames.set(frame.frameId, frame);
                    parent.addChild(frame);
                }
                frame.update(event.startTime, data);
                return;
            }
            if (event.name === TimelineModelImpl.DevToolsMetadataEvent.ProcessReadyInBrowser && this._browserFrameTracking) {
                const frame = this._pageFrames.get(data['frame']);
                if (frame) {
                    frame.processReady(data['processPseudoId'], data['processId']);
                }
                return;
            }
            if (event.name === TimelineModelImpl.DevToolsMetadataEvent.FrameDeletedInBrowser && this._browserFrameTracking) {
                const frame = this._pageFrames.get(data['frame']);
                if (frame) {
                    frame.deletedTime = event.startTime;
                }
                return;
            }
        }
    }
    /**
     * @param {!TrackType} type
     * @return {!Track}
     */
    _ensureNamedTrack(type) {
        if (!this._namedTracks.has(type)) {
            const track = new Track();
            track.type = type;
            this._tracks.push(track);
            this._namedTracks.set(type, track);
        }
        return this._namedTracks.get(type);
    }
    /**
     * @param {string} name
     * @return {?SDK.TracingModel.Event}
     */
    _findAncestorEvent(name) {
        for (let i = this._eventStack.length - 1; i >= 0; --i) {
            const event = this._eventStack[i];
            if (event.name === name) {
                return event;
            }
        }
        return null;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {!Object} payload
     * @return {boolean}
     */
    _addPageFrame(event, payload) {
        const parent = payload['parent'] && this._pageFrames.get(payload['parent']);
        if (payload['parent'] && !parent) {
            return false;
        }
        const pageFrame = new PageFrame(payload);
        this._pageFrames.set(pageFrame.frameId, pageFrame);
        pageFrame.update(event.startTime, payload);
        if (parent) {
            parent.addChild(pageFrame);
        }
        return true;
    }
    _reset() {
        this._isGenericTrace = false;
        /** @type {!Array<!Track>} */
        this._tracks = [];
        /** @type {!Map<!TrackType, !Track>} */
        this._namedTracks = new Map();
        /** @type {!Array<!SDK.TracingModel.Event>} */
        this._inspectedTargetEvents = [];
        /** @type {!Array<!SDK.TracingModel.Event>} */
        this._timeMarkerEvents = [];
        /** @type {?string} */
        this._sessionId = null;
        /** @type {?number} */
        this._mainFrameNodeId = null;
        /** @type {!Array<!SDK.CPUProfileDataModel.CPUProfileDataModel>} */
        this._cpuProfiles = [];
        /** @type {!WeakMap<!SDK.TracingModel.Thread, string>} */
        this._workerIdByThread = new WeakMap();
        /** @type {!Map<string, !PageFrame>} */
        this._pageFrames = new Map();
        this._mainFrame = null;
        /** @type {!Map<string, !SDK.TracingModel.Event>} */
        this._requestsFromBrowser = new Map();
        this._minimumRecordTime = 0;
        this._maximumRecordTime = 0;
        this._totalBlockingTime = -1;
        this._estimatedTotalBlockingTime = 0;
    }
    /**
     * @return {boolean}
     */
    isGenericTrace() {
        return this._isGenericTrace;
    }
    /**
     * @return {!SDK.TracingModel.TracingModel}
     */
    tracingModel() {
        return this._tracingModel;
    }
    /**
     * @return {number}
     */
    minimumRecordTime() {
        return this._minimumRecordTime;
    }
    /**
     * @return {number}
     */
    maximumRecordTime() {
        return this._maximumRecordTime;
    }
    /**
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    inspectedTargetEvents() {
        return this._inspectedTargetEvents;
    }
    /**
     * @return {!Array<!Track>}
     */
    tracks() {
        return this._tracks;
    }
    /**
     * @return {boolean}
     */
    isEmpty() {
        return this.minimumRecordTime() === 0 && this.maximumRecordTime() === 0;
    }
    /**
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    timeMarkerEvents() {
        return this._timeMarkerEvents;
    }
    /**
     * @return {!Array<!PageFrame>}
     */
    rootFrames() {
        return Array.from(this._pageFrames.values()).filter(frame => !frame.parent);
    }
    /**
     * @return {string}
     */
    pageURL() {
        return this._mainFrame && this._mainFrame.url || '';
    }
    /**
     * @param {string} frameId
     * @return {?PageFrame}
     */
    pageFrameById(frameId) {
        return frameId ? this._pageFrames.get(frameId) || null : null;
    }
    /**
     * @return {!Array<!NetworkRequest>}
     */
    networkRequests() {
        if (this.isGenericTrace()) {
            return [];
        }
        /** @type {!Map<string,!NetworkRequest>} */
        const requests = new Map();
        /** @type {!Array<!NetworkRequest>} */
        const requestsList = [];
        /** @type {!Array<!NetworkRequest>} */
        const zeroStartRequestsList = [];
        const types = RecordType;
        const resourceTypes = new Set([
            types.ResourceWillSendRequest, types.ResourceSendRequest, types.ResourceReceiveResponse,
            types.ResourceReceivedData, types.ResourceFinish, types.ResourceMarkAsCached
        ]);
        const events = this.inspectedTargetEvents();
        for (let i = 0; i < events.length; ++i) {
            const e = events[i];
            if (!resourceTypes.has(e.name)) {
                continue;
            }
            const id = TimelineModelImpl.globalEventId(e, 'requestId');
            if (e.name === types.ResourceSendRequest && this._requestsFromBrowser.has(e.args.data.requestId)) {
                addRequest(this._requestsFromBrowser.get(e.args.data.requestId), id);
            }
            addRequest(e, id);
        }
        function addRequest(e, id) {
            let request = requests.get(id);
            if (request) {
                request.addEvent(e);
            }
            else {
                request = new NetworkRequest(e);
                requests.set(id, request);
                if (request.startTime) {
                    requestsList.push(request);
                }
                else {
                    zeroStartRequestsList.push(request);
                }
            }
        }
        return zeroStartRequestsList.concat(requestsList);
    }
}
/**
 * @enum {string}
 */
const RecordType = {
    Task: 'RunTask',
    Program: 'Program',
    EventDispatch: 'EventDispatch',
    GPUTask: 'GPUTask',
    Animation: 'Animation',
    RequestMainThreadFrame: 'RequestMainThreadFrame',
    BeginFrame: 'BeginFrame',
    NeedsBeginFrameChanged: 'NeedsBeginFrameChanged',
    BeginMainThreadFrame: 'BeginMainThreadFrame',
    ActivateLayerTree: 'ActivateLayerTree',
    DrawFrame: 'DrawFrame',
    HitTest: 'HitTest',
    ScheduleStyleRecalculation: 'ScheduleStyleRecalculation',
    RecalculateStyles: 'RecalculateStyles',
    UpdateLayoutTree: 'UpdateLayoutTree',
    InvalidateLayout: 'InvalidateLayout',
    Layout: 'Layout',
    LayoutShift: 'LayoutShift',
    UpdateLayer: 'UpdateLayer',
    UpdateLayerTree: 'UpdateLayerTree',
    PaintSetup: 'PaintSetup',
    Paint: 'Paint',
    PaintImage: 'PaintImage',
    Rasterize: 'Rasterize',
    RasterTask: 'RasterTask',
    ScrollLayer: 'ScrollLayer',
    CompositeLayers: 'CompositeLayers',
    InteractiveTime: 'InteractiveTime',
    ScheduleStyleInvalidationTracking: 'ScheduleStyleInvalidationTracking',
    StyleRecalcInvalidationTracking: 'StyleRecalcInvalidationTracking',
    StyleInvalidatorInvalidationTracking: 'StyleInvalidatorInvalidationTracking',
    LayoutInvalidationTracking: 'LayoutInvalidationTracking',
    ParseHTML: 'ParseHTML',
    ParseAuthorStyleSheet: 'ParseAuthorStyleSheet',
    TimerInstall: 'TimerInstall',
    TimerRemove: 'TimerRemove',
    TimerFire: 'TimerFire',
    XHRReadyStateChange: 'XHRReadyStateChange',
    XHRLoad: 'XHRLoad',
    CompileScript: 'v8.compile',
    EvaluateScript: 'EvaluateScript',
    CompileModule: 'v8.compileModule',
    EvaluateModule: 'v8.evaluateModule',
    WasmStreamFromResponseCallback: 'v8.wasm.streamFromResponseCallback',
    WasmCompiledModule: 'v8.wasm.compiledModule',
    WasmCachedModule: 'v8.wasm.cachedModule',
    WasmModuleCacheHit: 'v8.wasm.moduleCacheHit',
    WasmModuleCacheInvalid: 'v8.wasm.moduleCacheInvalid',
    FrameStartedLoading: 'FrameStartedLoading',
    CommitLoad: 'CommitLoad',
    MarkLoad: 'MarkLoad',
    MarkDOMContent: 'MarkDOMContent',
    MarkFirstPaint: 'firstPaint',
    MarkFCP: 'firstContentfulPaint',
    MarkLCPCandidate: 'largestContentfulPaint::Candidate',
    MarkLCPInvalidate: 'largestContentfulPaint::Invalidate',
    NavigationStart: 'navigationStart',
    TimeStamp: 'TimeStamp',
    ConsoleTime: 'ConsoleTime',
    UserTiming: 'UserTiming',
    ResourceWillSendRequest: 'ResourceWillSendRequest',
    ResourceSendRequest: 'ResourceSendRequest',
    ResourceReceiveResponse: 'ResourceReceiveResponse',
    ResourceReceivedData: 'ResourceReceivedData',
    ResourceFinish: 'ResourceFinish',
    ResourceMarkAsCached: 'ResourceMarkAsCached',
    RunMicrotasks: 'RunMicrotasks',
    FunctionCall: 'FunctionCall',
    GCEvent: 'GCEvent',
    MajorGC: 'MajorGC',
    MinorGC: 'MinorGC',
    JSFrame: 'JSFrame',
    JSSample: 'JSSample',
    // V8Sample events are coming from tracing and contain raw stacks with function addresses.
    // After being processed with help of JitCodeAdded and JitCodeMoved events they
    // get translated into function infos and stored as stacks in JSSample events.
    V8Sample: 'V8Sample',
    JitCodeAdded: 'JitCodeAdded',
    JitCodeMoved: 'JitCodeMoved',
    StreamingCompileScript: 'v8.parseOnBackground',
    StreamingCompileScriptWaiting: 'v8.parseOnBackgroundWaiting',
    StreamingCompileScriptParsing: 'v8.parseOnBackgroundParsing',
    V8Execute: 'V8.Execute',
    UpdateCounters: 'UpdateCounters',
    RequestAnimationFrame: 'RequestAnimationFrame',
    CancelAnimationFrame: 'CancelAnimationFrame',
    FireAnimationFrame: 'FireAnimationFrame',
    RequestIdleCallback: 'RequestIdleCallback',
    CancelIdleCallback: 'CancelIdleCallback',
    FireIdleCallback: 'FireIdleCallback',
    WebSocketCreate: 'WebSocketCreate',
    WebSocketSendHandshakeRequest: 'WebSocketSendHandshakeRequest',
    WebSocketReceiveHandshakeResponse: 'WebSocketReceiveHandshakeResponse',
    WebSocketDestroy: 'WebSocketDestroy',
    EmbedderCallback: 'EmbedderCallback',
    SetLayerTreeId: 'SetLayerTreeId',
    TracingStartedInPage: 'TracingStartedInPage',
    TracingSessionIdForWorker: 'TracingSessionIdForWorker',
    DecodeImage: 'Decode Image',
    ResizeImage: 'Resize Image',
    DrawLazyPixelRef: 'Draw LazyPixelRef',
    DecodeLazyPixelRef: 'Decode LazyPixelRef',
    LazyPixelRef: 'LazyPixelRef',
    LayerTreeHostImplSnapshot: 'cc::LayerTreeHostImpl',
    PictureSnapshot: 'cc::Picture',
    DisplayItemListSnapshot: 'cc::DisplayItemList',
    LatencyInfo: 'LatencyInfo',
    LatencyInfoFlow: 'LatencyInfo.Flow',
    InputLatencyMouseMove: 'InputLatency::MouseMove',
    InputLatencyMouseWheel: 'InputLatency::MouseWheel',
    ImplSideFling: 'InputHandlerProxy::HandleGestureFling::started',
    GCCollectGarbage: 'BlinkGC.AtomicPhase',
    CryptoDoEncrypt: 'DoEncrypt',
    CryptoDoEncryptReply: 'DoEncryptReply',
    CryptoDoDecrypt: 'DoDecrypt',
    CryptoDoDecryptReply: 'DoDecryptReply',
    CryptoDoDigest: 'DoDigest',
    CryptoDoDigestReply: 'DoDigestReply',
    CryptoDoSign: 'DoSign',
    CryptoDoSignReply: 'DoSignReply',
    CryptoDoVerify: 'DoVerify',
    CryptoDoVerifyReply: 'DoVerifyReply',
    // CpuProfile is a virtual event created on frontend to support
    // serialization of CPU Profiles within tracing timeline data.
    CpuProfile: 'CpuProfile',
    Profile: 'Profile',
    AsyncTask: 'AsyncTask',
};
TimelineModelImpl.Category = {
    Console: 'blink.console',
    UserTiming: 'blink.user_timing',
    LatencyInfo: 'latencyInfo',
    Loading: 'loading',
};
/**
 * @enum {string}
 */
TimelineModelImpl.WarningType = {
    LongTask: 'LongTask',
    ForcedStyle: 'ForcedStyle',
    ForcedLayout: 'ForcedLayout',
    IdleDeadlineExceeded: 'IdleDeadlineExceeded',
    LongHandler: 'LongHandler',
    LongRecurringHandler: 'LongRecurringHandler',
    V8Deopt: 'V8Deopt'
};
TimelineModelImpl.WorkerThreadName = 'DedicatedWorker thread';
TimelineModelImpl.WorkerThreadNameLegacy = 'DedicatedWorker Thread';
TimelineModelImpl.RendererMainThreadName = 'CrRendererMain';
TimelineModelImpl.BrowserMainThreadName = 'CrBrowserMain';
TimelineModelImpl.DevToolsMetadataEvent = {
    TracingStartedInBrowser: 'TracingStartedInBrowser',
    TracingStartedInPage: 'TracingStartedInPage',
    TracingSessionIdForWorker: 'TracingSessionIdForWorker',
    FrameCommittedInBrowser: 'FrameCommittedInBrowser',
    ProcessReadyInBrowser: 'ProcessReadyInBrowser',
    FrameDeletedInBrowser: 'FrameDeletedInBrowser',
};
TimelineModelImpl.Thresholds = {
    LongTask: 50,
    Handler: 150,
    RecurringHandler: 50,
    ForcedLayout: 30,
    IdleCallbackAddon: 5
};
class Track {
    constructor() {
        this.name = '';
        this.type = TrackType.Other;
        // TODO(dgozman): replace forMainFrame with a list of frames, urls and time ranges.
        this.forMainFrame = false;
        this.url = '';
        // TODO(dgozman): do not distinguish between sync and async events.
        /** @type {!Array<!SDK.TracingModel.Event>} */
        this.events = [];
        /** @type {!Array<!SDK.TracingModel.AsyncEvent>} */
        this.asyncEvents = [];
        /** @type {!Array<!SDK.TracingModel.Event>} */
        this.tasks = [];
        this._syncEvents = null;
        /** @type {?SDK.TracingModel.Thread} */
        this.thread = null;
    }
    /**
     * @return {!Array<!SDK.TracingModel.Event>}
     */
    syncEvents() {
        if (this.events.length) {
            return this.events;
        }
        if (this._syncEvents) {
            return this._syncEvents;
        }
        const stack = [];
        this._syncEvents = [];
        for (const event of this.asyncEvents) {
            const startTime = event.startTime;
            const endTime = event.endTime;
            while (stack.length && startTime >= stack.peekLast().endTime) {
                stack.pop();
            }
            if (stack.length && endTime > stack.peekLast().endTime) {
                this._syncEvents = [];
                break;
            }
            const syncEvent = new TracingModel.Event(event.categoriesString, event.name, TracingModel.Phase.Complete, startTime, event.thread);
            syncEvent.setEndTime(endTime);
            syncEvent.addArgs(event.args);
            this._syncEvents.push(syncEvent);
            stack.push(syncEvent);
        }
        return this._syncEvents;
    }
}
/**
 * @enum {symbol}
 */
const TrackType = {
    MainThread: Symbol('MainThread'),
    Worker: Symbol('Worker'),
    Input: Symbol('Input'),
    Animation: Symbol('Animation'),
    Timings: Symbol('Timings'),
    Console: Symbol('Console'),
    Raster: Symbol('Raster'),
    GPU: Symbol('GPU'),
    Experience: Symbol('Experience'),
    Other: Symbol('Other'),
};
class PageFrame {
    /**
     * @param {!Object} payload
     */
    constructor(payload) {
        this.frameId = payload['frame'];
        this.url = payload['url'] || '';
        this.name = payload['name'];
        /** @type {!Array<!PageFrame>} */
        this.children = [];
        /** @type {?PageFrame} */
        this.parent = null;
        /** @type {!Array<!{time: number, processId: number, processPseudoId: ?string, url: string}>} */
        this.processes = [];
        /** @type {?number} */
        this.deletedTime = null;
        // TODO(dgozman): figure this out.
        // this.ownerNode = target && payload['nodeId'] ? new SDK.DOMModel.DeferredDOMNode(target, payload['nodeId']) : null;
        this.ownerNode = null;
    }
    /**
     * @param {number} time
     * @param {!Object} payload
     */
    update(time, payload) {
        this.url = payload['url'] || '';
        this.name = payload['name'];
        if (payload['processId']) {
            this.processes.push({ time: time, processId: payload['processId'], processPseudoId: '', url: payload['url'] || '' });
        }
        else {
            this.processes.push({ time: time, processId: -1, processPseudoId: payload['processPseudoId'], url: payload['url'] || '' });
        }
    }
    /**
     * @param {string} processPseudoId
     * @param {number} processId
     */
    processReady(processPseudoId, processId) {
        for (const process of this.processes) {
            if (process.processPseudoId === processPseudoId) {
                process.processPseudoId = '';
                process.processId = processId;
            }
        }
    }
    /**
     * @param {!PageFrame} child
     */
    addChild(child) {
        this.children.push(child);
        child.parent = this;
    }
}
/**
 * @unrestricted
 */
class NetworkRequest {
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    constructor(event) {
        const recordType = RecordType;
        const isInitial = event.name === recordType.ResourceSendRequest || event.name === recordType.ResourceWillSendRequest;
        this.startTime = isInitial ? event.startTime : 0;
        this.endTime = Infinity;
        this.encodedDataLength = 0;
        this.decodedBodyLength = 0;
        /** @type {!Array<!SDK.TracingModel.Event>} */
        this.children = [];
        /** @type {?Object} */
        this.timing;
        /** @type {string} */
        this.mimeType;
        /** @type {string} */
        this.url;
        /** @type {string} */
        this.requestMethod;
        /** @type {number} */
        this._transferSize = 0;
        /** @type {boolean} */
        this._maybeDiskCached = false;
        /** @type {boolean} */
        this._memoryCached = false;
        this.addEvent(event);
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    addEvent(event) {
        this.children.push(event);
        const recordType = RecordType;
        // This Math.min is likely because of BUG(chromium:865066).
        this.startTime = Math.min(this.startTime, event.startTime);
        const eventData = event.args['data'];
        if (eventData['mimeType']) {
            this.mimeType = eventData['mimeType'];
        }
        if ('priority' in eventData) {
            this.priority = eventData['priority'];
        }
        if (event.name === recordType.ResourceFinish) {
            this.endTime = event.startTime;
        }
        if (eventData['finishTime']) {
            this.finishTime = eventData['finishTime'] * 1000;
        }
        if (!this.responseTime &&
            (event.name === recordType.ResourceReceiveResponse || event.name === recordType.ResourceReceivedData)) {
            this.responseTime = event.startTime;
        }
        const encodedDataLength = eventData['encodedDataLength'] || 0;
        if (event.name === recordType.ResourceMarkAsCached) {
            // This is a reliable signal for memory caching.
            this._memoryCached = true;
        }
        if (event.name === recordType.ResourceReceiveResponse) {
            if (eventData['fromCache']) {
                // See BUG(chromium:998397): back-end over-approximates caching.
                this._maybeDiskCached = true;
            }
            if (eventData['fromServiceWorker']) {
                this.fromServiceWorker = true;
            }
            if (eventData['hasCachedResource']) {
                this.hasCachedResource = true;
            }
            this.encodedDataLength = encodedDataLength;
        }
        if (event.name === recordType.ResourceReceivedData) {
            this.encodedDataLength += encodedDataLength;
        }
        if (event.name === recordType.ResourceFinish && encodedDataLength) {
            this.encodedDataLength = encodedDataLength;
            // If a ResourceFinish event with an encoded data length is received,
            // then the resource was not cached; it was fetched before it was
            // requested, e.g. because it was pushed in this navigation.
            this._transferSize = encodedDataLength;
        }
        const decodedBodyLength = eventData['decodedBodyLength'];
        if (event.name === recordType.ResourceFinish && decodedBodyLength) {
            this.decodedBodyLength = decodedBodyLength;
        }
        if (!this.url) {
            this.url = eventData['url'];
        }
        if (!this.requestMethod) {
            this.requestMethod = eventData['requestMethod'];
        }
        if (!this.timing) {
            this.timing = eventData['timing'];
        }
        if (eventData['fromServiceWorker']) {
            this.fromServiceWorker = true;
        }
    }
    /**
     * Return whether this request was cached. This works around BUG(chromium:998397),
     * which reports pushed resources, and resources serverd by a service worker as
     * disk cached. Pushed resources that were not disk cached, however, have a non-zero
     * `_transferSize`.
     * @return {boolean}
     */
    cached() {
        return !!this._memoryCached || (!!this._maybeDiskCached && !this._transferSize && !this.fromServiceWorker);
    }
    /**
     * Return whether this request was served from a memory cache.
     * @return {boolean}
     */
    memoryCached() {
        return this._memoryCached;
    }
    /**
     * Get the timing information for this request. If the request was cached,
     * the timing refers to the original (uncached) load, and should not be used.
     * @return {!{sendStartTime: number, headersEndTime: number}}
     */
    getSendReceiveTiming() {
        if (this.cached() || !this.timing) {
            // If the request is served from cache, the timing refers to the original
            // resource load, and should not be used.
            return { sendStartTime: this.startTime, headersEndTime: this.startTime };
        }
        const requestTime = this.timing.requestTime * 1000;
        const sendStartTime = requestTime + this.timing.sendStart;
        const headersEndTime = requestTime + this.timing.receiveHeadersEnd;
        return { sendStartTime, headersEndTime };
    }
    /**
     * Get the start time of this request, i.e. the time when the browser or
     * renderer queued this request. There are two cases where request time is
     * earlier than `startTime`: (1) if the request is served from cache, because
     * it refers to the original load of the resource. (2) if the request was
     * initiated by the browser instead of the renderer. Only in case (2) the
     * the request time must be used instead of the start time to work around
     * BUG(chromium:865066).
     * @return {number}
     */
    getStartTime() {
        return Math.min(this.startTime, !this.cached() && this.timing && this.timing.requestTime * 1000 || Infinity);
    }
    /**
     * Returns the time where the earliest event belonging to this request starts.
     * This differs from `getStartTime()` if a previous HTTP/2 request pushed the
     * resource proactively: Then `beginTime()` refers to the time the push was received.
     * @return {number}
     */
    beginTime() {
        // `pushStart` is referring to the original push if the request was cached (i.e. in
        // general not the most recent push), and should hence only be used for requests that were not cached.
        return Math.min(this.getStartTime(), !this.cached() && this.timing && this.timing.pushStart * 1000 || Infinity);
    }
}
/**
 * @unrestricted
 */
class InvalidationTrackingEvent {
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    constructor(event) {
        /** @type {string} */
        this.type = event.name;
        /** @type {number} */
        this.startTime = event.startTime;
        /** @type {!SDK.TracingModel.Event} */
        this._tracingEvent = event;
        const eventData = event.args['data'];
        /** @type {number} */
        this.frame = eventData['frame'];
        /** @type {?number} */
        this.nodeId = eventData['nodeId'];
        /** @type {?string} */
        this.nodeName = eventData['nodeName'];
        /** @type {?number} */
        this.invalidationSet = eventData['invalidationSet'];
        /** @type {?string} */
        this.invalidatedSelectorId = eventData['invalidatedSelectorId'];
        /** @type {?string} */
        this.changedId = eventData['changedId'];
        /** @type {?string} */
        this.changedClass = eventData['changedClass'];
        /** @type {?string} */
        this.changedAttribute = eventData['changedAttribute'];
        /** @type {?string} */
        this.changedPseudo = eventData['changedPseudo'];
        /** @type {?string} */
        this.selectorPart = eventData['selectorPart'];
        /** @type {?string} */
        this.extraData = eventData['extraData'];
        /** @type {?Array.<!Object.<string, number>>} */
        this.invalidationList = eventData['invalidationList'];
        /** @type {!InvalidationCause} */
        this.cause = { reason: eventData['reason'], stackTrace: eventData['stackTrace'] };
        // FIXME: Move this to TimelineUIUtils.js.
        if (!this.cause.reason && this.cause.stackTrace && this.type === RecordType.LayoutInvalidationTracking) {
            this.cause.reason = 'Layout forced';
        }
    }
}
class InvalidationTracker {
    constructor() {
        /** @type {?SDK.TracingModel.Event} */
        this._lastRecalcStyle = null;
        /** @type {?SDK.TracingModel.Event} */
        this._lastPaintWithLayer = null;
        this._didPaint = false;
        this._initializePerFrameState();
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {?Array<!InvalidationTrackingEvent>}
     */
    static invalidationEventsFor(event) {
        return event[InvalidationTracker._invalidationTrackingEventsSymbol] || null;
    }
    /**
     * @param {!InvalidationTrackingEvent} invalidation
     */
    addInvalidation(invalidation) {
        this._startNewFrameIfNeeded();
        if (!invalidation.nodeId) {
            console.error('Invalidation lacks node information.');
            console.error(invalidation);
            return;
        }
        const recordTypes = RecordType;
        // Suppress StyleInvalidator StyleRecalcInvalidationTracking invalidations because they
        // will be handled by StyleInvalidatorInvalidationTracking.
        // FIXME: Investigate if we can remove StyleInvalidator invalidations entirely.
        if (invalidation.type === recordTypes.StyleRecalcInvalidationTracking &&
            invalidation.cause.reason === 'StyleInvalidator') {
            return;
        }
        // Style invalidation events can occur before and during recalc style. didRecalcStyle
        // handles style invalidations that occur before the recalc style event but we need to
        // handle style recalc invalidations during recalc style here.
        const styleRecalcInvalidation = (invalidation.type === recordTypes.ScheduleStyleInvalidationTracking ||
            invalidation.type === recordTypes.StyleInvalidatorInvalidationTracking ||
            invalidation.type === recordTypes.StyleRecalcInvalidationTracking);
        if (styleRecalcInvalidation) {
            const duringRecalcStyle = invalidation.startTime && this._lastRecalcStyle &&
                invalidation.startTime >= this._lastRecalcStyle.startTime &&
                invalidation.startTime <= this._lastRecalcStyle.endTime;
            if (duringRecalcStyle) {
                this._associateWithLastRecalcStyleEvent(invalidation);
            }
        }
        // Record the invalidation so later events can look it up.
        if (this._invalidations[invalidation.type]) {
            this._invalidations[invalidation.type].push(invalidation);
        }
        else {
            this._invalidations[invalidation.type] = [invalidation];
        }
        if (invalidation.nodeId) {
            if (this._invalidationsByNodeId[invalidation.nodeId]) {
                this._invalidationsByNodeId[invalidation.nodeId].push(invalidation);
            }
            else {
                this._invalidationsByNodeId[invalidation.nodeId] = [invalidation];
            }
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} recalcStyleEvent
     */
    didRecalcStyle(recalcStyleEvent) {
        this._lastRecalcStyle = recalcStyleEvent;
        const types = [
            RecordType.ScheduleStyleInvalidationTracking, RecordType.StyleInvalidatorInvalidationTracking,
            RecordType.StyleRecalcInvalidationTracking
        ];
        for (const invalidation of this._invalidationsOfTypes(types)) {
            this._associateWithLastRecalcStyleEvent(invalidation);
        }
    }
    /**
     * @param {!InvalidationTrackingEvent} invalidation
     */
    _associateWithLastRecalcStyleEvent(invalidation) {
        if (invalidation.linkedRecalcStyleEvent) {
            return;
        }
        const recordTypes = RecordType;
        const recalcStyleFrameId = this._lastRecalcStyle.args['beginData']['frame'];
        if (invalidation.type === recordTypes.StyleInvalidatorInvalidationTracking) {
            // Instead of calling _addInvalidationToEvent directly, we create synthetic
            // StyleRecalcInvalidationTracking events which will be added in _addInvalidationToEvent.
            this._addSyntheticStyleRecalcInvalidations(this._lastRecalcStyle, recalcStyleFrameId, invalidation);
        }
        else if (invalidation.type === recordTypes.ScheduleStyleInvalidationTracking) {
            // ScheduleStyleInvalidationTracking events are only used for adding information to
            // StyleInvalidatorInvalidationTracking events. See: _addSyntheticStyleRecalcInvalidations.
        }
        else {
            this._addInvalidationToEvent(this._lastRecalcStyle, recalcStyleFrameId, invalidation);
        }
        invalidation.linkedRecalcStyleEvent = true;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {number} frameId
     * @param {!InvalidationTrackingEvent} styleInvalidatorInvalidation
     */
    _addSyntheticStyleRecalcInvalidations(event, frameId, styleInvalidatorInvalidation) {
        if (!styleInvalidatorInvalidation.invalidationList) {
            this._addSyntheticStyleRecalcInvalidation(styleInvalidatorInvalidation._tracingEvent, styleInvalidatorInvalidation);
            return;
        }
        if (!styleInvalidatorInvalidation.nodeId) {
            console.error('Invalidation lacks node information.');
            console.error(styleInvalidatorInvalidation);
            return;
        }
        for (let i = 0; i < styleInvalidatorInvalidation.invalidationList.length; i++) {
            const setId = styleInvalidatorInvalidation.invalidationList[i]['id'];
            let lastScheduleStyleRecalculation;
            const nodeInvalidations = this._invalidationsByNodeId[styleInvalidatorInvalidation.nodeId] || [];
            for (let j = 0; j < nodeInvalidations.length; j++) {
                const invalidation = nodeInvalidations[j];
                if (invalidation.frame !== frameId || invalidation.invalidationSet !== setId ||
                    invalidation.type !== RecordType.ScheduleStyleInvalidationTracking) {
                    continue;
                }
                lastScheduleStyleRecalculation = invalidation;
            }
            if (!lastScheduleStyleRecalculation) {
                console.error('Failed to lookup the event that scheduled a style invalidator invalidation.');
                continue;
            }
            this._addSyntheticStyleRecalcInvalidation(lastScheduleStyleRecalculation._tracingEvent, styleInvalidatorInvalidation);
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} baseEvent
     * @param {!InvalidationTrackingEvent} styleInvalidatorInvalidation
     */
    _addSyntheticStyleRecalcInvalidation(baseEvent, styleInvalidatorInvalidation) {
        const invalidation = new InvalidationTrackingEvent(baseEvent);
        invalidation.type = RecordType.StyleRecalcInvalidationTracking;
        if (styleInvalidatorInvalidation.cause.reason) {
            invalidation.cause.reason = styleInvalidatorInvalidation.cause.reason;
        }
        if (styleInvalidatorInvalidation.selectorPart) {
            invalidation.selectorPart = styleInvalidatorInvalidation.selectorPart;
        }
        this.addInvalidation(invalidation);
        if (!invalidation.linkedRecalcStyleEvent) {
            this._associateWithLastRecalcStyleEvent(invalidation);
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} layoutEvent
     */
    didLayout(layoutEvent) {
        const layoutFrameId = layoutEvent.args['beginData']['frame'];
        for (const invalidation of this._invalidationsOfTypes([RecordType.LayoutInvalidationTracking])) {
            if (invalidation.linkedLayoutEvent) {
                continue;
            }
            this._addInvalidationToEvent(layoutEvent, layoutFrameId, invalidation);
            invalidation.linkedLayoutEvent = true;
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} paintEvent
     */
    didPaint(paintEvent) {
        this._didPaint = true;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {number} eventFrameId
     * @param {!InvalidationTrackingEvent} invalidation
     */
    _addInvalidationToEvent(event, eventFrameId, invalidation) {
        if (eventFrameId !== invalidation.frame) {
            return;
        }
        if (!event[InvalidationTracker._invalidationTrackingEventsSymbol]) {
            event[InvalidationTracker._invalidationTrackingEventsSymbol] = [invalidation];
        }
        else {
            event[InvalidationTracker._invalidationTrackingEventsSymbol].push(invalidation);
        }
    }
    /**
     * @param {!Array.<string>=} types
     * @return {!Generator<!InvalidationTrackingEvent>}
     */
    _invalidationsOfTypes(types) {
        const invalidations = this._invalidations;
        if (!types) {
            types = Object.keys(invalidations);
        }
        function* generator() {
            for (let i = 0; i < types.length; ++i) {
                const invalidationList = invalidations[types[i]] || [];
                for (let j = 0; j < invalidationList.length; ++j) {
                    yield invalidationList[j];
                }
            }
        }
        return generator();
    }
    _startNewFrameIfNeeded() {
        if (!this._didPaint) {
            return;
        }
        this._initializePerFrameState();
    }
    _initializePerFrameState() {
        /** @type {!Object.<string, !Array.<!InvalidationTrackingEvent>>} */
        this._invalidations = {};
        /** @type {!Object.<number, !Array.<!InvalidationTrackingEvent>>} */
        this._invalidationsByNodeId = {};
        this._lastRecalcStyle = null;
        this._lastPaintWithLayer = null;
        this._didPaint = false;
    }
}
InvalidationTracker._invalidationTrackingEventsSymbol = Symbol('invalidationTrackingEvents');
/**
 * @unrestricted
 */
class TimelineAsyncEventTracker {
    constructor() {
        TimelineAsyncEventTracker._initialize();
        /** @type {!Map<!RecordType, !Map<string, !SDK.TracingModel.Event>>} */
        this._initiatorByType = new Map();
        for (const initiator of TimelineAsyncEventTracker._asyncEvents.keys()) {
            this._initiatorByType.set(initiator, new Map());
        }
    }
    static _initialize() {
        if (TimelineAsyncEventTracker._asyncEvents) {
            return;
        }
        const events = new Map();
        let type = RecordType;
        events.set(type.TimerInstall, { causes: [type.TimerFire], joinBy: 'timerId' });
        events.set(type.ResourceSendRequest, {
            causes: [type.ResourceMarkAsCached, type.ResourceReceiveResponse, type.ResourceReceivedData, type.ResourceFinish],
            joinBy: 'requestId'
        });
        events.set(type.RequestAnimationFrame, { causes: [type.FireAnimationFrame], joinBy: 'id' });
        events.set(type.RequestIdleCallback, { causes: [type.FireIdleCallback], joinBy: 'id' });
        events.set(type.WebSocketCreate, {
            causes: [type.WebSocketSendHandshakeRequest, type.WebSocketReceiveHandshakeResponse, type.WebSocketDestroy],
            joinBy: 'identifier'
        });
        TimelineAsyncEventTracker._asyncEvents = events;
        /** @type {!Map<!RecordType, !RecordType>} */
        TimelineAsyncEventTracker._typeToInitiator = new Map();
        for (const entry of events) {
            const types = entry[1].causes;
            for (type of types) {
                TimelineAsyncEventTracker._typeToInitiator.set(type, entry[0]);
            }
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    processEvent(event) {
        let initiatorType = TimelineAsyncEventTracker._typeToInitiator.get(
        /** @type {!RecordType} */ (event.name));
        const isInitiator = !initiatorType;
        if (!initiatorType) {
            initiatorType = /** @type {!RecordType} */ (event.name);
        }
        const initiatorInfo = TimelineAsyncEventTracker._asyncEvents.get(initiatorType);
        if (!initiatorInfo) {
            return;
        }
        const id = TimelineModelImpl.globalEventId(event, initiatorInfo.joinBy);
        if (!id) {
            return;
        }
        /** @type {!Map<string, !SDK.TracingModel.Event>|undefined} */
        const initiatorMap = this._initiatorByType.get(initiatorType);
        if (isInitiator) {
            initiatorMap.set(id, event);
            return;
        }
        const initiator = initiatorMap.get(id) || null;
        const timelineData = TimelineData.forEvent(event);
        timelineData.setInitiator(initiator);
        if (!timelineData.frameId && initiator) {
            timelineData.frameId = TimelineModelImpl.eventFrameId(initiator);
        }
    }
}
class TimelineData {
    constructor() {
        /** @type {?string} */
        this.warning = null;
        /** @type {?Element} */
        this.previewElement = null;
        /** @type {?string} */
        this.url = null;
        /** @type {number} */
        this.backendNodeId = 0;
        /** @type {?Array<!Protocol.Runtime.CallFrame>} */
        this.stackTrace = null;
        /** @type {?SDK.TracingModel.ObjectSnapshot} */
        this.picture = null;
        /** @type {?SDK.TracingModel.Event} */
        this._initiator = null;
        this.frameId = '';
        /** @type {number|undefined} */
        this.timeWaitingForMainThread;
    }
    /**
     * @param {!SDK.TracingModel.Event} initiator
     */
    setInitiator(initiator) {
        this._initiator = initiator;
        if (!initiator || this.url) {
            return;
        }
        const initiatorURL = TimelineData.forEvent(initiator).url;
        if (initiatorURL) {
            this.url = initiatorURL;
        }
    }
    /**
     * @return {?SDK.TracingModel.Event}
     */
    initiator() {
        return this._initiator;
    }
    /**
     * @return {?Protocol.Runtime.CallFrame}
     */
    topFrame() {
        const stackTrace = this.stackTraceForSelfOrInitiator();
        return stackTrace && stackTrace[0] || null;
    }
    /**
     * @return {?Array<!Protocol.Runtime.CallFrame>}
     */
    stackTraceForSelfOrInitiator() {
        return this.stackTrace || (this._initiator && TimelineData.forEvent(this._initiator).stackTrace);
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {!TimelineData}
     */
    static forEvent(event) {
        let data = event[TimelineData._symbol];
        if (!data) {
            data = new TimelineData();
            event[TimelineData._symbol] = data;
        }
        return data;
    }
}
TimelineData._symbol = Symbol('timelineData');
/** @typedef {{reason: string, stackTrace: ?Array<!Protocol.Runtime.CallFrame>}} */
let InvalidationCause;
/** @typedef {!{page: !Array<!SDK.TracingModel.Event>, workers: !Array<!SDK.TracingModel.Event>}} */
let MetadataEvents;

var TimelineModel = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TimelineModelImpl: TimelineModelImpl,
    RecordType: RecordType,
    Track: Track,
    TrackType: TrackType,
    PageFrame: PageFrame,
    NetworkRequest: NetworkRequest,
    InvalidationTrackingEvent: InvalidationTrackingEvent,
    InvalidationTracker: InvalidationTracker,
    TimelineAsyncEventTracker: TimelineAsyncEventTracker,
    TimelineData: TimelineData,
    InvalidationCause: InvalidationCause,
    MetadataEvents: MetadataEvents
});

// Copyright 2017 The Chromium Authors. All rights reserved.
class TimelineModelFilter {
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    accept(event) {
        return true;
    }
}
class TimelineVisibleEventsFilter extends TimelineModelFilter {
    /**
     * @param {!Array<string>} visibleTypes
     */
    constructor(visibleTypes) {
        super();
        this._visibleTypes = new Set(visibleTypes);
    }
    /**
     * @override
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    accept(event) {
        return this._visibleTypes.has(TimelineVisibleEventsFilter._eventType(event));
    }
    /**
     * @return {!RecordType}
     */
    static _eventType(event) {
        if (event.hasCategory(TimelineModelImpl.Category.Console)) {
            return RecordType.ConsoleTime;
        }
        if (event.hasCategory(TimelineModelImpl.Category.UserTiming)) {
            return RecordType.UserTiming;
        }
        if (event.hasCategory(TimelineModelImpl.Category.LatencyInfo)) {
            return RecordType.LatencyInfo;
        }
        return /** @type !RecordType */ (event.name);
    }
}
class TimelineInvisibleEventsFilter extends TimelineModelFilter {
    /**
     * @param {!Array<string>} invisibleTypes
     */
    constructor(invisibleTypes) {
        super();
        this._invisibleTypes = new Set(invisibleTypes);
    }
    /**
     * @override
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    accept(event) {
        return !this._invisibleTypes.has(TimelineVisibleEventsFilter._eventType(event));
    }
}
class ExclusiveNameFilter extends TimelineModelFilter {
    /**
     * @param {!Array<string>} excludeNames
     */
    constructor(excludeNames) {
        super();
        this._excludeNames = new Set(excludeNames);
    }
    /**
     * @override
     * @param {!SDK.TracingModel.Event} event
     * @return {boolean}
     */
    accept(event) {
        return !this._excludeNames.has(event.name);
    }
}

var TimelineModelFilter$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TimelineModelFilter: TimelineModelFilter,
    TimelineVisibleEventsFilter: TimelineVisibleEventsFilter,
    TimelineInvisibleEventsFilter: TimelineInvisibleEventsFilter,
    ExclusiveNameFilter: ExclusiveNameFilter
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class TracingLayerTree extends LayerTreeBase.LayerTreeBase {
    /**
     * @param {?SDK.SDKModel.Target} target
     */
    constructor(target) {
        super(target);
        /** @type {!Map.<string, !TracingLayerTile>} */
        this._tileById = new Map();
        this._paintProfilerModel = target && target.model(PaintProfiler.PaintProfilerModel);
    }
    /**
     * @param {?TracingLayerPayload} root
     * @param {?Array<!TracingLayerPayload>} layers
     * @param {!Array<!LayerPaintEvent>} paints
     * @return {!Promise}
     */
    async setLayers(root, layers, paints) {
        const idsToResolve = new Set();
        if (root) {
            // This is a legacy code path for compatibility, as cc is removing
            // layer tree hierarchy, this code will eventually be removed.
            this._extractNodeIdsToResolve(idsToResolve, {}, root);
        }
        else {
            for (let i = 0; i < layers.length; ++i) {
                this._extractNodeIdsToResolve(idsToResolve, {}, layers[i]);
            }
        }
        await this.resolveBackendNodeIds(idsToResolve);
        const oldLayersById = this.layersById;
        this.layersById = new Map();
        this.setContentRoot(null);
        if (root) {
            const convertedLayers = this._innerSetLayers(oldLayersById, root);
            this.setRoot(convertedLayers);
        }
        else {
            const processedLayers = layers.map(this._innerSetLayers.bind(this, oldLayersById));
            const contentRoot = this.contentRoot();
            this.setRoot(contentRoot);
            for (let i = 0; i < processedLayers.length; ++i) {
                if (processedLayers[i].id() !== contentRoot.id()) {
                    contentRoot.addChild(processedLayers[i]);
                }
            }
        }
        this._setPaints(paints);
    }
    /**
     * @param {!Array.<!TracingLayerTile>} tiles
     */
    setTiles(tiles) {
        this._tileById = new Map();
        for (const tile of tiles) {
            this._tileById.set(tile.id, tile);
        }
    }
    /**
     * @param {string} tileId
     * @return {!Promise<?SDK.PaintProfiler.SnapshotWithRect>}
     */
    pictureForRasterTile(tileId) {
        const tile = this._tileById.get('cc::Tile/' + tileId);
        if (!tile) {
            Console.Console.instance().error(`Tile ${tileId} is missing`);
            return /** @type {!Promise<?SDK.PaintProfiler.SnapshotWithRect>} */ (Promise.resolve(null));
        }
        const layer = /** @type {?TracingLayer} */ (this.layerById(tile.layer_id));
        if (!layer) {
            Console.Console.instance().error(`Layer ${tile.layer_id} for tile ${tileId} is not found`);
            return /** @type {!Promise<?SDK.PaintProfiler.SnapshotWithRect>} */ (Promise.resolve(null));
        }
        return layer._pictureForRect(tile.content_rect);
    }
    /**
     * @param {!Array<!LayerPaintEvent>} paints
     */
    _setPaints(paints) {
        for (let i = 0; i < paints.length; ++i) {
            const layer = /** @type {?TracingLayer} */ (this.layersById.get(paints[i].layerId()));
            if (layer) {
                layer._addPaintEvent(paints[i]);
            }
        }
    }
    /**
     * @param {!Map<(string|number), !SDK.LayerTreeBase.Layer>} oldLayersById
     * @param {!TracingLayerPayload} payload
     * @return {!TracingLayer}
     */
    _innerSetLayers(oldLayersById, payload) {
        let layer = /** @type {?TracingLayer} */ (oldLayersById.get(payload.layer_id));
        if (layer) {
            layer._reset(payload);
        }
        else {
            layer = new TracingLayer(this._paintProfilerModel, payload);
        }
        this.layersById.set(payload.layer_id, layer);
        if (payload.owner_node) {
            layer._setNode(this.backendNodeIdToNode().get(payload.owner_node) || null);
        }
        if (!this.contentRoot() && layer.drawsContent()) {
            this.setContentRoot(layer);
        }
        for (let i = 0; payload.children && i < payload.children.length; ++i) {
            layer.addChild(this._innerSetLayers(oldLayersById, payload.children[i]));
        }
        return layer;
    }
    /**
     * @param {!Set<number>} nodeIdsToResolve
     * @param {!Object} seenNodeIds
     * @param {!TracingLayerPayload} payload
     */
    _extractNodeIdsToResolve(nodeIdsToResolve, seenNodeIds, payload) {
        const backendNodeId = payload.owner_node;
        if (backendNodeId && !this.backendNodeIdToNode().has(backendNodeId)) {
            nodeIdsToResolve.add(backendNodeId);
        }
        for (let i = 0; payload.children && i < payload.children.length; ++i) {
            this._extractNodeIdsToResolve(nodeIdsToResolve, seenNodeIds, payload.children[i]);
        }
    }
}
/**
 * @implements {SDK.LayerTreeBase.Layer}
 * @unrestricted
 */
class TracingLayer {
    /**
     * @param {?SDK.PaintProfiler.PaintProfilerModel} paintProfilerModel
     * @param {!TracingLayerPayload} payload
     */
    constructor(paintProfilerModel, payload) {
        this._paintProfilerModel = paintProfilerModel;
        this._reset(payload);
    }
    /**
     * @param {!TracingLayerPayload} payload
     */
    _reset(payload) {
        /** @type {?SDK.DOMModel.DOMNode} */
        this._node = null;
        this._layerId = String(payload.layer_id);
        this._offsetX = payload.position[0];
        this._offsetY = payload.position[1];
        this._width = payload.bounds.width;
        this._height = payload.bounds.height;
        this._children = [];
        this._parentLayerId = null;
        this._parent = null;
        this._quad = payload.layer_quad || [];
        this._createScrollRects(payload);
        // Keep payload.compositing_reasons as a default
        // but use the newer payload.debug_info.compositing_reasons
        // if the first one is not set.
        this._compositingReasonIds =
            payload.compositing_reason_ids || (payload.debug_info && payload.debug_info.compositing_reason_ids) || [];
        this._drawsContent = !!payload.draws_content;
        this._gpuMemoryUsage = payload.gpu_memory_usage;
        this._paints = [];
    }
    /**
     * @override
     * @return {string}
     */
    id() {
        return this._layerId;
    }
    /**
     * @override
     * @return {?string}
     */
    parentId() {
        return this._parentLayerId;
    }
    /**
     * @override
     * @return {?SDK.LayerTreeBase.Layer}
     */
    parent() {
        return this._parent;
    }
    /**
     * @override
     * @return {boolean}
     */
    isRoot() {
        return !this.parentId();
    }
    /**
     * @override
     * @return {!Array.<!SDK.LayerTreeBase.Layer>}
     */
    children() {
        return this._children;
    }
    /**
     * @override
     * @param {!SDK.LayerTreeBase.Layer} childParam
     */
    addChild(childParam) {
        const child = /** @type {!TracingLayer} */ (childParam);
        if (child._parent) {
            console.assert(false, 'Child already has a parent');
        }
        this._children.push(child);
        child._parent = this;
        child._parentLayerId = this._layerId;
    }
    /**
     * @param {?SDK.DOMModel.DOMNode} node
     */
    _setNode(node) {
        this._node = node;
    }
    /**
     * @override
     * @return {?SDK.DOMModel.DOMNode}
     */
    node() {
        return this._node;
    }
    /**
     * @override
     * @return {?SDK.DOMModel.DOMNode}
     */
    nodeForSelfOrAncestor() {
        for (let layer = this; layer; layer = layer._parent) {
            if (layer._node) {
                return layer._node;
            }
        }
        return null;
    }
    /**
     * @override
     * @return {number}
     */
    offsetX() {
        return this._offsetX;
    }
    /**
     * @override
     * @return {number}
     */
    offsetY() {
        return this._offsetY;
    }
    /**
     * @override
     * @return {number}
     */
    width() {
        return this._width;
    }
    /**
     * @override
     * @return {number}
     */
    height() {
        return this._height;
    }
    /**
     * @override
     * @return {?Array.<number>}
     */
    transform() {
        return null;
    }
    /**
     * @override
     * @return {!Array.<number>}
     */
    quad() {
        return this._quad;
    }
    /**
     * @override
     * @return {!Array.<number>}
     */
    anchorPoint() {
        return [0.5, 0.5, 0];
    }
    /**
     * @override
     * @return {boolean}
     */
    invisible() {
        return false;
    }
    /**
     * @override
     * @return {number}
     */
    paintCount() {
        return 0;
    }
    /**
     * @override
     * @return {?Protocol.DOM.Rect}
     */
    lastPaintRect() {
        return null;
    }
    /**
     * @override
     * @return {!Array.<!Protocol.LayerTree.ScrollRect>}
     */
    scrollRects() {
        return this._scrollRects;
    }
    /**
     * @override
     * @return {?SDK.LayerTreeBase.StickyPositionConstraint}
     */
    stickyPositionConstraint() {
        // TODO(smcgruer): Provide sticky layer information in traces.
        return null;
    }
    /**
     * @override
     * @return {number}
     */
    gpuMemoryUsage() {
        return this._gpuMemoryUsage;
    }
    /**
     * @override
     * @return {!Array<!Promise<?SDK.PaintProfiler.SnapshotWithRect>>}
     */
    snapshots() {
        return this._paints.map(paint => paint.snapshotPromise().then(snapshot => {
            if (!snapshot) {
                return null;
            }
            const rect = { x: snapshot.rect[0], y: snapshot.rect[1], width: snapshot.rect[2], height: snapshot.rect[3] };
            return { rect: rect, snapshot: snapshot.snapshot };
        }));
    }
    /**
     * @param {!Array<number>} targetRect
     * @return {!Promise<?SDK.PaintProfiler.SnapshotWithRect>}
     */
    _pictureForRect(targetRect) {
        return Promise.all(this._paints.map(paint => paint.picturePromise())).then(pictures => {
            const fragments = pictures.filter(picture => picture && rectsOverlap(picture.rect, targetRect))
                .map(picture => ({ x: picture.rect[0], y: picture.rect[1], picture: picture.serializedPicture }));
            if (!fragments.length || !this._paintProfilerModel) {
                return null;
            }
            const x0 = fragments.reduce((min, item) => Math.min(min, item.x), Infinity);
            const y0 = fragments.reduce((min, item) => Math.min(min, item.y), Infinity);
            // Rect is in layer content coordinates, make it relative to picture by offsetting to the top left corner.
            const rect = { x: targetRect[0] - x0, y: targetRect[1] - y0, width: targetRect[2], height: targetRect[3] };
            return this._paintProfilerModel.loadSnapshotFromFragments(fragments).then(snapshot => snapshot ? { rect: rect, snapshot: snapshot } : null);
        });
        /**
         * @param {number} a1
         * @param {number} a2
         * @param {number} b1
         * @param {number} b2
         * @return {boolean}
         */
        function segmentsOverlap(a1, a2, b1, b2) {
            console.assert(a1 <= a2 && b1 <= b2, 'segments should be specified as ordered pairs');
            return a2 > b1 && a1 < b2;
        }
        /**
         * @param {!Array.<number>} a
         * @param {!Array.<number>} b
         * @return {boolean}
         */
        function rectsOverlap(a, b) {
            return segmentsOverlap(a[0], a[0] + a[2], b[0], b[0] + b[2]) &&
                segmentsOverlap(a[1], a[1] + a[3], b[1], b[1] + b[3]);
        }
    }
    /**
     * @param {!Array.<number>} params
     * @param {string} type
     * @return {!Object}
     */
    _scrollRectsFromParams(params, type) {
        return { rect: { x: params[0], y: params[1], width: params[2], height: params[3] }, type: type };
    }
    /**
     * @param {!TracingLayerPayload} payload
     */
    _createScrollRects(payload) {
        this._scrollRects = [];
        if (payload.non_fast_scrollable_region) {
            this._scrollRects.push(this._scrollRectsFromParams(payload.non_fast_scrollable_region, LayerTreeBase.Layer.ScrollRectType.NonFastScrollable.name));
        }
        if (payload.touch_event_handler_region) {
            this._scrollRects.push(this._scrollRectsFromParams(payload.touch_event_handler_region, LayerTreeBase.Layer.ScrollRectType.TouchEventHandler.name));
        }
        if (payload.wheel_event_handler_region) {
            this._scrollRects.push(this._scrollRectsFromParams(payload.wheel_event_handler_region, LayerTreeBase.Layer.ScrollRectType.WheelEventHandler.name));
        }
        if (payload.scroll_event_handler_region) {
            this._scrollRects.push(this._scrollRectsFromParams(payload.scroll_event_handler_region, LayerTreeBase.Layer.ScrollRectType.RepaintsOnScroll.name));
        }
    }
    /**
     * @param {!LayerPaintEvent} paint
     */
    _addPaintEvent(paint) {
        this._paints.push(paint);
    }
    /**
     * @override
     * @return {!Promise<!Array<string>>}
     */
    requestCompositingReasonIds() {
        return Promise.resolve(this._compositingReasonIds);
    }
    /**
     * @override
     * @return {boolean}
     */
    drawsContent() {
        return this._drawsContent;
    }
}
/** @typedef {!{
 bounds: {height: number, width: number},
 children: Array.<!TracingLayerPayload>,
 layer_id: number,
 position: Array.<number>,
 scroll_offset: Array.<number>,
 layer_quad: Array.<number>,
 draws_content: number,
 gpu_memory_usage: number,
 transform: Array.<number>,
 owner_node: number,
 compositing_reasons: Array.<string>
}}
*/
let TracingLayerPayload;
/** @typedef {!{
 id: string,
 layer_id: string,
 gpu_memory_usage: number,
 content_rect: !Array.<number>
}}
*/
let TracingLayerTile;

var TracingLayerTree$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TracingLayerTree: TracingLayerTree,
    TracingLayer: TracingLayer,
    TracingLayerPayload: TracingLayerPayload,
    TracingLayerTile: TracingLayerTile
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class TimelineIRModel {
    constructor() {
        this.reset();
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     * @return {!Phases}
     */
    static phaseForEvent(event) {
        return event[TimelineIRModel._eventIRPhase];
    }
    /**
     * @param {?Array<!SDK.TracingModel.AsyncEvent>} inputLatencies
     * @param {?Array<!SDK.TracingModel.AsyncEvent>} animations
     */
    populate(inputLatencies, animations) {
        this.reset();
        if (!inputLatencies) {
            return;
        }
        this._processInputLatencies(inputLatencies);
        if (animations) {
            this._processAnimations(animations);
        }
        const range = new SegmentedRange.SegmentedRange();
        range.appendRange(this._drags); // Drags take lower precedence than animation, as we can't detect them reliably.
        range.appendRange(this._cssAnimations);
        range.appendRange(this._scrolls);
        range.appendRange(this._responses);
        this._segments = range.segments();
    }
    /**
     * @param {!Array<!SDK.TracingModel.AsyncEvent>} events
     */
    _processInputLatencies(events) {
        const eventTypes = InputEvents;
        const phases = Phases;
        const thresholdsMs = TimelineIRModel._mergeThresholdsMs;
        let scrollStart;
        let flingStart;
        let touchStart;
        let firstTouchMove;
        let mouseWheel;
        let mouseDown;
        let mouseMove;
        for (let i = 0; i < events.length; ++i) {
            const event = events[i];
            if (i > 0 && events[i].startTime < events[i - 1].startTime) {
                console.assert(false, 'Unordered input events');
            }
            const type = this._inputEventType(event.name);
            switch (type) {
                case eventTypes.ScrollBegin:
                    this._scrolls.append(this._segmentForEvent(event, phases.Scroll));
                    scrollStart = event;
                    break;
                case eventTypes.ScrollEnd:
                    if (scrollStart) {
                        this._scrolls.append(this._segmentForEventRange(scrollStart, event, phases.Scroll));
                    }
                    else {
                        this._scrolls.append(this._segmentForEvent(event, phases.Scroll));
                    }
                    scrollStart = null;
                    break;
                case eventTypes.ScrollUpdate:
                    touchStart = null; // Since we're scrolling now, disregard other touch gestures.
                    this._scrolls.append(this._segmentForEvent(event, phases.Scroll));
                    break;
                case eventTypes.FlingStart:
                    if (flingStart) {
                        Console.Console.instance().error(UIString.UIString('Two flings at the same time? %s vs %s', flingStart.startTime, event.startTime));
                        break;
                    }
                    flingStart = event;
                    break;
                case eventTypes.FlingCancel:
                    // FIXME: also process renderer fling events.
                    if (!flingStart) {
                        break;
                    }
                    this._scrolls.append(this._segmentForEventRange(flingStart, event, phases.Fling));
                    flingStart = null;
                    break;
                case eventTypes.ImplSideFling:
                    this._scrolls.append(this._segmentForEvent(event, phases.Fling));
                    break;
                case eventTypes.ShowPress:
                case eventTypes.Tap:
                case eventTypes.KeyDown:
                case eventTypes.KeyDownRaw:
                case eventTypes.KeyUp:
                case eventTypes.Char:
                case eventTypes.Click:
                case eventTypes.ContextMenu:
                    this._responses.append(this._segmentForEvent(event, phases.Response));
                    break;
                case eventTypes.TouchStart:
                    // We do not produce any response segment for TouchStart -- there's either going to be one upon
                    // TouchMove for drag, or one for GestureTap.
                    if (touchStart) {
                        Console.Console.instance().error(UIString.UIString('Two touches at the same time? %s vs %s', touchStart.startTime, event.startTime));
                        break;
                    }
                    touchStart = event;
                    event.steps[0][TimelineIRModel._eventIRPhase] = phases.Response;
                    firstTouchMove = null;
                    break;
                case eventTypes.TouchCancel:
                    touchStart = null;
                    break;
                case eventTypes.TouchMove:
                    if (firstTouchMove) {
                        this._drags.append(this._segmentForEvent(event, phases.Drag));
                    }
                    else if (touchStart) {
                        firstTouchMove = event;
                        this._responses.append(this._segmentForEventRange(touchStart, event, phases.Response));
                    }
                    break;
                case eventTypes.TouchEnd:
                    touchStart = null;
                    break;
                case eventTypes.MouseDown:
                    mouseDown = event;
                    mouseMove = null;
                    break;
                case eventTypes.MouseMove:
                    if (mouseDown && !mouseMove && mouseDown.startTime + thresholdsMs.mouse > event.startTime) {
                        this._responses.append(this._segmentForEvent(mouseDown, phases.Response));
                        this._responses.append(this._segmentForEvent(event, phases.Response));
                    }
                    else if (mouseDown) {
                        this._drags.append(this._segmentForEvent(event, phases.Drag));
                    }
                    mouseMove = event;
                    break;
                case eventTypes.MouseUp:
                    this._responses.append(this._segmentForEvent(event, phases.Response));
                    mouseDown = null;
                    break;
                case eventTypes.MouseWheel:
                    // Do not consider first MouseWheel as trace viewer's implementation does -- in case of MouseWheel it's not really special.
                    if (mouseWheel && canMerge(thresholdsMs.mouse, mouseWheel, event)) {
                        this._scrolls.append(this._segmentForEventRange(mouseWheel, event, phases.Scroll));
                    }
                    else {
                        this._scrolls.append(this._segmentForEvent(event, phases.Scroll));
                    }
                    mouseWheel = event;
                    break;
            }
        }
        /**
         * @param {number} threshold
         * @param {!SDK.TracingModel.AsyncEvent} first
         * @param {!SDK.TracingModel.AsyncEvent} second
         * @return {boolean}
         */
        function canMerge(threshold, first, second) {
            return first.endTime < second.startTime && second.startTime < first.endTime + threshold;
        }
    }
    /**
     * @param {!Array<!SDK.TracingModel.AsyncEvent>} events
     */
    _processAnimations(events) {
        for (let i = 0; i < events.length; ++i) {
            this._cssAnimations.append(this._segmentForEvent(events[i], Phases.Animation));
        }
    }
    /**
     * @param {!SDK.TracingModel.AsyncEvent} event
     * @param {!Phases} phase
     * @return {!Common.SegmentedRange.Segment}
     */
    _segmentForEvent(event, phase) {
        this._setPhaseForEvent(event, phase);
        return new SegmentedRange.Segment(event.startTime, event.endTime, phase);
    }
    /**
     * @param {!SDK.TracingModel.AsyncEvent} startEvent
     * @param {!SDK.TracingModel.AsyncEvent} endEvent
     * @param {!Phases} phase
     * @return {!Common.SegmentedRange.Segment}
     */
    _segmentForEventRange(startEvent, endEvent, phase) {
        this._setPhaseForEvent(startEvent, phase);
        this._setPhaseForEvent(endEvent, phase);
        return new SegmentedRange.Segment(startEvent.startTime, endEvent.endTime, phase);
    }
    /**
     * @param {!SDK.TracingModel.AsyncEvent} asyncEvent
     * @param {!Phases} phase
     */
    _setPhaseForEvent(asyncEvent, phase) {
        asyncEvent.steps[0][TimelineIRModel._eventIRPhase] = phase;
    }
    /**
     * @return {!Array<!Common.SegmentedRange.Segment>}
     */
    interactionRecords() {
        return this._segments;
    }
    reset() {
        const thresholdsMs = TimelineIRModel._mergeThresholdsMs;
        this._segments = [];
        this._drags = new SegmentedRange.SegmentedRange(merge.bind(null, thresholdsMs.mouse));
        this._cssAnimations = new SegmentedRange.SegmentedRange(merge.bind(null, thresholdsMs.animation));
        this._responses = new SegmentedRange.SegmentedRange(merge.bind(null, 0));
        this._scrolls = new SegmentedRange.SegmentedRange(merge.bind(null, thresholdsMs.animation));
        /**
         * @param {number} threshold
         * @param {!Common.SegmentedRange.Segment} first
         * @param {!Common.SegmentedRange.Segment} second
         */
        function merge(threshold, first, second) {
            return first.end + threshold >= second.begin && first.data === second.data ? first : null;
        }
    }
    /**
     * @param {string} eventName
     * @return {?InputEvents}
     */
    _inputEventType(eventName) {
        const prefix = 'InputLatency::';
        if (!eventName.startsWith(prefix)) {
            if (eventName === InputEvents.ImplSideFling) {
                return /** @type {!InputEvents} */ (eventName);
            }
            console.error('Unrecognized input latency event: ' + eventName);
            return null;
        }
        return /** @type {!InputEvents} */ (eventName.substr(prefix.length));
    }
}
/**
 * @enum {string}
 */
const Phases = {
    Idle: 'Idle',
    Response: 'Response',
    Scroll: 'Scroll',
    Fling: 'Fling',
    Drag: 'Drag',
    Animation: 'Animation',
    Uncategorized: 'Uncategorized'
};
/**
 * @enum {string}
 */
const InputEvents = {
    Char: 'Char',
    Click: 'GestureClick',
    ContextMenu: 'ContextMenu',
    FlingCancel: 'GestureFlingCancel',
    FlingStart: 'GestureFlingStart',
    ImplSideFling: RecordType.ImplSideFling,
    KeyDown: 'KeyDown',
    KeyDownRaw: 'RawKeyDown',
    KeyUp: 'KeyUp',
    LatencyScrollUpdate: 'ScrollUpdate',
    MouseDown: 'MouseDown',
    MouseMove: 'MouseMove',
    MouseUp: 'MouseUp',
    MouseWheel: 'MouseWheel',
    PinchBegin: 'GesturePinchBegin',
    PinchEnd: 'GesturePinchEnd',
    PinchUpdate: 'GesturePinchUpdate',
    ScrollBegin: 'GestureScrollBegin',
    ScrollEnd: 'GestureScrollEnd',
    ScrollUpdate: 'GestureScrollUpdate',
    ScrollUpdateRenderer: 'ScrollUpdate',
    ShowPress: 'GestureShowPress',
    Tap: 'GestureTap',
    TapCancel: 'GestureTapCancel',
    TapDown: 'GestureTapDown',
    TouchCancel: 'TouchCancel',
    TouchEnd: 'TouchEnd',
    TouchMove: 'TouchMove',
    TouchStart: 'TouchStart'
};
TimelineIRModel._mergeThresholdsMs = {
    animation: 1,
    mouse: 40,
};
TimelineIRModel._eventIRPhase = Symbol('eventIRPhase');

var TimelineIRModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TimelineIRModel: TimelineIRModel,
    Phases: Phases,
    InputEvents: InputEvents
});

/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @unrestricted
 */
class TimelineFrameModel {
    /**
     * @param {function(!SDK.TracingModel.Event):string} categoryMapper
     */
    constructor(categoryMapper) {
        this._categoryMapper = categoryMapper;
        /** @type {!Array<!TimelineFrame>} */
        this._frames;
        /** @type {!Object<number, !TimelineFrame>} */
        this._frameById;
        /** @type {number} */
        this._minimumRecordTime;
        /** @type {?TimelineFrame} */
        this._lastFrame;
        /** @type {boolean} */
        this._mainFrameCommitted;
        /** @type {boolean} */
        this._mainFrameRequested;
        /** @type {number} */
        this._minimumRecordTime;
        /** @type {?TracingFrameLayerTree} */
        this._lastLayerTree;
        /** @type {?PendingFrame} */
        this._framePendingActivation;
        /** @type {!Object<string, number>} */
        this._currentTaskTimeByCategory;
        /** @type {?SDK.SDKModel.Target} */
        this._target;
        this.reset();
    }
    /**
     * @param {number=} startTime
     * @param {number=} endTime
     * @return {!Array<!TimelineFrame>}
     */
    frames(startTime, endTime) {
        if (!startTime && !endTime) {
            return this._frames;
        }
        const firstFrame = this._frames.lowerBound(startTime || 0, (time, frame) => time - frame.endTime);
        const lastFrame = this._frames.lowerBound(endTime || Infinity, (time, frame) => time - frame.startTime);
        return this._frames.slice(firstFrame, lastFrame);
    }
    /**
     * @param {!SDK.TracingModel.Event} rasterTask
     * @return {boolean}
     */
    hasRasterTile(rasterTask) {
        const data = rasterTask.args['tileData'];
        if (!data) {
            return false;
        }
        const frameId = data['sourceFrameNumber'];
        const frame = frameId && this._frameById[frameId];
        if (!frame || !frame.layerTree) {
            return false;
        }
        return true;
    }
    /**
     * @param {!SDK.TracingModel.Event} rasterTask
     * @return {!Promise<?{rect: !Protocol.DOM.Rect, snapshot: !SDK.PaintProfiler.PaintProfilerSnapshot}>}
     */
    rasterTilePromise(rasterTask) {
        if (!this._target) {
            return Promise.resolve(null);
        }
        const data = rasterTask.args['tileData'];
        const frameId = /** @type {number} */ (data['sourceFrameNumber']);
        const tileId = data['tileId'] && data['tileId']['id_ref'];
        const frame = frameId && this._frameById[frameId];
        if (!frame || !frame.layerTree || !tileId) {
            return Promise.resolve(null);
        }
        return frame.layerTree.layerTreePromise().then(layerTree => layerTree && layerTree.pictureForRasterTile(tileId));
    }
    reset() {
        this._minimumRecordTime = Infinity;
        this._frames = [];
        this._frameById = {};
        this._lastFrame = null;
        this._lastLayerTree = null;
        this._mainFrameCommitted = false;
        this._mainFrameRequested = false;
        this._framePendingCommit = null;
        this._lastBeginFrame = null;
        this._lastNeedsBeginFrame = null;
        this._framePendingActivation = null;
        this._lastTaskBeginTime = null;
        this._target = null;
        this._layerTreeId = null;
        this._currentTaskTimeByCategory = {};
    }
    /**
     * @param {number} startTime
     */
    handleBeginFrame(startTime) {
        if (!this._lastFrame) {
            this._startFrame(startTime);
        }
        this._lastBeginFrame = startTime;
    }
    /**
     * @param {number} startTime
     */
    handleDrawFrame(startTime) {
        if (!this._lastFrame) {
            this._startFrame(startTime);
            return;
        }
        // - if it wasn't drawn, it didn't happen!
        // - only show frames that either did not wait for the main thread frame or had one committed.
        if (this._mainFrameCommitted || !this._mainFrameRequested) {
            if (this._lastNeedsBeginFrame) {
                const idleTimeEnd = this._framePendingActivation ? this._framePendingActivation.triggerTime :
                    (this._lastBeginFrame || this._lastNeedsBeginFrame);
                if (idleTimeEnd > this._lastFrame.startTime) {
                    this._lastFrame.idle = true;
                    this._startFrame(idleTimeEnd);
                    if (this._framePendingActivation) {
                        this._commitPendingFrame();
                    }
                    this._lastBeginFrame = null;
                }
                this._lastNeedsBeginFrame = null;
            }
            this._startFrame(startTime);
        }
        this._mainFrameCommitted = false;
    }
    handleActivateLayerTree() {
        if (!this._lastFrame) {
            return;
        }
        if (this._framePendingActivation && !this._lastNeedsBeginFrame) {
            this._commitPendingFrame();
        }
    }
    handleRequestMainThreadFrame() {
        if (!this._lastFrame) {
            return;
        }
        this._mainFrameRequested = true;
    }
    handleCompositeLayers() {
        if (!this._framePendingCommit) {
            return;
        }
        this._framePendingActivation = this._framePendingCommit;
        this._framePendingCommit = null;
        this._mainFrameRequested = false;
        this._mainFrameCommitted = true;
    }
    /**
     * @param {!TracingFrameLayerTree} layerTree
     */
    handleLayerTreeSnapshot(layerTree) {
        this._lastLayerTree = layerTree;
    }
    /**
     * @param {number} startTime
     * @param {boolean} needsBeginFrame
     */
    handleNeedFrameChanged(startTime, needsBeginFrame) {
        if (needsBeginFrame) {
            this._lastNeedsBeginFrame = startTime;
        }
    }
    /**
     * @param {number} startTime
     */
    _startFrame(startTime) {
        if (this._lastFrame) {
            this._flushFrame(this._lastFrame, startTime);
        }
        this._lastFrame = new TimelineFrame(startTime, startTime - this._minimumRecordTime);
    }
    /**
     * @param {!TimelineFrame} frame
     * @param {number} endTime
     */
    _flushFrame(frame, endTime) {
        frame._setLayerTree(this._lastLayerTree);
        frame._setEndTime(endTime);
        if (this._lastLayerTree) {
            this._lastLayerTree._setPaints(frame._paints);
        }
        const lastFrame = this._frames.peekLast();
        if (this._frames.length && lastFrame &&
            (frame.startTime !== lastFrame.endTime || frame.startTime > frame.endTime)) {
            console.assert(false, `Inconsistent frame time for frame ${this._frames.length} (${frame.startTime} - ${frame.endTime})`);
        }
        this._frames.push(frame);
        if (typeof frame._mainFrameId === 'number') {
            this._frameById[frame._mainFrameId] = frame;
        }
    }
    _commitPendingFrame() {
        if (!this._framePendingActivation || !this._lastFrame) {
            return;
        }
        this._lastFrame._addTimeForCategories(this._framePendingActivation.timeByCategory);
        this._lastFrame._paints = this._framePendingActivation.paints;
        this._lastFrame._mainFrameId = this._framePendingActivation.mainFrameId;
        this._framePendingActivation = null;
    }
    /**
     * @param {?SDK.SDKModel.Target} target
     * @param {!Array.<!SDK.TracingModel.Event>} events
     * @param {!Array<!{thread: !SDK.TracingModel.Thread, time: number}>} threadData
     */
    addTraceEvents(target, events, threadData) {
        this._target = target;
        let j = 0;
        this._currentProcessMainThread = threadData.length && threadData[0].thread || null;
        for (let i = 0; i < events.length; ++i) {
            while (j + 1 < threadData.length && threadData[j + 1].time <= events[i].startTime) {
                this._currentProcessMainThread = threadData[++j].thread;
            }
            this._addTraceEvent(events[i]);
        }
        this._currentProcessMainThread = null;
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    _addTraceEvent(event) {
        const eventNames = RecordType;
        if (event.startTime && event.startTime < this._minimumRecordTime) {
            this._minimumRecordTime = event.startTime;
        }
        if (event.name === eventNames.SetLayerTreeId) {
            this._layerTreeId = event.args['layerTreeId'] || event.args['data']['layerTreeId'];
        }
        else if (event.id && event.phase === TracingModel.Phase.SnapshotObject &&
            event.name === eventNames.LayerTreeHostImplSnapshot && parseInt(event.id, 0) === this._layerTreeId &&
            this._target) {
            const snapshot = /** @type {!SDK.TracingModel.ObjectSnapshot} */ (event);
            this.handleLayerTreeSnapshot(new TracingFrameLayerTree(this._target, snapshot));
        }
        else {
            this._processCompositorEvents(event);
            if (event.thread === this._currentProcessMainThread) {
                this._addMainThreadTraceEvent(event);
            }
            else if (this._lastFrame && event.selfTime && !TracingModel.TracingModel.isTopLevelEvent(event)) {
                this._lastFrame._addTimeForCategory(this._categoryMapper(event), event.selfTime);
            }
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    _processCompositorEvents(event) {
        const eventNames = RecordType;
        if (event.args['layerTreeId'] !== this._layerTreeId) {
            return;
        }
        const timestamp = event.startTime;
        if (event.name === eventNames.BeginFrame) {
            this.handleBeginFrame(timestamp);
        }
        else if (event.name === eventNames.DrawFrame) {
            this.handleDrawFrame(timestamp);
        }
        else if (event.name === eventNames.ActivateLayerTree) {
            this.handleActivateLayerTree();
        }
        else if (event.name === eventNames.RequestMainThreadFrame) {
            this.handleRequestMainThreadFrame();
        }
        else if (event.name === eventNames.NeedsBeginFrameChanged) {
            this.handleNeedFrameChanged(timestamp, event.args['data'] && event.args['data']['needsBeginFrame']);
        }
    }
    /**
     * @param {!SDK.TracingModel.Event} event
     */
    _addMainThreadTraceEvent(event) {
        const eventNames = RecordType;
        if (TracingModel.TracingModel.isTopLevelEvent(event)) {
            this._currentTaskTimeByCategory = {};
            this._lastTaskBeginTime = event.startTime;
        }
        if (!this._framePendingCommit && TimelineFrameModel._mainFrameMarkers.indexOf(event.name) >= 0) {
            this._framePendingCommit =
                new PendingFrame(this._lastTaskBeginTime || event.startTime, this._currentTaskTimeByCategory);
        }
        if (!this._framePendingCommit) {
            this._addTimeForCategory(this._currentTaskTimeByCategory, event);
            return;
        }
        this._addTimeForCategory(this._framePendingCommit.timeByCategory, event);
        if (event.name === eventNames.BeginMainThreadFrame && event.args['data'] && event.args['data']['frameId']) {
            this._framePendingCommit.mainFrameId = event.args['data']['frameId'];
        }
        if (event.name === eventNames.Paint && event.args['data']['layerId'] && TimelineData.forEvent(event).picture &&
            this._target) {
            this._framePendingCommit.paints.push(new LayerPaintEvent(event, this._target));
        }
        if (event.name === eventNames.CompositeLayers && event.args['layerTreeId'] === this._layerTreeId) {
            this.handleCompositeLayers();
        }
    }
    /**
     * @param {!Object.<string, number>} timeByCategory
     * @param {!SDK.TracingModel.Event} event
     */
    _addTimeForCategory(timeByCategory, event) {
        if (!event.selfTime) {
            return;
        }
        const categoryName = this._categoryMapper(event);
        timeByCategory[categoryName] = (timeByCategory[categoryName] || 0) + event.selfTime;
    }
}
TimelineFrameModel._mainFrameMarkers = [
    RecordType.ScheduleStyleRecalculation, RecordType.InvalidateLayout, RecordType.BeginMainThreadFrame,
    RecordType.ScrollLayer
];
/**
 * @unrestricted
 */
class TracingFrameLayerTree {
    /**
     * @param {!SDK.SDKModel.Target} target
     * @param {!SDK.TracingModel.ObjectSnapshot} snapshot
     */
    constructor(target, snapshot) {
        this._target = target;
        this._snapshot = snapshot;
        /** @type {!Array<!LayerPaintEvent>|undefined} */
        this._paints;
    }
    /**
     * @return {!Promise<?TracingLayerTree>}
     */
    async layerTreePromise() {
        const result = /** @type {!Object<string, *>} */ (await this._snapshot.objectPromise());
        if (!result) {
            return null;
        }
        const viewport = /** @type {!{width: number, height: number}} */ (result['device_viewport_size']);
        const tiles = /** @type {!Array<!TracingLayerTile>} */ (result['active_tiles']);
        const rootLayer = result['active_tree']['root_layer'];
        const layers = result['active_tree']['layers'];
        const layerTree = new TracingLayerTree(this._target);
        layerTree.setViewportSize(viewport);
        layerTree.setTiles(tiles);
        await layerTree.setLayers(rootLayer, layers, this._paints || []);
        return layerTree;
    }
    /**
     * @return {!Array<!LayerPaintEvent>}
     */
    paints() {
        return this._paints || [];
    }
    /**
     * @param {!Array<!LayerPaintEvent>} paints
     */
    _setPaints(paints) {
        this._paints = paints;
    }
}
/**
 * @unrestricted
 */
class TimelineFrame {
    /**
     * @param {number} startTime
     * @param {number} startTimeOffset
     */
    constructor(startTime, startTimeOffset) {
        this.startTime = startTime;
        this.startTimeOffset = startTimeOffset;
        this.endTime = this.startTime;
        this.duration = 0;
        /** @type {!Object<string, number>} */
        this.timeByCategory = {};
        this.cpuTime = 0;
        this.idle = false;
        /** @type {?TracingFrameLayerTree} */
        this.layerTree = null;
        /** @type {!Array.<!LayerPaintEvent>} */
        this._paints = [];
        /** @type {number|undefined} */
        this._mainFrameId = undefined;
    }
    /**
     * @return {boolean}
     */
    hasWarnings() {
        return false;
    }
    /**
     * @param {number} endTime
     */
    _setEndTime(endTime) {
        this.endTime = endTime;
        this.duration = this.endTime - this.startTime;
    }
    /**
     * @param {?TracingFrameLayerTree} layerTree
     */
    _setLayerTree(layerTree) {
        this.layerTree = layerTree;
    }
    /**
     * @param {!Object<string, number>} timeByCategory
     */
    _addTimeForCategories(timeByCategory) {
        for (const category in timeByCategory) {
            this._addTimeForCategory(category, timeByCategory[category]);
        }
    }
    /**
     * @param {string} category
     * @param {number} time
     */
    _addTimeForCategory(category, time) {
        this.timeByCategory[category] = (this.timeByCategory[category] || 0) + time;
        this.cpuTime += time;
    }
}
/**
 * @unrestricted
 */
class LayerPaintEvent {
    /**
     * @param {!SDK.TracingModel.Event} event
     * @param {?SDK.SDKModel.Target} target
     */
    constructor(event, target) {
        this._event = event;
        this._target = target;
    }
    /**
     * @return {string}
     */
    layerId() {
        return this._event.args['data']['layerId'];
    }
    /**
     * @return {!SDK.TracingModel.Event}
     */
    event() {
        return this._event;
    }
    /**
     * @return {!Promise<?{rect: !Array<number>, serializedPicture: string}>}
     */
    picturePromise() {
        const picture = TimelineData.forEvent(this._event).picture;
        if (!picture) {
            return Promise.resolve(null);
        }
        return picture.objectPromise().then(
        /**
       * @param {*} result
       */
        result => {
            if (!result) {
                return null;
            }
            const rect = result['params'] && result['params']['layer_rect'];
            const picture = result['skp64'];
            return rect && picture ? { rect: rect, serializedPicture: picture } : null;
        });
    }
    /**
     * @return !Promise<?{rect: !Array<number>, snapshot: !SDK.PaintProfiler.PaintProfilerSnapshot}>}
     */
    snapshotPromise() {
        const paintProfilerModel = this._target && this._target.model(PaintProfiler.PaintProfilerModel);
        return this.picturePromise().then(picture => {
            if (!picture || !paintProfilerModel) {
                return null;
            }
            return paintProfilerModel.loadSnapshot(picture.serializedPicture)
                .then(
            /**
           * @param {!SDK.PaintProfiler.PaintProfilerModel} snapshot
           */
            snapshot => snapshot ? { rect: picture.rect, snapshot: snapshot } : null);
        });
    }
}
/**
 * @unrestricted
 */
class PendingFrame {
    /**
     * @param {number} triggerTime
     * @param {!Object.<string, number>} timeByCategory
     */
    constructor(triggerTime, timeByCategory) {
        /** @type {!Object.<string, number>} */
        this.timeByCategory = timeByCategory;
        /** @type {!Array.<!LayerPaintEvent>} */
        this.paints = [];
        /** @type {number|undefined} */
        this.mainFrameId = undefined;
        this.triggerTime = triggerTime;
    }
}

var TimelineFrameModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TimelineFrameModel: TimelineFrameModel,
    TracingFrameLayerTree: TracingFrameLayerTree,
    TimelineFrame: TimelineFrame,
    LayerPaintEvent: LayerPaintEvent,
    PendingFrame: PendingFrame
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class Node {
    /**
     * @param {string|symbol} id
     * @param {?SDK.TracingModel.Event} event
     */
    constructor(id, event) {
        /** @type {number} */
        this.totalTime = 0;
        /** @type {number} */
        this.selfTime = 0;
        /** @type {string|symbol} */
        this.id = id;
        /** @type {?SDK.TracingModel.Event} */
        this.event = event;
        /** @type {?Node} */
        this.parent;
        /** @type {string} */
        this._groupId = '';
        this._isGroupNode = false;
    }
    /**
     * @return {boolean}
     */
    isGroupNode() {
        return this._isGroupNode;
    }
    /**
     * @return {boolean}
     */
    hasChildren() {
        throw 'Not implemented';
    }
    /**
     * @return {!ChildrenCache}
     */
    children() {
        throw 'Not implemented';
    }
    /**
     * @param {function(!SDK.TracingModel.Event):boolean} matchFunction
     * @param {!Array<!Node>=} results
     * @return {!Array<!Node>}
     */
    searchTree(matchFunction, results) {
        results = results || [];
        if (this.event && matchFunction(this.event)) {
            results.push(this);
        }
        for (const child of this.children().values()) {
            child.searchTree(matchFunction, results);
        }
        return results;
    }
}
class TopDownNode extends Node {
    /**
     * @param {string|symbol} id
     * @param {?SDK.TracingModel.Event} event
     * @param {?TopDownNode} parent
     */
    constructor(id, event, parent) {
        super(id, event);
        /** @type {?TopDownRootNode} */
        this._root = parent && parent._root;
        this._hasChildren = false;
        this._children = null;
        this.parent = parent;
    }
    /**
     * @override
     * @return {boolean}
     */
    hasChildren() {
        return this._hasChildren;
    }
    /**
     * @override
     * @return {!ChildrenCache}
     */
    children() {
        return this._children || this._buildChildren();
    }
    /**
     * @return {!ChildrenCache}
     */
    _buildChildren() {
        /** @type {!Array<!TopDownNode>} */
        const path = [];
        for (let node = this; node.parent && !node._isGroupNode; node = node.parent) {
            path.push(/** @type {!TopDownNode} */ (node));
        }
        path.reverse();
        /** @type {!ChildrenCache} */
        const children = new Map();
        const self = this;
        const root = this._root;
        const startTime = root._startTime;
        const endTime = root._endTime;
        const instantEventCallback = root._doNotAggregate ? onInstantEvent : undefined;
        const eventIdCallback = root._doNotAggregate ? undefined : _eventId;
        const eventGroupIdCallback = root._eventGroupIdCallback;
        let depth = 0;
        let matchedDepth = 0;
        let currentDirectChild = null;
        TimelineModelImpl.forEachEvent(root._events, onStartEvent, onEndEvent, instantEventCallback, startTime, endTime, root._filter);
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onStartEvent(e) {
            ++depth;
            if (depth > path.length + 2) {
                return;
            }
            if (!matchPath(e)) {
                return;
            }
            const duration = Math.min(endTime, e.endTime) - Math.max(startTime, e.startTime);
            if (duration < 0) {
                console.error('Negative event duration');
            }
            processEvent(e, duration);
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onInstantEvent(e) {
            ++depth;
            if (matchedDepth === path.length && depth <= path.length + 2) {
                processEvent(e, 0);
            }
            --depth;
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         * @param {number} duration
         */
        function processEvent(e, duration) {
            if (depth === path.length + 2) {
                currentDirectChild._hasChildren = true;
                currentDirectChild.selfTime -= duration;
                return;
            }
            let id;
            let groupId = '';
            if (!eventIdCallback) {
                id = Symbol('uniqueId');
            }
            else {
                id = eventIdCallback(e);
                groupId = eventGroupIdCallback ? eventGroupIdCallback(e) : '';
                if (groupId) {
                    id += '/' + groupId;
                }
            }
            let node = children.get(id);
            if (!node) {
                node = new TopDownNode(id, e, self);
                node._groupId = groupId;
                children.set(id, node);
            }
            node.selfTime += duration;
            node.totalTime += duration;
            currentDirectChild = node;
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         * @return {boolean}
         */
        function matchPath(e) {
            if (matchedDepth === path.length) {
                return true;
            }
            if (matchedDepth !== depth - 1) {
                return false;
            }
            if (!e.endTime) {
                return false;
            }
            if (!eventIdCallback) {
                if (e === path[matchedDepth].event) {
                    ++matchedDepth;
                }
                return false;
            }
            let id = eventIdCallback(e);
            const groupId = eventGroupIdCallback ? eventGroupIdCallback(e) : '';
            if (groupId) {
                id += '/' + groupId;
            }
            if (id === path[matchedDepth].id) {
                ++matchedDepth;
            }
            return false;
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onEndEvent(e) {
            --depth;
            if (matchedDepth > depth) {
                matchedDepth = depth;
            }
        }
        this._children = children;
        return children;
    }
}
class TopDownRootNode extends TopDownNode {
    /**
     * @param {!Array<!SDK.TracingModel.Event>} events
     * @param {!Array<!TimelineModelFilter>} filters
     * @param {number} startTime
     * @param {number} endTime
     * @param {boolean=} doNotAggregate
     * @param {?function(!SDK.TracingModel.Event):string=} eventGroupIdCallback
     */
    constructor(events, filters, startTime, endTime, doNotAggregate, eventGroupIdCallback) {
        super('', null, null);
        this._root = this;
        this._events = events;
        this._filter = e => filters.every(f => f.accept(e));
        this._startTime = startTime;
        this._endTime = endTime;
        this._eventGroupIdCallback = eventGroupIdCallback;
        this._doNotAggregate = doNotAggregate;
        this.totalTime = endTime - startTime;
        this.selfTime = this.totalTime;
    }
    /**
     * @override
     * @return {!ChildrenCache}
     */
    children() {
        return this._children || this._grouppedTopNodes();
    }
    /**
     * @return {!ChildrenCache}
     */
    _grouppedTopNodes() {
        const flatNodes = super.children();
        for (const node of flatNodes.values()) {
            this.selfTime -= node.totalTime;
        }
        if (!this._eventGroupIdCallback) {
            return flatNodes;
        }
        const groupNodes = new Map();
        for (const node of flatNodes.values()) {
            const groupId = this._eventGroupIdCallback(/** @type {!SDK.TracingModel.Event} */ (node.event));
            let groupNode = groupNodes.get(groupId);
            if (!groupNode) {
                groupNode = new GroupNode(groupId, this, /** @type {!SDK.TracingModel.Event} */ (node.event));
                groupNodes.set(groupId, groupNode);
            }
            groupNode.addChild(node, node.selfTime, node.totalTime);
        }
        this._children = groupNodes;
        return groupNodes;
    }
}
class BottomUpRootNode extends Node {
    /**
     * @param {!Array<!SDK.TracingModel.Event>} events
     * @param {!TimelineModelFilter} textFilter
     * @param {!Array<!TimelineModel.TimelineModelFilter>} filters
     * @param {number} startTime
     * @param {number} endTime
     * @param {?function(!SDK.TracingModel.Event):string} eventGroupIdCallback
     */
    constructor(events, textFilter, filters, startTime, endTime, eventGroupIdCallback) {
        super('', null);
        /** @type {?ChildrenCache} */
        this._children = null;
        this._events = events;
        this._textFilter = textFilter;
        this._filter = e => filters.every(f => f.accept(e));
        this._startTime = startTime;
        this._endTime = endTime;
        this._eventGroupIdCallback = eventGroupIdCallback;
        this.totalTime = endTime - startTime;
    }
    /**
     * @override
     * @return {boolean}
     */
    hasChildren() {
        return true;
    }
    /**
     * @param {!ChildrenCache} children
     * @return {!ChildrenCache}
     */
    _filterChildren(children) {
        for (const [id, child] of children) {
            if (child.event && !this._textFilter.accept(child.event)) {
                children.delete(/** @type {string|symbol} */ (id));
            }
        }
        return children;
    }
    /**
     * @override
     * @return {!ChildrenCache}
     */
    children() {
        if (!this._children) {
            this._children = this._filterChildren(this._grouppedTopNodes());
        }
        return this._children;
    }
    /**
     * @return {!ChildrenCache}
     */
    _ungrouppedTopNodes() {
        const root = this;
        const startTime = this._startTime;
        const endTime = this._endTime;
        /** @type {!ChildrenCache} */
        const nodeById = new Map();
        /** @type {!Array<number>} */
        const selfTimeStack = [endTime - startTime];
        /** @type {!Array<boolean>} */
        const firstNodeStack = [];
        /** @type {!Map<string, number>} */
        const totalTimeById = new Map();
        TimelineModelImpl.forEachEvent(this._events, onStartEvent, onEndEvent, undefined, startTime, endTime, this._filter);
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onStartEvent(e) {
            const duration = Math.min(e.endTime, endTime) - Math.max(e.startTime, startTime);
            selfTimeStack[selfTimeStack.length - 1] -= duration;
            selfTimeStack.push(duration);
            const id = _eventId(e);
            const noNodeOnStack = !totalTimeById.has(id);
            if (noNodeOnStack) {
                totalTimeById.set(id, duration);
            }
            firstNodeStack.push(noNodeOnStack);
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onEndEvent(e) {
            const id = _eventId(e);
            let node = nodeById.get(id);
            if (!node) {
                node = new BottomUpNode(root, id, e, false, root);
                nodeById.set(id, node);
            }
            node.selfTime += selfTimeStack.pop();
            if (firstNodeStack.pop()) {
                node.totalTime += totalTimeById.get(id);
                totalTimeById.delete(id);
            }
            if (firstNodeStack.length) {
                node.setHasChildren();
            }
        }
        this.selfTime = selfTimeStack.pop();
        for (const pair of nodeById) {
            if (pair[1].selfTime <= 0) {
                nodeById.delete(/** @type {string} */ (pair[0]));
            }
        }
        return nodeById;
    }
    /**
     * @return {!ChildrenCache}
     */
    _grouppedTopNodes() {
        const flatNodes = this._ungrouppedTopNodes();
        if (!this._eventGroupIdCallback) {
            return flatNodes;
        }
        const groupNodes = new Map();
        for (const node of flatNodes.values()) {
            const groupId = this._eventGroupIdCallback(/** @type {!SDK.TracingModel.Event} */ (node.event));
            let groupNode = groupNodes.get(groupId);
            if (!groupNode) {
                groupNode = new GroupNode(groupId, this, /** @type {!SDK.TracingModel.Event} */ (node.event));
                groupNodes.set(groupId, groupNode);
            }
            groupNode.addChild(node, node.selfTime, node.selfTime);
        }
        return groupNodes;
    }
}
class GroupNode extends Node {
    /**
     * @param {string} id
     * @param {!BottomUpRootNode|!TopDownRootNode} parent
     * @param {!SDK.TracingModel.Event} event
     */
    constructor(id, parent, event) {
        super(id, event);
        this._children = new Map();
        this.parent = parent;
        this._isGroupNode = true;
    }
    /**
     * @param {!BottomUpNode} child
     * @param {number} selfTime
     * @param {number} totalTime
     */
    addChild(child, selfTime, totalTime) {
        this._children.set(child.id, child);
        this.selfTime += selfTime;
        this.totalTime += totalTime;
        child.parent = this;
    }
    /**
     * @override
     * @return {boolean}
     */
    hasChildren() {
        return true;
    }
    /**
     * @override
     * @return {!ChildrenCache}
     */
    children() {
        return this._children;
    }
}
class BottomUpNode extends Node {
    /**
     * @param {!BottomUpRootNode} root
     * @param {string} id
     * @param {!SDK.TracingModel.Event} event
     * @param {boolean} hasChildren
     * @param {!Node} parent
     */
    constructor(root, id, event, hasChildren, parent) {
        super(id, event);
        this.parent = parent;
        this._root = root;
        this._depth = (parent._depth || 0) + 1;
        /** @type {?ChildrenCache} */
        this._cachedChildren = null;
        this._hasChildren = hasChildren;
    }
    setHasChildren() {
        this._hasChildren = true;
    }
    /**
     * @override
     * @return {boolean}
     */
    hasChildren() {
        return this._hasChildren;
    }
    /**
     * @override
     * @return {!ChildrenCache}
     */
    children() {
        if (this._cachedChildren) {
            return this._cachedChildren;
        }
        /** @type {!Array<number>} */
        const selfTimeStack = [0];
        /** @type {!Array<string>} */
        const eventIdStack = [];
        /** @type {!Array<!SDK.TracingModel.Event>} */
        const eventStack = [];
        /** @type {!ChildrenCache} */
        const nodeById = new Map();
        const startTime = this._root._startTime;
        const endTime = this._root._endTime;
        let lastTimeMarker = startTime;
        const self = this;
        TimelineModelImpl.forEachEvent(this._root._events, onStartEvent, onEndEvent, undefined, startTime, endTime, this._root._filter);
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onStartEvent(e) {
            const duration = Math.min(e.endTime, endTime) - Math.max(e.startTime, startTime);
            if (duration < 0) {
                console.assert(false, 'Negative duration of an event');
            }
            selfTimeStack[selfTimeStack.length - 1] -= duration;
            selfTimeStack.push(duration);
            const id = _eventId(e);
            eventIdStack.push(id);
            eventStack.push(e);
        }
        /**
         * @param {!SDK.TracingModel.Event} e
         */
        function onEndEvent(e) {
            const selfTime = selfTimeStack.pop();
            const id = eventIdStack.pop();
            eventStack.pop();
            let node;
            for (node = self; node._depth > 1; node = node.parent) {
                if (node.id !== eventIdStack[eventIdStack.length + 1 - node._depth]) {
                    return;
                }
            }
            if (node.id !== id || eventIdStack.length < self._depth) {
                return;
            }
            const childId = eventIdStack[eventIdStack.length - self._depth];
            node = nodeById.get(childId);
            if (!node) {
                const event = eventStack[eventStack.length - self._depth];
                const hasChildren = eventStack.length > self._depth;
                node = new BottomUpNode(self._root, childId, event, hasChildren, self);
                nodeById.set(childId, node);
            }
            const totalTime = Math.min(e.endTime, endTime) - Math.max(e.startTime, lastTimeMarker);
            node.selfTime += selfTime;
            node.totalTime += totalTime;
            lastTimeMarker = Math.min(e.endTime, endTime);
        }
        this._cachedChildren = this._root._filterChildren(nodeById);
        return this._cachedChildren;
    }
    /**
     * @override
     * @param {function(!SDK.TracingModel.Event):boolean} matchFunction
     * @param {!Array<!Node>=} results
     * @return {!Array<!Node>}
     */
    searchTree(matchFunction, results) {
        results = results || [];
        if (this.event && matchFunction(this.event)) {
            results.push(this);
        }
        return results;
    }
}
/**
 * @param {!SDK.TracingModel.Event} event
 * @return {?string}
 */
function eventURL(event) {
    const data = event.args['data'] || event.args['beginData'];
    if (data && data['url']) {
        return data['url'];
    }
    let frame = eventStackFrame(event);
    while (frame) {
        const url = frame['url'];
        if (url) {
            return url;
        }
        frame = frame.parent;
    }
    return null;
}
/**
 * @param {!SDK.TracingModel.Event} event
 * @return {?Protocol.Runtime.CallFrame}
 */
function eventStackFrame(event) {
    if (event.name === RecordType.JSFrame) {
        return /** @type {?Protocol.Runtime.CallFrame} */ (event.args['data'] || null);
    }
    return TimelineData.forEvent(event).topFrame();
}
/**
 * @param {!SDK.TracingModel.Event} event
 * @return {string}
 */
function _eventId(event) {
    if (event.name === RecordType.TimeStamp) {
        return `${event.name}:${event.args.data.message}`;
    }
    if (event.name !== RecordType.JSFrame) {
        return event.name;
    }
    const frame = event.args['data'];
    const location = frame['scriptId'] || frame['url'] || '';
    const functionName = frame['functionName'];
    const name = TimelineJSProfileProcessor.isNativeRuntimeFrame(frame) ?
        TimelineJSProfileProcessor.nativeGroup(functionName) || functionName :
        `${functionName}:${frame['lineNumber']}:${frame['columnNumber']}`;
    return `f:${name}@${location}`;
}
/**
 * @typedef {Map<string|symbol, !Node>}
 */
let ChildrenCache;

var TimelineProfileTree = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Node: Node,
    TopDownNode: TopDownNode,
    TopDownRootNode: TopDownRootNode,
    BottomUpRootNode: BottomUpRootNode,
    GroupNode: GroupNode,
    BottomUpNode: BottomUpNode,
    eventURL: eventURL,
    eventStackFrame: eventStackFrame,
    _eventId: _eventId,
    ChildrenCache: ChildrenCache
});

// Copyright 2019 The Chromium Authors. All rights reserved.

export { TimelineFrameModel$1 as TimelineFrameModel, TimelineIRModel$1 as TimelineIRModel, TimelineJSProfile, TimelineModel, TimelineModelFilter$1 as TimelineModelFilter, TimelineProfileTree, TracingLayerTree$1 as TracingLayerTree };
