import { Runtime } from '../../root/root.js';
import { ThemeSupport } from '../../theme_support/theme_support.js';

// Copyright 2019 The Chromium Authors. All rights reserved.
/**
 * @param {!Node} node
 * @param {string} cssFile
 * @suppressGlobalPropertiesCheck
 */
function appendStyle(node, cssFile) {
    const content = Runtime.cachedResources.get(cssFile) || '';
    if (!content) {
        console.error(cssFile + ' not preloaded. Check module.json');
    }
    let styleElement = createElement('style');
    styleElement.textContent = content;
    node.appendChild(styleElement);
    const themeStyleSheet = ThemeSupport.instance().themeStyleSheet(cssFile, content);
    if (themeStyleSheet) {
        styleElement = createElement('style');
        styleElement.textContent = themeStyleSheet + '\n' + Runtime.Runtime.resolveSourceURL(cssFile + '.theme');
        node.appendChild(styleElement);
    }
}

// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
// @ts-nocheck
// TODO(crbug.com/1011811): Enable TypeScript compiler checks
/**
 * @param {?Node} node
 */
function WidgetfocusWidgetForNode(node) {
    while (node) {
        if (node.__widget) {
            break;
        }
        node = node.parentNodeOrShadowHost();
    }
    if (!node) {
        return;
    }
    let widget = node.__widget;
    while (widget._parentWidget) {
        widget._parentWidget._defaultFocusedChild = widget;
        widget = widget._parentWidget;
    }
}
/**
 * @param {?Node} node
 */
function XWidgetfocusWidgetForNode(node) {
    node = node && node.parentNodeOrShadowHost();
    const XWidgetCtor = customElements.get('x-widget');
    let widget = null;
    while (node) {
        if (node instanceof XWidgetCtor) {
            if (widget) {
                node._defaultFocusedElement = widget;
            }
            widget = node;
        }
        node = node.parentNodeOrShadowHost();
    }
}
/**
 * @param {!Event} event
 */
function focusChanged(event) {
    const document = event.target && event.target.ownerDocument;
    const element = document ? document.deepActiveElement() : null;
    WidgetfocusWidgetForNode(element);
    XWidgetfocusWidgetForNode(element);
}

// Copyright 2019 The Chromium Authors. All rights reserved.
/**
 * @param {!Element|!ShadowRoot} root
 */
function injectCoreStyles(root) {
    appendStyle(root, 'ui/inspectorCommon.css');
    appendStyle(root, 'ui/textButton.css');
    ThemeSupport.instance().injectHighlightStyleSheets(root);
    ThemeSupport.instance().injectCustomStyleSheets(root);
}

// Copyright 2019 The Chromium Authors. All rights reserved.
/**
 * @param {!Element} element
 * @param {string=} cssFile
 * @param {boolean=} delegatesFocus
 * @return {!DocumentFragment}
 */
function createShadowRootWithCoreStyles(element, cssFile, delegatesFocus) {
    const shadowRoot = element.attachShadow({ mode: 'open', delegatesFocus });
    injectCoreStyles(shadowRoot);
    if (cssFile) {
        appendStyle(shadowRoot, cssFile);
    }
    shadowRoot.addEventListener('focus', focusChanged.bind(UI), true);
    return shadowRoot;
}

// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
// @ts-nocheck
// TODO(crbug.com/1011811): Enable TypeScript compiler checks
/** @type {number} */
let _measuredScrollbarWidth;
/**
 * @param {?Document=} document
 * @return {number}
 */
function measuredScrollbarWidth(document) {
    if (typeof _measuredScrollbarWidth === 'number') {
        return _measuredScrollbarWidth;
    }
    if (!document) {
        return 16;
    }
    const scrollDiv = document.createElement('div');
    const innerDiv = document.createElement('div');
    scrollDiv.setAttribute('style', 'display: block; width: 100px; height: 100px; overflow: scroll;');
    innerDiv.setAttribute('style', 'height: 200px');
    scrollDiv.appendChild(innerDiv);
    document.body.appendChild(scrollDiv);
    _measuredScrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
    document.body.removeChild(scrollDiv);
    return _measuredScrollbarWidth;
}

// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
// @ts-nocheck
// TODO(crbug.com/1011811): Enable TypeScript compiler checks
/**
 * @param {string} localName
 * @param {string} typeExtension
 * @param {function(new:HTMLElement, *):void} definition
 * @return {function():!Element}
 * @suppressGlobalPropertiesCheck
 */
function registerCustomElement(localName, typeExtension, definition) {
    self.customElements.define(typeExtension, class extends definition {
        constructor() {
            super();
            // TODO(einbinder) convert to classes and custom element tags
            this.setAttribute('is', typeExtension);
        }
    }, { extends: localName });
    return () => createElement(localName, typeExtension);
}

// Copyright 2019 The Chromium Authors. All rights reserved.

export { appendStyle, createShadowRootWithCoreStyles, focusChanged, injectCoreStyles, measuredScrollbarWidth, registerCustomElement };
