import { ObjectWrapper, ParsedURL, Color, Settings, ResourceType, UIString, ls as ls$1, Console, EventTarget, Base64, Throttler, Revealer, Debouncer } from '../common/common.js';
import { InspectorFrontendHost, ResourceLoader, userMetrics, UserMetrics, Platform, InspectorFrontendHostAPI } from '../host/host.js';
import { Multimap, SetUtilities, StringUtilities, ArrayUtilities, DateUtilities, ls as ls$2, NumberUtilities } from '../platform/platform.js';
import { InspectorBackend } from '../protocol_client/protocol_client.js';
import { TextUtils, ContentProvider, TextRange, Text, StaticContentProvider } from '../text_utils/text_utils.js';
import { Runtime } from '../root/root.js';

// Copyright 2019 The Chromium Authors. All rights reserved.
/** @type {!Map<function(new:SDKModel, !Target), !{capabilities: number, autostart: boolean}>} */
const _registeredModels = new Map();
class SDKModel extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super();
        this._target = target;
    }
    /**
     * @return {!Target}
     */
    target() {
        return this._target;
    }
    /**
     * Override this method to perform tasks that are required to suspend the
     * model and that still need other models in an unsuspended state.
     * @param {string=} reason - optionally provide a reason, the model can respond accordingly
     * @return {!Promise<void>}
     */
    preSuspendModel(reason) {
        return Promise.resolve();
    }
    /**
     * @param {string=} reason - optionally provide a reason, the model can respond accordingly
     * @return {!Promise<void>}
     */
    suspendModel(reason) {
        return Promise.resolve();
    }
    /**
     * @return {!Promise<void>}
     */
    resumeModel() {
        return Promise.resolve();
    }
    /**
     * Override this method to perform tasks that are required to after resuming
     * the model and that require all models already in an unsuspended state.
     * @return {!Promise<void>}
     */
    postResumeModel() {
        return Promise.resolve();
    }
    dispose() {
    }
    /**
     * @param {function(new:SDKModel, !Target)} modelClass
     * @param {number} capabilities
     * @param {boolean} autostart
     */
    static register(modelClass, capabilities, autostart) {
        _registeredModels.set(modelClass, { capabilities, autostart });
    }
    static get registeredModels() {
        return _registeredModels;
    }
}
/**
 * @unrestricted
 */
class Target extends InspectorBackend.TargetBase {
    /**
     * @param {!TargetManager} targetManager
     * @param {string} id
     * @param {string} name
     * @param {!Type} type
     * @param {?Target} parentTarget
     * @param {string} sessionId
     * @param {boolean} suspended
     * @param {?ProtocolClient.InspectorBackend.Connection} connection
     */
    constructor(targetManager, id, name, type, parentTarget, sessionId, suspended, connection) {
        const needsNodeJSPatching = type === Type.Node;
        super(needsNodeJSPatching, parentTarget, sessionId, connection);
        this._targetManager = targetManager;
        this._name = name;
        this._inspectedURL = '';
        this._inspectedURLName = '';
        this._capabilitiesMask = 0;
        switch (type) {
            case Type.Frame:
                this._capabilitiesMask = Capability.Browser | Capability.Storage | Capability.DOM | Capability.JS |
                    Capability.Log | Capability.Network | Capability.Target | Capability.Tracing | Capability.Emulation |
                    Capability.Input | Capability.Inspector | Capability.Audits | Capability.WebAuthn | Capability.IO;
                if (!parentTarget) {
                    // This matches backend exposing certain capabilities only for the main frame.
                    this._capabilitiesMask |=
                        Capability.DeviceEmulation | Capability.ScreenCapture | Capability.Security | Capability.ServiceWorker;
                    // TODO(dgozman): we report service workers for the whole frame tree on the main frame,
                    // while we should be able to only cover the subtree corresponding to the target.
                }
                break;
            case Type.ServiceWorker:
                this._capabilitiesMask = Capability.JS | Capability.Log | Capability.Network | Capability.Target |
                    Capability.Inspector | Capability.IO;
                if (!parentTarget) {
                    this._capabilitiesMask |= Capability.Browser;
                }
                break;
            case Type.Worker:
                this._capabilitiesMask =
                    Capability.JS | Capability.Log | Capability.Network | Capability.Target | Capability.IO;
                break;
            case Type.Node:
                this._capabilitiesMask = Capability.JS;
                break;
            case Type.Browser:
                this._capabilitiesMask = Capability.Target | Capability.IO;
                break;
        }
        this._type = type;
        this._parentTarget = parentTarget;
        this._id = id;
        this._modelByConstructor = new Map();
        this._isSuspended = suspended;
    }
    /**
     * TODO(1011811): Replace type with !Set<function(new:SDKModel, !Target)> once we no longer type-check with closure.
     * @param {*} required
     */
    createModels(required) {
        this._creatingModels = true;
        // TODO(dgozman): fix this in bindings layer.
        // @ts-ignore ResourceTreeModel inherits from SDKModel introducing a cyclic dependency. Use the global for now.
        this.model(SDK.ResourceTreeModel);
        const registered = Array.from(SDKModel.registeredModels.keys());
        for (const modelClass of registered) {
            const info = 
            /** @type {!{capabilities: number, autostart: boolean}} */ (SDKModel.registeredModels.get(modelClass));
            if (info.autostart || required.has(modelClass)) {
                this.model(modelClass);
            }
        }
        this._creatingModels = false;
    }
    /**
     * @return {string}
     */
    id() {
        return this._id;
    }
    /**
     * @return {string}
     */
    name() {
        return this._name || this._inspectedURLName;
    }
    /**
     * @return {!Type}
     */
    type() {
        return this._type;
    }
    /**
     * @override
     */
    markAsNodeJSForTest() {
        super.markAsNodeJSForTest();
        this._type = Type.Node;
    }
    /**
     * @return {!TargetManager}
     */
    targetManager() {
        return this._targetManager;
    }
    /**
     * @param {number} capabilitiesMask
     * @return {boolean}
     */
    hasAllCapabilities(capabilitiesMask) {
        // TODO(dgozman): get rid of this method, once we never observe targets with
        // capability mask.
        return (this._capabilitiesMask & capabilitiesMask) === capabilitiesMask;
    }
    /**
     * @param {string} label
     * @return {string}
     */
    decorateLabel(label) {
        return (this._type === Type.Worker || this._type === Type.ServiceWorker) ? '\u2699 ' + label : label;
    }
    /**
     * @return {?Target}
     */
    parentTarget() {
        return this._parentTarget;
    }
    /**
     * @override
     * @param {string} reason
     */
    dispose(reason) {
        super.dispose(reason);
        this._targetManager.removeTarget(this);
        for (const model of this._modelByConstructor.values()) {
            model.dispose();
        }
    }
    /**
     * @param {function(new:SDKModel, !Target)} modelClass
     * @return {?T}
     * @template T
     */
    model(modelClass) {
        if (!this._modelByConstructor.get(modelClass)) {
            const info = SDKModel.registeredModels.get(modelClass);
            if (info === undefined) {
                throw 'Model class is not registered @' + new Error().stack;
            }
            if ((this._capabilitiesMask & info.capabilities) === info.capabilities) {
                const model = new modelClass(this);
                this._modelByConstructor.set(modelClass, model);
                if (!this._creatingModels) {
                    this._targetManager.modelAdded(this, modelClass, model);
                }
            }
        }
        return this._modelByConstructor.get(modelClass) || null;
    }
    /**
     * @return {!Map<function(new:SDKModel, !Target), !SDKModel>}
     */
    models() {
        return this._modelByConstructor;
    }
    /**
     * @return {string}
     */
    inspectedURL() {
        return this._inspectedURL;
    }
    /**
     * @param {string} inspectedURL
     */
    setInspectedURL(inspectedURL) {
        this._inspectedURL = inspectedURL;
        const parsedURL = ParsedURL.ParsedURL.fromString(inspectedURL);
        this._inspectedURLName = parsedURL ? parsedURL.lastPathComponentWithFragment() : '#' + this._id;
        if (!this.parentTarget()) {
            InspectorFrontendHost.InspectorFrontendHostInstance.inspectedURLChanged(inspectedURL || '');
        }
        this._targetManager.dispatchEventToListeners(Events.InspectedURLChanged, this);
        if (!this._name) {
            this._targetManager.dispatchEventToListeners(Events.NameChanged, this);
        }
    }
    /**
     * @param {string=} reason - optionally provide a reason, so models can respond accordingly
     * @return {!Promise<void>}
     */
    async suspend(reason) {
        if (this._isSuspended) {
            return Promise.resolve();
        }
        this._isSuspended = true;
        await Promise.all(Array.from(this.models().values(), m => m.preSuspendModel(reason)));
        await Promise.all(Array.from(this.models().values(), m => m.suspendModel(reason)));
    }
    /**
     * @return {!Promise<void>}
     */
    async resume() {
        if (!this._isSuspended) {
            return Promise.resolve();
        }
        this._isSuspended = false;
        await Promise.all(Array.from(this.models().values(), m => m.resumeModel()));
        await Promise.all(Array.from(this.models().values(), m => m.postResumeModel()));
    }
    /**
     * @return {boolean}
     */
    suspended() {
        return this._isSuspended;
    }
}
/**
 * @enum {number}
 */
const Capability = {
    Browser: 1 << 0,
    DOM: 1 << 1,
    JS: 1 << 2,
    Log: 1 << 3,
    Network: 1 << 4,
    Target: 1 << 5,
    ScreenCapture: 1 << 6,
    Tracing: 1 << 7,
    Emulation: 1 << 8,
    Security: 1 << 9,
    Input: 1 << 10,
    Inspector: 1 << 11,
    DeviceEmulation: 1 << 12,
    Storage: 1 << 13,
    ServiceWorker: 1 << 14,
    Audits: 1 << 15,
    WebAuthn: 1 << 16,
    IO: 1 << 17,
    None: 0,
};
/**
 * @enum {string}
 */
const Type = {
    Frame: 'frame',
    ServiceWorker: 'service-worker',
    Worker: 'worker',
    Node: 'node',
    Browser: 'browser',
};
/**
 * @type {!TargetManager}
 */
let targetManagerInstance;
class TargetManager extends ObjectWrapper.ObjectWrapper {
    /**
     * @private
     */
    constructor() {
        super();
        /** @type {!Set.<!Target>} */
        this._targets = new Set();
        /** @type {!Set.<!Observer>} */
        this._observers = new Set();
        /** @type {!Platform.Multimap<symbol, !{modelClass: function(new:SDKModel, !Target), thisObject: (!Object|undefined), listener: function(!Common.EventTarget.EventTargetEvent):void}>} */
        this._modelListeners = new Multimap();
        /** @type {!Platform.Multimap<function(new:SDKModel, !Target), !SDKModelObserver<*>>} */
        this._modelObservers = new Multimap();
        this._isSuspended = false;
    }
    /**
     * @param {{forceNew: boolean}} opts
     */
    static instance({ forceNew } = { forceNew: false }) {
        if (!targetManagerInstance || forceNew) {
            targetManagerInstance = new TargetManager();
        }
        return targetManagerInstance;
    }
    /**
     * @param {string=} reason - optionally provide a reason, so targets can respond accordingly
     * @return {!Promise<void>}
     */
    async suspendAllTargets(reason) {
        if (this._isSuspended) {
            return;
        }
        this._isSuspended = true;
        this.dispatchEventToListeners(Events.SuspendStateChanged);
        const suspendPromises = Array.from(this._targets.values(), target => target.suspend(reason));
        await Promise.all(suspendPromises);
    }
    /**
     * @return {!Promise<void>}
     */
    async resumeAllTargets() {
        if (!this._isSuspended) {
            return;
        }
        this._isSuspended = false;
        this.dispatchEventToListeners(Events.SuspendStateChanged);
        const resumePromises = Array.from(this._targets.values(), target => target.resume());
        await Promise.all(resumePromises);
    }
    /**
     * @return {boolean}
     */
    allTargetsSuspended() {
        return this._isSuspended;
    }
    /**
     * @param {function(new:SDKModel,!Target)} modelClass
     * @return {!Array<!T>}
     * @template T
     */
    models(modelClass) {
        const result = [];
        for (const target of this._targets) {
            const model = target.model(modelClass);
            if (model) {
                result.push(model);
            }
        }
        return result;
    }
    /**
     * @return {string}
     */
    inspectedURL() {
        const mainTarget = this.mainTarget();
        return mainTarget ? mainTarget.inspectedURL() : '';
    }
    /**
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {!SDKModelObserver<T>} observer
     * @template T
     */
    observeModels(modelClass, observer) {
        const models = this.models(modelClass);
        this._modelObservers.set(modelClass, observer);
        for (const model of models) {
            observer.modelAdded(model);
        }
    }
    /**
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {!SDKModelObserver<T>} observer
     * @template T
     */
    unobserveModels(modelClass, observer) {
        this._modelObservers.delete(modelClass, observer);
    }
    /**
     * @param {!Target} target
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {!SDKModel} model
     */
    modelAdded(target, modelClass, model) {
        for (const observer of this._modelObservers.get(modelClass).values()) {
            observer.modelAdded(model);
        }
    }
    /**
     * @param {!Target} target
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {!SDKModel} model
     */
    _modelRemoved(target, modelClass, model) {
        for (const observer of this._modelObservers.get(modelClass).values()) {
            observer.modelRemoved(model);
        }
    }
    /**
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {symbol} eventType
     * @param {function(!Common.EventTarget.EventTargetEvent):void} listener
     * @param {!Object=} thisObject
     */
    addModelListener(modelClass, eventType, listener, thisObject) {
        for (const model of this.models(modelClass)) {
            model.addEventListener(eventType, listener, thisObject);
        }
        this._modelListeners.set(eventType, { modelClass: modelClass, thisObject: thisObject, listener: listener });
    }
    /**
     * @param {function(new:SDKModel,!Target)} modelClass
     * @param {symbol} eventType
     * @param {function(!Common.EventTarget.EventTargetEvent):void} listener
     * @param {!Object=} thisObject
     */
    removeModelListener(modelClass, eventType, listener, thisObject) {
        if (!this._modelListeners.has(eventType)) {
            return;
        }
        for (const model of this.models(modelClass)) {
            model.removeEventListener(eventType, listener, thisObject);
        }
        for (const info of this._modelListeners.get(eventType)) {
            if (info.modelClass === modelClass && info.listener === listener && info.thisObject === thisObject) {
                this._modelListeners.delete(eventType, info);
            }
        }
    }
    /**
     * @param {!Observer} targetObserver
     */
    observeTargets(targetObserver) {
        if (this._observers.has(targetObserver)) {
            throw new Error('Observer can only be registered once');
        }
        for (const target of this._targets) {
            targetObserver.targetAdded(target);
        }
        this._observers.add(targetObserver);
    }
    /**
     * @param {!Observer} targetObserver
     */
    unobserveTargets(targetObserver) {
        this._observers.delete(targetObserver);
    }
    /**
     * @param {string} id
     * @param {string} name
     * @param {!Type} type
     * @param {?Target} parentTarget
     * @param {string=} sessionId
     * @param {boolean=} waitForDebuggerInPage
     * @param {!ProtocolClient.InspectorBackend.Connection=} connection
     * @return {!Target}
     */
    createTarget(id, name, type, parentTarget, sessionId, waitForDebuggerInPage, connection) {
        const target = new Target(this, id, name, type, parentTarget, sessionId || '', this._isSuspended, connection || null);
        if (waitForDebuggerInPage) {
            // @ts-ignore TODO(1063322): Find out where pageAgent() is set on Target/TargetBase.
            target.pageAgent().waitForDebugger();
        }
        target.createModels(new Set(this._modelObservers.keysArray()));
        this._targets.add(target);
        // Iterate over a copy. _observers might be modified during iteration.
        for (const observer of [...this._observers]) {
            observer.targetAdded(target);
        }
        for (const modelClass of target.models().keys()) {
            const model = /** @type {!SDKModel} */ (target.models().get(modelClass));
            this.modelAdded(target, modelClass, model);
        }
        for (const key of this._modelListeners.keysArray()) {
            for (const info of this._modelListeners.get(key)) {
                const model = target.model(info.modelClass);
                if (model) {
                    model.addEventListener(key, info.listener, info.thisObject);
                }
            }
        }
        return target;
    }
    /**
     * @param {!Target} target
     */
    removeTarget(target) {
        if (!this._targets.has(target)) {
            return;
        }
        this._targets.delete(target);
        for (const modelClass of target.models().keys()) {
            const model = /** @type {!SDKModel} */ (target.models().get(modelClass));
            this._modelRemoved(target, modelClass, model);
        }
        // Iterate over a copy. _observers might be modified during iteration.
        for (const observer of [...this._observers]) {
            observer.targetRemoved(target);
        }
        for (const key of this._modelListeners.keysArray()) {
            for (const info of this._modelListeners.get(key)) {
                const model = target.model(info.modelClass);
                if (model) {
                    model.removeEventListener(key, info.listener, info.thisObject);
                }
            }
        }
    }
    /**
     * @return {!Array.<!Target>}
     */
    targets() {
        return [...this._targets];
    }
    /**
     * @param {string} id
     * @return {?Target}
     */
    targetById(id) {
        // TODO(dgozman): add a map id -> target.
        return this.targets().find(target => target.id() === id) || null;
    }
    /**
     * @return {?Target}
     */
    mainTarget() {
        return this._targets.size ? this._targets.values().next().value : null;
    }
}
/** @enum {symbol} */
const Events = {
    AvailableTargetsChanged: Symbol('AvailableTargetsChanged'),
    InspectedURLChanged: Symbol('InspectedURLChanged'),
    NameChanged: Symbol('NameChanged'),
    SuspendStateChanged: Symbol('SuspendStateChanged')
};
/**
 * @interface
 */
class Observer {
    /**
     * @param {!Target} target
     */
    targetAdded(target) {
    }
    /**
     * @param {!Target} target
     */
    targetRemoved(target) {
    }
}
/**
 * @interface
 * @template T
 */
class SDKModelObserver {
    /**
     * @param {!T} model
     */
    modelAdded(model) {
    }
    /**
     * @param {!T} model
     */
    modelRemoved(model) {
    }
}

var SDKModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    SDKModel: SDKModel,
    Target: Target,
    Capability: Capability,
    Type: Type,
    TargetManager: TargetManager,
    Events: Events,
    Observer: Observer,
    SDKModelObserver: SDKModelObserver
});

// Copyright 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
const generatedProperties = [
    { 'name': '-webkit-app-region', 'keywords': ['none', 'drag', 'no-drag'] },
    { 'name': '-webkit-border-horizontal-spacing', 'inherited': true },
    { 'name': '-webkit-border-image' },
    { 'name': '-webkit-border-vertical-spacing', 'inherited': true },
    { 'name': '-webkit-box-align', 'keywords': ['stretch', 'start', 'center', 'end', 'baseline'] },
    { 'name': '-webkit-box-decoration-break', 'keywords': ['slice', 'clone'] },
    { 'name': '-webkit-box-direction', 'inherited': true, 'keywords': ['normal', 'reverse'] },
    { 'name': '-webkit-box-flex' },
    { 'name': '-webkit-box-ordinal-group' },
    { 'name': '-webkit-box-orient', 'keywords': ['horizontal', 'vertical'] },
    { 'name': '-webkit-box-pack', 'keywords': ['start', 'center', 'end', 'justify'] },
    { 'name': '-webkit-box-reflect' },
    { 'longhands': ['break-after'], 'name': '-webkit-column-break-after' },
    { 'longhands': ['break-before'], 'name': '-webkit-column-break-before' },
    { 'longhands': ['break-inside'], 'name': '-webkit-column-break-inside' },
    { 'name': '-webkit-font-smoothing', 'inherited': true },
    { 'name': '-webkit-highlight', 'inherited': true },
    { 'name': '-webkit-hyphenate-character', 'inherited': true },
    {
        'name': '-webkit-line-break',
        'inherited': true,
        'keywords': ['auto', 'loose', 'normal', 'strict', 'after-white-space', 'anywhere']
    },
    { 'name': '-webkit-line-clamp' },
    { 'name': '-webkit-locale', 'inherited': true },
    {
        'longhands': [
            '-webkit-mask-image', '-webkit-mask-position-x', '-webkit-mask-position-y', '-webkit-mask-size',
            '-webkit-mask-repeat-x', '-webkit-mask-repeat-y', '-webkit-mask-origin', '-webkit-mask-clip'
        ],
        'name': '-webkit-mask'
    },
    {
        'longhands': [
            '-webkit-mask-box-image-source', '-webkit-mask-box-image-slice', '-webkit-mask-box-image-width',
            '-webkit-mask-box-image-outset', '-webkit-mask-box-image-repeat'
        ],
        'name': '-webkit-mask-box-image'
    },
    { 'name': '-webkit-mask-box-image-outset' },
    { 'name': '-webkit-mask-box-image-repeat' },
    { 'name': '-webkit-mask-box-image-slice' },
    { 'name': '-webkit-mask-box-image-source' },
    { 'name': '-webkit-mask-box-image-width' },
    { 'name': '-webkit-mask-clip' },
    { 'name': '-webkit-mask-composite' },
    { 'name': '-webkit-mask-image' },
    { 'name': '-webkit-mask-origin' },
    { 'longhands': ['-webkit-mask-position-x', '-webkit-mask-position-y'], 'name': '-webkit-mask-position' },
    { 'name': '-webkit-mask-position-x' },
    { 'name': '-webkit-mask-position-y' },
    { 'longhands': ['-webkit-mask-repeat-x', '-webkit-mask-repeat-y'], 'name': '-webkit-mask-repeat' },
    { 'name': '-webkit-mask-repeat-x' },
    { 'name': '-webkit-mask-repeat-y' },
    { 'name': '-webkit-mask-size' },
    { 'name': '-webkit-perspective-origin-x' },
    { 'name': '-webkit-perspective-origin-y' },
    { 'name': '-webkit-print-color-adjust', 'inherited': true, 'keywords': ['economy', 'exact'] },
    { 'name': '-webkit-rtl-ordering', 'inherited': true, 'keywords': ['logical', 'visual'] },
    { 'name': '-webkit-ruby-position', 'inherited': true, 'keywords': ['before', 'after'] },
    { 'name': '-webkit-tap-highlight-color', 'inherited': true },
    { 'name': '-webkit-text-combine', 'inherited': true },
    { 'name': '-webkit-text-decorations-in-effect', 'inherited': true },
    {
        'longhands': ['-webkit-text-emphasis-style', '-webkit-text-emphasis-color'],
        'name': '-webkit-text-emphasis',
        'inherited': true
    },
    { 'name': '-webkit-text-emphasis-color', 'inherited': true },
    { 'name': '-webkit-text-emphasis-position', 'inherited': true },
    { 'name': '-webkit-text-emphasis-style', 'inherited': true },
    { 'name': '-webkit-text-fill-color', 'inherited': true },
    { 'name': '-webkit-text-orientation', 'inherited': true },
    { 'name': '-webkit-text-security', 'inherited': true, 'keywords': ['none', 'disc', 'circle', 'square'] },
    {
        'longhands': ['-webkit-text-stroke-width', '-webkit-text-stroke-color'],
        'name': '-webkit-text-stroke',
        'inherited': true
    },
    { 'name': '-webkit-text-stroke-color', 'inherited': true },
    { 'name': '-webkit-text-stroke-width', 'inherited': true },
    { 'name': '-webkit-transform-origin-x' },
    { 'name': '-webkit-transform-origin-y' },
    { 'name': '-webkit-transform-origin-z' },
    { 'name': '-webkit-user-drag', 'keywords': ['auto', 'none', 'element'] },
    {
        'name': '-webkit-user-modify',
        'inherited': true,
        'keywords': ['read-only', 'read-write', 'read-write-plaintext-only']
    },
    { 'name': '-webkit-writing-mode', 'inherited': true },
    { 'name': 'advance-override' },
    { 'name': 'align-content' },
    { 'name': 'align-items' },
    { 'name': 'align-self' },
    {
        'keywords': ['baseline', 'alphabetic', 'ideographic', 'middle', 'central', 'mathematical'],
        'svg': true,
        'name': 'alignment-baseline'
    },
    { 'name': 'all' },
    {
        'longhands': [
            'animation-duration', 'animation-timing-function', 'animation-delay', 'animation-iteration-count',
            'animation-direction', 'animation-fill-mode', 'animation-play-state', 'animation-name', 'animation-timeline'
        ],
        'name': 'animation'
    },
    { 'name': 'animation-delay' },
    { 'name': 'animation-direction', 'keywords': ['normal', 'reverse', 'alternate', 'alternate-reverse'] },
    { 'name': 'animation-duration' },
    { 'name': 'animation-fill-mode', 'keywords': ['none', 'forwards', 'backwards', 'both'] },
    { 'name': 'animation-iteration-count', 'keywords': ['infinite'] },
    { 'name': 'animation-name', 'keywords': ['none'] },
    { 'name': 'animation-play-state', 'keywords': ['running', 'paused'] },
    { 'name': 'animation-timeline', 'keywords': ['none', 'auto'] },
    {
        'name': 'animation-timing-function',
        'keywords': [
            'linear', 'ease', 'ease-in', 'ease-out', 'ease-in-out', 'jump-both', 'jump-end', 'jump-none', 'jump-start',
            'step-start', 'step-end'
        ]
    },
    { 'name': 'appearance' },
    { 'name': 'ascent-override' },
    { 'name': 'aspect-ratio', 'keywords': ['auto'] },
    { 'name': 'backdrop-filter', 'keywords': ['none'] },
    { 'name': 'backface-visibility', 'keywords': ['visible', 'hidden'] },
    {
        'longhands': [
            'background-image', 'background-position-x', 'background-position-y', 'background-size', 'background-repeat-x',
            'background-repeat-y', 'background-attachment', 'background-origin', 'background-clip', 'background-color'
        ],
        'name': 'background'
    },
    { 'name': 'background-attachment', 'keywords': ['scroll', 'fixed', 'local'] },
    {
        'name': 'background-blend-mode',
        'keywords': [
            'normal', 'multiply', 'screen', 'overlay', 'darken', 'lighten', 'color-dodge', 'color-burn', 'hard-light',
            'soft-light', 'difference', 'exclusion', 'hue', 'saturation', 'color', 'luminosity'
        ]
    },
    { 'name': 'background-clip', 'keywords': ['border-box', 'padding-box', 'content-box'] },
    { 'name': 'background-color', 'keywords': ['currentcolor'] },
    { 'name': 'background-image', 'keywords': ['auto', 'none'] },
    { 'name': 'background-origin', 'keywords': ['border-box', 'padding-box', 'content-box'] },
    { 'longhands': ['background-position-x', 'background-position-y'], 'name': 'background-position' },
    { 'name': 'background-position-x' },
    { 'name': 'background-position-y' },
    { 'longhands': ['background-repeat-x', 'background-repeat-y'], 'name': 'background-repeat' },
    { 'name': 'background-repeat-x' },
    { 'name': 'background-repeat-y' },
    { 'name': 'background-size', 'keywords': ['auto', 'cover', 'contain'] },
    { 'name': 'baseline-shift', 'svg': true, 'keywords': ['sub', 'super'] },
    { 'name': 'block-size', 'keywords': ['auto'] },
    {
        'longhands': [
            'border-top-color', 'border-top-style', 'border-top-width', 'border-right-color', 'border-right-style',
            'border-right-width', 'border-bottom-color', 'border-bottom-style', 'border-bottom-width', 'border-left-color',
            'border-left-style', 'border-left-width', 'border-image-source', 'border-image-slice', 'border-image-width',
            'border-image-outset', 'border-image-repeat'
        ],
        'name': 'border'
    },
    {
        'longhands': [
            'border-block-start-color', 'border-block-start-style', 'border-block-start-width', 'border-block-end-color',
            'border-block-end-style', 'border-block-end-width'
        ],
        'name': 'border-block'
    },
    { 'longhands': ['border-block-start-color', 'border-block-end-color'], 'name': 'border-block-color' },
    {
        'longhands': ['border-block-end-width', 'border-block-end-style', 'border-block-end-color'],
        'name': 'border-block-end'
    },
    { 'name': 'border-block-end-color' },
    { 'name': 'border-block-end-style' },
    { 'name': 'border-block-end-width' },
    {
        'longhands': ['border-block-start-width', 'border-block-start-style', 'border-block-start-color'],
        'name': 'border-block-start'
    },
    { 'name': 'border-block-start-color' },
    { 'name': 'border-block-start-style' },
    { 'name': 'border-block-start-width' },
    { 'longhands': ['border-block-start-style', 'border-block-end-style'], 'name': 'border-block-style' },
    { 'longhands': ['border-block-start-width', 'border-block-end-width'], 'name': 'border-block-width' },
    { 'longhands': ['border-bottom-width', 'border-bottom-style', 'border-bottom-color'], 'name': 'border-bottom' },
    { 'name': 'border-bottom-color', 'keywords': ['currentcolor'] },
    { 'name': 'border-bottom-left-radius' },
    { 'name': 'border-bottom-right-radius' },
    {
        'name': 'border-bottom-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'border-bottom-width', 'keywords': ['thin', 'medium', 'thick'] },
    { 'name': 'border-collapse', 'inherited': true, 'keywords': ['separate', 'collapse'] },
    {
        'longhands': ['border-top-color', 'border-right-color', 'border-bottom-color', 'border-left-color'],
        'name': 'border-color'
    },
    {
        'longhands': [
            'border-image-source', 'border-image-slice', 'border-image-width', 'border-image-outset', 'border-image-repeat'
        ],
        'name': 'border-image'
    },
    { 'name': 'border-image-outset' },
    { 'name': 'border-image-repeat', 'keywords': ['stretch', 'repeat', 'round', 'space'] },
    { 'name': 'border-image-slice' },
    { 'name': 'border-image-source', 'keywords': ['none'] },
    { 'name': 'border-image-width', 'keywords': ['auto'] },
    {
        'longhands': [
            'border-inline-start-color', 'border-inline-start-style', 'border-inline-start-width', 'border-inline-end-color',
            'border-inline-end-style', 'border-inline-end-width'
        ],
        'name': 'border-inline'
    },
    { 'longhands': ['border-inline-start-color', 'border-inline-end-color'], 'name': 'border-inline-color' },
    {
        'longhands': ['border-inline-end-width', 'border-inline-end-style', 'border-inline-end-color'],
        'name': 'border-inline-end'
    },
    { 'name': 'border-inline-end-color' },
    { 'name': 'border-inline-end-style' },
    { 'name': 'border-inline-end-width' },
    {
        'longhands': ['border-inline-start-width', 'border-inline-start-style', 'border-inline-start-color'],
        'name': 'border-inline-start'
    },
    { 'name': 'border-inline-start-color' },
    { 'name': 'border-inline-start-style' },
    { 'name': 'border-inline-start-width' },
    { 'longhands': ['border-inline-start-style', 'border-inline-end-style'], 'name': 'border-inline-style' },
    { 'longhands': ['border-inline-start-width', 'border-inline-end-width'], 'name': 'border-inline-width' },
    { 'longhands': ['border-left-width', 'border-left-style', 'border-left-color'], 'name': 'border-left' },
    { 'name': 'border-left-color', 'keywords': ['currentcolor'] },
    {
        'name': 'border-left-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'border-left-width', 'keywords': ['thin', 'medium', 'thick'] },
    {
        'longhands': [
            'border-top-left-radius', 'border-top-right-radius', 'border-bottom-right-radius', 'border-bottom-left-radius'
        ],
        'name': 'border-radius'
    },
    { 'longhands': ['border-right-width', 'border-right-style', 'border-right-color'], 'name': 'border-right' },
    { 'name': 'border-right-color', 'keywords': ['currentcolor'] },
    {
        'name': 'border-right-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'border-right-width', 'keywords': ['thin', 'medium', 'thick'] },
    {
        'longhands': ['-webkit-border-horizontal-spacing', '-webkit-border-vertical-spacing'],
        'name': 'border-spacing',
        'inherited': true
    },
    {
        'keywords': ['none'],
        'longhands': ['border-top-style', 'border-right-style', 'border-bottom-style', 'border-left-style'],
        'name': 'border-style'
    },
    { 'longhands': ['border-top-width', 'border-top-style', 'border-top-color'], 'name': 'border-top' },
    { 'name': 'border-top-color', 'keywords': ['currentcolor'] },
    { 'name': 'border-top-left-radius' },
    { 'name': 'border-top-right-radius' },
    {
        'name': 'border-top-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'border-top-width', 'keywords': ['thin', 'medium', 'thick'] },
    {
        'longhands': ['border-top-width', 'border-right-width', 'border-bottom-width', 'border-left-width'],
        'name': 'border-width'
    },
    { 'name': 'bottom', 'keywords': ['auto'] },
    { 'name': 'box-shadow', 'keywords': ['none'] },
    { 'name': 'box-sizing', 'keywords': ['content-box', 'border-box'] },
    {
        'name': 'break-after',
        'keywords': ['auto', 'avoid', 'avoid-column', 'avoid-page', 'column', 'left', 'page', 'recto', 'right', 'verso']
    },
    {
        'name': 'break-before',
        'keywords': ['auto', 'avoid', 'avoid-column', 'avoid-page', 'column', 'left', 'page', 'recto', 'right', 'verso']
    },
    { 'name': 'break-inside', 'keywords': ['auto', 'avoid', 'avoid-column', 'avoid-page'] },
    { 'svg': true, 'name': 'buffered-rendering' },
    { 'name': 'caption-side', 'inherited': true, 'keywords': ['top', 'bottom'] },
    { 'name': 'caret-color', 'inherited': true, 'keywords': ['auto', 'currentcolor'] },
    { 'name': 'clear', 'keywords': ['none', 'left', 'right', 'both', 'inline-start', 'inline-end'] },
    { 'name': 'clip', 'keywords': ['auto'] },
    { 'name': 'clip-path', 'keywords': ['none'] },
    { 'name': 'clip-rule', 'svg': true, 'inherited': true, 'keywords': ['nonzero', 'evenodd'] },
    { 'name': 'color', 'inherited': true, 'keywords': ['currentcolor'] },
    { 'name': 'color-interpolation', 'svg': true, 'inherited': true, 'keywords': ['auto', 'srgb', 'linearrgb'] },
    { 'svg': true, 'name': 'color-interpolation-filters', 'inherited': true },
    { 'name': 'color-rendering', 'svg': true, 'inherited': true, 'keywords': ['auto', 'optimizespeed', 'optimizequality'] },
    { 'name': 'color-scheme', 'inherited': true },
    { 'name': 'column-count', 'keywords': ['auto'] },
    { 'name': 'column-fill', 'keywords': ['balance', 'auto'] },
    { 'name': 'column-gap', 'keywords': ['normal'] },
    { 'longhands': ['column-rule-width', 'column-rule-style', 'column-rule-color'], 'name': 'column-rule' },
    { 'name': 'column-rule-color', 'keywords': ['currentcolor'] },
    {
        'name': 'column-rule-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'column-rule-width', 'keywords': ['thin', 'medium', 'thick'] },
    { 'name': 'column-span', 'keywords': ['none', 'all'] },
    { 'name': 'column-width', 'keywords': ['auto'] },
    { 'longhands': ['column-width', 'column-count'], 'name': 'columns' },
    { 'name': 'contain', 'keywords': ['none', 'strict', 'content', 'size', 'layout', 'style', 'paint'] },
    { 'name': 'contain-intrinsic-size', 'keywords': ['auto'] },
    { 'name': 'content' },
    { 'name': 'content-visibility', 'keywords': ['visible', 'auto', 'hidden', 'hidden-matchable'] },
    { 'name': 'counter-increment', 'keywords': ['none'] },
    { 'name': 'counter-reset', 'keywords': ['none'] },
    { 'name': 'counter-set', 'keywords': ['none'] },
    {
        'name': 'cursor',
        'inherited': true,
        'keywords': [
            'auto', 'default', 'none', 'context-menu', 'help', 'pointer', 'progress', 'wait',
            'cell', 'crosshair', 'text', 'vertical-text', 'alias', 'copy', 'move', 'no-drop',
            'not-allowed', 'e-resize', 'n-resize', 'ne-resize', 'nw-resize', 's-resize', 'se-resize', 'sw-resize',
            'w-resize', 'ew-resize', 'ns-resize', 'nesw-resize', 'nwse-resize', 'col-resize', 'row-resize', 'all-scroll',
            'zoom-in', 'zoom-out', 'grab', 'grabbing'
        ]
    },
    { 'svg': true, 'name': 'cx' },
    { 'svg': true, 'name': 'cy' },
    { 'name': 'd', 'svg': true, 'keywords': ['none'] },
    { 'name': 'descent-override' },
    { 'name': 'direction', 'inherited': true, 'keywords': ['ltr', 'rtl'] },
    {
        'keywords': [
            'inline',
            'block',
            'list-item',
            'inline-block',
            'table',
            'inline-table',
            'table-row-group',
            'table-header-group',
            'table-footer-group',
            'table-row',
            'table-column-group',
            'table-column',
            'table-cell',
            'table-caption',
            '-webkit-box',
            '-webkit-inline-box',
            'flex',
            'inline-flex',
            'grid',
            'inline-grid',
            'contents',
            'flow-root',
            'none'
        ],
        'name': 'display'
    },
    {
        'name': 'dominant-baseline',
        'svg': true,
        'inherited': true,
        'keywords': ['auto', 'alphabetic', 'ideographic', 'middle', 'central', 'mathematical', 'hanging']
    },
    { 'name': 'empty-cells', 'inherited': true, 'keywords': ['show', 'hide'] },
    { 'name': 'end' },
    { 'name': 'fill', 'svg': true, 'inherited': true },
    { 'svg': true, 'inherited': true, 'name': 'fill-opacity' },
    { 'name': 'fill-rule', 'svg': true, 'inherited': true, 'keywords': ['nonzero', 'evenodd'] },
    { 'name': 'filter', 'keywords': ['none'] },
    { 'longhands': ['flex-grow', 'flex-shrink', 'flex-basis'], 'name': 'flex' },
    { 'name': 'flex-basis', 'keywords': ['auto'] },
    { 'name': 'flex-direction', 'keywords': ['row', 'row-reverse', 'column', 'column-reverse'] },
    { 'longhands': ['flex-direction', 'flex-wrap'], 'name': 'flex-flow' },
    { 'name': 'flex-grow' },
    { 'name': 'flex-shrink' },
    { 'name': 'flex-wrap', 'keywords': ['nowrap', 'wrap', 'wrap-reverse'] },
    { 'name': 'float', 'keywords': ['none', 'left', 'right', 'inline-start', 'inline-end'] },
    { 'name': 'flood-color', 'svg': true, 'keywords': ['currentcolor'] },
    { 'name': 'flood-opacity', 'svg': true },
    {
        'longhands': [
            'font-style', 'font-variant-ligatures', 'font-variant-caps', 'font-variant-numeric', 'font-variant-east-asian',
            'font-weight', 'font-stretch', 'font-size', 'line-height', 'font-family'
        ],
        'name': 'font',
        'inherited': true
    },
    { 'name': 'font-display' },
    { 'name': 'font-family', 'inherited': true },
    { 'name': 'font-feature-settings', 'inherited': true, 'keywords': ['normal'] },
    { 'name': 'font-kerning', 'inherited': true, 'keywords': ['auto', 'normal', 'none'] },
    { 'name': 'font-optical-sizing', 'inherited': true, 'keywords': ['auto', 'none'] },
    {
        'name': 'font-size',
        'inherited': true,
        'keywords': [
            'xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large', 'xxx-large', 'larger', 'smaller',
            '-webkit-xxx-large'
        ]
    },
    { 'name': 'font-size-adjust', 'inherited': true, 'keywords': ['none'] },
    {
        'name': 'font-stretch',
        'inherited': true,
        'keywords': [
            'normal', 'ultra-condensed', 'extra-condensed', 'condensed', 'semi-condensed', 'semi-expanded', 'expanded',
            'extra-expanded', 'ultra-expanded'
        ]
    },
    { 'name': 'font-style', 'inherited': true, 'keywords': ['normal', 'italic', 'oblique'] },
    {
        'longhands': ['font-variant-ligatures', 'font-variant-caps', 'font-variant-numeric', 'font-variant-east-asian'],
        'name': 'font-variant',
        'inherited': true
    },
    {
        'inherited': true,
        'keywords': ['normal', 'small-caps', 'all-small-caps', 'petite-caps', 'all-petite-caps', 'unicase', 'titling-caps'],
        'name': 'font-variant-caps'
    },
    {
        'inherited': true,
        'keywords': [
            'normal', 'jis78', 'jis83', 'jis90', 'jis04', 'simplified', 'traditional', 'full-width', 'proportional-width',
            'ruby'
        ],
        'name': 'font-variant-east-asian'
    },
    {
        'name': 'font-variant-ligatures',
        'inherited': true,
        'keywords': [
            'normal', 'none', 'common-ligatures', 'no-common-ligatures', 'discretionary-ligatures',
            'no-discretionary-ligatures', 'historical-ligatures', 'no-historical-ligatures', 'contextual', 'no-contextual'
        ]
    },
    {
        'inherited': true,
        'keywords': [
            'normal', 'lining-nums', 'oldstyle-nums', 'proportional-nums', 'tabular-nums', 'diagonal-fractions',
            'stacked-fractions', 'ordinal', 'slashed-zero'
        ],
        'name': 'font-variant-numeric'
    },
    { 'name': 'font-variation-settings', 'inherited': true, 'keywords': ['normal'] },
    { 'name': 'font-weight', 'inherited': true, 'keywords': ['normal', 'bold', 'bolder', 'lighter'] },
    { 'name': 'forced-color-adjust', 'inherited': true, 'keywords': ['auto', 'none'] },
    { 'longhands': ['row-gap', 'column-gap'], 'name': 'gap' },
    {
        'longhands': [
            'grid-template-rows', 'grid-template-columns', 'grid-template-areas', 'grid-auto-flow', 'grid-auto-rows',
            'grid-auto-columns'
        ],
        'name': 'grid'
    },
    { 'longhands': ['grid-row-start', 'grid-column-start', 'grid-row-end', 'grid-column-end'], 'name': 'grid-area' },
    { 'name': 'grid-auto-columns', 'keywords': ['auto', 'min-content', 'max-content'] },
    { 'name': 'grid-auto-flow', 'keywords': ['row', 'column'] },
    { 'name': 'grid-auto-rows', 'keywords': ['auto', 'min-content', 'max-content'] },
    { 'longhands': ['grid-column-start', 'grid-column-end'], 'name': 'grid-column' },
    { 'name': 'grid-column-end', 'keywords': ['auto'] },
    { 'longhands': ['column-gap'], 'name': 'grid-column-gap' },
    { 'name': 'grid-column-start', 'keywords': ['auto'] },
    { 'longhands': ['row-gap', 'column-gap'], 'name': 'grid-gap' },
    { 'longhands': ['grid-row-start', 'grid-row-end'], 'name': 'grid-row' },
    { 'name': 'grid-row-end', 'keywords': ['auto'] },
    { 'longhands': ['row-gap'], 'name': 'grid-row-gap' },
    { 'name': 'grid-row-start', 'keywords': ['auto'] },
    { 'longhands': ['grid-template-rows', 'grid-template-columns', 'grid-template-areas'], 'name': 'grid-template' },
    { 'keywords': ['none'], 'name': 'grid-template-areas' },
    { 'name': 'grid-template-columns', 'keywords': ['none'] },
    { 'name': 'grid-template-rows', 'keywords': ['none'] },
    { 'name': 'height', 'keywords': ['auto', 'fit-content', 'min-content', 'max-content'] },
    { 'name': 'hyphens', 'inherited': true, 'keywords': ['none', 'manual', 'auto'] },
    { 'name': 'image-orientation', 'inherited': true },
    {
        'name': 'image-rendering',
        'inherited': true,
        'keywords': ['auto', 'optimizespeed', 'optimizequality', '-webkit-optimize-contrast', 'pixelated']
    },
    { 'name': 'inherits' },
    { 'name': 'initial-value' },
    { 'name': 'inline-size', 'keywords': ['auto'] },
    { 'longhands': ['top', 'right', 'bottom', 'left'], 'name': 'inset' },
    { 'longhands': ['inset-block-start', 'inset-block-end'], 'name': 'inset-block' },
    { 'name': 'inset-block-end' },
    { 'name': 'inset-block-start' },
    { 'longhands': ['inset-inline-start', 'inset-inline-end'], 'name': 'inset-inline' },
    { 'name': 'inset-inline-end' },
    { 'name': 'inset-inline-start' },
    { 'name': 'isolation', 'keywords': ['auto', 'isolate'] },
    { 'name': 'justify-content' },
    { 'name': 'justify-items' },
    { 'name': 'justify-self' },
    { 'name': 'left', 'keywords': ['auto'] },
    { 'inherited': true, 'keywords': ['normal'], 'name': 'letter-spacing' },
    { 'name': 'lighting-color', 'svg': true, 'keywords': ['currentcolor'] },
    { 'name': 'line-break', 'inherited': true, 'keywords': ['auto', 'loose', 'normal', 'strict', 'anywhere'] },
    { 'name': 'line-gap-override' },
    { 'name': 'line-height', 'inherited': true, 'keywords': ['normal'] },
    { 'name': 'line-height-step', 'inherited': true },
    {
        'longhands': ['list-style-position', 'list-style-image', 'list-style-type'],
        'name': 'list-style',
        'inherited': true
    },
    { 'name': 'list-style-image', 'inherited': true, 'keywords': ['none'] },
    { 'name': 'list-style-position', 'inherited': true, 'keywords': ['outside', 'inside'] },
    {
        'keywords': [
            'disc',
            'circle',
            'square',
            'decimal',
            'decimal-leading-zero',
            'arabic-indic',
            'bengali',
            'cambodian',
            'khmer',
            'devanagari',
            'gujarati',
            'gurmukhi',
            'kannada',
            'lao',
            'malayalam',
            'mongolian',
            'myanmar',
            'oriya',
            'persian',
            'urdu',
            'telugu',
            'tibetan',
            'thai',
            'lower-roman',
            'upper-roman',
            'lower-greek',
            'lower-alpha',
            'lower-latin',
            'upper-alpha',
            'upper-latin',
            'cjk-earthly-branch',
            'cjk-heavenly-stem',
            'ethiopic-halehame',
            'ethiopic-halehame-am',
            'ethiopic-halehame-ti-er',
            'ethiopic-halehame-ti-et',
            'hangul',
            'hangul-consonant',
            'korean-hangul-formal',
            'korean-hanja-formal',
            'korean-hanja-informal',
            'hebrew',
            'armenian',
            'lower-armenian',
            'upper-armenian',
            'georgian',
            'cjk-ideographic',
            'simp-chinese-formal',
            'simp-chinese-informal',
            'trad-chinese-formal',
            'trad-chinese-informal',
            'hiragana',
            'katakana',
            'hiragana-iroha',
            'katakana-iroha',
            'none'
        ],
        'name': 'list-style-type',
        'inherited': true
    },
    { 'longhands': ['margin-top', 'margin-right', 'margin-bottom', 'margin-left'], 'name': 'margin' },
    { 'longhands': ['margin-block-start', 'margin-block-end'], 'name': 'margin-block' },
    { 'name': 'margin-block-end', 'keywords': ['auto'] },
    { 'name': 'margin-block-start', 'keywords': ['auto'] },
    { 'name': 'margin-bottom', 'keywords': ['auto'] },
    { 'longhands': ['margin-inline-start', 'margin-inline-end'], 'name': 'margin-inline' },
    { 'name': 'margin-inline-end', 'keywords': ['auto'] },
    { 'name': 'margin-inline-start', 'keywords': ['auto'] },
    { 'name': 'margin-left', 'keywords': ['auto'] },
    { 'name': 'margin-right', 'keywords': ['auto'] },
    { 'name': 'margin-top', 'keywords': ['auto'] },
    { 'longhands': ['marker-start', 'marker-mid', 'marker-end'], 'inherited': true, 'name': 'marker', 'svg': true },
    { 'svg': true, 'inherited': true, 'keywords': ['none'], 'name': 'marker-end' },
    { 'svg': true, 'inherited': true, 'keywords': ['none'], 'name': 'marker-mid' },
    { 'svg': true, 'inherited': true, 'keywords': ['none'], 'name': 'marker-start' },
    { 'svg': true, 'name': 'mask' },
    { 'keywords': ['luminance', 'alpha'], 'svg': true, 'name': 'mask-type' },
    { 'name': 'math-style', 'inherited': true, 'keywords': ['inline', 'display'] },
    { 'name': 'math-superscript-shift-style', 'inherited': true, 'keywords': ['inline', 'display'] },
    { 'keywords': ['none'], 'name': 'max-block-size' },
    { 'name': 'max-height', 'keywords': ['none'] },
    { 'keywords': ['none'], 'name': 'max-inline-size' },
    { 'name': 'max-width', 'keywords': ['none'] },
    { 'name': 'max-zoom' },
    { 'name': 'min-block-size' },
    { 'name': 'min-height' },
    { 'name': 'min-inline-size' },
    { 'name': 'min-width' },
    { 'name': 'min-zoom' },
    {
        'name': 'mix-blend-mode',
        'keywords': [
            'normal', 'multiply', 'screen', 'overlay', 'darken', 'lighten', 'color-dodge', 'color-burn', 'hard-light',
            'soft-light', 'difference', 'exclusion', 'hue', 'saturation', 'color', 'luminosity'
        ]
    },
    { 'name': 'object-fit', 'keywords': ['fill', 'contain', 'cover', 'none', 'scale-down'] },
    { 'name': 'object-position' },
    {
        'longhands': ['offset-position', 'offset-path', 'offset-distance', 'offset-rotate', 'offset-anchor'],
        'name': 'offset'
    },
    { 'name': 'offset-anchor', 'keywords': ['auto'] },
    { 'name': 'offset-distance' },
    { 'name': 'offset-path', 'keywords': ['none'] },
    { 'name': 'offset-position', 'keywords': ['auto'] },
    { 'name': 'offset-rotate', 'keywords': ['auto', 'reverse'] },
    { 'name': 'opacity' },
    { 'name': 'order' },
    { 'name': 'orientation' },
    { 'name': 'origin-trial-test-property', 'keywords': ['normal', 'none'] },
    { 'name': 'orphans', 'inherited': true },
    { 'longhands': ['outline-color', 'outline-style', 'outline-width'], 'name': 'outline' },
    { 'name': 'outline-color', 'keywords': ['currentcolor'] },
    { 'name': 'outline-offset' },
    {
        'name': 'outline-style',
        'keywords': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double']
    },
    { 'name': 'outline-width', 'keywords': ['thin', 'medium', 'thick'] },
    { 'longhands': ['overflow-x', 'overflow-y'], 'name': 'overflow' },
    { 'name': 'overflow-anchor', 'inherited': false, 'keywords': ['visible', 'none', 'auto'] },
    { 'name': 'overflow-block' },
    { 'name': 'overflow-inline' },
    { 'name': 'overflow-wrap', 'inherited': true, 'keywords': ['normal', 'break-word', 'anywhere'] },
    { 'name': 'overflow-x', 'keywords': ['visible', 'hidden', 'scroll', 'auto', 'overlay', 'clip'] },
    { 'name': 'overflow-y', 'keywords': ['visible', 'hidden', 'scroll', 'auto', 'overlay', 'clip'] },
    { 'longhands': ['overscroll-behavior-x', 'overscroll-behavior-y'], 'name': 'overscroll-behavior' },
    { 'name': 'overscroll-behavior-block' },
    { 'name': 'overscroll-behavior-inline' },
    { 'name': 'overscroll-behavior-x', 'keywords': ['auto', 'contain', 'none'] },
    { 'name': 'overscroll-behavior-y', 'keywords': ['auto', 'contain', 'none'] },
    { 'longhands': ['padding-top', 'padding-right', 'padding-bottom', 'padding-left'], 'name': 'padding' },
    { 'longhands': ['padding-block-start', 'padding-block-end'], 'name': 'padding-block' },
    { 'name': 'padding-block-end' },
    { 'name': 'padding-block-start' },
    { 'name': 'padding-bottom' },
    { 'longhands': ['padding-inline-start', 'padding-inline-end'], 'name': 'padding-inline' },
    { 'name': 'padding-inline-end' },
    { 'name': 'padding-inline-start' },
    { 'name': 'padding-left' },
    { 'name': 'padding-right' },
    { 'name': 'padding-top' },
    { 'name': 'page', 'keywords': ['auto'] },
    { 'longhands': ['break-after'], 'name': 'page-break-after' },
    { 'longhands': ['break-before'], 'name': 'page-break-before' },
    { 'longhands': ['break-inside'], 'name': 'page-break-inside' },
    { 'name': 'page-orientation' },
    { 'svg': true, 'inherited': true, 'keywords': ['normal', 'fill', 'stroke', 'markers'], 'name': 'paint-order' },
    { 'name': 'perspective', 'keywords': ['none'] },
    { 'name': 'perspective-origin' },
    { 'longhands': ['align-content', 'justify-content'], 'name': 'place-content' },
    { 'longhands': ['align-items', 'justify-items'], 'name': 'place-items' },
    { 'longhands': ['align-self', 'justify-self'], 'name': 'place-self' },
    {
        'name': 'pointer-events',
        'inherited': true,
        'keywords': [
            'none', 'auto', 'stroke', 'fill', 'painted', 'visible', 'visiblestroke', 'visiblefill', 'visiblepainted',
            'bounding-box', 'all'
        ]
    },
    { 'name': 'position', 'keywords': ['static', 'relative', 'absolute', 'fixed', 'sticky'] },
    { 'name': 'quotes', 'inherited': true, 'keywords': ['auto', 'none'] },
    { 'svg': true, 'name': 'r' },
    { 'name': 'resize', 'keywords': ['none', 'both', 'horizontal', 'vertical', 'block', 'inline'] },
    { 'name': 'right', 'keywords': ['auto'] },
    { 'name': 'rotate' },
    { 'name': 'row-gap', 'keywords': ['normal'] },
    { 'inherited': true, 'name': 'ruby-position' },
    { 'name': 'rx', 'svg': true, 'keywords': ['auto'] },
    { 'name': 'ry', 'svg': true, 'keywords': ['auto'] },
    { 'name': 'scale' },
    { 'name': 'scroll-behavior', 'keywords': ['auto', 'smooth'] },
    { 'name': 'scroll-customization' },
    {
        'longhands': ['scroll-margin-top', 'scroll-margin-right', 'scroll-margin-bottom', 'scroll-margin-left'],
        'name': 'scroll-margin'
    },
    { 'longhands': ['scroll-margin-block-start', 'scroll-margin-block-end'], 'name': 'scroll-margin-block' },
    { 'name': 'scroll-margin-block-end' },
    { 'name': 'scroll-margin-block-start' },
    { 'name': 'scroll-margin-bottom' },
    { 'longhands': ['scroll-margin-inline-start', 'scroll-margin-inline-end'], 'name': 'scroll-margin-inline' },
    { 'name': 'scroll-margin-inline-end' },
    { 'name': 'scroll-margin-inline-start' },
    { 'name': 'scroll-margin-left' },
    { 'name': 'scroll-margin-right' },
    { 'name': 'scroll-margin-top' },
    {
        'longhands': ['scroll-padding-top', 'scroll-padding-right', 'scroll-padding-bottom', 'scroll-padding-left'],
        'name': 'scroll-padding'
    },
    { 'longhands': ['scroll-padding-block-start', 'scroll-padding-block-end'], 'name': 'scroll-padding-block' },
    { 'name': 'scroll-padding-block-end', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-block-start', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-bottom', 'keywords': ['auto'] },
    { 'longhands': ['scroll-padding-inline-start', 'scroll-padding-inline-end'], 'name': 'scroll-padding-inline' },
    { 'name': 'scroll-padding-inline-end', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-inline-start', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-left', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-right', 'keywords': ['auto'] },
    { 'name': 'scroll-padding-top', 'keywords': ['auto'] },
    { 'name': 'scroll-snap-align', 'keywords': ['none', 'start', 'end', 'center'] },
    { 'name': 'scroll-snap-stop', 'keywords': ['normal', 'always'] },
    { 'name': 'scroll-snap-type', 'keywords': ['none', 'x', 'y', 'block', 'inline', 'both', 'mandatory', 'proximity'] },
    { 'name': 'scrollbar-gutter', 'inherited': false, 'keywords': ['auto', 'stable', 'always'] },
    { 'name': 'shape-image-threshold' },
    { 'name': 'shape-margin', 'keywords': ['none'] },
    { 'name': 'shape-outside', 'keywords': ['none'] },
    {
        'name': 'shape-rendering',
        'svg': true,
        'inherited': true,
        'keywords': ['auto', 'optimizespeed', 'crispedges', 'geometricprecision']
    },
    { 'name': 'size' },
    { 'name': 'source' },
    {
        'name': 'speak',
        'inherited': true,
        'keywords': ['none', 'normal', 'spell-out', 'digits', 'literal-punctuation', 'no-punctuation']
    },
    { 'name': 'src' },
    { 'name': 'start' },
    { 'name': 'stop-color', 'svg': true, 'keywords': ['currentcolor'] },
    { 'name': 'stop-opacity', 'svg': true },
    { 'name': 'stroke', 'svg': true, 'inherited': true },
    { 'name': 'stroke-dasharray', 'svg': true, 'inherited': true, 'keywords': ['none'] },
    { 'name': 'stroke-dashoffset', 'svg': true, 'inherited': true },
    { 'name': 'stroke-linecap', 'svg': true, 'inherited': true, 'keywords': ['butt', 'round', 'square'] },
    { 'name': 'stroke-linejoin', 'svg': true, 'inherited': true, 'keywords': ['miter', 'bevel', 'round'] },
    { 'name': 'stroke-miterlimit', 'svg': true, 'inherited': true },
    { 'svg': true, 'inherited': true, 'name': 'stroke-opacity' },
    { 'svg': true, 'inherited': true, 'name': 'stroke-width' },
    { 'name': 'syntax' },
    { 'name': 'tab-size', 'inherited': true },
    { 'name': 'table-layout', 'keywords': ['auto', 'fixed'] },
    {
        'name': 'text-align',
        'inherited': true,
        'keywords': ['left', 'right', 'center', 'justify', '-webkit-left', '-webkit-right', '-webkit-center', 'start', 'end']
    },
    {
        'name': 'text-align-last',
        'inherited': true,
        'keywords': ['auto', 'start', 'end', 'left', 'right', 'center', 'justify']
    },
    { 'name': 'text-anchor', 'svg': true, 'inherited': true, 'keywords': ['start', 'middle', 'end'] },
    { 'name': 'text-combine-upright', 'inherited': true, 'keywords': ['none', 'all'] },
    {
        'longhands': ['text-decoration-line', 'text-decoration-thickness', 'text-decoration-style', 'text-decoration-color'],
        'name': 'text-decoration'
    },
    { 'name': 'text-decoration-color', 'keywords': ['currentcolor'] },
    { 'name': 'text-decoration-line', 'keywords': ['none', 'underline', 'overline', 'line-through', 'blink'] },
    { 'name': 'text-decoration-skip-ink', 'inherited': true, 'keywords': ['none', 'auto'] },
    { 'name': 'text-decoration-style', 'keywords': ['solid', 'double', 'dotted', 'dashed', 'wavy'] },
    { 'name': 'text-decoration-thickness', 'inherited': true, 'keywords': ['auto', 'from-font'] },
    { 'name': 'text-indent', 'inherited': true },
    { 'name': 'text-justify', 'inherited': true, 'keywords': ['auto', 'none', 'inter-word', 'distribute'] },
    { 'name': 'text-orientation', 'inherited': true, 'keywords': ['sideways', 'mixed', 'upright'] },
    { 'name': 'text-overflow', 'keywords': ['clip', 'ellipsis'] },
    {
        'name': 'text-rendering',
        'inherited': true,
        'keywords': ['auto', 'optimizespeed', 'optimizelegibility', 'geometricprecision']
    },
    { 'name': 'text-shadow', 'inherited': true, 'keywords': ['none'] },
    { 'name': 'text-size-adjust', 'inherited': true, 'keywords': ['none', 'auto'] },
    {
        'name': 'text-transform',
        'inherited': true,
        'keywords': ['capitalize', 'uppercase', 'lowercase', 'none', 'math-auto']
    },
    { 'name': 'text-underline-offset', 'inherited': true, 'keywords': ['auto'] },
    { 'name': 'text-underline-position', 'inherited': true, 'keywords': ['auto', 'from-font', 'under', 'left', 'right'] },
    { 'name': 'time-range' },
    { 'name': 'top', 'keywords': ['auto'] },
    {
        'name': 'touch-action',
        'keywords': ['auto', 'none', 'pan-x', 'pan-left', 'pan-right', 'pan-y', 'pan-up', 'pan-down', 'pinch-zoom', 'manipulation']
    },
    { 'name': 'transform', 'keywords': ['none'] },
    { 'name': 'transform-box', 'keywords': ['fill-box', 'view-box'] },
    { 'name': 'transform-origin' },
    { 'name': 'transform-style', 'keywords': ['flat', 'preserve-3d'] },
    {
        'longhands': ['transition-property', 'transition-duration', 'transition-timing-function', 'transition-delay'],
        'name': 'transition'
    },
    { 'name': 'transition-delay' },
    { 'name': 'transition-duration' },
    { 'name': 'transition-property', 'keywords': ['none'] },
    {
        'name': 'transition-timing-function',
        'keywords': [
            'linear', 'ease', 'ease-in', 'ease-out', 'ease-in-out', 'jump-both', 'jump-end', 'jump-none', 'jump-start',
            'step-start', 'step-end'
        ]
    },
    { 'name': 'translate' },
    {
        'name': 'unicode-bidi',
        'keywords': ['normal', 'embed', 'bidi-override', 'isolate', 'plaintext', 'isolate-override']
    },
    { 'name': 'unicode-range' },
    { 'name': 'user-select', 'inherited': true, 'keywords': ['auto', 'none', 'text', 'all'] },
    { 'name': 'user-zoom' },
    { 'keywords': ['none', 'non-scaling-stroke'], 'svg': true, 'name': 'vector-effect' },
    { 'name': 'vertical-align', 'keywords': ['baseline', 'sub', 'super', 'text-top', 'text-bottom', 'middle'] },
    { 'name': 'viewport-fit' },
    { 'name': 'visibility', 'inherited': true, 'keywords': ['visible', 'hidden', 'collapse'] },
    {
        'name': 'white-space',
        'inherited': true,
        'keywords': ['none', 'normal', 'pre', 'pre-wrap', 'pre-line', 'nowrap', '-webkit-nowrap', 'break-spaces']
    },
    { 'name': 'widows', 'inherited': true },
    { 'name': 'width', 'keywords': ['auto', 'fit-content', 'min-content', 'max-content'] },
    { 'keywords': ['auto'], 'name': 'will-change' },
    { 'name': 'word-break', 'inherited': true, 'keywords': ['normal', 'break-all', 'keep-all', 'break-word'] },
    { 'inherited': true, 'keywords': ['normal'], 'name': 'word-spacing' },
    { 'name': 'writing-mode', 'inherited': true, 'keywords': ['horizontal-tb', 'vertical-rl', 'vertical-lr'] },
    { 'svg': true, 'name': 'x' },
    { 'svg': true, 'name': 'y' },
    { 'name': 'z-index', 'keywords': ['auto'] },
    { 'name': 'zoom' }
];
const generatedPropertyValues = {
    '-webkit-app-region': { 'values': ['none', 'drag', 'no-drag'] },
    '-webkit-box-align': { 'values': ['stretch', 'start', 'center', 'end', 'baseline'] },
    '-webkit-box-decoration-break': { 'values': ['slice', 'clone'] },
    '-webkit-box-direction': { 'values': ['normal', 'reverse'] },
    '-webkit-box-orient': { 'values': ['horizontal', 'vertical'] },
    '-webkit-box-pack': { 'values': ['start', 'center', 'end', 'justify'] },
    '-webkit-line-break': { 'values': ['auto', 'loose', 'normal', 'strict', 'after-white-space', 'anywhere'] },
    '-webkit-print-color-adjust': { 'values': ['economy', 'exact'] },
    '-webkit-rtl-ordering': { 'values': ['logical', 'visual'] },
    '-webkit-ruby-position': { 'values': ['before', 'after'] },
    '-webkit-text-security': { 'values': ['none', 'disc', 'circle', 'square'] },
    '-webkit-user-drag': { 'values': ['auto', 'none', 'element'] },
    '-webkit-user-modify': { 'values': ['read-only', 'read-write', 'read-write-plaintext-only'] },
    'alignment-baseline': { 'values': ['baseline', 'alphabetic', 'ideographic', 'middle', 'central', 'mathematical'] },
    'animation-direction': { 'values': ['normal', 'reverse', 'alternate', 'alternate-reverse'] },
    'animation-fill-mode': { 'values': ['none', 'forwards', 'backwards', 'both'] },
    'animation-iteration-count': { 'values': ['infinite'] },
    'animation-name': { 'values': ['none'] },
    'animation-play-state': { 'values': ['running', 'paused'] },
    'animation-timeline': { 'values': ['none', 'auto'] },
    'animation-timing-function': {
        'values': [
            'linear', 'ease', 'ease-in', 'ease-out', 'ease-in-out', 'jump-both', 'jump-end', 'jump-none', 'jump-start',
            'step-start', 'step-end'
        ]
    },
    'aspect-ratio': { 'values': ['auto'] },
    'backdrop-filter': { 'values': ['none'] },
    'backface-visibility': { 'values': ['visible', 'hidden'] },
    'background-attachment': { 'values': ['scroll', 'fixed', 'local'] },
    'background-blend-mode': {
        'values': [
            'normal', 'multiply', 'screen', 'overlay', 'darken', 'lighten', 'color-dodge', 'color-burn', 'hard-light',
            'soft-light', 'difference', 'exclusion', 'hue', 'saturation', 'color', 'luminosity'
        ]
    },
    'background-clip': { 'values': ['border-box', 'padding-box', 'content-box'] },
    'background-color': { 'values': ['currentcolor'] },
    'background-image': { 'values': ['auto', 'none'] },
    'background-origin': { 'values': ['border-box', 'padding-box', 'content-box'] },
    'background-size': { 'values': ['auto', 'cover', 'contain'] },
    'baseline-shift': { 'values': ['sub', 'super'] },
    'block-size': { 'values': ['auto'] },
    'border-bottom-color': { 'values': ['currentcolor'] },
    'border-bottom-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'border-bottom-width': { 'values': ['thin', 'medium', 'thick'] },
    'border-collapse': { 'values': ['separate', 'collapse'] },
    'border-image-repeat': { 'values': ['stretch', 'repeat', 'round', 'space'] },
    'border-image-source': { 'values': ['none'] },
    'border-image-width': { 'values': ['auto'] },
    'border-left-color': { 'values': ['currentcolor'] },
    'border-left-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'border-left-width': { 'values': ['thin', 'medium', 'thick'] },
    'border-right-color': { 'values': ['currentcolor'] },
    'border-right-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'border-right-width': { 'values': ['thin', 'medium', 'thick'] },
    'border-style': { 'values': ['none'] },
    'border-top-color': { 'values': ['currentcolor'] },
    'border-top-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'border-top-width': { 'values': ['thin', 'medium', 'thick'] },
    'bottom': { 'values': ['auto'] },
    'box-shadow': { 'values': ['none'] },
    'box-sizing': { 'values': ['content-box', 'border-box'] },
    'break-after': { 'values': ['auto', 'avoid', 'avoid-column', 'avoid-page', 'column', 'left', 'page', 'recto', 'right', 'verso'] },
    'break-before': { 'values': ['auto', 'avoid', 'avoid-column', 'avoid-page', 'column', 'left', 'page', 'recto', 'right', 'verso'] },
    'break-inside': { 'values': ['auto', 'avoid', 'avoid-column', 'avoid-page'] },
    'caption-side': { 'values': ['top', 'bottom'] },
    'caret-color': { 'values': ['auto', 'currentcolor'] },
    'clear': { 'values': ['none', 'left', 'right', 'both', 'inline-start', 'inline-end'] },
    'clip': { 'values': ['auto'] },
    'clip-path': { 'values': ['none'] },
    'clip-rule': { 'values': ['nonzero', 'evenodd'] },
    'color': { 'values': ['currentcolor'] },
    'color-interpolation': { 'values': ['auto', 'srgb', 'linearrgb'] },
    'color-rendering': { 'values': ['auto', 'optimizespeed', 'optimizequality'] },
    'column-count': { 'values': ['auto'] },
    'column-fill': { 'values': ['balance', 'auto'] },
    'column-gap': { 'values': ['normal'] },
    'column-rule-color': { 'values': ['currentcolor'] },
    'column-rule-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'column-rule-width': { 'values': ['thin', 'medium', 'thick'] },
    'column-span': { 'values': ['none', 'all'] },
    'column-width': { 'values': ['auto'] },
    'contain': { 'values': ['none', 'strict', 'content', 'size', 'layout', 'style', 'paint'] },
    'contain-intrinsic-size': { 'values': ['auto'] },
    'content-visibility': { 'values': ['visible', 'auto', 'hidden', 'hidden-matchable'] },
    'counter-increment': { 'values': ['none'] },
    'counter-reset': { 'values': ['none'] },
    'counter-set': { 'values': ['none'] },
    'cursor': {
        'values': [
            'auto', 'default', 'none', 'context-menu', 'help', 'pointer', 'progress', 'wait',
            'cell', 'crosshair', 'text', 'vertical-text', 'alias', 'copy', 'move', 'no-drop',
            'not-allowed', 'e-resize', 'n-resize', 'ne-resize', 'nw-resize', 's-resize', 'se-resize', 'sw-resize',
            'w-resize', 'ew-resize', 'ns-resize', 'nesw-resize', 'nwse-resize', 'col-resize', 'row-resize', 'all-scroll',
            'zoom-in', 'zoom-out', 'grab', 'grabbing'
        ]
    },
    'd': { 'values': ['none'] },
    'direction': { 'values': ['ltr', 'rtl'] },
    'display': {
        'values': [
            'inline',
            'block',
            'list-item',
            'inline-block',
            'table',
            'inline-table',
            'table-row-group',
            'table-header-group',
            'table-footer-group',
            'table-row',
            'table-column-group',
            'table-column',
            'table-cell',
            'table-caption',
            '-webkit-box',
            '-webkit-inline-box',
            'flex',
            'inline-flex',
            'grid',
            'inline-grid',
            'contents',
            'flow-root',
            'none'
        ]
    },
    'dominant-baseline': { 'values': ['auto', 'alphabetic', 'ideographic', 'middle', 'central', 'mathematical', 'hanging'] },
    'empty-cells': { 'values': ['show', 'hide'] },
    'fill-rule': { 'values': ['nonzero', 'evenodd'] },
    'filter': { 'values': ['none'] },
    'flex-basis': { 'values': ['auto'] },
    'flex-direction': { 'values': ['row', 'row-reverse', 'column', 'column-reverse'] },
    'flex-wrap': { 'values': ['nowrap', 'wrap', 'wrap-reverse'] },
    'float': { 'values': ['none', 'left', 'right', 'inline-start', 'inline-end'] },
    'flood-color': { 'values': ['currentcolor'] },
    'font-feature-settings': { 'values': ['normal'] },
    'font-kerning': { 'values': ['auto', 'normal', 'none'] },
    'font-optical-sizing': { 'values': ['auto', 'none'] },
    'font-size': {
        'values': [
            'xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large', 'xxx-large', 'larger', 'smaller',
            '-webkit-xxx-large'
        ]
    },
    'font-size-adjust': { 'values': ['none'] },
    'font-stretch': {
        'values': [
            'normal', 'ultra-condensed', 'extra-condensed', 'condensed', 'semi-condensed', 'semi-expanded', 'expanded',
            'extra-expanded', 'ultra-expanded'
        ]
    },
    'font-style': { 'values': ['normal', 'italic', 'oblique'] },
    'font-variant-caps': {
        'values': ['normal', 'small-caps', 'all-small-caps', 'petite-caps', 'all-petite-caps', 'unicase', 'titling-caps']
    },
    'font-variant-east-asian': {
        'values': [
            'normal', 'jis78', 'jis83', 'jis90', 'jis04', 'simplified', 'traditional', 'full-width', 'proportional-width',
            'ruby'
        ]
    },
    'font-variant-ligatures': {
        'values': [
            'normal', 'none', 'common-ligatures', 'no-common-ligatures', 'discretionary-ligatures',
            'no-discretionary-ligatures', 'historical-ligatures', 'no-historical-ligatures', 'contextual', 'no-contextual'
        ]
    },
    'font-variant-numeric': {
        'values': [
            'normal', 'lining-nums', 'oldstyle-nums', 'proportional-nums', 'tabular-nums', 'diagonal-fractions',
            'stacked-fractions', 'ordinal', 'slashed-zero'
        ]
    },
    'font-variation-settings': { 'values': ['normal'] },
    'font-weight': { 'values': ['normal', 'bold', 'bolder', 'lighter'] },
    'forced-color-adjust': { 'values': ['auto', 'none'] },
    'grid-auto-columns': { 'values': ['auto', 'min-content', 'max-content'] },
    'grid-auto-flow': { 'values': ['row', 'column'] },
    'grid-auto-rows': { 'values': ['auto', 'min-content', 'max-content'] },
    'grid-column-end': { 'values': ['auto'] },
    'grid-column-start': { 'values': ['auto'] },
    'grid-row-end': { 'values': ['auto'] },
    'grid-row-start': { 'values': ['auto'] },
    'grid-template-areas': { 'values': ['none'] },
    'grid-template-columns': { 'values': ['none'] },
    'grid-template-rows': { 'values': ['none'] },
    'height': { 'values': ['auto', 'fit-content', 'min-content', 'max-content'] },
    'hyphens': { 'values': ['none', 'manual', 'auto'] },
    'image-rendering': { 'values': ['auto', 'optimizespeed', 'optimizequality', '-webkit-optimize-contrast', 'pixelated'] },
    'inline-size': { 'values': ['auto'] },
    'isolation': { 'values': ['auto', 'isolate'] },
    'left': { 'values': ['auto'] },
    'letter-spacing': { 'values': ['normal'] },
    'lighting-color': { 'values': ['currentcolor'] },
    'line-break': { 'values': ['auto', 'loose', 'normal', 'strict', 'anywhere'] },
    'line-height': { 'values': ['normal'] },
    'list-style-image': { 'values': ['none'] },
    'list-style-position': { 'values': ['outside', 'inside'] },
    'list-style-type': {
        'values': [
            'disc',
            'circle',
            'square',
            'decimal',
            'decimal-leading-zero',
            'arabic-indic',
            'bengali',
            'cambodian',
            'khmer',
            'devanagari',
            'gujarati',
            'gurmukhi',
            'kannada',
            'lao',
            'malayalam',
            'mongolian',
            'myanmar',
            'oriya',
            'persian',
            'urdu',
            'telugu',
            'tibetan',
            'thai',
            'lower-roman',
            'upper-roman',
            'lower-greek',
            'lower-alpha',
            'lower-latin',
            'upper-alpha',
            'upper-latin',
            'cjk-earthly-branch',
            'cjk-heavenly-stem',
            'ethiopic-halehame',
            'ethiopic-halehame-am',
            'ethiopic-halehame-ti-er',
            'ethiopic-halehame-ti-et',
            'hangul',
            'hangul-consonant',
            'korean-hangul-formal',
            'korean-hanja-formal',
            'korean-hanja-informal',
            'hebrew',
            'armenian',
            'lower-armenian',
            'upper-armenian',
            'georgian',
            'cjk-ideographic',
            'simp-chinese-formal',
            'simp-chinese-informal',
            'trad-chinese-formal',
            'trad-chinese-informal',
            'hiragana',
            'katakana',
            'hiragana-iroha',
            'katakana-iroha',
            'none'
        ]
    },
    'margin-block-end': { 'values': ['auto'] },
    'margin-block-start': { 'values': ['auto'] },
    'margin-bottom': { 'values': ['auto'] },
    'margin-inline-end': { 'values': ['auto'] },
    'margin-inline-start': { 'values': ['auto'] },
    'margin-left': { 'values': ['auto'] },
    'margin-right': { 'values': ['auto'] },
    'margin-top': { 'values': ['auto'] },
    'marker-end': { 'values': ['none'] },
    'marker-mid': { 'values': ['none'] },
    'marker-start': { 'values': ['none'] },
    'mask-type': { 'values': ['luminance', 'alpha'] },
    'math-style': { 'values': ['inline', 'display'] },
    'math-superscript-shift-style': { 'values': ['inline', 'display'] },
    'max-block-size': { 'values': ['none'] },
    'max-height': { 'values': ['none'] },
    'max-inline-size': { 'values': ['none'] },
    'max-width': { 'values': ['none'] },
    'mix-blend-mode': {
        'values': [
            'normal', 'multiply', 'screen', 'overlay', 'darken', 'lighten', 'color-dodge', 'color-burn', 'hard-light',
            'soft-light', 'difference', 'exclusion', 'hue', 'saturation', 'color', 'luminosity'
        ]
    },
    'object-fit': { 'values': ['fill', 'contain', 'cover', 'none', 'scale-down'] },
    'offset-anchor': { 'values': ['auto'] },
    'offset-path': { 'values': ['none'] },
    'offset-position': { 'values': ['auto'] },
    'offset-rotate': { 'values': ['auto', 'reverse'] },
    'origin-trial-test-property': { 'values': ['normal', 'none'] },
    'outline-color': { 'values': ['currentcolor'] },
    'outline-style': { 'values': ['none', 'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    'outline-width': { 'values': ['thin', 'medium', 'thick'] },
    'overflow-anchor': { 'values': ['visible', 'none', 'auto'] },
    'overflow-wrap': { 'values': ['normal', 'break-word', 'anywhere'] },
    'overflow-x': { 'values': ['visible', 'hidden', 'scroll', 'auto', 'overlay', 'clip'] },
    'overflow-y': { 'values': ['visible', 'hidden', 'scroll', 'auto', 'overlay', 'clip'] },
    'overscroll-behavior-x': { 'values': ['auto', 'contain', 'none'] },
    'overscroll-behavior-y': { 'values': ['auto', 'contain', 'none'] },
    'page': { 'values': ['auto'] },
    'paint-order': { 'values': ['normal', 'fill', 'stroke', 'markers'] },
    'perspective': { 'values': ['none'] },
    'pointer-events': {
        'values': [
            'none', 'auto', 'stroke', 'fill', 'painted', 'visible', 'visiblestroke', 'visiblefill', 'visiblepainted',
            'bounding-box', 'all'
        ]
    },
    'position': { 'values': ['static', 'relative', 'absolute', 'fixed', 'sticky'] },
    'quotes': { 'values': ['auto', 'none'] },
    'resize': { 'values': ['none', 'both', 'horizontal', 'vertical', 'block', 'inline'] },
    'right': { 'values': ['auto'] },
    'row-gap': { 'values': ['normal'] },
    'rx': { 'values': ['auto'] },
    'ry': { 'values': ['auto'] },
    'scroll-behavior': { 'values': ['auto', 'smooth'] },
    'scroll-padding-block-end': { 'values': ['auto'] },
    'scroll-padding-block-start': { 'values': ['auto'] },
    'scroll-padding-bottom': { 'values': ['auto'] },
    'scroll-padding-inline-end': { 'values': ['auto'] },
    'scroll-padding-inline-start': { 'values': ['auto'] },
    'scroll-padding-left': { 'values': ['auto'] },
    'scroll-padding-right': { 'values': ['auto'] },
    'scroll-padding-top': { 'values': ['auto'] },
    'scroll-snap-align': { 'values': ['none', 'start', 'end', 'center'] },
    'scroll-snap-stop': { 'values': ['normal', 'always'] },
    'scroll-snap-type': { 'values': ['none', 'x', 'y', 'block', 'inline', 'both', 'mandatory', 'proximity'] },
    'scrollbar-gutter': { 'values': ['auto', 'stable', 'always'] },
    'shape-margin': { 'values': ['none'] },
    'shape-outside': { 'values': ['none'] },
    'shape-rendering': { 'values': ['auto', 'optimizespeed', 'crispedges', 'geometricprecision'] },
    'speak': { 'values': ['none', 'normal', 'spell-out', 'digits', 'literal-punctuation', 'no-punctuation'] },
    'stop-color': { 'values': ['currentcolor'] },
    'stroke-dasharray': { 'values': ['none'] },
    'stroke-linecap': { 'values': ['butt', 'round', 'square'] },
    'stroke-linejoin': { 'values': ['miter', 'bevel', 'round'] },
    'table-layout': { 'values': ['auto', 'fixed'] },
    'text-align': {
        'values': ['left', 'right', 'center', 'justify', '-webkit-left', '-webkit-right', '-webkit-center', 'start', 'end']
    },
    'text-align-last': { 'values': ['auto', 'start', 'end', 'left', 'right', 'center', 'justify'] },
    'text-anchor': { 'values': ['start', 'middle', 'end'] },
    'text-combine-upright': { 'values': ['none', 'all'] },
    'text-decoration-color': { 'values': ['currentcolor'] },
    'text-decoration-line': { 'values': ['none', 'underline', 'overline', 'line-through', 'blink'] },
    'text-decoration-skip-ink': { 'values': ['none', 'auto'] },
    'text-decoration-style': { 'values': ['solid', 'double', 'dotted', 'dashed', 'wavy'] },
    'text-decoration-thickness': { 'values': ['auto', 'from-font'] },
    'text-justify': { 'values': ['auto', 'none', 'inter-word', 'distribute'] },
    'text-orientation': { 'values': ['sideways', 'mixed', 'upright'] },
    'text-overflow': { 'values': ['clip', 'ellipsis'] },
    'text-rendering': { 'values': ['auto', 'optimizespeed', 'optimizelegibility', 'geometricprecision'] },
    'text-shadow': { 'values': ['none'] },
    'text-size-adjust': { 'values': ['none', 'auto'] },
    'text-transform': { 'values': ['capitalize', 'uppercase', 'lowercase', 'none', 'math-auto'] },
    'text-underline-offset': { 'values': ['auto'] },
    'text-underline-position': { 'values': ['auto', 'from-font', 'under', 'left', 'right'] },
    'top': { 'values': ['auto'] },
    'touch-action': {
        'values': ['auto', 'none', 'pan-x', 'pan-left', 'pan-right', 'pan-y', 'pan-up', 'pan-down', 'pinch-zoom', 'manipulation']
    },
    'transform': { 'values': ['none'] },
    'transform-box': { 'values': ['fill-box', 'view-box'] },
    'transform-style': { 'values': ['flat', 'preserve-3d'] },
    'transition-property': { 'values': ['none'] },
    'transition-timing-function': {
        'values': [
            'linear', 'ease', 'ease-in', 'ease-out', 'ease-in-out', 'jump-both', 'jump-end', 'jump-none', 'jump-start',
            'step-start', 'step-end'
        ]
    },
    'unicode-bidi': { 'values': ['normal', 'embed', 'bidi-override', 'isolate', 'plaintext', 'isolate-override'] },
    'user-select': { 'values': ['auto', 'none', 'text', 'all'] },
    'vector-effect': { 'values': ['none', 'non-scaling-stroke'] },
    'vertical-align': { 'values': ['baseline', 'sub', 'super', 'text-top', 'text-bottom', 'middle'] },
    'visibility': { 'values': ['visible', 'hidden', 'collapse'] },
    'white-space': { 'values': ['none', 'normal', 'pre', 'pre-wrap', 'pre-line', 'nowrap', '-webkit-nowrap', 'break-spaces'] },
    'width': { 'values': ['auto', 'fit-content', 'min-content', 'max-content'] },
    'will-change': { 'values': ['auto'] },
    'word-break': { 'values': ['normal', 'break-all', 'keep-all', 'break-word'] },
    'word-spacing': { 'values': ['normal'] },
    'writing-mode': { 'values': ['horizontal-tb', 'vertical-rl', 'vertical-lr'] },
    'z-index': { 'values': ['auto'] }
};
const generatedAliasesFor = new Map([
    ['-epub-caption-side', 'caption-side'],
    ['-epub-text-combine', '-webkit-text-combine'],
    ['-epub-text-emphasis', '-webkit-text-emphasis'],
    ['-epub-text-emphasis-color', '-webkit-text-emphasis-color'],
    ['-epub-text-emphasis-style', '-webkit-text-emphasis-style'],
    ['-epub-text-orientation', '-webkit-text-orientation'],
    ['-epub-text-transform', 'text-transform'],
    ['-epub-word-break', 'word-break'],
    ['-epub-writing-mode', '-webkit-writing-mode'],
    ['-webkit-align-content', 'align-content'],
    ['-webkit-align-items', 'align-items'],
    ['-webkit-align-self', 'align-self'],
    ['-webkit-animation', 'animation'],
    ['-webkit-animation-delay', 'animation-delay'],
    ['-webkit-animation-direction', 'animation-direction'],
    ['-webkit-animation-duration', 'animation-duration'],
    ['-webkit-animation-fill-mode', 'animation-fill-mode'],
    ['-webkit-animation-iteration-count', 'animation-iteration-count'],
    ['-webkit-animation-name', 'animation-name'],
    ['-webkit-animation-play-state', 'animation-play-state'],
    ['-webkit-animation-timing-function', 'animation-timing-function'],
    ['-webkit-appearance', 'appearance'],
    ['-webkit-backface-visibility', 'backface-visibility'],
    ['-webkit-background-clip', 'background-clip'],
    ['-webkit-background-origin', 'background-origin'],
    ['-webkit-background-size', 'background-size'],
    ['-webkit-border-after', 'border-block-end'],
    ['-webkit-border-after-color', 'border-block-end-color'],
    ['-webkit-border-after-style', 'border-block-end-style'],
    ['-webkit-border-after-width', 'border-block-end-width'],
    ['-webkit-border-before', 'border-block-start'],
    ['-webkit-border-before-color', 'border-block-start-color'],
    ['-webkit-border-before-style', 'border-block-start-style'],
    ['-webkit-border-before-width', 'border-block-start-width'],
    ['-webkit-border-bottom-left-radius', 'border-bottom-left-radius'],
    ['-webkit-border-bottom-right-radius', 'border-bottom-right-radius'],
    ['-webkit-border-end', 'border-inline-end'],
    ['-webkit-border-end-color', 'border-inline-end-color'],
    ['-webkit-border-end-style', 'border-inline-end-style'],
    ['-webkit-border-end-width', 'border-inline-end-width'],
    ['-webkit-border-radius', 'border-radius'],
    ['-webkit-border-start', 'border-inline-start'],
    ['-webkit-border-start-color', 'border-inline-start-color'],
    ['-webkit-border-start-style', 'border-inline-start-style'],
    ['-webkit-border-start-width', 'border-inline-start-width'],
    ['-webkit-border-top-left-radius', 'border-top-left-radius'],
    ['-webkit-border-top-right-radius', 'border-top-right-radius'],
    ['-webkit-box-shadow', 'box-shadow'],
    ['-webkit-box-sizing', 'box-sizing'],
    ['-webkit-clip-path', 'clip-path'],
    ['-webkit-column-count', 'column-count'],
    ['-webkit-column-gap', 'column-gap'],
    ['-webkit-column-rule', 'column-rule'],
    ['-webkit-column-rule-color', 'column-rule-color'],
    ['-webkit-column-rule-style', 'column-rule-style'],
    ['-webkit-column-rule-width', 'column-rule-width'],
    ['-webkit-column-span', 'column-span'],
    ['-webkit-column-width', 'column-width'],
    ['-webkit-columns', 'columns'],
    ['-webkit-filter', 'filter'],
    ['-webkit-flex', 'flex'],
    ['-webkit-flex-basis', 'flex-basis'],
    ['-webkit-flex-direction', 'flex-direction'],
    ['-webkit-flex-flow', 'flex-flow'],
    ['-webkit-flex-grow', 'flex-grow'],
    ['-webkit-flex-shrink', 'flex-shrink'],
    ['-webkit-flex-wrap', 'flex-wrap'],
    ['-webkit-font-feature-settings', 'font-feature-settings'],
    ['-webkit-justify-content', 'justify-content'],
    ['-webkit-logical-height', 'block-size'],
    ['-webkit-logical-width', 'inline-size'],
    ['-webkit-margin-after', 'margin-block-end'],
    ['-webkit-margin-before', 'margin-block-start'],
    ['-webkit-margin-end', 'margin-inline-end'],
    ['-webkit-margin-start', 'margin-inline-start'],
    ['-webkit-max-logical-height', 'max-block-size'],
    ['-webkit-max-logical-width', 'max-inline-size'],
    ['-webkit-min-logical-height', 'min-block-size'],
    ['-webkit-min-logical-width', 'min-inline-size'],
    ['-webkit-opacity', 'opacity'],
    ['-webkit-order', 'order'],
    ['-webkit-padding-after', 'padding-block-end'],
    ['-webkit-padding-before', 'padding-block-start'],
    ['-webkit-padding-end', 'padding-inline-end'],
    ['-webkit-padding-start', 'padding-inline-start'],
    ['-webkit-perspective', 'perspective'],
    ['-webkit-perspective-origin', 'perspective-origin'],
    ['-webkit-shape-image-threshold', 'shape-image-threshold'],
    ['-webkit-shape-margin', 'shape-margin'],
    ['-webkit-shape-outside', 'shape-outside'],
    ['-webkit-text-size-adjust', 'text-size-adjust'],
    ['-webkit-transform', 'transform'],
    ['-webkit-transform-origin', 'transform-origin'],
    ['-webkit-transform-style', 'transform-style'],
    ['-webkit-transition', 'transition'],
    ['-webkit-transition-delay', 'transition-delay'],
    ['-webkit-transition-duration', 'transition-duration'],
    ['-webkit-transition-property', 'transition-property'],
    ['-webkit-transition-timing-function', 'transition-timing-function'],
    ['-webkit-user-select', 'user-select'],
    ['word-wrap', 'overflow-wrap']
]);

/*
 * Copyright (C) 2010 Nikita Vasilyev. All rights reserved.
 * Copyright (C) 2010 Joseph Pecoraro. All rights reserved.
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class CSSMetadata {
    /**
     * @param {!Array<!CSSPropertyDefinition>} properties
     * @param {!Map<string, string>} aliasesFor
     */
    constructor(properties, aliasesFor) {
        /** @type {!Array<string>} */
        this._values = [];
        /** @type {!Map<string, !Array<string>>} */
        this._longhands = new Map();
        /** @type {!Map<string, !Array<string>>} */
        this._shorthands = new Map();
        /** @type {!Set<string>} */
        this._inherited = new Set();
        /** @type {!Set<string>} */
        this._svgProperties = new Set();
        /** @type {!Map<string, !Array<string>>} */
        this._propertyValues = new Map();
        /** @type {!Map<string, string>} */
        this._aliasesFor = aliasesFor;
        for (let i = 0; i < properties.length; ++i) {
            const property = properties[i];
            const propertyName = property.name;
            if (!CSS.supports(propertyName, 'initial')) {
                continue;
            }
            this._values.push(propertyName);
            if (property.inherited) {
                this._inherited.add(propertyName);
            }
            if (property.svg) {
                this._svgProperties.add(propertyName);
            }
            const longhands = properties[i].longhands;
            if (longhands) {
                this._longhands.set(propertyName, longhands);
                for (let j = 0; j < longhands.length; ++j) {
                    const longhandName = longhands[j];
                    let shorthands = this._shorthands.get(longhandName);
                    if (!shorthands) {
                        shorthands = [];
                        this._shorthands.set(longhandName, shorthands);
                    }
                    shorthands.push(propertyName);
                }
            }
        }
        this._values.sort(CSSMetadata._sortPrefixesToEnd);
        this._valuesSet = new Set(this._values);
        // Reads in auto-generated property names and values from blink/public/renderer/core/css/css_properties.json5
        // treats _generatedPropertyValues as basis
        const propertyValueSets = new Map();
        for (const [propertyName, basisValueObj] of Object.entries(generatedPropertyValues)) {
            propertyValueSets.set(propertyName, new Set(basisValueObj.values));
        }
        // and add manually maintained map of extra prop-value pairs
        for (const [propertyName, extraValueObj] of Object.entries(_extraPropertyValues)) {
            if (propertyValueSets.has(propertyName)) {
                SetUtilities.addAll(propertyValueSets.get(propertyName), extraValueObj.values);
            }
            else {
                propertyValueSets.set(propertyName, new Set(extraValueObj.values));
            }
        }
        // finally add common keywords to value sets and convert property values
        // into arrays since callers expect arrays
        for (const [propertyName, values] of propertyValueSets) {
            for (const commonKeyword of CommonKeywords) {
                if (!values.has(commonKeyword) && CSS.supports(propertyName, commonKeyword)) {
                    values.add(commonKeyword);
                }
            }
            this._propertyValues.set(propertyName, [...values]);
        }
        /** @type {!Array<string>} */
        this._nameValuePresets = [];
        /** @type {!Array<string>} */
        this._nameValuePresetsIncludingSVG = [];
        for (const name of this._valuesSet) {
            const values = this._specificPropertyValues(name)
                .filter(value => CSS.supports(name, value))
                .sort(CSSMetadata._sortPrefixesToEnd);
            const presets = values.map(value => `${name}: ${value}`);
            if (!this.isSVGProperty(name)) {
                this._nameValuePresets.push(...presets);
            }
            this._nameValuePresetsIncludingSVG.push(...presets);
        }
    }
    /**
     * @param {string} a
     * @param {string} b
     */
    static _sortPrefixesToEnd(a, b) {
        const aIsPrefixed = a.startsWith('-webkit-');
        const bIsPrefixed = b.startsWith('-webkit-');
        if (aIsPrefixed && !bIsPrefixed) {
            return 1;
        }
        if (!aIsPrefixed && bIsPrefixed) {
            return -1;
        }
        return a < b ? -1 : (a > b ? 1 : 0);
    }
    /**
     * @return {!Array<string>}
     */
    allProperties() {
        return this._values;
    }
    /**
     * @param {boolean=} includeSVG
     * @return {!Array<string>}
     */
    nameValuePresets(includeSVG) {
        return includeSVG ? this._nameValuePresetsIncludingSVG : this._nameValuePresets;
    }
    /**
     * @param {string} name
     * @return {boolean}
     */
    isSVGProperty(name) {
        name = name.toLowerCase();
        return this._svgProperties.has(name);
    }
    /**
     * @param {string} shorthand
     * @return {?Array<string>}
     */
    longhands(shorthand) {
        return this._longhands.get(shorthand) || null;
    }
    /**
     * @param {string} longhand
     * @return {?Array<string>}
     */
    shorthands(longhand) {
        return this._shorthands.get(longhand) || null;
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isColorAwareProperty(propertyName) {
        return !!_colorAwareProperties.has(propertyName.toLowerCase()) || this.isCustomProperty(propertyName.toLowerCase());
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isFontFamilyProperty(propertyName) {
        return propertyName.toLowerCase() === 'font-family';
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isGridAreaDefiningProperty(propertyName) {
        propertyName = propertyName.toLowerCase();
        return propertyName === 'grid' || propertyName === 'grid-template' || propertyName === 'grid-template-areas';
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isLengthProperty(propertyName) {
        propertyName = propertyName.toLowerCase();
        if (propertyName === 'line-height') {
            return false;
        }
        return _distanceProperties.has(propertyName) || propertyName.startsWith('margin') ||
            propertyName.startsWith('padding') || propertyName.indexOf('width') !== -1 ||
            propertyName.indexOf('height') !== -1;
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isBezierAwareProperty(propertyName) {
        propertyName = propertyName.toLowerCase();
        return !!_bezierAwareProperties.has(propertyName) || this.isCustomProperty(propertyName);
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isCustomProperty(propertyName) {
        return propertyName.startsWith('--');
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isShadowProperty(propertyName) {
        propertyName = propertyName.toLowerCase();
        return propertyName === 'box-shadow' || propertyName === 'text-shadow' || propertyName === '-webkit-box-shadow';
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isStringProperty(propertyName) {
        propertyName = propertyName.toLowerCase();
        // TODO(crbug.com/1033910): Generalize this to all CSS properties
        // that accept <string> values.
        return propertyName === 'content';
    }
    /**
     * @param {string} name
     * @return {string}
     */
    canonicalPropertyName(name) {
        if (this.isCustomProperty(name)) {
            return name;
        }
        name = name.toLowerCase();
        const aliasFor = this._aliasesFor.get(name);
        if (aliasFor) {
            return aliasFor;
        }
        if (!name || name.length < 9 || name.charAt(0) !== '-') {
            return name;
        }
        const match = name.match(/(?:-webkit-)(.+)/);
        if (!match || !this._valuesSet.has(match[1])) {
            return name;
        }
        return match[1];
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isCSSPropertyName(propertyName) {
        propertyName = propertyName.toLowerCase();
        if ((propertyName.startsWith('--') && propertyName.length > 2) || propertyName.startsWith('-moz-') ||
            propertyName.startsWith('-ms-') || propertyName.startsWith('-o-') || propertyName.startsWith('-webkit-')) {
            return true;
        }
        return this._valuesSet.has(propertyName);
    }
    /**
     * @param {string} propertyName
     * @return {boolean}
     */
    isPropertyInherited(propertyName) {
        propertyName = propertyName.toLowerCase();
        return propertyName.startsWith('--') || this._inherited.has(this.canonicalPropertyName(propertyName)) ||
            this._inherited.has(propertyName);
    }
    /**
     * @param {string} propertyName
     * @return {!Array<string>}
     */
    _specificPropertyValues(propertyName) {
        const unprefixedName = propertyName.replace(/^-webkit-/, '');
        const propertyValues = this._propertyValues;
        // _propertyValues acts like cache; missing properties are added with possible common keywords
        let keywords = propertyValues.get(propertyName) || propertyValues.get(unprefixedName);
        if (!keywords) {
            keywords = [];
            for (const commonKeyword of CommonKeywords) {
                if (CSS.supports(propertyName, commonKeyword)) {
                    keywords.push(commonKeyword);
                }
            }
            propertyValues.set(propertyName, keywords);
        }
        return keywords;
    }
    /**
     * @param {string} propertyName
     * @return {!Array<string>}
     */
    propertyValues(propertyName) {
        const acceptedKeywords = ['inherit', 'initial', 'revert', 'unset'];
        propertyName = propertyName.toLowerCase();
        acceptedKeywords.push(...this._specificPropertyValues(propertyName));
        if (this.isColorAwareProperty(propertyName)) {
            acceptedKeywords.push('currentColor');
            for (const color in Color.Nicknames) {
                acceptedKeywords.push(color);
            }
        }
        return acceptedKeywords.sort(CSSMetadata._sortPrefixesToEnd);
    }
    /**
     * @param {string} property
     * @return {number}
     */
    propertyUsageWeight(property) {
        return Weight.get(property) || Weight.get(this.canonicalPropertyName(property)) || 0;
    }
    /**
     * @param {string} key
     * @param {string} value
     * @return {?{text: string, startColumn: number, endColumn: number}}
     */
    getValuePreset(key, value) {
        const values = _valuePresets.get(key);
        let text = values ? values.get(value) : null;
        if (!text) {
            return null;
        }
        let startColumn = text.length;
        let endColumn = text.length;
        if (text) {
            startColumn = text.indexOf('|');
            endColumn = text.lastIndexOf('|');
            endColumn = startColumn === endColumn ? endColumn : endColumn - 1;
            text = text.replace(/\|/g, '');
        }
        return { text, startColumn, endColumn };
    }
}
const VariableRegex = /(var\(--.*?\))/g;
const URLRegex = /url\(\s*('.+?'|".+?"|[^)]+)\s*\)/g;
/**
 * Matches an instance of a grid area 'row' definition.
 * 'grid-template-areas', e.g.
 *    "a a ."
 *
 * 'grid', 'grid-template', e.g.
 *    [track-name] "a a ." minmax(50px, auto) [track-name]
 */
const GridAreaRowRegex = /((?:\[[\w\- ]+\]\s*)*(?:"[^"]+"|'[^']+'))[^'"\[]*\[?[^'"\[]*/;
/** @type {?CSSMetadata} */
let _instance = null;
/**
 * @return {!CSSMetadata}
 */
function cssMetadata() {
    if (!_instance) {
        const supportedProperties = 
        /** @type {!Array<!CSSPropertyDefinition>} */ (generatedProperties);
        _instance = new CSSMetadata(supportedProperties, generatedAliasesFor);
    }
    return _instance;
}
/**
 * The pipe character '|' indicates where text selection should be set.
 */
const _imageValuePresetMap = new Map([
    ['linear-gradient', 'linear-gradient(|45deg, black, transparent|)'],
    ['radial-gradient', 'radial-gradient(|black, transparent|)'],
    ['repeating-linear-gradient', 'repeating-linear-gradient(|45deg, black, transparent 100px|)'],
    ['repeating-radial-gradient', 'repeating-radial-gradient(|black, transparent 100px|)'],
    ['url', 'url(||)'],
]);
const _filterValuePresetMap = new Map([
    ['blur', 'blur(|1px|)'],
    ['brightness', 'brightness(|0.5|)'],
    ['contrast', 'contrast(|0.5|)'],
    ['drop-shadow', 'drop-shadow(|2px 4px 6px black|)'],
    ['grayscale', 'grayscale(|1|)'],
    ['hue-rotate', 'hue-rotate(|45deg|)'],
    ['invert', 'invert(|1|)'],
    ['opacity', 'opacity(|0.5|)'],
    ['saturate', 'saturate(|0.5|)'],
    ['sepia', 'sepia(|1|)'],
    ['url', 'url(||)'],
]);
const _valuePresets = new Map([
    ['filter', _filterValuePresetMap], ['backdrop-filter', _filterValuePresetMap], ['background', _imageValuePresetMap],
    ['background-image', _imageValuePresetMap], ['-webkit-mask-image', _imageValuePresetMap],
    [
        'transform', new Map([
            ['scale', 'scale(|1.5|)'],
            ['scaleX', 'scaleX(|1.5|)'],
            ['scaleY', 'scaleY(|1.5|)'],
            ['scale3d', 'scale3d(|1.5, 1.5, 1.5|)'],
            ['rotate', 'rotate(|45deg|)'],
            ['rotateX', 'rotateX(|45deg|)'],
            ['rotateY', 'rotateY(|45deg|)'],
            ['rotateZ', 'rotateZ(|45deg|)'],
            ['rotate3d', 'rotate3d(|1, 1, 1, 45deg|)'],
            ['skew', 'skew(|10deg, 10deg|)'],
            ['skewX', 'skewX(|10deg|)'],
            ['skewY', 'skewY(|10deg|)'],
            ['translate', 'translate(|10px, 10px|)'],
            ['translateX', 'translateX(|10px|)'],
            ['translateY', 'translateY(|10px|)'],
            ['translateZ', 'translateZ(|10px|)'],
            ['translate3d', 'translate3d(|10px, 10px, 10px|)'],
            ['matrix', 'matrix(|1, 0, 0, 1, 0, 0|)'],
            ['matrix3d', 'matrix3d(|1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1|)'],
            ['perspective', 'perspective(|10px|)']
        ])
    ]
]);
const _distanceProperties = new Set([
    'background-position', 'border-spacing', 'bottom', 'font-size', 'height', 'left', 'letter-spacing', 'max-height',
    'max-width', 'min-height', 'min-width', 'right', 'text-indent', 'top', 'width', 'word-spacing', 'grid-row-gap',
    'grid-column-gap', 'row-gap'
]);
const _bezierAwareProperties = new Set([
    'animation', 'animation-timing-function', 'transition', 'transition-timing-function', '-webkit-animation',
    '-webkit-animation-timing-function', '-webkit-transition', '-webkit-transition-timing-function'
]);
const _colorAwareProperties = new Set([
    'background',
    'background-color',
    'background-image',
    'border',
    'border-color',
    'border-image',
    'border-image-source',
    'border-bottom',
    'border-bottom-color',
    'border-left',
    'border-left-color',
    'border-right',
    'border-right-color',
    'border-top',
    'border-top-color',
    'box-shadow',
    'caret-color',
    'color',
    'column-rule',
    'column-rule-color',
    'fill',
    'list-style-image',
    'outline',
    'outline-color',
    'stroke',
    'text-decoration-color',
    'text-shadow',
    '-webkit-border-after',
    '-webkit-border-after-color',
    '-webkit-border-before',
    '-webkit-border-before-color',
    '-webkit-border-end',
    '-webkit-border-end-color',
    '-webkit-border-start',
    '-webkit-border-start-color',
    '-webkit-box-reflect',
    '-webkit-box-shadow',
    '-webkit-column-rule-color',
    '-webkit-mask',
    '-webkit-mask-box-image',
    '-webkit-mask-box-image-source',
    '-webkit-mask-image',
    '-webkit-tap-highlight-color',
    '-webkit-text-decoration-color',
    '-webkit-text-emphasis',
    '-webkit-text-emphasis-color',
    '-webkit-text-fill-color',
    '-webkit-text-stroke',
    '-webkit-text-stroke-color'
]);
// manually maintained list of property values to add into autocomplete list
const _extraPropertyValues = {
    'background-repeat': { values: ['repeat', 'repeat-x', 'repeat-y', 'no-repeat', 'space', 'round'] },
    'content': { values: ['normal', 'close-quote', 'no-close-quote', 'no-open-quote', 'open-quote'] },
    'baseline-shift': { values: ['baseline'] },
    'max-height': { values: ['min-content', 'max-content', '-webkit-fill-available', 'fit-content'] },
    'color': { values: ['black'] },
    'background-color': { values: ['white'] },
    'box-shadow': { values: ['inset'] },
    'text-shadow': { values: ['0 0 black'] },
    '-webkit-writing-mode': { values: ['horizontal-tb', 'vertical-rl', 'vertical-lr'] },
    'writing-mode': { values: ['lr', 'rl', 'tb', 'lr-tb', 'rl-tb', 'tb-rl'] },
    'page-break-inside': { values: ['avoid'] },
    'cursor': { values: ['-webkit-zoom-in', '-webkit-zoom-out', '-webkit-grab', '-webkit-grabbing'] },
    'border-width': { values: ['medium', 'thick', 'thin'] },
    'border-style': { values: ['hidden', 'inset', 'groove', 'ridge', 'outset', 'dotted', 'dashed', 'solid', 'double'] },
    'size': { values: ['a3', 'a4', 'a5', 'b4', 'b5', 'landscape', 'ledger', 'legal', 'letter', 'portrait'] },
    'overflow': { values: ['hidden', 'visible', 'overlay', 'scroll'] },
    'overscroll-behavior': { values: ['contain'] },
    'text-rendering': { values: ['optimizeSpeed', 'optimizeLegibility', 'geometricPrecision'] },
    'text-align': { values: ['-webkit-auto', '-webkit-match-parent'] },
    'color-interpolation': { values: ['sRGB', 'linearRGB'] },
    'word-wrap': { values: ['normal', 'break-word'] },
    'font-weight': { values: ['100', '200', '300', '400', '500', '600', '700', '800', '900'] },
    '-webkit-text-emphasis': { values: ['circle', 'filled', 'open', 'dot', 'double-circle', 'triangle', 'sesame'] },
    'color-rendering': { values: ['optimizeSpeed', 'optimizeQuality'] },
    '-webkit-text-combine': { values: ['horizontal'] },
    'text-orientation': { values: ['sideways-right'] },
    'outline': {
        values: ['inset', 'groove', 'ridge', 'outset', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin']
    },
    'font': {
        values: [
            'caption', 'icon', 'menu', 'message-box', 'small-caption', '-webkit-mini-control', '-webkit-small-control',
            '-webkit-control', 'status-bar'
        ]
    },
    'dominant-baseline': { values: ['text-before-edge', 'text-after-edge', 'use-script', 'no-change', 'reset-size'] },
    '-webkit-text-emphasis-position': { values: ['over', 'under'] },
    'alignment-baseline': { values: ['before-edge', 'after-edge', 'text-before-edge', 'text-after-edge', 'hanging'] },
    'page-break-before': { values: ['left', 'right', 'always', 'avoid'] },
    'border-image': { values: ['repeat', 'stretch', 'space', 'round'] },
    'text-decoration': { values: ['blink', 'line-through', 'overline', 'underline', 'wavy', 'double', 'solid', 'dashed', 'dotted'] },
    // List taken from https://drafts.csswg.org/css-fonts-4/#generic-font-families
    'font-family': {
        values: [
            'serif', 'sans-serif', 'cursive', 'fantasy', 'monospace', 'system-ui', 'emoji', 'math', 'fangsong', 'ui-serif',
            'ui-sans-serif', 'ui-monospace', 'ui-rounded', '-webkit-body', '-webkit-pictograph'
        ]
    },
    'zoom': { values: ['normal'] },
    'max-width': { values: ['min-content', 'max-content', '-webkit-fill-available', 'fit-content'] },
    '-webkit-font-smoothing': { values: ['antialiased', 'subpixel-antialiased'] },
    'border': {
        values: [
            'hidden', 'inset', 'groove', 'ridge', 'outset', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'font-variant': {
        values: [
            'small-caps',
            'normal',
            'common-ligatures',
            'no-common-ligatures',
            'discretionary-ligatures',
            'no-discretionary-ligatures',
            'historical-ligatures',
            'no-historical-ligatures',
            'contextual',
            'no-contextual',
            'all-small-caps',
            'petite-caps',
            'all-petite-caps',
            'unicase',
            'titling-caps',
            'lining-nums',
            'oldstyle-nums',
            'proportional-nums',
            'tabular-nums',
            'diagonal-fractions',
            'stacked-fractions',
            'ordinal',
            'slashed-zero',
            'jis78',
            'jis83',
            'jis90',
            'jis04',
            'simplified',
            'traditional',
            'full-width',
            'proportional-width',
            'ruby'
        ]
    },
    'vertical-align': { values: ['top', 'bottom', '-webkit-baseline-middle'] },
    'page-break-after': { values: ['left', 'right', 'always', 'avoid'] },
    '-webkit-text-emphasis-style': { values: ['circle', 'filled', 'open', 'dot', 'double-circle', 'triangle', 'sesame'] },
    'transform': {
        values: [
            'scale', 'scaleX', 'scaleY', 'scale3d', 'rotate', 'rotateX', 'rotateY',
            'rotateZ', 'rotate3d', 'skew', 'skewX', 'skewY', 'translate', 'translateX',
            'translateY', 'translateZ', 'translate3d', 'matrix', 'matrix3d', 'perspective'
        ]
    },
    'align-content': {
        values: [
            'normal', 'baseline', 'space-between', 'space-around', 'space-evenly', 'stretch', 'center', 'start', 'end',
            'flex-start', 'flex-end'
        ]
    },
    'justify-content': {
        values: [
            'normal', 'space-between', 'space-around', 'space-evenly', 'stretch', 'center', 'start', 'end', 'flex-start',
            'flex-end', 'left', 'right'
        ]
    },
    'place-content': {
        values: [
            'normal', 'space-between', 'space-around', 'space-evenly', 'stretch', 'center', 'start', 'end', 'flex-start',
            'flex-end', 'baseline'
        ]
    },
    'align-items': {
        values: ['normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end']
    },
    'justify-items': {
        values: [
            'normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end',
            'left', 'right', 'legacy'
        ]
    },
    'place-items': {
        values: ['normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end']
    },
    'align-self': {
        values: ['normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end']
    },
    'justify-self': {
        values: [
            'normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end',
            'left', 'right'
        ]
    },
    'place-self': {
        values: ['normal', 'stretch', 'baseline', 'center', 'start', 'end', 'self-start', 'self-end', 'flex-start', 'flex-end']
    },
    'perspective-origin': { values: ['left', 'center', 'right', 'top', 'bottom'] },
    'transform-origin': { values: ['left', 'center', 'right', 'top', 'bottom'] },
    'transition-timing-function': { values: ['cubic-bezier', 'steps'] },
    'animation-timing-function': { values: ['cubic-bezier', 'steps'] },
    '-webkit-backface-visibility': { values: ['visible', 'hidden'] },
    '-webkit-column-break-after': { values: ['always', 'avoid'] },
    '-webkit-column-break-before': { values: ['always', 'avoid'] },
    '-webkit-column-break-inside': { values: ['avoid'] },
    '-webkit-column-span': { values: ['all'] },
    '-webkit-column-gap': { values: ['normal'] },
    'filter': {
        values: [
            'url', 'blur', 'brightness', 'contrast', 'drop-shadow', 'grayscale', 'hue-rotate', 'invert', 'opacity',
            'saturate', 'sepia'
        ]
    },
    'backdrop-filter': {
        values: [
            'url', 'blur', 'brightness', 'contrast', 'drop-shadow', 'grayscale', 'hue-rotate', 'invert', 'opacity',
            'saturate', 'sepia'
        ]
    },
    'mix-blend-mode': { values: ['unset'] },
    'background-blend-mode': { values: ['unset'] },
    'grid-template-columns': { values: ['min-content', 'max-content'] },
    'grid-template-rows': { values: ['min-content', 'max-content'] },
    'grid-auto-flow': { values: ['dense'] },
    'background': {
        values: [
            'repeat',
            'repeat-x',
            'repeat-y',
            'no-repeat',
            'top',
            'bottom',
            'left',
            'right',
            'center',
            'fixed',
            'local',
            'scroll',
            'space',
            'round',
            'border-box',
            'content-box',
            'padding-box',
            'linear-gradient',
            'radial-gradient',
            'repeating-linear-gradient',
            'repeating-radial-gradient',
            'url'
        ]
    },
    'background-image': { values: ['linear-gradient', 'radial-gradient', 'repeating-linear-gradient', 'repeating-radial-gradient', 'url'] },
    'background-position': { values: ['top', 'bottom', 'left', 'right', 'center'] },
    'background-position-x': { values: ['left', 'right', 'center'] },
    'background-position-y': { values: ['top', 'bottom', 'center'] },
    'background-repeat-x': { values: ['repeat', 'no-repeat'] },
    'background-repeat-y': { values: ['repeat', 'no-repeat'] },
    'border-bottom': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'border-left': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'border-right': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'border-top': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'buffered-rendering': { values: ['static', 'dynamic'] },
    'color-interpolation-filters': { values: ['srgb', 'linearrgb'] },
    'column-rule': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    'flex-flow': { values: ['nowrap', 'row', 'row-reverse', 'column', 'column-reverse', 'wrap', 'wrap-reverse'] },
    'height': { values: ['-webkit-fill-available'] },
    'inline-size': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'list-style': {
        values: [
            'outside',
            'inside',
            'disc',
            'circle',
            'square',
            'decimal',
            'decimal-leading-zero',
            'arabic-indic',
            'bengali',
            'cambodian',
            'khmer',
            'devanagari',
            'gujarati',
            'gurmukhi',
            'kannada',
            'lao',
            'malayalam',
            'mongolian',
            'myanmar',
            'oriya',
            'persian',
            'urdu',
            'telugu',
            'tibetan',
            'thai',
            'lower-roman',
            'upper-roman',
            'lower-greek',
            'lower-alpha',
            'lower-latin',
            'upper-alpha',
            'upper-latin',
            'cjk-earthly-branch',
            'cjk-heavenly-stem',
            'ethiopic-halehame',
            'ethiopic-halehame-am',
            'ethiopic-halehame-ti-er',
            'ethiopic-halehame-ti-et',
            'hangul',
            'hangul-consonant',
            'korean-hangul-formal',
            'korean-hanja-formal',
            'korean-hanja-informal',
            'hebrew',
            'armenian',
            'lower-armenian',
            'upper-armenian',
            'georgian',
            'cjk-ideographic',
            'simp-chinese-formal',
            'simp-chinese-informal',
            'trad-chinese-formal',
            'trad-chinese-informal',
            'hiragana',
            'katakana',
            'hiragana-iroha',
            'katakana-iroha'
        ]
    },
    'max-block-size': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'max-inline-size': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'min-block-size': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'min-height': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'min-inline-size': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'min-width': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    'object-position': { values: ['top', 'bottom', 'left', 'right', 'center'] },
    'shape-outside': { values: ['border-box', 'content-box', 'padding-box', 'margin-box'] },
    '-webkit-appearance': {
        values: [
            'checkbox',
            'radio',
            'push-button',
            'square-button',
            'button',
            'inner-spin-button',
            'listbox',
            'media-slider',
            'media-sliderthumb',
            'media-volume-slider',
            'media-volume-sliderthumb',
            'menulist',
            'menulist-button',
            'meter',
            'progress-bar',
            'slider-horizontal',
            'slider-vertical',
            'sliderthumb-horizontal',
            'sliderthumb-vertical',
            'searchfield',
            'searchfield-cancel-button',
            'textfield',
            'textarea'
        ]
    },
    '-webkit-border-after': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    '-webkit-border-after-style': { values: ['hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    '-webkit-border-after-width': { values: ['medium', 'thick', 'thin'] },
    '-webkit-border-before': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    '-webkit-border-before-style': { values: ['hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    '-webkit-border-before-width': { values: ['medium', 'thick', 'thin'] },
    '-webkit-border-end': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    '-webkit-border-end-style': { values: ['hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    '-webkit-border-end-width': { values: ['medium', 'thick', 'thin'] },
    '-webkit-border-start': {
        values: [
            'hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double', 'medium', 'thick', 'thin'
        ]
    },
    '-webkit-border-start-style': { values: ['hidden', 'inset', 'groove', 'outset', 'ridge', 'dotted', 'dashed', 'solid', 'double'] },
    '-webkit-border-start-width': { values: ['medium', 'thick', 'thin'] },
    '-webkit-logical-height': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-logical-width': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-margin-collapse': { values: ['collapse', 'separate', 'discard'] },
    '-webkit-mask-box-image': { values: ['repeat', 'stretch', 'space', 'round'] },
    '-webkit-mask-box-image-repeat': { values: ['repeat', 'stretch', 'space', 'round'] },
    '-webkit-mask-clip': { values: ['text', 'border', 'border-box', 'content', 'content-box', 'padding', 'padding-box'] },
    '-webkit-mask-composite': {
        values: [
            'clear', 'copy', 'source-over', 'source-in', 'source-out', 'source-atop', 'destination-over', 'destination-in',
            'destination-out', 'destination-atop', 'xor', 'plus-lighter'
        ]
    },
    '-webkit-mask-image': { values: ['linear-gradient', 'radial-gradient', 'repeating-linear-gradient', 'repeating-radial-gradient', 'url'] },
    '-webkit-mask-origin': { values: ['border', 'border-box', 'content', 'content-box', 'padding', 'padding-box'] },
    '-webkit-mask-position': { values: ['top', 'bottom', 'left', 'right', 'center'] },
    '-webkit-mask-position-x': { values: ['left', 'right', 'center'] },
    '-webkit-mask-position-y': { values: ['top', 'bottom', 'center'] },
    '-webkit-mask-repeat': { values: ['repeat', 'repeat-x', 'repeat-y', 'no-repeat', 'space', 'round'] },
    '-webkit-mask-size': { values: ['contain', 'cover'] },
    '-webkit-max-logical-height': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-max-logical-width': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-min-logical-height': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-min-logical-width': { values: ['-webkit-fill-available', 'min-content', 'max-content', 'fit-content'] },
    '-webkit-perspective-origin-x': { values: ['left', 'right', 'center'] },
    '-webkit-perspective-origin-y': { values: ['top', 'bottom', 'center'] },
    '-webkit-text-decorations-in-effect': { values: ['blink', 'line-through', 'overline', 'underline'] },
    '-webkit-text-stroke': { values: ['medium', 'thick', 'thin'] },
    '-webkit-text-stroke-width': { values: ['medium', 'thick', 'thin'] },
    '-webkit-transform-origin-x': { values: ['left', 'right', 'center'] },
    '-webkit-transform-origin-y': { values: ['top', 'bottom', 'center'] },
    'width': { values: ['-webkit-fill-available'] }
};
// Weight of CSS properties based on their usage from https://www.chromestatus.com/metrics/css/popularity
const Weight = new Map([
    ['align-content', 57],
    ['align-items', 129],
    ['align-self', 55],
    ['animation', 175],
    ['animation-delay', 114],
    ['animation-direction', 113],
    ['animation-duration', 137],
    ['animation-fill-mode', 132],
    ['animation-iteration-count', 124],
    ['animation-name', 139],
    ['animation-play-state', 104],
    ['animation-timing-function', 141],
    ['backface-visibility', 123],
    ['background', 260],
    ['background-attachment', 119],
    ['background-clip', 165],
    ['background-color', 259],
    ['background-image', 246],
    ['background-origin', 107],
    ['background-position', 237],
    ['background-position-x', 108],
    ['background-position-y', 93],
    ['background-repeat', 234],
    ['background-size', 203],
    ['border', 263],
    ['border-bottom', 233],
    ['border-bottom-color', 190],
    ['border-bottom-left-radius', 186],
    ['border-bottom-right-radius', 185],
    ['border-bottom-style', 150],
    ['border-bottom-width', 179],
    ['border-collapse', 209],
    ['border-color', 226],
    ['border-image', 89],
    ['border-image-outset', 50],
    ['border-image-repeat', 49],
    ['border-image-slice', 58],
    ['border-image-source', 32],
    ['border-image-width', 52],
    ['border-left', 221],
    ['border-left-color', 174],
    ['border-left-style', 142],
    ['border-left-width', 172],
    ['border-radius', 224],
    ['border-right', 223],
    ['border-right-color', 182],
    ['border-right-style', 130],
    ['border-right-width', 178],
    ['border-spacing', 198],
    ['border-style', 206],
    ['border-top', 231],
    ['border-top-color', 192],
    ['border-top-left-radius', 187],
    ['border-top-right-radius', 189],
    ['border-top-style', 152],
    ['border-top-width', 180],
    ['border-width', 214],
    ['bottom', 227],
    ['box-shadow', 213],
    ['box-sizing', 216],
    ['caption-side', 96],
    ['clear', 229],
    ['clip', 173],
    ['clip-rule', 5],
    ['color', 256],
    ['content', 219],
    ['counter-increment', 111],
    ['counter-reset', 110],
    ['cursor', 250],
    ['direction', 176],
    ['display', 262],
    ['empty-cells', 99],
    ['fill', 140],
    ['fill-opacity', 82],
    ['fill-rule', 22],
    ['filter', 160],
    ['flex', 133],
    ['flex-basis', 66],
    ['flex-direction', 85],
    ['flex-flow', 94],
    ['flex-grow', 112],
    ['flex-shrink', 61],
    ['flex-wrap', 68],
    ['float', 252],
    ['font', 211],
    ['font-family', 254],
    ['font-kerning', 18],
    ['font-size', 264],
    ['font-stretch', 77],
    ['font-style', 220],
    ['font-variant', 161],
    ['font-weight', 257],
    ['height', 266],
    ['image-rendering', 90],
    ['justify-content', 127],
    ['left', 248],
    ['letter-spacing', 188],
    ['line-height', 244],
    ['list-style', 215],
    ['list-style-image', 145],
    ['list-style-position', 149],
    ['list-style-type', 199],
    ['margin', 267],
    ['margin-bottom', 241],
    ['margin-left', 243],
    ['margin-right', 238],
    ['margin-top', 253],
    ['mask', 20],
    ['max-height', 205],
    ['max-width', 225],
    ['min-height', 217],
    ['min-width', 218],
    ['object-fit', 33],
    ['opacity', 251],
    ['order', 117],
    ['orphans', 146],
    ['outline', 222],
    ['outline-color', 153],
    ['outline-offset', 147],
    ['outline-style', 151],
    ['outline-width', 148],
    ['overflow', 255],
    ['overflow-wrap', 105],
    ['overflow-x', 184],
    ['overflow-y', 196],
    ['padding', 265],
    ['padding-bottom', 230],
    ['padding-left', 235],
    ['padding-right', 232],
    ['padding-top', 240],
    ['page', 8],
    ['page-break-after', 120],
    ['page-break-before', 69],
    ['page-break-inside', 121],
    ['perspective', 92],
    ['perspective-origin', 103],
    ['pointer-events', 183],
    ['position', 261],
    ['quotes', 158],
    ['resize', 168],
    ['right', 245],
    ['shape-rendering', 38],
    ['size', 64],
    ['speak', 118],
    ['src', 170],
    ['stop-color', 42],
    ['stop-opacity', 31],
    ['stroke', 98],
    ['stroke-dasharray', 36],
    ['stroke-dashoffset', 3],
    ['stroke-linecap', 30],
    ['stroke-linejoin', 21],
    ['stroke-miterlimit', 12],
    ['stroke-opacity', 34],
    ['stroke-width', 87],
    ['table-layout', 171],
    ['tab-size', 46],
    ['text-align', 260],
    ['text-anchor', 35],
    ['text-decoration', 247],
    ['text-indent', 207],
    ['text-overflow', 204],
    ['text-rendering', 155],
    ['text-shadow', 208],
    ['text-transform', 202],
    ['top', 258],
    ['touch-action', 80],
    ['transform', 181],
    ['transform-origin', 162],
    ['transform-style', 86],
    ['transition', 193],
    ['transition-delay', 134],
    ['transition-duration', 135],
    ['transition-property', 131],
    ['transition-timing-function', 122],
    ['unicode-bidi', 156],
    ['unicode-range', 136],
    ['vertical-align', 236],
    ['visibility', 242],
    ['-webkit-appearance', 191],
    ['-webkit-backface-visibility', 154],
    ['-webkit-background-clip', 164],
    ['-webkit-background-origin', 40],
    ['-webkit-background-size', 163],
    ['-webkit-border-end', 9],
    ['-webkit-border-horizontal-spacing', 81],
    ['-webkit-border-image', 75],
    ['-webkit-border-radius', 212],
    ['-webkit-border-start', 10],
    ['-webkit-border-start-color', 16],
    ['-webkit-border-start-width', 13],
    ['-webkit-border-vertical-spacing', 43],
    ['-webkit-box-align', 101],
    ['-webkit-box-direction', 51],
    ['-webkit-box-flex', 128],
    ['-webkit-box-ordinal-group', 91],
    ['-webkit-box-orient', 144],
    ['-webkit-box-pack', 106],
    ['-webkit-box-reflect', 39],
    ['-webkit-box-shadow', 210],
    ['-webkit-column-break-inside', 60],
    ['-webkit-column-count', 84],
    ['-webkit-column-gap', 76],
    ['-webkit-column-rule', 25],
    ['-webkit-column-rule-color', 23],
    ['-webkit-columns', 44],
    ['-webkit-column-span', 29],
    ['-webkit-column-width', 47],
    ['-webkit-filter', 159],
    ['-webkit-font-feature-settings', 59],
    ['-webkit-font-smoothing', 177],
    ['-webkit-highlight', 1],
    ['-webkit-line-break', 45],
    ['-webkit-line-clamp', 126],
    ['-webkit-margin-after', 67],
    ['-webkit-margin-before', 70],
    ['-webkit-margin-collapse', 14],
    ['-webkit-margin-end', 65],
    ['-webkit-margin-start', 100],
    ['-webkit-margin-top-collapse', 78],
    ['-webkit-mask', 19],
    ['-webkit-mask-box-image', 72],
    ['-webkit-mask-image', 88],
    ['-webkit-mask-position', 54],
    ['-webkit-mask-repeat', 63],
    ['-webkit-mask-size', 79],
    ['-webkit-padding-after', 15],
    ['-webkit-padding-before', 28],
    ['-webkit-padding-end', 48],
    ['-webkit-padding-start', 73],
    ['-webkit-print-color-adjust', 83],
    ['-webkit-rtl-ordering', 7],
    ['-webkit-tap-highlight-color', 169],
    ['-webkit-text-emphasis-color', 11],
    ['-webkit-text-fill-color', 71],
    ['-webkit-text-security', 17],
    ['-webkit-text-stroke', 56],
    ['-webkit-text-stroke-color', 37],
    ['-webkit-text-stroke-width', 53],
    ['-webkit-user-drag', 95],
    ['-webkit-user-modify', 62],
    ['-webkit-user-select', 194],
    ['-webkit-writing-mode', 4],
    ['white-space', 228],
    ['widows', 115],
    ['width', 268],
    ['will-change', 74],
    ['word-break', 166],
    ['word-spacing', 157],
    ['word-wrap', 197],
    ['writing-mode', 41],
    ['z-index', 239],
    ['zoom', 200]
]);
// Common keywords to CSS properties
const CommonKeywords = ['auto', 'none'];
/**
 * @typedef {{name: string, longhands: ?Array<string>, inherited: ?boolean, svg: ?boolean}}
 */
// @ts-ignore typedef
let CSSPropertyDefinition;

var CSSMetadata$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSMetadata: CSSMetadata,
    VariableRegex: VariableRegex,
    URLRegex: URLRegex,
    GridAreaRowRegex: GridAreaRowRegex,
    cssMetadata: cssMetadata,
    CSSPropertyDefinition: CSSPropertyDefinition
});

// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 * @unrestricted
 */
class ProfileNode {
    /**
     * @param {!Protocol.Runtime.CallFrame} callFrame
     */
    constructor(callFrame) {
        /** @type {!Protocol.Runtime.CallFrame} */
        this.callFrame = callFrame;
        /** @type {string} */
        this.callUID = `${callFrame.functionName}@${callFrame.scriptId}:${callFrame.lineNumber}:${callFrame.columnNumber}`;
        /** @type {number} */
        this.self = 0;
        /** @type {number} */
        this.total = 0;
        /** @type {number} */
        this.id = 0;
        /** @type {?ProfileNode} */
        this.parent = null;
        /** @type {!Array<!ProfileNode>} */
        this.children = [];
        /** @type {number} */
        this.depth;
    }
    /**
     * @return {string}
     */
    get functionName() {
        return this.callFrame.functionName;
    }
    /**
     * @return {string}
     */
    get scriptId() {
        return this.callFrame.scriptId;
    }
    /**
     * @return {string}
     */
    get url() {
        return this.callFrame.url;
    }
    /**
     * @return {number}
     */
    get lineNumber() {
        return this.callFrame.lineNumber;
    }
    /**
     * @return {number}
     */
    get columnNumber() {
        return this.callFrame.columnNumber;
    }
}
/**
 * @unrestricted
 */
class ProfileTreeModel {
    /**
     * @param {?Target=} target
     */
    constructor(target) {
        this._target = target || null;
        /** @type {!ProfileNode} */
        this.root;
        /** @type {number} */
        this.total;
    }
    /**
     * @param {!ProfileNode} root
     * @protected
     */
    initialize(root) {
        this.root = root;
        this._assignDepthsAndParents();
        this.total = this._calculateTotals(this.root);
    }
    _assignDepthsAndParents() {
        const root = this.root;
        root.depth = -1;
        root.parent = null;
        this.maxDepth = 0;
        const nodesToTraverse = [root];
        while (nodesToTraverse.length) {
            const parent = /** @type {!ProfileNode} */ (nodesToTraverse.pop());
            const depth = parent.depth + 1;
            if (depth > this.maxDepth) {
                this.maxDepth = depth;
            }
            const children = parent.children;
            for (const child of children) {
                child.depth = depth;
                child.parent = parent;
                if (child.children.length) {
                    nodesToTraverse.push(child);
                }
            }
        }
    }
    /**
     * @param {!ProfileNode} root
     * @return {number}
     */
    _calculateTotals(root) {
        const nodesToTraverse = [root];
        const dfsList = [];
        while (nodesToTraverse.length) {
            const node = /** @type {!ProfileNode} */ (nodesToTraverse.pop());
            node.total = node.self;
            dfsList.push(node);
            nodesToTraverse.push(...node.children);
        }
        while (dfsList.length > 1) {
            const node = /** @type {!ProfileNode} */ (dfsList.pop());
            if (node.parent) {
                node.parent.total += node.total;
            }
        }
        return root.total;
    }
    /**
     * @return {?Target}
     */
    target() {
        return this._target;
    }
}

var ProfileTreeModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ProfileNode: ProfileNode,
    ProfileTreeModel: ProfileTreeModel
});

// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/**
 *  @unrestricted
 */
class Cookie {
    /**
       * @param {string} name
       * @param {string} value
       * @param {?Type=} type
       * @param {!Protocol.Network.CookiePriority=} priority
       */
    constructor(name, value, type, priority) {
        this._name = name;
        this._value = value;
        this._type = type;
        /** @type {!Object<string, *>} */
        this._attributes = {};
        this._size = 0;
        this._priority = /** @type {!Protocol.Network.CookiePriority} */ (priority || 'Medium');
        /** @type {string|null} */
        this._cookieLine = null;
    }
    /**
       * @param {!Protocol.Network.Cookie} protocolCookie
       * @return {!Cookie}
       */
    static fromProtocolCookie(protocolCookie) {
        const cookie = new Cookie(protocolCookie.name, protocolCookie.value, null, protocolCookie.priority);
        cookie.addAttribute('domain', protocolCookie['domain']);
        cookie.addAttribute('path', protocolCookie['path']);
        if (protocolCookie['expires']) {
            cookie.addAttribute('expires', protocolCookie['expires'] * 1000);
        }
        if (protocolCookie['httpOnly']) {
            cookie.addAttribute('httpOnly');
        }
        if (protocolCookie['secure']) {
            cookie.addAttribute('secure');
        }
        if (protocolCookie['sameSite']) {
            cookie.addAttribute('sameSite', protocolCookie['sameSite']);
        }
        cookie.setSize(protocolCookie['size']);
        return cookie;
    }
    /**
     * @returns {string}
     */
    key() {
        return (this.domain() || '-') + ' ' + this.name() + ' ' + (this.path() || '-');
    }
    /**
       * @return {string}
       */
    name() {
        return this._name;
    }
    /**
       * @return {string}
       */
    value() {
        return this._value;
    }
    /**
       * @return {?Type|undefined}
       */
    type() {
        return this._type;
    }
    /**
       * @return {boolean}
       */
    httpOnly() {
        return 'httponly' in this._attributes;
    }
    /**
       * @return {boolean}
       */
    secure() {
        return 'secure' in this._attributes;
    }
    /**
       * @return {!Protocol.Network.CookieSameSite}
       */
    sameSite() {
        // TODO(allada) This should not rely on _attributes and instead store them individually.
        // when attributes get added via addAttribute() they are lowercased, hence the lowercasing of samesite here
        return /** @type {!Protocol.Network.CookieSameSite} */ (this._attributes['samesite']);
    }
    /**
     * @return {!Protocol.Network.CookiePriority}
     */
    priority() {
        return this._priority;
    }
    /**
       * @return {boolean}
       */
    session() {
        // RFC 2965 suggests using Discard attribute to mark session cookies, but this does not seem to be widely used.
        // Check for absence of explicitly max-age or expiry date instead.
        return !('expires' in this._attributes || 'max-age' in this._attributes);
    }
    /**
       * @return {string}
       */
    path() {
        return /** @type {string} */ (this._attributes['path']);
    }
    /**
       * @return {string}
       */
    domain() {
        return /** @type {string} */ (this._attributes['domain']);
    }
    /**
       * @return {number}
       */
    expires() {
        return /** @type {number} */ (this._attributes['expires']);
    }
    /**
       * @return {number}
       */
    maxAge() {
        return /** @type {number} */ (this._attributes['max-age']);
    }
    /**
       * @return {number}
       */
    size() {
        return this._size;
    }
    /**
       * @return {string|null}
       */
    url() {
        if (!this.domain() || !this.path()) {
            return null;
        }
        return (this.secure() ? 'https://' : 'http://') + this.domain() + this.path();
    }
    /**
       * @param {number} size
       */
    setSize(size) {
        this._size = size;
    }
    /**
     * @param {!Date} requestDate
     * @return {!Date|null}
     */
    expiresDate(requestDate) {
        // RFC 6265 indicates that the max-age attribute takes precedence over the expires attribute
        if (this.maxAge()) {
            return new Date(requestDate.getTime() + 1000 * this.maxAge());
        }
        if (this.expires()) {
            return new Date(this.expires());
        }
        return null;
    }
    /**
     * @param {string} key
     * @param {string|number=} value
     */
    addAttribute(key, value) {
        const normalizedKey = key.toLowerCase();
        switch (normalizedKey) {
            case 'priority':
                this._priority = /** @type {!Protocol.Network.CookiePriority} */ (value);
                break;
            default:
                this._attributes[normalizedKey] = value;
        }
    }
    /**
       * @param {string} cookieLine
       */
    setCookieLine(cookieLine) {
        this._cookieLine = cookieLine;
    }
    /**
       * @return {string|null}
       */
    getCookieLine() {
        return this._cookieLine;
    }
}
/**
 * @enum {number}
 */
const Type$1 = {
    Request: 0,
    Response: 1
};
/**
 * @enum {string}
 */
const Attributes = {
    Name: 'name',
    Value: 'value',
    Size: 'size',
    Domain: 'domain',
    Path: 'path',
    Expires: 'expires',
    HttpOnly: 'httpOnly',
    Secure: 'secure',
    SameSite: 'sameSite',
    Priority: 'priority',
};
/**
 * A `CookieReference` uniquely identifies a cookie by the triple (name,domain,path). Additionally, a context may be
 * included to make it clear which site under Application>Cookies should be opened when revealing a `CookieReference`.
 */
class CookieReference {
    /**
     * @param {string} name
     * @param {string} domain
     * @param {string} path
     * @param {string|undefined} contextUrl - Context in which to reveal the cookie.
     */
    constructor(name, domain, path, contextUrl) {
        this._name = name;
        this._domain = domain;
        this._path = path;
        this._contextUrl = contextUrl;
    }
    /**
     * @returns {string}
     */
    domain() {
        return this._domain;
    }
    /**
     * @returns {string|undefined}
     */
    contextUrl() {
        return this._contextUrl;
    }
}

var Cookie$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Cookie: Cookie,
    Type: Type$1,
    Attributes: Attributes,
    CookieReference: CookieReference
});

/*
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @unrestricted
 */
class CookieParser {
    /**
     * @param {string=} domain
     */
    constructor(domain) {
        if (domain) {
            // Handle domain according to
            // https://tools.ietf.org/html/draft-ietf-httpbis-rfc6265bis-03#section-5.3.3
            this._domain = domain.toLowerCase().replace(/^\./, '');
        }
        /** @type {!Array<!Cookie>} */
        this._cookies = [];
        /** @type {string|undefined} */
        this._input;
        this._originalInputLength = 0;
    }
    /**
     * @param {string|undefined} header
     * @return {?Array<!Cookie>}
     */
    static parseCookie(header) {
        return (new CookieParser()).parseCookie(header);
    }
    /**
     * @param {string|undefined} header
     * @param {string=} domain
     * @return {?Array<!Cookie>}
     */
    static parseSetCookie(header, domain) {
        return (new CookieParser(domain)).parseSetCookie(header);
    }
    /**
     * @return {!Array<!Cookie>}
     */
    cookies() {
        return this._cookies;
    }
    /**
     * @param {string|undefined} cookieHeader
     * @return {?Array<!Cookie>}
     */
    parseCookie(cookieHeader) {
        if (!this._initialize(cookieHeader)) {
            return null;
        }
        for (let kv = this._extractKeyValue(); kv; kv = this._extractKeyValue()) {
            if (kv.key.charAt(0) === '$' && this._lastCookie) {
                this._lastCookie.addAttribute(kv.key.slice(1), kv.value);
            }
            else if (kv.key.toLowerCase() !== '$version' && typeof kv.value === 'string') {
                this._addCookie(kv, Type$1.Request);
            }
            this._advanceAndCheckCookieDelimiter();
        }
        this._flushCookie();
        return this._cookies;
    }
    /**
     * @param {string|undefined} setCookieHeader
     * @return {?Array<!Cookie>}
     */
    parseSetCookie(setCookieHeader) {
        if (!this._initialize(setCookieHeader)) {
            return null;
        }
        for (let kv = this._extractKeyValue(); kv; kv = this._extractKeyValue()) {
            if (this._lastCookie) {
                this._lastCookie.addAttribute(kv.key, kv.value);
            }
            else {
                this._addCookie(kv, Type$1.Response);
            }
            if (this._advanceAndCheckCookieDelimiter()) {
                this._flushCookie();
            }
        }
        this._flushCookie();
        return this._cookies;
    }
    /**
     * @param {string|undefined} headerValue
     * @return {boolean}
     */
    _initialize(headerValue) {
        this._input = headerValue;
        if (typeof headerValue !== 'string') {
            return false;
        }
        this._cookies = [];
        this._lastCookie = null;
        this._lastCookieLine = '';
        this._originalInputLength = /** @type {string} */ (this._input).length;
        return true;
    }
    _flushCookie() {
        if (this._lastCookie) {
            // if we have a last cookie we know that these valeus all exist, hence the typecasts
            this._lastCookie.setSize(this._originalInputLength - /** @type {string} */ (this._input).length -
                /** @type {number} */ (this._lastCookiePosition));
            this._lastCookie.setCookieLine(/** @type {string} */ (this._lastCookieLine).replace('\n', ''));
        }
        this._lastCookie = null;
        this._lastCookieLine = '';
    }
    /**
     * @return {?KeyValue}
     */
    _extractKeyValue() {
        if (!this._input || !this._input.length) {
            return null;
        }
        // Note: RFCs offer an option for quoted values that may contain commas and semicolons.
        // Many browsers/platforms do not support this, however (see http://webkit.org/b/16699
        // and http://crbug.com/12361). The logic below matches latest versions of IE, Firefox,
        // Chrome and Safari on some old platforms. The latest version of Safari supports quoted
        // cookie values, though.
        const keyValueMatch = /^[ \t]*([^\s=;]+)[ \t]*(?:=[ \t]*([^;\n]*))?/.exec(this._input);
        if (!keyValueMatch) {
            console.error('Failed parsing cookie header before: ' + this._input);
            return null;
        }
        const result = new KeyValue(keyValueMatch[1], keyValueMatch[2] && keyValueMatch[2].trim(), 
        /** @type {number} */ (this._originalInputLength) - this._input.length);
        this._lastCookieLine += keyValueMatch[0];
        this._input = this._input.slice(keyValueMatch[0].length);
        return result;
    }
    /**
     * @return {boolean}
     */
    _advanceAndCheckCookieDelimiter() {
        if (!this._input) {
            return false;
        }
        const match = /^\s*[\n;]\s*/.exec(this._input);
        if (!match) {
            return false;
        }
        this._lastCookieLine += match[0];
        this._input = this._input.slice(match[0].length);
        return match[0].match('\n') !== null;
    }
    /**
     * @param {!KeyValue} keyValue
     * @param {!Type} type
     */
    _addCookie(keyValue, type) {
        if (this._lastCookie) {
            this._lastCookie.setSize(keyValue.position - /** @type {number} */ (this._lastCookiePosition));
        }
        // Mozilla bug 169091: Mozilla, IE and Chrome treat single token (w/o "=") as
        // specifying a value for a cookie with empty name.
        this._lastCookie = typeof keyValue.value === 'string' ? new Cookie(keyValue.key, keyValue.value, type) :
            new Cookie('', keyValue.key, type);
        if (this._domain) {
            this._lastCookie.addAttribute('domain', this._domain);
        }
        this._lastCookiePosition = keyValue.position;
        this._cookies.push(this._lastCookie);
    }
}
class KeyValue {
    /**
     * @param {string} key
     * @param {string|undefined} value
     * @param {number} position
     */
    constructor(key, value, position) {
        this.key = key;
        this.value = value;
        this.position = position;
    }
}

var CookieParser$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CookieParser: CookieParser
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/** @type {!WeakMap<!NetworkRequest, !NetworkManager>} */
const requestToManagerMap = new WeakMap();
const CONNECTION_TYPES = new Map([
    ['2g', Protocol.Network.ConnectionType.Cellular2g],
    ['3g', Protocol.Network.ConnectionType.Cellular3g],
    ['4g', Protocol.Network.ConnectionType.Cellular4g],
    ['bluetooth', Protocol.Network.ConnectionType.Bluetooth],
    ['wifi', Protocol.Network.ConnectionType.Wifi],
    ['wimax', Protocol.Network.ConnectionType.Wimax],
]);
class NetworkManager extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._dispatcher = new NetworkDispatcher(this);
        this._networkAgent = target.networkAgent();
        target.registerNetworkDispatcher(this._dispatcher);
        if (Settings.Settings.instance().moduleSetting('cacheDisabled').get()) {
            this._networkAgent.invoke_setCacheDisabled({ cacheDisabled: true });
        }
        this._networkAgent.invoke_enable({ maxPostDataSize: MAX_EAGER_POST_REQUEST_BODY_LENGTH });
        this._bypassServiceWorkerSetting = Settings.Settings.instance().createSetting('bypassServiceWorker', false);
        if (this._bypassServiceWorkerSetting.get()) {
            this._bypassServiceWorkerChanged();
        }
        this._bypassServiceWorkerSetting.addChangeListener(this._bypassServiceWorkerChanged, this);
        Settings.Settings.instance()
            .moduleSetting('cacheDisabled')
            .addChangeListener(this._cacheDisabledSettingChanged, this);
    }
    /**
     * @param {!NetworkRequest} request
     * @return {?NetworkManager}
     */
    static forRequest(request) {
        return requestToManagerMap.get(request) || null;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {boolean}
     */
    static canReplayRequest(request) {
        return !!requestToManagerMap.get(request) && request.resourceType() === ResourceType.resourceTypes.XHR;
    }
    /**
     * @param {!NetworkRequest} request
     */
    static replayRequest(request) {
        const manager = requestToManagerMap.get(request);
        if (!manager) {
            return;
        }
        manager._networkAgent.invoke_replayXHR({ requestId: request.requestId() });
    }
    /**
     * @param {!NetworkRequest} request
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    static async searchInRequest(request, query, caseSensitive, isRegex) {
        const manager = NetworkManager.forRequest(request);
        if (!manager) {
            return [];
        }
        const response = await manager._networkAgent.invoke_searchInResponseBody({ requestId: request.requestId(), query: query, caseSensitive: caseSensitive, isRegex: isRegex });
        return response.result || [];
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!Promise<!ContentData>}
     */
    static async requestContentData(request) {
        if (request.resourceType() === ResourceType.resourceTypes.WebSocket) {
            return { error: 'Content for WebSockets is currently not supported', content: null, encoded: false };
        }
        if (!request.finished) {
            await request.once(Events$2.FinishedLoading);
        }
        const manager = NetworkManager.forRequest(request);
        if (!manager) {
            return { error: 'No network manager for request', content: null, encoded: false };
        }
        const response = await manager._networkAgent.invoke_getResponseBody({ requestId: request.requestId() });
        const error = response.getError() || null;
        return { error: error, content: error ? null : response.body, encoded: response.base64Encoded };
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!Promise<?string>}
     */
    static async requestPostData(request) {
        const manager = NetworkManager.forRequest(request);
        if (manager) {
            try {
                const { postData } = await manager._networkAgent.invoke_getRequestPostData({ requestId: request.backendRequestId() });
                return postData;
            }
            catch (e) {
                return e.message;
            }
        }
        console.error('No network manager for request');
        return /** @type {!Promise<?string>} */ (Promise.resolve(null));
    }
    /**
     * @param {!Conditions} conditions
     * @return {!Protocol.Network.ConnectionType}
     * TODO(allada): this belongs to NetworkConditionsSelector, which should hardcode/guess it.
     */
    static _connectionType(conditions) {
        if (!conditions.download && !conditions.upload) {
            return Protocol.Network.ConnectionType.None;
        }
        const title = conditions.title.toLowerCase();
        for (const [name, protocolType] of CONNECTION_TYPES) {
            if (title.includes(name)) {
                return protocolType;
            }
        }
        return Protocol.Network.ConnectionType.Other;
    }
    /**
     * @param {!Object<string,string>} headers
     * @return {!Object<string, string>}
     */
    static lowercaseHeaders(headers) {
        /** @type {!Object<string, string>} */
        const newHeaders = {};
        for (const headerName in headers) {
            newHeaders[headerName.toLowerCase()] = headers[headerName];
        }
        return newHeaders;
    }
    /**
     * @param {string} url
     * @return {!NetworkRequest}
     */
    inflightRequestForURL(url) {
        return this._dispatcher._inflightRequestsByURL[url];
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _cacheDisabledSettingChanged(event) {
        const enabled = /** @type {boolean} */ (event.data);
        this._networkAgent.invoke_setCacheDisabled({ cacheDisabled: enabled });
    }
    /**
     * @override
     */
    dispose() {
        Settings.Settings.instance()
            .moduleSetting('cacheDisabled')
            .removeChangeListener(this._cacheDisabledSettingChanged, this);
    }
    _bypassServiceWorkerChanged() {
        this._networkAgent.invoke_setBypassServiceWorker({ bypass: this._bypassServiceWorkerSetting.get() });
    }
    /**
     * @param {string} frameId
     * @returns {!Promise<?Protocol.Network.SecurityIsolationStatus>}
     */
    async getSecurityIsolationStatus(frameId) {
        const result = await this._networkAgent.invoke_getSecurityIsolationStatus({ frameId });
        if (result.getError()) {
            return null;
        }
        return result.status;
    }
    /**
      * @param {string} frameId
      * @param {string} url
      * @param {!Protocol.Network.LoadNetworkResourceOptions} options
      * @return {!Promise<!Protocol.Network.LoadNetworkResourcePageResult>}
      */
    async loadNetworkResource(frameId, url, options) {
        const result = await this._networkAgent.invoke_loadNetworkResource({ frameId, url, options });
        if (result.getError()) {
            throw new Error(result.getError());
        }
        return result.resource;
    }
}
/** @enum {symbol} */
const Events$1 = {
    RequestStarted: Symbol('RequestStarted'),
    RequestUpdated: Symbol('RequestUpdated'),
    RequestFinished: Symbol('RequestFinished'),
    RequestUpdateDropped: Symbol('RequestUpdateDropped'),
    ResponseReceived: Symbol('ResponseReceived'),
    MessageGenerated: Symbol('MessageGenerated'),
    RequestRedirected: Symbol('RequestRedirected'),
    LoadingFinished: Symbol('LoadingFinished'),
};
/** @type {!Conditions} */
const NoThrottlingConditions = {
    title: UIString.UIString('Online'),
    download: -1,
    upload: -1,
    latency: 0
};
/** @type {!Conditions} */
const OfflineConditions = {
    title: UIString.UIString('Offline'),
    download: 0,
    upload: 0,
    latency: 0,
};
/** @type {!Conditions} */
const Slow3GConditions = {
    title: UIString.UIString('Slow 3G'),
    download: 500 * 1024 / 8 * .8,
    upload: 500 * 1024 / 8 * .8,
    latency: 400 * 5,
};
/** @type {!Conditions} */
const Fast3GConditions = {
    title: UIString.UIString('Fast 3G'),
    download: 1.6 * 1024 * 1024 / 8 * .9,
    upload: 750 * 1024 / 8 * .9,
    latency: 150 * 3.75,
};
const MAX_EAGER_POST_REQUEST_BODY_LENGTH = 64 * 1024; // bytes
/**
 * @implements {ProtocolProxyApi.NetworkDispatcher}
 * @unrestricted
 */
class NetworkDispatcher {
    /**
     * @param {!NetworkManager} manager
     */
    constructor(manager) {
        this._manager = manager;
        /** @type {!Map<!Protocol.Network.RequestId, !NetworkRequest>} */
        this._inflightRequestsById = new Map();
        /** @type {!Object<string, !NetworkRequest>} */
        this._inflightRequestsByURL = {};
        /** @type {!Map<string, !RedirectExtraInfoBuilder>} */
        this._requestIdToRedirectExtraInfoBuilder = new Map();
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @param {!Protocol.Network.Headers} headersMap
     * @return {!Array.<!NameValue>}
     */
    _headersMapToHeadersArray(headersMap) {
        const result = [];
        for (const name in headersMap) {
            const values = headersMap[name].split('\n');
            for (let i = 0; i < values.length; ++i) {
                result.push({ name: name, value: values[i] });
            }
        }
        return result;
    }
    /**
     * @param {!NetworkRequest} networkRequest
     * @param {!Protocol.Network.Request} request
     */
    _updateNetworkRequestWithRequest(networkRequest, request) {
        networkRequest.requestMethod = request.method;
        networkRequest.setRequestHeaders(this._headersMapToHeadersArray(request.headers));
        networkRequest.setRequestFormData(!!request.hasPostData, request.postData || null);
        networkRequest.setInitialPriority(request.initialPriority);
        networkRequest.mixedContentType = request.mixedContentType || Protocol.Security.MixedContentType.None;
        networkRequest.setReferrerPolicy(request.referrerPolicy);
    }
    /**
     * @param {!NetworkRequest} networkRequest
     * @param {!Protocol.Network.Response} response
     */
    _updateNetworkRequestWithResponse(networkRequest, response) {
        if (response.url && networkRequest.url() !== response.url) {
            networkRequest.setUrl(response.url);
        }
        networkRequest.mimeType = /** @type {!MIME_TYPE} */ (response.mimeType);
        networkRequest.statusCode = response.status;
        networkRequest.statusText = response.statusText;
        if (!networkRequest.hasExtraResponseInfo()) {
            networkRequest.responseHeaders = this._headersMapToHeadersArray(response.headers);
        }
        if (response.encodedDataLength >= 0) {
            networkRequest.setTransferSize(response.encodedDataLength);
        }
        if (response.requestHeaders && !networkRequest.hasExtraRequestInfo()) {
            // TODO(http://crbug.com/1004979): Stop using response.requestHeaders and
            //   response.requestHeadersText once shared workers
            //   emit Network.*ExtraInfo events for their network requests.
            networkRequest.setRequestHeaders(this._headersMapToHeadersArray(response.requestHeaders));
            networkRequest.setRequestHeadersText(response.requestHeadersText || '');
        }
        networkRequest.connectionReused = response.connectionReused;
        networkRequest.connectionId = String(response.connectionId);
        if (response.remoteIPAddress) {
            networkRequest.setRemoteAddress(response.remoteIPAddress, response.remotePort || -1);
        }
        if (response.fromServiceWorker) {
            networkRequest.fetchedViaServiceWorker = true;
        }
        if (response.fromDiskCache) {
            networkRequest.setFromDiskCache();
        }
        if (response.fromPrefetchCache) {
            networkRequest.setFromPrefetchCache();
        }
        if (response.cacheStorageCacheName) {
            networkRequest.setResponseCacheStorageCacheName(response.cacheStorageCacheName);
        }
        if (response.responseTime) {
            networkRequest.setResponseRetrievalTime(new Date(response.responseTime));
        }
        networkRequest.timing = response.timing;
        networkRequest.protocol = response.protocol || '';
        if (response.serviceWorkerResponseSource) {
            networkRequest.setServiceWorkerResponseSource(response.serviceWorkerResponseSource);
        }
        networkRequest.setSecurityState(response.securityState);
        if (!this._mimeTypeIsConsistentWithType(networkRequest)) {
            const message = UIString.UIString('Resource interpreted as %s but transferred with MIME type %s: "%s".', networkRequest.resourceType().title(), networkRequest.mimeType, networkRequest.url());
            this._manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: networkRequest.requestId(), warning: true });
        }
        if (response.securityDetails) {
            networkRequest.setSecurityDetails(response.securityDetails);
        }
    }
    /**
     * @param {!NetworkRequest} networkRequest
     * @return {boolean}
     */
    _mimeTypeIsConsistentWithType(networkRequest) {
        // If status is an error, content is likely to be of an inconsistent type,
        // as it's going to be an error message. We do not want to emit a warning
        // for this, though, as this will already be reported as resource loading failure.
        // Also, if a URL like http://localhost/wiki/load.php?debug=true&lang=en produces text/css and gets reloaded,
        // it is 304 Not Modified and its guessed mime-type is text/php, which is wrong.
        // Don't check for mime-types in 304-resources.
        if (networkRequest.hasErrorStatusCode() || networkRequest.statusCode === 304 || networkRequest.statusCode === 204) {
            return true;
        }
        const resourceType = networkRequest.resourceType();
        if (resourceType !== ResourceType.resourceTypes.Stylesheet &&
            resourceType !== ResourceType.resourceTypes.Document &&
            resourceType !== ResourceType.resourceTypes.TextTrack) {
            return true;
        }
        if (!networkRequest.mimeType) {
            return true;
        } // Might be not known for cached resources with null responses.
        if (MIME_TYPE_TO_RESOURCE_TYPE.has(networkRequest.mimeType)) {
            return resourceType.name() in MIME_TYPE_TO_RESOURCE_TYPE.get(networkRequest.mimeType);
        }
        return false;
    }
    /**
     * @override
     * @param {!Protocol.Network.ResourceChangedPriorityEvent} request
     */
    resourceChangedPriority({ requestId, newPriority, timestamp }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (networkRequest) {
            networkRequest.setPriority(newPriority);
        }
    }
    /**
     * @override
     * @param {!Protocol.Network.SignedExchangeReceivedEvent} request
     */
    signedExchangeReceived({ requestId, info }) {
        // While loading a signed exchange, a signedExchangeReceived event is sent
        // between two requestWillBeSent events.
        // 1. The first requestWillBeSent is sent while starting the navigation (or
        //    prefetching).
        // 2. This signedExchangeReceived event is sent when the browser detects the
        //    signed exchange.
        // 3. The second requestWillBeSent is sent with the generated redirect
        //    response and a new redirected request which URL is the inner request
        //    URL of the signed exchange.
        let networkRequest = this._inflightRequestsById.get(requestId);
        // |requestId| is available only for navigation requests. If the request was
        // sent from a renderer process for prefetching, it is not available. In the
        // case, need to fallback to look for the URL.
        // TODO(crbug/841076): Sends the request ID of prefetching to the browser
        // process and DevTools to find the matching request.
        if (!networkRequest) {
            networkRequest = this._inflightRequestsByURL[info.outerResponse.url];
            if (!networkRequest) {
                return;
            }
        }
        networkRequest.setSignedExchangeInfo(info);
        networkRequest.setResourceType(ResourceType.resourceTypes.SignedExchange);
        this._updateNetworkRequestWithResponse(networkRequest, info.outerResponse);
        this._updateNetworkRequest(networkRequest);
        this._manager.dispatchEventToListeners(Events$1.ResponseReceived, { request: networkRequest, response: info.outerResponse });
    }
    /**
     * @override
     * @param {!Protocol.Network.RequestWillBeSentEvent} request
     */
    requestWillBeSent({ requestId, loaderId, documentURL, request, timestamp, wallTime, initiator, redirectResponse, type, frameId }) {
        let networkRequest = this._inflightRequestsById.get(requestId);
        if (networkRequest) {
            // FIXME: move this check to the backend.
            if (!redirectResponse) {
                return;
            }
            // If signedExchangeReceived event has already been sent for the request,
            // ignores the internally generated |redirectResponse|. The
            // |outerResponse| of SignedExchangeInfo was set to |networkRequest| in
            // signedExchangeReceived().
            if (!networkRequest.signedExchangeInfo()) {
                this.responseReceived({
                    requestId,
                    loaderId,
                    timestamp,
                    type: type || Protocol.Network.ResourceType.Other,
                    response: redirectResponse,
                    frameId
                });
            }
            networkRequest = this._appendRedirect(requestId, timestamp, request.url);
            this._manager.dispatchEventToListeners(Events$1.RequestRedirected, networkRequest);
        }
        else {
            networkRequest =
                this._createNetworkRequest(requestId, frameId || '', loaderId, request.url, documentURL, initiator);
        }
        networkRequest.hasNetworkData = true;
        this._updateNetworkRequestWithRequest(networkRequest, request);
        networkRequest.setIssueTime(timestamp, wallTime);
        networkRequest.setResourceType(type ? ResourceType.resourceTypes[type] : ResourceType.resourceTypes.Other);
        this._getExtraInfoBuilder(requestId).addRequest(networkRequest);
        this._startNetworkRequest(networkRequest, request);
    }
    /**
     * @override
     * @param {!Protocol.Network.RequestServedFromCacheEvent} request
     */
    requestServedFromCache({ requestId }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.setFromMemoryCache();
    }
    /**
     * @override
     * @param {!Protocol.Network.ResponseReceivedEvent} request
     */
    responseReceived({ requestId, loaderId, timestamp, type, response, frameId }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        const lowercaseHeaders = NetworkManager.lowercaseHeaders(response.headers);
        if (!networkRequest) {
            const lastModifiedHeader = lowercaseHeaders['last-modified'];
            // We missed the requestWillBeSent.
            /** @type {!RequestUpdateDroppedEventData} */
            const eventData = {
                url: response.url,
                frameId: frameId || '',
                loaderId: loaderId,
                resourceType: type,
                mimeType: response.mimeType,
                lastModified: lastModifiedHeader ? new Date(lastModifiedHeader) : null,
            };
            this._manager.dispatchEventToListeners(Events$1.RequestUpdateDropped, eventData);
            return;
        }
        networkRequest.responseReceivedTime = timestamp;
        networkRequest.setResourceType(ResourceType.resourceTypes[type]);
        // net::ParsedCookie::kMaxCookieSize = 4096 (net/cookies/parsed_cookie.h)
        if ('set-cookie' in lowercaseHeaders && lowercaseHeaders['set-cookie'].length > 4096) {
            const values = lowercaseHeaders['set-cookie'].split('\n');
            for (let i = 0; i < values.length; ++i) {
                if (values[i].length <= 4096) {
                    continue;
                }
                const message = UIString.UIString('Set-Cookie header is ignored in response from url: %s. Cookie length should be less than or equal to 4096 characters.', response.url);
                this._manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: requestId, warning: true });
            }
        }
        this._updateNetworkRequestWithResponse(networkRequest, response);
        this._updateNetworkRequest(networkRequest);
        this._manager.dispatchEventToListeners(Events$1.ResponseReceived, { request: networkRequest, response });
    }
    /**
     * @override
     * @param {!Protocol.Network.DataReceivedEvent} request
     */
    dataReceived({ requestId, timestamp, dataLength, encodedDataLength }) {
        /** @type {?(!NetworkRequest|undefined)} */
        let networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            networkRequest = this._maybeAdoptMainResourceRequest(requestId);
        }
        if (!networkRequest) {
            return;
        }
        networkRequest.resourceSize += dataLength;
        if (encodedDataLength !== -1) {
            networkRequest.increaseTransferSize(encodedDataLength);
        }
        networkRequest.endTime = timestamp;
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.LoadingFinishedEvent} request
     */
    loadingFinished({ requestId, timestamp: finishTime, encodedDataLength, shouldReportCorbBlocking }) {
        /** @type {?(!NetworkRequest|undefined)} */
        let networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            networkRequest = this._maybeAdoptMainResourceRequest(requestId);
        }
        if (!networkRequest) {
            return;
        }
        this._getExtraInfoBuilder(requestId).finished();
        this._finishNetworkRequest(networkRequest, finishTime, encodedDataLength, shouldReportCorbBlocking);
        this._manager.dispatchEventToListeners(Events$1.LoadingFinished, networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.LoadingFailedEvent} request
     */
    loadingFailed({ requestId, timestamp: time, type: resourceType, errorText: localizedDescription, canceled, blockedReason }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.failed = true;
        networkRequest.setResourceType(ResourceType.resourceTypes[resourceType]);
        networkRequest.canceled = !!canceled;
        if (blockedReason) {
            networkRequest.setBlockedReason(blockedReason);
            if (blockedReason === Protocol.Network.BlockedReason.Inspector) {
                const message = UIString.UIString('Request was blocked by DevTools: "%s".', networkRequest.url());
                this._manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: requestId, warning: true });
            }
        }
        networkRequest.localizedFailDescription = localizedDescription;
        this._getExtraInfoBuilder(requestId).finished();
        this._finishNetworkRequest(networkRequest, time, -1);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketCreatedEvent} request
     */
    webSocketCreated({ requestId, url: requestURL, initiator }) {
        const networkRequest = new NetworkRequest(requestId, requestURL, '', '', '', initiator || null);
        requestToManagerMap.set(networkRequest, this._manager);
        networkRequest.setResourceType(ResourceType.resourceTypes.WebSocket);
        this._startNetworkRequest(networkRequest, null);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketWillSendHandshakeRequestEvent} request
     */
    webSocketWillSendHandshakeRequest({ requestId, timestamp: time, wallTime, request }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.requestMethod = 'GET';
        networkRequest.setRequestHeaders(this._headersMapToHeadersArray(request.headers));
        networkRequest.setIssueTime(time, wallTime);
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketHandshakeResponseReceivedEvent} request
     */
    webSocketHandshakeResponseReceived({ requestId, timestamp: time, response }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.statusCode = response.status;
        networkRequest.statusText = response.statusText;
        networkRequest.responseHeaders = this._headersMapToHeadersArray(response.headers);
        networkRequest.responseHeadersText = response.headersText || '';
        if (response.requestHeaders) {
            networkRequest.setRequestHeaders(this._headersMapToHeadersArray(response.requestHeaders));
        }
        if (response.requestHeadersText) {
            networkRequest.setRequestHeadersText(response.requestHeadersText);
        }
        networkRequest.responseReceivedTime = time;
        networkRequest.protocol = 'websocket';
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketFrameReceivedEvent} request
     */
    webSocketFrameReceived({ requestId, timestamp: time, response }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.addProtocolFrame(response, time, false);
        networkRequest.responseReceivedTime = time;
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketFrameSentEvent} request
     */
    webSocketFrameSent({ requestId, timestamp: time, response }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.addProtocolFrame(response, time, true);
        networkRequest.responseReceivedTime = time;
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketFrameErrorEvent} request
     */
    webSocketFrameError({ requestId, timestamp: time, errorMessage }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.addProtocolFrameError(errorMessage, time);
        networkRequest.responseReceivedTime = time;
        this._updateNetworkRequest(networkRequest);
    }
    /**
     * @override
     * @param {!Protocol.Network.WebSocketClosedEvent} request
     */
    webSocketClosed({ requestId, timestamp: time }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        this._finishNetworkRequest(networkRequest, time, -1);
    }
    /**
     * @override
     * @param {!Protocol.Network.EventSourceMessageReceivedEvent} request
     */
    eventSourceMessageReceived({ requestId, timestamp: time, eventName, eventId, data }) {
        const networkRequest = this._inflightRequestsById.get(requestId);
        if (!networkRequest) {
            return;
        }
        networkRequest.addEventSourceMessage(time, eventName, eventId, data);
    }
    /**
     * @override
     * @param {!Protocol.Network.RequestInterceptedEvent} request
     */
    requestIntercepted({ interceptionId, request, frameId, resourceType, isNavigationRequest, isDownload, redirectUrl, authChallenge, responseErrorReason, responseStatusCode, responseHeaders, requestId }) {
        MultitargetNetworkManager.instance()._requestIntercepted(new InterceptedRequest(this._manager.target().networkAgent(), interceptionId, request, frameId, resourceType, isNavigationRequest, isDownload, redirectUrl, authChallenge, responseErrorReason, responseStatusCode, responseHeaders, requestId));
    }
    /**
     * @override
     * @param {!Protocol.Network.RequestWillBeSentExtraInfoEvent} request
     */
    requestWillBeSentExtraInfo({ requestId, associatedCookies, headers }) {
        /** @type {!Array<!BlockedCookieWithReason>} */
        const blockedRequestCookies = [];
        const includedRequestCookies = [];
        for (const { blockedReasons, cookie } of associatedCookies) {
            if (blockedReasons.length === 0) {
                includedRequestCookies.push(Cookie.fromProtocolCookie(cookie));
            }
            else {
                blockedRequestCookies.push({ blockedReasons, cookie: Cookie.fromProtocolCookie(cookie) });
            }
        }
        const extraRequestInfo = {
            blockedRequestCookies,
            includedRequestCookies,
            requestHeaders: this._headersMapToHeadersArray(headers)
        };
        this._getExtraInfoBuilder(requestId).addRequestExtraInfo(extraRequestInfo);
    }
    /**
     * @override
     * @param {!Protocol.Network.ResponseReceivedExtraInfoEvent} request
     */
    responseReceivedExtraInfo({ requestId, blockedCookies, headers, headersText }) {
        /** @type {!ExtraResponseInfo} */
        const extraResponseInfo = {
            blockedResponseCookies: blockedCookies.map(blockedCookie => {
                return {
                    blockedReasons: blockedCookie.blockedReasons,
                    cookieLine: blockedCookie.cookieLine,
                    cookie: blockedCookie.cookie ? Cookie.fromProtocolCookie(blockedCookie.cookie) : null
                };
            }),
            responseHeaders: this._headersMapToHeadersArray(headers),
            responseHeadersText: headersText
        };
        this._getExtraInfoBuilder(requestId).addResponseExtraInfo(extraResponseInfo);
    }
    /**
     * @param {string} requestId
     * @return {!RedirectExtraInfoBuilder}
     */
    _getExtraInfoBuilder(requestId) {
        /** @type {!RedirectExtraInfoBuilder} */
        let builder;
        if (!this._requestIdToRedirectExtraInfoBuilder.has(requestId)) {
            const deleteCallback = () => {
                this._requestIdToRedirectExtraInfoBuilder.delete(requestId);
            };
            builder = new RedirectExtraInfoBuilder(deleteCallback);
            this._requestIdToRedirectExtraInfoBuilder.set(requestId, builder);
        }
        else {
            builder = /** @type {!RedirectExtraInfoBuilder} */ (this._requestIdToRedirectExtraInfoBuilder.get(requestId));
        }
        return builder;
    }
    /**
     * @param {!Protocol.Network.RequestId} requestId
     * @param {!Protocol.Network.MonotonicTime} time
     * @param {string} redirectURL
     * @return {!NetworkRequest}
     */
    _appendRedirect(requestId, time, redirectURL) {
        const originalNetworkRequest = this._inflightRequestsById.get(requestId);
        if (!originalNetworkRequest) {
            throw new Error(`Could not find original network request for ${requestId}`);
        }
        let redirectCount = 0;
        for (let redirect = originalNetworkRequest.redirectSource(); redirect; redirect = redirect.redirectSource()) {
            redirectCount++;
        }
        originalNetworkRequest.markAsRedirect(redirectCount);
        this._finishNetworkRequest(originalNetworkRequest, time, -1);
        const newNetworkRequest = this._createNetworkRequest(requestId, originalNetworkRequest.frameId, originalNetworkRequest.loaderId, redirectURL, originalNetworkRequest.documentURL, originalNetworkRequest.initiator());
        newNetworkRequest.setRedirectSource(originalNetworkRequest);
        originalNetworkRequest.setRedirectDestination(newNetworkRequest);
        return newNetworkRequest;
    }
    /**
     * @param {string} requestId
     * @return {?NetworkRequest}
     */
    _maybeAdoptMainResourceRequest(requestId) {
        const request = MultitargetNetworkManager.instance()._inflightMainResourceRequests.get(requestId);
        if (!request) {
            return null;
        }
        const oldDispatcher = /** @type {!NetworkManager} */ (NetworkManager.forRequest(request))._dispatcher;
        oldDispatcher._inflightRequestsById.delete(requestId);
        delete oldDispatcher._inflightRequestsByURL[request.url()];
        this._inflightRequestsById.set(requestId, request);
        this._inflightRequestsByURL[request.url()] = request;
        requestToManagerMap.set(request, this._manager);
        return request;
    }
    /**
     * @param {!NetworkRequest} networkRequest
     * @param {?Protocol.Network.Request} originalRequest
     */
    _startNetworkRequest(networkRequest, originalRequest) {
        this._inflightRequestsById.set(networkRequest.requestId(), networkRequest);
        this._inflightRequestsByURL[networkRequest.url()] = networkRequest;
        // The following relies on the fact that loaderIds and requestIds are
        // globally unique and that the main request has them equal.
        if (networkRequest.loaderId === networkRequest.requestId()) {
            MultitargetNetworkManager.instance()._inflightMainResourceRequests.set(networkRequest.requestId(), networkRequest);
        }
        this._manager.dispatchEventToListeners(Events$1.RequestStarted, { request: networkRequest, originalRequest });
    }
    /**
     * @param {!NetworkRequest} networkRequest
     */
    _updateNetworkRequest(networkRequest) {
        this._manager.dispatchEventToListeners(Events$1.RequestUpdated, networkRequest);
    }
    /**
     * @param {!NetworkRequest} networkRequest
     * @param {!Protocol.Network.MonotonicTime} finishTime
     * @param {number} encodedDataLength
     * @param {boolean=} shouldReportCorbBlocking
     */
    _finishNetworkRequest(networkRequest, finishTime, encodedDataLength, shouldReportCorbBlocking) {
        networkRequest.endTime = finishTime;
        networkRequest.finished = true;
        if (encodedDataLength >= 0) {
            const redirectSource = networkRequest.redirectSource();
            if (redirectSource && redirectSource.signedExchangeInfo()) {
                networkRequest.setTransferSize(0);
                redirectSource.setTransferSize(encodedDataLength);
                this._updateNetworkRequest(redirectSource);
            }
            else {
                networkRequest.setTransferSize(encodedDataLength);
            }
        }
        this._manager.dispatchEventToListeners(Events$1.RequestFinished, networkRequest);
        this._inflightRequestsById.delete(networkRequest.requestId());
        delete this._inflightRequestsByURL[networkRequest.url()];
        MultitargetNetworkManager.instance()._inflightMainResourceRequests.delete(networkRequest.requestId());
        if (shouldReportCorbBlocking) {
            const message = UIString.UIString('Cross-Origin Read Blocking (CORB) blocked cross-origin response %s with MIME type %s. See https://www.chromestatus.com/feature/5629709824032768 for more details.', networkRequest.url(), networkRequest.mimeType);
            this._manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: networkRequest.requestId(), warning: true });
        }
        if (Settings.Settings.instance().moduleSetting('monitoringXHREnabled').get() &&
            networkRequest.resourceType().category() === ResourceType.resourceCategories.XHR) {
            let message;
            const failedToLoad = networkRequest.failed || networkRequest.hasErrorStatusCode();
            if (failedToLoad) {
                message = UIString.UIString('%s failed loading: %s "%s".', networkRequest.resourceType().title(), networkRequest.requestMethod, networkRequest.url());
            }
            else {
                message = UIString.UIString('%s finished loading: %s "%s".', networkRequest.resourceType().title(), networkRequest.requestMethod, networkRequest.url());
            }
            this._manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: networkRequest.requestId(), warning: false });
        }
    }
    /**
     * @param {!Protocol.Network.RequestId} requestId
     * @param {string} frameId
     * @param {!Protocol.Network.LoaderId} loaderId
     * @param {string} url
     * @param {string} documentURL
     * @param {?Protocol.Network.Initiator} initiator
     */
    _createNetworkRequest(requestId, frameId, loaderId, url, documentURL, initiator) {
        const request = new NetworkRequest(requestId, url, documentURL, frameId, loaderId, initiator);
        requestToManagerMap.set(request, this._manager);
        return request;
    }
}
/**
 * @type {?MultitargetNetworkManager}
 */
let multiTargetNetworkManagerInstance;
/**
 * @implements {SDKModelObserver<!NetworkManager>}
 * @unrestricted
 */
class MultitargetNetworkManager extends ObjectWrapper.ObjectWrapper {
    constructor() {
        super();
        /** @type {string} */
        this._userAgentOverride = '';
        /** @type {?Protocol.Emulation.UserAgentMetadata} */
        this._userAgentMetadataOverride = null;
        /** @type {!Set<!ProtocolProxyApi.NetworkApi>} */
        this._agents = new Set();
        /** @type {!Map<string, !NetworkRequest>} */
        this._inflightMainResourceRequests = new Map();
        /** @type {!Conditions} */
        this._networkConditions = NoThrottlingConditions;
        /** @type {?Promise<void>} */
        this._updatingInterceptionPatternsPromise = null;
        // TODO(allada) Remove these and merge it with request interception.
        this._blockingEnabledSetting = Settings.Settings.instance().moduleSetting('requestBlockingEnabled');
        this._blockedPatternsSetting = Settings.Settings.instance().createSetting('networkBlockedPatterns', []);
        /** @type {!Array<string>} */
        this._effectiveBlockedURLs = [];
        this._updateBlockedPatterns();
        /** @type {!Platform.Multimap<!RequestInterceptor, !InterceptionPattern>} */
        this._urlsForRequestInterceptor = new Multimap();
        TargetManager.instance().observeModels(NetworkManager, this);
    }
    /**
     * @param {{forceNew: ?boolean}} opts
     * @return {!MultitargetNetworkManager}
     */
    static instance(opts = { forceNew: null }) {
        const { forceNew } = opts;
        if (!multiTargetNetworkManagerInstance || forceNew) {
            multiTargetNetworkManagerInstance = new MultitargetNetworkManager();
        }
        return multiTargetNetworkManagerInstance;
    }
    /**
     * @return {string}
     */
    static getChromeVersion() {
        const chromeRegex = new RegExp('(?:^|\\W)Chrome/(\\S+)');
        const chromeMatch = navigator.userAgent.match(chromeRegex);
        if (chromeMatch && chromeMatch.length > 1) {
            return chromeMatch[1];
        }
        return '';
    }
    /**
      * Generate a brand version list for Chrome, including some randomization
      * to try to ensure proper parsing behavior. See
      *
      * https://wicg.github.io/ua-client-hints/#grease
      *
      * This implementation matches GenerateBrandVersionList() in
      * chrome_content_browser_client.cc
      * @param {number} seed
      * @param {string} brand
      * @param {string} majorVersion
      *
      * @return {!Array<!Protocol.Emulation.UserAgentBrandVersion>}
      */
    static _generateBrandVersionList(seed, brand, majorVersion) {
        // Pick a stable permutation seeded by major version number. any values here
        // and in order should be under three.
        const orders = [[0, 1, 2], [0, 2, 1], [1, 0, 2], [1, 2, 0], [2, 0, 1], [2, 1, 0]];
        const permutation = seed % orders.length;
        const order = orders[permutation];
        const escapedChars = ['\\', '\"', ';'];
        const greaseyBrand = escapedChars[order[0]] + 'Not' + escapedChars[order[1]] + 'A' + escapedChars[order[2]] + 'Brand';
        const greasey = { brand: greaseyBrand, version: '99' };
        const chromium = { brand: 'Chromium', version: majorVersion };
        const branded = { brand: brand, version: majorVersion };
        const greasedBrandVersionList = /** @type {!Array<!Protocol.Emulation.UserAgentBrandVersion>} */ ([]);
        greasedBrandVersionList[order[0]] = greasey;
        greasedBrandVersionList[order[1]] = chromium;
        greasedBrandVersionList[order[2]] = branded;
        return greasedBrandVersionList;
    }
    /**
     * @return {!Array<!Protocol.Emulation.UserAgentBrandVersion>}
     */
    static getChromeBrands() {
        const chromeVersion = MultitargetNetworkManager.getChromeVersion();
        if (chromeVersion.length > 0) {
            const majorVersion = chromeVersion.split('.', 1)[0];
            return MultitargetNetworkManager._generateBrandVersionList(Number.parseInt(majorVersion, 10), 'Google Chrome', majorVersion);
        }
        return [];
    }
    /**
     * @param {string} uaString
     * @return {string}
     */
    static patchUserAgentWithChromeVersion(uaString) {
        // Patches Chrome/CriOS version from user agent ("1.2.3.4" when user agent is: "Chrome/1.2.3.4").
        // Edge also contains an appVersion which should be patched to match the Chrome major version.
        // Otherwise, ignore it. This assumes additional appVersions appear after the Chrome version.
        const chromeVersion = MultitargetNetworkManager.getChromeVersion();
        if (chromeVersion.length > 0) {
            // "1.2.3.4" becomes "1.0.100.0"
            const additionalAppVersion = chromeVersion.split('.', 1)[0] + '.0.100.0';
            return StringUtilities.sprintf(uaString, chromeVersion, additionalAppVersion);
        }
        return uaString;
    }
    /**
     * @override
     * @param {!NetworkManager} networkManager
     */
    modelAdded(networkManager) {
        const networkAgent = networkManager.target().networkAgent();
        if (this._extraHeaders) {
            networkAgent.invoke_setExtraHTTPHeaders({ headers: this._extraHeaders });
        }
        if (this.currentUserAgent()) {
            networkAgent.invoke_setUserAgentOverride({ userAgent: this.currentUserAgent(), userAgentMetadata: this._userAgentMetadataOverride || undefined });
        }
        if (this._effectiveBlockedURLs.length) {
            networkAgent.invoke_setBlockedURLs({ urls: this._effectiveBlockedURLs });
        }
        if (this.isIntercepting()) {
            networkAgent.invoke_setRequestInterception({ patterns: this._urlsForRequestInterceptor.valuesArray() });
        }
        this._agents.add(networkAgent);
        if (this.isThrottling()) {
            this._updateNetworkConditions(networkAgent);
        }
    }
    /**
     * @override
     * @param {!NetworkManager} networkManager
     */
    modelRemoved(networkManager) {
        for (const entry of this._inflightMainResourceRequests) {
            const manager = NetworkManager.forRequest(/** @type {!NetworkRequest} */ (entry[1]));
            if (manager !== networkManager) {
                continue;
            }
            this._inflightMainResourceRequests.delete(/** @type {string} */ (entry[0]));
        }
        this._agents.delete(networkManager.target().networkAgent());
    }
    /**
     * @return {boolean}
     */
    isThrottling() {
        return this._networkConditions.download >= 0 || this._networkConditions.upload >= 0 ||
            this._networkConditions.latency > 0;
    }
    /**
     * @return {boolean}
     */
    isOffline() {
        return !this._networkConditions.download && !this._networkConditions.upload;
    }
    /**
     * @param {!Conditions} conditions
     */
    setNetworkConditions(conditions) {
        this._networkConditions = conditions;
        for (const agent of this._agents) {
            this._updateNetworkConditions(agent);
        }
        this.dispatchEventToListeners(MultitargetNetworkManager.Events.ConditionsChanged);
    }
    /**
     * @return {!Conditions}
     */
    networkConditions() {
        return this._networkConditions;
    }
    /**
     * @param {!ProtocolProxyApi.NetworkApi} networkAgent
     */
    _updateNetworkConditions(networkAgent) {
        const conditions = this._networkConditions;
        if (!this.isThrottling()) {
            networkAgent.invoke_emulateNetworkConditions({ offline: false, latency: 0, downloadThroughput: 0, uploadThroughput: 0 });
        }
        else {
            networkAgent.invoke_emulateNetworkConditions({
                offline: this.isOffline(),
                latency: conditions.latency,
                downloadThroughput: conditions.download < 0 ? 0 : conditions.download,
                uploadThroughput: conditions.upload < 0 ? 0 : conditions.upload,
                connectionType: NetworkManager._connectionType(conditions)
            });
        }
    }
    /**
     * @param {!Protocol.Network.Headers} headers
     */
    setExtraHTTPHeaders(headers) {
        this._extraHeaders = headers;
        for (const agent of this._agents) {
            agent.invoke_setExtraHTTPHeaders({ headers: this._extraHeaders });
        }
    }
    /**
     * @return {string}
     */
    currentUserAgent() {
        return this._customUserAgent ? this._customUserAgent : this._userAgentOverride;
    }
    _updateUserAgentOverride() {
        const userAgent = this.currentUserAgent();
        for (const agent of this._agents) {
            agent.invoke_setUserAgentOverride({ userAgent: userAgent, userAgentMetadata: this._userAgentMetadataOverride || undefined });
        }
    }
    /**
     * @param {string} userAgent
     * @param {?Protocol.Emulation.UserAgentMetadata} userAgentMetadataOverride
     */
    setUserAgentOverride(userAgent, userAgentMetadataOverride) {
        if (this._userAgentOverride === userAgent) {
            return;
        }
        this._userAgentOverride = userAgent;
        if (!this._customUserAgent) {
            this._userAgentMetadataOverride = userAgentMetadataOverride;
            this._updateUserAgentOverride();
        }
        else {
            this._userAgentMetadataOverride = null;
        }
        this.dispatchEventToListeners(MultitargetNetworkManager.Events.UserAgentChanged);
    }
    /**
     * @return {string}
     */
    userAgentOverride() {
        return this._userAgentOverride;
    }
    /**
     * @param {string} userAgent
     */
    setCustomUserAgentOverride(userAgent) {
        this._customUserAgent = userAgent;
        this._userAgentMetadataOverride = null;
        this._updateUserAgentOverride();
    }
    // TODO(allada) Move all request blocking into interception and let view manage blocking.
    /**
     * @return {!Array<!BlockedPattern>}
     */
    blockedPatterns() {
        return this._blockedPatternsSetting.get().slice();
    }
    /**
     * @return {boolean}
     */
    blockingEnabled() {
        return this._blockingEnabledSetting.get();
    }
    /**
     * @return {boolean}
     */
    isBlocking() {
        return !!this._effectiveBlockedURLs.length;
    }
    /**
     * @param {!Array<!BlockedPattern>} patterns
     */
    setBlockedPatterns(patterns) {
        this._blockedPatternsSetting.set(patterns);
        this._updateBlockedPatterns();
        this.dispatchEventToListeners(MultitargetNetworkManager.Events.BlockedPatternsChanged);
    }
    /**
     * @param {boolean} enabled
     */
    setBlockingEnabled(enabled) {
        if (this._blockingEnabledSetting.get() === enabled) {
            return;
        }
        this._blockingEnabledSetting.set(enabled);
        this._updateBlockedPatterns();
        this.dispatchEventToListeners(MultitargetNetworkManager.Events.BlockedPatternsChanged);
    }
    _updateBlockedPatterns() {
        const urls = [];
        if (this._blockingEnabledSetting.get()) {
            for (const pattern of this._blockedPatternsSetting.get()) {
                if (pattern.enabled) {
                    urls.push(pattern.url);
                }
            }
        }
        if (!urls.length && !this._effectiveBlockedURLs.length) {
            return;
        }
        this._effectiveBlockedURLs = urls;
        for (const agent of this._agents) {
            agent.invoke_setBlockedURLs({ urls: this._effectiveBlockedURLs });
        }
    }
    /**
     * @return {boolean}
     */
    isIntercepting() {
        return !!this._urlsForRequestInterceptor.size;
    }
    /**
     * @param {!Array<!InterceptionPattern>} patterns
     * @param {!RequestInterceptor} requestInterceptor
     * @return {!Promise<void>}
     */
    setInterceptionHandlerForPatterns(patterns, requestInterceptor) {
        // Note: requestInterceptors may recieve interception requests for patterns they did not subscribe to.
        this._urlsForRequestInterceptor.deleteAll(requestInterceptor);
        for (const newPattern of patterns) {
            this._urlsForRequestInterceptor.set(requestInterceptor, newPattern);
        }
        return this._updateInterceptionPatternsOnNextTick();
    }
    /**
     * @return {!Promise<void>}
     */
    _updateInterceptionPatternsOnNextTick() {
        // This is used so we can register and unregister patterns in loops without sending lots of protocol messages.
        if (!this._updatingInterceptionPatternsPromise) {
            this._updatingInterceptionPatternsPromise = Promise.resolve().then(this._updateInterceptionPatterns.bind(this));
        }
        return this._updatingInterceptionPatternsPromise;
    }
    /**
     * @return {!Promise<void>}
     */
    _updateInterceptionPatterns() {
        if (!Settings.Settings.instance().moduleSetting('cacheDisabled').get()) {
            Settings.Settings.instance().moduleSetting('cacheDisabled').set(true);
        }
        this._updatingInterceptionPatternsPromise = null;
        const promises = /** @type {!Array<!Promise<*>>} */ ([]);
        for (const agent of this._agents) {
            promises.push(agent.invoke_setRequestInterception({ patterns: this._urlsForRequestInterceptor.valuesArray() }));
        }
        this.dispatchEventToListeners(MultitargetNetworkManager.Events.InterceptorsChanged);
        return Promise.all(promises).then(values => Promise.resolve());
    }
    /**
     * @param {!InterceptedRequest} interceptedRequest
     */
    async _requestIntercepted(interceptedRequest) {
        for (const requestInterceptor of this._urlsForRequestInterceptor.keysArray()) {
            await requestInterceptor(interceptedRequest);
            if (interceptedRequest.hasResponded()) {
                return;
            }
        }
        if (!interceptedRequest.hasResponded()) {
            interceptedRequest.continueRequestWithoutChange();
        }
    }
    clearBrowserCache() {
        for (const agent of this._agents) {
            agent.invoke_clearBrowserCache();
        }
    }
    clearBrowserCookies() {
        for (const agent of this._agents) {
            agent.invoke_clearBrowserCookies();
        }
    }
    /**
     * @param {string} origin
     * @return {!Promise<!Array<string>>}
     */
    getCertificate(origin) {
        const target = TargetManager.instance().mainTarget();
        if (!target) {
            return Promise.resolve([]);
        }
        return target.networkAgent().invoke_getCertificate({ origin }).then(certificate => (certificate && certificate.tableNames) || []);
    }
    /**
     * @param {string} url
     * @return {!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>}
     */
    async loadResource(url) {
        /** @type {!Object<string, string>} */
        const headers = {};
        const currentUserAgent = this.currentUserAgent();
        if (currentUserAgent) {
            headers['User-Agent'] = currentUserAgent;
        }
        if (Settings.Settings.instance().moduleSetting('cacheDisabled').get()) {
            headers['Cache-Control'] = 'no-cache';
        }
        return new Promise(resolve => ResourceLoader.load(url, headers, (success, _responseHeaders, content, errorDescription) => {
            resolve({ success, content, errorDescription });
        }));
    }
}
/** @enum {symbol} */
MultitargetNetworkManager.Events = {
    BlockedPatternsChanged: Symbol('BlockedPatternsChanged'),
    ConditionsChanged: Symbol('ConditionsChanged'),
    UserAgentChanged: Symbol('UserAgentChanged'),
    InterceptorsChanged: Symbol('InterceptorsChanged')
};
class InterceptedRequest {
    /**
     * @param {!ProtocolProxyApi.NetworkApi} networkAgent
     * @param {!Protocol.Network.InterceptionId} interceptionId
     * @param {!Protocol.Network.Request} request
     * @param {!Protocol.Page.FrameId} frameId
     * @param {!Protocol.Network.ResourceType} resourceType
     * @param {boolean} isNavigationRequest
     * @param {boolean=} isDownload
     * @param {string=} redirectUrl
     * @param {!Protocol.Network.AuthChallenge=} authChallenge
     * @param {!Protocol.Network.ErrorReason=} responseErrorReason
     * @param {number=} responseStatusCode
     * @param {!Protocol.Network.Headers=} responseHeaders
     * @param {!Protocol.Network.RequestId=} requestId
     */
    constructor(networkAgent, interceptionId, request, frameId, resourceType, isNavigationRequest, isDownload, redirectUrl, authChallenge, responseErrorReason, responseStatusCode, responseHeaders, requestId) {
        this._networkAgent = networkAgent;
        this._interceptionId = interceptionId;
        this._hasResponded = false;
        this.request = request;
        this.frameId = frameId;
        this.resourceType = resourceType;
        this.isNavigationRequest = isNavigationRequest;
        this.isDownload = !!isDownload;
        this.redirectUrl = redirectUrl;
        this.authChallenge = authChallenge;
        this.responseErrorReason = responseErrorReason;
        this.responseStatusCode = responseStatusCode;
        this.responseHeaders = responseHeaders;
        this.requestId = requestId;
    }
    /**
     * @return {boolean}
     */
    hasResponded() {
        return this._hasResponded;
    }
    /**
     * @param {!Blob} contentBlob
     */
    async continueRequestWithContent(contentBlob) {
        this._hasResponded = true;
        const headers = [
            'HTTP/1.1 200 OK',
            'Date: ' + (new Date()).toUTCString(),
            'Server: Chrome Devtools Request Interceptor',
            'Connection: closed',
            'Content-Length: ' + contentBlob.size,
            'Content-Type: ' + contentBlob.type || 'text/x-unknown',
        ];
        const encodedResponse = await blobToBase64(new Blob([headers.join('\r\n'), '\r\n\r\n', contentBlob]));
        this._networkAgent.invoke_continueInterceptedRequest({ interceptionId: this._interceptionId, rawResponse: encodedResponse });
        /**
         * @param {!Blob} blob
         * @return {!Promise<string>}
         */
        async function blobToBase64(blob) {
            const reader = new FileReader();
            const fileContentsLoadedPromise = new Promise(resolve => {
                reader.onloadend = resolve;
            });
            reader.readAsDataURL(blob);
            await fileContentsLoadedPromise;
            if (reader.error) {
                console.error('Could not convert blob to base64.', reader.error);
                return '';
            }
            const result = reader.result;
            if (result === undefined || result === null || typeof result !== 'string') {
                console.error('Could not convert blob to base64.');
                return '';
            }
            return result.substring(result.indexOf(',') + 1);
        }
    }
    continueRequestWithoutChange() {
        console.assert(!this._hasResponded);
        this._hasResponded = true;
        this._networkAgent.invoke_continueInterceptedRequest({ interceptionId: this._interceptionId });
    }
    /**
     * @param {!Protocol.Network.ErrorReason} errorReason
     */
    continueRequestWithError(errorReason) {
        console.assert(!this._hasResponded);
        this._hasResponded = true;
        this._networkAgent.invoke_continueInterceptedRequest({ interceptionId: this._interceptionId, errorReason });
    }
    /**
     * @return {!Promise<!ContentData>}
     */
    async responseBody() {
        const response = await this._networkAgent.invoke_getResponseBodyForInterception({ interceptionId: this._interceptionId });
        const error = response.getError() || null;
        return { error: error, content: error ? null : response.body, encoded: response.base64Encoded };
    }
}
/**
 * Helper class to match requests created from requestWillBeSent with
 * requestWillBeSentExtraInfo and responseReceivedExtraInfo when they have the
 * same requestId due to redirects.
 */
class RedirectExtraInfoBuilder {
    /**
     * @param {function():void} deleteCallback
     */
    constructor(deleteCallback) {
        /** @type {!Array<!NetworkRequest>} */
        this._requests = [];
        /** @type {!Array<?ExtraRequestInfo>} */
        this._requestExtraInfos = [];
        /** @type {!Array<?ExtraResponseInfo>} */
        this._responseExtraInfos = [];
        /** @type {boolean} */
        this._finished = false;
        /** @type {boolean} */
        this._hasExtraInfo = false;
        /** @type {function():void} */
        this._deleteCallback = deleteCallback;
    }
    /**
     * @param {!NetworkRequest} req
     */
    addRequest(req) {
        this._requests.push(req);
        this._sync(this._requests.length - 1);
    }
    /**
     * @param {!ExtraRequestInfo} info
     */
    addRequestExtraInfo(info) {
        this._hasExtraInfo = true;
        this._requestExtraInfos.push(info);
        this._sync(this._requestExtraInfos.length - 1);
    }
    /**
     * @param {!ExtraResponseInfo} info
     */
    addResponseExtraInfo(info) {
        this._responseExtraInfos.push(info);
        this._sync(this._responseExtraInfos.length - 1);
    }
    finished() {
        this._finished = true;
        this._deleteIfComplete();
    }
    /**
     * @param {number} index
     */
    _sync(index) {
        const req = this._requests[index];
        if (!req) {
            return;
        }
        const requestExtraInfo = this._requestExtraInfos[index];
        if (requestExtraInfo) {
            req.addExtraRequestInfo(requestExtraInfo);
            this._requestExtraInfos[index] = null;
        }
        const responseExtraInfo = this._responseExtraInfos[index];
        if (responseExtraInfo) {
            req.addExtraResponseInfo(responseExtraInfo);
            this._responseExtraInfos[index] = null;
        }
        this._deleteIfComplete();
    }
    _deleteIfComplete() {
        if (!this._finished) {
            return;
        }
        if (this._hasExtraInfo) {
            // if we haven't gotten the last responseExtraInfo event, we have to wait for it.
            const lastItem = this._requests.peekLast();
            if (lastItem && !lastItem.hasExtraResponseInfo()) {
                return;
            }
        }
        this._deleteCallback();
    }
}
SDKModel.register(NetworkManager, Capability.Network, true);
/**
 * @typedef {{
  *   download: number,
  *   upload: number,
  *   latency: number,
  *   title: string,
  * }}
  */
// @ts-ignore typedef
let Conditions;
/** @typedef {{url: string, enabled: boolean}} */
// @ts-ignore typedef
let BlockedPattern;
/** @typedef {{message: string, requestId: string, warning: boolean}} */
// @ts-ignore typedef
let Message;
/** @typedef {!{urlPattern: string, interceptionStage: !Protocol.Network.InterceptionStage}} */
// @ts-ignore typedef
let InterceptionPattern;
/** @typedef {!function(!InterceptedRequest):!Promise<void>} */
// @ts-ignore typedef
let RequestInterceptor;
/** @typedef {!{url: string, frameId: string, loaderId: string, resourceType: Protocol.Network.ResourceType, mimeType: string, lastModified: ?Date}} */
// @ts-ignore typedef
let RequestUpdateDroppedEventData;

var NetworkManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    NetworkManager: NetworkManager,
    Events: Events$1,
    NoThrottlingConditions: NoThrottlingConditions,
    OfflineConditions: OfflineConditions,
    Slow3GConditions: Slow3GConditions,
    Fast3GConditions: Fast3GConditions,
    NetworkDispatcher: NetworkDispatcher,
    MultitargetNetworkManager: MultitargetNetworkManager,
    InterceptedRequest: InterceptedRequest,
    Conditions: Conditions,
    BlockedPattern: BlockedPattern,
    Message: Message,
    InterceptionPattern: InterceptionPattern,
    RequestInterceptor: RequestInterceptor,
    RequestUpdateDroppedEventData: RequestUpdateDroppedEventData
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class ServerTiming {
    /**
     * @param {string} metric
     * @param {?number} value
     * @param {?string} description
     */
    constructor(metric, value, description) {
        this.metric = metric;
        this.value = value;
        this.description = description;
    }
    /**
     * @param {!Array<!NameValue>} headers
     * @return {?Array<!ServerTiming>}
     */
    static parseHeaders(headers) {
        const rawServerTimingHeaders = headers.filter(item => item.name.toLowerCase() === 'server-timing');
        if (!rawServerTimingHeaders.length) {
            return null;
        }
        const serverTimings = rawServerTimingHeaders.reduce((memo, header) => {
            const timing = this.createFromHeaderValue(header.value);
            memo.push(...timing.map(function (entry) {
                return new ServerTiming(entry.name, entry.hasOwnProperty('dur') ? entry.dur : null, entry.hasOwnProperty('desc') ? entry.desc : '');
            }));
            return memo;
        }, /** @type {!Array<!ServerTiming>} */ ([]));
        serverTimings.sort((a, b) => a.metric.toLowerCase().compareTo(b.metric.toLowerCase()));
        return serverTimings;
    }
    /**
     * TODO(crbug.com/1011811): Instead of using !Object<string, *> we should have a proper type
     *                          with name, desc and dur properties.
     * @param {string} valueString
     * @return {!Array<!Object<string, *>>}
     */
    static createFromHeaderValue(valueString) {
        function trimLeadingWhiteSpace() {
            valueString = valueString.replace(/^\s*/, '');
        }
        /** @param {string} char */
        function consumeDelimiter(char) {
            console.assert(char.length === 1);
            trimLeadingWhiteSpace();
            if (valueString.charAt(0) !== char) {
                return false;
            }
            valueString = valueString.substring(1);
            return true;
        }
        function consumeToken() {
            // https://tools.ietf.org/html/rfc7230#appendix-B
            const result = /^(?:\s*)([\w!#$%&'*+\-.^`|~]+)(?:\s*)(.*)/.exec(valueString);
            if (!result) {
                return null;
            }
            valueString = result[2];
            return result[1];
        }
        function consumeTokenOrQuotedString() {
            trimLeadingWhiteSpace();
            if (valueString.charAt(0) === '"') {
                return consumeQuotedString();
            }
            return consumeToken();
        }
        function consumeQuotedString() {
            console.assert(valueString.charAt(0) === '"');
            valueString = valueString.substring(1); // remove leading DQUOTE
            let value = '';
            while (valueString.length) {
                // split into two parts:
                //  -everything before the first " or \
                //  -everything else
                const result = /^([^"\\]*)(.*)/.exec(valueString);
                if (!result) {
                    return null; // not a valid quoted-string
                }
                value += result[1];
                if (result[2].charAt(0) === '"') {
                    // we have found our closing "
                    valueString = result[2].substring(1); // strip off everything after the closing "
                    return value; // we are done here
                }
                console.assert(result[2].charAt(0) === '\\');
                // special rules for \ found in quoted-string (https://tools.ietf.org/html/rfc7230#section-3.2.6)
                value += result[2].charAt(1); // grab the character AFTER the \ (if there was one)
                valueString = result[2].substring(2); // strip off \ and next character
            }
            return null; // not a valid quoted-string
        }
        function consumeExtraneous() {
            const result = /([,;].*)/.exec(valueString);
            if (result) {
                valueString = result[1];
            }
        }
        const result = [];
        let name;
        while ((name = consumeToken()) !== null) {
            const entry = { name };
            if (valueString.charAt(0) === '=') {
                this.showWarning(ls$1 `Deprecated syntax found. Please use: <name>;dur=<duration>;desc=<description>`);
            }
            while (consumeDelimiter(';')) {
                let paramName;
                if ((paramName = consumeToken()) === null) {
                    continue;
                }
                paramName = paramName.toLowerCase();
                const parseParameter = this.getParserForParameter(paramName);
                let paramValue = null;
                if (consumeDelimiter('=')) {
                    // always parse the value, even if we don't recognize the parameter name
                    paramValue = consumeTokenOrQuotedString();
                    consumeExtraneous();
                }
                if (parseParameter) {
                    // paramName is valid
                    if (entry.hasOwnProperty(paramName)) {
                        this.showWarning(ls$1 `Duplicate parameter "${paramName}" ignored.`);
                        continue;
                    }
                    if (paramValue === null) {
                        this.showWarning(ls$1 `No value found for parameter "${paramName}".`);
                    }
                    parseParameter.call(this, entry, paramValue);
                }
                else {
                    // paramName is not valid
                    this.showWarning(ls$1 `Unrecognized parameter "${paramName}".`);
                }
            }
            result.push(entry);
            if (!consumeDelimiter(',')) {
                break;
            }
        }
        if (valueString.length) {
            this.showWarning(ls$1 `Extraneous trailing characters.`);
        }
        return result;
    }
    /**
     * @param {string} paramName
     * @return {?function(!Object<string, *>, ?string):void}
     */
    static getParserForParameter(paramName) {
        switch (paramName) {
            case 'dur': {
                /**
                 * @param {!Object<string, *>} entry
                 * @param {*} paramValue
                 */
                function durParser(entry, paramValue) {
                    entry.dur = 0;
                    if (paramValue !== null) {
                        const duration = parseFloat(paramValue);
                        if (isNaN(duration)) {
                            ServerTiming.showWarning(ls$1 `Unable to parse "${paramName}" value "${paramValue}".`);
                            return;
                        }
                        entry.dur = duration;
                    }
                }
                return durParser;
            }
            case 'desc': {
                /**
                 * @param {!Object<string, *>} entry
                 * @param {?string} paramValue
                 */
                function descParser(entry, paramValue) {
                    entry.desc = paramValue || '';
                }
                return descParser;
            }
            default: {
                return null;
            }
        }
    }
    /**
     * @param {string} msg
     */
    static showWarning(msg) {
        Console.Console.instance().warn(UIString.UIString(`ServerTiming: ${msg}`));
    }
}

var ServerTiming$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ServerTiming: ServerTiming
});

/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/** @enum {string} */
const MIME_TYPE = {
    HTML: 'text/html',
    XML: 'text/xml',
    PLAIN: 'text/plain',
    XHTML: 'application/xhtml+xml',
    SVG: 'image/svg+xml',
    CSS: 'text/css',
    XSL: 'text/xsl',
    VTT: 'text/vtt',
    PDF: 'application/pdf',
};
/** @type {!Map<!MIME_TYPE, *>} */
const MIME_TYPE_TO_RESOURCE_TYPE = new Map([
    [MIME_TYPE.HTML, { 'document': true }],
    [MIME_TYPE.XML, { 'document': true }],
    [MIME_TYPE.PLAIN, { 'document': true }],
    [MIME_TYPE.XHTML, { 'document': true }],
    [MIME_TYPE.SVG, { 'document': true }],
    [MIME_TYPE.CSS, { 'stylesheet': true }],
    [MIME_TYPE.XSL, { 'stylesheet': true }],
    [MIME_TYPE.VTT, { 'texttrack': true }],
    [MIME_TYPE.PDF, { 'document': true }],
]);
/**
 * @implements {TextUtils.ContentProvider.ContentProvider}
 * @unrestricted
 */
class NetworkRequest extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {!Protocol.Network.RequestId} requestId
     * @param {string} url
     * @param {string} documentURL
     * @param {!Protocol.Page.FrameId} frameId
     * @param {!Protocol.Network.LoaderId} loaderId
     * @param {?Protocol.Network.Initiator} initiator
     */
    constructor(requestId, url, documentURL, frameId, loaderId, initiator) {
        super();
        this._requestId = requestId;
        this._backendRequestId = requestId;
        this.setUrl(url);
        this._documentURL = documentURL;
        this._frameId = frameId;
        this._loaderId = loaderId;
        /** @type {?Protocol.Network.Initiator} */
        this._initiator = initiator;
        /** @type {?NetworkRequest} */
        this._redirectSource = null;
        /** @type {boolean} */
        this._isRedirect = false;
        /** @type {?NetworkRequest} */
        this._redirectDestination = null;
        this._issueTime = -1;
        this._startTime = -1;
        this._endTime = -1;
        /** @type {!Protocol.Network.BlockedReason|undefined} */
        this._blockedReason = undefined;
        this.statusCode = 0;
        this.statusText = '';
        this.requestMethod = '';
        this.requestTime = 0;
        /** @type {string} */
        this.protocol = '';
        /** @type {!Protocol.Security.MixedContentType} */
        this.mixedContentType = Protocol.Security.MixedContentType.None;
        /** @type {?Protocol.Network.ResourcePriority} */
        this._initialPriority = null;
        /** @type {?Protocol.Network.ResourcePriority} */
        this._currentPriority = null;
        /** @type {?Protocol.Network.SignedExchangeInfo} */
        this._signedExchangeInfo = null;
        /** @type {!Common.ResourceType.ResourceType} */
        this._resourceType = ResourceType.resourceTypes.Other;
        /** @type {?Promise<!ContentData>} */
        this._contentData = null;
        /** @type {!Array.<!WebSocketFrame>} */
        this._frames = [];
        /** @type {!Array.<!EventSourceMessage>} */
        this._eventSourceMessages = [];
        /** @type {!Object<string, (string|undefined)>} */
        this._responseHeaderValues = {};
        this._responseHeadersText = '';
        /** @type {!Array<!NameValue>} */
        this._requestHeaders = [];
        /** @type {!Object<string, (string|undefined)>} */
        this._requestHeaderValues = {};
        this._remoteAddress = '';
        /** @type {?Protocol.Network.RequestReferrerPolicy} */
        this._referrerPolicy = null;
        /** @type {!Protocol.Security.SecurityState} */
        this._securityState = Protocol.Security.SecurityState.Unknown;
        /** @type {?Protocol.Network.SecurityDetails} */
        this._securityDetails = null;
        /** @type {string} */
        this.connectionId = '0';
        /** @type {boolean} */
        this.connectionReused = false;
        /** @type {boolean} */
        this.hasNetworkData = false;
        /** @type {?Promise<?Array.<!NameValue>>} */
        this._formParametersPromise = null;
        // Assume no body initially
        /** @type {?Promise<?string>} */
        this._requestFormDataPromise = /** @type {?Promise<?string>} */ (Promise.resolve(null));
        /** @type {boolean} */
        this._hasExtraRequestInfo = false;
        /** @type {boolean} */
        this._hasExtraResponseInfo = false;
        /** @type {!Array<!BlockedCookieWithReason>} */
        this._blockedRequestCookies = [];
        /** @type {!Array<!Cookie>} */
        this._includedRequestCookies = [];
        /** @type {!Array<!BlockedSetCookieWithReason>} */
        this._blockedResponseCookies = [];
        /** @type {?string} */
        this.localizedFailDescription = null;
        /** @type {string} */
        this._url;
        /** @type {number} */
        this._responseReceivedTime;
        /** @type {number} */
        this._transferSize;
        /** @type {boolean} */
        this._finished;
        /** @type {boolean} */
        this._failed;
        /** @type {boolean} */
        this._canceled;
        /** @type {!MIME_TYPE} */
        this._mimeType;
        /** @type {!Common.ParsedURL.ParsedURL} */
        this._parsedURL;
        /** @type {(string|undefined)} */
        this._name;
        /** @type {(string|undefined)} */
        this._path;
    }
    /**
     * @param {!NetworkRequest} other
     * @return {number}
     */
    identityCompare(other) {
        const thisId = this.requestId();
        const thatId = other.requestId();
        if (thisId > thatId) {
            return 1;
        }
        if (thisId < thatId) {
            return -1;
        }
        return 0;
    }
    /**
     * @return {!Protocol.Network.RequestId}
     */
    requestId() {
        return this._requestId;
    }
    /**
     * @return {!Protocol.Network.RequestId}
     */
    backendRequestId() {
        return this._backendRequestId;
    }
    /**
     * @return {string}
     */
    url() {
        return this._url;
    }
    /**
     * @return {boolean}
     */
    isBlobRequest() {
        return this._url.startsWith('blob:');
    }
    /**
     * @param {string} x
     */
    setUrl(x) {
        if (this._url === x) {
            return;
        }
        this._url = x;
        this._parsedURL = new ParsedURL.ParsedURL(x);
        delete this._queryString;
        delete this._parsedQueryParameters;
        delete this._name;
        delete this._path;
    }
    /**
     * @return {string}
     */
    get documentURL() {
        return this._documentURL;
    }
    get parsedURL() {
        return this._parsedURL;
    }
    /**
     * @return {!Protocol.Page.FrameId}
     */
    get frameId() {
        return this._frameId;
    }
    /**
     * @return {!Protocol.Network.LoaderId}
     */
    get loaderId() {
        return this._loaderId;
    }
    /**
     * @param {string} ip
     * @param {number} port
     */
    setRemoteAddress(ip, port) {
        this._remoteAddress = ip + ':' + port;
        this.dispatchEventToListeners(Events$2.RemoteAddressChanged, this);
    }
    /**
     * @return {string}
     */
    remoteAddress() {
        return this._remoteAddress;
    }
    /**
     * @return {string | undefined}
     * The cache name of the CacheStorage from where the response is served via
     * the ServiceWorker.
     */
    getResponseCacheStorageCacheName() {
        return this._responseCacheStorageCacheName;
    }
    /**
     * @param {string} x
     */
    setResponseCacheStorageCacheName(x) {
        this._responseCacheStorageCacheName = x;
    }
    /**
     * @return {!Protocol.Network.ServiceWorkerResponseSource | undefined}
     */
    serviceWorkerResponseSource() {
        return this._serviceWorkerResponseSource;
    }
    /**
     * @param {!Protocol.Network.ServiceWorkerResponseSource} serviceWorkerResponseSource
     */
    setServiceWorkerResponseSource(serviceWorkerResponseSource) {
        this._serviceWorkerResponseSource = serviceWorkerResponseSource;
    }
    /**
     * @param {!Protocol.Network.RequestReferrerPolicy} referrerPolicy
     */
    setReferrerPolicy(referrerPolicy) {
        this._referrerPolicy = referrerPolicy;
    }
    /**
     * @return {?Protocol.Network.RequestReferrerPolicy}
     */
    referrerPolicy() {
        return this._referrerPolicy;
    }
    /**
     * @return {!Protocol.Security.SecurityState}
     */
    securityState() {
        return this._securityState;
    }
    /**
     * @param {!Protocol.Security.SecurityState} securityState
     */
    setSecurityState(securityState) {
        this._securityState = securityState;
    }
    /**
     * @return {?Protocol.Network.SecurityDetails}
     */
    securityDetails() {
        return this._securityDetails;
    }
    /**
     * @return {string}
     */
    securityOrigin() {
        return this._parsedURL.securityOrigin();
    }
    /**
     * @param {!Protocol.Network.SecurityDetails} securityDetails
     */
    setSecurityDetails(securityDetails) {
        this._securityDetails = securityDetails;
    }
    /**
     * @return {number}
     */
    get startTime() {
        return this._startTime || -1;
    }
    /**
     * @param {number} monotonicTime
     * @param {number} wallTime
     */
    setIssueTime(monotonicTime, wallTime) {
        this._issueTime = monotonicTime;
        this._wallIssueTime = wallTime;
        this._startTime = monotonicTime;
    }
    /**
     * @return {number}
     */
    issueTime() {
        return this._issueTime;
    }
    /**
     * @param {number} monotonicTime
     * @return {number}
     */
    pseudoWallTime(monotonicTime) {
        return this._wallIssueTime ? this._wallIssueTime - this._issueTime + monotonicTime : monotonicTime;
    }
    /**
     * @return {number}
     */
    get responseReceivedTime() {
        return this._responseReceivedTime || -1;
    }
    /**
     * @param {number} x
     */
    set responseReceivedTime(x) {
        this._responseReceivedTime = x;
    }
    /**
     * @return {!Date | undefined}
     * The time at which the returned response was generated. For cached
     * responses, this is the last time the cache entry was validated.
     */
    getResponseRetrievalTime() {
        return this._responseRetrievalTime;
    }
    /**
     * @param {!Date} x
     */
    setResponseRetrievalTime(x) {
        this._responseRetrievalTime = x;
    }
    /**
     * @return {number}
     */
    get endTime() {
        return this._endTime || -1;
    }
    /**
     * @param {number} x
     */
    set endTime(x) {
        if (this.timing && this.timing.requestTime) {
            // Check against accurate responseReceivedTime.
            this._endTime = Math.max(x, this.responseReceivedTime);
        }
        else {
            // Prefer endTime since it might be from the network stack.
            this._endTime = x;
            if (this._responseReceivedTime > x) {
                this._responseReceivedTime = x;
            }
        }
        this.dispatchEventToListeners(Events$2.TimingChanged, this);
    }
    /**
     * @return {number}
     */
    get duration() {
        if (this._endTime === -1 || this._startTime === -1) {
            return -1;
        }
        return this._endTime - this._startTime;
    }
    /**
     * @return {number}
     */
    get latency() {
        if (this._responseReceivedTime === -1 || this._startTime === -1) {
            return -1;
        }
        return this._responseReceivedTime - this._startTime;
    }
    /**
     * @return {number}
     */
    get resourceSize() {
        return this._resourceSize || 0;
    }
    /**
     * @param {number} x
     */
    set resourceSize(x) {
        this._resourceSize = x;
    }
    /**
     * @return {number}
     */
    get transferSize() {
        return this._transferSize || 0;
    }
    /**
     * @param {number} x
     */
    increaseTransferSize(x) {
        this._transferSize = (this._transferSize || 0) + x;
    }
    /**
     * @param {number} x
     */
    setTransferSize(x) {
        this._transferSize = x;
    }
    /**
     * @return {boolean}
     */
    get finished() {
        return this._finished;
    }
    /**
     * @param {boolean} x
     */
    set finished(x) {
        if (this._finished === x) {
            return;
        }
        this._finished = x;
        if (x) {
            this.dispatchEventToListeners(Events$2.FinishedLoading, this);
        }
    }
    /**
     * @return {boolean}
     */
    get failed() {
        return this._failed;
    }
    /**
     * @param {boolean} x
     */
    set failed(x) {
        this._failed = x;
    }
    /**
     * @return {boolean}
     */
    get canceled() {
        return this._canceled;
    }
    /**
     * @param {boolean} x
     */
    set canceled(x) {
        this._canceled = x;
    }
    /**
     * @return {!Protocol.Network.BlockedReason|undefined}
     */
    blockedReason() {
        return this._blockedReason;
    }
    /**
     * @param {!Protocol.Network.BlockedReason} reason
     */
    setBlockedReason(reason) {
        this._blockedReason = reason;
    }
    /**
     * @return {boolean}
     */
    wasBlocked() {
        return !!this._blockedReason;
    }
    /**
     * @return {boolean}
     */
    cached() {
        return (!!this._fromMemoryCache || !!this._fromDiskCache) && !this._transferSize;
    }
    /**
     * @return {boolean}
     */
    cachedInMemory() {
        return !!this._fromMemoryCache && !this._transferSize;
    }
    /**
     * @return {boolean}
     */
    fromPrefetchCache() {
        return !!this._fromPrefetchCache;
    }
    setFromMemoryCache() {
        this._fromMemoryCache = true;
        delete this._timing;
    }
    setFromDiskCache() {
        this._fromDiskCache = true;
    }
    setFromPrefetchCache() {
        this._fromPrefetchCache = true;
    }
    /**
     * Returns true if the request was intercepted by a service worker and it
     * provided its own response.
     * @return {boolean}
     */
    get fetchedViaServiceWorker() {
        return !!this._fetchedViaServiceWorker;
    }
    /**
     * @param {boolean} x
     */
    set fetchedViaServiceWorker(x) {
        this._fetchedViaServiceWorker = x;
    }
    /**
     * Returns true if the request was sent by a service worker.
     * @return {boolean}
     */
    initiatedByServiceWorker() {
        const networkManager = NetworkManager.forRequest(this);
        if (!networkManager) {
            return false;
        }
        return networkManager.target().type() === Type.ServiceWorker;
    }
    /**
     * @return {!Protocol.Network.ResourceTiming|undefined}
     */
    get timing() {
        return this._timing;
    }
    /**
     * @param {!Protocol.Network.ResourceTiming|undefined} timingInfo
     */
    set timing(timingInfo) {
        if (!timingInfo || this._fromMemoryCache) {
            return;
        }
        // Take startTime and responseReceivedTime from timing data for better accuracy.
        // Timing's requestTime is a baseline in seconds, rest of the numbers there are ticks in millis.
        this._startTime = timingInfo.requestTime;
        const headersReceivedTime = timingInfo.requestTime + timingInfo.receiveHeadersEnd / 1000.0;
        if ((this._responseReceivedTime || -1) < 0 || this._responseReceivedTime > headersReceivedTime) {
            this._responseReceivedTime = headersReceivedTime;
        }
        if (this._startTime > this._responseReceivedTime) {
            this._responseReceivedTime = this._startTime;
        }
        this._timing = timingInfo;
        this.dispatchEventToListeners(Events$2.TimingChanged, this);
    }
    /**
     * @return {!MIME_TYPE}
     */
    get mimeType() {
        return this._mimeType;
    }
    /**
     * @param {!MIME_TYPE} x
     */
    set mimeType(x) {
        this._mimeType = x;
    }
    /**
     * @return {string}
     */
    get displayName() {
        return this._parsedURL.displayName;
    }
    /**
     * @return {string}
     */
    name() {
        if (this._name) {
            return this._name;
        }
        this._parseNameAndPathFromURL();
        return /** @type {string} */ (this._name);
    }
    /**
     * @return {string}
     */
    path() {
        if (this._path) {
            return this._path;
        }
        this._parseNameAndPathFromURL();
        return /** @type {string} */ (this._path);
    }
    _parseNameAndPathFromURL() {
        if (this._parsedURL.isDataURL()) {
            this._name = this._parsedURL.dataURLDisplayName();
            this._path = '';
        }
        else if (this._parsedURL.isBlobURL()) {
            this._name = this._parsedURL.url;
            this._path = '';
        }
        else if (this._parsedURL.isAboutBlank()) {
            this._name = this._parsedURL.url;
            this._path = '';
        }
        else {
            this._path = this._parsedURL.host + this._parsedURL.folderPathComponents;
            const networkManager = NetworkManager.forRequest(this);
            const inspectedURL = networkManager ? ParsedURL.ParsedURL.fromString(networkManager.target().inspectedURL()) : null;
            this._path = StringUtilities.trimURL(this._path, inspectedURL ? inspectedURL.host : '');
            if (this._parsedURL.lastPathComponent || this._parsedURL.queryParams) {
                this._name =
                    this._parsedURL.lastPathComponent + (this._parsedURL.queryParams ? '?' + this._parsedURL.queryParams : '');
            }
            else if (this._parsedURL.folderPathComponents) {
                this._name =
                    this._parsedURL.folderPathComponents.substring(this._parsedURL.folderPathComponents.lastIndexOf('/') + 1) +
                        '/';
                this._path = this._path.substring(0, this._path.lastIndexOf('/'));
            }
            else {
                this._name = this._parsedURL.host;
                this._path = '';
            }
        }
    }
    /**
     * @return {string}
     */
    get folder() {
        let path = this._parsedURL.path;
        const indexOfQuery = path.indexOf('?');
        if (indexOfQuery !== -1) {
            path = path.substring(0, indexOfQuery);
        }
        const lastSlashIndex = path.lastIndexOf('/');
        return lastSlashIndex !== -1 ? path.substring(0, lastSlashIndex) : '';
    }
    /**
     * @return {string}
     */
    get pathname() {
        return this._parsedURL.path;
    }
    /**
     * @return {!Common.ResourceType.ResourceType}
     */
    resourceType() {
        return this._resourceType;
    }
    /**
     * @param {!Common.ResourceType.ResourceType} resourceType
     */
    setResourceType(resourceType) {
        this._resourceType = resourceType;
    }
    /**
     * @return {string}
     */
    get domain() {
        return this._parsedURL.host;
    }
    /**
     * @return {string}
     */
    get scheme() {
        return this._parsedURL.scheme;
    }
    /**
     * @return {?NetworkRequest}
     */
    redirectSource() {
        return this._redirectSource;
    }
    /**
     * @param {?NetworkRequest} originatingRequest
     */
    setRedirectSource(originatingRequest) {
        this._redirectSource = originatingRequest;
    }
    /**
     * @return {?NetworkRequest}
     */
    redirectDestination() {
        return this._redirectDestination;
    }
    /**
     * @param {?NetworkRequest} redirectDestination
     */
    setRedirectDestination(redirectDestination) {
        this._redirectDestination = redirectDestination;
    }
    /**
     * @return {!Array.<!NameValue>}
     */
    requestHeaders() {
        return this._requestHeaders;
    }
    /**
     * @param {!Array.<!NameValue>} headers
     */
    setRequestHeaders(headers) {
        this._requestHeaders = headers;
        delete this._requestCookies;
        this.dispatchEventToListeners(Events$2.RequestHeadersChanged);
    }
    /**
     * @return {string|undefined}
     */
    requestHeadersText() {
        return this._requestHeadersText;
    }
    /**
     * @param {string} text
     */
    setRequestHeadersText(text) {
        this._requestHeadersText = text;
        this.dispatchEventToListeners(Events$2.RequestHeadersChanged);
    }
    /**
     * @param {string} headerName
     * @return {string|undefined}
     */
    requestHeaderValue(headerName) {
        if (this._requestHeaderValues[headerName]) {
            return this._requestHeaderValues[headerName];
        }
        this._requestHeaderValues[headerName] = this._computeHeaderValue(this.requestHeaders(), headerName);
        return this._requestHeaderValues[headerName];
    }
    /**
     * @return {!Array.<!Cookie>}
     */
    get requestCookies() {
        if (!this._requestCookies) {
            this._requestCookies = CookieParser.parseCookie(this.requestHeaderValue('Cookie')) || [];
        }
        return this._requestCookies;
    }
    /**
     * @return {!Promise<?string>}
     */
    requestFormData() {
        if (!this._requestFormDataPromise) {
            this._requestFormDataPromise = NetworkManager.requestPostData(this);
        }
        return this._requestFormDataPromise;
    }
    /**
     * @param {boolean} hasData
     * @param {?string} data
     */
    setRequestFormData(hasData, data) {
        this._requestFormDataPromise = (hasData && data === null) ? null : Promise.resolve(data);
        this._formParametersPromise = null;
    }
    /**
     * @return {string}
     */
    _filteredProtocolName() {
        const protocol = this.protocol.toLowerCase();
        if (protocol === 'h2') {
            return 'http/2.0';
        }
        return protocol.replace(/^http\/2(\.0)?\+/, 'http/2.0+');
    }
    /**
     * @return {string}
     */
    requestHttpVersion() {
        const headersText = this.requestHeadersText();
        if (!headersText) {
            const version = this.requestHeaderValue('version') || this.requestHeaderValue(':version');
            if (version) {
                return version;
            }
            return this._filteredProtocolName();
        }
        const firstLine = headersText.split(/\r\n/)[0];
        const match = firstLine.match(/(HTTP\/\d+\.\d+)$/);
        return match ? match[1] : 'HTTP/0.9';
    }
    /**
     * @return {!Array.<!NameValue>}
     */
    get responseHeaders() {
        return this._responseHeaders || [];
    }
    /**
     * @param {!Array.<!NameValue>} x
     */
    set responseHeaders(x) {
        this._responseHeaders = x;
        delete this._sortedResponseHeaders;
        delete this._serverTimings;
        delete this._responseCookies;
        this._responseHeaderValues = {};
        this.dispatchEventToListeners(Events$2.ResponseHeadersChanged);
    }
    /**
     * @return {string}
     */
    get responseHeadersText() {
        return this._responseHeadersText;
    }
    /**
     * @param {string} x
     */
    set responseHeadersText(x) {
        this._responseHeadersText = x;
        this.dispatchEventToListeners(Events$2.ResponseHeadersChanged);
    }
    /**
     * @return {!Array.<!NameValue>}
     */
    get sortedResponseHeaders() {
        if (this._sortedResponseHeaders !== undefined) {
            return this._sortedResponseHeaders;
        }
        this._sortedResponseHeaders = this.responseHeaders.slice();
        this._sortedResponseHeaders.sort(function (a, b) {
            return a.name.toLowerCase().compareTo(b.name.toLowerCase());
        });
        return this._sortedResponseHeaders;
    }
    /**
     * @param {string} headerName
     * @return {string|undefined}
     */
    responseHeaderValue(headerName) {
        if (headerName in this._responseHeaderValues) {
            return this._responseHeaderValues[headerName];
        }
        this._responseHeaderValues[headerName] = this._computeHeaderValue(this.responseHeaders, headerName);
        return this._responseHeaderValues[headerName];
    }
    /**
     * @return {!Array.<!Cookie>}
     */
    get responseCookies() {
        if (!this._responseCookies) {
            this._responseCookies = CookieParser.parseSetCookie(this.responseHeaderValue('Set-Cookie'), this.domain) || [];
        }
        return this._responseCookies;
    }
    /**
     * @return {string|undefined}
     */
    responseLastModified() {
        return this.responseHeaderValue('last-modified');
    }
    /**
     * @return {!Array.<!Cookie>}
     */
    allCookiesIncludingBlockedOnes() {
        return /** @type {!Array.<!Cookie>} */ ([
            ...this.includedRequestCookies(), ...this.responseCookies,
            ...this.blockedRequestCookies().map(blockedRequestCookie => blockedRequestCookie.cookie),
            ...this.blockedResponseCookies().map(blockedResponseCookie => blockedResponseCookie.cookie),
        ].filter(v => !!v));
    }
    /**
     * @return {?Array.<!ServerTiming>}
     */
    get serverTimings() {
        if (typeof this._serverTimings === 'undefined') {
            this._serverTimings = ServerTiming.parseHeaders(this.responseHeaders);
        }
        return this._serverTimings;
    }
    /**
     * @return {?string}
     */
    queryString() {
        if (this._queryString !== undefined) {
            return this._queryString;
        }
        let queryString = null;
        const url = this.url();
        const questionMarkPosition = url.indexOf('?');
        if (questionMarkPosition !== -1) {
            queryString = url.substring(questionMarkPosition + 1);
            const hashSignPosition = queryString.indexOf('#');
            if (hashSignPosition !== -1) {
                queryString = queryString.substring(0, hashSignPosition);
            }
        }
        this._queryString = queryString;
        return this._queryString;
    }
    /**
     * @return {?Array.<!NameValue>}
     */
    get queryParameters() {
        if (this._parsedQueryParameters) {
            return this._parsedQueryParameters;
        }
        const queryString = this.queryString();
        if (!queryString) {
            return null;
        }
        this._parsedQueryParameters = this._parseParameters(queryString);
        return this._parsedQueryParameters;
    }
    /**
     * @return {!Promise<?Array<!NameValue>>}
     */
    async _parseFormParameters() {
        const requestContentType = this.requestContentType();
        if (!requestContentType) {
            return null;
        }
        // Handling application/x-www-form-urlencoded request bodies.
        if (requestContentType.match(/^application\/x-www-form-urlencoded\s*(;.*)?$/i)) {
            const formData = await this.requestFormData();
            if (!formData) {
                return null;
            }
            return this._parseParameters(formData);
        }
        // Handling multipart/form-data request bodies.
        const multipartDetails = requestContentType.match(/^multipart\/form-data\s*;\s*boundary\s*=\s*(\S+)\s*$/);
        if (!multipartDetails) {
            return null;
        }
        const boundary = multipartDetails[1];
        if (!boundary) {
            return null;
        }
        const formData = await this.requestFormData();
        if (!formData) {
            return null;
        }
        return this._parseMultipartFormDataParameters(formData, boundary);
    }
    /**
     * @return {!Promise<?Array<!NameValue>>}
     */
    formParameters() {
        if (!this._formParametersPromise) {
            this._formParametersPromise = this._parseFormParameters();
        }
        return this._formParametersPromise;
    }
    /**
     * @return {string}
     */
    responseHttpVersion() {
        const headersText = this._responseHeadersText;
        if (!headersText) {
            const version = this.responseHeaderValue('version') || this.responseHeaderValue(':version');
            if (version) {
                return version;
            }
            return this._filteredProtocolName();
        }
        const firstLine = headersText.split(/\r\n/)[0];
        const match = firstLine.match(/^(HTTP\/\d+\.\d+)/);
        return match ? match[1] : 'HTTP/0.9';
    }
    /**
     * @param {string} queryString
     * @return {!Array.<!NameValue>}
     */
    _parseParameters(queryString) {
        /**
         * @param {string} pair
         */
        function parseNameValue(pair) {
            const position = pair.indexOf('=');
            if (position === -1) {
                return { name: pair, value: '' };
            }
            return { name: pair.substring(0, position), value: pair.substring(position + 1) };
        }
        return queryString.split('&').map(parseNameValue);
    }
    /**
     * Parses multipart/form-data; boundary=boundaryString request bodies -
     * --boundaryString
     * Content-Disposition: form-data; name="field-name"; filename="r.gif"
     * Content-Type: application/octet-stream
     *
     * optionalValue
     * --boundaryString
     * Content-Disposition: form-data; name="field-name-2"
     *
     * optionalValue2
     * --boundaryString--
     *
     * @param {string} data
     * @param {string} boundary
     * @return {!Array.<!NameValue>}
     */
    _parseMultipartFormDataParameters(data, boundary) {
        const sanitizedBoundary = boundary.escapeForRegExp();
        const keyValuePattern = new RegExp(
        // Header with an optional file name.
        '^\\r\\ncontent-disposition\\s*:\\s*form-data\\s*;\\s*name="([^"]*)"(?:\\s*;\\s*filename="([^"]*)")?' +
            // Optional secondary header with the content type.
            '(?:\\r\\ncontent-type\\s*:\\s*([^\\r\\n]*))?' +
            // Padding.
            '\\r\\n\\r\\n' +
            // Value
            '(.*)' +
            // Padding.
            '\\r\\n$', 'is');
        const fields = data.split(new RegExp(`--${sanitizedBoundary}(?:--\s*$)?`, 'g'));
        return fields.reduce(parseMultipartField, []);
        /**
         * @param {!Array.<!NameValue>} result
         * @param {string} field
         * @return {!Array.<!NameValue>}
         */
        function parseMultipartField(result, field) {
            const [match, name, filename, contentType, value] = field.match(keyValuePattern) || [];
            if (!match) {
                return result;
            }
            const processedValue = (filename || contentType) ? ls `(binary)` : value;
            result.push({ name, value: processedValue });
            return result;
        }
    }
    /**
     * @param {!Array.<!NameValue>} headers
     * @param {string} headerName
     * @return {string|undefined}
     */
    _computeHeaderValue(headers, headerName) {
        headerName = headerName.toLowerCase();
        const values = [];
        for (let i = 0; i < headers.length; ++i) {
            if (headers[i].name.toLowerCase() === headerName) {
                values.push(headers[i].value);
            }
        }
        if (!values.length) {
            return undefined;
        }
        // Set-Cookie values should be separated by '\n', not comma, otherwise cookies could not be parsed.
        if (headerName === 'set-cookie') {
            return values.join('\n');
        }
        return values.join(', ');
    }
    /**
     * @return {!Promise<!ContentData>}
     */
    contentData() {
        if (this._contentData) {
            return this._contentData;
        }
        if (this._contentDataProvider) {
            this._contentData = this._contentDataProvider();
        }
        else {
            this._contentData = NetworkManager.requestContentData(this);
        }
        return this._contentData;
    }
    /**
     * @param {function():!Promise<!ContentData>} dataProvider
     */
    setContentDataProvider(dataProvider) {
        console.assert(!this._contentData, 'contentData can only be set once.');
        this._contentDataProvider = dataProvider;
    }
    /**
     * @override
     * @return {string}
     */
    contentURL() {
        return this._url;
    }
    /**
     * @override
     * @return {!Common.ResourceType.ResourceType}
     */
    contentType() {
        return this._resourceType;
    }
    /**
     * @override
     * @return {!Promise<boolean>}
     */
    async contentEncoded() {
        return (await this.contentData()).encoded;
    }
    /**
     * @override
     * @return {!Promise<!TextUtils.ContentProvider.DeferredContent>}
     */
    async requestContent() {
        const { content, error, encoded } = await this.contentData();
        return /** @type{!TextUtils.ContentProvider.DeferredContent} */ ({
            content,
            error,
            isEncoded: encoded,
        });
    }
    /**
     * @override
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    async searchInContent(query, caseSensitive, isRegex) {
        if (!this._contentDataProvider) {
            return NetworkManager.searchInRequest(this, query, caseSensitive, isRegex);
        }
        const contentData = await this.contentData();
        let content = contentData.content;
        if (!content) {
            return [];
        }
        if (contentData.encoded) {
            content = window.atob(content);
        }
        return TextUtils.performSearchInContent(content, query, caseSensitive, isRegex);
    }
    /**
     * @return {boolean}
     */
    isHttpFamily() {
        return !!this.url().match(/^https?:/i);
    }
    /**
     * @return {string|undefined}
     */
    requestContentType() {
        return this.requestHeaderValue('Content-Type');
    }
    /**
     * @return {boolean}
     */
    hasErrorStatusCode() {
        return this.statusCode >= 400;
    }
    /**
     * @param {!Protocol.Network.ResourcePriority} priority
     */
    setInitialPriority(priority) {
        this._initialPriority = priority;
    }
    /**
     * @return {?Protocol.Network.ResourcePriority}
     */
    initialPriority() {
        return this._initialPriority;
    }
    /**
     * @param {!Protocol.Network.ResourcePriority} priority
     */
    setPriority(priority) {
        this._currentPriority = priority;
    }
    /**
     * @return {?Protocol.Network.ResourcePriority}
     */
    priority() {
        return this._currentPriority || this._initialPriority || null;
    }
    /**
     * @param {!Protocol.Network.SignedExchangeInfo} info
     */
    setSignedExchangeInfo(info) {
        this._signedExchangeInfo = info;
    }
    /**
     * @return {?Protocol.Network.SignedExchangeInfo}
     */
    signedExchangeInfo() {
        return this._signedExchangeInfo;
    }
    /**
     * @param {!HTMLImageElement} image
     */
    async populateImageSource(image) {
        const { content, encoded } = await this.contentData();
        let imageSrc = ContentProvider.contentAsDataURL(content, this._mimeType, encoded);
        if (imageSrc === null && !this._failed) {
            const cacheControl = this.responseHeaderValue('cache-control') || '';
            if (!cacheControl.includes('no-cache')) {
                imageSrc = this._url;
            }
        }
        if (imageSrc !== null) {
            image.src = imageSrc;
        }
    }
    /**
     * @return {?Protocol.Network.Initiator}
     */
    initiator() {
        return this._initiator;
    }
    /**
     * @return {!Array.<!WebSocketFrame>}
     */
    frames() {
        return this._frames;
    }
    /**
     * @param {string} errorMessage
     * @param {number} time
     */
    addProtocolFrameError(errorMessage, time) {
        this.addFrame({ type: WebSocketFrameType.Error, text: errorMessage, time: this.pseudoWallTime(time), opCode: -1, mask: false });
    }
    /**
     * @param {!Protocol.Network.WebSocketFrame} response
     * @param {number} time
     * @param {boolean} sent
     */
    addProtocolFrame(response, time, sent) {
        const type = sent ? WebSocketFrameType.Send : WebSocketFrameType.Receive;
        this.addFrame({
            type: type,
            text: response.payloadData,
            time: this.pseudoWallTime(time),
            opCode: response.opcode,
            mask: response.mask
        });
    }
    /**
     * @param {!WebSocketFrame} frame
     */
    addFrame(frame) {
        this._frames.push(frame);
        this.dispatchEventToListeners(Events$2.WebsocketFrameAdded, frame);
    }
    /**
     * @return {!Array.<!EventSourceMessage>}
     */
    eventSourceMessages() {
        return this._eventSourceMessages;
    }
    /**
     * @param {number} time
     * @param {string} eventName
     * @param {string} eventId
     * @param {string} data
     */
    addEventSourceMessage(time, eventName, eventId, data) {
        const message = { time: this.pseudoWallTime(time), eventName: eventName, eventId: eventId, data: data };
        this._eventSourceMessages.push(message);
        this.dispatchEventToListeners(Events$2.EventSourceMessageAdded, message);
    }
    /**
     * @param {number} redirectCount
     */
    markAsRedirect(redirectCount) {
        this._isRedirect = true;
        this._requestId = `${this._backendRequestId}:redirected.${redirectCount}`;
    }
    /**
     * @return {boolean}
     */
    isRedirect() {
        return this._isRedirect;
    }
    /**
     * @param {string} requestId
     */
    setRequestIdForTest(requestId) {
        this._backendRequestId = requestId;
        this._requestId = requestId;
    }
    /**
     * @return {?string}
     */
    charset() {
        const contentTypeHeader = this.responseHeaderValue('content-type');
        if (!contentTypeHeader) {
            return null;
        }
        const responseCharsets = contentTypeHeader.replace(/ /g, '')
            .split(';')
            .filter(parameter => parameter.toLowerCase().startsWith('charset='))
            .map(parameter => parameter.slice('charset='.length));
        if (responseCharsets.length) {
            return responseCharsets[0];
        }
        return null;
    }
    /**
     * @param {!ExtraRequestInfo} extraRequestInfo
     */
    addExtraRequestInfo(extraRequestInfo) {
        this._blockedRequestCookies = extraRequestInfo.blockedRequestCookies;
        this._includedRequestCookies = extraRequestInfo.includedRequestCookies;
        this.setRequestHeaders(extraRequestInfo.requestHeaders);
        this._hasExtraRequestInfo = true;
        this.setRequestHeadersText(''); // Mark request headers as non-provisional
    }
    /**
     * @return {boolean}
     */
    hasExtraRequestInfo() {
        return this._hasExtraRequestInfo;
    }
    /**
     * @return {!Array<!BlockedCookieWithReason>}
     */
    blockedRequestCookies() {
        return this._blockedRequestCookies;
    }
    /**
     * @return {!Array<!Cookie>}
     */
    includedRequestCookies() {
        return this._includedRequestCookies;
    }
    /**
     * @return {boolean}
     */
    hasRequestCookies() {
        return this._includedRequestCookies.length > 0 || this._blockedRequestCookies.length > 0;
    }
    /**
     * @param {!ExtraResponseInfo} extraResponseInfo
     */
    addExtraResponseInfo(extraResponseInfo) {
        this._blockedResponseCookies = extraResponseInfo.blockedResponseCookies;
        this.responseHeaders = extraResponseInfo.responseHeaders;
        if (extraResponseInfo.responseHeadersText) {
            this.responseHeadersText = extraResponseInfo.responseHeadersText;
            if (!this.requestHeadersText()) {
                // Generate request headers text from raw headers in extra request info because
                // Network.requestWillBeSentExtraInfo doesn't include headers text.
                let requestHeadersText = `${this.requestMethod} ${this.parsedURL.path}`;
                if (this.parsedURL.queryParams) {
                    requestHeadersText += `?${this.parsedURL.queryParams}`;
                }
                requestHeadersText += ' HTTP/1.1\r\n';
                for (const { name, value } of this.requestHeaders()) {
                    requestHeadersText += `${name}: ${value}\r\n`;
                }
                this.setRequestHeadersText(requestHeadersText);
            }
        }
        this._hasExtraResponseInfo = true;
    }
    /**
     * @return {boolean}
     */
    hasExtraResponseInfo() {
        return this._hasExtraResponseInfo;
    }
    /**
     * @return {!Array<!BlockedSetCookieWithReason>}
     */
    blockedResponseCookies() {
        return this._blockedResponseCookies;
    }
}
/** @enum {symbol} */
const Events$2 = {
    FinishedLoading: Symbol('FinishedLoading'),
    TimingChanged: Symbol('TimingChanged'),
    RemoteAddressChanged: Symbol('RemoteAddressChanged'),
    RequestHeadersChanged: Symbol('RequestHeadersChanged'),
    ResponseHeadersChanged: Symbol('ResponseHeadersChanged'),
    WebsocketFrameAdded: Symbol('WebsocketFrameAdded'),
    EventSourceMessageAdded: Symbol('EventSourceMessageAdded')
};
/** @enum {string} */
const InitiatorType = {
    Other: 'other',
    Parser: 'parser',
    Redirect: 'redirect',
    Script: 'script',
    Preload: 'preload',
    SignedExchange: 'signedExchange'
};
/** @enum {string} */
const WebSocketFrameType = {
    Send: 'send',
    Receive: 'receive',
    Error: 'error'
};
/**
 * @param {!Protocol.Network.CookieBlockedReason} blockedReason
 * @return {string}
 */
const cookieBlockedReasonToUiString = function (blockedReason) {
    switch (blockedReason) {
        case Protocol.Network.CookieBlockedReason.SecureOnly:
            return ls `This cookie was blocked because it had the "Secure" attribute and the connection was not secure.`;
        case Protocol.Network.CookieBlockedReason.NotOnPath:
            return ls `This cookie was blocked because its path was not an exact match for or a superdirectory of the request url's path.`;
        case Protocol.Network.CookieBlockedReason.DomainMismatch:
            return ls `This cookie was blocked because neither did the request URL's domain exactly match the cookie's domain, nor was the request URL's domain a subdomain of the cookie's Domain attribute value.`;
        case Protocol.Network.CookieBlockedReason.SameSiteStrict:
            return ls `This cookie was blocked because it had the "SameSite=Strict" attribute and the request was made from a different site. This includes top-level navigation requests initiated by other sites.`;
        case Protocol.Network.CookieBlockedReason.SameSiteLax:
            return ls `This cookie was blocked because it had the "SameSite=Lax" attribute and the request was made from a different site and was not initiated by a top-level navigation.`;
        case Protocol.Network.CookieBlockedReason.SameSiteUnspecifiedTreatedAsLax:
            return ls `This cookie didn't specify a "SameSite" attribute when it was stored and was defaulted to "SameSite=Lax," and was blocked because the request was made from a different site and was not initiated by a top-level navigation. The cookie had to have been set with "SameSite=None" to enable cross-site usage.`;
        case Protocol.Network.CookieBlockedReason.SameSiteNoneInsecure:
            return ls `This cookie was blocked because it had the "SameSite=None" attribute but was not marked "Secure". Cookies without SameSite restrictions must be marked "Secure" and sent over a secure connection.`;
        case Protocol.Network.CookieBlockedReason.UserPreferences:
            return ls `This cookie was blocked due to user preferences.`;
        case Protocol.Network.CookieBlockedReason.UnknownError:
            return ls `An unknown error was encountered when trying to send this cookie.`;
    }
    return '';
};
/**
 * @param {!Protocol.Network.SetCookieBlockedReason} blockedReason
 * @return {string}
 */
const setCookieBlockedReasonToUiString = function (blockedReason) {
    switch (blockedReason) {
        case Protocol.Network.SetCookieBlockedReason.SecureOnly:
            return ls `This Set-Cookie was blocked because it had the "Secure" attribute but was not received over a secure connection.`;
        case Protocol.Network.SetCookieBlockedReason.SameSiteStrict:
            return ls `This Set-Cookie was blocked because it had the "SameSite=Strict" attribute but came from a cross-site response which was not the response to a top-level navigation.`;
        case Protocol.Network.SetCookieBlockedReason.SameSiteLax:
            return ls `This Set-Cookie was blocked because it had the "SameSite=Lax" attribute but came from a cross-site response which was not the response to a top-level navigation.`;
        case Protocol.Network.SetCookieBlockedReason.SameSiteUnspecifiedTreatedAsLax:
            return ls `This Set-Cookie didn't specify a "SameSite" attribute and was defaulted to "SameSite=Lax," and was blocked because it came from a cross-site response which was not the response to a top-level navigation. The Set-Cookie had to have been set with "SameSite=None" to enable cross-site usage.`;
        case Protocol.Network.SetCookieBlockedReason.SameSiteNoneInsecure:
            return ls `This Set-Cookie was blocked because it had the "SameSite=None" attribute but did not have the "Secure" attribute, which is required in order to use "SameSite=None".`;
        case Protocol.Network.SetCookieBlockedReason.UserPreferences:
            return ls `This Set-Cookie was blocked due to user preferences.`;
        case Protocol.Network.SetCookieBlockedReason.SyntaxError:
            return ls `This Set-Cookie had invalid syntax.`;
        case Protocol.Network.SetCookieBlockedReason.SchemeNotSupported:
            return ls `The scheme of this connection is not allowed to store cookies.`;
        case Protocol.Network.SetCookieBlockedReason.OverwriteSecure:
            return ls `This Set-Cookie was blocked because it was not sent over a secure connection and would have overwritten a cookie with the Secure attribute.`;
        case Protocol.Network.SetCookieBlockedReason.InvalidDomain:
            return ls `This Set-Cookie was blocked because its Domain attribute was invalid with regards to the current host url.`;
        case Protocol.Network.SetCookieBlockedReason.InvalidPrefix:
            return ls `This Set-Cookie was blocked because it used the "__Secure-" or "__Host-" prefix in its name and broke the additional rules applied to cookies with these prefixes as defined in https://tools.ietf.org/html/draft-west-cookie-prefixes-05.`;
        case Protocol.Network.SetCookieBlockedReason.UnknownError:
            return ls `An unknown error was encountered when trying to store this cookie.`;
    }
    return '';
};
/**
 * @param {!Protocol.Network.CookieBlockedReason} blockedReason
 * @return {?Attributes}
 */
const cookieBlockedReasonToAttribute = function (blockedReason) {
    switch (blockedReason) {
        case Protocol.Network.CookieBlockedReason.SecureOnly:
            return Attributes.Secure;
        case Protocol.Network.CookieBlockedReason.NotOnPath:
            return Attributes.Path;
        case Protocol.Network.CookieBlockedReason.DomainMismatch:
            return Attributes.Domain;
        case Protocol.Network.CookieBlockedReason.SameSiteStrict:
        case Protocol.Network.CookieBlockedReason.SameSiteLax:
        case Protocol.Network.CookieBlockedReason.SameSiteUnspecifiedTreatedAsLax:
        case Protocol.Network.CookieBlockedReason.SameSiteNoneInsecure:
            return Attributes.SameSite;
        case Protocol.Network.CookieBlockedReason.UserPreferences:
        case Protocol.Network.CookieBlockedReason.UnknownError:
            return null;
    }
    return null;
};
/**
 * @param {!Protocol.Network.SetCookieBlockedReason} blockedReason
 * @return {?Attributes}
 */
const setCookieBlockedReasonToAttribute = function (blockedReason) {
    switch (blockedReason) {
        case Protocol.Network.SetCookieBlockedReason.SecureOnly:
        case Protocol.Network.SetCookieBlockedReason.OverwriteSecure:
            return Attributes.Secure;
        case Protocol.Network.SetCookieBlockedReason.SameSiteStrict:
        case Protocol.Network.SetCookieBlockedReason.SameSiteLax:
        case Protocol.Network.SetCookieBlockedReason.SameSiteUnspecifiedTreatedAsLax:
        case Protocol.Network.SetCookieBlockedReason.SameSiteNoneInsecure:
            return Attributes.SameSite;
        case Protocol.Network.SetCookieBlockedReason.InvalidDomain:
            return Attributes.Domain;
        case Protocol.Network.SetCookieBlockedReason.InvalidPrefix:
            return Attributes.Name;
        case Protocol.Network.SetCookieBlockedReason.UserPreferences:
        case Protocol.Network.SetCookieBlockedReason.SyntaxError:
        case Protocol.Network.SetCookieBlockedReason.SchemeNotSupported:
        case Protocol.Network.SetCookieBlockedReason.UnknownError:
            return null;
    }
    return null;
};
/** @typedef {!{name: string, value: string}} */
// @ts-ignore typedef
let NameValue;
/** @typedef {!{type: WebSocketFrameType, time: number, text: string, opCode: number, mask: boolean}} */
// @ts-ignore typedef
let WebSocketFrame;
/**
  * @typedef {!{
  *   blockedReasons: !Array<!Protocol.Network.SetCookieBlockedReason>,
  *   cookieLine: string,
  *   cookie: ?Cookie
  * }}
  */
// @ts-ignore typedef
let BlockedSetCookieWithReason;
/**
 * @typedef {!{
 *   blockedReasons: !Array<!Protocol.Network.CookieBlockedReason>,
 *   cookie: !Cookie
 * }}
 */
// @ts-ignore typedef
let BlockedCookieWithReason;
/** @typedef {!{error: ?string, content: ?string, encoded: boolean}} */
// @ts-ignore typedef
let ContentData;
/** @typedef {!{time: number, eventName: string, eventId: string, data: string}} */
// @ts-ignore typedef
let EventSourceMessage;
/**
  * @typedef {!{
  *   blockedRequestCookies: !Array<!BlockedCookieWithReason>,
  *   requestHeaders: !Array<!NameValue>,
  *   includedRequestCookies: !Array<!Cookie>
  * }}
  */
// @ts-ignore typedef
let ExtraRequestInfo;
/**
  * @typedef {!{
  *   blockedResponseCookies: !Array<!BlockedSetCookieWithReason>,
  *   responseHeaders: !Array<!NameValue>,
  *   responseHeadersText: (string|undefined)
  * }}
  */
// @ts-ignore typedef
let ExtraResponseInfo;

var NetworkRequest$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    MIME_TYPE: MIME_TYPE,
    MIME_TYPE_TO_RESOURCE_TYPE: MIME_TYPE_TO_RESOURCE_TYPE,
    NetworkRequest: NetworkRequest,
    Events: Events$2,
    InitiatorType: InitiatorType,
    WebSocketFrameType: WebSocketFrameType,
    cookieBlockedReasonToUiString: cookieBlockedReasonToUiString,
    setCookieBlockedReasonToUiString: setCookieBlockedReasonToUiString,
    cookieBlockedReasonToAttribute: cookieBlockedReasonToAttribute,
    setCookieBlockedReasonToAttribute: setCookieBlockedReasonToAttribute,
    NameValue: NameValue,
    WebSocketFrame: WebSocketFrame,
    BlockedSetCookieWithReason: BlockedSetCookieWithReason,
    BlockedCookieWithReason: BlockedCookieWithReason,
    ContentData: ContentData,
    EventSourceMessage: EventSourceMessage,
    ExtraRequestInfo: ExtraRequestInfo,
    ExtraResponseInfo: ExtraResponseInfo
});

/*
 * Copyright (C) 2009 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class RemoteObject {
    /**
     * This may not be an interface due to "instanceof RemoteObject" checks in the code.
     */
    /**
     * @param {*} value
     * @return {!RemoteObject}
     */
    static fromLocalObject(value) {
        return new LocalJSONObject(value);
    }
    /**
     * @param {!RemoteObject} remoteObject
     * @return {string}
     */
    static type(remoteObject) {
        if (remoteObject === null) {
            return 'null';
        }
        const type = typeof remoteObject;
        if (type !== 'object' && type !== 'function') {
            return type;
        }
        return remoteObject.type;
    }
    /**
     * @param {string} description
     * @return {string}
     */
    static arrayNameFromDescription(description) {
        return description.replace(_descriptionLengthParenRegex, '').replace(_descriptionLengthSquareRegex, '');
    }
    /**
     * @param {!RemoteObject|!Protocol.Runtime.RemoteObject|!Protocol.Runtime.ObjectPreview} object
     * @return {number}
     */
    static arrayLength(object) {
        if (object.subtype !== 'array' && object.subtype !== 'typedarray') {
            return 0;
        }
        // Array lengths in V8-generated descriptions switched from square brackets to parentheses.
        // Both formats are checked in case the front end is dealing with an old version of V8.
        const parenMatches = object.description && object.description.match(_descriptionLengthParenRegex);
        const squareMatches = object.description && object.description.match(_descriptionLengthSquareRegex);
        return parenMatches ? parseInt(parenMatches[1], 10) : (squareMatches ? parseInt(squareMatches[1], 10) : 0);
    }
    /**
     * @param {*} object
     * @return {?string}
     */
    static unserializableDescription(object) {
        const type = typeof object;
        if (type === 'number') {
            const description = String(object);
            if (object === 0 && 1 / object < 0) {
                return UnserializableNumber.Negative0;
            }
            if (description === UnserializableNumber.NaN || description === UnserializableNumber.Infinity ||
                description === UnserializableNumber.NegativeInfinity) {
                return description;
            }
        }
        if (type === 'bigint') {
            return object + 'n';
        }
        return null;
    }
    /**
     * @param {!Protocol.Runtime.RemoteObject|!RemoteObject|number|string|boolean|undefined|null|bigint} object
     * @return {!Protocol.Runtime.CallArgument}
     */
    static toCallArgument(object) {
        const type = typeof object;
        if (type === 'undefined') {
            return {};
        }
        const unserializableDescription = RemoteObject.unserializableDescription(object);
        if (type === 'number') {
            if (unserializableDescription !== null) {
                return { unserializableValue: unserializableDescription };
            }
            return { value: object };
        }
        if (type === 'bigint') {
            return { unserializableValue: /** @type {!Protocol.Runtime.UnserializableValue} */ (unserializableDescription) };
        }
        if (type === 'string' || type === 'boolean') {
            return { value: object };
        }
        if (!object) {
            return { value: null };
        }
        // The unserializableValue is a function on RemoteObject's and a simple property on
        // Protocol.Runtime.RemoteObject's.
        const objectAsProtocolRemoteObject = /** @type {!Protocol.Runtime.RemoteObject} */ (object);
        if (object instanceof RemoteObject) {
            const unserializableValue = object.unserializableValue();
            if (unserializableValue !== undefined) {
                return { unserializableValue: unserializableValue };
            }
        }
        else if (objectAsProtocolRemoteObject.unserializableValue !== undefined) {
            return { unserializableValue: objectAsProtocolRemoteObject.unserializableValue };
        }
        if (typeof objectAsProtocolRemoteObject.objectId !== 'undefined') {
            return { objectId: objectAsProtocolRemoteObject.objectId };
        }
        return { value: objectAsProtocolRemoteObject.value };
    }
    /**
     * @param {!RemoteObject} object
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    static async loadFromObjectPerProto(object, generatePreview) {
        const result = await Promise.all([
            object.getAllProperties(true /* accessorPropertiesOnly */, generatePreview),
            object.getOwnProperties(generatePreview)
        ]);
        const accessorProperties = result[0].properties;
        const ownProperties = result[1].properties;
        const internalProperties = result[1].internalProperties;
        if (!ownProperties || !accessorProperties) {
            return /** @type {!GetPropertiesResult} */ ({ properties: null, internalProperties: null });
        }
        const propertiesMap = new Map();
        const propertySymbols = [];
        for (let i = 0; i < accessorProperties.length; i++) {
            const property = accessorProperties[i];
            if (property.symbol) {
                propertySymbols.push(property);
            }
            else {
                propertiesMap.set(property.name, property);
            }
        }
        for (let i = 0; i < ownProperties.length; i++) {
            const property = ownProperties[i];
            if (property.isAccessorProperty()) {
                continue;
            }
            if (property.symbol) {
                propertySymbols.push(property);
            }
            else {
                propertiesMap.set(property.name, property);
            }
        }
        return {
            properties: [...propertiesMap.values()].concat(propertySymbols),
            internalProperties: internalProperties ? internalProperties : null
        };
    }
    /**
     * @return {?Protocol.Runtime.CustomPreview}
     */
    customPreview() {
        return null;
    }
    /** @return {!Protocol.Runtime.RemoteObjectId|undefined} */
    get objectId() {
        return 'Not implemented';
    }
    /** @return {string} */
    get type() {
        throw 'Not implemented';
    }
    /** @return {string|undefined} */
    get subtype() {
        throw 'Not implemented';
    }
    /** @return {*} */
    get value() {
        throw 'Not implemented';
    }
    /** @return {string|undefined} */
    unserializableValue() {
        throw 'Not implemented';
    }
    /** @return {string|undefined} */
    get description() {
        throw 'Not implemented';
    }
    /** @return {boolean} */
    get hasChildren() {
        throw 'Not implemented';
    }
    /**
     * @return {!Protocol.Runtime.ObjectPreview|undefined}
     */
    get preview() {
        return undefined;
    }
    /**
     * @return {?string}
     */
    get className() {
        return null;
    }
    /**
     * @return {number}
     */
    arrayLength() {
        throw 'Not implemented';
    }
    /**
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getOwnProperties(generatePreview) {
        throw 'Not implemented';
    }
    /**
     * @param {boolean} accessorPropertiesOnly
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getAllProperties(accessorPropertiesOnly, generatePreview) {
        throw 'Not implemented';
    }
    /**
     * @param {!Protocol.Runtime.CallArgument} name
     * @return {!Promise<string|undefined>}
     */
    async deleteProperty(name) {
        throw 'Not implemented';
    }
    /**
     * @param {string|!Protocol.Runtime.CallArgument} name
     * @param {string} value
     * @return {!Promise<string|undefined>}
     */
    async setPropertyValue(name, value) {
        throw 'Not implemented';
    }
    /**
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>=} args
     * @return {!Promise<!CallFunctionResult>}
     * @template T
     */
    callFunction(functionDeclaration, args) {
        throw 'Not implemented';
    }
    /**
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>|undefined} args
     * @return {!Promise<T>}
     * @template T
     */
    callFunctionJSON(functionDeclaration, args) {
        throw 'Not implemented';
    }
    release() {
    }
    /**
     * @return {!DebuggerModel}
     */
    debuggerModel() {
        throw new Error('DebuggerModel-less object');
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        throw new Error('RuntimeModel-less object');
    }
    /**
     * @return {boolean}
     */
    isNode() {
        return false;
    }
}
class RemoteObjectImpl extends RemoteObject {
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {string|undefined} objectId
     * @param {string} type
     * @param {string|undefined} subtype
     * @param {*} value
     * @param {!Protocol.Runtime.UnserializableValue=} unserializableValue
     * @param {string=} description
     * @param {!Protocol.Runtime.ObjectPreview=} preview
     * @param {!Protocol.Runtime.CustomPreview=} customPreview
     * @param {string=} className
     */
    constructor(runtimeModel, objectId, type, subtype, value, unserializableValue, description, preview, customPreview, className) {
        super();
        this._runtimeModel = runtimeModel;
        this._runtimeAgent = runtimeModel.target().runtimeAgent();
        this._type = type;
        this._subtype = subtype;
        if (objectId) {
            // handle
            this._objectId = objectId;
            this._description = description;
            this._hasChildren = (type !== 'symbol');
            this._preview = preview;
        }
        else {
            this._description = description;
            if (!this.description && unserializableValue) {
                this._description = unserializableValue;
            }
            if (!this._description && (typeof value !== 'object' || value === null)) {
                this._description = value + '';
            }
            this._hasChildren = false;
            if (typeof unserializableValue === 'string') {
                this._unserializableValue = unserializableValue;
                if (unserializableValue === UnserializableNumber.Infinity ||
                    unserializableValue === UnserializableNumber.NegativeInfinity ||
                    unserializableValue === UnserializableNumber.Negative0 ||
                    unserializableValue === UnserializableNumber.NaN) {
                    this._value = Number(unserializableValue);
                }
                else if (type === 'bigint' && unserializableValue.endsWith('n')) {
                    this._value = BigInt(unserializableValue.substring(0, unserializableValue.length - 1));
                }
                else {
                    this._value = unserializableValue;
                }
            }
            else {
                this._value = value;
            }
        }
        this._customPreview = customPreview || null;
        this._className = typeof className === 'string' ? className : null;
    }
    /**
     * @override
     * @return {?Protocol.Runtime.CustomPreview}
     */
    customPreview() {
        return this._customPreview;
    }
    /**
     * @override
     * @return {!Protocol.Runtime.RemoteObjectId|undefined}
     */
    get objectId() {
        return this._objectId;
    }
    /**
     * @override
     * @return {string}
     */
    get type() {
        return this._type;
    }
    /**
     * @override
     * @return {string|undefined}
     */
    get subtype() {
        return this._subtype;
    }
    /**
     * @override
     * @return {*}
     */
    get value() {
        return this._value;
    }
    /**
     * @override
     * @return {string|undefined}
     */
    unserializableValue() {
        return this._unserializableValue;
    }
    /**
     * @override
     * @return {string|undefined}
     */
    get description() {
        return this._description;
    }
    /**
     * @override
     * @return {boolean}
     */
    get hasChildren() {
        return this._hasChildren;
    }
    /**
     * @override
     * @return {!Protocol.Runtime.ObjectPreview|undefined}
     */
    get preview() {
        return this._preview;
    }
    /**
     * @override
     * @return {?string}
     */
    get className() {
        return this._className;
    }
    /**
     * @override
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getOwnProperties(generatePreview) {
        return this.doGetProperties(true, false, generatePreview);
    }
    /**
     * @override
     * @param {boolean} accessorPropertiesOnly
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getAllProperties(accessorPropertiesOnly, generatePreview) {
        return this.doGetProperties(false, accessorPropertiesOnly, generatePreview);
    }
    /**
     * @param {boolean} ownProperties
     * @param {boolean} accessorPropertiesOnly
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    async doGetProperties(ownProperties, accessorPropertiesOnly, generatePreview) {
        if (!this._objectId) {
            return /** @type {!GetPropertiesResult} */ ({ properties: null, internalProperties: null });
        }
        const response = await this._runtimeAgent.invoke_getProperties({ objectId: this._objectId, ownProperties, accessorPropertiesOnly, generatePreview });
        if (response.getError()) {
            return /** @type {!GetPropertiesResult} */ ({ properties: null, internalProperties: null });
        }
        if (response.exceptionDetails) {
            this._runtimeModel.exceptionThrown(Date.now(), response.exceptionDetails);
            return /** @type {!GetPropertiesResult} */ ({ properties: null, internalProperties: null });
        }
        const { result: properties = [], internalProperties = [], privateProperties = [] } = response;
        const result = [];
        for (const property of properties) {
            const propertyValue = property.value ? this._runtimeModel.createRemoteObject(property.value) : null;
            const propertySymbol = property.symbol ? this._runtimeModel.createRemoteObject(property.symbol) : null;
            const remoteProperty = new RemoteObjectProperty(property.name, propertyValue, !!property.enumerable, !!property.writable, !!property.isOwn, !!property.wasThrown, propertySymbol);
            if (typeof property.value === 'undefined') {
                if (property.get && property.get.type !== 'undefined') {
                    remoteProperty.getter = this._runtimeModel.createRemoteObject(property.get);
                }
                if (property.set && property.set.type !== 'undefined') {
                    remoteProperty.setter = this._runtimeModel.createRemoteObject(property.set);
                }
            }
            result.push(remoteProperty);
        }
        for (const property of privateProperties) {
            const propertyValue = this._runtimeModel.createRemoteObject(/** @type {!Protocol.Runtime.RemoteObject} */ (property.value));
            const remoteProperty = new RemoteObjectProperty(property.name, propertyValue, true, true, true, false, undefined, false, undefined, true);
            result.push(remoteProperty);
        }
        const internalPropertiesResult = [];
        for (const property of internalProperties) {
            if (!property.value) {
                continue;
            }
            if (property.name === '[[StableObjectId]]') {
                continue;
            }
            const propertyValue = this._runtimeModel.createRemoteObject(property.value);
            internalPropertiesResult.push(new RemoteObjectProperty(property.name, propertyValue, true, false, undefined, undefined, undefined, true));
        }
        return { properties: result, internalProperties: internalPropertiesResult };
    }
    /**
     * @override
     * @param {string|!Protocol.Runtime.CallArgument} name
     * @param {string} value
     * @return {!Promise<string|undefined>}
     */
    async setPropertyValue(name, value) {
        if (!this._objectId) {
            return 'Can’t set a property of non-object.';
        }
        const response = await this._runtimeAgent.invoke_evaluate({ expression: value, silent: true });
        if (response.getError() || response.exceptionDetails) {
            return response.getError() ||
                (response.result.type !== 'string' ? response.result.description :
                    /** @type {string} */ (response.result.value));
        }
        if (typeof name === 'string') {
            name = RemoteObject.toCallArgument(name);
        }
        const resultPromise = this.doSetObjectPropertyValue(response.result, name);
        if (response.result.objectId) {
            this._runtimeAgent.invoke_releaseObject({ objectId: response.result.objectId });
        }
        return resultPromise;
    }
    /**
     * @param {!Protocol.Runtime.RemoteObject} result
     * @param {!Protocol.Runtime.CallArgument} name
     * @return {!Promise<string|undefined>}
     */
    async doSetObjectPropertyValue(result, name) {
        // This assignment may be for a regular (data) property, and for an accessor property (with getter/setter).
        // Note the sensitive matter about accessor property: the property may be physically defined in some proto object,
        // but logically it is bound to the object in question. JavaScript passes this object to getters/setters, not the object
        // where property was defined; so do we.
        const setPropertyValueFunction = 'function(a, b) { this[a] = b; }';
        const argv = [name, RemoteObject.toCallArgument(result)];
        const response = await this._runtimeAgent.invoke_callFunctionOn({ objectId: this._objectId, functionDeclaration: setPropertyValueFunction, arguments: argv, silent: true });
        const error = response.getError();
        return error || response.exceptionDetails ? error || response.result.description : undefined;
    }
    /**
     * @override
     * @param {!Protocol.Runtime.CallArgument} name
     * @return {!Promise<string|undefined>}
     */
    async deleteProperty(name) {
        if (!this._objectId) {
            return 'Can’t delete a property of non-object.';
        }
        const deletePropertyFunction = 'function(a) { delete this[a]; return !(a in this); }';
        const response = await this._runtimeAgent.invoke_callFunctionOn({ objectId: this._objectId, functionDeclaration: deletePropertyFunction, arguments: [name], silent: true });
        if (response.getError() || response.exceptionDetails) {
            return response.getError() || response.result.description;
        }
        if (!response.result.value) {
            return 'Failed to delete property.';
        }
        return undefined;
    }
    /**
     * @override
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>=} args
     * @return {!Promise<!CallFunctionResult>}
     * @template T
     */
    async callFunction(functionDeclaration, args) {
        const response = await this._runtimeAgent.invoke_callFunctionOn({ objectId: this._objectId, functionDeclaration: functionDeclaration.toString(), arguments: args, silent: true });
        if (response.getError()) {
            return { object: null, wasThrown: false };
        }
        // TODO: release exceptionDetails object
        return { object: this._runtimeModel.createRemoteObject(response.result), wasThrown: !!response.exceptionDetails };
    }
    /**
     * @override
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>|undefined} args
     * @return {!Promise<T>}
     * @template T
     */
    async callFunctionJSON(functionDeclaration, args) {
        const response = await this._runtimeAgent.invoke_callFunctionOn({
            objectId: this._objectId,
            functionDeclaration: functionDeclaration.toString(),
            arguments: args,
            silent: true,
            returnByValue: true
        });
        return response.getError() || response.exceptionDetails ? null : response.result.value;
    }
    /**
     * @override
     */
    release() {
        if (!this._objectId) {
            return;
        }
        this._runtimeAgent.invoke_releaseObject({ objectId: this._objectId });
    }
    /**
     * @override
     * @return {number}
     */
    arrayLength() {
        return RemoteObject.arrayLength(this);
    }
    /**
     * @override
     * @return {!DebuggerModel}
     */
    debuggerModel() {
        return this._runtimeModel.debuggerModel();
    }
    /**
     * @override
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    /**
     * @override
     * @return {boolean}
     */
    isNode() {
        return !!this._objectId && this.type === 'object' && this.subtype === 'node';
    }
}
class ScopeRemoteObject extends RemoteObjectImpl {
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {string|undefined} objectId
     * @param {!ScopeRef} scopeRef
     * @param {string} type
     * @param {string|undefined} subtype
     * @param {*} value
     * @param {!Protocol.Runtime.UnserializableValue=} unserializableValue
     * @param {string=} description
     * @param {!Protocol.Runtime.ObjectPreview=} preview
     */
    constructor(runtimeModel, objectId, scopeRef, type, subtype, value, unserializableValue, description, preview) {
        super(runtimeModel, objectId, type, subtype, value, unserializableValue, description, preview);
        this._scopeRef = scopeRef;
        this._savedScopeProperties = undefined;
    }
    /**
     * @override
     * @param {boolean} ownProperties
     * @param {boolean} accessorPropertiesOnly
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    async doGetProperties(ownProperties, accessorPropertiesOnly, generatePreview) {
        if (accessorPropertiesOnly) {
            return /** @type {!GetPropertiesResult} */ ({ properties: [], internalProperties: [] });
        }
        if (this._savedScopeProperties) {
            // No need to reload scope variables, as the remote object never
            // changes its properties. If variable is updated, the properties
            // array is patched locally.
            return { properties: this._savedScopeProperties.slice(), internalProperties: null };
        }
        const allProperties = await super.doGetProperties(ownProperties, accessorPropertiesOnly, true /* generatePreview */);
        if (this._scopeRef && Array.isArray(allProperties.properties)) {
            this._savedScopeProperties = allProperties.properties.slice();
            if (!this._scopeRef.callFrameId) {
                for (const property of this._savedScopeProperties) {
                    property.writable = false;
                }
            }
        }
        return allProperties;
    }
    /**
     * @override
     * @param {!Protocol.Runtime.RemoteObject} result
     * @param {!Protocol.Runtime.CallArgument} argumentName
     * @return {!Promise<string|undefined>}
     */
    async doSetObjectPropertyValue(result, argumentName) {
        const name = /** @type {string} */ (argumentName.value);
        const error = await this.debuggerModel().setVariableValue(this._scopeRef.number, name, RemoteObject.toCallArgument(result), 
        /** @type {string} */ (this._scopeRef.callFrameId));
        if (error) {
            return error;
        }
        if (this._savedScopeProperties) {
            for (const property of this._savedScopeProperties) {
                if (property.name === name) {
                    property.value = this._runtimeModel.createRemoteObject(result);
                }
            }
        }
        return;
    }
}
class ScopeRef {
    /**
     * @param {number} number
     * @param {string=} callFrameId
     */
    constructor(number, callFrameId) {
        this.number = number;
        this.callFrameId = callFrameId;
    }
}
/**
 * @unrestricted
 */
class RemoteObjectProperty {
    /**
     * @param {string} name
     * @param {?RemoteObject} value
     * @param {boolean=} enumerable
     * @param {boolean=} writable
     * @param {boolean=} isOwn
     * @param {boolean=} wasThrown
     * @param {?RemoteObject=} symbol
     * @param {boolean=} synthetic
     * @param {function(string):!Promise<?RemoteObject>=} syntheticSetter
     * @param {boolean=} isPrivate
     */
    constructor(name, value, enumerable, writable, isOwn, wasThrown, symbol, synthetic, syntheticSetter, isPrivate) {
        this.name = name;
        if (value !== null) {
            this.value = value;
        }
        this.enumerable = typeof enumerable !== 'undefined' ? enumerable : true;
        const isNonSyntheticOrSyntheticWritable = !synthetic || !!syntheticSetter;
        this.writable = typeof writable !== 'undefined' ? writable : isNonSyntheticOrSyntheticWritable;
        this.isOwn = !!isOwn;
        this.wasThrown = !!wasThrown;
        if (symbol) {
            this.symbol = symbol;
        }
        this.synthetic = !!synthetic;
        if (syntheticSetter) {
            this.syntheticSetter = syntheticSetter;
        }
        this.private = !!isPrivate;
        /** @type {(!RemoteObject|undefined)} */
        this.getter;
        /** @type {(!RemoteObject|undefined)} */
        this.setter;
    }
    /**
     * @param {string} expression
     * @return {!Promise<boolean>}
     */
    async setSyntheticValue(expression) {
        if (!this.syntheticSetter) {
            return false;
        }
        const result = await this.syntheticSetter(expression);
        if (result) {
            this.value = result;
        }
        return !!result;
    }
    /**
     * @return {boolean}
     */
    isAccessorProperty() {
        return !!(this.getter || this.setter);
    }
}
// Below is a wrapper around a local object that implements the RemoteObject interface,
// which can be used by the UI code (primarily ObjectPropertiesSection).
// Note that only JSON-compliant objects are currently supported, as there's no provision
// for traversing prototypes, extracting class names via constructor, handling properties
// or functions.
class LocalJSONObject extends RemoteObject {
    /**
     * @param {*} value
     */
    constructor(value) {
        super();
        this._value = value;
        /** @type {string} */
        this._cachedDescription;
        /** @type {!Array<!RemoteObjectProperty>} */
        this._cachedChildren;
    }
    /**
     * @override
     * @return {!Protocol.Runtime.RemoteObjectId|undefined}
     * */
    get objectId() {
        return undefined;
    }
    /**
     * @override
     * @return {*}
     */
    get value() {
        return this._value;
    }
    /**
     * @override
     * @return {string|undefined}
     */
    unserializableValue() {
        const unserializableDescription = RemoteObject.unserializableDescription(this._value);
        return unserializableDescription || undefined;
    }
    /**
     * @override
     * @return {string}
     */
    get description() {
        if (this._cachedDescription) {
            return this._cachedDescription;
        }
        /**
         * @param {!RemoteObjectProperty} property
         * @return {string}
         * @this {LocalJSONObject}
         */
        function formatArrayItem(property) {
            return this._formatValue(property.value || null);
        }
        /**
         * @param {!RemoteObjectProperty} property
         * @return {string}
         * @this {LocalJSONObject}
         */
        function formatObjectItem(property) {
            let name = property.name;
            if (/^\s|\s$|^$|\n/.test(name)) {
                name = '"' + name.replace(/\n/g, '\u21B5') + '"';
            }
            return name + ': ' + this._formatValue(property.value || null);
        }
        if (this.type === 'object') {
            switch (this.subtype) {
                case 'array':
                    this._cachedDescription = this._concatenate('[', ']', formatArrayItem.bind(this));
                    break;
                case 'date':
                    this._cachedDescription = '' + this._value;
                    break;
                case 'null':
                    this._cachedDescription = 'null';
                    break;
                default:
                    this._cachedDescription = this._concatenate('{', '}', formatObjectItem.bind(this));
            }
        }
        else {
            this._cachedDescription = String(this._value);
        }
        return this._cachedDescription;
    }
    /**
     * @param {?RemoteObject} value
     * @return {string}
     */
    _formatValue(value) {
        if (!value) {
            return 'undefined';
        }
        const description = value.description || '';
        if (value.type === 'string') {
            return '"' + description.replace(/\n/g, '\u21B5') + '"';
        }
        return description;
    }
    /**
     * @param {string} prefix
     * @param {string} suffix
     * @param {function(!RemoteObjectProperty):string} formatProperty
     * @return {string}
     */
    _concatenate(prefix, suffix, formatProperty) {
        const previewChars = 100;
        let buffer = prefix;
        const children = this._children();
        for (let i = 0; i < children.length; ++i) {
            const itemDescription = formatProperty(children[i]);
            if (buffer.length + itemDescription.length > previewChars) {
                buffer += ',…';
                break;
            }
            if (i) {
                buffer += ', ';
            }
            buffer += itemDescription;
        }
        buffer += suffix;
        return buffer;
    }
    /**
     * @override
     * @return {string}
     */
    get type() {
        return typeof this._value;
    }
    /**
     * @override
     * @return {string|undefined}
     */
    get subtype() {
        if (this._value === null) {
            return 'null';
        }
        if (Array.isArray(this._value)) {
            return 'array';
        }
        if (this._value instanceof Date) {
            return 'date';
        }
        return undefined;
    }
    /**
     * @override
     * @return {boolean}
     */
    get hasChildren() {
        if ((typeof this._value !== 'object') || (this._value === null)) {
            return false;
        }
        return !!Object.keys(/** @type {!Object} */ (this._value)).length;
    }
    /**
     * @override
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getOwnProperties(generatePreview) {
        return Promise.resolve(
        /** @type {!GetPropertiesResult} */ ({ properties: this._children(), internalProperties: null }));
    }
    /**
     * @override
     * @param {boolean} accessorPropertiesOnly
     * @param {boolean} generatePreview
     * @return {!Promise<!GetPropertiesResult>}
     */
    getAllProperties(accessorPropertiesOnly, generatePreview) {
        if (accessorPropertiesOnly) {
            return Promise.resolve(/** @type {!GetPropertiesResult} */ ({ properties: [], internalProperties: null }));
        }
        return Promise.resolve(
        /** @type {!GetPropertiesResult} */ ({ properties: this._children(), internalProperties: null }));
    }
    /**
     * @return {!Array.<!RemoteObjectProperty>}
     */
    _children() {
        if (!this.hasChildren) {
            return [];
        }
        const value = /** @type {*} */ (this._value);
        /**
         * @param {string} propName
         * @return {!RemoteObjectProperty}
         */
        function buildProperty(propName) {
            let propValue = value[propName];
            if (!(propValue instanceof RemoteObject)) {
                propValue = RemoteObject.fromLocalObject(propValue);
            }
            return new RemoteObjectProperty(propName, propValue);
        }
        if (!this._cachedChildren) {
            this._cachedChildren = Object.keys(/** @type {!Object} */ (value)).map(buildProperty);
        }
        return this._cachedChildren;
    }
    /**
     * @override
     * @return {number}
     */
    arrayLength() {
        return Array.isArray(this._value) ? this._value.length : 0;
    }
    /**
     * @override
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>=} args
     * @return {!Promise<!CallFunctionResult>}
     * @template T
     */
    callFunction(functionDeclaration, args) {
        const target = /** @type {!Object} */ (this._value);
        const rawArgs = args ? args.map(arg => arg.value) : [];
        let result;
        let wasThrown = false;
        try {
            result = functionDeclaration.apply(target, rawArgs);
        }
        catch (e) {
            wasThrown = true;
        }
        const object = RemoteObject.fromLocalObject(result);
        return Promise.resolve(
        /** @type {!CallFunctionResult} */ ({ object, wasThrown }));
    }
    /**
     * @override
     * @param {function(this:Object, ...?):T} functionDeclaration
     * @param {!Array<!Protocol.Runtime.CallArgument>|undefined} args
     * @return {!Promise<T>}
     * @template T
     */
    callFunctionJSON(functionDeclaration, args) {
        const target = /** @type {!Object} */ (this._value);
        const rawArgs = args ? args.map(arg => arg.value) : [];
        let result;
        try {
            result = functionDeclaration.apply(target, rawArgs);
        }
        catch (e) {
            result = null;
        }
        return Promise.resolve(/** @type {T} */ (result));
    }
}
class RemoteArray {
    /**
     * @param {!RemoteObject} object
     */
    constructor(object) {
        this._object = object;
    }
    /**
     * @param {?RemoteObject} object
     * @return {!RemoteArray}
     */
    static objectAsArray(object) {
        if (!object || object.type !== 'object' || (object.subtype !== 'array' && object.subtype !== 'typedarray')) {
            throw new Error('Object is empty or not an array');
        }
        return new RemoteArray(object);
    }
    /**
     * @param {!Array<!RemoteObject>} objects
     * @return {!Promise<!RemoteArray>}
     */
    static createFromRemoteObjects(objects) {
        if (!objects.length) {
            throw new Error('Input array is empty');
        }
        const objectArguments = [];
        for (let i = 0; i < objects.length; ++i) {
            objectArguments.push(RemoteObject.toCallArgument(objects[i]));
        }
        return objects[0].callFunction(createArray, objectArguments).then(returnRemoteArray);
        /**
         * @return {!Array<*>}
         */
        function createArray() {
            if (arguments.length > 1) {
                return new Array(arguments);
            }
            return [arguments[0]];
        }
        /**
         * @param {!CallFunctionResult} result
         * @return {!RemoteArray}
         */
        function returnRemoteArray(result) {
            if (result.wasThrown || !result.object) {
                throw new Error('Call function throws exceptions or returns empty value');
            }
            return RemoteArray.objectAsArray(result.object);
        }
    }
    /**
     * @param {number} index
     * @return {!Promise<!RemoteObject>}
     */
    at(index) {
        if (index < 0 || index > this._object.arrayLength()) {
            throw new Error('Out of range');
        }
        return this._object.callFunction(at, [RemoteObject.toCallArgument(index)]).then(assertCallFunctionResult);
        /**
         * @suppressReceiverCheck
         * @param {number} index
         * @return {*}
         * @this {*}
         */
        function at(index) {
            return this[index];
        }
        /**
         * @param {!CallFunctionResult} result
         * @return {!RemoteObject}
         */
        function assertCallFunctionResult(result) {
            if (result.wasThrown || !result.object) {
                throw new Error('Exception in callFunction or result value is empty');
            }
            return result.object;
        }
    }
    /**
     * @return {number}
     */
    length() {
        return this._object.arrayLength();
    }
    /**
     * @param {function(!RemoteObject):!Promise<T>} func
     * @return {!Promise<!Array<T>>}
     * @template T
     */
    map(func) {
        const promises = [];
        for (let i = 0; i < this.length(); ++i) {
            promises.push(this.at(i).then(func));
        }
        return Promise.all(promises);
    }
    /**
     * @return {!RemoteObject}
     */
    object() {
        return this._object;
    }
}
class RemoteFunction {
    /**
     * @param {!RemoteObject} object
     */
    constructor(object) {
        this._object = object;
    }
    /**
     * @param {?RemoteObject} object
     * @return {!RemoteFunction}
     */
    static objectAsFunction(object) {
        if (!object || object.type !== 'function') {
            throw new Error('Object is empty or not a function');
        }
        return new RemoteFunction(object);
    }
    /**
     * @return {!Promise<!RemoteObject>}
     */
    targetFunction() {
        return this._object.getOwnProperties(false /* generatePreview */).then(targetFunction.bind(this));
        /**
         * @param {!GetPropertiesResult} ownProperties
         * @return {!RemoteObject}
         * @this {RemoteFunction}
         */
        function targetFunction(ownProperties) {
            if (!ownProperties.internalProperties) {
                return this._object;
            }
            const internalProperties = ownProperties.internalProperties;
            for (const property of internalProperties) {
                if (property.name === '[[TargetFunction]]') {
                    return /** @type {!RemoteObject} */ (property.value);
                }
            }
            return this._object;
        }
    }
    /**
     * @return {!Promise<?FunctionDetails>}
     */
    targetFunctionDetails() {
        return this.targetFunction().then(functionDetails.bind(this));
        /**
         * @param {!RemoteObject} targetFunction
         * @return {!Promise<?FunctionDetails>}
         * @this {RemoteFunction}
         */
        function functionDetails(targetFunction) {
            const boundReleaseFunctionDetails = releaseTargetFunction.bind(null, this._object !== targetFunction ? targetFunction : null);
            return targetFunction.debuggerModel().functionDetailsPromise(targetFunction).then(boundReleaseFunctionDetails);
        }
        /**
         * @param {?RemoteObject} targetFunction
         * @param {?FunctionDetails} functionDetails
         * @return {?FunctionDetails}
         */
        function releaseTargetFunction(targetFunction, functionDetails) {
            if (targetFunction) {
                targetFunction.release();
            }
            return functionDetails;
        }
    }
    /**
     * @return {!RemoteObject}
     */
    object() {
        return this._object;
    }
}
/**
 * @const
 * @type {!RegExp}
 */
const _descriptionLengthParenRegex = /\(([0-9]+)\)/;
/**
 * @const
 * @type {!RegExp}
 */
const _descriptionLengthSquareRegex = /\[([0-9]+)\]/;
/**
 * @const
 * @enum {!Protocol.Runtime.UnserializableValue}
 */
const UnserializableNumber = {
    Negative0: /** @type {!Protocol.Runtime.UnserializableValue} */ ('-0'),
    NaN: /** @type {!Protocol.Runtime.UnserializableValue} */ ('NaN'),
    Infinity: /** @type {!Protocol.Runtime.UnserializableValue} */ ('Infinity'),
    NegativeInfinity: /** @type {!Protocol.Runtime.UnserializableValue} */ ('-Infinity')
};
/**
 * @typedef {{object: ?RemoteObject, wasThrown: (boolean|undefined)}}
 */
// @ts-ignore typedef
let CallFunctionResult;
/**
 * @typedef {{properties: ?Array<!RemoteObjectProperty>, internalProperties: ?Array<!RemoteObjectProperty>}}
 */
// @ts-ignore typedef
let GetPropertiesResult;

var RemoteObject$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    RemoteObject: RemoteObject,
    RemoteObjectImpl: RemoteObjectImpl,
    ScopeRemoteObject: ScopeRemoteObject,
    ScopeRef: ScopeRef,
    RemoteObjectProperty: RemoteObjectProperty,
    LocalJSONObject: LocalJSONObject,
    RemoteArray: RemoteArray,
    RemoteFunction: RemoteFunction,
    CallFunctionResult: CallFunctionResult,
    GetPropertiesResult: GetPropertiesResult
});

// Copyright 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
class CSSFontFace {
    /**
     * @param {!Protocol.CSS.FontFace} payload
     */
    constructor(payload) {
        this._fontFamily = payload.fontFamily;
        this._fontVariationAxes = payload.fontVariationAxes || [];
        /** @type {!Map<string, !Protocol.CSS.FontVariationAxis>} */
        this._fontVariationAxesByTag = new Map();
        for (const axis of this._fontVariationAxes) {
            this._fontVariationAxesByTag.set(axis.tag, axis);
        }
    }
    getFontFamily() {
        return this._fontFamily;
    }
    /**
     * @param {string} tag
     */
    getVariationAxisByTag(tag) {
        return this._fontVariationAxesByTag.get(tag);
    }
}

var CSSFontFace$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSFontFace: CSSFontFace
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class CSSMediaQuery {
    /**
     * @param {!Protocol.CSS.MediaQuery} payload
     */
    constructor(payload) {
        this._active = payload.active;
        /** @type {?Array<!CSSMediaQueryExpression>} */
        this._expressions = [];
        for (let j = 0; j < payload.expressions.length; ++j) {
            this._expressions.push(CSSMediaQueryExpression.parsePayload(payload.expressions[j]));
        }
    }
    /**
     * @param {!Protocol.CSS.MediaQuery} payload
     * @return {!CSSMediaQuery}
     */
    static parsePayload(payload) {
        return new CSSMediaQuery(payload);
    }
    /**
     * @return {boolean}
     */
    active() {
        return this._active;
    }
    /**
     * @return {?Array<!CSSMediaQueryExpression>}
     */
    expressions() {
        return this._expressions;
    }
}
class CSSMediaQueryExpression {
    /**
     * @param {!Protocol.CSS.MediaQueryExpression} payload
     */
    constructor(payload) {
        this._value = payload.value;
        this._unit = payload.unit;
        this._feature = payload.feature;
        /** @type {?TextUtils.TextRange.TextRange} */
        this._valueRange = payload.valueRange ? TextRange.TextRange.fromObject(payload.valueRange) : null;
        this._computedLength = payload.computedLength || null;
    }
    /**
     * @param {!Protocol.CSS.MediaQueryExpression} payload
     * @return {!CSSMediaQueryExpression}
     */
    static parsePayload(payload) {
        return new CSSMediaQueryExpression(payload);
    }
    /**
     * @return {number}
     */
    value() {
        return this._value;
    }
    /**
     * @return {string}
     */
    unit() {
        return this._unit;
    }
    /**
     * @return {string}
     */
    feature() {
        return this._feature;
    }
    /**
     * @return {?TextUtils.TextRange.TextRange}
     */
    valueRange() {
        return this._valueRange;
    }
    /**
     * @return {?number}
     */
    computedLength() {
        return this._computedLength;
    }
}
/**
 * @unrestricted
 */
class CSSMedia {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSMedia} payload
     */
    constructor(cssModel, payload) {
        this._cssModel = cssModel;
        this._reinitialize(payload);
    }
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSMedia} payload
     * @return {!CSSMedia}
     */
    static parsePayload(cssModel, payload) {
        return new CSSMedia(cssModel, payload);
    }
    /**
     * @param {!CSSModel} cssModel
     * @param {!Array.<!Protocol.CSS.CSSMedia>} payload
     * @return {!Array.<!CSSMedia>}
     */
    static parseMediaArrayPayload(cssModel, payload) {
        const result = [];
        for (let i = 0; i < payload.length; ++i) {
            result.push(CSSMedia.parsePayload(cssModel, payload[i]));
        }
        return result;
    }
    /**
     * @param {!Protocol.CSS.CSSMedia} payload
     */
    _reinitialize(payload) {
        this.text = payload.text;
        this.source = payload.source;
        this.sourceURL = payload.sourceURL || '';
        /** @type {?TextUtils.TextRange.TextRange} */
        this.range = payload.range ? TextRange.TextRange.fromObject(payload.range) : null;
        this.styleSheetId = payload.styleSheetId;
        /** @type {?Array<!CSSMediaQuery>} */
        this.mediaList = null;
        if (payload.mediaList) {
            this.mediaList = [];
            for (let i = 0; i < payload.mediaList.length; ++i) {
                this.mediaList.push(CSSMediaQuery.parsePayload(payload.mediaList[i]));
            }
        }
    }
    /**
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.styleSheetId !== edit.styleSheetId || !this.range) {
            return;
        }
        if (edit.oldRange.equal(this.range)) {
            this._reinitialize(/** @type {!Protocol.CSS.CSSMedia} */ (edit.payload));
        }
        else {
            this.range = this.range.rebaseAfterTextEdit(edit.oldRange, edit.newRange);
        }
    }
    /**
     * @param {!CSSMedia} other
     * @return {boolean}
     */
    equal(other) {
        if (!this.styleSheetId || !this.range || !other.range) {
            return false;
        }
        return this.styleSheetId === other.styleSheetId && this.range.equal(other.range);
    }
    /**
     * @return {boolean}
     */
    active() {
        if (!this.mediaList) {
            return true;
        }
        for (let i = 0; i < this.mediaList.length; ++i) {
            if (this.mediaList[i].active()) {
                return true;
            }
        }
        return false;
    }
    /**
     * @return {number|undefined}
     */
    lineNumberInSource() {
        if (!this.range) {
            return undefined;
        }
        const header = this.header();
        if (!header) {
            return undefined;
        }
        return header.lineNumberInSource(this.range.startLine);
    }
    /**
     * @return {number|undefined}
     */
    columnNumberInSource() {
        if (!this.range) {
            return undefined;
        }
        const header = this.header();
        if (!header) {
            return undefined;
        }
        return header.columnNumberInSource(this.range.startLine, this.range.startColumn);
    }
    /**
     * @return {?CSSStyleSheetHeader}
     */
    header() {
        return this.styleSheetId ? this._cssModel.styleSheetHeaderForId(this.styleSheetId) : null;
    }
    /**
     * @return {?CSSLocation}
     */
    rawLocation() {
        const header = this.header();
        if (!header || this.lineNumberInSource() === undefined) {
            return null;
        }
        const lineNumber = Number(this.lineNumberInSource());
        return new CSSLocation(header, lineNumber, this.columnNumberInSource());
    }
}
const Source = {
    LINKED_SHEET: 'linkedSheet',
    INLINE_SHEET: 'inlineSheet',
    MEDIA_RULE: 'mediaRule',
    IMPORT_RULE: 'importRule',
};

var CSSMedia$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSMediaQuery: CSSMediaQuery,
    CSSMediaQueryExpression: CSSMediaQueryExpression,
    CSSMedia: CSSMedia,
    Source: Source
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class CSSProperty {
    /**
     * @param {!CSSStyleDeclaration} ownerStyle
     * @param {number} index
     * @param {string} name
     * @param {string} value
     * @param {boolean} important
     * @param {boolean} disabled
     * @param {boolean} parsedOk
     * @param {boolean} implicit
     * @param {?string=} text
     * @param {!Protocol.CSS.SourceRange=} range
     */
    constructor(ownerStyle, index, name, value, important, disabled, parsedOk, implicit, text, range) {
        this.ownerStyle = ownerStyle;
        this.index = index;
        this.name = name;
        this.value = value;
        this.important = important;
        this.disabled = disabled;
        this.parsedOk = parsedOk;
        this.implicit = implicit; // A longhand, implicitly set by missing values of shorthand.
        this.text = text;
        /** @type {?TextUtils.TextRange.TextRange} */
        this.range = range ? TextRange.TextRange.fromObject(range) : null;
        this._active = true;
        this._nameRange = null;
        this._valueRange = null;
    }
    /**
     * @param {!CSSStyleDeclaration} ownerStyle
     * @param {number} index
     * @param {!Protocol.CSS.CSSProperty} payload
     * @return {!CSSProperty}
     */
    static parsePayload(ownerStyle, index, payload) {
        // The following default field values are used in the payload:
        // important: false
        // parsedOk: true
        // implicit: false
        // disabled: false
        const result = new CSSProperty(ownerStyle, index, payload.name, payload.value, payload.important || false, payload.disabled || false, ('parsedOk' in payload) ? !!payload.parsedOk : true, !!payload.implicit, payload.text, payload.range);
        return result;
    }
    _ensureRanges() {
        if (this._nameRange && this._valueRange) {
            return;
        }
        const range = this.range;
        const text = this.text ? new Text.Text(this.text) : null;
        if (!range || !text) {
            return;
        }
        const nameIndex = text.value().indexOf(this.name);
        const valueIndex = text.value().lastIndexOf(this.value);
        if (nameIndex === -1 || valueIndex === -1 || nameIndex > valueIndex) {
            return;
        }
        const nameSourceRange = new TextRange.SourceRange(nameIndex, this.name.length);
        const valueSourceRange = new TextRange.SourceRange(valueIndex, this.value.length);
        this._nameRange = rebase(text.toTextRange(nameSourceRange), range.startLine, range.startColumn);
        this._valueRange = rebase(text.toTextRange(valueSourceRange), range.startLine, range.startColumn);
        /**
         * @param {!TextUtils.TextRange.TextRange} oneLineRange
         * @param {number} lineOffset
         * @param {number} columnOffset
         * @return {!TextUtils.TextRange.TextRange}
         */
        function rebase(oneLineRange, lineOffset, columnOffset) {
            if (oneLineRange.startLine === 0) {
                oneLineRange.startColumn += columnOffset;
                oneLineRange.endColumn += columnOffset;
            }
            oneLineRange.startLine += lineOffset;
            oneLineRange.endLine += lineOffset;
            return oneLineRange;
        }
    }
    /**
     * @return {?TextUtils.TextRange.TextRange}
     */
    nameRange() {
        this._ensureRanges();
        return this._nameRange;
    }
    /**
     * @return {?TextUtils.TextRange.TextRange}
     */
    valueRange() {
        this._ensureRanges();
        return this._valueRange;
    }
    /**
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.ownerStyle.styleSheetId !== edit.styleSheetId) {
            return;
        }
        if (this.range) {
            this.range = this.range.rebaseAfterTextEdit(edit.oldRange, edit.newRange);
        }
    }
    /**
     * @param {boolean} active
     */
    setActive(active) {
        this._active = active;
    }
    get propertyText() {
        if (this.text !== undefined) {
            return this.text;
        }
        if (this.name === '') {
            return '';
        }
        return this.name + ': ' + this.value + (this.important ? ' !important' : '') + ';';
    }
    /**
     * @return {boolean}
     */
    activeInStyle() {
        return this._active;
    }
    /**
     * @param {string} propertyText
     * @param {boolean} majorChange
     * @param {boolean=} overwrite
     * @return {!Promise.<boolean>}
     */
    async setText(propertyText, majorChange, overwrite) {
        if (!this.ownerStyle) {
            return Promise.reject(new Error('No ownerStyle for property'));
        }
        if (!this.ownerStyle.styleSheetId) {
            return Promise.reject(new Error('No owner style id'));
        }
        if (!this.range || !this.ownerStyle.range) {
            return Promise.reject(new Error('Style not editable'));
        }
        if (majorChange) {
            userMetrics.actionTaken(UserMetrics.Action.StyleRuleEdited);
        }
        if (overwrite && propertyText === this.propertyText) {
            this.ownerStyle.cssModel().domModel().markUndoableState(!majorChange);
            return Promise.resolve(true);
        }
        const range = this.range.relativeTo(this.ownerStyle.range.startLine, this.ownerStyle.range.startColumn);
        const indentation = this.ownerStyle.cssText ?
            this._detectIndentation(this.ownerStyle.cssText) :
            Settings.Settings.instance().moduleSetting('textEditorIndent').get();
        const endIndentation = this.ownerStyle.cssText ? indentation.substring(0, this.ownerStyle.range.endColumn) : '';
        const text = new Text.Text(this.ownerStyle.cssText || '');
        const newStyleText = text.replaceRange(range, StringUtilities.sprintf(';%s;', propertyText));
        const tokenizerFactory = 
        /** @type {!TextUtils.TextUtils.TokenizerFactory} */ (await /** @type {!Root.Runtime.Extension} */ (Runtime.Runtime.instance().extension(TextUtils.TokenizerFactory))
            .instance());
        const styleText = CSSProperty._formatStyle(newStyleText, indentation, endIndentation, tokenizerFactory);
        return this.ownerStyle.setText(styleText, majorChange);
    }
    /**
     * @param {string} styleText
     * @param {string} indentation
     * @param {string} endIndentation
     * @param {!TextUtils.TextUtils.TokenizerFactory} tokenizerFactory
     * @return {string}
     */
    static _formatStyle(styleText, indentation, endIndentation, tokenizerFactory) {
        const doubleIndent = indentation.substring(endIndentation.length) + indentation;
        if (indentation) {
            indentation = '\n' + indentation;
        }
        let result = '';
        let propertyName = '';
        let propertyText = '';
        let insideProperty = false;
        let needsSemi = false;
        const tokenize = tokenizerFactory.createTokenizer('text/css');
        tokenize('*{' + styleText + '}', processToken);
        if (insideProperty) {
            result += propertyText;
        }
        result = result.substring(2, result.length - 1).trimRight();
        return result + (indentation ? '\n' + endIndentation : '');
        /**
         * @param {string} token
         * @param {?string} tokenType
         * @param {number} column
         * @param {number} newColumn
         */
        function processToken(token, tokenType, column, newColumn) {
            if (!insideProperty) {
                const disabledProperty = tokenType && tokenType.includes('css-comment') && isDisabledProperty(token);
                const isPropertyStart = tokenType &&
                    (tokenType.includes('css-string') || tokenType.includes('css-meta') || tokenType.includes('css-property') ||
                        tokenType.includes('css-variable-2'));
                if (disabledProperty) {
                    result = result.trimRight() + indentation + token;
                }
                else if (isPropertyStart) {
                    insideProperty = true;
                    propertyText = token;
                }
                else if (token !== ';' || needsSemi) {
                    result += token;
                    if (token.trim() && !(tokenType && tokenType.includes('css-comment'))) {
                        needsSemi = token !== ';';
                    }
                }
                if (token === '{' && !tokenType) {
                    needsSemi = false;
                }
                return;
            }
            if (token === '}' || token === ';') {
                // While `propertyText` can generally be trimmed, doing so
                // breaks valid CSS declarations such as `--foo:  ;` which would
                // then produce invalid CSS of the form `--foo:;`. This
                // implementation takes special care to restore a single
                // whitespace token in this edge case. https://crbug.com/1071296
                const trimmedPropertyText = propertyText.trim();
                result = result.trimRight() + indentation + trimmedPropertyText +
                    (trimmedPropertyText.endsWith(':') ? ' ' : '') + ';';
                needsSemi = false;
                insideProperty = false;
                propertyName = '';
                if (token === '}') {
                    result += '}';
                }
            }
            else {
                if (cssMetadata().isGridAreaDefiningProperty(propertyName)) {
                    const rowResult = GridAreaRowRegex.exec(token);
                    if (rowResult && rowResult.index === 0 && !propertyText.trimRight().endsWith(']')) {
                        propertyText = propertyText.trimRight() + '\n' + doubleIndent;
                    }
                }
                if (!propertyName && token === ':') {
                    propertyName = propertyText;
                }
                propertyText += token;
            }
        }
        /**
         * @param {string} text
         * @return {boolean}
         */
        function isDisabledProperty(text) {
            const colon = text.indexOf(':');
            if (colon === -1) {
                return false;
            }
            const propertyName = text.substring(2, colon).trim();
            return cssMetadata().isCSSPropertyName(propertyName);
        }
    }
    /**
     * @param {string} text
     * @return {string}
     */
    _detectIndentation(text) {
        const lines = text.split('\n');
        if (lines.length < 2) {
            return '';
        }
        return TextUtils.Utils.lineIndent(lines[1]);
    }
    /**
     * @param {string} newValue
     * @param {boolean} majorChange
     * @param {boolean} overwrite
     * @param {function(boolean):void=} userCallback
     */
    setValue(newValue, majorChange, overwrite, userCallback) {
        const text = this.name + ': ' + newValue + (this.important ? ' !important' : '') + ';';
        this.setText(text, majorChange, overwrite).then(userCallback);
    }
    /**
     * @param {boolean} disabled
     * @return {!Promise.<boolean>}
     */
    setDisabled(disabled) {
        if (!this.ownerStyle) {
            return Promise.resolve(false);
        }
        if (disabled === this.disabled) {
            return Promise.resolve(true);
        }
        if (!this.text) {
            return Promise.resolve(true);
        }
        const propertyText = this.text.trim();
        const text = disabled ? '/* ' + propertyText + ' */' : this.text.substring(2, propertyText.length - 2).trim();
        return this.setText(text, true, true);
    }
}

var CSSProperty$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSProperty: CSSProperty
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class CSSStyleDeclaration {
    /**
     * @param {!CSSModel} cssModel
     * @param {?CSSRule} parentRule
     * @param {!Protocol.CSS.CSSStyle} payload
     * @param {!Type} type
     */
    constructor(cssModel, parentRule, payload, type) {
        this._cssModel = cssModel;
        this.parentRule = parentRule;
        /** @type {!Array<!CSSProperty>} */
        this._allProperties;
        /** @type {string|undefined} */
        this.styleSheetId;
        /** @type {?TextUtils.TextRange.TextRange} */
        this.range;
        /** @type {string|undefined} */
        this.cssText;
        /** @type {!Map<string, string>} */
        this._shorthandValues;
        /** @type {!Set<string>} */
        this._shorthandIsImportant;
        /** @type {!Map<string, !CSSProperty>} */
        this._activePropertyMap;
        /** @type {?Array<!CSSProperty>} */
        this._leadingProperties;
        this._reinitialize(payload);
        this.type = type;
    }
    /**
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.styleSheetId !== edit.styleSheetId || !this.range) {
            return;
        }
        if (edit.oldRange.equal(this.range)) {
            this._reinitialize(/** @type {!Protocol.CSS.CSSStyle} */ (edit.payload));
        }
        else {
            this.range = this.range.rebaseAfterTextEdit(edit.oldRange, edit.newRange);
            for (let i = 0; i < this._allProperties.length; ++i) {
                this._allProperties[i].rebase(edit);
            }
        }
    }
    /**
     * @param {!Protocol.CSS.CSSStyle} payload
     */
    _reinitialize(payload) {
        this.styleSheetId = payload.styleSheetId;
        this.range = payload.range ? TextRange.TextRange.fromObject(payload.range) : null;
        const shorthandEntries = payload.shorthandEntries;
        this._shorthandValues = new Map();
        this._shorthandIsImportant = new Set();
        for (let i = 0; i < shorthandEntries.length; ++i) {
            this._shorthandValues.set(shorthandEntries[i].name, shorthandEntries[i].value);
            if (shorthandEntries[i].important) {
                this._shorthandIsImportant.add(shorthandEntries[i].name);
            }
        }
        this._allProperties = [];
        if (payload.cssText && this.range) {
            const cssText = new Text.Text(payload.cssText);
            let start = { line: this.range.startLine, column: this.range.startColumn };
            for (const cssProperty of payload.cssProperties) {
                const range = cssProperty.range;
                if (range) {
                    parseUnusedText.call(this, cssText, start.line, start.column, range.startLine, range.startColumn);
                    start = { line: range.endLine, column: range.endColumn };
                }
                this._allProperties.push(CSSProperty.parsePayload(this, this._allProperties.length, cssProperty));
            }
            parseUnusedText.call(this, cssText, start.line, start.column, this.range.endLine, this.range.endColumn);
        }
        else {
            for (const cssProperty of payload.cssProperties) {
                this._allProperties.push(CSSProperty.parsePayload(this, this._allProperties.length, cssProperty));
            }
        }
        this._generateSyntheticPropertiesIfNeeded();
        this._computeInactiveProperties();
        this._activePropertyMap = new Map();
        for (const property of this._allProperties) {
            if (!property.activeInStyle()) {
                continue;
            }
            this._activePropertyMap.set(property.name, property);
        }
        this.cssText = payload.cssText;
        this._leadingProperties = null;
        /**
         * @this {CSSStyleDeclaration}
         * @param {!TextUtils.Text.Text} cssText
         * @param {number} startLine
         * @param {number} startColumn
         * @param {number} endLine
         * @param {number} endColumn
         */
        function parseUnusedText(cssText, startLine, startColumn, endLine, endColumn) {
            const tr = new TextRange.TextRange(startLine, startColumn, endLine, endColumn);
            if (!this.range) {
                return;
            }
            const missingText = cssText.extract(tr.relativeTo(this.range.startLine, this.range.startColumn));
            // Try to fit the malformed css into properties.
            const lines = missingText.split('\n');
            let lineNumber = 0;
            let inComment = false;
            for (const line of lines) {
                let column = 0;
                for (const property of line.split(';')) {
                    const strippedProperty = stripComments(property, inComment);
                    const trimmedProperty = strippedProperty.text.trim();
                    inComment = strippedProperty.inComment;
                    if (trimmedProperty) {
                        let name;
                        let value;
                        const colonIndex = trimmedProperty.indexOf(':');
                        if (colonIndex === -1) {
                            name = trimmedProperty;
                            value = '';
                        }
                        else {
                            name = trimmedProperty.substring(0, colonIndex).trim();
                            value = trimmedProperty.substring(colonIndex + 1).trim();
                        }
                        const range = new TextRange.TextRange(lineNumber, column, lineNumber, column + property.length);
                        this._allProperties.push(new CSSProperty(this, this._allProperties.length, name, value, false, false, false, false, property, range.relativeFrom(startLine, startColumn)));
                    }
                    column += property.length + 1;
                }
                lineNumber++;
            }
        }
        /**
         * @param {string} text
         * @param {boolean} inComment
         * @return {{text: string, inComment: boolean}}
         */
        function stripComments(text, inComment) {
            let output = '';
            for (let i = 0; i < text.length; i++) {
                if (!inComment && text.substring(i, i + 2) === '/*') {
                    inComment = true;
                    i++;
                }
                else if (inComment && text.substring(i, i + 2) === '*/') {
                    inComment = false;
                    i++;
                }
                else if (!inComment) {
                    output += text[i];
                }
            }
            return { text: output, inComment };
        }
    }
    _generateSyntheticPropertiesIfNeeded() {
        if (this.range) {
            return;
        }
        if (!this._shorthandValues.size) {
            return;
        }
        const propertiesSet = new Set();
        for (const property of this._allProperties) {
            propertiesSet.add(property.name);
        }
        const generatedProperties = [];
        // For style-based properties, generate shorthands with values when possible.
        for (const property of this._allProperties) {
            // For style-based properties, try generating shorthands.
            const shorthands = cssMetadata().shorthands(property.name) || [];
            for (const shorthand of shorthands) {
                if (propertiesSet.has(shorthand)) {
                    continue;
                } // There already is a shorthand this longhands falls under.
                const shorthandValue = this._shorthandValues.get(shorthand);
                if (!shorthandValue) {
                    continue;
                } // Never generate synthetic shorthands when no value is available.
                // Generate synthetic shorthand we have a value for.
                const shorthandImportance = !!this._shorthandIsImportant.has(shorthand);
                const shorthandProperty = new CSSProperty(this, this.allProperties().length, shorthand, shorthandValue, shorthandImportance, false, true, false);
                generatedProperties.push(shorthandProperty);
                propertiesSet.add(shorthand);
            }
        }
        this._allProperties = this._allProperties.concat(generatedProperties);
    }
    /**
     * @return {!Array.<!CSSProperty>}
     */
    _computeLeadingProperties() {
        /**
         * @param {!CSSProperty} property
         * @return {boolean}
         */
        function propertyHasRange(property) {
            return !!property.range;
        }
        if (this.range) {
            return this._allProperties.filter(propertyHasRange);
        }
        const leadingProperties = [];
        for (const property of this._allProperties) {
            const shorthands = cssMetadata().shorthands(property.name) || [];
            let belongToAnyShorthand = false;
            for (const shorthand of shorthands) {
                if (this._shorthandValues.get(shorthand)) {
                    belongToAnyShorthand = true;
                    break;
                }
            }
            if (!belongToAnyShorthand) {
                leadingProperties.push(property);
            }
        }
        return leadingProperties;
    }
    /**
     * @return {!Array.<!CSSProperty>}
     */
    leadingProperties() {
        if (!this._leadingProperties) {
            this._leadingProperties = this._computeLeadingProperties();
        }
        return this._leadingProperties;
    }
    /**
     * @return {!Target}
     */
    target() {
        return this._cssModel.target();
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return this._cssModel;
    }
    _computeInactiveProperties() {
        /** @type {!Map<string, !CSSProperty>} */
        const activeProperties = new Map();
        for (let i = 0; i < this._allProperties.length; ++i) {
            const property = this._allProperties[i];
            if (property.disabled || !property.parsedOk) {
                property.setActive(false);
                continue;
            }
            const canonicalName = cssMetadata().canonicalPropertyName(property.name);
            const activeProperty = activeProperties.get(canonicalName);
            if (!activeProperty) {
                activeProperties.set(canonicalName, property);
            }
            else if (!activeProperty.important || property.important) {
                activeProperty.setActive(false);
                activeProperties.set(canonicalName, property);
            }
            else {
                property.setActive(false);
            }
        }
    }
    /**
     * @return {!Array<!CSSProperty>}
     */
    allProperties() {
        return this._allProperties;
    }
    /**
     * @param {string} name
     * @return {string}
     */
    getPropertyValue(name) {
        const property = this._activePropertyMap.get(name);
        return property ? property.value : '';
    }
    /**
     * @param {string} name
     * @return {boolean}
     */
    isPropertyImplicit(name) {
        const property = this._activePropertyMap.get(name);
        return property ? property.implicit : false;
    }
    /**
     * @param {string} name
     * @return {!Array.<!CSSProperty>}
     */
    longhandProperties(name) {
        const longhands = cssMetadata().longhands(name);
        const result = [];
        for (let i = 0; longhands && i < longhands.length; ++i) {
            const property = this._activePropertyMap.get(longhands[i]);
            if (property) {
                result.push(property);
            }
        }
        return result;
    }
    /**
     * @param {number} index
     * @return {?CSSProperty}
     */
    propertyAt(index) {
        return (index < this.allProperties().length) ? this.allProperties()[index] : null;
    }
    /**
     * @return {number}
     */
    pastLastSourcePropertyIndex() {
        for (let i = this.allProperties().length - 1; i >= 0; --i) {
            if (this.allProperties()[i].range) {
                return i + 1;
            }
        }
        return 0;
    }
    /**
     * @param {number} index
     * @return {!TextUtils.TextRange.TextRange}
     */
    _insertionRange(index) {
        const property = this.propertyAt(index);
        if (property && property.range) {
            return property.range.collapseToStart();
        }
        if (!this.range) {
            throw new Error('CSSStyleDeclaration.range is null');
        }
        return this.range.collapseToEnd();
    }
    /**
     * @param {number=} index
     * @return {!CSSProperty}
     */
    newBlankProperty(index) {
        index = (typeof index === 'undefined') ? this.pastLastSourcePropertyIndex() : index;
        const property = new CSSProperty(this, index, '', '', false, false, true, false, '', this._insertionRange(index));
        return property;
    }
    /**
     * @param {string} text
     * @param {boolean} majorChange
     * @return {!Promise.<boolean>}
     */
    setText(text, majorChange) {
        if (!this.range || !this.styleSheetId) {
            return Promise.resolve(false);
        }
        return this._cssModel.setStyleText(this.styleSheetId, this.range, text, majorChange);
    }
    /**
     * @param {number} index
     * @param {string} name
     * @param {string} value
     * @param {function(boolean):void=} userCallback
     */
    insertPropertyAt(index, name, value, userCallback) {
        this.newBlankProperty(index).setText(name + ': ' + value + ';', false, true).then(userCallback);
    }
    /**
     * @param {string} name
     * @param {string} value
     * @param {function(boolean):void=} userCallback
     */
    appendProperty(name, value, userCallback) {
        this.insertPropertyAt(this.allProperties().length, name, value, userCallback);
    }
}
/** @enum {string} */
const Type$2 = {
    Regular: 'Regular',
    Inline: 'Inline',
    Attributes: 'Attributes'
};

var CSSStyleDeclaration$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSStyleDeclaration: CSSStyleDeclaration,
    Type: Type$2
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class CSSRule {
    /**
     * @param {!CSSModel} cssModel
     * @param {{style: !Protocol.CSS.CSSStyle, styleSheetId: (string|undefined), origin: !Protocol.CSS.StyleSheetOrigin}} payload
     */
    constructor(cssModel, payload) {
        this._cssModel = cssModel;
        this.styleSheetId = payload.styleSheetId;
        if (this.styleSheetId) {
            const styleSheetHeader = this._getStyleSheetHeader(this.styleSheetId);
            this.sourceURL = styleSheetHeader.sourceURL;
        }
        this.origin = payload.origin;
        this.style = new CSSStyleDeclaration(this._cssModel, this, payload.style, Type$2.Regular);
    }
    /**
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.styleSheetId !== edit.styleSheetId) {
            return;
        }
        this.style.rebase(edit);
    }
    /**
     * @return {string}
     */
    resourceURL() {
        if (!this.styleSheetId) {
            return '';
        }
        const styleSheetHeader = this._getStyleSheetHeader(this.styleSheetId);
        return styleSheetHeader.resourceURL();
    }
    /**
     * @return {boolean}
     */
    isUserAgent() {
        return this.origin === Protocol.CSS.StyleSheetOrigin.UserAgent;
    }
    /**
     * @return {boolean}
     */
    isInjected() {
        return this.origin === Protocol.CSS.StyleSheetOrigin.Injected;
    }
    /**
     * @return {boolean}
     */
    isViaInspector() {
        return this.origin === Protocol.CSS.StyleSheetOrigin.Inspector;
    }
    /**
     * @return {boolean}
     */
    isRegular() {
        return this.origin === Protocol.CSS.StyleSheetOrigin.Regular;
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return this._cssModel;
    }
    /**
     * @param {string} styleSheetId
     * @return {!CSSStyleSheetHeader}
     */
    _getStyleSheetHeader(styleSheetId) {
        const styleSheetHeader = this._cssModel.styleSheetHeaderForId(styleSheetId);
        console.assert(styleSheetHeader !== null);
        return /** @type {!CSSStyleSheetHeader} */ (styleSheetHeader);
    }
}
class CSSValue {
    /**
     * @param {!Protocol.CSS.Value} payload
     */
    constructor(payload) {
        this.text = payload.text;
        if (payload.range) {
            this.range = TextRange.TextRange.fromObject(payload.range);
        }
    }
    /**
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (!this.range) {
            return;
        }
        this.range = this.range.rebaseAfterTextEdit(edit.oldRange, edit.newRange);
    }
}
class CSSStyleRule extends CSSRule {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSRule} payload
     * @param {boolean=} wasUsed
     */
    constructor(cssModel, payload, wasUsed) {
        // TODO(crbug.com/1011811): Replace with spread operator or better types once Closure is gone.
        super(cssModel, { origin: payload.origin, style: payload.style, styleSheetId: payload.styleSheetId });
        /** @type {!Array.<!CSSValue>} */
        this.selectors;
        this._reinitializeSelectors(payload.selectorList);
        this.media = payload.media ? CSSMedia.parseMediaArrayPayload(cssModel, payload.media) : [];
        this.wasUsed = wasUsed || false;
    }
    /**
     * @param {!CSSModel} cssModel
     * @param {string} selectorText
     * @return {!CSSStyleRule}
     */
    static createDummyRule(cssModel, selectorText) {
        const dummyPayload = {
            selectorList: {
                text: '',
                selectors: [{ text: selectorText, value: undefined }],
            },
            style: {
                styleSheetId: '0',
                range: new TextRange.TextRange(0, 0, 0, 0),
                shorthandEntries: [],
                cssProperties: []
            },
            origin: Protocol.CSS.StyleSheetOrigin.Inspector,
        };
        return new CSSStyleRule(cssModel, /** @type {!Protocol.CSS.CSSRule} */ (dummyPayload));
    }
    /**
     * @param {!Protocol.CSS.SelectorList} selectorList
     */
    _reinitializeSelectors(selectorList) {
        this.selectors = [];
        for (let i = 0; i < selectorList.selectors.length; ++i) {
            this.selectors.push(new CSSValue(selectorList.selectors[i]));
        }
    }
    /**
     * @param {string} newSelector
     * @return {!Promise.<boolean>}
     */
    setSelectorText(newSelector) {
        const styleSheetId = this.styleSheetId;
        if (!styleSheetId) {
            throw 'No rule stylesheet id';
        }
        const range = this.selectorRange();
        if (!range) {
            throw 'Rule selector is not editable';
        }
        return this._cssModel.setSelectorText(styleSheetId, range, newSelector);
    }
    /**
     * @return {string}
     */
    selectorText() {
        return this.selectors.map(selector => selector.text).join(', ');
    }
    /**
     * @return {?TextUtils.TextRange.TextRange}
     */
    selectorRange() {
        const firstRange = this.selectors[0].range;
        const lastRange = this.selectors[this.selectors.length - 1].range;
        if (!firstRange || !lastRange) {
            return null;
        }
        return new TextRange.TextRange(firstRange.startLine, firstRange.startColumn, lastRange.endLine, lastRange.endColumn);
    }
    /**
     * @param {number} selectorIndex
     * @return {number}
     */
    lineNumberInSource(selectorIndex) {
        const selector = this.selectors[selectorIndex];
        if (!selector || !selector.range || !this.styleSheetId) {
            return 0;
        }
        const styleSheetHeader = this._getStyleSheetHeader(this.styleSheetId);
        return styleSheetHeader.lineNumberInSource(selector.range.startLine);
    }
    /**
     * @param {number} selectorIndex
     * @return {number|undefined}
     */
    columnNumberInSource(selectorIndex) {
        const selector = this.selectors[selectorIndex];
        if (!selector || !selector.range || !this.styleSheetId) {
            return undefined;
        }
        const styleSheetHeader = this._getStyleSheetHeader(this.styleSheetId);
        return styleSheetHeader.columnNumberInSource(selector.range.startLine, selector.range.startColumn);
    }
    /**
     * @override
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.styleSheetId !== edit.styleSheetId) {
            return;
        }
        const range = this.selectorRange();
        if (range && range.equal(edit.oldRange)) {
            this._reinitializeSelectors(/** @type {!Protocol.CSS.SelectorList} */ (edit.payload));
        }
        else {
            for (let i = 0; i < this.selectors.length; ++i) {
                this.selectors[i].rebase(edit);
            }
        }
        for (const media of this.media) {
            media.rebase(edit);
        }
        super.rebase(edit);
    }
}
class CSSKeyframesRule {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSKeyframesRule} payload
     */
    constructor(cssModel, payload) {
        this._cssModel = cssModel;
        this._animationName = new CSSValue(payload.animationName);
        this._keyframes = payload.keyframes.map(keyframeRule => new CSSKeyframeRule(cssModel, keyframeRule));
    }
    /**
     * @return {!CSSValue}
     */
    name() {
        return this._animationName;
    }
    /**
     * @return {!Array.<!CSSKeyframeRule>}
     */
    keyframes() {
        return this._keyframes;
    }
}
/**
 * @unrestricted
 */
class CSSKeyframeRule extends CSSRule {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSKeyframeRule} payload
     */
    constructor(cssModel, payload) {
        // TODO(crbug.com/1011811): Replace with spread operator or better types once Closure is gone.
        super(cssModel, { origin: payload.origin, style: payload.style, styleSheetId: payload.styleSheetId });
        /** @type {!CSSValue} */
        this._keyText;
        this._reinitializeKey(payload.keyText);
    }
    /**
     * @return {!CSSValue}
     */
    key() {
        return this._keyText;
    }
    /**
     * @param {!Protocol.CSS.Value} payload
     */
    _reinitializeKey(payload) {
        this._keyText = new CSSValue(payload);
    }
    /**
     * @override
     * @param {!Edit} edit
     */
    rebase(edit) {
        if (this.styleSheetId !== edit.styleSheetId || !this._keyText.range) {
            return;
        }
        if (edit.oldRange.equal(this._keyText.range)) {
            this._reinitializeKey(/** @type {!Protocol.CSS.Value} */ (edit.payload));
        }
        else {
            this._keyText.rebase(edit);
        }
        super.rebase(edit);
    }
    /**
     * @param {string} newKeyText
     * @return {!Promise.<boolean>}
     */
    setKeyText(newKeyText) {
        const styleSheetId = this.styleSheetId;
        if (!styleSheetId) {
            throw 'No rule stylesheet id';
        }
        const range = this._keyText.range;
        if (!range) {
            throw 'Keyframe key is not editable';
        }
        return this._cssModel.setKeyframeKey(styleSheetId, range, newKeyText);
    }
}

var CSSRule$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSRule: CSSRule,
    CSSStyleRule: CSSStyleRule,
    CSSKeyframesRule: CSSKeyframesRule,
    CSSKeyframeRule: CSSKeyframeRule
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class CSSMatchedStyles {
    /**
     * @param {!CSSModel} cssModel
     * @param {!DOMNode} node
     * @param {?Protocol.CSS.CSSStyle} inlinePayload
     * @param {?Protocol.CSS.CSSStyle} attributesPayload
     * @param {!Array.<!Protocol.CSS.RuleMatch>} matchedPayload
     * @param {!Array.<!Protocol.CSS.PseudoElementMatches>} pseudoPayload
     * @param {!Array.<!Protocol.CSS.InheritedStyleEntry>} inheritedPayload
     * @param {!Array.<!Protocol.CSS.CSSKeyframesRule>} animationsPayload
     */
    constructor(cssModel, node, inlinePayload, attributesPayload, matchedPayload, pseudoPayload, inheritedPayload, animationsPayload) {
        this._cssModel = cssModel;
        this._node = node;
        /** @type {!Map<!CSSStyleDeclaration, !DOMNode>} */
        this._addedStyles = new Map();
        /** @type {!Map<!Protocol.DOM.NodeId, !Map<string, boolean>>} */
        this._matchingSelectors = new Map();
        /** @type {!Array.<!CSSKeyframesRule>} */
        this._keyframes = [];
        if (animationsPayload) {
            this._keyframes = animationsPayload.map(rule => new CSSKeyframesRule(cssModel, rule));
        }
        /** @type {!Map<!CSSStyleDeclaration, ?DOMNode>} */
        this._nodeForStyle = new Map();
        /** @type {!Set<!CSSStyleDeclaration>} */
        this._inheritedStyles = new Set();
        matchedPayload = cleanUserAgentPayload(matchedPayload);
        for (const inheritedResult of inheritedPayload) {
            inheritedResult.matchedCSSRules = cleanUserAgentPayload(inheritedResult.matchedCSSRules);
        }
        this._mainDOMCascade = this._buildMainCascade(inlinePayload, attributesPayload, matchedPayload, inheritedPayload);
        this._pseudoDOMCascades = this._buildPseudoCascades(pseudoPayload);
        /** @type {!Map<!CSSStyleDeclaration, !DOMInheritanceCascade>} */
        this._styleToDOMCascade = new Map();
        for (const domCascade of Array.from(this._pseudoDOMCascades.values()).concat(this._mainDOMCascade)) {
            for (const style of domCascade.styles()) {
                this._styleToDOMCascade.set(style, domCascade);
            }
        }
        /**
         * @param {!Array<!Protocol.CSS.RuleMatch>} payload
         * @return {!Array<!Protocol.CSS.RuleMatch>}
         */
        function cleanUserAgentPayload(payload) {
            for (const ruleMatch of payload) {
                cleanUserAgentSelectors(ruleMatch);
            }
            // Merge UA rules that are sequential and have similar selector/media.
            const cleanMatchedPayload = [];
            for (const ruleMatch of payload) {
                const lastMatch = cleanMatchedPayload.peekLast();
                if (!lastMatch || ruleMatch.rule.origin !== 'user-agent' || lastMatch.rule.origin !== 'user-agent' ||
                    ruleMatch.rule.selectorList.text !== lastMatch.rule.selectorList.text ||
                    mediaText(ruleMatch) !== mediaText(lastMatch)) {
                    cleanMatchedPayload.push(ruleMatch);
                    continue;
                }
                mergeRule(ruleMatch, lastMatch);
            }
            return cleanMatchedPayload;
            /**
             * @param {!Protocol.CSS.RuleMatch} from
             * @param {!Protocol.CSS.RuleMatch} to
             */
            function mergeRule(from, to) {
                const shorthands = /** @type {!Map<string, string>} */ (new Map());
                const properties = /** @type {!Map<string, string>} */ (new Map());
                for (const entry of to.rule.style.shorthandEntries) {
                    shorthands.set(entry.name, entry.value);
                }
                for (const entry of to.rule.style.cssProperties) {
                    properties.set(entry.name, entry.value);
                }
                for (const entry of from.rule.style.shorthandEntries) {
                    shorthands.set(entry.name, entry.value);
                }
                for (const entry of from.rule.style.cssProperties) {
                    properties.set(entry.name, entry.value);
                }
                to.rule.style.shorthandEntries = [...shorthands.entries()].map(([name, value]) => ({ name, value }));
                to.rule.style.cssProperties = [...properties.entries()].map(([name, value]) => ({ name, value }));
            }
            /**
             * @param {!Protocol.CSS.RuleMatch} ruleMatch
             * @return {?string}
             */
            function mediaText(ruleMatch) {
                if (!ruleMatch.rule.media) {
                    return null;
                }
                return ruleMatch.rule.media.map(media => media.text).join(', ');
            }
            /**
             * @param {!Protocol.CSS.RuleMatch} ruleMatch
             */
            function cleanUserAgentSelectors(ruleMatch) {
                const { matchingSelectors, rule } = ruleMatch;
                if (rule.origin !== 'user-agent' || !matchingSelectors.length) {
                    return;
                }
                rule.selectorList.selectors = rule.selectorList.selectors.filter((item, i) => matchingSelectors.includes(i));
                rule.selectorList.text = rule.selectorList.selectors.map(item => item.text).join(', ');
                ruleMatch.matchingSelectors = matchingSelectors.map((item, i) => i);
            }
        }
    }
    /**
     * @param {?Protocol.CSS.CSSStyle} inlinePayload
     * @param {?Protocol.CSS.CSSStyle} attributesPayload
     * @param {!Array.<!Protocol.CSS.RuleMatch>} matchedPayload
     * @param {!Array.<!Protocol.CSS.InheritedStyleEntry>} inheritedPayload
     * @return {!DOMInheritanceCascade}
     */
    _buildMainCascade(inlinePayload, attributesPayload, matchedPayload, inheritedPayload) {
        /** @type {!Array<!NodeCascade>} */
        const nodeCascades = [];
        /** @type {!Array<!CSSStyleDeclaration>} */
        const nodeStyles = [];
        /**
         * @this {CSSMatchedStyles}
         */
        function addAttributesStyle() {
            if (!attributesPayload) {
                return;
            }
            const style = new CSSStyleDeclaration(this._cssModel, null, attributesPayload, Type$2.Attributes);
            this._nodeForStyle.set(style, this._node);
            nodeStyles.push(style);
        }
        // Inline style has the greatest specificity.
        if (inlinePayload && this._node.nodeType() === Node.ELEMENT_NODE) {
            const style = new CSSStyleDeclaration(this._cssModel, null, inlinePayload, Type$2.Inline);
            this._nodeForStyle.set(style, this._node);
            nodeStyles.push(style);
        }
        // Add rules in reverse order to match the cascade order.
        let addedAttributesStyle;
        for (let i = matchedPayload.length - 1; i >= 0; --i) {
            const rule = new CSSStyleRule(this._cssModel, matchedPayload[i].rule);
            if ((rule.isInjected() || rule.isUserAgent()) && !addedAttributesStyle) {
                // Show element's Style Attributes after all author rules.
                addedAttributesStyle = true;
                addAttributesStyle.call(this);
            }
            this._nodeForStyle.set(rule.style, this._node);
            nodeStyles.push(rule.style);
            this._addMatchingSelectors(this._node, rule, matchedPayload[i].matchingSelectors);
        }
        if (!addedAttributesStyle) {
            addAttributesStyle.call(this);
        }
        nodeCascades.push(new NodeCascade(this, nodeStyles, false /* isInherited */));
        // Walk the node structure and identify styles with inherited properties.
        let parentNode = this._node.parentNode;
        for (let i = 0; parentNode && inheritedPayload && i < inheritedPayload.length; ++i) {
            const inheritedStyles = [];
            const entryPayload = inheritedPayload[i];
            const inheritedInlineStyle = entryPayload.inlineStyle ?
                new CSSStyleDeclaration(this._cssModel, null, entryPayload.inlineStyle, Type$2.Inline) :
                null;
            if (inheritedInlineStyle && this._containsInherited(inheritedInlineStyle)) {
                this._nodeForStyle.set(inheritedInlineStyle, parentNode);
                inheritedStyles.push(inheritedInlineStyle);
                this._inheritedStyles.add(inheritedInlineStyle);
            }
            const inheritedMatchedCSSRules = entryPayload.matchedCSSRules || [];
            for (let j = inheritedMatchedCSSRules.length - 1; j >= 0; --j) {
                const inheritedRule = new CSSStyleRule(this._cssModel, inheritedMatchedCSSRules[j].rule);
                this._addMatchingSelectors(parentNode, inheritedRule, inheritedMatchedCSSRules[j].matchingSelectors);
                if (!this._containsInherited(inheritedRule.style)) {
                    continue;
                }
                if (containsStyle(nodeStyles, inheritedRule.style) ||
                    containsStyle(this._inheritedStyles, inheritedRule.style)) {
                    continue;
                }
                this._nodeForStyle.set(inheritedRule.style, parentNode);
                inheritedStyles.push(inheritedRule.style);
                this._inheritedStyles.add(inheritedRule.style);
            }
            parentNode = parentNode.parentNode;
            nodeCascades.push(new NodeCascade(this, inheritedStyles, true /* isInherited */));
        }
        return new DOMInheritanceCascade(nodeCascades);
        /**
         * @param {!Array<!CSSStyleDeclaration>|!Set<!CSSStyleDeclaration>} styles
         * @param {!CSSStyleDeclaration} query
         * @return {boolean}
         */
        function containsStyle(styles, query) {
            if (!query.styleSheetId || !query.range) {
                return false;
            }
            for (const style of styles) {
                if (query.styleSheetId === style.styleSheetId && style.range && query.range.equal(style.range)) {
                    return true;
                }
            }
            return false;
        }
    }
    /**
     * @param {!Array.<!Protocol.CSS.PseudoElementMatches>} pseudoPayload
     * @return {!Map<!Protocol.DOM.PseudoType, !DOMInheritanceCascade>}
     */
    _buildPseudoCascades(pseudoPayload) {
        /** @type {!Map<!Protocol.DOM.PseudoType, !DOMInheritanceCascade>} */
        const pseudoCascades = new Map();
        if (!pseudoPayload) {
            return pseudoCascades;
        }
        for (let i = 0; i < pseudoPayload.length; ++i) {
            const entryPayload = pseudoPayload[i];
            // PseudoElement nodes are not created unless "content" css property is set.
            const pseudoElement = this._node.pseudoElements().get(entryPayload.pseudoType) || null;
            const pseudoStyles = [];
            const rules = entryPayload.matches || [];
            for (let j = rules.length - 1; j >= 0; --j) {
                const pseudoRule = new CSSStyleRule(this._cssModel, rules[j].rule);
                pseudoStyles.push(pseudoRule.style);
                this._nodeForStyle.set(pseudoRule.style, pseudoElement);
                if (pseudoElement) {
                    this._addMatchingSelectors(pseudoElement, pseudoRule, rules[j].matchingSelectors);
                }
            }
            const nodeCascade = new NodeCascade(this, pseudoStyles, false /* isInherited */);
            pseudoCascades.set(entryPayload.pseudoType, new DOMInheritanceCascade([nodeCascade]));
        }
        return pseudoCascades;
    }
    /**
     * @param {!DOMNode} node
     * @param {!CSSStyleRule} rule
     * @param {!Array<number>} matchingSelectorIndices
     * @this {CSSMatchedStyles}
     */
    _addMatchingSelectors(node, rule, matchingSelectorIndices) {
        for (const matchingSelectorIndex of matchingSelectorIndices) {
            const selector = rule.selectors[matchingSelectorIndex];
            this._setSelectorMatches(node, selector.text, true);
        }
    }
    /**
     * @return {!DOMNode}
     */
    node() {
        return this._node;
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return this._cssModel;
    }
    /**
     * @param {!CSSStyleRule} rule
     * @return {boolean}
     */
    hasMatchingSelectors(rule) {
        const matchingSelectors = this.matchingSelectors(rule);
        return matchingSelectors.length > 0 && this.mediaMatches(rule.style);
    }
    /**
     * @param {!CSSStyleRule} rule
     * @return {!Array<number>}
     */
    matchingSelectors(rule) {
        const node = this.nodeForStyle(rule.style);
        if (!node || typeof node.id !== 'number') {
            return [];
        }
        const map = this._matchingSelectors.get(node.id);
        if (!map) {
            return [];
        }
        const result = [];
        for (let i = 0; i < rule.selectors.length; ++i) {
            if (map.get(rule.selectors[i].text)) {
                result.push(i);
            }
        }
        return result;
    }
    /**
     * @param {!CSSStyleRule} rule
     * @return {!Promise<?>}
     */
    recomputeMatchingSelectors(rule) {
        const node = this.nodeForStyle(rule.style);
        if (!node) {
            return Promise.resolve();
        }
        const promises = [];
        for (const selector of rule.selectors) {
            promises.push(querySelector.call(this, node, selector.text));
        }
        return Promise.all(promises);
        /**
         * @param {!DOMNode} node
         * @param {string} selectorText
         * @this {CSSMatchedStyles}
         */
        async function querySelector(node, selectorText) {
            const ownerDocument = node.ownerDocument;
            if (!ownerDocument) {
                return;
            }
            // We assume that "matching" property does not ever change during the
            // MatchedStyleResult's lifetime.
            if (typeof node.id === 'number') {
                const map = this._matchingSelectors.get(node.id);
                if (map && map.has(selectorText)) {
                    return;
                }
            }
            if (typeof ownerDocument.id !== 'number') {
                return;
            }
            const matchingNodeIds = await this._node.domModel().querySelectorAll(ownerDocument.id, selectorText);
            if (matchingNodeIds) {
                if (typeof node.id === 'number') {
                    this._setSelectorMatches(node, selectorText, matchingNodeIds.indexOf(node.id) !== -1);
                }
                else {
                    this._setSelectorMatches(node, selectorText, false);
                }
            }
        }
    }
    /**
     * @param {!CSSStyleRule} rule
     * @param {!DOMNode} node
     * @return {!Promise<?>}
     */
    addNewRule(rule, node) {
        this._addedStyles.set(rule.style, node);
        return this.recomputeMatchingSelectors(rule);
    }
    /**
     * @param {!DOMNode} node
     * @param {string} selectorText
     * @param {boolean} value
     */
    _setSelectorMatches(node, selectorText, value) {
        if (typeof node.id !== 'number') {
            return;
        }
        let map = this._matchingSelectors.get(node.id);
        if (!map) {
            map = new Map();
            this._matchingSelectors.set(node.id, map);
        }
        map.set(selectorText, value);
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {boolean}
     */
    mediaMatches(style) {
        if (!style.parentRule) {
            return true;
        }
        const media = /** @type {!CSSStyleRule} */ (style.parentRule).media;
        for (let i = 0; media && i < media.length; ++i) {
            if (!media[i].active()) {
                return false;
            }
        }
        return true;
    }
    /**
     * @return {!Array<!CSSStyleDeclaration>}
     */
    nodeStyles() {
        return this._mainDOMCascade.styles();
    }
    /**
     * @return {!Array.<!CSSKeyframesRule>}
     */
    keyframes() {
        return this._keyframes;
    }
    /**
     * @param {!Protocol.DOM.PseudoType} pseudoType
     * @return {!Array<!CSSStyleDeclaration>}
     */
    pseudoStyles(pseudoType) {
        const domCascade = this._pseudoDOMCascades.get(pseudoType);
        return domCascade ? domCascade.styles() : [];
    }
    /**
     * @return {!Set<!Protocol.DOM.PseudoType>}
     */
    pseudoTypes() {
        return new Set(this._pseudoDOMCascades.keys());
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {boolean}
     */
    _containsInherited(style) {
        const properties = style.allProperties();
        for (let i = 0; i < properties.length; ++i) {
            const property = properties[i];
            // Does this style contain non-overridden inherited property?
            if (property.activeInStyle() && cssMetadata().isPropertyInherited(property.name)) {
                return true;
            }
        }
        return false;
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {?DOMNode}
     */
    nodeForStyle(style) {
        return this._addedStyles.get(style) || this._nodeForStyle.get(style) || null;
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {!Array<string>}
     */
    availableCSSVariables(style) {
        const domCascade = this._styleToDOMCascade.get(style) || null;
        return domCascade ? domCascade.availableCSSVariables(style) : [];
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @param {string} variableName
     * @return {?string}
     */
    computeCSSVariable(style, variableName) {
        const domCascade = this._styleToDOMCascade.get(style) || null;
        return domCascade ? domCascade.computeCSSVariable(style, variableName) : null;
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @param {string} value
     * @return {?string}
     */
    computeValue(style, value) {
        const domCascade = this._styleToDOMCascade.get(style) || null;
        return domCascade ? domCascade.computeValue(style, value) : null;
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {boolean}
     */
    isInherited(style) {
        return this._inheritedStyles.has(style);
    }
    /**
     * @param {!CSSProperty} property
     * @return {?PropertyState}
     */
    propertyState(property) {
        const domCascade = this._styleToDOMCascade.get(property.ownerStyle);
        return domCascade ? domCascade.propertyState(property) : null;
    }
    resetActiveProperties() {
        this._mainDOMCascade.reset();
        for (const domCascade of this._pseudoDOMCascades.values()) {
            domCascade.reset();
        }
    }
}
class NodeCascade {
    /**
     * @param {!CSSMatchedStyles} matchedStyles
     * @param {!Array<!CSSStyleDeclaration>} styles
     * @param {boolean} isInherited
     */
    constructor(matchedStyles, styles, isInherited) {
        this._matchedStyles = matchedStyles;
        this._styles = styles;
        this._isInherited = isInherited;
        /** @type {!Map<!CSSProperty, !PropertyState>} */
        this._propertiesState = new Map();
        /** @type {!Map.<string, !CSSProperty>} */
        this._activeProperties = new Map();
    }
    _computeActiveProperties() {
        this._propertiesState.clear();
        this._activeProperties.clear();
        for (const style of this._styles) {
            const rule = style.parentRule;
            // Compute cascade for CSSStyleRules only.
            if (rule && !(rule instanceof CSSStyleRule)) {
                continue;
            }
            if (rule && !this._matchedStyles.hasMatchingSelectors(rule)) {
                continue;
            }
            for (const property of style.allProperties()) {
                // Do not pick non-inherited properties from inherited styles.
                if (this._isInherited && !cssMetadata().isPropertyInherited(property.name)) {
                    continue;
                }
                if (!property.activeInStyle()) {
                    this._propertiesState.set(property, PropertyState.Overloaded);
                    continue;
                }
                const canonicalName = cssMetadata().canonicalPropertyName(property.name);
                const activeProperty = this._activeProperties.get(canonicalName);
                if (activeProperty && (activeProperty.important || !property.important)) {
                    this._propertiesState.set(property, PropertyState.Overloaded);
                    continue;
                }
                if (activeProperty) {
                    this._propertiesState.set(activeProperty, PropertyState.Overloaded);
                }
                this._propertiesState.set(property, PropertyState.Active);
                this._activeProperties.set(canonicalName, property);
            }
        }
    }
}
class DOMInheritanceCascade {
    /**
     * @param {!Array<!NodeCascade>} nodeCascades
     */
    constructor(nodeCascades) {
        this._nodeCascades = nodeCascades;
        /** @type {!Map<!CSSProperty, !PropertyState>} */
        this._propertiesState = new Map();
        /** @type {!Map<!NodeCascade, !Map<string, string>>} */
        this._availableCSSVariables = new Map();
        /** @type {!Map<!NodeCascade, !Map<string, ?string>>} */
        this._computedCSSVariables = new Map();
        this._initialized = false;
        /** @type {!Map<!CSSStyleDeclaration, !NodeCascade>} */
        this._styleToNodeCascade = new Map();
        for (const nodeCascade of nodeCascades) {
            for (const style of nodeCascade._styles) {
                this._styleToNodeCascade.set(style, nodeCascade);
            }
        }
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @return {!Array<string>}
     */
    availableCSSVariables(style) {
        const nodeCascade = this._styleToNodeCascade.get(style);
        if (!nodeCascade) {
            return [];
        }
        this._ensureInitialized();
        const availableCSSVariables = this._availableCSSVariables.get(nodeCascade);
        if (!availableCSSVariables) {
            return [];
        }
        return Array.from(availableCSSVariables.keys());
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @param {string} variableName
     * @return {?string}
     */
    computeCSSVariable(style, variableName) {
        const nodeCascade = this._styleToNodeCascade.get(style);
        if (!nodeCascade) {
            return null;
        }
        this._ensureInitialized();
        const availableCSSVariables = this._availableCSSVariables.get(nodeCascade);
        const computedCSSVariables = this._computedCSSVariables.get(nodeCascade);
        if (!availableCSSVariables || !computedCSSVariables) {
            return null;
        }
        return this._innerComputeCSSVariable(availableCSSVariables, computedCSSVariables, variableName);
    }
    /**
     * @param {!CSSStyleDeclaration} style
     * @param {string} value
     * @return {?string}
     */
    computeValue(style, value) {
        const nodeCascade = this._styleToNodeCascade.get(style);
        if (!nodeCascade) {
            return null;
        }
        this._ensureInitialized();
        const availableCSSVariables = this._availableCSSVariables.get(nodeCascade);
        const computedCSSVariables = this._computedCSSVariables.get(nodeCascade);
        if (!availableCSSVariables || !computedCSSVariables) {
            return null;
        }
        return this._innerComputeValue(availableCSSVariables, computedCSSVariables, value);
    }
    /**
     * @param {!Map<string, string>} availableCSSVariables
     * @param {!Map<string, ?string>} computedCSSVariables
     * @param {string} variableName
     * @return {?string}
     */
    _innerComputeCSSVariable(availableCSSVariables, computedCSSVariables, variableName) {
        if (!availableCSSVariables.has(variableName)) {
            return null;
        }
        if (computedCSSVariables.has(variableName)) {
            return computedCSSVariables.get(variableName) || null;
        }
        // Set dummy value to avoid infinite recursion.
        computedCSSVariables.set(variableName, null);
        const definedValue = availableCSSVariables.get(variableName);
        if (definedValue === undefined || definedValue === null) {
            return null;
        }
        const computedValue = this._innerComputeValue(availableCSSVariables, computedCSSVariables, definedValue);
        computedCSSVariables.set(variableName, computedValue);
        return computedValue;
    }
    /**
     * @param {!Map<string, string>} availableCSSVariables
     * @param {!Map<string, ?string>} computedCSSVariables
     * @param {string} value
     * @return {?string}
     */
    _innerComputeValue(availableCSSVariables, computedCSSVariables, value) {
        const results = TextUtils.Utils.splitStringByRegexes(value, [VariableRegex]);
        const tokens = [];
        for (const result of results) {
            if (result.regexIndex === -1) {
                tokens.push(result.value);
                continue;
            }
            // process var() function
            const regexMatch = result.value.match(/^var\((--[a-zA-Z0-9-_]+)[,]?\s*(.*)\)$/);
            if (!regexMatch) {
                return null;
            }
            const cssVariable = regexMatch[1];
            const computedValue = this._innerComputeCSSVariable(availableCSSVariables, computedCSSVariables, cssVariable);
            if (computedValue === null && !regexMatch[2]) {
                return null;
            }
            if (computedValue === null) {
                tokens.push(regexMatch[2]);
            }
            else {
                tokens.push(computedValue);
            }
        }
        return tokens.map(token => token.trim()).join(' ');
    }
    /**
     * @return {!Array<!CSSStyleDeclaration>}
     */
    styles() {
        return Array.from(this._styleToNodeCascade.keys());
    }
    /**
     * @param {!CSSProperty} property
     * @return {?PropertyState}
     */
    propertyState(property) {
        this._ensureInitialized();
        return this._propertiesState.get(property) || null;
    }
    reset() {
        this._initialized = false;
        this._propertiesState.clear();
        this._availableCSSVariables.clear();
        this._computedCSSVariables.clear();
    }
    _ensureInitialized() {
        if (this._initialized) {
            return;
        }
        this._initialized = true;
        const activeProperties = new Map();
        for (const nodeCascade of this._nodeCascades) {
            nodeCascade._computeActiveProperties();
            for (const entry of nodeCascade._propertiesState.entries()) {
                const property = /** @type {!CSSProperty} */ (entry[0]);
                const state = /** @type {!PropertyState} */ (entry[1]);
                if (state === PropertyState.Overloaded) {
                    this._propertiesState.set(property, PropertyState.Overloaded);
                    continue;
                }
                const canonicalName = cssMetadata().canonicalPropertyName(property.name);
                if (activeProperties.has(canonicalName)) {
                    this._propertiesState.set(property, PropertyState.Overloaded);
                    continue;
                }
                activeProperties.set(canonicalName, property);
                this._propertiesState.set(property, PropertyState.Active);
            }
        }
        // If every longhand of the shorthand is not active, then the shorthand is not active too.
        for (const entry of activeProperties.entries()) {
            const canonicalName = /** @type {string} */ (entry[0]);
            const shorthandProperty = /** @type {!CSSProperty} */ (entry[1]);
            const shorthandStyle = shorthandProperty.ownerStyle;
            const longhands = shorthandStyle.longhandProperties(shorthandProperty.name);
            if (!longhands.length) {
                continue;
            }
            let hasActiveLonghands = false;
            for (const longhand of longhands) {
                const longhandCanonicalName = cssMetadata().canonicalPropertyName(longhand.name);
                const longhandActiveProperty = activeProperties.get(longhandCanonicalName);
                if (!longhandActiveProperty) {
                    continue;
                }
                if (longhandActiveProperty.ownerStyle === shorthandStyle) {
                    hasActiveLonghands = true;
                    break;
                }
            }
            if (hasActiveLonghands) {
                continue;
            }
            activeProperties.delete(canonicalName);
            this._propertiesState.set(shorthandProperty, PropertyState.Overloaded);
        }
        // Work inheritance chain backwards to compute visible CSS Variables.
        const accumulatedCSSVariables = new Map();
        for (let i = this._nodeCascades.length - 1; i >= 0; --i) {
            const nodeCascade = this._nodeCascades[i];
            const variableNames = [];
            for (const entry of nodeCascade._activeProperties.entries()) {
                const propertyName = /** @type {string} */ (entry[0]);
                const property = /** @type {!CSSProperty} */ (entry[1]);
                if (propertyName.startsWith('--')) {
                    accumulatedCSSVariables.set(propertyName, property.value);
                    variableNames.push(propertyName);
                }
            }
            const availableCSSVariablesMap = new Map(accumulatedCSSVariables);
            const computedVariablesMap = new Map();
            this._availableCSSVariables.set(nodeCascade, availableCSSVariablesMap);
            this._computedCSSVariables.set(nodeCascade, computedVariablesMap);
            for (const variableName of variableNames) {
                accumulatedCSSVariables.delete(variableName);
                accumulatedCSSVariables.set(variableName, this._innerComputeCSSVariable(availableCSSVariablesMap, computedVariablesMap, variableName));
            }
        }
    }
}
/** @enum {string} */
const PropertyState = {
    Active: 'Active',
    Overloaded: 'Overloaded'
};

var CSSMatchedStyles$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSMatchedStyles: CSSMatchedStyles,
    PropertyState: PropertyState
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @implements {TextUtils.ContentProvider.ContentProvider}
 * TODO(chromium:1011811): make `implements {FrameAssociated}` annotation work here.
 */
class CSSStyleSheetHeader {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Protocol.CSS.CSSStyleSheetHeader} payload
     */
    constructor(cssModel, payload) {
        this._cssModel = cssModel;
        this.id = payload.styleSheetId;
        this.frameId = payload.frameId;
        this.sourceURL = payload.sourceURL;
        this.hasSourceURL = !!payload.hasSourceURL;
        this.origin = payload.origin;
        this.title = payload.title;
        this.disabled = payload.disabled;
        this.isInline = payload.isInline;
        this.isMutable = payload.isMutable;
        // TODO(alexrudenko): Needs a roll of the browser_protocol.pdl.
        this.isConstructed = /** @type {*} */ (payload).isConstructed;
        this.startLine = payload.startLine;
        this.startColumn = payload.startColumn;
        this.endLine = payload.endLine;
        this.endColumn = payload.endColumn;
        this.contentLength = payload.length;
        if (payload.ownerNode) {
            this.ownerNode = new DeferredDOMNode(cssModel.target(), payload.ownerNode);
        }
        this.sourceMapURL = payload.sourceMapURL;
        this._originalContentProvider = null;
    }
    /**
     * @return {!TextUtils.ContentProvider.ContentProvider}
     */
    originalContentProvider() {
        if (!this._originalContentProvider) {
            const lazyContent = /** @type {function():!Promise<!TextUtils.ContentProvider.DeferredContent>} */ (async () => {
                const originalText = await this._cssModel.originalStyleSheetText(this);
                // originalText might be an empty string which should not trigger the error
                if (originalText === null) {
                    return { content: null, error: ls `Could not find the original style sheet.`, isEncoded: false };
                }
                return { content: originalText, isEncoded: false };
            });
            this._originalContentProvider =
                new StaticContentProvider.StaticContentProvider(this.contentURL(), this.contentType(), lazyContent);
        }
        return this._originalContentProvider;
    }
    /**
     * @param {string=} sourceMapURL
     */
    setSourceMapURL(sourceMapURL) {
        this.sourceMapURL = sourceMapURL;
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return this._cssModel;
    }
    /**
     * @return {boolean}
     */
    isAnonymousInlineStyleSheet() {
        return !this.resourceURL() && !this._cssModel.sourceMapManager().sourceMapForClient(this);
    }
    /**
     * @return {string}
     */
    resourceURL() {
        return this.isViaInspector() ? this._viaInspectorResourceURL() : this.sourceURL;
    }
    /**
     * @return {string}
     */
    _viaInspectorResourceURL() {
        const frame = this._cssModel.target().model(ResourceTreeModel).frameForId(this.frameId);
        console.assert(frame);
        const parsedURL = new ParsedURL.ParsedURL(frame.url);
        let fakeURL = 'inspector://' + parsedURL.host + parsedURL.folderPathComponents;
        if (!fakeURL.endsWith('/')) {
            fakeURL += '/';
        }
        fakeURL += 'inspector-stylesheet';
        return fakeURL;
    }
    /**
     * @param {number} lineNumberInStyleSheet
     * @return {number}
     */
    lineNumberInSource(lineNumberInStyleSheet) {
        return this.startLine + lineNumberInStyleSheet;
    }
    /**
     * @param {number} lineNumberInStyleSheet
     * @param {number} columnNumberInStyleSheet
     * @return {number|undefined}
     */
    columnNumberInSource(lineNumberInStyleSheet, columnNumberInStyleSheet) {
        return (lineNumberInStyleSheet ? 0 : this.startColumn) + columnNumberInStyleSheet;
    }
    /**
     * Checks whether the position is in this style sheet. Assumes that the
     * position's columnNumber is consistent with line endings.
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {boolean}
     */
    containsLocation(lineNumber, columnNumber) {
        const afterStart = (lineNumber === this.startLine && columnNumber >= this.startColumn) || lineNumber > this.startLine;
        const beforeEnd = lineNumber < this.endLine || (lineNumber === this.endLine && columnNumber <= this.endColumn);
        return afterStart && beforeEnd;
    }
    /**
     * @override
     * @return {string}
     */
    contentURL() {
        return this.resourceURL();
    }
    /**
     * @override
     * @return {!Common.ResourceType.ResourceType}
     */
    contentType() {
        return ResourceType.resourceTypes.Stylesheet;
    }
    /**
     * @override
     * @return {!Promise<boolean>}
     */
    contentEncoded() {
        return Promise.resolve(false);
    }
    /**
     * @override
     * @return {!Promise<!TextUtils.ContentProvider.DeferredContent>}
     */
    async requestContent() {
        try {
            const cssText = await this._cssModel.getStyleSheetText(this.id);
            return { content: /** @type{string} */ (cssText), isEncoded: false };
        }
        catch (err) {
            return {
                content: null,
                error: ls `There was an error retrieving the source styles.`,
                isEncoded: false,
            };
        }
    }
    /**
     * @override
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    async searchInContent(query, caseSensitive, isRegex) {
        const requestedContent = await this.requestContent();
        if (requestedContent.content === null) {
            return [];
        }
        return TextUtils.performSearchInContent(requestedContent.content, query, caseSensitive, isRegex);
    }
    /**
     * @return {boolean}
     */
    isViaInspector() {
        return this.origin === 'inspector';
    }
    /**
     * @returns {!PageResourceLoadInitiator}
     */
    createPageResourceLoadInitiator() {
        return { target: null, frameId: this.frameId, initiatorUrl: this.hasSourceURL ? '' : this.sourceURL };
    }
}

var CSSStyleSheetHeader$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSStyleSheetHeader: CSSStyleSheetHeader
});

// Copyright 2020 The Chromium Authors. All rights reserved.
/** @type {?FrameManager} */
let frameManagerInstance = null;
/**
 * The FrameManager is a central storage for all frames. It collects frames from all
 * ResourceTreeModel-instances (one per target), so that frames can be found by id
 * without needing to know their target.
 * @implements {SDKModelObserver<!ResourceTreeModel>}
 */
class FrameManager extends ObjectWrapper.ObjectWrapper {
    constructor() {
        super();
        /** @type {!WeakMap<!ResourceTreeModel, !Array<!Common.EventTarget.EventDescriptor>>} */
        this._eventListeners = new WeakMap();
        TargetManager.instance().observeModels(ResourceTreeModel, this);
        // Maps frameIds to frames and a count of how many ResourceTreeModels contain this frame.
        // (OOPIFs are first attached to a new target and then detached from their old target,
        // therefore being contained in 2 models for a short period of time.)
        /** @type {!Map<string, {frame: !ResourceTreeFrame, count: number}>} */
        this._frames = new Map();
        // Maps targetIds to a set of frameIds.
        /** @type {!Map<string, !Set<string>>} */
        this._framesForTarget = new Map();
        /** @type {?ResourceTreeFrame} */
        this._topFrame = null;
    }
    /**
     * @param {{forceNew: boolean}} opts
     * @return {!FrameManager}
     */
    static instance({ forceNew } = { forceNew: false }) {
        if (!frameManagerInstance || forceNew) {
            frameManagerInstance = new FrameManager();
        }
        return frameManagerInstance;
    }
    /**
     * @override
     * @param {!ResourceTreeModel} resourceTreeModel
     */
    modelAdded(resourceTreeModel) {
        const addListener = resourceTreeModel.addEventListener(Events$a.FrameAdded, this._frameAdded, this);
        const detachListener = resourceTreeModel.addEventListener(Events$a.FrameDetached, this._frameDetached, this);
        const navigatedListener = resourceTreeModel.addEventListener(Events$a.FrameNavigated, this._frameNavigated, this);
        const resourceAddedListener = resourceTreeModel.addEventListener(Events$a.ResourceAdded, this._resourceAdded, this);
        this._eventListeners.set(resourceTreeModel, [addListener, detachListener, navigatedListener, resourceAddedListener]);
        this._framesForTarget.set(resourceTreeModel.target().id(), new Set());
    }
    /**
     * @override
     * @param {!ResourceTreeModel} resourceTreeModel
     */
    modelRemoved(resourceTreeModel) {
        const listeners = this._eventListeners.get(resourceTreeModel);
        if (listeners) {
            EventTarget.EventTarget.removeEventListeners(listeners);
        }
        // Iterate over this model's frames and decrease their count or remove them.
        // (The ResourceTreeModel does not send FrameDetached events when a model
        // is removed.)
        const frameSet = this._framesForTarget.get(resourceTreeModel.target().id());
        if (frameSet) {
            for (const frameId of frameSet) {
                this._decreaseOrRemoveFrame(frameId);
            }
        }
        this._framesForTarget.delete(resourceTreeModel.target().id());
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameAdded(event) {
        const frame = /** @type {!ResourceTreeFrame} */ (event.data);
        const frameData = this._frames.get(frame.id);
        // If the frame is already in the map, increase its count, otherwise add it to the map.
        if (frameData) {
            this._frames.set(frame.id, { frame, count: frameData.count + 1 });
        }
        else {
            this._frames.set(frame.id, { frame, count: 1 });
        }
        this._resetTopFrame();
        // Add the frameId to the the targetId's set of frameIds.
        const frameSet = this._framesForTarget.get(frame.resourceTreeModel().target().id());
        if (frameSet) {
            frameSet.add(frame.id);
        }
        this.dispatchEventToListeners(Events$3.FrameAddedToTarget, { frame });
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameDetached(event) {
        const frame = /** @type {!ResourceTreeFrame} */ (event.data);
        // Decrease the frame's count or remove it entirely from the map.
        this._decreaseOrRemoveFrame(frame.id);
        // Remove the frameId from the target's set of frameIds.
        const frameSet = this._framesForTarget.get(frame.resourceTreeModel().target().id());
        if (frameSet) {
            frameSet.delete(frame.id);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _frameNavigated(event) {
        const frame = /** @type {!ResourceTreeFrame} */ (event.data);
        this.dispatchEventToListeners(Events$3.FrameNavigated, { frame });
        if (frame.isTopFrame()) {
            this.dispatchEventToListeners(Events$3.TopFrameNavigated, { frame });
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _resourceAdded(event) {
        const resource = /** @type {!Resource} */ (event.data);
        this.dispatchEventToListeners(Events$3.ResourceAdded, { resource });
    }
    /**
     * @param {string} frameId
     */
    _decreaseOrRemoveFrame(frameId) {
        const frameData = this._frames.get(frameId);
        if (frameData) {
            if (frameData.count === 1) {
                this._frames.delete(frameId);
                this._resetTopFrame();
                this.dispatchEventToListeners(Events$3.FrameRemoved, { frameId });
            }
            else {
                frameData.count--;
            }
        }
    }
    /**
     * Looks for the top frame in `_frames` and sets `_topFrame` accordingly.
     *
     * Important: This method needs to be called everytime `_frames` is updated.
     */
    _resetTopFrame() {
        const topFrames = this.getAllFrames().filter(frame => frame.isTopFrame());
        this._topFrame = topFrames.length > 0 ? topFrames[0] : null;
    }
    /**
     * Returns the ResourceTreeFrame with a given frameId.
     * When a frame is being detached a new ResourceTreeFrame but with the same
     * frameId is created. Consequently getFrame() will return a different
     * ResourceTreeFrame after detachment. Callers of getFrame() should therefore
     * immediately use the function return value and not store it for later use.
     * @param {string} frameId
     * @return {?ResourceTreeFrame}
     */
    getFrame(frameId) {
        const frameData = this._frames.get(frameId);
        if (frameData) {
            return frameData.frame;
        }
        return null;
    }
    /**
     * @return {!Array<!ResourceTreeFrame>}
     */
    getAllFrames() {
        return Array.from(this._frames.values(), frameData => frameData.frame);
    }
    /**
     * @return {?ResourceTreeFrame}
     */
    getTopFrame() {
        return this._topFrame;
    }
}
/** @enum {symbol} */
const Events$3 = {
    // The FrameAddedToTarget event is sent whenever a frame is added to a target.
    // This means that for OOPIFs it is sent twice: once when it's added to a
    // parent frame and a second time when it's added to its own frame.
    FrameAddedToTarget: Symbol('FrameAddedToTarget'),
    FrameNavigated: Symbol('FrameNavigated'),
    // The FrameRemoved event is only sent when a frame has been detached from
    // all targets.
    FrameRemoved: Symbol('FrameRemoved'),
    ResourceAdded: Symbol('ResourceAdded'),
    TopFrameNavigated: Symbol('TopFrameNavigated'),
};

var FrameManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    FrameManager: FrameManager,
    Events: Events$3
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class IOModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
    }
    /**
     * @param {!Protocol.IO.StreamHandle} handle
     * @param {number=} size
     * @param {number=} offset
     * @return {!Promise<string|!ArrayBuffer|null>}
     * @throws {!Error}
     */
    async read(handle, size, offset) {
        const result = await this.target().ioAgent().invoke_read({ handle, offset, size });
        if (result.getError()) {
            throw new Error(result.getError());
        }
        if (result.eof) {
            return null;
        }
        if (result.base64Encoded) {
            return Base64.decode(result.data);
        }
        return result.data;
    }
    /**
     * @param {!Protocol.IO.StreamHandle} handle
     */
    async close(handle) {
        const result = await this.target().ioAgent().invoke_close({ handle });
        if (result.getError()) {
            console.error('Could not close stream.');
        }
    }
    /**
     * @param {!Protocol.Runtime.RemoteObjectId|!RemoteObject} objectOrObjectId
     * @returns {!Promise<!Protocol.IO.StreamHandle>}
     * @throws {!Error}
     */
    async resolveBlob(objectOrObjectId) {
        const objectId = objectOrObjectId instanceof RemoteObject ? objectOrObjectId.objectId : objectOrObjectId;
        if (!objectId) {
            throw new Error('Remote object has undefined objectId');
        }
        const result = await this.target().ioAgent().invoke_resolveBlob({ objectId });
        if (result.getError()) {
            throw new Error(result.getError());
        }
        return `blob:${result.uuid}`;
    }
    /**
     * @param {!Protocol.IO.StreamHandle} handle
     * @throws {!Error}
     */
    async readToString(handle) {
        /** @type {!Array<string>} */
        const strings = [];
        const decoder = new TextDecoder();
        for (;;) {
            const data = await this.read(handle, 1024 * 1024);
            if (data === null) {
                strings.push(decoder.decode());
                break;
            }
            if (data instanceof ArrayBuffer) {
                strings.push(decoder.decode(data, { stream: true }));
            }
            else {
                strings.push(data);
            }
        }
        return strings.join('');
    }
}
SDKModel.register(IOModel, Capability.IO, true);

var IOModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IOModel: IOModel
});

// Copyright 2020 The Chromium Authors. All rights reserved.
/** @typedef {{target: null, frameId: Protocol.Page.FrameId, initiatorUrl: ?string}|{target: Target, frameId: ?Protocol.Page.FrameId, initiatorUrl: ?string}} */
// @ts-ignore typedef
let PageResourceLoadInitiator; // eslint-disable-line no-unused-vars
/** @typedef {{success: ?boolean, errorMessage: (undefined|string), initiator: !PageResourceLoadInitiator, url: string, size: ?number}} */
// @ts-ignore typedef
let PageResource; // eslint-disable-line no-unused-vars
/** @type {?PageResourceLoader} */
let pageResourceLoader = null;
/**
 * The page resource loader is a bottleneck for all DevTools-initiated resource loads. For each such load, it keeps a
 * `PageResource` object around that holds meta information. This can be as the basis for reporting to the user which
 * resources were loaded, and whether there was a load error.
 */
class PageResourceLoader extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {null|function(string):!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>} loadOverride
     * @param {number} maxConcurrentLoads - Maximal number of concurrently dispatched loads. If this limit is reached, additional loads are queued up.
     * @param {number} loadTimeout - Timeout for the back-end of this loader. If the timeout is reached, the load promise is rejected and the result from the back-end gets discarded.
     */
    constructor(loadOverride, maxConcurrentLoads, loadTimeout) {
        super();
        this._currentlyLoading = 0;
        this._maxConcurrentLoads = maxConcurrentLoads;
        /** @type{!Map<string, !PageResource>} */
        this._pageResources = new Map();
        /** @type {!Array<!{resolve:function(*):void, reject:function(*):void}>} */
        this._queuedLoads = [];
        TargetManager.instance().addModelListener(ResourceTreeModel, Events$a.MainFrameNavigated, this._onMainFrameNavigated, this);
        this._loadOverride = loadOverride;
        this._loadTimeout = loadTimeout;
    }
    /**
     * @param {{forceNew: boolean, loadOverride:(null|function(string):!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>), maxConcurrentLoads: number, loadTimeout: number}} _
     */
    static instance({ forceNew, loadOverride, maxConcurrentLoads, loadTimeout } = {
        forceNew: false,
        loadOverride: null,
        maxConcurrentLoads: 500,
        loadTimeout: 30000
    }) {
        if (!pageResourceLoader || forceNew) {
            pageResourceLoader = new PageResourceLoader(loadOverride, maxConcurrentLoads, loadTimeout);
        }
        return pageResourceLoader;
    }
    /**
     *
     * @param {*} event
     */
    _onMainFrameNavigated(event) {
        const mainFrame = /** @type {!ResourceTreeFrame} */ (event.data);
        if (!mainFrame.isTopFrame()) {
            return;
        }
        for (const { reject } of this._queuedLoads) {
            reject(new Error(ls$1 `Load canceled due to reload of inspected page`));
        }
        this._queuedLoads = [];
        this._pageResources.clear();
        this.dispatchEventToListeners(Events$4.Update);
    }
    getResourcesLoaded() {
        return this._pageResources;
    }
    /**
     * Loading is the number of currently loading and queued items. Resources is the total number of resources,
     * including loading and queued resources, but not including resources that are still loading but scheduled
     * for cancelation.
     * @returns {!{loading: number, queued: number, resources: number}};
     */
    getNumberOfResources() {
        return { loading: this._currentlyLoading, queued: this._queuedLoads.length, resources: this._pageResources.size };
    }
    async _acquireLoadSlot() {
        this._currentlyLoading++;
        if (this._currentlyLoading > this._maxConcurrentLoads) {
            /** @type {!{resolve:function(*):void, reject:function(*):void}} */
            const pair = { resolve: () => { }, reject: () => { } };
            const waitForCapacity = new Promise((resolve, reject) => {
                pair.resolve = resolve;
                pair.reject = reject;
            });
            this._queuedLoads.push(pair);
            await waitForCapacity;
        }
    }
    _releaseLoadSlot() {
        this._currentlyLoading--;
        const entry = this._queuedLoads.shift();
        if (entry) {
            entry.resolve(undefined);
        }
    }
    /**
     * @template T
     * @param {!Promise<T>} promise
     * @param {number} timeout
     */
    static async _withTimeout(promise, timeout) {
        /** @type {!Promise<T>} */
        const timeoutPromise = new Promise((_, reject) => setTimeout(reject, timeout, new Error(ls$1 `Load canceled due to load timeout`)));
        return Promise.race([promise, timeoutPromise]);
    }
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @returns {string}
     */
    static makeKey(url, initiator) {
        if (initiator.frameId) {
            return `${url}-${initiator.frameId}`;
        }
        if (initiator.target) {
            return `${url}-${initiator.target.id()}`;
        }
        throw new Error('Invalid initiator');
    }
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @return {!Promise<!{content: string}>}
     */
    async loadResource(url, initiator) {
        const key = PageResourceLoader.makeKey(url, initiator);
        /** @type {!PageResource} */
        const pageResource = { success: null, size: null, errorMessage: undefined, url, initiator };
        this._pageResources.set(key, pageResource);
        this.dispatchEventToListeners(Events$4.Update);
        try {
            await this._acquireLoadSlot();
            const resultPromise = this._dispatchLoad(url, initiator);
            const result = await PageResourceLoader._withTimeout(resultPromise, this._loadTimeout);
            pageResource.errorMessage = result.errorDescription.message;
            pageResource.success = result.success;
            if (result.success) {
                pageResource.size = result.content.length;
                return { content: result.content };
            }
            throw new Error(result.errorDescription.message);
        }
        catch (e) {
            if (pageResource.errorMessage === undefined) {
                pageResource.errorMessage = e.message;
            }
            if (pageResource.success === null) {
                pageResource.success = false;
            }
            throw e;
        }
        finally {
            this._releaseLoadSlot();
            this.dispatchEventToListeners(Events$4.Update);
        }
    }
    /**
     * @param {string} url
     * @param {!PageResourceLoadInitiator} initiator
     * @return {!Promise<!{success: boolean, content: string, errorDescription: !Host.ResourceLoader.LoadErrorDescription}>}
     */
    async _dispatchLoad(url, initiator) {
        /** @type {string|null} */
        let failureReason = null;
        if (this._loadOverride) {
            return this._loadOverride(url);
        }
        const parsedURL = ParsedURL.ParsedURL.fromString(url);
        if (getLoadThroughTargetSetting().get() && parsedURL && parsedURL.isHttpOrHttps()) {
            try {
                if (initiator.target) {
                    const result = await this._loadFromTarget(initiator.target, initiator.frameId, url);
                    return result;
                }
                const frame = FrameManager.instance().getFrame(initiator.frameId || '');
                if (frame) {
                    const result = await this._loadFromTarget(frame.resourceTreeModel().target(), initiator.frameId, url);
                    return result;
                }
            }
            catch (e) {
                if (e instanceof Error) {
                    failureReason = e.message;
                }
            }
            console.warn('Fallback triggered', url, initiator);
        }
        const result = await MultitargetNetworkManager.instance().loadResource(url);
        if (failureReason) {
            // In case we have a success, add a note about why the load through the target failed.
            result.errorDescription.message =
                `Fetch through target failed: ${failureReason}; Fallback: ${result.errorDescription.message}`;
        }
        return result;
    }
    /**
     * @param {!Target} target
     * @param {?Protocol.Page.FrameId} frameId
     * @param {string} url
     */
    async _loadFromTarget(target, frameId, url) {
        const networkManager = /** @type {!NetworkManager} */ (target.model(NetworkManager));
        const ioModel = /** @type {!IOModel} */ (target.model(IOModel));
        const resource = await networkManager.loadNetworkResource(frameId || '', url, { disableCache: true, includeCredentials: true });
        try {
            const content = resource.stream ? await ioModel.readToString(resource.stream) : '';
            return {
                success: resource.success,
                content,
                errorDescription: {
                    statusCode: resource.httpStatusCode || 0,
                    netError: resource.netError,
                    netErrorName: resource.netErrorName,
                    message: ResourceLoader.netErrorToMessage(resource.netError, resource.httpStatusCode, resource.netErrorName) ||
                        '',
                    urlValid: undefined
                }
            };
        }
        finally {
            if (resource.stream) {
                ioModel.close(resource.stream);
            }
        }
    }
}
/** @return {!Common.Settings.Setting<boolean>} */
function getLoadThroughTargetSetting() {
    return Settings.Settings.instance().createSetting('loadThroughTarget', true);
}
/** @enum {symbol} */
const Events$4 = {
    Update: Symbol('Update')
};

var PageResourceLoader$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    PageResourceLoadInitiator: PageResourceLoadInitiator,
    PageResource: PageResource,
    PageResourceLoader: PageResourceLoader,
    getLoadThroughTargetSetting: getLoadThroughTargetSetting,
    Events: Events$4
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @implements {TextUtils.ContentProvider.ContentProvider}
 * @unrestricted
 */
class CompilerSourceMappingContentProvider {
    /**
     * @param {string} sourceURL
     * @param {!Common.ResourceType.ResourceType} contentType
     * @param {!PageResourceLoadInitiator} initiator
     */
    constructor(sourceURL, contentType, initiator) {
        this._sourceURL = sourceURL;
        this._contentType = contentType;
        this._initiator = initiator;
    }
    /**
     * @override
     * @return {string}
     */
    contentURL() {
        return this._sourceURL;
    }
    /**
     * @override
     * @return {!Common.ResourceType.ResourceType}
     */
    contentType() {
        return this._contentType;
    }
    /**
     * @override
     * @return {!Promise<boolean>}
     */
    contentEncoded() {
        return Promise.resolve(false);
    }
    /**
     * @override
     * @return {!Promise<!TextUtils.ContentProvider.DeferredContent>}
     */
    async requestContent() {
        try {
            const { content } = await PageResourceLoader.instance().loadResource(this._sourceURL, this._initiator);
            return { content, isEncoded: false };
        }
        catch (e) {
            const error = ls `Could not load content for ${this._sourceURL} (${e.message})`;
            console.error(error);
            return { content: null, error, isEncoded: false };
        }
    }
    /**
     * @override
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    async searchInContent(query, caseSensitive, isRegex) {
        const { content } = await this.requestContent();
        if (typeof content !== 'string') {
            return [];
        }
        return TextUtils.performSearchInContent(content, query, caseSensitive, isRegex);
    }
}

var CompilerSourceMappingContentProvider$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CompilerSourceMappingContentProvider: CompilerSourceMappingContentProvider
});

let wasm;

const heap = new Array(32).fill(undefined);

heap.push(undefined, null, true, false);

function getObject(idx) { return heap[idx]; }

let heap_next = heap.length;

function dropObject(idx) {
    if (idx < 36) return;
    heap[idx] = heap_next;
    heap_next = idx;
}

function takeObject(idx) {
    const ret = getObject(idx);
    dropObject(idx);
    return ret;
}

function addHeapObject(obj) {
    if (heap_next === heap.length) heap.push(heap.length + 1);
    const idx = heap_next;
    heap_next = heap[idx];

    heap[idx] = obj;
    return idx;
}

let cachedTextDecoder = new TextDecoder('utf-8', { ignoreBOM: true, fatal: true });

cachedTextDecoder.decode();

let cachegetUint8Memory0 = null;
function getUint8Memory0() {
    if (cachegetUint8Memory0 === null || cachegetUint8Memory0.buffer !== wasm.memory.buffer) {
        cachegetUint8Memory0 = new Uint8Array(wasm.memory.buffer);
    }
    return cachegetUint8Memory0;
}

function getStringFromWasm0(ptr, len) {
    return cachedTextDecoder.decode(getUint8Memory0().subarray(ptr, ptr + len));
}

let WASM_VECTOR_LEN = 0;

function passArray8ToWasm0(arg, malloc) {
    const ptr = malloc(arg.length * 1);
    getUint8Memory0().set(arg, ptr / 1);
    WASM_VECTOR_LEN = arg.length;
    return ptr;
}

let cachedTextEncoder = new TextEncoder('utf-8');

const encodeString = (typeof cachedTextEncoder.encodeInto === 'function'
    ? function (arg, view) {
    return cachedTextEncoder.encodeInto(arg, view);
}
    : function (arg, view) {
    const buf = cachedTextEncoder.encode(arg);
    view.set(buf);
    return {
        read: arg.length,
        written: buf.length
    };
});

function passStringToWasm0(arg, malloc, realloc) {

    if (realloc === undefined) {
        const buf = cachedTextEncoder.encode(arg);
        const ptr = malloc(buf.length);
        getUint8Memory0().subarray(ptr, ptr + buf.length).set(buf);
        WASM_VECTOR_LEN = buf.length;
        return ptr;
    }

    let len = arg.length;
    let ptr = malloc(len);

    const mem = getUint8Memory0();

    let offset = 0;

    for (; offset < len; offset++) {
        const code = arg.charCodeAt(offset);
        if (code > 0x7F) break;
        mem[ptr + offset] = code;
    }

    if (offset !== len) {
        if (offset !== 0) {
            arg = arg.slice(offset);
        }
        ptr = realloc(ptr, len, len = offset + arg.length * 3);
        const view = getUint8Memory0().subarray(ptr + offset, ptr + len);
        const ret = encodeString(arg, view);

        offset += ret.written;
    }

    WASM_VECTOR_LEN = offset;
    return ptr;
}
/**
*/
class Resolver {

    static __wrap(ptr) {
        const obj = Object.create(Resolver.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;

        wasm.__wbg_resolver_free(ptr);
    }
    /**
    * @param {Uint8Array} src
    */
    constructor(src) {
        var ptr0 = passArray8ToWasm0(src, wasm.__wbindgen_malloc);
        var len0 = WASM_VECTOR_LEN;
        var ret = wasm.resolver_from_slice(ptr0, len0);
        return Resolver.__wrap(ret);
    }
    /**
    * @returns {Array<any>}
    */
    listFiles() {
        var ret = wasm.resolver_listFiles(this.ptr);
        return takeObject(ret);
    }
    /**
    * @returns {Array<any>}
    */
    listMappings() {
        var ret = wasm.resolver_listMappings(this.ptr);
        return takeObject(ret);
    }
    /**
    * @param {number} addr
    * @returns {SourceMapEntry | undefined}
    */
    resolve(addr) {
        var ret = wasm.resolver_resolve(this.ptr, addr);
        return takeObject(ret);
    }
    /**
    * @param {string} file
    * @param {number} line
    * @param {number} column
    * @returns {SourceMapEntry | undefined}
    */
    resolveReverse(file, line, column) {
        var ptr0 = passStringToWasm0(file, wasm.__wbindgen_malloc, wasm.__wbindgen_realloc);
        var len0 = WASM_VECTOR_LEN;
        var ret = wasm.resolver_resolveReverse(this.ptr, ptr0, len0, line, column);
        return takeObject(ret);
    }
}

async function load(module, imports) {
    if (typeof Response === 'function' && module instanceof Response) {

        if (typeof WebAssembly.instantiateStreaming === 'function') {
            try {
                return await WebAssembly.instantiateStreaming(module, imports);

            } catch (e) {
                if (module.headers.get('Content-Type') != 'application/wasm') {
                    console.warn("`WebAssembly.instantiateStreaming` failed because your server does not serve wasm with `application/wasm` MIME type. Falling back to `WebAssembly.instantiate` which is slower. Original error:\n", e);

                } else {
                    throw e;
                }
            }
        }

        const bytes = await module.arrayBuffer();
        return await WebAssembly.instantiate(bytes, imports);

    } else {

        const instance = await WebAssembly.instantiate(module, imports);

        if (instance instanceof WebAssembly.Instance) {
            return { instance, module };

        } else {
            return instance;
        }
    }
}

async function init(input) {
    if (typeof input === 'undefined') {
        input = import.meta.url.replace(/\.js$/, '_bg.wasm');
    }
    const imports = {};
    imports.wbg = {};
    imports.wbg.__wbindgen_object_drop_ref = function(arg0) {
        takeObject(arg0);
    };
    imports.wbg.__wbg_new_8344aa4d8326b816 = function(arg0, arg1, arg2, arg3, arg4) {
        var ret = new SourceMapEntry(arg0 >>> 0, arg1 >>> 0, getObject(arg2), arg3 >>> 0, arg4 >>> 0);
        return addHeapObject(ret);
    };
    imports.wbg.__wbindgen_object_clone_ref = function(arg0) {
        var ret = getObject(arg0);
        return addHeapObject(ret);
    };
    imports.wbg.__wbindgen_string_new = function(arg0, arg1) {
        var ret = getStringFromWasm0(arg0, arg1);
        return addHeapObject(ret);
    };
    imports.wbg.__wbg_new_0d50725e1ae68303 = function() {
        var ret = new Array();
        return addHeapObject(ret);
    };
    imports.wbg.__wbg_push_46274b393147c746 = function(arg0, arg1) {
        var ret = getObject(arg0).push(getObject(arg1));
        return ret;
    };
    imports.wbg.__wbg_new_1d56e97b8de3067f = function(arg0, arg1) {
        var ret = new Error(getStringFromWasm0(arg0, arg1));
        return addHeapObject(ret);
    };
    imports.wbg.__wbindgen_throw = function(arg0, arg1) {
        throw new Error(getStringFromWasm0(arg0, arg1));
    };
    imports.wbg.__wbindgen_rethrow = function(arg0) {
        throw takeObject(arg0);
    };

    if (typeof input === 'string' || (typeof Request === 'function' && input instanceof Request) || (typeof URL === 'function' && input instanceof URL)) {
        input = fetch(input);
    }

    const { instance, module } = await load(await input, imports);

    wasm = instance.exports;
    init.__wbindgen_wasm_module = module;

    return wasm;
}

/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @interface
 */
class SourceMap {
    /**
     * @return {string}
     */
    compiledURL() {
        throw new Error('Not implemented');
    }
    /**
     * @return {string}
     */
    url() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array<string>}
     */
    sourceURLs() {
        throw new Error('Not implemented');
    }
    /**
     * @param {string} sourceURL
     * @param {!Common.ResourceType.ResourceType} contentType
     * @return {!TextUtils.ContentProvider.ContentProvider}
     */
    sourceContentProvider(sourceURL, contentType) {
        throw new Error('Not implemented');
    }
    /**
     * @param {string} sourceURL
     * @return {?string}
     */
    embeddedContentByURL(sourceURL) {
        throw new Error('Not implemented');
    }
    /**
     * @param {number} lineNumber in compiled resource
     * @param {number} columnNumber in compiled resource
     * @return {?SourceMapEntry}
     */
    findEntry(lineNumber, columnNumber) {
        throw new Error('Not implemented');
    }
    /**
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?SourceMapEntry}
     */
    sourceLineMapping(sourceURL, lineNumber, columnNumber) {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array<!SourceMapEntry>}
     */
    mappings() {
        throw new Error('Not implemented');
    }
    dispose() {
    }
}
// eslint-disable-next-line no-unused-vars
class SourceMapV3 {
    constructor() {
        /** @type {number} */ this.version;
        /** @type {string|undefined} */ this.file;
        /** @type {!Array.<string>} */ this.sources;
        /** @type {!Array.<!Section>|undefined} */ this.sections;
        /** @type {string} */ this.mappings;
        /** @type {string|undefined} */ this.sourceRoot;
        /** @type {!Array.<string>|undefined} */ this.names;
        /** @type {string|undefined} */ this.sourcesContent;
    }
}
// eslint-disable-next-line no-unused-vars
class Section {
    constructor() {
        /** @type {!SourceMapV3} */ this.map;
        /** @type {!Offset} */ this.offset;
        /** @type {string|undefined} */ this.url;
    }
}
// eslint-disable-next-line no-unused-vars
class Offset {
    constructor() {
        /** @type {number} */ this.line;
        /** @type {number} */ this.column;
    }
}
class SourceMapEntry {
    /**
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @param {string=} sourceURL
     * @param {number=} sourceLineNumber
     * @param {number=} sourceColumnNumber
     * @param {string=} name
     */
    constructor(lineNumber, columnNumber, sourceURL, sourceLineNumber, sourceColumnNumber, name) {
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber;
        this.sourceURL = sourceURL;
        // These two type-casts are wrong. However, when you try to fix them,
        // a lot of comparator functions defined below start to produce incorrect
        // behavior. Instead of trying to fix this, we should cast away and
        // pretend that everything is all right.
        this.sourceLineNumber = /** @type {number} */ (sourceLineNumber);
        this.sourceColumnNumber = /** @type {number} */ (sourceColumnNumber);
        this.name = name;
    }
    /**
     * @param {!SourceMapEntry} entry1
     * @param {!SourceMapEntry} entry2
     * @return {number}
     */
    static compare(entry1, entry2) {
        if (entry1.lineNumber !== entry2.lineNumber) {
            return entry1.lineNumber - entry2.lineNumber;
        }
        return entry1.columnNumber - entry2.columnNumber;
    }
}
class EditResult {
    /**
     * @param {!SourceMap} map
     * @param {!Array<!TextUtils.TextRange.SourceEdit>} compiledEdits
     * @param {!Map<string, string>} newSources
     */
    constructor(map, compiledEdits, newSources) {
        this.map = map;
        this.compiledEdits = compiledEdits;
        this.newSources = newSources;
    }
}
const base64Digits = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
/** @type {!Map<string, number>} */
const base64Map = new Map();
for (let i = 0; i < base64Digits.length; ++i) {
    base64Map.set(base64Digits.charAt(i), i);
}
/** @type {!WeakMap<!SourceMapV3, !Array<string>>} */
const sourceMapToSourceList = new WeakMap();
/**
 * @implements {SourceMap}
 */
class TextSourceMap {
    /**
     * Implements Source Map V3 model. See https://github.com/google/closure-compiler/wiki/Source-Maps
     * for format description.
     * @param {string} compiledURL
     * @param {string} sourceMappingURL
     * @param {!SourceMapV3} payload
     * @param {!PageResourceLoadInitiator} initiator
     */
    constructor(compiledURL, sourceMappingURL, payload, initiator) {
        this._initiator = initiator;
        /** @type {?SourceMapV3} */
        this._json = payload;
        this._compiledURL = compiledURL;
        this._sourceMappingURL = sourceMappingURL;
        this._baseURL = sourceMappingURL.startsWith('data:') ? compiledURL : sourceMappingURL;
        /** @type {?Array<!SourceMapEntry>} */
        this._mappings = null;
        /** @type {!Map<string, !TextSourceMap.SourceInfo>} */
        this._sourceInfos = new Map();
        if (this._json.sections) {
            const sectionWithURL = !!this._json.sections.find(section => !!section.url);
            if (sectionWithURL) {
                Console.Console.instance().warn(`SourceMap "${sourceMappingURL}" contains unsupported "URL" field in one of its sections.`);
            }
        }
        this._eachSection(this._parseSources.bind(this));
    }
    /**
     * @param {string} sourceMapURL
     * @param {string} compiledURL
     * @param {!PageResourceLoadInitiator} initiator
     * @return {!Promise<!TextSourceMap>}
     * @throws {!Error}
     * @this {TextSourceMap}
     */
    static async load(sourceMapURL, compiledURL, initiator) {
        let updatedContent;
        try {
            const { content } = await PageResourceLoader.instance().loadResource(sourceMapURL, initiator);
            updatedContent = content;
            if (content.slice(0, 3) === ')]}') {
                updatedContent = content.substring(content.indexOf('\n'));
            }
        }
        catch (error) {
            throw new Error(ls `Could not load content for ${sourceMapURL}: ${error.message}`);
        }
        try {
            const payload = /** @type {!SourceMapV3} */ (JSON.parse(updatedContent));
            return new TextSourceMap(compiledURL, sourceMapURL, payload, initiator);
        }
        catch (error) {
            throw new Error(ls `Could not parse content for ${sourceMapURL}: ${error.message}`);
        }
    }
    /**
     * @override
     * @return {string}
     */
    compiledURL() {
        return this._compiledURL;
    }
    /**
     * @override
     * @return {string}
     */
    url() {
        return this._sourceMappingURL;
    }
    /**
     * @override
     * @return {!Array.<string>}
     */
    sourceURLs() {
        return [...this._sourceInfos.keys()];
    }
    /**
     * @override
     * @param {string} sourceURL
     * @param {!Common.ResourceType.ResourceType} contentType
     * @return {!TextUtils.ContentProvider.ContentProvider}
     */
    sourceContentProvider(sourceURL, contentType) {
        const info = this._sourceInfos.get(sourceURL);
        if (info && info.content) {
            return StaticContentProvider.StaticContentProvider.fromString(sourceURL, contentType, info.content);
        }
        return new CompilerSourceMappingContentProvider(sourceURL, contentType, this._initiator);
    }
    /**
     * @override
     * @param {string} sourceURL
     * @return {?string}
     */
    embeddedContentByURL(sourceURL) {
        const entry = this._sourceInfos.get(sourceURL);
        if (!entry) {
            return null;
        }
        return entry.content;
    }
    /**
     * @override
     * @param {number} lineNumber in compiled resource
     * @param {number} columnNumber in compiled resource
     * @return {?SourceMapEntry}
     */
    findEntry(lineNumber, columnNumber) {
        const mappings = this.mappings();
        const index = mappings.upperBound(undefined, (unused, entry) => lineNumber - entry.lineNumber || columnNumber - entry.columnNumber);
        return index ? mappings[index - 1] : null;
    }
    /**
     * @override
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?SourceMapEntry}
     */
    sourceLineMapping(sourceURL, lineNumber, columnNumber) {
        const mappings = this._reversedMappings(sourceURL);
        const first = mappings.lowerBound(lineNumber, lineComparator);
        const last = mappings.upperBound(lineNumber, lineComparator);
        if (first >= mappings.length || mappings[first].sourceLineNumber !== lineNumber) {
            return null;
        }
        const columnMappings = mappings.slice(first, last);
        if (!columnMappings.length) {
            return null;
        }
        const index = columnMappings.lowerBound(columnNumber, (columnNumber, mapping) => columnNumber - mapping.sourceColumnNumber);
        return index >= columnMappings.length ? columnMappings[columnMappings.length - 1] : columnMappings[index];
        /**
         * @param {number} lineNumber
         * @param {!SourceMapEntry} mapping
         * @return {number}
         */
        function lineComparator(lineNumber, mapping) {
            return lineNumber - mapping.sourceLineNumber;
        }
    }
    /**
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {!Array<!SourceMapEntry>}
     */
    findReverseEntries(sourceURL, lineNumber, columnNumber) {
        const mappings = this._reversedMappings(sourceURL);
        const endIndex = mappings.upperBound(undefined, (unused, entry) => lineNumber - entry.sourceLineNumber || columnNumber - entry.sourceColumnNumber);
        let startIndex = endIndex;
        while (startIndex > 0 && mappings[startIndex - 1].sourceLineNumber === mappings[endIndex - 1].sourceLineNumber &&
            mappings[startIndex - 1].sourceColumnNumber === mappings[endIndex - 1].sourceColumnNumber) {
            --startIndex;
        }
        return mappings.slice(startIndex, endIndex);
    }
    /**
     * @override
     * @return {!Array<!SourceMapEntry>}
     */
    mappings() {
        if (this._mappings === null) {
            this._mappings = [];
            this._eachSection(this._parseMap.bind(this));
            this._json = null;
        }
        return /** @type {!Array<!SourceMapEntry>} */ (this._mappings);
    }
    /**
     * @param {string} sourceURL
     * @return {!Array.<!SourceMapEntry>}
     */
    _reversedMappings(sourceURL) {
        const info = this._sourceInfos.get(sourceURL);
        if (!info) {
            return [];
        }
        const mappings = this.mappings();
        if (info.reverseMappings === null) {
            info.reverseMappings = mappings.filter(mapping => mapping.sourceURL === sourceURL).sort(sourceMappingComparator);
        }
        return info.reverseMappings;
        /**
         * @param {!SourceMapEntry} a
         * @param {!SourceMapEntry} b
         * @return {number}
         */
        function sourceMappingComparator(a, b) {
            if (a.sourceLineNumber !== b.sourceLineNumber) {
                return a.sourceLineNumber - b.sourceLineNumber;
            }
            if (a.sourceColumnNumber !== b.sourceColumnNumber) {
                return a.sourceColumnNumber - b.sourceColumnNumber;
            }
            if (a.lineNumber !== b.lineNumber) {
                return a.lineNumber - b.lineNumber;
            }
            return a.columnNumber - b.columnNumber;
        }
    }
    /**
     * @param {function(!SourceMapV3, number, number):void} callback
     */
    _eachSection(callback) {
        if (!this._json) {
            return;
        }
        if (!this._json.sections) {
            callback(this._json, 0, 0);
            return;
        }
        for (const section of this._json.sections) {
            callback(section.map, section.offset.line, section.offset.column);
        }
    }
    /**
     * @param {!SourceMapV3} sourceMap
     */
    _parseSources(sourceMap) {
        const sourcesList = [];
        let sourceRoot = sourceMap.sourceRoot || '';
        if (sourceRoot && !sourceRoot.endsWith('/')) {
            sourceRoot += '/';
        }
        for (let i = 0; i < sourceMap.sources.length; ++i) {
            const href = sourceRoot + sourceMap.sources[i];
            let url = ParsedURL.ParsedURL.completeURL(this._baseURL, href) || href;
            const source = sourceMap.sourcesContent && sourceMap.sourcesContent[i];
            if (url === this._compiledURL && source) {
                url += UIString.UIString('? [sm]');
            }
            this._sourceInfos.set(url, new TextSourceMap.SourceInfo(source || null, null));
            sourcesList.push(url);
        }
        sourceMapToSourceList.set(sourceMap, sourcesList);
    }
    /**
     * @param {!SourceMapV3} map
     * @param {number} lineNumber
     * @param {number} columnNumber
     */
    _parseMap(map, lineNumber, columnNumber) {
        let sourceIndex = 0;
        let sourceLineNumber = 0;
        let sourceColumnNumber = 0;
        let nameIndex = 0;
        // TODO(crbug.com/1011811): refactor away map.
        // `sources` can be undefined if it wasn't previously
        // processed and added to the list. However, that
        // is not WAI and we should make sure that we can
        // only reach this point when we are certain
        // we have the list available.
        const sources = sourceMapToSourceList.get(map);
        const names = map.names || [];
        const stringCharIterator = new TextSourceMap.StringCharIterator(map.mappings);
        let sourceURL = sources && sources[sourceIndex];
        while (true) {
            if (stringCharIterator.peek() === ',') {
                stringCharIterator.next();
            }
            else {
                while (stringCharIterator.peek() === ';') {
                    lineNumber += 1;
                    columnNumber = 0;
                    stringCharIterator.next();
                }
                if (!stringCharIterator.hasNext()) {
                    break;
                }
            }
            columnNumber += this._decodeVLQ(stringCharIterator);
            if (!stringCharIterator.hasNext() || this._isSeparator(stringCharIterator.peek())) {
                this.mappings().push(new SourceMapEntry(lineNumber, columnNumber));
                continue;
            }
            const sourceIndexDelta = this._decodeVLQ(stringCharIterator);
            if (sourceIndexDelta) {
                sourceIndex += sourceIndexDelta;
                if (sources) {
                    sourceURL = sources[sourceIndex];
                }
            }
            sourceLineNumber += this._decodeVLQ(stringCharIterator);
            sourceColumnNumber += this._decodeVLQ(stringCharIterator);
            if (!stringCharIterator.hasNext() || this._isSeparator(stringCharIterator.peek())) {
                this.mappings().push(new SourceMapEntry(lineNumber, columnNumber, sourceURL, sourceLineNumber, sourceColumnNumber));
                continue;
            }
            nameIndex += this._decodeVLQ(stringCharIterator);
            this.mappings().push(new SourceMapEntry(lineNumber, columnNumber, sourceURL, sourceLineNumber, sourceColumnNumber, names[nameIndex]));
        }
        // As per spec, mappings are not necessarily sorted.
        this.mappings().sort(SourceMapEntry.compare);
    }
    /**
     * @param {string} char
     * @return {boolean}
     */
    _isSeparator(char) {
        return char === ',' || char === ';';
    }
    /**
     * @param {!TextSourceMap.StringCharIterator} stringCharIterator
     * @return {number}
     */
    _decodeVLQ(stringCharIterator) {
        // Read unsigned value.
        let result = 0;
        let shift = 0;
        let digit = TextSourceMap._VLQ_CONTINUATION_MASK;
        while (digit & TextSourceMap._VLQ_CONTINUATION_MASK) {
            digit = base64Map.get(stringCharIterator.next()) || 0;
            result += (digit & TextSourceMap._VLQ_BASE_MASK) << shift;
            shift += TextSourceMap._VLQ_BASE_SHIFT;
        }
        // Fix the sign.
        const negative = result & 1;
        result >>= 1;
        return negative ? -result : result;
    }
    /**
     * @param {string} url
     * @param {!TextUtils.TextRange.TextRange} textRange
     * @return {?TextUtils.TextRange.TextRange}
     */
    reverseMapTextRange(url, textRange) {
        /**
         * @param {!{lineNumber: number, columnNumber: number}} position
         * @param {!SourceMapEntry} mapping
         * @return {number}
         */
        function comparator(position, mapping) {
            if (position.lineNumber !== mapping.sourceLineNumber) {
                return position.lineNumber - mapping.sourceLineNumber;
            }
            return position.columnNumber - mapping.sourceColumnNumber;
        }
        const mappings = this._reversedMappings(url);
        if (!mappings.length) {
            return null;
        }
        const startIndex = mappings.lowerBound({ lineNumber: textRange.startLine, columnNumber: textRange.startColumn }, comparator);
        const endIndex = mappings.upperBound({ lineNumber: textRange.endLine, columnNumber: textRange.endColumn }, comparator);
        const startMapping = mappings[startIndex];
        const endMapping = mappings[endIndex];
        return new TextRange.TextRange(startMapping.lineNumber, startMapping.columnNumber, endMapping.lineNumber, endMapping.columnNumber);
    }
    /**
     * @override
     */
    dispose() {
    }
}
TextSourceMap._VLQ_BASE_SHIFT = 5;
TextSourceMap._VLQ_BASE_MASK = (1 << 5) - 1;
TextSourceMap._VLQ_CONTINUATION_MASK = 1 << 5;
TextSourceMap.StringCharIterator = class {
    /**
     * @param {string} string
     */
    constructor(string) {
        this._string = string;
        this._position = 0;
    }
    /**
     * @return {string}
     */
    next() {
        return this._string.charAt(this._position++);
    }
    /**
     * @return {string}
     */
    peek() {
        return this._string.charAt(this._position);
    }
    /**
     * @return {boolean}
     */
    hasNext() {
        return this._position < this._string.length;
    }
};
TextSourceMap.SourceInfo = class {
    /**
     * @param {?string} content
     * @param {?Array<!SourceMapEntry>} reverseMappings
     */
    constructor(content, reverseMappings) {
        this.content = content;
        this.reverseMappings = reverseMappings;
    }
};
/** @type {?Promise<typeof WasmResolver>} */
let asyncResolver;
/**
 * @implements {SourceMap}
 */
class WasmSourceMap {
    /**
     * Implements SourceMap interface for DWARF information in Wasm.
     * @param {string} wasmUrl
     * @param {!WasmResolver} resolver
     * @param {!PageResourceLoadInitiator} initiator
     */
    constructor(wasmUrl, resolver, initiator) {
        this._wasmUrl = wasmUrl;
        this._resolver = resolver;
        this._initiator = initiator;
    }
    /**
     * @private
     */
    static async _loadBindings() {
        const arrayBuffer = await Runtime.Runtime.instance().loadBinaryResourcePromise('./sdk/wasm_source_map/pkg/wasm_source_map_bg.wasm');
        await init(arrayBuffer);
        return Resolver;
    }
    /**
     * @private
     */
    static _loadBindingsOnce() {
        if (!asyncResolver) {
            asyncResolver = WasmSourceMap._loadBindings();
        }
        return asyncResolver;
    }
    /**
     *
     * @param {!Script} script
     * @param {string} wasmUrl
     * @returns {!Promise<!WasmSourceMap>}
     */
    static async load(script, wasmUrl) {
        const [Resolver, wasm] = await Promise.all([WasmSourceMap._loadBindingsOnce(), script.getWasmBytecode()]);
        return new WasmSourceMap(wasmUrl, new Resolver(new Uint8Array(wasm)), script.createPageResourceLoadInitiator());
    }
    /**
     * @override
     * @return {string}
     */
    compiledURL() {
        return this._wasmUrl;
    }
    /**
     * @override
     * @return {string}
     */
    url() {
        return WasmSourceMap.FAKE_URL;
    }
    /**
     * @override
     * @return {!Array.<string>}
     */
    sourceURLs() {
        return this._resolver.listFiles();
    }
    /**
     * @override
     * @param {string} sourceURL
     * @param {!Common.ResourceType.ResourceType} contentType
     * @return {!TextUtils.ContentProvider.ContentProvider}
     */
    sourceContentProvider(sourceURL, contentType) {
        return new CompilerSourceMappingContentProvider(sourceURL, contentType, this._initiator);
    }
    /**
     * @override
     * @param {string} sourceURL
     * @return {?string}
     */
    embeddedContentByURL(sourceURL) {
        return null;
    }
    /**
     * @override
     * @param {number} lineNumber in compiled resource
     * @param {number} columnNumber in compiled resource
     * @return {?SourceMapEntry}
     */
    findEntry(lineNumber, columnNumber) {
        if (lineNumber !== 0) {
            console.warn(new Error('Invalid non-zero line number.'));
        }
        return this._resolver.resolve(columnNumber) || null;
    }
    /**
     * @override
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?SourceMapEntry}
     */
    sourceLineMapping(sourceURL, lineNumber, columnNumber) {
        return this._resolver.resolveReverse(sourceURL, lineNumber, columnNumber) || null;
    }
    /**
     * @override
     * @return {!Array<!SourceMapEntry>}
     */
    mappings() {
        return this._resolver.listMappings();
    }
    /**
     * @override
     */
    dispose() {
        this._resolver.free();
    }
}
/* Special URL that should be kept in sync with one in V8 */
WasmSourceMap.FAKE_URL = 'wasm://dwarf';

var SourceMap$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    SourceMap: SourceMap,
    SourceMapEntry: SourceMapEntry,
    EditResult: EditResult,
    TextSourceMap: TextSourceMap,
    WasmSourceMap: WasmSourceMap
});

// Copyright 2017 The Chromium Authors. All rights reserved.
/**
 * @template {!FrameAssociated} T
 */
class SourceMapManager extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super();
        this._target = target;
        /** @type {boolean} */
        this._isEnabled = true;
        /** @type {!Map<!T, string>} */
        this._relativeSourceURL = new Map();
        /** @type {!Map<!T, string>} */
        this._relativeSourceMapURL = new Map();
        /** @type {!Map<!T, string>} */
        this._resolvedSourceMapId = new Map();
        /** @type {!Map<string, !SourceMap>} */
        this._sourceMapById = new Map();
        /** @type {!Platform.Multimap<string, !T>} */
        this._sourceMapIdToLoadingClients = new Multimap();
        /** @type {!Platform.Multimap<string, !T>} */
        this._sourceMapIdToClients = new Multimap();
        TargetManager.instance().addEventListener(Events.InspectedURLChanged, this._inspectedURLChanged, this);
    }
    /**
     * @param {boolean} isEnabled
     */
    setEnabled(isEnabled) {
        if (isEnabled === this._isEnabled) {
            return;
        }
        this._isEnabled = isEnabled;
        // We need this copy, because `this._resolvedSourceMapId` is getting modified
        // in the loop body and trying to iterate over it at the same time leads to
        // an infinite loop.
        const clients = [...this._resolvedSourceMapId.keys()];
        for (const client of clients) {
            const relativeSourceURL = this._relativeSourceURL.get(client);
            const relativeSourceMapURL = this._relativeSourceMapURL.get(client);
            this.detachSourceMap(client);
            this.attachSourceMap(client, relativeSourceURL, relativeSourceMapURL);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _inspectedURLChanged(event) {
        if (event.data !== this._target) {
            return;
        }
        // We need this copy, because `this._resolvedSourceMapId` is getting modified
        // in the loop body and trying to iterate over it at the same time leads to
        // an infinite loop.
        const prevSourceMapIds = new Map(this._resolvedSourceMapId);
        for (const [client, prevSourceMapId] of prevSourceMapIds) {
            const relativeSourceURL = this._relativeSourceURL.get(client);
            const relativeSourceMapURL = this._relativeSourceMapURL.get(client);
            if (relativeSourceURL === undefined || relativeSourceMapURL === undefined) {
                continue;
            }
            const resolvedUrls = this._resolveRelativeURLs(relativeSourceURL, relativeSourceMapURL);
            if (resolvedUrls !== null && prevSourceMapId !== resolvedUrls.sourceMapId) {
                this.detachSourceMap(client);
                this.attachSourceMap(client, relativeSourceURL, relativeSourceMapURL);
            }
        }
    }
    /**
     * @param {!T} client
     * @return {?SourceMap}
     */
    sourceMapForClient(client) {
        const sourceMapId = this._resolvedSourceMapId.get(client);
        if (!sourceMapId) {
            return null;
        }
        return this._sourceMapById.get(sourceMapId) || null;
    }
    /**
     * @param {!SourceMap} sourceMap
     * @return {!Array<!T>}
     */
    clientsForSourceMap(sourceMap) {
        const sourceMapId = this._getSourceMapId(sourceMap.compiledURL(), sourceMap.url());
        if (this._sourceMapIdToClients.has(sourceMapId)) {
            return [...this._sourceMapIdToClients.get(sourceMapId)];
        }
        return [...this._sourceMapIdToLoadingClients.get(sourceMapId)];
    }
    /**
     * @param {string} sourceURL
     * @param {string} sourceMapURL
     */
    _getSourceMapId(sourceURL, sourceMapURL) {
        return `${sourceURL}:${sourceMapURL}`;
    }
    /**
     * @param {string} sourceURL
     * @param {string} sourceMapURL
     * @return {?{sourceURL: string, sourceMapURL: string, sourceMapId: string}}
     */
    _resolveRelativeURLs(sourceURL, sourceMapURL) {
        // |sourceURL| can be a random string, but is generally an absolute path.
        // Complete it to inspected page url for relative links.
        const resolvedSourceURL = ParsedURL.ParsedURL.completeURL(this._target.inspectedURL(), sourceURL);
        if (!resolvedSourceURL) {
            return null;
        }
        const resolvedSourceMapURL = ParsedURL.ParsedURL.completeURL(resolvedSourceURL, sourceMapURL);
        if (!resolvedSourceMapURL) {
            return null;
        }
        return {
            sourceURL: resolvedSourceURL,
            sourceMapURL: resolvedSourceMapURL,
            sourceMapId: this._getSourceMapId(resolvedSourceURL, resolvedSourceMapURL)
        };
    }
    /**
     * @param {!T} client
     * @param {string|undefined} relativeSourceURL
     * @param {string|undefined} relativeSourceMapURL
     */
    attachSourceMap(client, relativeSourceURL, relativeSourceMapURL) {
        // TODO(chromium:1011811): Strengthen the type to obsolte the undefined check once sdk/ is fully typescriptified.
        if (relativeSourceURL === undefined || !relativeSourceMapURL) {
            return;
        }
        console.assert(!this._resolvedSourceMapId.has(client), 'SourceMap is already attached to client');
        const resolvedURLs = this._resolveRelativeURLs(relativeSourceURL, relativeSourceMapURL);
        if (!resolvedURLs) {
            return;
        }
        this._relativeSourceURL.set(client, relativeSourceURL);
        this._relativeSourceMapURL.set(client, relativeSourceMapURL);
        const { sourceURL, sourceMapURL, sourceMapId } = resolvedURLs;
        this._resolvedSourceMapId.set(client, sourceMapId);
        if (!this._isEnabled) {
            return;
        }
        this.dispatchEventToListeners(Events$5.SourceMapWillAttach, client);
        if (this._sourceMapById.has(sourceMapId)) {
            attach.call(this, sourceMapId, client);
            return;
        }
        if (!this._sourceMapIdToLoadingClients.has(sourceMapId)) {
            /** @type {!Promise<?SourceMap>} */
            let sourceMapPromise;
            if (sourceMapURL === WasmSourceMap.FAKE_URL && client instanceof Script) {
                if (Runtime.experiments.isEnabled('wasmDWARFDebugging')) {
                    sourceMapPromise = Promise.resolve(null);
                }
                else {
                    sourceMapPromise = WasmSourceMap.load(client, sourceURL);
                }
            }
            else {
                sourceMapPromise = TextSourceMap.load(sourceMapURL, sourceURL, client.createPageResourceLoadInitiator());
            }
            sourceMapPromise
                .catch(error => {
                Console.Console.instance().warn(ls `DevTools failed to load SourceMap: ${error.message}`);
                return null;
            })
                .then(onSourceMap.bind(this, sourceMapId));
        }
        this._sourceMapIdToLoadingClients.set(sourceMapId, client);
        /**
         * @param {string} sourceMapId
         * @param {?SourceMap} sourceMap
         * @this {SourceMapManager<T>}
         */
        function onSourceMap(sourceMapId, sourceMap) {
            this._sourceMapLoadedForTest();
            const clients = this._sourceMapIdToLoadingClients.get(sourceMapId);
            this._sourceMapIdToLoadingClients.deleteAll(sourceMapId);
            if (!clients.size) {
                return;
            }
            if (!sourceMap) {
                for (const client of clients) {
                    this.dispatchEventToListeners(Events$5.SourceMapFailedToAttach, client);
                }
                return;
            }
            this._sourceMapById.set(sourceMapId, sourceMap);
            for (const client of clients) {
                attach.call(this, sourceMapId, client);
            }
        }
        /**
         * @param {string} sourceMapId
         * @param {!T} client
         * @this {SourceMapManager<T>}
         */
        function attach(sourceMapId, client) {
            this._sourceMapIdToClients.set(sourceMapId, client);
            const sourceMap = this._sourceMapById.get(sourceMapId);
            this.dispatchEventToListeners(Events$5.SourceMapAttached, { client: client, sourceMap: sourceMap });
        }
    }
    /**
     * @param {!T} client
     */
    detachSourceMap(client) {
        const sourceMapId = this._resolvedSourceMapId.get(client);
        this._relativeSourceURL.delete(client);
        this._relativeSourceMapURL.delete(client);
        this._resolvedSourceMapId.delete(client);
        if (!sourceMapId) {
            return;
        }
        if (!this._sourceMapIdToClients.hasValue(sourceMapId, client)) {
            if (this._sourceMapIdToLoadingClients.delete(sourceMapId, client)) {
                this.dispatchEventToListeners(Events$5.SourceMapFailedToAttach, client);
            }
            return;
        }
        this._sourceMapIdToClients.delete(sourceMapId, client);
        const sourceMap = this._sourceMapById.get(sourceMapId);
        if (!sourceMap) {
            return;
        }
        this.dispatchEventToListeners(Events$5.SourceMapDetached, { client: client, sourceMap: sourceMap });
        if (!this._sourceMapIdToClients.has(sourceMapId)) {
            sourceMap.dispose();
            this._sourceMapById.delete(sourceMapId);
        }
    }
    _sourceMapLoadedForTest() {
    }
    dispose() {
        for (const sourceMap of this._sourceMapById.values()) {
            sourceMap.dispose();
        }
        TargetManager.instance().removeEventListener(Events.InspectedURLChanged, this._inspectedURLChanged, this);
    }
}
const Events$5 = {
    SourceMapWillAttach: Symbol('SourceMapWillAttach'),
    SourceMapFailedToAttach: Symbol('SourceMapFailedToAttach'),
    SourceMapAttached: Symbol('SourceMapAttached'),
    SourceMapDetached: Symbol('SourceMapDetached'),
    SourceMapChanged: Symbol('SourceMapChanged')
};

var SourceMapManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    SourceMapManager: SourceMapManager,
    Events: Events$5
});

/*
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @unrestricted
 */
class CSSModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._domModel = /** @type {!DOMModel} */ (target.model(DOMModel));
        /** @type {!SourceMapManager<!CSSStyleSheetHeader>} */
        this._sourceMapManager = new SourceMapManager(target);
        this._agent = target.cssAgent();
        this._styleLoader = new ComputedStyleLoader(this);
        this._resourceTreeModel = target.model(ResourceTreeModel);
        if (this._resourceTreeModel) {
            this._resourceTreeModel.addEventListener(Events$a.MainFrameNavigated, this._resetStyleSheets, this);
        }
        target.registerCSSDispatcher(new CSSDispatcher(this));
        if (!target.suspended()) {
            this._enable();
        }
        /** @type {!Map.<string, !CSSStyleSheetHeader>} */
        this._styleSheetIdToHeader = new Map();
        /** @type {!Map.<string, !Map.<!Protocol.Page.FrameId, !Set.<!Protocol.CSS.StyleSheetId>>>} */
        this._styleSheetIdsForURL = new Map();
        /** @type {!Map.<!CSSStyleSheetHeader, !Promise<?string>>} */
        this._originalStyleSheetText = new Map();
        /** @type {boolean} */
        this._isRuleUsageTrackingEnabled = false;
        /** @type {!Map<string, !CSSFontFace>} */
        this._fontFaces = new Map();
        /** @type {?CSSPropertyTracker} */
        this._cssPropertyTracker = null; // TODO: support multiple trackers when we refactor the backend
        this._isCSSPropertyTrackingEnabled = false;
        this._isTrackingRequestPending = false;
        /** @type {!Map<number, !Array<!Protocol.CSS.CSSComputedStyleProperty>>} */
        this._trackedCSSProperties = new Map();
        this._stylePollingThrottler = new Throttler.Throttler(StylePollingInterval);
        this._sourceMapManager.setEnabled(Settings.Settings.instance().moduleSetting('cssSourceMapsEnabled').get());
        Settings.Settings.instance()
            .moduleSetting('cssSourceMapsEnabled')
            .addChangeListener(event => this._sourceMapManager.setEnabled(/** @type {boolean} */ (event.data)));
    }
    /**
     * @param {string} sourceURL
     * @return {!Array<!CSSStyleSheetHeader>}
     */
    headersForSourceURL(sourceURL) {
        const headers = [];
        for (const headerId of this.styleSheetIdsForURL(sourceURL)) {
            const header = this.styleSheetHeaderForId(headerId);
            if (header) {
                headers.push(header);
            }
        }
        return headers;
    }
    /**
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {!Array<!CSSLocation>}
     */
    createRawLocationsByURL(sourceURL, lineNumber, columnNumber) {
        const headers = this.headersForSourceURL(sourceURL);
        headers.sort(stylesheetComparator);
        const compareToArgLocation = (_, header) => lineNumber - header.startLine || columnNumber - header.startColumn;
        const endIndex = headers.upperBound(undefined, compareToArgLocation);
        if (!endIndex) {
            return [];
        }
        const locations = [];
        const last = headers[endIndex - 1];
        for (let index = endIndex - 1; index >= 0 && headers[index].startLine === last.startLine && headers[index].startColumn === last.startColumn; --index) {
            if (headers[index].containsLocation(lineNumber, columnNumber)) {
                locations.push(new CSSLocation(headers[index], lineNumber, columnNumber));
            }
        }
        return locations;
        /**
         * @param {!CSSStyleSheetHeader} a
         * @param {!CSSStyleSheetHeader} b
         * @return {number}
         */
        function stylesheetComparator(a, b) {
            return a.startLine - b.startLine || a.startColumn - b.startColumn || a.id.localeCompare(b.id);
        }
    }
    /**
     * @return {!SourceMapManager<!CSSStyleSheetHeader>}
     */
    sourceMapManager() {
        return this._sourceMapManager;
    }
    /**
     * @param {string} text
     * @return {string}
     */
    static trimSourceURL(text) {
        let sourceURLIndex = text.lastIndexOf('/*# sourceURL=');
        if (sourceURLIndex === -1) {
            sourceURLIndex = text.lastIndexOf('/*@ sourceURL=');
            if (sourceURLIndex === -1) {
                return text;
            }
        }
        const sourceURLLineIndex = text.lastIndexOf('\n', sourceURLIndex);
        if (sourceURLLineIndex === -1) {
            return text;
        }
        const sourceURLLine = text.substr(sourceURLLineIndex + 1).split('\n', 1)[0];
        const sourceURLRegex = /[\040\t]*\/\*[#@] sourceURL=[\040\t]*([^\s]*)[\040\t]*\*\/[\040\t]*$/;
        if (sourceURLLine.search(sourceURLRegex) === -1) {
            return text;
        }
        return text.substr(0, sourceURLLineIndex) + text.substr(sourceURLLineIndex + sourceURLLine.length + 1);
    }
    /**
     * @return {!DOMModel}
     */
    domModel() {
        return this._domModel;
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!TextUtils.TextRange.TextRange} range
     * @param {string} text
     * @param {boolean} majorChange
     * @return {!Promise<boolean>}
     */
    async setStyleText(styleSheetId, range, text, majorChange) {
        try {
            await this._ensureOriginalStyleSheetText(styleSheetId);
            const stylePayloads = await this._agent.setStyleTexts([{ styleSheetId: styleSheetId, range: range.serializeToObject(), text: text }]);
            if (!stylePayloads || stylePayloads.length !== 1) {
                return false;
            }
            this._domModel.markUndoableState(!majorChange);
            const edit = new Edit(styleSheetId, range, text, stylePayloads[0]);
            this._fireStyleSheetChanged(styleSheetId, edit);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!TextUtils.TextRange.TextRange} range
     * @param {string} text
     * @return {!Promise<boolean>}
     */
    async setSelectorText(styleSheetId, range, text) {
        userMetrics.actionTaken(Host.UserMetrics.Action.StyleRuleEdited);
        try {
            await this._ensureOriginalStyleSheetText(styleSheetId);
            const selectorPayload = await this._agent.setRuleSelector(styleSheetId, range, text);
            if (!selectorPayload) {
                return false;
            }
            this._domModel.markUndoableState();
            const edit = new Edit(styleSheetId, range, text, selectorPayload);
            this._fireStyleSheetChanged(styleSheetId, edit);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!TextUtils.TextRange.TextRange} range
     * @param {string} text
     * @return {!Promise<boolean>}
     */
    async setKeyframeKey(styleSheetId, range, text) {
        userMetrics.actionTaken(Host.UserMetrics.Action.StyleRuleEdited);
        try {
            await this._ensureOriginalStyleSheetText(styleSheetId);
            const payload = await this._agent.setKeyframeKey(styleSheetId, range, text);
            if (!payload) {
                return false;
            }
            this._domModel.markUndoableState();
            const edit = new Edit(styleSheetId, range, text, payload);
            this._fireStyleSheetChanged(styleSheetId, edit);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    startCoverage() {
        this._isRuleUsageTrackingEnabled = true;
        return this._agent.startRuleUsageTracking();
    }
    /**
     * @return {!Promise<{timestamp: number, coverage:!Array<!Protocol.CSS.RuleUsage>}>}
     */
    async takeCoverageDelta() {
        const r = await this._agent.invoke_takeCoverageDelta({});
        const timestamp = (r && r.timestamp) || 0;
        const coverage = (r && r.coverage) || [];
        return { timestamp, coverage };
    }
    setLocalFontsEnabled(enabled) {
        return this._agent.invoke_setLocalFontsEnabled({
            enabled,
        });
    }
    /**
     * @return {!Promise<?>}
     */
    stopCoverage() {
        this._isRuleUsageTrackingEnabled = false;
        return this._agent.stopRuleUsageTracking();
    }
    /**
     * @return {!Promise<!Array<!CSSMedia>>}
     */
    async mediaQueriesPromise() {
        const payload = await this._agent.getMediaQueries();
        return payload ? CSSMedia.parseMediaArrayPayload(this, payload) : [];
    }
    /**
     * @return {boolean}
     */
    isEnabled() {
        return this._isEnabled;
    }
    /**
     * @return {!Promise<?>}
     */
    async _enable() {
        await this._agent.enable();
        this._isEnabled = true;
        if (this._isRuleUsageTrackingEnabled) {
            await this.startCoverage();
        }
        this.dispatchEventToListeners(Events$6.ModelWasEnabled);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {!Promise<?CSSMatchedStyles>}
     */
    async matchedStylesPromise(nodeId) {
        const response = await this._agent.invoke_getMatchedStylesForNode({ nodeId });
        if (response[InspectorBackend.ProtocolError]) {
            return null;
        }
        const node = this._domModel.nodeForId(nodeId);
        if (!node) {
            return null;
        }
        return new CSSMatchedStyles(this, /** @type {!DOMNode} */ (node), response.inlineStyle || null, response.attributesStyle || null, response.matchedCSSRules || [], response.pseudoElements || [], response.inherited || [], response.cssKeyframesRules || []);
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @return {!Promise<!Array<string>>}
     */
    classNamesPromise(styleSheetId) {
        return this._agent.collectClassNames(styleSheetId).then(classNames => classNames || []);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {!Promise<?Map<string, string>>}
     */
    computedStylePromise(nodeId) {
        return this._styleLoader.computedStylePromise(nodeId);
    }
    /**
     * @param {number} nodeId
     * @return {!Promise<?ContrastInfo>}
     */
    async backgroundColorsPromise(nodeId) {
        const response = this._agent.invoke_getBackgroundColors({ nodeId });
        if (response[InspectorBackend.ProtocolError]) {
            return null;
        }
        return response;
    }
    /**
     * @param {number} nodeId
     * @return {!Promise<?Array<!Protocol.CSS.PlatformFontUsage>>}
     */
    platformFontsPromise(nodeId) {
        return this._agent.getPlatformFontsForNode(nodeId);
    }
    /**
     * @return {!Array.<!CSSStyleSheetHeader>}
     */
    allStyleSheets() {
        const values = [...this._styleSheetIdToHeader.values()];
        /**
         * @param {!CSSStyleSheetHeader} a
         * @param {!CSSStyleSheetHeader} b
         * @return {number}
         */
        function styleSheetComparator(a, b) {
            if (a.sourceURL < b.sourceURL) {
                return -1;
            }
            if (a.sourceURL > b.sourceURL) {
                return 1;
            }
            return a.startLine - b.startLine || a.startColumn - b.startColumn;
        }
        values.sort(styleSheetComparator);
        return values;
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {!Promise<?InlineStyleResult>}
     */
    async inlineStylesPromise(nodeId) {
        const response = await this._agent.invoke_getInlineStylesForNode({ nodeId });
        if (response[InspectorBackend.ProtocolError] || !response.inlineStyle) {
            return null;
        }
        const inlineStyle = new CSSStyleDeclaration(this, null, response.inlineStyle, Type$2.Inline);
        const attributesStyle = response.attributesStyle ?
            new CSSStyleDeclaration(this, null, response.attributesStyle, Type$2.Attributes) :
            null;
        return new InlineStyleResult(inlineStyle, attributesStyle);
    }
    /**
     * @param {!DOMNode} node
     * @param {string} pseudoClass
     * @param {boolean} enable
     * @return {boolean}
     */
    forcePseudoState(node, pseudoClass, enable) {
        const pseudoClasses = node.marker(PseudoStateMarker) || [];
        if (enable) {
            if (pseudoClasses.indexOf(pseudoClass) >= 0) {
                return false;
            }
            pseudoClasses.push(pseudoClass);
            node.setMarker(PseudoStateMarker, pseudoClasses);
        }
        else {
            if (pseudoClasses.indexOf(pseudoClass) < 0) {
                return false;
            }
            ArrayUtilities.removeElement(pseudoClasses, pseudoClass);
            if (pseudoClasses.length) {
                node.setMarker(PseudoStateMarker, pseudoClasses);
            }
            else {
                node.setMarker(PseudoStateMarker, null);
            }
        }
        this._agent.forcePseudoState(node.id, pseudoClasses);
        this.dispatchEventToListeners(Events$6.PseudoStateForced, { node: node, pseudoClass: pseudoClass, enable: enable });
        return true;
    }
    /**
     * @param {!DOMNode} node
     * @return {?Array<string>} state
     */
    pseudoState(node) {
        return node.marker(PseudoStateMarker) || [];
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!TextUtils.TextRange.TextRange} range
     * @param {string} newMediaText
     * @return {!Promise<boolean>}
     */
    async setMediaText(styleSheetId, range, newMediaText) {
        userMetrics.actionTaken(Host.UserMetrics.Action.StyleRuleEdited);
        try {
            await this._ensureOriginalStyleSheetText(styleSheetId);
            const mediaPayload = await this._agent.setMediaText(styleSheetId, range, newMediaText);
            if (!mediaPayload) {
                return false;
            }
            this._domModel.markUndoableState();
            const edit = new Edit(styleSheetId, range, newMediaText, mediaPayload);
            this._fireStyleSheetChanged(styleSheetId, edit);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {string} ruleText
     * @param {!TextUtils.TextRange.TextRange} ruleLocation
     * @return {!Promise<?CSSStyleRule>}
     */
    async addRule(styleSheetId, ruleText, ruleLocation) {
        try {
            await this._ensureOriginalStyleSheetText(styleSheetId);
            const rulePayload = await this._agent.addRule(styleSheetId, ruleText, ruleLocation);
            if (!rulePayload) {
                return null;
            }
            this._domModel.markUndoableState();
            const edit = new Edit(styleSheetId, ruleLocation, ruleText, rulePayload);
            this._fireStyleSheetChanged(styleSheetId, edit);
            return new CSSStyleRule(this, rulePayload);
        }
        catch (e) {
            return null;
        }
    }
    /**
     * @param {!DOMNode} node
     * @return {!Promise<?CSSStyleSheetHeader>}
     */
    async requestViaInspectorStylesheet(node) {
        const frameId = node.frameId() || (this._resourceTreeModel ? this._resourceTreeModel.mainFrame.id : '');
        const headers = [...this._styleSheetIdToHeader.values()];
        const styleSheetHeader = headers.find(header => header.frameId === frameId && header.isViaInspector());
        if (styleSheetHeader) {
            return styleSheetHeader;
        }
        try {
            const styleSheetId = await this._agent.createStyleSheet(frameId);
            return styleSheetId && this._styleSheetIdToHeader.get(styleSheetId) || null;
        }
        catch (e) {
            return null;
        }
    }
    mediaQueryResultChanged() {
        this.dispatchEventToListeners(Events$6.MediaQueryResultChanged);
    }
    /**
     * @param {?Protocol.CSS.FontFace=} fontFace
     */
    fontsUpdated(fontFace) {
        if (fontFace) {
            this._fontFaces.set(fontFace.src, new CSSFontFace(fontFace));
        }
        this.dispatchEventToListeners(Events$6.FontsUpdated);
    }
    /**
     * @return {!Array<!CSSFontFace>}
     */
    fontFaces() {
        return [...this._fontFaces.values()];
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} id
     * @return {?CSSStyleSheetHeader}
     */
    styleSheetHeaderForId(id) {
        return this._styleSheetIdToHeader.get(id) || null;
    }
    /**
     * @return {!Array.<!CSSStyleSheetHeader>}
     */
    styleSheetHeaders() {
        return [...this._styleSheetIdToHeader.values()];
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!Edit=} edit
     */
    _fireStyleSheetChanged(styleSheetId, edit) {
        this.dispatchEventToListeners(Events$6.StyleSheetChanged, { styleSheetId: styleSheetId, edit: edit });
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @return {!Promise<?string>}
     */
    _ensureOriginalStyleSheetText(styleSheetId) {
        const header = this.styleSheetHeaderForId(styleSheetId);
        if (!header) {
            return Promise.resolve(/** @type {?string} */ (null));
        }
        let promise = this._originalStyleSheetText.get(header);
        if (!promise) {
            promise = this.getStyleSheetText(header.id);
            this._originalStyleSheetText.set(header, promise);
            this._originalContentRequestedForTest(header);
        }
        return promise;
    }
    /**
     * @param {!CSSStyleSheetHeader} header
     */
    _originalContentRequestedForTest(header) {
    }
    /**
     * @param {!CSSStyleSheetHeader} header
     * @return {!Promise<?string>}
     */
    originalStyleSheetText(header) {
        return this._ensureOriginalStyleSheetText(header.id);
    }
    /**
     *
     * @return {!Iterable<!CSSStyleSheetHeader>}
     */
    getAllStyleSheetHeaders() {
        return this._styleSheetIdToHeader.values();
    }
    /**
     * @param {!Protocol.CSS.CSSStyleSheetHeader} header
     */
    _styleSheetAdded(header) {
        console.assert(!this._styleSheetIdToHeader.get(header.styleSheetId));
        const styleSheetHeader = new CSSStyleSheetHeader(this, header);
        this._styleSheetIdToHeader.set(header.styleSheetId, styleSheetHeader);
        const url = styleSheetHeader.resourceURL();
        if (!this._styleSheetIdsForURL.get(url)) {
            this._styleSheetIdsForURL.set(url, new Map());
        }
        const frameIdToStyleSheetIds = this._styleSheetIdsForURL.get(url);
        let styleSheetIds = frameIdToStyleSheetIds.get(styleSheetHeader.frameId);
        if (!styleSheetIds) {
            styleSheetIds = new Set();
            frameIdToStyleSheetIds.set(styleSheetHeader.frameId, styleSheetIds);
        }
        styleSheetIds.add(styleSheetHeader.id);
        this._sourceMapManager.attachSourceMap(styleSheetHeader, styleSheetHeader.sourceURL, styleSheetHeader.sourceMapURL);
        this.dispatchEventToListeners(Events$6.StyleSheetAdded, styleSheetHeader);
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} id
     */
    _styleSheetRemoved(id) {
        const header = this._styleSheetIdToHeader.get(id);
        console.assert(header);
        if (!header) {
            return;
        }
        this._styleSheetIdToHeader.delete(id);
        const url = header.resourceURL();
        const frameIdToStyleSheetIds = this._styleSheetIdsForURL.get(url);
        console.assert(frameIdToStyleSheetIds, 'No frameId to styleSheetId map is available for given style sheet URL.');
        frameIdToStyleSheetIds.get(header.frameId).delete(id);
        if (!frameIdToStyleSheetIds.get(header.frameId).size) {
            frameIdToStyleSheetIds.delete(header.frameId);
            if (!frameIdToStyleSheetIds.size) {
                this._styleSheetIdsForURL.delete(url);
            }
        }
        this._originalStyleSheetText.delete(header);
        this._sourceMapManager.detachSourceMap(header);
        this.dispatchEventToListeners(Events$6.StyleSheetRemoved, header);
    }
    /**
     * @param {string} url
     * @return {!Array.<!Protocol.CSS.StyleSheetId>}
     */
    styleSheetIdsForURL(url) {
        const frameIdToStyleSheetIds = this._styleSheetIdsForURL.get(url);
        if (!frameIdToStyleSheetIds) {
            return [];
        }
        const result = [];
        for (const styleSheetIds of frameIdToStyleSheetIds.values()) {
            result.push(...styleSheetIds);
        }
        return result;
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {string} newText
     * @param {boolean} majorChange
     * @return {!Promise<?string>}
     */
    async setStyleSheetText(styleSheetId, newText, majorChange) {
        const header = /** @type {!CSSStyleSheetHeader} */ (this._styleSheetIdToHeader.get(styleSheetId));
        console.assert(header);
        newText = CSSModel.trimSourceURL(newText);
        if (header.hasSourceURL) {
            newText += '\n/*# sourceURL=' + header.sourceURL + ' */';
        }
        await this._ensureOriginalStyleSheetText(styleSheetId);
        const response = await this._agent.invoke_setStyleSheetText({ styleSheetId: header.id, text: newText });
        const sourceMapURL = response.sourceMapURL;
        this._sourceMapManager.detachSourceMap(header);
        header.setSourceMapURL(sourceMapURL);
        this._sourceMapManager.attachSourceMap(header, header.sourceURL, header.sourceMapURL);
        if (sourceMapURL === null) {
            return 'Error in CSS.setStyleSheetText';
        }
        this._domModel.markUndoableState(!majorChange);
        this._fireStyleSheetChanged(styleSheetId);
        return null;
    }
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @return {!Promise<?string>}
     */
    async getStyleSheetText(styleSheetId) {
        try {
            const text = await this._agent.getStyleSheetText(styleSheetId);
            return text && CSSModel.trimSourceURL(text);
        }
        catch (e) {
            return null;
        }
    }
    _resetStyleSheets() {
        const headers = [...this._styleSheetIdToHeader.values()];
        this._styleSheetIdsForURL.clear();
        this._styleSheetIdToHeader.clear();
        for (const header of headers) {
            this._sourceMapManager.detachSourceMap(header);
            this.dispatchEventToListeners(Events$6.StyleSheetRemoved, header);
        }
    }
    _resetFontFaces() {
        this._fontFaces.clear();
    }
    /**
     * @override
     * @return {!Promise<?>}
     */
    async suspendModel() {
        this._isEnabled = false;
        await this._agent.disable();
        this._resetStyleSheets();
        this._resetFontFaces();
    }
    /**
     * @override
     * @return {!Promise<?>}
     */
    async resumeModel() {
        return this._enable();
    }
    /**
     * @param {number} nodeId
     * @param {string} name
     * @param {string} value
     */
    setEffectivePropertyValueForNode(nodeId, name, value) {
        this._agent.setEffectivePropertyValueForNode(nodeId, name, value);
    }
    /**
     * @param {!DOMNode} node
     * @return {!Promise.<?CSSMatchedStyles>}
     */
    cachedMatchedCascadeForNode(node) {
        if (this._cachedMatchedCascadeNode !== node) {
            this.discardCachedMatchedCascade();
        }
        this._cachedMatchedCascadeNode = node;
        if (!this._cachedMatchedCascadePromise) {
            this._cachedMatchedCascadePromise = this.matchedStylesPromise(node.id);
        }
        return this._cachedMatchedCascadePromise;
    }
    discardCachedMatchedCascade() {
        delete this._cachedMatchedCascadeNode;
        delete this._cachedMatchedCascadePromise;
    }
    /**
     @param {!Array.<!Protocol.CSS.CSSComputedStyleProperty>} propertiesToTrack
     */
    createCSSPropertyTracker(propertiesToTrack) {
        const gridStyleTracker = new CSSPropertyTracker(this, propertiesToTrack);
        return gridStyleTracker;
    }
    /**
     * @param {!CSSPropertyTracker} cssPropertyTracker
     */
    enableCSSPropertyTracker(cssPropertyTracker) {
        const propertiesToTrack = cssPropertyTracker.getTrackedProperties();
        if (propertiesToTrack.length === 0) {
            return;
        }
        this._agent.invoke_trackComputedStyleUpdates({ propertiesToTrack });
        this._isCSSPropertyTrackingEnabled = true;
        this._cssPropertyTracker = cssPropertyTracker;
        this._pollComputedStyleUpdates();
    }
    // Since we only support one tracker at a time, this call effectively disables
    // style tracking.
    disableCSSPropertyTracker() {
        this._isCSSPropertyTrackingEnabled = false;
        this._cssPropertyTracker = null;
        // Sending an empty list to the backend signals the close of style tracking
        this._agent.invoke_trackComputedStyleUpdates({ propertiesToTrack: [] });
    }
    async _pollComputedStyleUpdates() {
        if (this._isTrackingRequestPending) {
            return;
        }
        if (this._isCSSPropertyTrackingEnabled) {
            this._isTrackingRequestPending = true;
            const result = await this._agent.invoke_takeComputedStyleUpdates();
            this._isTrackingRequestPending = false;
            if (result.getError() || !result.nodeIds || !this._isCSSPropertyTrackingEnabled) {
                return;
            }
            this._cssPropertyTracker.dispatchEventToListeners(CSSPropertyTrackerEvents.TrackedCSSPropertiesUpdated, {
                domNodes: result.nodeIds.map(nodeId => this._domModel.nodeForId(nodeId)),
            });
        }
        if (this._isCSSPropertyTrackingEnabled) {
            this._stylePollingThrottler.schedule(this._pollComputedStyleUpdates.bind(this));
        }
    }
    /**
     * @override
     */
    dispose() {
        if (Runtime.experiments.isEnabled('cssGridFeatures')) {
            this.disableCSSPropertyTracker();
        }
        super.dispose();
        this._sourceMapManager.dispose();
    }
}
/** @enum {symbol} */
const Events$6 = {
    FontsUpdated: Symbol('FontsUpdated'),
    MediaQueryResultChanged: Symbol('MediaQueryResultChanged'),
    ModelWasEnabled: Symbol('ModelWasEnabled'),
    PseudoStateForced: Symbol('PseudoStateForced'),
    StyleSheetAdded: Symbol('StyleSheetAdded'),
    StyleSheetChanged: Symbol('StyleSheetChanged'),
    StyleSheetRemoved: Symbol('StyleSheetRemoved'),
};
const PseudoStateMarker = 'pseudo-state-marker';
class Edit {
    /**
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     * @param {!TextUtils.TextRange.TextRange} oldRange
     * @param {string} newText
     * @param {?Object} payload
     */
    constructor(styleSheetId, oldRange, newText, payload) {
        this.styleSheetId = styleSheetId;
        this.oldRange = oldRange;
        this.newRange = TextRange.TextRange.fromEdit(oldRange, newText);
        this.newText = newText;
        this.payload = payload;
    }
}
class CSSLocation {
    /**
     * @param {!CSSStyleSheetHeader} header
     * @param {number} lineNumber
     * @param {number=} columnNumber
     */
    constructor(header, lineNumber, columnNumber) {
        this._cssModel = header.cssModel();
        this.styleSheetId = header.id;
        this.url = header.resourceURL();
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber || 0;
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return this._cssModel;
    }
    /**
     * @return {?CSSStyleSheetHeader}
     */
    header() {
        return this._cssModel.styleSheetHeaderForId(this.styleSheetId);
    }
}
/**
 * @implements {Protocol.CSSDispatcher}
 * @unrestricted
 */
class CSSDispatcher {
    /**
     * @param {!CSSModel} cssModel
     */
    constructor(cssModel) {
        this._cssModel = cssModel;
    }
    /**
     * @override
     */
    mediaQueryResultChanged() {
        this._cssModel.mediaQueryResultChanged();
    }
    /**
     * @override
     * @param {?Protocol.CSS.FontFace=} fontFace
     */
    fontsUpdated(fontFace) {
        this._cssModel.fontsUpdated(fontFace);
    }
    /**
     * @override
     * @param {!Protocol.CSS.StyleSheetId} styleSheetId
     */
    styleSheetChanged(styleSheetId) {
        this._cssModel._fireStyleSheetChanged(styleSheetId);
    }
    /**
     * @override
     * @param {!Protocol.CSS.CSSStyleSheetHeader} header
     */
    styleSheetAdded(header) {
        this._cssModel._styleSheetAdded(header);
    }
    /**
     * @override
     * @param {!Protocol.CSS.StyleSheetId} id
     */
    styleSheetRemoved(id) {
        this._cssModel._styleSheetRemoved(id);
    }
}
class ComputedStyleLoader {
    /**
     * @param {!CSSModel} cssModel
     */
    constructor(cssModel) {
        this._cssModel = cssModel;
        /** @type {!Map<!Protocol.DOM.NodeId, !Promise<?Map<string, string>>>} */
        this._nodeIdToPromise = new Map();
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {!Promise<?Map<string, string>>}
     */
    computedStylePromise(nodeId) {
        let promise = this._nodeIdToPromise.get(nodeId);
        if (promise) {
            return promise;
        }
        promise = this._cssModel._agent.getComputedStyleForNode(nodeId).then(parsePayload.bind(this));
        this._nodeIdToPromise.set(nodeId, promise);
        return promise;
        /**
         * @param {?Array<!Protocol.CSS.CSSComputedStyleProperty>} computedPayload
         * @return {?Map<string, string>}
         * @this {ComputedStyleLoader}
         */
        function parsePayload(computedPayload) {
            this._nodeIdToPromise.delete(nodeId);
            if (!computedPayload || !computedPayload.length) {
                return null;
            }
            const result = new Map();
            for (const property of computedPayload) {
                result.set(property.name, property.value);
            }
            return result;
        }
    }
}
class InlineStyleResult {
    /**
     * @param {?CSSStyleDeclaration} inlineStyle
     * @param {?CSSStyleDeclaration} attributesStyle
     */
    constructor(inlineStyle, attributesStyle) {
        this.inlineStyle = inlineStyle;
        this.attributesStyle = attributesStyle;
    }
}
class CSSPropertyTracker extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {!CSSModel} cssModel
     * @param {!Array.<!Protocol.CSS.CSSComputedStyleProperty>} propertiesToTrack
     */
    constructor(cssModel, propertiesToTrack) {
        super();
        this._cssModel = cssModel;
        this._properties = propertiesToTrack;
    }
    start() {
        this._cssModel.enableCSSPropertyTracker(this);
    }
    stop() {
        this._cssModel.disableCSSPropertyTracker();
    }
    /**
     * @return {!Array.<!Protocol.CSS.CSSComputedStyleProperty>}
     */
    getTrackedProperties() {
        return this._properties;
    }
}
const StylePollingInterval = 1000; // throttling interval for style polling, in milliseconds
/** @enum {symbol} */
const CSSPropertyTrackerEvents = {
    TrackedCSSPropertiesUpdated: Symbol('TrackedCSSPropertiesUpdated'),
};
SDKModel.register(CSSModel, Capability.DOM, true);
/** @typedef {{backgroundColors: ?Array<string>, computedFontSize: string, computedFontWeight: string}} */
let ContrastInfo;

var CSSModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CSSModel: CSSModel,
    Events: Events$6,
    Edit: Edit,
    CSSLocation: CSSLocation,
    InlineStyleResult: InlineStyleResult,
    CSSPropertyTracker: CSSPropertyTracker,
    CSSPropertyTrackerEvents: CSSPropertyTrackerEvents,
    ContrastInfo: ContrastInfo
});

// Copyright 2017 The Chromium Authors. All rights reserved.
/**
 * @typedef {{r: number, g: number, b: number, a: number}}
 */
// @ts-ignore typedef
let HighlightColor;
/**
 * @typedef {{x: number, y: number, width: number, height: number, color: HighlightColor, outlineColor: HighlightColor}}
 */
// @ts-ignore typedef
let HighlightRect;
/** @typedef {!{width: number, height: number, x: number, y: number, contentColor:HighlightColor, outlineColor: HighlightColor}} */
// @ts-ignore typedef
let Hinge;
/**
 * @implements {ProtocolProxyApi.OverlayDispatcher}
 */
class OverlayModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._domModel = /** @type {!DOMModel} */ (target.model(DOMModel));
        target.registerOverlayDispatcher(this);
        this._overlayAgent = target.overlayAgent();
        this._debuggerModel = target.model(DebuggerModel);
        if (this._debuggerModel) {
            Settings.Settings.instance()
                .moduleSetting('disablePausedStateOverlay')
                .addChangeListener(this._updatePausedInDebuggerMessage, this);
            this._debuggerModel.addEventListener(Events$b.DebuggerPaused, this._updatePausedInDebuggerMessage, this);
            this._debuggerModel.addEventListener(Events$b.DebuggerResumed, this._updatePausedInDebuggerMessage, this);
            // TODO(dgozman): we should get DebuggerResumed on navigations instead of listening to GlobalObjectCleared.
            this._debuggerModel.addEventListener(Events$b.GlobalObjectCleared, this._updatePausedInDebuggerMessage, this);
        }
        this._inspectModeEnabled = false;
        this._gridFeaturesExperimentEnabled = Runtime.experiments.isEnabled('cssGridFeatures');
        this._hideHighlightTimeout = null;
        /** @type {!Highlighter} */
        this._defaultHighlighter = new DefaultHighlighter(this);
        this._highlighter = this._defaultHighlighter;
        this._showPaintRectsSetting = Settings.Settings.instance().moduleSetting('showPaintRects');
        this._showLayoutShiftRegionsSetting = Settings.Settings.instance().moduleSetting('showLayoutShiftRegions');
        this._showAdHighlightsSetting = Settings.Settings.instance().moduleSetting('showAdHighlights');
        this._showDebugBordersSetting = Settings.Settings.instance().moduleSetting('showDebugBorders');
        this._showFPSCounterSetting = Settings.Settings.instance().moduleSetting('showFPSCounter');
        this._showScrollBottleneckRectsSetting =
            Settings.Settings.instance().moduleSetting('showScrollBottleneckRects');
        this._showHitTestBordersSetting = Settings.Settings.instance().moduleSetting('showHitTestBorders');
        /**
         * @type {!Array<!Common.EventTarget.EventDescriptor>}
         */
        this._registeredListeners = [];
        /** @type {boolean} */
        this._showViewportSizeOnResize = true;
        if (!target.suspended()) {
            this._overlayAgent.invoke_enable();
            this._wireAgentToSettings();
        }
        /** @type {?DefaultPersistentGridHighlighter} */
        this._persistentGridHighlighter = null;
        if (this._gridFeaturesExperimentEnabled) {
            this._persistentGridHighlighter = new DefaultPersistentGridHighlighter(this);
            this._domModel.addEventListener(Events$8.NodeRemoved, () => {
                if (!this._persistentGridHighlighter) {
                    return;
                }
                this._persistentGridHighlighter.refreshHighlights();
            });
            this._domModel.addEventListener(Events$8.DocumentUpdated, () => {
                if (!this._persistentGridHighlighter) {
                    return;
                }
                this._persistentGridHighlighter.hideAllInOverlay();
            });
        }
        this._sourceOrderHighlighter = new SourceOrderHighlighter(this);
        this._sourceOrderModeActive = false;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @param {!RemoteObject} object
     */
    static highlightObjectAsDOMNode(object) {
        const domModel = object.runtimeModel().target().model(DOMModel);
        if (domModel) {
            domModel.overlayModel().highlightInOverlay({ object });
        }
    }
    static hideDOMNodeHighlight() {
        for (const overlayModel of TargetManager.instance().models(OverlayModel)) {
            overlayModel._delayedHideHighlight(0);
        }
    }
    static async muteHighlight() {
        return Promise.all(TargetManager.instance().models(OverlayModel).map(model => model.suspendModel()));
    }
    static async unmuteHighlight() {
        return Promise.all(TargetManager.instance().models(OverlayModel).map(model => model.resumeModel()));
    }
    /**
     * @param {!HighlightRect} rect
     */
    static highlightRect(rect) {
        for (const overlayModel of TargetManager.instance().models(OverlayModel)) {
            overlayModel.highlightRect(rect);
        }
    }
    static clearHighlight() {
        for (const overlayModel of TargetManager.instance().models(OverlayModel)) {
            overlayModel.clearHighlight();
        }
    }
    /**
     * @return {!DOMModel}
     */
    getDOMModel() {
        return this._domModel;
    }
    /**
     * @param {!HighlightRect} rect
     * @return {!Promise<*>}
     */
    highlightRect({ x, y, width, height, color, outlineColor }) {
        const highlightColor = color || { r: 255, g: 0, b: 255, a: 0.3 };
        const highlightOutlineColor = outlineColor || { r: 255, g: 0, b: 255, a: 0.5 };
        return this._overlayAgent.invoke_highlightRect({ x, y, width, height, color: highlightColor, outlineColor: highlightOutlineColor });
    }
    /**
     * @return {!Promise<*>}
     */
    clearHighlight() {
        return this._overlayAgent.invoke_hideHighlight();
    }
    /**
     * @return {!Promise<void>}
     */
    async _wireAgentToSettings() {
        this._registeredListeners = [
            this._showPaintRectsSetting.addChangeListener(() => this._overlayAgent.invoke_setShowPaintRects({ result: this._showPaintRectsSetting.get() })),
            this._showLayoutShiftRegionsSetting.addChangeListener(() => this._overlayAgent.invoke_setShowLayoutShiftRegions({ result: this._showLayoutShiftRegionsSetting.get() })),
            this._showAdHighlightsSetting.addChangeListener(() => this._overlayAgent.invoke_setShowAdHighlights({ show: this._showAdHighlightsSetting.get() })),
            this._showDebugBordersSetting.addChangeListener(() => this._overlayAgent.invoke_setShowDebugBorders({ show: this._showDebugBordersSetting.get() })),
            this._showFPSCounterSetting.addChangeListener(() => this._overlayAgent.invoke_setShowFPSCounter({ show: this._showFPSCounterSetting.get() })),
            this._showScrollBottleneckRectsSetting.addChangeListener(() => this._overlayAgent.invoke_setShowScrollBottleneckRects({ show: this._showScrollBottleneckRectsSetting.get() })),
            this._showHitTestBordersSetting.addChangeListener(() => this._overlayAgent.invoke_setShowHitTestBorders({ show: this._showHitTestBordersSetting.get() }))
        ];
        if (this._showPaintRectsSetting.get()) {
            this._overlayAgent.invoke_setShowPaintRects({ result: true });
        }
        if (this._showLayoutShiftRegionsSetting.get()) {
            this._overlayAgent.invoke_setShowLayoutShiftRegions({ result: true });
        }
        if (this._showAdHighlightsSetting.get()) {
            this._overlayAgent.invoke_setShowAdHighlights({ show: true });
        }
        if (this._showDebugBordersSetting.get()) {
            this._overlayAgent.invoke_setShowDebugBorders({ show: true });
        }
        if (this._showFPSCounterSetting.get()) {
            this._overlayAgent.invoke_setShowFPSCounter({ show: true });
        }
        if (this._showScrollBottleneckRectsSetting.get()) {
            this._overlayAgent.invoke_setShowScrollBottleneckRects({ show: true });
        }
        if (this._showHitTestBordersSetting.get()) {
            this._overlayAgent.invoke_setShowHitTestBorders({ show: true });
        }
        if (this._debuggerModel.isPaused()) {
            this._updatePausedInDebuggerMessage();
        }
        await this._overlayAgent.invoke_setShowViewportSizeOnResize({ show: this._showViewportSizeOnResize });
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    async suspendModel() {
        EventTarget.EventTarget.removeEventListeners(this._registeredListeners);
        await this._overlayAgent.invoke_disable();
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    async resumeModel() {
        await Promise.all([this._overlayAgent.invoke_enable(), this._wireAgentToSettings()]);
    }
    /**
     * @param {boolean} show
     */
    setShowViewportSizeOnResize(show) {
        if (this._showViewportSizeOnResize === show) {
            return;
        }
        this._showViewportSizeOnResize = show;
        if (this.target().suspended()) {
            return;
        }
        this._overlayAgent.invoke_setShowViewportSizeOnResize({ show });
    }
    _updatePausedInDebuggerMessage() {
        if (this.target().suspended()) {
            return;
        }
        const message = this._debuggerModel.isPaused() &&
            !Settings.Settings.instance().moduleSetting('disablePausedStateOverlay').get() ?
            UIString.UIString('Paused in debugger') :
            undefined;
        this._overlayAgent.invoke_setPausedInDebuggerMessage({ message });
    }
    /**
     * @param {?Highlighter} highlighter
     */
    setHighlighter(highlighter) {
        this._highlighter = highlighter || this._defaultHighlighter;
    }
    /**
     * @param {!Protocol.Overlay.InspectMode} mode
     * @param {boolean=} showDetailedTooltip
     * @return {!Promise<void>}
     */
    async setInspectMode(mode, showDetailedTooltip = true) {
        await this._domModel.requestDocument();
        this._inspectModeEnabled = mode !== Protocol.Overlay.InspectMode.None;
        this.dispatchEventToListeners(Events$7.InspectModeWillBeToggled, this);
        this._highlighter.setInspectMode(mode, this._buildHighlightConfig('all', showDetailedTooltip));
    }
    /**
     * @return {boolean}
     */
    inspectModeEnabled() {
        return this._inspectModeEnabled;
    }
    /**
     * @param {!HighlightData} data
     * @param {string=} mode
     * @param {boolean=} showInfo
     */
    highlightInOverlay(data, mode, showInfo) {
        if (this._sourceOrderModeActive) {
            // Return early if the source order is currently being shown the in the
            // overlay, so that it is not cleared by the highlight
            return;
        }
        if (this._hideHighlightTimeout) {
            clearTimeout(this._hideHighlightTimeout);
            this._hideHighlightTimeout = null;
        }
        const highlightConfig = this._buildHighlightConfig(mode);
        if (typeof showInfo !== 'undefined') {
            highlightConfig.showInfo = showInfo;
        }
        this._highlighter.highlightInOverlay(data, highlightConfig);
    }
    /**
     * @param {!HighlightData} data
     */
    highlightInOverlayForTwoSeconds(data) {
        this.highlightInOverlay(data);
        this._delayedHideHighlight(2000);
    }
    /**
     * @param {number} nodeId
     * @param {!Host.UserMetrics.GridOverlayOpener} gridOverlayOpener
     */
    highlightGridInPersistentOverlay(nodeId, gridOverlayOpener) {
        if (!this._persistentGridHighlighter) {
            return;
        }
        this._persistentGridHighlighter.highlightInOverlay(nodeId);
        userMetrics.gridOverlayOpenedFrom(gridOverlayOpener);
        this.dispatchEventToListeners(Events$7.PersistentGridOverlayStateChanged, { nodeId, enabled: true });
    }
    /**
     * @param {number} nodeId
     */
    isHighlightedGridInPersistentOverlay(nodeId) {
        if (!this._persistentGridHighlighter) {
            return;
        }
        return this._persistentGridHighlighter.isHighlighted(nodeId);
    }
    /**
     * @param {number} nodeId
     */
    hideGridInPersistentOverlay(nodeId) {
        if (!this._persistentGridHighlighter) {
            return;
        }
        this._persistentGridHighlighter.hideInOverlay(nodeId);
        this.dispatchEventToListeners(Events$7.PersistentGridOverlayStateChanged, { nodeId, enabled: false });
    }
    /**
     * @param {!DOMNode} node
     */
    highlightSourceOrderInOverlay(node) {
        const sourceOrderConfig = {
            parentOutlineColor: Color.SourceOrderHighlight.ParentOutline.toProtocolRGBA(),
            childOutlineColor: Color.SourceOrderHighlight.ChildOutline.toProtocolRGBA(),
        };
        this._sourceOrderHighlighter.highlightSourceOrderInOverlay(node, sourceOrderConfig);
    }
    /**
     * @param {number} nodeId
     * @return {string|null}
     */
    colorOfGridInPersistentOverlay(nodeId) {
        if (!this._persistentGridHighlighter) {
            return null;
        }
        return this._persistentGridHighlighter.colorOfGrid(nodeId).asString(Color.Format.HEX);
    }
    /**
     * @param {number} nodeId
     * @param {string} colorStr
     */
    setColorOfGridInPersistentOverlay(nodeId, colorStr) {
        if (!this._persistentGridHighlighter) {
            return;
        }
        const color = Color.Color.parse(colorStr);
        if (!color) {
            return;
        }
        this._persistentGridHighlighter.setColorOfGrid(nodeId, color);
        this._persistentGridHighlighter._resetOverlay();
    }
    hideSourceOrderInOverlay() {
        this._sourceOrderHighlighter.hideSourceOrderHighlight();
    }
    /**
     * @param {boolean} isActive
     */
    setSourceOrderActive(isActive) {
        this._sourceOrderModeActive = isActive;
    }
    sourceOrderModeActive() {
        return this._sourceOrderModeActive;
    }
    /**
     * @param {number} delay
     */
    _delayedHideHighlight(delay) {
        if (this._hideHighlightTimeout === null) {
            this._hideHighlightTimeout = setTimeout(() => this.highlightInOverlay({ clear: true }), delay);
        }
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     */
    highlightFrame(frameId) {
        if (this._hideHighlightTimeout) {
            clearTimeout(this._hideHighlightTimeout);
            this._hideHighlightTimeout = null;
        }
        this._highlighter.highlightFrame(frameId);
    }
    /**
     * @param {?Hinge} hinge
     */
    showHingeForDualScreen(hinge) {
        if (hinge) {
            const { x, y, width, height, contentColor, outlineColor } = hinge;
            this._overlayAgent.invoke_setShowHinge({
                hingeConfig: { rect: { x: x, y: y, width: width, height: height }, contentColor: contentColor, outlineColor: outlineColor }
            });
        }
        else {
            this._overlayAgent.invoke_setShowHinge({});
        }
    }
    /**
     * @param {string=} mode
     * @param {boolean=} showDetailedToolip
     * @return {!Protocol.Overlay.HighlightConfig}
     */
    _buildHighlightConfig(mode = 'all', showDetailedToolip = false) {
        const showRulers = Settings.Settings.instance().moduleSetting('showMetricsRulers').get();
        const colorFormat = Settings.Settings.instance().moduleSetting('colorFormat').get();
        /** @type {!Protocol.Overlay.HighlightConfig} */
        const highlightConfig = {
            showInfo: mode === 'all',
            showRulers: showRulers,
            showStyles: showDetailedToolip,
            showAccessibilityInfo: showDetailedToolip,
            showExtensionLines: showRulers,
            gridHighlightConfig: {},
        };
        if (mode === 'all' || mode === 'content') {
            highlightConfig.contentColor = Color.PageHighlight.Content.toProtocolRGBA();
        }
        if (mode === 'all' || mode === 'padding') {
            highlightConfig.paddingColor = Color.PageHighlight.Padding.toProtocolRGBA();
        }
        if (mode === 'all' || mode === 'border') {
            highlightConfig.borderColor = Color.PageHighlight.Border.toProtocolRGBA();
        }
        if (mode === 'all' || mode === 'margin') {
            highlightConfig.marginColor = Color.PageHighlight.Margin.toProtocolRGBA();
        }
        if (mode === 'all') {
            highlightConfig.eventTargetColor = Color.PageHighlight.EventTarget.toProtocolRGBA();
            highlightConfig.shapeColor = Color.PageHighlight.Shape.toProtocolRGBA();
            highlightConfig.shapeMarginColor = Color.PageHighlight.ShapeMargin.toProtocolRGBA();
            highlightConfig.gridHighlightConfig = {
                rowGapColor: Color.PageHighlight.GridRowGapBackground.toProtocolRGBA(),
                rowHatchColor: Color.PageHighlight.GridRowGapHatch.toProtocolRGBA(),
                columnGapColor: Color.PageHighlight.GridColumnGapBackground.toProtocolRGBA(),
                columnHatchColor: Color.PageHighlight.GridColumnGapHatch.toProtocolRGBA(),
                rowLineColor: Color.PageHighlight.GridRowLine.toProtocolRGBA(),
                columnLineColor: Color.PageHighlight.GridColumnLine.toProtocolRGBA(),
                rowLineDash: true,
                columnLineDash: true,
            };
        }
        if (mode.endsWith('gap') && this._gridFeaturesExperimentEnabled) {
            highlightConfig.gridHighlightConfig = {
                gridBorderColor: Color.PageHighlight.GridBorder.toProtocolRGBA(),
                gridBorderDash: true
            };
            if (mode === 'gap' || mode === 'row-gap') {
                highlightConfig.gridHighlightConfig.rowGapColor =
                    Color.PageHighlight.GridRowGapBackground.toProtocolRGBA();
                highlightConfig.gridHighlightConfig.rowHatchColor = Color.PageHighlight.GridRowGapHatch.toProtocolRGBA();
            }
            if (mode === 'gap' || mode === 'column-gap') {
                highlightConfig.gridHighlightConfig.columnGapColor =
                    Color.PageHighlight.GridColumnGapBackground.toProtocolRGBA();
                highlightConfig.gridHighlightConfig.columnHatchColor =
                    Color.PageHighlight.GridColumnGapHatch.toProtocolRGBA();
            }
        }
        if (mode === 'grid-areas' && this._gridFeaturesExperimentEnabled) {
            highlightConfig.gridHighlightConfig = {
                rowLineColor: Color.PageHighlight.GridRowLine.toProtocolRGBA(),
                columnLineColor: Color.PageHighlight.GridColumnLine.toProtocolRGBA(),
                rowLineDash: true,
                columnLineDash: true,
                showAreaNames: true,
                areaBorderColor: Color.PageHighlight.GridAreaBorder.toProtocolRGBA()
            };
        }
        if (mode === 'grid-template-columns' && this._gridFeaturesExperimentEnabled) {
            highlightConfig.contentColor = Color.PageHighlight.Content.toProtocolRGBA();
            highlightConfig.gridHighlightConfig = {
                columnLineColor: Color.PageHighlight.GridColumnLine.toProtocolRGBA(),
                columnLineDash: true
            };
        }
        if (mode === 'grid-template-rows' && this._gridFeaturesExperimentEnabled) {
            highlightConfig.contentColor = Color.PageHighlight.Content.toProtocolRGBA();
            highlightConfig.gridHighlightConfig = {
                rowLineColor: Color.PageHighlight.GridRowLine.toProtocolRGBA(),
                rowLineDash: true
            };
        }
        // the backend does not support the 'original' format because
        // it currently cannot retrieve the original format using computed styles
        const supportedColorFormats = new Set(['rgb', 'hsl', 'hex']);
        if (supportedColorFormats.has(colorFormat)) {
            highlightConfig.colorFormat = colorFormat;
        }
        return highlightConfig;
    }
    /**
     * @override
     * @param {!Protocol.Overlay.NodeHighlightRequestedEvent} event
     */
    nodeHighlightRequested({ nodeId }) {
        const node = this._domModel.nodeForId(nodeId);
        if (node) {
            this.dispatchEventToListeners(Events$7.HighlightNodeRequested, node);
        }
    }
    /**
     * @param {function(!DOMNode):void} handler
     */
    static setInspectNodeHandler(handler) {
        OverlayModel._inspectNodeHandler = handler;
    }
    /**
     * @override
     * @param {!Protocol.Overlay.InspectNodeRequestedEvent} event
     */
    inspectNodeRequested({ backendNodeId }) {
        const deferredNode = new DeferredDOMNode(this.target(), backendNodeId);
        if (OverlayModel._inspectNodeHandler) {
            deferredNode.resolvePromise().then(node => {
                if (node && OverlayModel._inspectNodeHandler) {
                    OverlayModel._inspectNodeHandler(node);
                }
            });
        }
        else {
            Revealer.reveal(deferredNode);
        }
        this.dispatchEventToListeners(Events$7.ExitedInspectMode);
    }
    /**
     * @override
     * @param {!Protocol.Overlay.ScreenshotRequestedEvent} event
     */
    screenshotRequested({ viewport }) {
        this.dispatchEventToListeners(Events$7.ScreenshotRequested, viewport);
        this.dispatchEventToListeners(Events$7.ExitedInspectMode);
    }
    /**
     * @override
     */
    inspectModeCanceled() {
        this.dispatchEventToListeners(Events$7.ExitedInspectMode);
    }
}
/** @type {?function(!DOMNode):void} */
OverlayModel._inspectNodeHandler = null;
/** @enum {symbol} */
const Events$7 = {
    InspectModeWillBeToggled: Symbol('InspectModeWillBeToggled'),
    ExitedInspectMode: Symbol('InspectModeExited'),
    HighlightNodeRequested: Symbol('HighlightNodeRequested'),
    ScreenshotRequested: Symbol('ScreenshotRequested'),
    PersistentGridOverlayCleared: Symbol('PersistentGridOverlayCleared'),
    PersistentGridOverlayStateChanged: Symbol('PersistentGridOverlayStateChanged'),
};
/**
 * @interface
 */
class Highlighter {
    /**
     * @param {!HighlightData} data
     * @param {!Protocol.Overlay.HighlightConfig} config
     */
    highlightInOverlay(data, config) {
    }
    /**
     * @param {!Protocol.Overlay.InspectMode} mode
     * @param {!Protocol.Overlay.HighlightConfig} config
     * @return {!Promise<void>}
     */
    setInspectMode(mode, config) {
        throw Error('Unimplemented method');
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     */
    highlightFrame(frameId) { }
}
/**
 * @implements {Highlighter}
 */
class DefaultHighlighter {
    /**
     * @param {!OverlayModel} model
     */
    constructor(model) {
        this._model = model;
    }
    /**
     * @override
     * @param {!HighlightData} data
     * @param {!Protocol.Overlay.HighlightConfig} highlightConfig
     */
    highlightInOverlay(data, highlightConfig) {
        const { node, deferredNode, object, selectorList } = { node: undefined, deferredNode: undefined, object: undefined, selectorList: undefined, ...data };
        const nodeId = node ? node.id : undefined;
        const backendNodeId = deferredNode ? deferredNode.backendNodeId() : undefined;
        const objectId = object ? object.objectId : undefined;
        if (nodeId || backendNodeId || objectId) {
            this._model.target().overlayAgent().invoke_highlightNode({ highlightConfig, nodeId, backendNodeId, objectId, selector: selectorList });
        }
        else {
            this._model.target().overlayAgent().invoke_hideHighlight();
        }
    }
    /**
     * @override
     * @param {!Protocol.Overlay.InspectMode} mode
     * @param {!Protocol.Overlay.HighlightConfig} highlightConfig
     * @return {!Promise<void>}
     */
    async setInspectMode(mode, highlightConfig) {
        await this._model.target().overlayAgent().invoke_setInspectMode({ mode, highlightConfig });
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameId} frameId
     */
    highlightFrame(frameId) {
        this._model.target().overlayAgent().invoke_highlightFrame({
            frameId,
            contentColor: Color.PageHighlight.Content.toProtocolRGBA(),
            contentOutlineColor: Color.PageHighlight.ContentOutline.toProtocolRGBA()
        });
    }
}
/**
 * @interface
 */
class PersistentGridHighlighter {
    /**
     * @param {number} nodeId
     * @param {!Protocol.Overlay.GridHighlightConfig} config
     */
    highlightInOverlay(nodeId, config) {
    }
    /**
     * @param {number} nodeId
     */
    hideInOverlay(nodeId) {
    }
    hideAllInOverlay() {
    }
    refreshHighlights() {
    }
    /**
     * @param {number} nodeId
     * @return {boolean}
     */
    isHighlighted(nodeId) {
        return false;
    }
}
/**
 * @implements {PersistentGridHighlighter}
 */
class DefaultPersistentGridHighlighter {
    /**
     * @param {!OverlayModel} model
     */
    constructor(model) {
        this._model = model;
        /** @type {!Map<number, !Protocol.Overlay.GridHighlightConfig>} */
        this._gridHighlights = new Map();
        /** @type {!Map<number, !Common.Color.Color>} */
        this._gridColors = new Map();
        this._colorGenerator = new OverlayColorGenerator();
        /** @type {!Common.Settings.Setting<*>} */
        this._showGridLineLabelsSetting = Settings.Settings.instance().moduleSetting('showGridLineLabels');
        this._showGridLineLabelsSetting.addChangeListener(this._onSettingChange, this);
        /** @type {!Common.Settings.Setting<*>} */
        this._extendGridLinesSetting = Settings.Settings.instance().moduleSetting('extendGridLines');
        this._extendGridLinesSetting.addChangeListener(this._onSettingChange, this);
        /** @type {!Common.Settings.Setting<*>} */
        this._showGridAreasSetting = Settings.Settings.instance().moduleSetting('showGridAreas');
        this._showGridAreasSetting.addChangeListener(this._onSettingChange, this);
        /** @type {!Common.Settings.Setting<*>} */
        this._showGridTrackSizesSetting = Settings.Settings.instance().moduleSetting('showGridTrackSizes');
        this._showGridTrackSizesSetting.addChangeListener(this._onSettingChange, this);
        this._logCurrentGridSettings();
        // Debounce recording highlighted grids in order to avoid counting rapidly turning grids on and off.
        this._recordHighlightedGridCount = Debouncer.debounce(this._doRecordHighlightedGridCount.bind(this), 1000);
        /** @type {!Array<number>} */
        this._previouslyRecordedGridCountNodeIds = [];
    }
    /**
     * @returns {boolean}
     */
    static getGridTelemetryLogged() {
        return DefaultPersistentGridHighlighter.gridTelemetryLogged;
    }
    /**
     * @param {boolean} isLogged
     */
    static setGridTelemetryLogged(isLogged) {
        DefaultPersistentGridHighlighter.gridTelemetryLogged = isLogged;
    }
    _onSettingChange() {
        this._resetOverlay();
    }
    _logCurrentGridSettings() {
        if (DefaultPersistentGridHighlighter.getGridTelemetryLogged()) {
            return;
        }
        this._recordGridSetting(this._showGridLineLabelsSetting);
        this._recordGridSetting(this._extendGridLinesSetting);
        this._recordGridSetting(this._showGridAreasSetting);
        this._recordGridSetting(this._showGridTrackSizesSetting);
        DefaultPersistentGridHighlighter.setGridTelemetryLogged(true);
    }
    /**
     * @param {?Common.Settings.Setting<*>} setting
     */
    _recordGridSetting(setting) {
        if (!setting) {
            return;
        }
        userMetrics.cssGridSettings(`${setting.name}.${setting.get()}`);
    }
    _doRecordHighlightedGridCount() {
        const recordedNodeIds = [...this._gridHighlights.keys()];
        // If only settings changed, but not the list of highlighted grids, bail out.
        if (arraysEqual(recordedNodeIds, this._previouslyRecordedGridCountNodeIds)) {
            return;
        }
        userMetrics.highlightedPersistentCssGridCount(recordedNodeIds.length);
        this._previouslyRecordedGridCountNodeIds = recordedNodeIds;
    }
    /**
     * @param {number} nodeId
     * @return {!Protocol.Overlay.GridHighlightConfig}
     */
    _buildGridHighlightConfig(nodeId) {
        const mainColor = this.colorOfGrid(nodeId);
        const background = mainColor.setAlpha(0.1);
        const gapBackground = mainColor.setAlpha(0.3);
        const gapHatch = mainColor.setAlpha(0.8);
        const showGridExtensionLines = /** @type {boolean} */ (this._extendGridLinesSetting.get());
        const showPositiveLineNumbers = this._showGridLineLabelsSetting.get() === 'lineNumbers';
        const showNegativeLineNumbers = showPositiveLineNumbers;
        const showLineNames = this._showGridLineLabelsSetting.get() === 'lineNames';
        return {
            rowGapColor: gapBackground.toProtocolRGBA(),
            rowHatchColor: gapHatch.toProtocolRGBA(),
            columnGapColor: gapBackground.toProtocolRGBA(),
            columnHatchColor: gapHatch.toProtocolRGBA(),
            gridBorderColor: mainColor.toProtocolRGBA(),
            gridBorderDash: false,
            rowLineColor: mainColor.toProtocolRGBA(),
            columnLineColor: mainColor.toProtocolRGBA(),
            rowLineDash: true,
            columnLineDash: true,
            showGridExtensionLines,
            showPositiveLineNumbers,
            showNegativeLineNumbers,
            showLineNames,
            showAreaNames: /** @type {boolean} */ (this._showGridAreasSetting.get()),
            showTrackSizes: /** @type {boolean} */ (this._showGridTrackSizesSetting.get()),
            areaBorderColor: mainColor.toProtocolRGBA(),
            gridBackgroundColor: background.toProtocolRGBA(),
        };
    }
    /**
     * @override
     * @param {number} nodeId
     */
    highlightInOverlay(nodeId) {
        this._gridHighlights.set(nodeId, this._buildGridHighlightConfig(nodeId));
        this._updateHighlightsInOverlay();
    }
    /**
     * @override
     * @param {number} nodeId
     * @return {boolean}
     */
    isHighlighted(nodeId) {
        return this._gridHighlights.has(nodeId);
    }
    /**
     * @param {number} nodeId
     * @return {!Common.Color.Color}
     */
    colorOfGrid(nodeId) {
        let color = this._gridColors.get(nodeId);
        if (!color) {
            color = this._colorGenerator.next();
            this._gridColors.set(nodeId, color);
        }
        return color;
    }
    /**
     * @param {number} nodeId
     * @param {!Common.Color.Color} color
     */
    setColorOfGrid(nodeId, color) {
        this._gridColors.set(nodeId, color);
    }
    /**
     * @override
     * @param {number} nodeId
     */
    hideInOverlay(nodeId) {
        if (this._gridHighlights.has(nodeId)) {
            this._gridHighlights.delete(nodeId);
            this._updateHighlightsInOverlay();
        }
    }
    /**
     * @override
     */
    hideAllInOverlay() {
        this._gridHighlights.clear();
        this._updateHighlightsInOverlay();
    }
    /**
     * @override
     */
    refreshHighlights() {
        let needsUpdate = false;
        for (const nodeId of this._gridHighlights.keys()) {
            if (this._model.getDOMModel().nodeForId(nodeId) === null) {
                this._gridHighlights.delete(nodeId);
                needsUpdate = true;
            }
        }
        if (needsUpdate) {
            this._updateHighlightsInOverlay();
        }
    }
    _resetOverlay() {
        for (const nodeId of this._gridHighlights.keys()) {
            this._gridHighlights.set(nodeId, this._buildGridHighlightConfig(nodeId));
        }
        this._updateHighlightsInOverlay();
    }
    _updateHighlightsInOverlay() {
        const hasGridNodesToHighlight = this._gridHighlights.size > 0;
        this._model.setShowViewportSizeOnResize(!hasGridNodesToHighlight);
        const overlayModel = this._model;
        const gridNodeHighlightConfigs = [];
        for (const [nodeId, gridHighlightConfig] of this._gridHighlights.entries()) {
            gridNodeHighlightConfigs.push({ nodeId, gridHighlightConfig });
        }
        overlayModel.target().overlayAgent().invoke_setShowGridOverlays({ gridNodeHighlightConfigs });
        this._recordHighlightedGridCount();
    }
}
DefaultPersistentGridHighlighter.gridTelemetryLogged = false;
/**
 * Used to cycle through a list of predetermined colors for the grid overlay.
 * This helps users differentiate between overlays when several are shown at the
 * same time.
 */
class OverlayColorGenerator {
    constructor() {
        this._colors = [
            // F59794
            new Color.Color([0.9607843137254902, 0.592156862745098, 0.5803921568627451, 1], Color.Format.RGBA),
            // F0BF4C
            new Color.Color([0.9411764705882353, 0.7490196078431373, 0.2980392156862745, 1], Color.Format.RGBA),
            // D4ED31
            new Color.Color([0.8313725490196079, 0.9294117647058824, 0.19215686274509805, 1], Color.Format.RGBA),
            // 9EEB47
            new Color.Color([0.6196078431372549, 0.9215686274509803, 0.2784313725490196, 1], Color.Format.RGBA),
            // 5BD1D7
            new Color.Color([0.3568627450980392, 0.8196078431372549, 0.8431372549019608, 1], Color.Format.RGBA),
            // BCCEFB
            new Color.Color([0.7372549019607844, 0.807843137254902, 0.984313725490196, 1], Color.Format.RGBA),
            // C6BEEE
            new Color.Color([0.7764705882352941, 0.7450980392156863, 0.9333333333333333, 1], Color.Format.RGBA),
            // D094EA
            new Color.Color([0.8156862745098039, 0.5803921568627451, 0.9176470588235294, 1], Color.Format.RGBA),
            // EB94CF
            new Color.Color([0.9215686274509803, 0.5803921568627451, 0.8117647058823529, 1], Color.Format.RGBA),
        ];
        this._index = 0;
    }
    /**
     * Generate the next color in the spectrum
     * @return {!Common.Color.Color}
     */
    next() {
        const color = this._colors[this._index];
        this._index++;
        if (this._index >= this._colors.length) {
            this._index = 0;
        }
        return color;
    }
}
class SourceOrderHighlighter {
    /**
     * @param {!OverlayModel} model
     */
    constructor(model) {
        this._model = model;
    }
    /**
     * @param {!DOMNode} node
     * @param {!Protocol.Overlay.SourceOrderConfig} sourceOrderConfig
     */
    highlightSourceOrderInOverlay(node, sourceOrderConfig) {
        this._model.setSourceOrderActive(true);
        this._model.setShowViewportSizeOnResize(false);
        this._model._overlayAgent.invoke_highlightSourceOrder({ sourceOrderConfig, nodeId: node.id });
    }
    hideSourceOrderHighlight() {
        this._model.setSourceOrderActive(false);
        this._model.setShowViewportSizeOnResize(true);
        this._model.clearHighlight();
    }
}
SDKModel.register(OverlayModel, Capability.DOM, true);
/** @typedef {!{node: !DOMNode, selectorList: (string|undefined)} | !{deferredNode: DeferredDOMNode, selectorList: (string|undefined)} | !{object: !RemoteObject, selectorList: (string|undefined)} | !{clear: *}} */
// @ts-ignore typedef
let HighlightData;
/**
 * Checks if 2 arrays are equal.
 * @param {!Array<*>} a1 The first array
 * @param {!Array<*>} a2 The second array
 * @return {boolean}
 */
const arraysEqual = (a1, a2) => {
    if (a1.length !== a2.length) {
        return false;
    }
    a1 = [...a1].sort();
    a2 = [...a2].sort();
    for (let i = 0; i < a1.length; i++) {
        if (a1[i] !== a2[i]) {
            return false;
        }
    }
    return true;
};

var OverlayModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    HighlightColor: HighlightColor,
    HighlightRect: HighlightRect,
    Hinge: Hinge,
    OverlayModel: OverlayModel,
    Events: Events$7,
    Highlighter: Highlighter,
    PersistentGridHighlighter: PersistentGridHighlighter,
    SourceOrderHighlighter: SourceOrderHighlighter,
    HighlightData: HighlightData
});

/*
 * Copyright (C) 2009, 2010 Google Inc. All rights reserved.
 * Copyright (C) 2009 Joseph Pecoraro
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @unrestricted
 */
class DOMNode {
    /**
     * @param {!DOMModel} domModel
     */
    constructor(domModel) {
        this._domModel = domModel;
    }
    /**
     * @param {!DOMModel} domModel
     * @param {?DOMDocument} doc
     * @param {boolean} isInShadowTree
     * @param {!Protocol.DOM.Node} payload
     * @return {!DOMNode}
     */
    static create(domModel, doc, isInShadowTree, payload) {
        const node = new DOMNode(domModel);
        node._init(doc, isInShadowTree, payload);
        return node;
    }
    /**
     * @param {?DOMDocument} doc
     * @param {boolean} isInShadowTree
     * @param {!Protocol.DOM.Node} payload
     */
    _init(doc, isInShadowTree, payload) {
        this._agent = this._domModel._agent;
        this.ownerDocument = doc;
        this._isInShadowTree = isInShadowTree;
        this.id = payload.nodeId;
        this._backendNodeId = payload.backendNodeId;
        this._domModel._idToDOMNode[this.id] = this;
        this._nodeType = payload.nodeType;
        this._nodeName = payload.nodeName;
        this._localName = payload.localName;
        this._nodeValue = payload.nodeValue;
        this._pseudoType = payload.pseudoType;
        this._shadowRootType = payload.shadowRootType;
        this._frameOwnerFrameId = payload.frameId || null;
        this._xmlVersion = payload.xmlVersion;
        this._isSVGNode = !!payload.isSVG;
        this._creationStackTrace = null;
        this._shadowRoots = [];
        /** @type {!Map<string, !Attribute>} */
        this._attributes = new Map();
        if (payload.attributes) {
            this._setAttributesPayload(payload.attributes);
        }
        /** @type {!Map<string, ?>} */
        this._markers = new Map();
        this._subtreeMarkerCount = 0;
        this._childNodeCount = payload.childNodeCount || 0;
        this._children = null;
        this.nextSibling = null;
        this.previousSibling = null;
        this.firstChild = null;
        this.lastChild = null;
        this.parentNode = null;
        if (payload.shadowRoots) {
            for (let i = 0; i < payload.shadowRoots.length; ++i) {
                const root = payload.shadowRoots[i];
                const node = DOMNode.create(this._domModel, this.ownerDocument, true, root);
                this._shadowRoots.push(node);
                node.parentNode = this;
            }
        }
        if (payload.templateContent) {
            this._templateContent = DOMNode.create(this._domModel, this.ownerDocument, true, payload.templateContent);
            this._templateContent.parentNode = this;
            this._children = [];
        }
        if (payload.contentDocument) {
            this._contentDocument = new DOMDocument(this._domModel, payload.contentDocument);
            this._contentDocument.parentNode = this;
            this._children = [];
        }
        else if ((payload.nodeName === 'IFRAME' || payload.nodeName === 'PORTAL') && payload.frameId) {
            const childTarget = TargetManager.instance().targetById(payload.frameId);
            const childModel = childTarget ? childTarget.model(DOMModel) : null;
            if (childModel) {
                this._childDocumentPromiseForTesting = childModel.requestDocument();
            }
            this._children = [];
        }
        if (payload.importedDocument) {
            this._importedDocument = DOMNode.create(this._domModel, this.ownerDocument, true, payload.importedDocument);
            this._importedDocument.parentNode = this;
            this._children = [];
        }
        if (payload.distributedNodes) {
            this._setDistributedNodePayloads(payload.distributedNodes);
        }
        if (payload.children) {
            this._setChildrenPayload(payload.children);
        }
        this._setPseudoElements(payload.pseudoElements);
        if (this._nodeType === Node.ELEMENT_NODE) {
            // HTML and BODY from internal iframes should not overwrite top-level ones.
            if (this.ownerDocument && !this.ownerDocument.documentElement && this._nodeName === 'HTML') {
                this.ownerDocument.documentElement = this;
            }
            if (this.ownerDocument && !this.ownerDocument.body && this._nodeName === 'BODY') {
                this.ownerDocument.body = this;
            }
        }
        else if (this._nodeType === Node.DOCUMENT_TYPE_NODE) {
            this.publicId = payload.publicId;
            this.systemId = payload.systemId;
            this.internalSubset = payload.internalSubset;
        }
        else if (this._nodeType === Node.ATTRIBUTE_NODE) {
            this.name = payload.name;
            this.value = payload.value;
        }
    }
    /**
     * @return {boolean}
     */
    isAdFrameNode() {
        if (this.isIframe() && this._frameOwnerFrameId) {
            const frame = FrameManager.instance().getFrame(this._frameOwnerFrameId);
            if (!frame) {
                return false;
            }
            return frame.adFrameType() !== Protocol.Page.AdFrameType.None;
        }
        return false;
    }
    /**
     * @return {boolean}
     */
    isSVGNode() {
        return this._isSVGNode;
    }
    /**
     * @return {!Promise<?Protocol.Runtime.StackTrace>}
     */
    creationStackTrace() {
        if (this._creationStackTrace) {
            return this._creationStackTrace;
        }
        const stackTracesPromise = this._agent.invoke_getNodeStackTraces({ nodeId: this.id });
        this._creationStackTrace = stackTracesPromise.then(res => res.creation);
        return this._creationStackTrace;
    }
    /**
     * @return {!DOMModel}
     */
    domModel() {
        return this._domModel;
    }
    /**
     * @return {number}
     */
    backendNodeId() {
        return this._backendNodeId;
    }
    /**
     * @return {?Array.<!DOMNode>}
     */
    children() {
        return this._children ? this._children.slice() : null;
    }
    /**
     * @return {boolean}
     */
    hasAttributes() {
        return this._attributes.size > 0;
    }
    /**
     * @return {number}
     */
    childNodeCount() {
        return this._childNodeCount;
    }
    /**
     * @return {boolean}
     */
    hasShadowRoots() {
        return !!this._shadowRoots.length;
    }
    /**
     * @return {!Array.<!DOMNode>}
     */
    shadowRoots() {
        return this._shadowRoots.slice();
    }
    /**
     * @return {?DOMNode}
     */
    templateContent() {
        return this._templateContent || null;
    }
    /**
     * @return {?DOMNode}
     */
    contentDocument() {
        return this._contentDocument || null;
    }
    /**
     * @return {boolean}
     */
    isIframe() {
        return this._nodeName === 'IFRAME';
    }
    /**
     * @return {boolean}
     */
    isPortal() {
        return this._nodeName === 'PORTAL';
    }
    /**
     * @return {?DOMNode}
     */
    importedDocument() {
        return this._importedDocument || null;
    }
    /**
     * @return {number}
     */
    nodeType() {
        return this._nodeType;
    }
    /**
     * @return {string}
     */
    nodeName() {
        return this._nodeName;
    }
    /**
     * @return {string|undefined}
     */
    pseudoType() {
        return this._pseudoType;
    }
    /**
     * @return {boolean}
     */
    hasPseudoElements() {
        return this._pseudoElements.size > 0;
    }
    /**
     * @return {!Map<string, !DOMNode>}
     */
    pseudoElements() {
        return this._pseudoElements;
    }
    /**
     * @return {?DOMNode}
     */
    beforePseudoElement() {
        if (!this._pseudoElements) {
            return null;
        }
        return this._pseudoElements.get(DOMNode.PseudoElementNames.Before);
    }
    /**
     * @return {?DOMNode}
     */
    afterPseudoElement() {
        if (!this._pseudoElements) {
            return null;
        }
        return this._pseudoElements.get(DOMNode.PseudoElementNames.After);
    }
    /**
     * @return {?DOMNode}
     */
    markerPseudoElement() {
        if (!this._pseudoElements) {
            return null;
        }
        return this._pseudoElements.get(DOMNode.PseudoElementNames.Marker);
    }
    /**
     * @return {boolean}
     */
    isInsertionPoint() {
        return !this.isXMLNode() &&
            (this._nodeName === 'SHADOW' || this._nodeName === 'CONTENT' || this._nodeName === 'SLOT');
    }
    /**
     * @return {!Array.<!DOMNodeShortcut>}
     */
    distributedNodes() {
        return this._distributedNodes || [];
    }
    /**
     * @return {boolean}
     */
    isInShadowTree() {
        return this._isInShadowTree;
    }
    /**
     * @return {?DOMNode}
     */
    ancestorShadowHost() {
        const ancestorShadowRoot = this.ancestorShadowRoot();
        return ancestorShadowRoot ? ancestorShadowRoot.parentNode : null;
    }
    /**
     * @return {?DOMNode}
     */
    ancestorShadowRoot() {
        if (!this._isInShadowTree) {
            return null;
        }
        let current = this;
        while (current && !current.isShadowRoot()) {
            current = current.parentNode;
        }
        return current;
    }
    /**
     * @return {?DOMNode}
     */
    ancestorUserAgentShadowRoot() {
        const ancestorShadowRoot = this.ancestorShadowRoot();
        if (!ancestorShadowRoot) {
            return null;
        }
        return ancestorShadowRoot.shadowRootType() === DOMNode.ShadowRootTypes.UserAgent ? ancestorShadowRoot : null;
    }
    /**
     * @return {boolean}
     */
    isShadowRoot() {
        return !!this._shadowRootType;
    }
    /**
     * @return {?string}
     */
    shadowRootType() {
        return this._shadowRootType || null;
    }
    /**
     * @return {string}
     */
    nodeNameInCorrectCase() {
        const shadowRootType = this.shadowRootType();
        if (shadowRootType) {
            return '#shadow-root (' + shadowRootType + ')';
        }
        // If there is no local name, it's case sensitive
        if (!this.localName()) {
            return this.nodeName();
        }
        // If the names are different lengths, there is a prefix and it's case sensitive
        if (this.localName().length !== this.nodeName().length) {
            return this.nodeName();
        }
        // Return the localname, which will be case insensitive if its an html node
        return this.localName();
    }
    /**
     * @param {string} name
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, ?DOMNode)=} callback
     */
    setNodeName(name, callback) {
        this._agent.invoke_setNodeName({ nodeId: this.id, name }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null, this._domModel.nodeForId(response.nodeId));
            }
        });
    }
    /**
     * @return {string}
     */
    localName() {
        return this._localName;
    }
    /**
     * @return {string}
     */
    nodeValue() {
        return this._nodeValue;
    }
    /**
     * @param {string} value
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError)=} callback
     */
    setNodeValue(value, callback) {
        this._agent.invoke_setNodeValue({ nodeId: this.id, value }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null);
            }
        });
    }
    /**
     * @param {string} name
     * @return {string|undefined}
     */
    getAttribute(name) {
        const attr = this._attributes.get(name);
        return attr ? attr.value : undefined;
    }
    /**
     * @param {string} name
     * @param {string} text
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError)=} callback
     */
    setAttribute(name, text, callback) {
        this._agent.invoke_setAttributesAsText({ nodeId: this.id, text, name }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null);
            }
        });
    }
    /**
     * @param {string} name
     * @param {string} value
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError)=} callback
     */
    setAttributeValue(name, value, callback) {
        this._agent.invoke_setAttributeValue({ nodeId: this.id, name, value }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null);
            }
        });
    }
    /**
    * @param {string} name
    * @param {string} value
    * @return {!Promise<?ProtocolClient.InspectorBackend.ProtocolError>}
    */
    setAttributeValuePromise(name, value) {
        return new Promise(fulfill => this.setAttributeValue(name, value, fulfill));
    }
    /**
     * @return {!Array<!Attribute>}
     */
    attributes() {
        return [...this._attributes.values()];
    }
    /**
     * @param {string} name
     * @return {!Promise<void>}
     */
    async removeAttribute(name) {
        const response = await this._agent.invoke_removeAttribute({ nodeId: this.id, name });
        if (response[InspectorBackend.ProtocolError]) {
            return;
        }
        this._attributes.delete(name);
        this._domModel.markUndoableState();
    }
    /**
     * @param {function(?Array<!DOMNode>):void} callback
     */
    getChildNodes(callback) {
        if (this._children) {
            callback(this.children());
            return;
        }
        this._agent.invoke_requestChildNodes({ nodeId: this.id }).then(response => {
            callback(response[InspectorBackend.ProtocolError] ? null : this.children());
        });
    }
    /**
     * @param {number} depth
     * @param {boolean} pierce
     * @return {!Promise<?Array<!DOMNode>>}
     */
    async getSubtree(depth, pierce) {
        const response = await this._agent.invoke_requestChildNodes({ nodeId: this.id, depth: depth, pierce: pierce });
        return response[InspectorBackend.ProtocolError] ? null : this._children;
    }
    /**
     * @return {!Promise<?string>}
     */
    getOuterHTML() {
        return this._agent.getOuterHTML(this.id);
    }
    /**
     * @param {string} html
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError)=} callback
     */
    setOuterHTML(html, callback) {
        this._agent.invoke_setOuterHTML({ nodeId: this.id, outerHTML: html }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null);
            }
        });
    }
    /**
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, !Protocol.DOM.NodeId=)=} callback
     */
    removeNode(callback) {
        this._agent.invoke_removeNode({ nodeId: this.id }).then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null);
            }
        });
    }
    /**
     * @return {!Promise<?string>}
     */
    async copyNode() {
        const text = await this._agent.getOuterHTML(this.id);
        if (text !== null) {
            InspectorFrontendHost.InspectorFrontendHostInstance.copyText(text);
        }
        return text;
    }
    /**
     * @return {string}
     */
    path() {
        /**
         * @param {?DOMNode} node
         */
        function canPush(node) {
            return node && ('index' in node || (node.isShadowRoot() && node.parentNode)) && node._nodeName.length;
        }
        const path = [];
        let node = this;
        while (canPush(node)) {
            const index = typeof node.index === 'number' ?
                node.index :
                (node.shadowRootType() === DOMNode.ShadowRootTypes.UserAgent ? 'u' : 'a');
            path.push([index, node._nodeName]);
            node = node.parentNode;
        }
        path.reverse();
        return path.join(',');
    }
    /**
     * @param {!DOMNode} node
     * @return {boolean}
     */
    isAncestor(node) {
        if (!node) {
            return false;
        }
        let currentNode = node.parentNode;
        while (currentNode) {
            if (this === currentNode) {
                return true;
            }
            currentNode = currentNode.parentNode;
        }
        return false;
    }
    /**
     * @param {!DOMNode} descendant
     * @return {boolean}
     */
    isDescendant(descendant) {
        return descendant !== null && descendant.isAncestor(this);
    }
    /**
     * @return {?Protocol.Page.FrameId}
     */
    frameId() {
        let node = this.parentNode || this;
        while (!node._frameOwnerFrameId && node.parentNode) {
            node = node.parentNode;
        }
        return node._frameOwnerFrameId;
    }
    /**
     * @param {!Array.<string>} attrs
     * @return {boolean}
     */
    _setAttributesPayload(attrs) {
        let attributesChanged = !this._attributes || attrs.length !== this._attributes.size * 2;
        const oldAttributesMap = this._attributes || new Map();
        this._attributes = new Map();
        for (let i = 0; i < attrs.length; i += 2) {
            const name = attrs[i];
            const value = attrs[i + 1];
            this._addAttribute(name, value);
            if (attributesChanged) {
                continue;
            }
            if (!oldAttributesMap.has(name) || oldAttributesMap.get(name).value !== value) {
                attributesChanged = true;
            }
        }
        return attributesChanged;
    }
    /**
     * @param {!DOMNode} prev
     * @param {!Protocol.DOM.Node} payload
     * @return {!DOMNode}
     */
    _insertChild(prev, payload) {
        const node = DOMNode.create(this._domModel, this.ownerDocument, this._isInShadowTree, payload);
        this._children.splice(this._children.indexOf(prev) + 1, 0, node);
        this._renumber();
        return node;
    }
    /**
     * @param {!DOMNode} node
     */
    _removeChild(node) {
        if (node.pseudoType()) {
            this._pseudoElements.delete(node.pseudoType());
        }
        else {
            const shadowRootIndex = this._shadowRoots.indexOf(node);
            if (shadowRootIndex !== -1) {
                this._shadowRoots.splice(shadowRootIndex, 1);
            }
            else {
                console.assert(this._children.indexOf(node) !== -1);
                this._children.splice(this._children.indexOf(node), 1);
            }
        }
        node.parentNode = null;
        this._subtreeMarkerCount -= node._subtreeMarkerCount;
        if (node._subtreeMarkerCount) {
            this._domModel.dispatchEventToListeners(Events$8.MarkersChanged, this);
        }
        this._renumber();
    }
    /**
     * @param {!Array.<!Protocol.DOM.Node>} payloads
     */
    _setChildrenPayload(payloads) {
        this._children = [];
        for (let i = 0; i < payloads.length; ++i) {
            const payload = payloads[i];
            const node = DOMNode.create(this._domModel, this.ownerDocument, this._isInShadowTree, payload);
            this._children.push(node);
        }
        this._renumber();
    }
    /**
     * @param {!Array.<!Protocol.DOM.Node>|undefined} payloads
     */
    _setPseudoElements(payloads) {
        this._pseudoElements = new Map();
        if (!payloads) {
            return;
        }
        for (let i = 0; i < payloads.length; ++i) {
            const node = DOMNode.create(this._domModel, this.ownerDocument, this._isInShadowTree, payloads[i]);
            node.parentNode = this;
            this._pseudoElements.set(node.pseudoType(), node);
        }
    }
    /**
     * @param {!Array.<!Protocol.DOM.BackendNode>} payloads
     */
    _setDistributedNodePayloads(payloads) {
        this._distributedNodes = [];
        for (const payload of payloads) {
            this._distributedNodes.push(new DOMNodeShortcut(this._domModel.target(), payload.backendNodeId, payload.nodeType, payload.nodeName));
        }
    }
    _renumber() {
        this._childNodeCount = this._children.length;
        if (this._childNodeCount === 0) {
            this.firstChild = null;
            this.lastChild = null;
            return;
        }
        this.firstChild = this._children[0];
        this.lastChild = this._children[this._childNodeCount - 1];
        for (let i = 0; i < this._childNodeCount; ++i) {
            const child = this._children[i];
            child.index = i;
            child.nextSibling = i + 1 < this._childNodeCount ? this._children[i + 1] : null;
            child.previousSibling = i - 1 >= 0 ? this._children[i - 1] : null;
            child.parentNode = this;
        }
    }
    /**
     * @param {string} name
     * @param {string} value
     */
    _addAttribute(name, value) {
        const attr = { name: name, value: value, _node: this };
        this._attributes.set(name, attr);
    }
    /**
     * @param {string} name
     * @param {string} value
     */
    _setAttribute(name, value) {
        const attr = this._attributes.get(name);
        if (attr) {
            attr.value = value;
        }
        else {
            this._addAttribute(name, value);
        }
    }
    /**
     * @param {string} name
     */
    _removeAttribute(name) {
        this._attributes.delete(name);
    }
    /**
     * @param {!DOMNode} targetNode
     * @param {?DOMNode} anchorNode
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, !Protocol.DOM.NodeId=)=} callback
     */
    copyTo(targetNode, anchorNode, callback) {
        this._agent
            .invoke_copyTo({ nodeId: this.id, targetNodeId: targetNode.id, insertBeforeNodeId: anchorNode ? anchorNode.id : undefined })
            .then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null, response.nodeId);
            }
        });
    }
    /**
     * @param {!DOMNode} targetNode
     * @param {?DOMNode} anchorNode
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, ?SDK.DOMNode)=} callback
     */
    moveTo(targetNode, anchorNode, callback) {
        this._agent
            .invoke_moveTo({ nodeId: this.id, targetNodeId: targetNode.id, insertBeforeNodeId: anchorNode ? anchorNode.id : undefined })
            .then(response => {
            if (!response[InspectorBackend.ProtocolError]) {
                this._domModel.markUndoableState();
            }
            if (callback) {
                callback(response[InspectorBackend.ProtocolError] || null, this._domModel.nodeForId(response.nodeId));
            }
        });
    }
    /**
     * @return {boolean}
     */
    isXMLNode() {
        return !!this._xmlVersion;
    }
    /**
     * @param {string} name
     * @param {?*} value
     */
    setMarker(name, value) {
        if (value === null) {
            if (!this._markers.has(name)) {
                return;
            }
            this._markers.delete(name);
            for (let node = this; node; node = node.parentNode) {
                --node._subtreeMarkerCount;
            }
            for (let node = this; node; node = node.parentNode) {
                this._domModel.dispatchEventToListeners(Events$8.MarkersChanged, node);
            }
            return;
        }
        if (this.parentNode && !this._markers.has(name)) {
            for (let node = this; node; node = node.parentNode) {
                ++node._subtreeMarkerCount;
            }
        }
        this._markers.set(name, value);
        for (let node = this; node; node = node.parentNode) {
            this._domModel.dispatchEventToListeners(Events$8.MarkersChanged, node);
        }
    }
    /**
     * @param {string} name
     * @return {?T}
     * @template T
     */
    marker(name) {
        return this._markers.get(name) || null;
    }
    /**
     * @param {function(!DOMNode, string):void} visitor
     */
    traverseMarkers(visitor) {
        /**
         * @param {!DOMNode} node
         */
        function traverse(node) {
            if (!node._subtreeMarkerCount) {
                return;
            }
            for (const marker of node._markers.keys()) {
                visitor(node, marker);
            }
            if (!node._children) {
                return;
            }
            for (const child of node._children) {
                traverse(child);
            }
        }
        traverse(this);
    }
    /**
     * @param {string} url
     * @return {?string}
     */
    resolveURL(url) {
        if (!url) {
            return url;
        }
        for (let frameOwnerCandidate = this; frameOwnerCandidate; frameOwnerCandidate = frameOwnerCandidate.parentNode) {
            if (frameOwnerCandidate.baseURL) {
                return ParsedURL.ParsedURL.completeURL(frameOwnerCandidate.baseURL, url);
            }
        }
        return null;
    }
    /**
     * @param {string=} mode
     */
    highlight(mode) {
        this._domModel.overlayModel().highlightInOverlay({ node: this }, mode);
    }
    highlightForTwoSeconds() {
        this._domModel.overlayModel().highlightInOverlayForTwoSeconds({ node: this });
    }
    /**
     * @param {string=} objectGroup
     * @return {!Promise<?RemoteObject>}
     */
    async resolveToObject(objectGroup) {
        const object = await this._agent.resolveNode(this.id, undefined, objectGroup);
        return object && this._domModel._runtimeModel.createRemoteObject(object);
    }
    /**
     * @return {!Promise<?Protocol.DOM.BoxModel>}
     */
    boxModel() {
        return this._agent.getBoxModel(this.id);
    }
    setAsInspectedNode() {
        let node = this;
        if (node.pseudoType()) {
            node = node.parentNode;
        }
        while (true) {
            let ancestor = node.ancestorUserAgentShadowRoot();
            if (!ancestor) {
                break;
            }
            ancestor = node.ancestorShadowHost();
            if (!ancestor) {
                break;
            }
            // User agent shadow root, keep climbing up.
            node = ancestor;
        }
        this._agent.setInspectedNode(node.id);
    }
    /**
     *  @return {?DOMNode}
     */
    enclosingElementOrSelf() {
        let node = this;
        if (node && node.nodeType() === Node.TEXT_NODE && node.parentNode) {
            node = node.parentNode;
        }
        if (node && node.nodeType() !== Node.ELEMENT_NODE) {
            node = null;
        }
        return node;
    }
    async scrollIntoView() {
        const node = this.enclosingElementOrSelf();
        if (!node) {
            return;
        }
        const object = await node.resolveToObject();
        if (!object) {
            return;
        }
        object.callFunction(scrollIntoView);
        object.release();
        node.highlightForTwoSeconds();
        /**
         * @suppressReceiverCheck
         * @this {!Element}
         */
        function scrollIntoView() {
            this.scrollIntoViewIfNeeded(true);
        }
    }
    async focus() {
        const node = this.enclosingElementOrSelf();
        const object = await node.resolveToObject();
        if (!object) {
            return;
        }
        await object.callFunction(focusInPage);
        object.release();
        node.highlightForTwoSeconds();
        this._domModel.target().pageAgent().bringToFront();
        /**
         * @suppressReceiverCheck
         * @this {!Element}
         */
        function focusInPage() {
            this.focus();
        }
    }
    /**
     * @return {string}
     */
    simpleSelector() {
        const lowerCaseName = this.localName() || this.nodeName().toLowerCase();
        if (this.nodeType() !== Node.ELEMENT_NODE) {
            return lowerCaseName;
        }
        if (lowerCaseName === 'input' && this.getAttribute('type') && !this.getAttribute('id') &&
            !this.getAttribute('class')) {
            return lowerCaseName + '[type="' + this.getAttribute('type') + '"]';
        }
        if (this.getAttribute('id')) {
            return lowerCaseName + '#' + this.getAttribute('id');
        }
        if (this.getAttribute('class')) {
            return (lowerCaseName === 'div' ? '' : lowerCaseName) + '.' +
                this.getAttribute('class').trim().replace(/\s+/g, '.');
        }
        return lowerCaseName;
    }
}
/**
 * @enum {string}
 */
DOMNode.PseudoElementNames = {
    Before: 'before',
    After: 'after',
    Marker: 'marker'
};
/**
 * @enum {string}
 */
DOMNode.ShadowRootTypes = {
    UserAgent: 'user-agent',
    Open: 'open',
    Closed: 'closed'
};
class DeferredDOMNode {
    /**
     * @param {!Target} target
     * @param {number} backendNodeId
     */
    constructor(target, backendNodeId) {
        this._domModel = /** @type {!DOMModel} */ (target.model(DOMModel));
        this._backendNodeId = backendNodeId;
    }
    /**
     * @param {function(?DOMNode):void} callback
     */
    resolve(callback) {
        this.resolvePromise().then(callback);
    }
    /**
     * @return {!Promise<?DOMNode>}
     */
    async resolvePromise() {
        const nodeIds = await this._domModel.pushNodesByBackendIdsToFrontend(new Set([this._backendNodeId]));
        return nodeIds && nodeIds.get(this._backendNodeId) || null;
    }
    /**
     * @return {number}
     */
    backendNodeId() {
        return this._backendNodeId;
    }
    /**
     * @return {!DOMModel}
     */
    domModel() {
        return this._domModel;
    }
    highlight() {
        this._domModel.overlayModel().highlightInOverlay({ deferredNode: this });
    }
}
class DOMNodeShortcut {
    /**
     * @param {!Target} target
     * @param {number} backendNodeId
     * @param {number} nodeType
     * @param {string} nodeName
     */
    constructor(target, backendNodeId, nodeType, nodeName) {
        this.nodeType = nodeType;
        this.nodeName = nodeName;
        this.deferredNode = new DeferredDOMNode(target, backendNodeId);
    }
}
/**
 * @unrestricted
 */
class DOMDocument extends DOMNode {
    /**
     * @param {!DOMModel} domModel
     * @param {!Protocol.DOM.Node} payload
     */
    constructor(domModel, payload) {
        super(domModel);
        this.body = null;
        this.documentElement = null;
        this._init(this, false, payload);
        this.documentURL = payload.documentURL || '';
        this.baseURL = payload.baseURL || '';
    }
}
/**
 * @unrestricted
 */
class DOMModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._agent = target.domAgent();
        /** @type {!Object.<number, !DOMNode>} */
        this._idToDOMNode = {};
        /** @type {?DOMDocument} */
        this._document = null;
        /** @type {!Set<number>} */
        this._attributeLoadNodeIds = new Set();
        target.registerDOMDispatcher(new DOMDispatcher(this));
        this._runtimeModel = /** @type {!RuntimeModel} */ (target.model(RuntimeModel));
        /** @type {number} */
        this._lastMutationId;
        /** @type {?Promise<?DOMDocument>} */
        this._pendingDocumentRequestPromise;
        if (!target.suspended()) {
            this._agent.invoke_enable({});
        }
        if (Runtime.experiments.isEnabled('captureNodeCreationStacks')) {
            this._agent.invoke_setNodeStackTracesEnabled({ enable: true });
        }
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    /**
     * @return {!CSSModel}
     */
    cssModel() {
        return /** @type {!CSSModel} */ (this.target().model(CSSModel));
    }
    /**
     * @return {!OverlayModel}
     */
    overlayModel() {
        return /** @type {!OverlayModel} */ (this.target().model(OverlayModel));
    }
    static cancelSearch() {
        for (const domModel of TargetManager.instance().models(DOMModel)) {
            domModel._cancelSearch();
        }
    }
    /**
     * @param {!DOMNode} node
     */
    _scheduleMutationEvent(node) {
        if (!this.hasEventListeners(Events$8.DOMMutated)) {
            return;
        }
        this._lastMutationId = (this._lastMutationId || 0) + 1;
        Promise.resolve().then(callObserve.bind(this, node, this._lastMutationId));
        /**
         * @this {DOMModel}
         * @param {!DOMNode} node
         * @param {number} mutationId
         */
        function callObserve(node, mutationId) {
            if (!this.hasEventListeners(Events$8.DOMMutated) || this._lastMutationId !== mutationId) {
                return;
            }
            this.dispatchEventToListeners(Events$8.DOMMutated, node);
        }
    }
    /**
     * @return {!Promise<?DOMDocument>}
     */
    requestDocument() {
        if (this._document) {
            return Promise.resolve(this._document);
        }
        if (!this._pendingDocumentRequestPromise) {
            this._pendingDocumentRequestPromise = this._requestDocument();
        }
        return this._pendingDocumentRequestPromise;
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     * @returns {!Promise<?DeferredDOMNode>}
     */
    async getOwnerNodeForFrame(frameId) {
        // Returns an error if the frameId does not belong to the current target.
        const response = await this._agent.invoke_getFrameOwner({ frameId });
        if (response.getError()) {
            return null;
        }
        return new DeferredDOMNode(this.target(), response.backendNodeId);
    }
    /**
     * @return {!Promise<?DOMDocument>}
     */
    async _requestDocument() {
        const { root: documentPayload } = await this._agent.invoke_getDocument();
        delete this._pendingDocumentRequestPromise;
        if (documentPayload) {
            this._setDocument(documentPayload);
        }
        if (!this._document) {
            console.error('No document');
            return null;
        }
        const parentModel = this.parentModel();
        if (parentModel && !this._frameOwnerNode) {
            await parentModel.requestDocument();
            const response = await parentModel._agent.invoke_getFrameOwner({ frameId: this.target().id() });
            if (!response.getError() && response.nodeId) {
                this._frameOwnerNode = parentModel.nodeForId(response.nodeId);
            }
        }
        // Document could have been cleared by now.
        if (this._frameOwnerNode) {
            const oldDocument = this._frameOwnerNode._contentDocument;
            this._frameOwnerNode._contentDocument = this._document;
            this._frameOwnerNode._children = [];
            if (this._document) {
                this._document.parentNode = this._frameOwnerNode;
                this.dispatchEventToListeners(Events$8.NodeInserted, this._document);
            }
            else if (oldDocument) {
                this.dispatchEventToListeners(Events$8.NodeRemoved, { node: oldDocument, parent: this._frameOwnerNode });
            }
        }
        return this._document;
    }
    /**
     * @return {?DOMDocument}
     */
    existingDocument() {
        return this._document;
    }
    /**
     * @param {!Protocol.Runtime.RemoteObjectId} objectId
     * @return {!Promise<?DOMNode>}
     */
    async pushNodeToFrontend(objectId) {
        await this.requestDocument();
        const { nodeId } = await this._agent.invoke_requestNode({ objectId });
        return nodeId ? this.nodeForId(nodeId) : null;
    }
    /**
     * @param {string} path
     * @return {!Promise<?Protocol.DOM.NodeId>}
     */
    pushNodeByPathToFrontend(path) {
        return this.requestDocument()
            .then(() => this._agent.invoke_pushNodeByPathToFrontend({ path }))
            .then(({ nodeId }) => nodeId);
    }
    /**
     * @param {!Set<number>} backendNodeIds
     * @return {!Promise<?Map<number, ?DOMNode>>}
     */
    async pushNodesByBackendIdsToFrontend(backendNodeIds) {
        await this.requestDocument();
        const backendNodeIdsArray = [...backendNodeIds];
        const { nodeIds } = await this._agent.invoke_pushNodesByBackendIdsToFrontend({ backendNodeIds: backendNodeIdsArray });
        if (!nodeIds) {
            return null;
        }
        /** @type {!Map<number, ?DOMNode>} */
        const map = new Map();
        for (let i = 0; i < nodeIds.length; ++i) {
            if (nodeIds[i]) {
                map.set(backendNodeIdsArray[i], this.nodeForId(nodeIds[i]));
            }
        }
        return map;
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {string} name
     * @param {string} value
     */
    _attributeModified(nodeId, name, value) {
        const node = this._idToDOMNode[nodeId];
        if (!node) {
            return;
        }
        node._setAttribute(name, value);
        this.dispatchEventToListeners(Events$8.AttrModified, { node: node, name: name });
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {string} name
     */
    _attributeRemoved(nodeId, name) {
        const node = this._idToDOMNode[nodeId];
        if (!node) {
            return;
        }
        node._removeAttribute(name);
        this.dispatchEventToListeners(Events$8.AttrRemoved, { node: node, name: name });
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Array<!Protocol.DOM.NodeId>} nodeIds
     */
    _inlineStyleInvalidated(nodeIds) {
        SetUtilities.addAll(this._attributeLoadNodeIds, nodeIds);
        if (!this._loadNodeAttributesTimeout) {
            this._loadNodeAttributesTimeout = setTimeout(this._loadNodeAttributes.bind(this), 20);
        }
    }
    _loadNodeAttributes() {
        delete this._loadNodeAttributesTimeout;
        for (const nodeId of this._attributeLoadNodeIds) {
            this._agent.invoke_getAttributes({ nodeId }).then(({ attributes }) => {
                if (!attributes) {
                    // We are calling _loadNodeAttributes asynchronously, it is ok if node is not found.
                    return;
                }
                const node = this._idToDOMNode[nodeId];
                if (!node) {
                    return;
                }
                if (node._setAttributesPayload(attributes)) {
                    this.dispatchEventToListeners(Events$8.AttrModified, { node: node, name: 'style' });
                    this._scheduleMutationEvent(node);
                }
            });
        }
        this._attributeLoadNodeIds.clear();
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {string} newValue
     */
    _characterDataModified(nodeId, newValue) {
        const node = this._idToDOMNode[nodeId];
        node._nodeValue = newValue;
        this.dispatchEventToListeners(Events$8.CharacterDataModified, node);
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {?DOMNode}
     */
    nodeForId(nodeId) {
        return this._idToDOMNode[nodeId] || null;
    }
    _documentUpdated() {
        // If we have this._pendingDocumentRequestPromise in flight,
        // if it hits backend post document update, it will contain most recent result.
        const documentWasRequested = this._document || this._pendingDocumentRequestPromise;
        this._setDocument(null);
        if (this.parentModel() && documentWasRequested) {
            this.requestDocument();
        }
    }
    /**
     * @param {?Protocol.DOM.Node} payload
     */
    _setDocument(payload) {
        this._idToDOMNode = {};
        if (payload && 'nodeId' in payload) {
            this._document = new DOMDocument(this, payload);
        }
        else {
            this._document = null;
        }
        DOMModelUndoStack.instance()._dispose(this);
        if (!this.parentModel()) {
            this.dispatchEventToListeners(Events$8.DocumentUpdated, this);
        }
    }
    /**
     * @param {!Protocol.DOM.Node} payload
     */
    _setDetachedRoot(payload) {
        if (payload.nodeName === '#document') {
            new DOMDocument(this, payload);
        }
        else {
            DOMNode.create(this, null, false, payload);
        }
    }
    /**
     * @param {!Protocol.DOM.NodeId} parentId
     * @param {!Array.<!Protocol.DOM.Node>} payloads
     */
    _setChildNodes(parentId, payloads) {
        if (!parentId && payloads.length) {
            this._setDetachedRoot(payloads[0]);
            return;
        }
        const parent = this._idToDOMNode[parentId];
        parent._setChildrenPayload(payloads);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {number} newValue
     */
    _childNodeCountUpdated(nodeId, newValue) {
        const node = this._idToDOMNode[nodeId];
        node._childNodeCount = newValue;
        this.dispatchEventToListeners(Events$8.ChildNodeCountUpdated, node);
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} parentId
     * @param {!Protocol.DOM.NodeId} prevId
     * @param {!Protocol.DOM.Node} payload
     */
    _childNodeInserted(parentId, prevId, payload) {
        const parent = this._idToDOMNode[parentId];
        const prev = this._idToDOMNode[prevId];
        const node = parent._insertChild(prev, payload);
        this._idToDOMNode[node.id] = node;
        this.dispatchEventToListeners(Events$8.NodeInserted, node);
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} parentId
     * @param {!Protocol.DOM.NodeId} nodeId
     */
    _childNodeRemoved(parentId, nodeId) {
        const parent = this._idToDOMNode[parentId];
        const node = this._idToDOMNode[nodeId];
        parent._removeChild(node);
        this._unbind(node);
        this.dispatchEventToListeners(Events$8.NodeRemoved, { node: node, parent: parent });
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} hostId
     * @param {!Protocol.DOM.Node} root
     */
    _shadowRootPushed(hostId, root) {
        const host = this._idToDOMNode[hostId];
        if (!host) {
            return;
        }
        const node = DOMNode.create(this, host.ownerDocument, true, root);
        node.parentNode = host;
        this._idToDOMNode[node.id] = node;
        host._shadowRoots.unshift(node);
        this.dispatchEventToListeners(Events$8.NodeInserted, node);
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} hostId
     * @param {!Protocol.DOM.NodeId} rootId
     */
    _shadowRootPopped(hostId, rootId) {
        const host = this._idToDOMNode[hostId];
        if (!host) {
            return;
        }
        const root = this._idToDOMNode[rootId];
        if (!root) {
            return;
        }
        host._removeChild(root);
        this._unbind(root);
        this.dispatchEventToListeners(Events$8.NodeRemoved, { node: root, parent: host });
        this._scheduleMutationEvent(root);
    }
    /**
     * @param {!Protocol.DOM.NodeId} parentId
     * @param {!Protocol.DOM.Node} pseudoElement
     */
    _pseudoElementAdded(parentId, pseudoElement) {
        const parent = this._idToDOMNode[parentId];
        if (!parent) {
            return;
        }
        const node = DOMNode.create(this, parent.ownerDocument, false, pseudoElement);
        node.parentNode = parent;
        this._idToDOMNode[node.id] = node;
        console.assert(!parent._pseudoElements.get(node.pseudoType()));
        parent._pseudoElements.set(node.pseudoType(), node);
        this.dispatchEventToListeners(Events$8.NodeInserted, node);
        this._scheduleMutationEvent(node);
    }
    /**
     * @param {!Protocol.DOM.NodeId} parentId
     * @param {!Protocol.DOM.NodeId} pseudoElementId
     */
    _pseudoElementRemoved(parentId, pseudoElementId) {
        const parent = this._idToDOMNode[parentId];
        if (!parent) {
            return;
        }
        const pseudoElement = this._idToDOMNode[pseudoElementId];
        if (!pseudoElement) {
            return;
        }
        parent._removeChild(pseudoElement);
        this._unbind(pseudoElement);
        this.dispatchEventToListeners(Events$8.NodeRemoved, { node: pseudoElement, parent: parent });
        this._scheduleMutationEvent(pseudoElement);
    }
    /**
     * @param {!Protocol.DOM.NodeId} insertionPointId
     * @param {!Array.<!Protocol.DOM.BackendNode>} distributedNodes
     */
    _distributedNodesUpdated(insertionPointId, distributedNodes) {
        const insertionPoint = this._idToDOMNode[insertionPointId];
        if (!insertionPoint) {
            return;
        }
        insertionPoint._setDistributedNodePayloads(distributedNodes);
        this.dispatchEventToListeners(Events$8.DistributedNodesChanged, insertionPoint);
        this._scheduleMutationEvent(insertionPoint);
    }
    /**
     * @param {!DOMNode} node
     */
    _unbind(node) {
        delete this._idToDOMNode[node.id];
        for (let i = 0; node._children && i < node._children.length; ++i) {
            this._unbind(node._children[i]);
        }
        for (let i = 0; i < node._shadowRoots.length; ++i) {
            this._unbind(node._shadowRoots[i]);
        }
        const pseudoElements = node.pseudoElements();
        for (const value of pseudoElements.values()) {
            this._unbind(value);
        }
        if (node._templateContent) {
            this._unbind(node._templateContent);
        }
    }
    /**
     * @param {!Array<!{name: string, value: string}>} computedStyles
     * @param {boolean} pierce
     * @return {!Promise<!Array<number>>}
     */
    async getNodesByStyle(computedStyles, pierce = false) {
        await this.requestDocument();
        const response = await this._agent.invoke_getNodesForSubtreeByStyle({ nodeId: this._document.id, computedStyles, pierce });
        if (response.getError()) {
            throw response.getError();
        }
        return response.nodeIds;
    }
    /**
     * @param {string} query
     * @param {boolean} includeUserAgentShadowDOM
     * @return {!Promise<number>}
     */
    async performSearch(query, includeUserAgentShadowDOM) {
        const response = await this._agent.invoke_performSearch({ query, includeUserAgentShadowDOM });
        if (!response.getError()) {
            this._searchId = response.searchId;
        }
        return response.getError() ? 0 : response.resultCount;
    }
    /**
     * @param {number} index
     * @return {!Promise<?DOMNode>}
     */
    async searchResult(index) {
        if (!this._searchId) {
            return null;
        }
        const { nodeIds } = await this._agent.invoke_getSearchResults({ searchId: this._searchId, fromIndex: index, toIndex: index + 1 });
        return nodeIds && nodeIds.length === 1 ? this.nodeForId(nodeIds[0]) : null;
    }
    _cancelSearch() {
        if (!this._searchId) {
            return;
        }
        this._agent.invoke_discardSearchResults({ searchId: this._searchId });
        delete this._searchId;
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @return {!Promise<!Array<string>>}
     */
    classNamesPromise(nodeId) {
        return this._agent.invoke_collectClassNamesFromSubtree({ nodeId }).then(({ classNames }) => classNames || []);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {string} selector
     * @return {!Promise<?Protocol.DOM.NodeId>}
     */
    querySelector(nodeId, selector) {
        return this._agent.invoke_querySelector({ nodeId, selector }).then(({ nodeId }) => nodeId);
    }
    /**
     * @param {!Protocol.DOM.NodeId} nodeId
     * @param {string} selector
     * @return {!Promise<?Array<!Protocol.DOM.NodeId>>}
     */
    querySelectorAll(nodeId, selector) {
        return this._agent.invoke_querySelectorAll({ nodeId, selector }).then(({ nodeIds }) => nodeIds);
    }
    /**
     * @param {boolean=} minorChange
     */
    markUndoableState(minorChange) {
        DOMModelUndoStack.instance()._markUndoableState(this, minorChange || false);
    }
    /**
     * @param {number} x
     * @param {number} y
     * @param {boolean} includeUserAgentShadowDOM
     * @return {!Promise<?DOMNode>}
     */
    async nodeForLocation(x, y, includeUserAgentShadowDOM) {
        const response = await this._agent.invoke_getNodeForLocation({ x, y, includeUserAgentShadowDOM });
        if (response.getError() || !response.nodeId) {
            return null;
        }
        return this.nodeForId(response.nodeId);
    }
    /**
     * @param {!RemoteObject} object
     * @return {!Promise<?DOMNode>}
     */
    pushObjectAsNodeToFrontend(object) {
        return object.isNode() ? this.pushNodeToFrontend(/** @type {string} */ (object.objectId)) : Promise.resolve(null);
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    suspendModel() {
        return this._agent.invoke_disable().then(() => this._setDocument(null));
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    async resumeModel() {
        await this._agent.invoke_enable();
    }
    /**
     * @override
     */
    dispose() {
        DOMModelUndoStack.instance()._dispose(this);
    }
    /**
     * @return {?DOMModel}
     */
    parentModel() {
        const parentTarget = this.target().parentTarget();
        return parentTarget ? parentTarget.model(DOMModel) : null;
    }
}
/** @enum {symbol} */
const Events$8 = {
    AttrModified: Symbol('AttrModified'),
    AttrRemoved: Symbol('AttrRemoved'),
    CharacterDataModified: Symbol('CharacterDataModified'),
    DOMMutated: Symbol('DOMMutated'),
    NodeInserted: Symbol('NodeInserted'),
    NodeRemoved: Symbol('NodeRemoved'),
    DocumentUpdated: Symbol('DocumentUpdated'),
    ChildNodeCountUpdated: Symbol('ChildNodeCountUpdated'),
    DistributedNodesChanged: Symbol('DistributedNodesChanged'),
    MarkersChanged: Symbol('MarkersChanged')
};
/**
 * @implements {ProtocolProxyApi.DOMDispatcher}
 */
class DOMDispatcher {
    /**
     * @param {!DOMModel} domModel
     */
    constructor(domModel) {
        this._domModel = domModel;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @override
     */
    documentUpdated() {
        this._domModel._documentUpdated();
    }
    /**
     * @override
     * @param {!Protocol.DOM.AttributeModifiedEvent} event
     */
    attributeModified({ nodeId, name, value }) {
        this._domModel._attributeModified(nodeId, name, value);
    }
    /**
     * @override
     * @param {!Protocol.DOM.AttributeRemovedEvent} event
     */
    attributeRemoved({ nodeId, name }) {
        this._domModel._attributeRemoved(nodeId, name);
    }
    /**
     * @override
     * @param {!Protocol.DOM.InlineStyleInvalidatedEvent} event
     */
    inlineStyleInvalidated({ nodeIds }) {
        this._domModel._inlineStyleInvalidated(nodeIds);
    }
    /**
     * @override
     * @param {!Protocol.DOM.CharacterDataModifiedEvent} event
     */
    characterDataModified({ nodeId, characterData }) {
        this._domModel._characterDataModified(nodeId, characterData);
    }
    /**
     * @override
     * @param {!Protocol.DOM.SetChildNodesEvent} event
     */
    setChildNodes({ parentId, nodes }) {
        this._domModel._setChildNodes(parentId, nodes);
    }
    /**
     * @override
     * @param {!Protocol.DOM.ChildNodeCountUpdatedEvent} event
     */
    childNodeCountUpdated({ nodeId, childNodeCount }) {
        this._domModel._childNodeCountUpdated(nodeId, childNodeCount);
    }
    /**
     * @override
     * @param {!Protocol.DOM.ChildNodeInsertedEvent} event
     */
    childNodeInserted({ parentNodeId, previousNodeId, node }) {
        this._domModel._childNodeInserted(parentNodeId, previousNodeId, node);
    }
    /**
     * @override
     * @param {!Protocol.DOM.ChildNodeRemovedEvent} event
     */
    childNodeRemoved({ parentNodeId, nodeId }) {
        this._domModel._childNodeRemoved(parentNodeId, nodeId);
    }
    /**
     * @override
     * @param {!Protocol.DOM.ShadowRootPushedEvent} event
     */
    shadowRootPushed({ hostId, root }) {
        this._domModel._shadowRootPushed(hostId, root);
    }
    /**
     * @override
     * @param {!Protocol.DOM.ShadowRootPoppedEvent} event
     */
    shadowRootPopped({ hostId, rootId }) {
        this._domModel._shadowRootPopped(hostId, rootId);
    }
    /**
     * @override
     * @param {!Protocol.DOM.PseudoElementAddedEvent} event
     */
    pseudoElementAdded({ parentId, pseudoElement }) {
        this._domModel._pseudoElementAdded(parentId, pseudoElement);
    }
    /**
     * @override
     * @param {!Protocol.DOM.PseudoElementRemovedEvent} event
     */
    pseudoElementRemoved({ parentId, pseudoElementId }) {
        this._domModel._pseudoElementRemoved(parentId, pseudoElementId);
    }
    /**
     * @override
     * @param {!Protocol.DOM.DistributedNodesUpdatedEvent} event
     */
    distributedNodesUpdated({ insertionPointId, distributedNodes }) {
        this._domModel._distributedNodesUpdated(insertionPointId, distributedNodes);
    }
}
/**
 * @type {?DOMModelUndoStack}
 */
let DOMModelUndoStackInstance;
class DOMModelUndoStack {
    constructor() {
        /** @type {!Array<!DOMModel>} */
        this._stack = [];
        this._index = 0;
        /** @type {?DOMModel} */
        this._lastModelWithMinorChange = null;
    }
    /**
     * @param {{forceNew: ?boolean}} opts
     * @return {!DOMModelUndoStack}
     */
    static instance(opts = { forceNew: null }) {
        const { forceNew } = opts;
        if (!DOMModelUndoStackInstance || forceNew) {
            DOMModelUndoStackInstance = new DOMModelUndoStack();
        }
        return DOMModelUndoStackInstance;
    }
    /**
     * @param {!DOMModel} model
     * @param {boolean} minorChange
     */
    _markUndoableState(model, minorChange) {
        // Both minor and major changes get into the stack, but minor updates are coalesced.
        // Commit major undoable state in the old model upon model switch.
        if (this._lastModelWithMinorChange && model !== this._lastModelWithMinorChange) {
            this._lastModelWithMinorChange.markUndoableState();
            this._lastModelWithMinorChange = null;
        }
        // Previous minor change is already in the stack.
        if (minorChange && this._lastModelWithMinorChange === model) {
            return;
        }
        this._stack = this._stack.slice(0, this._index);
        this._stack.push(model);
        this._index = this._stack.length;
        // Delay marking as major undoable states in case of minor operations until the
        // major or model switch.
        if (minorChange) {
            this._lastModelWithMinorChange = model;
        }
        else {
            model._agent.markUndoableState();
            this._lastModelWithMinorChange = null;
        }
    }
    /**
     * @return {!Promise<void>}
     */
    undo() {
        if (this._index === 0) {
            return Promise.resolve();
        }
        --this._index;
        this._lastModelWithMinorChange = null;
        return this._stack[this._index]._agent.undo();
    }
    /**
     * @return {!Promise<void>}
     */
    redo() {
        if (this._index >= this._stack.length) {
            return Promise.resolve();
        }
        ++this._index;
        this._lastModelWithMinorChange = null;
        return this._stack[this._index - 1]._agent.redo();
    }
    /**
     * @param {!DOMModel} model
     */
    _dispose(model) {
        let shift = 0;
        for (let i = 0; i < this._index; ++i) {
            if (this._stack[i] === model) {
                ++shift;
            }
        }
        ArrayUtilities.removeElement(this._stack, model);
        this._index -= shift;
        if (this._lastModelWithMinorChange === model) {
            this._lastModelWithMinorChange = null;
        }
    }
}
SDKModel.register(DOMModel, Capability.DOM, true);
/** @typedef {{name: string, value: string, _node: DOMNode}} */
let Attribute;

var DOMModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    DOMNode: DOMNode,
    DeferredDOMNode: DeferredDOMNode,
    DOMNodeShortcut: DOMNodeShortcut,
    DOMDocument: DOMDocument,
    DOMModel: DOMModel,
    Events: Events$8,
    DOMModelUndoStack: DOMModelUndoStack,
    Attribute: Attribute
});

/*
 * Copyright (C) 2007, 2008 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @implements {TextUtils.ContentProvider.ContentProvider}
 * @unrestricted
 */
class Resource {
    /**
     * @param {!ResourceTreeModel} resourceTreeModel
     * @param {?NetworkRequest} request
     * @param {string} url
     * @param {string} documentURL
     * @param {!Protocol.Page.FrameId} frameId
     * @param {!Protocol.Network.LoaderId} loaderId
     * @param {!Common.ResourceType.ResourceType} type
     * @param {string} mimeType
     * @param {?Date} lastModified
     * @param {?number} contentSize
     */
    constructor(resourceTreeModel, request, url, documentURL, frameId, loaderId, type, mimeType, lastModified, contentSize) {
        this._resourceTreeModel = resourceTreeModel;
        this._request = request;
        this.url = url;
        /** @type {string} */
        this._url;
        this._documentURL = documentURL;
        this._frameId = frameId;
        this._loaderId = loaderId;
        this._type = type || ResourceType.resourceTypes.Other;
        this._mimeType = mimeType;
        this._lastModified = lastModified && DateUtilities.isValid(lastModified) ? lastModified : null;
        this._contentSize = contentSize;
        /** @type {?string} */ this._content;
        /** @type {?string} */ this._contentLoadError;
        /** @type {boolean} */ this._contentEncoded;
        /** @type {!Array<function(?Object):void>} */
        this._pendingContentCallbacks = [];
        if (this._request && !this._request.finished) {
            this._request.addEventListener(Events$2.FinishedLoading, this._requestFinished, this);
        }
    }
    /**
     * @return {?Date}
     */
    lastModified() {
        if (this._lastModified || !this._request) {
            return this._lastModified;
        }
        const lastModifiedHeader = this._request.responseLastModified();
        const date = lastModifiedHeader ? new Date(lastModifiedHeader) : null;
        this._lastModified = date && DateUtilities.isValid(date) ? date : null;
        return this._lastModified;
    }
    /**
     * @return {?number}
     */
    contentSize() {
        if (typeof this._contentSize === 'number' || !this._request) {
            return this._contentSize;
        }
        return this._request.resourceSize;
    }
    /**
     * @return {?NetworkRequest}
     */
    get request() {
        return this._request;
    }
    /**
     * @return {string}
     */
    get url() {
        return this._url;
    }
    /**
     * @param {string} x
     */
    set url(x) {
        this._url = x;
        this._parsedURL = new ParsedURL.ParsedURL(x);
    }
    get parsedURL() {
        return this._parsedURL;
    }
    /**
     * @return {string}
     */
    get documentURL() {
        return this._documentURL;
    }
    /**
     * @return {!Protocol.Page.FrameId}
     */
    get frameId() {
        return this._frameId;
    }
    /**
     * @return {!Protocol.Network.LoaderId}
     */
    get loaderId() {
        return this._loaderId;
    }
    /**
     * @return {string}
     */
    get displayName() {
        return this._parsedURL ? this._parsedURL.displayName : '';
    }
    /**
     * @return {!Common.ResourceType.ResourceType}
     */
    resourceType() {
        return this._request ? this._request.resourceType() : this._type;
    }
    /**
     * @return {string}
     */
    get mimeType() {
        return this._request ? this._request.mimeType : this._mimeType;
    }
    /**
     * @return {?string}
     */
    get content() {
        return this._content;
    }
    /**
     * @override
     * @return {string}
     */
    contentURL() {
        return this._url;
    }
    /**
     * @override
     * @return {!Common.ResourceType.ResourceType}
     */
    contentType() {
        if (this.resourceType() === ResourceType.resourceTypes.Document &&
            this.mimeType.indexOf('javascript') !== -1) {
            return ResourceType.resourceTypes.Script;
        }
        return this.resourceType();
    }
    /**
     * @override
     * @return {!Promise<boolean>}
     */
    async contentEncoded() {
        await this.requestContent();
        return this._contentEncoded;
    }
    /**
     * @override
     * @return {!Promise<!TextUtils.ContentProvider.DeferredContent>}
     */
    requestContent() {
        if (typeof this._content !== 'undefined') {
            return Promise.resolve({ content: /** @type {string} */ (this._content), isEncoded: this._contentEncoded });
        }
        return new Promise(resolve => {
            this._pendingContentCallbacks.push(/** @type {function(?Object):void} */ (resolve));
            if (!this._request || this._request.finished) {
                this._innerRequestContent();
            }
        });
    }
    /**
     * @return {string}
     */
    canonicalMimeType() {
        return this.contentType().canonicalMimeType() || this.mimeType;
    }
    /**
     * @override
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    async searchInContent(query, caseSensitive, isRegex) {
        if (!this.frameId) {
            return [];
        }
        if (this.request) {
            return this.request.searchInContent(query, caseSensitive, isRegex);
        }
        const result = await this._resourceTreeModel.target().pageAgent().invoke_searchInResource({ frameId: this.frameId, url: this.url, query, caseSensitive, isRegex });
        return result.result || [];
    }
    /**
     * @param {!HTMLImageElement} image
     */
    async populateImageSource(image) {
        const { content } = await this.requestContent();
        const encoded = this._contentEncoded;
        image.src = ContentProvider.contentAsDataURL(content, this._mimeType, encoded) || this._url;
    }
    _requestFinished() {
        if (this._request) {
            this._request.removeEventListener(Events$2.FinishedLoading, this._requestFinished, this);
        }
        if (this._pendingContentCallbacks.length) {
            this._innerRequestContent();
        }
    }
    async _innerRequestContent() {
        if (this._contentRequested) {
            return;
        }
        this._contentRequested = true;
        /** @type {!TextUtils.ContentProvider.DeferredContent} */
        let loadResult;
        if (this.request) {
            const contentData = await this.request.contentData();
            this._content = contentData.content;
            this._contentEncoded = contentData.encoded;
            loadResult = { content: /** @type{string} */ (contentData.content), isEncoded: contentData.encoded };
        }
        else {
            const response = await this._resourceTreeModel.target().pageAgent().invoke_getResourceContent({ frameId: this.frameId, url: this.url });
            const protocolError = response.getError();
            if (protocolError) {
                this._contentLoadError = protocolError;
                this._content = null;
                loadResult = { content: null, error: protocolError, isEncoded: false };
            }
            else {
                this._content = response.content;
                this._contentLoadError = null;
                loadResult = { content: response.content, isEncoded: response.base64Encoded };
            }
            this._contentEncoded = response.base64Encoded;
        }
        if (this._content === null) {
            this._contentEncoded = false;
        }
        for (const callback of this._pendingContentCallbacks.splice(0)) {
            callback(loadResult);
        }
        delete this._contentRequested;
    }
    /**
     * @return {boolean}
     */
    hasTextContent() {
        if (this._type.isTextType()) {
            return true;
        }
        if (this._type === ResourceType.resourceTypes.Other) {
            return !!this._content && !this._contentEncoded;
        }
        return false;
    }
    /**
     * @return {?ResourceTreeFrame}
     */
    frame() {
        return this._resourceTreeModel.frameForId(this._frameId);
    }
}

var Resource$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Resource: Resource
});

// Copyright 2016 The Chromium Authors. All rights reserved.
class SecurityOriginManager extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        // if a URL is unreachable, the browser will jump to an error page at
        // 'chrome-error://chromewebdata/', and |this._mainSecurityOrigin| stores
        // its origin. In this situation, the original unreachable URL's security
        // origin will be stored in |this._unreachableMainSecurityOrigin|.
        this._mainSecurityOrigin = '';
        /** @type {?string} */
        this._unreachableMainSecurityOrigin = '';
        /** @type {!Set<string>} */
        this._securityOrigins = new Set();
    }
    /**
     * @param {!Set<string>} securityOrigins
     */
    updateSecurityOrigins(securityOrigins) {
        const oldOrigins = this._securityOrigins;
        this._securityOrigins = securityOrigins;
        for (const origin of oldOrigins) {
            if (!this._securityOrigins.has(origin)) {
                this.dispatchEventToListeners(Events$9.SecurityOriginRemoved, origin);
            }
        }
        for (const origin of this._securityOrigins) {
            if (!oldOrigins.has(origin)) {
                this.dispatchEventToListeners(Events$9.SecurityOriginAdded, origin);
            }
        }
    }
    /**
     * @return {!Array<string>}
     */
    securityOrigins() {
        return [...this._securityOrigins];
    }
    /**
     * @return {string}
     */
    mainSecurityOrigin() {
        return this._mainSecurityOrigin;
    }
    /**
     * @return {?string}
     */
    unreachableMainSecurityOrigin() {
        return this._unreachableMainSecurityOrigin;
    }
    /**
     * @param {string} securityOrigin
     * @param {string} unreachableSecurityOrigin
     */
    setMainSecurityOrigin(securityOrigin, unreachableSecurityOrigin) {
        this._mainSecurityOrigin = securityOrigin;
        this._unreachableMainSecurityOrigin = unreachableSecurityOrigin || null;
        this.dispatchEventToListeners(Events$9.MainSecurityOriginChanged, {
            mainSecurityOrigin: this._mainSecurityOrigin,
            unreachableMainSecurityOrigin: this._unreachableMainSecurityOrigin
        });
    }
}
/** @enum {symbol} */
const Events$9 = {
    SecurityOriginAdded: Symbol('SecurityOriginAdded'),
    SecurityOriginRemoved: Symbol('SecurityOriginRemoved'),
    MainSecurityOriginChanged: Symbol('MainSecurityOriginChanged')
};
// TODO(jarhar): this is the only usage of Capability.None. Do something about it!
SDKModel.register(SecurityOriginManager, Capability.None, false);

var SecurityOriginManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    SecurityOriginManager: SecurityOriginManager,
    Events: Events$9
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class ResourceTreeModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        const networkManager = target.model(NetworkManager);
        if (networkManager) {
            networkManager.addEventListener(Events$1.RequestFinished, this._onRequestFinished, this);
            networkManager.addEventListener(Events$1.RequestUpdateDropped, this._onRequestUpdateDropped, this);
        }
        this._agent = target.pageAgent();
        this._agent.invoke_enable();
        this._securityOriginManager = target.model(SecurityOriginManager);
        target.registerPageDispatcher(new PageDispatcher(this));
        /** @type {!Map<string, !ResourceTreeFrame>} */
        this._frames = new Map();
        this._cachedResourcesProcessed = false;
        /** @type {?{ignoreCache:(boolean|undefined), scriptToEvaluateOnLoad: (string|undefined)}}} */
        this._pendingReloadOptions = null;
        this._reloadSuspensionCount = 0;
        this._isInterstitialShowing = false;
        /** @type {?ResourceTreeFrame} */
        this.mainFrame = null;
        this._agent.invoke_getResourceTree().then(event => {
            this._processCachedResources(event.getError() ? null : event.frameTree);
        });
    }
    /**
     * @param {!NetworkRequest} request
     * @return {?ResourceTreeFrame}
     */
    static frameForRequest(request) {
        const networkManager = NetworkManager.forRequest(request);
        const resourceTreeModel = networkManager ? networkManager.target().model(ResourceTreeModel) : null;
        if (!resourceTreeModel) {
            return null;
        }
        return resourceTreeModel.frameForId(request.frameId);
    }
    /**
     * @return {!Array.<!ResourceTreeFrame>}
     */
    static frames() {
        const result = [];
        for (const resourceTreeModel of TargetManager.instance().models(ResourceTreeModel)) {
            result.push(...resourceTreeModel._frames.values());
        }
        return result;
    }
    /**
     * @param {string} url
     * @return {?Resource}
     */
    static resourceForURL(url) {
        for (const resourceTreeModel of TargetManager.instance().models(ResourceTreeModel)) {
            const mainFrame = resourceTreeModel.mainFrame;
            const result = mainFrame ? mainFrame.resourceForURL(url) : null;
            if (result) {
                return result;
            }
        }
        return null;
    }
    /**
     * @param {boolean=} bypassCache
     * @param {string=} scriptToEvaluateOnLoad
     */
    static reloadAllPages(bypassCache, scriptToEvaluateOnLoad) {
        for (const resourceTreeModel of TargetManager.instance().models(ResourceTreeModel)) {
            if (!resourceTreeModel.target().parentTarget()) {
                resourceTreeModel.reloadPage(bypassCache, scriptToEvaluateOnLoad);
            }
        }
    }
    /**
     * @return {!DOMModel}
     */
    domModel() {
        return /** @type {!DOMModel} */ (this.target().model(DOMModel));
    }
    /**
     * @param {?Protocol.Page.FrameResourceTree} mainFramePayload
     */
    _processCachedResources(mainFramePayload) {
        // TODO(caseq): the url check below is a mergeable, conservative
        // workaround for a problem caused by us requesting resources from a
        // subtarget frame before it has committed. The proper fix is likely
        // to be too complicated to be safely merged.
        // See https://crbug.com/1081270 for details.
        if (mainFramePayload && mainFramePayload.frame.url !== ':') {
            this.dispatchEventToListeners(Events$a.WillLoadCachedResources);
            this._addFramesRecursively(null, mainFramePayload);
            this.target().setInspectedURL(mainFramePayload.frame.url);
        }
        this._cachedResourcesProcessed = true;
        const runtimeModel = this.target().model(RuntimeModel);
        if (runtimeModel) {
            runtimeModel.setExecutionContextComparator(this._executionContextComparator.bind(this));
            runtimeModel.fireExecutionContextOrderChanged();
        }
        this.dispatchEventToListeners(Events$a.CachedResourcesLoaded, this);
    }
    /**
     * @return {boolean}
     */
    cachedResourcesLoaded() {
        return this._cachedResourcesProcessed;
    }
    /**
     * @return {boolean}
     */
    isInterstitialShowing() {
        return this._isInterstitialShowing;
    }
    /**
     * @param {!ResourceTreeFrame} frame
     * @param {boolean=} aboutToNavigate
     */
    _addFrame(frame, aboutToNavigate) {
        this._frames.set(frame.id, frame);
        if (frame.isMainFrame()) {
            this.mainFrame = frame;
        }
        this.dispatchEventToListeners(Events$a.FrameAdded, frame);
        this._updateSecurityOrigins();
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     * @param {?Protocol.Page.FrameId} parentFrameId
     * @param {!Protocol.Runtime.StackTrace=} stackTrace
     * @return {?ResourceTreeFrame}
     */
    _frameAttached(frameId, parentFrameId, stackTrace) {
        const sameTargetParentFrame = parentFrameId ? (this._frames.get(parentFrameId) || null) : null;
        // Do nothing unless cached resource tree is processed - it will overwrite everything.
        if (!this._cachedResourcesProcessed && sameTargetParentFrame) {
            return null;
        }
        if (this._frames.has(frameId)) {
            return null;
        }
        const frame = new ResourceTreeFrame(this, sameTargetParentFrame, frameId, null, stackTrace || null);
        if (parentFrameId && !sameTargetParentFrame) {
            frame._crossTargetParentFrameId = parentFrameId;
        }
        if (frame.isMainFrame() && this.mainFrame) {
            // Navigation to the new backend process.
            this._frameDetached(this.mainFrame.id);
        }
        this._addFrame(frame, true);
        return frame;
    }
    /**
     * @param {!Protocol.Page.Frame} framePayload
     */
    _frameNavigated(framePayload) {
        const sameTargetParentFrame = framePayload.parentId ? (this._frames.get(framePayload.parentId) || null) : null;
        // Do nothing unless cached resource tree is processed - it will overwrite everything.
        if (!this._cachedResourcesProcessed && sameTargetParentFrame) {
            return;
        }
        let frame = this._frames.get(framePayload.id) || null;
        if (!frame) {
            // Simulate missed "frameAttached" for a main frame navigation to the new backend process.
            frame = this._frameAttached(framePayload.id, framePayload.parentId || '');
            console.assert(!!frame);
            if (!frame) {
                return;
            }
        }
        this.dispatchEventToListeners(Events$a.FrameWillNavigate, frame);
        frame._navigate(framePayload);
        this.dispatchEventToListeners(Events$a.FrameNavigated, frame);
        if (frame.isMainFrame()) {
            this.dispatchEventToListeners(Events$a.MainFrameNavigated, frame);
        }
        // Fill frame with retained resources (the ones loaded using new loader).
        const resources = frame.resources();
        for (let i = 0; i < resources.length; ++i) {
            this.dispatchEventToListeners(Events$a.ResourceAdded, resources[i]);
        }
        if (frame.isMainFrame()) {
            this.target().setInspectedURL(frame.url);
        }
        this._updateSecurityOrigins();
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     */
    _frameDetached(frameId) {
        // Do nothing unless cached resource tree is processed - it will overwrite everything.
        if (!this._cachedResourcesProcessed) {
            return;
        }
        const frame = this._frames.get(frameId);
        if (!frame) {
            return;
        }
        const sameTargetParentFrame = frame.sameTargetParentFrame();
        if (sameTargetParentFrame) {
            sameTargetParentFrame._removeChildFrame(frame);
        }
        else {
            frame._remove();
        }
        this._updateSecurityOrigins();
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onRequestFinished(event) {
        if (!this._cachedResourcesProcessed) {
            return;
        }
        const request = /** @type {!NetworkRequest} */ (event.data);
        if (request.failed || request.resourceType() === ResourceType.resourceTypes.XHR) {
            return;
        }
        const frame = this._frames.get(request.frameId);
        if (frame) {
            frame._addRequest(request);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onRequestUpdateDropped(event) {
        if (!this._cachedResourcesProcessed) {
            return;
        }
        const data = /** @type {!RequestUpdateDroppedEventData} */ (event.data);
        const frameId = data.frameId;
        const frame = this._frames.get(frameId);
        if (!frame) {
            return;
        }
        const url = data.url;
        if (frame._resourcesMap.get(url)) {
            return;
        }
        const resource = new Resource(this, null, url, frame.url, frameId, data.loaderId, ResourceType.resourceTypes[data.resourceType], data.mimeType, data.lastModified, null);
        frame.addResource(resource);
    }
    /**
     * @param {!Protocol.Page.FrameId} frameId
     * @return {?ResourceTreeFrame}
     */
    frameForId(frameId) {
        return this._frames.get(frameId) || null;
    }
    /**
     * @param {function(!Resource):boolean} callback
     * @return {boolean}
     */
    forAllResources(callback) {
        if (this.mainFrame) {
            return this.mainFrame._callForFrameResources(callback);
        }
        return false;
    }
    /**
     * @return {!Array<!ResourceTreeFrame>}
     */
    frames() {
        return [...this._frames.values()];
    }
    /**
     * @param {string} url
     * @return {?Resource}
     */
    resourceForURL(url) {
        // Workers call into this with no frames available.
        return this.mainFrame ? this.mainFrame.resourceForURL(url) : null;
    }
    /**
     * @param {?ResourceTreeFrame} sameTargetParentFrame
     * @param {!Protocol.Page.FrameResourceTree} frameTreePayload
     */
    _addFramesRecursively(sameTargetParentFrame, frameTreePayload) {
        const framePayload = frameTreePayload.frame;
        const frame = new ResourceTreeFrame(this, sameTargetParentFrame, framePayload.id, framePayload, null);
        if (!sameTargetParentFrame && framePayload.parentId) {
            frame._crossTargetParentFrameId = framePayload.parentId;
        }
        this._addFrame(frame);
        for (const childFrame of frameTreePayload.childFrames || []) {
            this._addFramesRecursively(frame, childFrame);
        }
        for (let i = 0; i < frameTreePayload.resources.length; ++i) {
            const subresource = frameTreePayload.resources[i];
            const resource = this._createResourceFromFramePayload(framePayload, subresource.url, ResourceType.resourceTypes[subresource.type], subresource.mimeType, subresource.lastModified || null, subresource.contentSize || null);
            frame.addResource(resource);
        }
        if (!frame._resourcesMap.get(framePayload.url)) {
            const frameResource = this._createResourceFromFramePayload(framePayload, framePayload.url, ResourceType.resourceTypes.Document, framePayload.mimeType, null, null);
            frame.addResource(frameResource);
        }
    }
    /**
     * @param {!Protocol.Page.Frame} frame
     * @param {string} url
     * @param {!Common.ResourceType.ResourceType} type
     * @param {string} mimeType
     * @param {?number} lastModifiedTime
     * @param {?number} contentSize
     * @return {!Resource}
     */
    _createResourceFromFramePayload(frame, url, type, mimeType, lastModifiedTime, contentSize) {
        const lastModified = typeof lastModifiedTime === 'number' ? new Date(lastModifiedTime * 1000) : null;
        return new Resource(this, null, url, frame.url, frame.id, frame.loaderId, type, mimeType, lastModified, contentSize);
    }
    suspendReload() {
        this._reloadSuspensionCount++;
    }
    resumeReload() {
        this._reloadSuspensionCount--;
        console.assert(this._reloadSuspensionCount >= 0, 'Unbalanced call to ResourceTreeModel.resumeReload()');
        if (!this._reloadSuspensionCount && this._pendingReloadOptions) {
            const { ignoreCache, scriptToEvaluateOnLoad } = this._pendingReloadOptions;
            this.reloadPage(ignoreCache, scriptToEvaluateOnLoad);
        }
    }
    /**
     * @param {boolean=} ignoreCache
     * @param {string=} scriptToEvaluateOnLoad
     */
    reloadPage(ignoreCache, scriptToEvaluateOnLoad) {
        // Only dispatch PageReloadRequested upon first reload request to simplify client logic.
        if (!this._pendingReloadOptions) {
            this.dispatchEventToListeners(Events$a.PageReloadRequested, this);
        }
        if (this._reloadSuspensionCount) {
            this._pendingReloadOptions = { ignoreCache, scriptToEvaluateOnLoad };
            return;
        }
        this._pendingReloadOptions = null;
        this.dispatchEventToListeners(Events$a.WillReloadPage);
        this._agent.invoke_reload({ ignoreCache, scriptToEvaluateOnLoad });
    }
    /**
     * @param {string} url
     * @return {!Promise<?>}
     */
    navigate(url) {
        return this._agent.invoke_navigate({ url });
    }
    /**
     * @return {!Promise<?{currentIndex: number, entries: !Array<!Protocol.Page.NavigationEntry>}>}
     */
    async navigationHistory() {
        const response = await this._agent.invoke_getNavigationHistory();
        if (response.getError()) {
            return null;
        }
        return { currentIndex: response.currentIndex, entries: response.entries };
    }
    /**
     * @param {!Protocol.Page.NavigationEntry} entry
     */
    navigateToHistoryEntry(entry) {
        this._agent.invoke_navigateToHistoryEntry({ entryId: entry.id });
    }
    /**
     * @return {!Promise<{url: string, data: ?string, errors: !Array<!Protocol.Page.AppManifestError>}>}
     */
    async fetchAppManifest() {
        const response = await this._agent.invoke_getAppManifest();
        if (response.getError()) {
            return { url: response.url, data: null, errors: [] };
        }
        return { url: response.url, data: response.data || null, errors: response.errors };
    }
    /**
     * @return {!Promise<!Array<!Protocol.Page.InstallabilityError>>}
     */
    async getInstallabilityErrors() {
        const response = await this._agent.invoke_getInstallabilityErrors();
        return response.installabilityErrors || [];
    }
    /**
     * @return {!Promise<{primaryIcon: ?string}>}
     */
    async getManifestIcons() {
        const response = await this._agent.invoke_getManifestIcons();
        return { primaryIcon: response.primaryIcon || null };
    }
    /**
     * @param {!ExecutionContext} a
     * @param {!ExecutionContext} b
     * @return {number}
     */
    _executionContextComparator(a, b) {
        /**
         * @param {?ResourceTreeFrame} frame
         * @return {!Array<!ResourceTreeFrame>}
         */
        function framePath(frame) {
            let currentFrame = frame;
            const parents = [];
            while (currentFrame) {
                parents.push(currentFrame);
                currentFrame = currentFrame.sameTargetParentFrame();
            }
            return parents.reverse();
        }
        if (a.target() !== b.target()) {
            return ExecutionContext.comparator(a, b);
        }
        const framesA = a.frameId ? framePath(this.frameForId(a.frameId)) : [];
        const framesB = b.frameId ? framePath(this.frameForId(b.frameId)) : [];
        let frameA;
        let frameB;
        for (let i = 0;; i++) {
            if (!framesA[i] || !framesB[i] || (framesA[i] !== framesB[i])) {
                frameA = framesA[i];
                frameB = framesB[i];
                break;
            }
        }
        if (!frameA && frameB) {
            return -1;
        }
        if (!frameB && frameA) {
            return 1;
        }
        if (frameA && frameB) {
            return frameA.id.localeCompare(frameB.id);
        }
        return ExecutionContext.comparator(a, b);
    }
    /**
     * @return {!SecurityOriginData}
     */
    _getSecurityOriginData() {
        /** @type {!Set<string>} */
        const securityOrigins = new Set();
        let mainSecurityOrigin = null;
        let unreachableMainSecurityOrigin = null;
        for (const frame of this._frames.values()) {
            const origin = frame.securityOrigin;
            if (!origin) {
                continue;
            }
            securityOrigins.add(origin);
            if (frame.isMainFrame()) {
                mainSecurityOrigin = origin;
                if (frame.unreachableUrl()) {
                    const unreachableParsed = new ParsedURL.ParsedURL(frame.unreachableUrl());
                    unreachableMainSecurityOrigin = unreachableParsed.securityOrigin();
                }
            }
        }
        return {
            securityOrigins: securityOrigins,
            mainSecurityOrigin: mainSecurityOrigin,
            unreachableMainSecurityOrigin: unreachableMainSecurityOrigin
        };
    }
    _updateSecurityOrigins() {
        const data = this._getSecurityOriginData();
        this._securityOriginManager.setMainSecurityOrigin(data.mainSecurityOrigin || '', data.unreachableMainSecurityOrigin || '');
        this._securityOriginManager.updateSecurityOrigins(data.securityOrigins);
    }
    /**
     * @return {?string}
     */
    getMainSecurityOrigin() {
        const data = this._getSecurityOriginData();
        return data.mainSecurityOrigin || data.unreachableMainSecurityOrigin;
    }
}
/** @enum {symbol} */
const Events$a = {
    FrameAdded: Symbol('FrameAdded'),
    FrameNavigated: Symbol('FrameNavigated'),
    FrameDetached: Symbol('FrameDetached'),
    FrameResized: Symbol('FrameResized'),
    FrameWillNavigate: Symbol('FrameWillNavigate'),
    MainFrameNavigated: Symbol('MainFrameNavigated'),
    ResourceAdded: Symbol('ResourceAdded'),
    WillLoadCachedResources: Symbol('WillLoadCachedResources'),
    CachedResourcesLoaded: Symbol('CachedResourcesLoaded'),
    DOMContentLoaded: Symbol('DOMContentLoaded'),
    LifecycleEvent: Symbol('LifecycleEvent'),
    Load: Symbol('Load'),
    PageReloadRequested: Symbol('PageReloadRequested'),
    WillReloadPage: Symbol('WillReloadPage'),
    InterstitialShown: Symbol('InterstitialShown'),
    InterstitialHidden: Symbol('InterstitialHidden')
};
class ResourceTreeFrame {
    /**
     * @param {!ResourceTreeModel} model
     * @param {?ResourceTreeFrame} parentFrame
     * @param {!Protocol.Page.FrameId} frameId
     * @param {?Protocol.Page.Frame} payload
     * @param {?Protocol.Runtime.StackTrace} creationStackTrace
     */
    constructor(model, parentFrame, frameId, payload, creationStackTrace) {
        this._model = model;
        this._sameTargetParentFrame = parentFrame;
        this._id = frameId;
        /** @type {?Protocol.Page.FrameId} */
        this._crossTargetParentFrameId = null;
        this._loaderId = (payload && payload.loaderId) || '';
        this._name = payload && payload.name;
        this._url = (payload && payload.url) || '';
        this._domainAndRegistry = (payload && payload.domainAndRegistry) || '';
        this._securityOrigin = payload && payload.securityOrigin;
        this._mimeType = payload && payload.mimeType;
        this._unreachableUrl = (payload && payload.unreachableUrl) || '';
        this._adFrameType = (payload && payload.adFrameType) || Protocol.Page.AdFrameType.None;
        this._secureContextType = payload && payload.secureContextType;
        this._crossOriginIsolatedContextType = payload && payload.crossOriginIsolatedContextType;
        this._creationStackTrace = creationStackTrace;
        /**
         * @type {!Set.<!ResourceTreeFrame>}
         */
        this._childFrames = new Set();
        /**
         * @type {!Map<string, !Resource>}
         */
        this._resourcesMap = new Map();
        if (this._sameTargetParentFrame) {
            this._sameTargetParentFrame._childFrames.add(this);
        }
    }
    /**
     * @returns {boolean}
     */
    isSecureContext() {
        return !!this._secureContextType && this._secureContextType.startsWith('Secure');
    }
    /**
     * @returns {?Protocol.Page.SecureContextType}
     */
    getSecureContextType() {
        return this._secureContextType;
    }
    /**
     * @returns {boolean}
     */
    isCrossOriginIsolated() {
        return !!this._crossOriginIsolatedContextType && this._crossOriginIsolatedContextType.startsWith('Isolated');
    }
    /**
     * @returns {?Protocol.Page.CrossOriginIsolatedContextType}
     */
    getCrossOriginIsolatedContextType() {
        return this._crossOriginIsolatedContextType;
    }
    /**
     * @param {!Protocol.Page.Frame} framePayload
     */
    _navigate(framePayload) {
        this._loaderId = framePayload.loaderId;
        this._name = framePayload.name;
        this._url = framePayload.url;
        this._domainAndRegistry = framePayload.domainAndRegistry;
        this._securityOrigin = framePayload.securityOrigin;
        this._mimeType = framePayload.mimeType;
        this._unreachableUrl = framePayload.unreachableUrl || '';
        this._adFrameType = framePayload.adFrameType || Protocol.Page.AdFrameType.None;
        this._secureContextType = framePayload.secureContextType;
        this._crossOriginIsolatedContextType = framePayload.crossOriginIsolatedContextType;
        const mainResource = this._resourcesMap.get(this._url);
        this._resourcesMap.clear();
        this._removeChildFrames();
        if (mainResource && mainResource.loaderId === this._loaderId) {
            this.addResource(mainResource);
        }
    }
    /**
     * @return {!ResourceTreeModel}
     */
    resourceTreeModel() {
        return this._model;
    }
    /**
     * @return {string}
     */
    get id() {
        return this._id;
    }
    /**
     * @return {string}
     */
    get name() {
        return this._name || '';
    }
    /**
     * @return {string}
     */
    get url() {
        return this._url;
    }
    /**
     * @return {string}
     */
    domainAndRegistry() {
        return this._domainAndRegistry;
    }
    /**
     * @return {?string}
     */
    get securityOrigin() {
        return this._securityOrigin;
    }
    /**
     * @return {string}
     */
    unreachableUrl() {
        return this._unreachableUrl;
    }
    /**
     * @return {string}
     */
    get loaderId() {
        return this._loaderId;
    }
    /**
     * @return {!Protocol.Page.AdFrameType}
     */
    adFrameType() {
        return this._adFrameType;
    }
    /**
     * @return {!Array.<!ResourceTreeFrame>}
     */
    get childFrames() {
        return [...this._childFrames];
    }
    /**
     * Returns the parent frame if both frames are part of the same process/target.
     * @return {?ResourceTreeFrame}
     */
    sameTargetParentFrame() {
        return this._sameTargetParentFrame;
    }
    /**
     * Returns the parent frame if both frames are part of different processes/targets (child is an OOPIF).
     * @return {?ResourceTreeFrame}
     */
    crossTargetParentFrame() {
        if (!this._crossTargetParentFrameId) {
            return null;
        }
        const parentTarget = this._model.target().parentTarget();
        if (!parentTarget) {
            return null;
        }
        const parentModel = parentTarget.model(ResourceTreeModel);
        if (!parentModel) {
            return null;
        }
        // Note that parent model has already processed cached resources:
        // - when parent target was created, we issued getResourceTree call;
        // - strictly after we issued setAutoAttach call;
        // - both of them were handled in renderer in the same order;
        // - cached resource tree got processed on parent model;
        // - child target was created as a result of setAutoAttach call.
        return parentModel._frames.get(this._crossTargetParentFrameId) || null;
    }
    /**
     * Returns the parent frame. There is only 1 parent and it's either in the
     * same target or it's cross-target.
     * @return {?ResourceTreeFrame}
     */
    parentFrame() {
        return this.sameTargetParentFrame() || this.crossTargetParentFrame();
    }
    /**
     * @param {function(!Protocol.Runtime.CallFrame):boolean} searchFn
     * @return {?Protocol.Runtime.CallFrame}
     */
    findCreationCallFrame(searchFn) {
        let stackTrace = this._creationStackTrace;
        while (stackTrace) {
            const foundEntry = stackTrace.callFrames.find(searchFn);
            if (foundEntry) {
                return foundEntry;
            }
            stackTrace = stackTrace.parent || null;
        }
        return null;
    }
    /**
     * Returns true if this is the main frame of its target. For example, this returns true for the main frame
     * of an out-of-process iframe (OOPIF).
     * @return {boolean}
     */
    isMainFrame() {
        return !this._sameTargetParentFrame;
    }
    /**
     * Returns true if this is the top frame of the main target, i.e. if this is the top-most frame in the inspected
     * tab.
     * @return {boolean}
     */
    isTopFrame() {
        return !this._model.target().parentTarget() && !this._sameTargetParentFrame && !this._crossTargetParentFrameId;
    }
    /**
     * @param {!ResourceTreeFrame} frame
     */
    _removeChildFrame(frame) {
        this._childFrames.delete(frame);
        frame._remove();
    }
    _removeChildFrames() {
        const frames = this._childFrames;
        this._childFrames = new Set();
        for (const frame of frames) {
            frame._remove();
        }
    }
    _remove() {
        this._removeChildFrames();
        this._model._frames.delete(this.id);
        this._model.dispatchEventToListeners(Events$a.FrameDetached, this);
    }
    /**
     * @param {!Resource} resource
     */
    addResource(resource) {
        if (this._resourcesMap.get(resource.url) === resource) {
            // Already in the tree, we just got an extra update.
            return;
        }
        this._resourcesMap.set(resource.url, resource);
        this._model.dispatchEventToListeners(Events$a.ResourceAdded, resource);
    }
    /**
     * @param {!NetworkRequest} request
     */
    _addRequest(request) {
        let resource = this._resourcesMap.get(request.url());
        if (resource && resource.request === request) {
            // Already in the tree, we just got an extra update.
            return;
        }
        resource = new Resource(this._model, request, request.url(), request.documentURL, request.frameId, request.loaderId, request.resourceType(), request.mimeType, null, null);
        this._resourcesMap.set(resource.url, resource);
        this._model.dispatchEventToListeners(Events$a.ResourceAdded, resource);
    }
    /**
     * @return {!Array.<!Resource>}
     */
    resources() {
        return Array.from(this._resourcesMap.values());
    }
    /**
     * @param {string} url
     * @return {?Resource}
     */
    resourceForURL(url) {
        const resource = this._resourcesMap.get(url);
        if (resource) {
            return resource;
        }
        for (const frame of this._childFrames) {
            const resource = frame.resourceForURL(url);
            if (resource) {
                return resource;
            }
        }
        return null;
    }
    /**
     * @param {function(!Resource):boolean} callback
     * @return {boolean}
     */
    _callForFrameResources(callback) {
        for (const resource of this._resourcesMap.values()) {
            if (callback(resource)) {
                return true;
            }
        }
        for (const frame of this._childFrames) {
            if (frame._callForFrameResources(callback)) {
                return true;
            }
        }
        return false;
    }
    /**
     * @return {string}
     */
    displayName() {
        if (this.isTopFrame()) {
            return UIString.UIString('top');
        }
        const subtitle = new ParsedURL.ParsedURL(this._url).displayName;
        if (subtitle) {
            if (!this._name) {
                return subtitle;
            }
            return this._name + ' (' + subtitle + ')';
        }
        return UIString.UIString('<iframe>');
    }
    /**
     * @returns {!Promise<?DeferredDOMNode>}
     */
    async getOwnerDeferredDOMNode() {
        const parentFrame = this.parentFrame();
        if (!parentFrame) {
            return null;
        }
        return parentFrame.resourceTreeModel().domModel().getOwnerNodeForFrame(this._id);
    }
    /**
     * @returns {!Promise<?DOMNode>}
     */
    async getOwnerDOMNodeOrDocument() {
        const deferredNode = await this.getOwnerDeferredDOMNode();
        if (deferredNode) {
            return deferredNode.resolvePromise();
        }
        if (this.isTopFrame()) {
            return this.resourceTreeModel().domModel().requestDocument();
        }
        return null;
    }
    /**
     * @returns {!Promise<void>}
     */
    async highlight() {
        const parentFrame = this.parentFrame();
        const parentTarget = this.resourceTreeModel().target().parentTarget();
        /**
         * @param {!DOMModel} domModel
         */
        const highlightFrameOwner = async (domModel) => {
            const deferredNode = await domModel.getOwnerNodeForFrame(this._id);
            if (deferredNode) {
                domModel.overlayModel().highlightInOverlay({ deferredNode, selectorList: '' }, 'all', true);
            }
        };
        if (parentFrame) {
            return highlightFrameOwner(parentFrame.resourceTreeModel().domModel());
        }
        // Portals.
        if (parentTarget) {
            const domModel = parentTarget.model(DOMModel);
            if (domModel) {
                return highlightFrameOwner(domModel);
            }
        }
        // For the top frame there is no owner node. Highlight the whole document instead.
        const document = await this.resourceTreeModel().domModel().requestDocument();
        if (document) {
            this.resourceTreeModel().domModel().overlayModel().highlightInOverlay({ node: document, selectorList: '' }, 'all', true);
        }
    }
}
/**
 * @implements {ProtocolProxyApi.PageDispatcher}
 */
class PageDispatcher {
    /**
     * @param {!ResourceTreeModel} resourceTreeModel
     */
    constructor(resourceTreeModel) {
        this._resourceTreeModel = resourceTreeModel;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @override
     * @param {!Protocol.Page.DomContentEventFiredEvent} event
     */
    domContentEventFired({ timestamp }) {
        this._resourceTreeModel.dispatchEventToListeners(Events$a.DOMContentLoaded, timestamp);
    }
    /**
     * @override
     * @param {!Protocol.Page.LoadEventFiredEvent} event
     */
    loadEventFired({ timestamp }) {
        this._resourceTreeModel.dispatchEventToListeners(Events$a.Load, { resourceTreeModel: this._resourceTreeModel, loadTime: timestamp });
    }
    /**
     * @override
     * @param {!Protocol.Page.LifecycleEventEvent} event
     */
    lifecycleEvent({ frameId, loaderId, name, timestamp }) {
        this._resourceTreeModel.dispatchEventToListeners(Events$a.LifecycleEvent, { frameId, name });
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameAttachedEvent} event
     */
    frameAttached({ frameId, parentFrameId, stack }) {
        this._resourceTreeModel._frameAttached(frameId, parentFrameId, stack);
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameNavigatedEvent} event
     */
    frameNavigated({ frame }) {
        this._resourceTreeModel._frameNavigated(frame);
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameDetachedEvent} event
     */
    frameDetached({ frameId }) {
        this._resourceTreeModel._frameDetached(frameId);
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameStartedLoadingEvent} event
     */
    frameStartedLoading({ frameId }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameStoppedLoadingEvent} event
     */
    frameStoppedLoading({ frameId }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameRequestedNavigationEvent} event
     */
    frameRequestedNavigation({ frameId }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameScheduledNavigationEvent} event
     */
    frameScheduledNavigation({ frameId, delay }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameClearedScheduledNavigationEvent} event
     */
    frameClearedScheduledNavigation({ frameId }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.NavigatedWithinDocumentEvent} event
     */
    navigatedWithinDocument({ frameId, url }) {
    }
    /**
     * @override
     */
    frameResized() {
        this._resourceTreeModel.dispatchEventToListeners(Events$a.FrameResized, null);
    }
    /**
     * @override
     * @param {!Protocol.Page.JavascriptDialogOpeningEvent} event
     */
    javascriptDialogOpening({ url, message, type, hasBrowserHandler, defaultPrompt }) {
        if (!hasBrowserHandler) {
            this._resourceTreeModel._agent.invoke_handleJavaScriptDialog({ accept: false });
        }
    }
    /**
     * @override
     * @param {!Protocol.Page.JavascriptDialogClosedEvent} event
     */
    javascriptDialogClosed({ result, userInput }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.ScreencastFrameEvent} event
     */
    screencastFrame({ data, metadata, sessionId }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.ScreencastVisibilityChangedEvent} event
     */
    screencastVisibilityChanged({ visible }) {
    }
    /**
     * @override
     */
    interstitialShown() {
        this._resourceTreeModel._isInterstitialShowing = true;
        this._resourceTreeModel.dispatchEventToListeners(Events$a.InterstitialShown);
    }
    /**
     * @override
     */
    interstitialHidden() {
        this._resourceTreeModel._isInterstitialShowing = false;
        this._resourceTreeModel.dispatchEventToListeners(Events$a.InterstitialHidden);
    }
    /**
     * @override
     * @param {!Protocol.Page.WindowOpenEvent} event
     */
    windowOpen({ url, windowName, windowFeatures, userGesture }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.CompilationCacheProducedEvent} event
     */
    compilationCacheProduced({ url, data }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FileChooserOpenedEvent} event
     */
    fileChooserOpened({ mode }) {
    }
    /**
     * @override
     * @param {!Protocol.Page.DownloadWillBeginEvent} event
     */
    downloadWillBegin({ frameId, url }) {
    }
    /**
     * @override
     */
    downloadProgress() {
    }
}
SDKModel.register(ResourceTreeModel, Capability.DOM, true);
/**
 * @typedef {{
  *      securityOrigins: !Set<string>,
  *      mainSecurityOrigin: ?string,
  *      unreachableMainSecurityOrigin: ?string
  * }}
  */
// @ts-ignore typedef
let SecurityOriginData;

var ResourceTreeModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ResourceTreeModel: ResourceTreeModel,
    Events: Events$a,
    ResourceTreeFrame: ResourceTreeFrame,
    PageDispatcher: PageDispatcher,
    SecurityOriginData: SecurityOriginData
});

/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @implements {TextUtils.ContentProvider.ContentProvider}
 * TODO(chromium:1011811): make `implements {FrameAssociated}` annotation work here.
 * @unrestricted
 */
class Script {
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {string} scriptId
     * @param {string} sourceURL
     * @param {number} startLine
     * @param {number} startColumn
     * @param {number} endLine
     * @param {number} endColumn
     * @param {!Protocol.Runtime.ExecutionContextId} executionContextId
     * @param {string} hash
     * @param {boolean} isContentScript
     * @param {boolean} isLiveEdit
     * @param {string|undefined} sourceMapURL
     * @param {boolean} hasSourceURL
     * @param {number} length
     * @param {?Protocol.Runtime.StackTrace} originStackTrace
     * @param {?number} codeOffset
     * @param {?string} scriptLanguage
     * @param {?Protocol.Debugger.DebugSymbols} debugSymbols
     * @param {?string} embedderName
     */
    constructor(debuggerModel, scriptId, sourceURL, startLine, startColumn, endLine, endColumn, executionContextId, hash, isContentScript, isLiveEdit, sourceMapURL, hasSourceURL, length, originStackTrace, codeOffset, scriptLanguage, debugSymbols, embedderName) {
        this.debuggerModel = debuggerModel;
        this.scriptId = scriptId;
        this.sourceURL = sourceURL;
        this.lineOffset = startLine;
        this.columnOffset = startColumn;
        this.endLine = endLine;
        this.endColumn = endColumn;
        this.executionContextId = executionContextId;
        this.hash = hash;
        this._isContentScript = isContentScript;
        this._isLiveEdit = isLiveEdit;
        this.sourceMapURL = sourceMapURL;
        if (!sourceMapURL && debugSymbols && debugSymbols.type === 'EmbeddedDWARF') {
            // TODO(chromium:1064248) Remove this once we either drop gimli or support DebugSymbols all the way down.
            this.sourceMapURL = 'wasm://dwarf';
        }
        this.debugSymbols = debugSymbols;
        this.hasSourceURL = hasSourceURL;
        this.contentLength = length;
        /** @type {?TextUtils.ContentProvider.ContentProvider} */
        this._originalContentProvider = null;
        this.originStackTrace = originStackTrace;
        this._codeOffset = codeOffset;
        this._language = scriptLanguage;
        /** @type {?Promise<!TextUtils.ContentProvider.DeferredContent>} */
        this._contentPromise = null;
        this._embedderName = embedderName;
    }
    /**
     * @returns {?string}
     */
    embedderName() {
        return this._embedderName;
    }
    /**
     * @returns {!Target}
     */
    target() {
        return this.debuggerModel.target();
    }
    /**
     * @param {string} source
     * @return {string}
     */
    static _trimSourceURLComment(source) {
        let sourceURLIndex = source.lastIndexOf('//# sourceURL=');
        if (sourceURLIndex === -1) {
            sourceURLIndex = source.lastIndexOf('//@ sourceURL=');
            if (sourceURLIndex === -1) {
                return source;
            }
        }
        const sourceURLLineIndex = source.lastIndexOf('\n', sourceURLIndex);
        if (sourceURLLineIndex === -1) {
            return source;
        }
        const sourceURLLine = source.substr(sourceURLLineIndex + 1);
        if (!sourceURLLine.match(sourceURLRegex)) {
            return source;
        }
        return source.substr(0, sourceURLLineIndex);
    }
    /**
     * @return {boolean}
     */
    isContentScript() {
        return this._isContentScript;
    }
    /**
     * @return {?number}
     */
    codeOffset() {
        return this._codeOffset;
    }
    /**
     * @return {boolean}
     */
    isWasm() {
        return this._language === Protocol.Debugger.ScriptLanguage.WebAssembly;
    }
    /**
     * @return {string|null}
     */
    scriptLanguage() {
        return this._language;
    }
    /**
     * @return {?ExecutionContext}
     */
    executionContext() {
        return this.debuggerModel.runtimeModel().executionContext(this.executionContextId);
    }
    /**
     * @return {boolean}
     */
    isLiveEdit() {
        return this._isLiveEdit;
    }
    /**
     * @override
     * @return {string}
     */
    contentURL() {
        return this.sourceURL;
    }
    /**
     * @override
     * @return {!Common.ResourceType.ResourceType}
     */
    contentType() {
        return ResourceType.resourceTypes.Script;
    }
    /**
     * @override
     * @return {!Promise<boolean>}
     */
    contentEncoded() {
        return Promise.resolve(false);
    }
    /**
     * @override
     * @return {!Promise<!TextUtils.ContentProvider.DeferredContent>}
     */
    requestContent() {
        if (!this._contentPromise) {
            this._contentPromise = this.originalContentProvider().requestContent();
        }
        return this._contentPromise;
    }
    /**
     * @return {!Promise<!ArrayBuffer>}
     */
    async getWasmBytecode() {
        const base64 = await this.debuggerModel.target().debuggerAgent().invoke_getWasmBytecode({ scriptId: this.scriptId });
        const response = await fetch(`data:application/wasm;base64,${base64.bytecode}`);
        return response.arrayBuffer();
    }
    /**
     * @return {!TextUtils.ContentProvider.ContentProvider}
     */
    originalContentProvider() {
        if (!this._originalContentProvider) {
            /** @type {?Promise<!TextUtils.ContentProvider.DeferredContent>} } */
            let lazyContentPromise;
            this._originalContentProvider =
                new StaticContentProvider.StaticContentProvider(this.contentURL(), this.contentType(), () => {
                    if (!lazyContentPromise) {
                        lazyContentPromise = (async () => {
                            if (!this.scriptId) {
                                return { content: null, error: ls `Script removed or deleted.`, isEncoded: false };
                            }
                            try {
                                const result = await this.debuggerModel.target().debuggerAgent().invoke_getScriptSource({ scriptId: this.scriptId });
                                if (result.getError()) {
                                    throw new Error(result.getError());
                                }
                                const { scriptSource, bytecode } = result;
                                if (bytecode) {
                                    return { content: bytecode, isEncoded: true };
                                }
                                let content = scriptSource || '';
                                if (this.hasSourceURL) {
                                    content = Script._trimSourceURLComment(content);
                                }
                                return { content, isEncoded: false };
                            }
                            catch (err) {
                                // TODO(bmeurer): Propagate errors as exceptions / rejections.
                                return { content: null, error: ls `Unable to fetch script source.`, isEncoded: false };
                            }
                        })();
                    }
                    return lazyContentPromise;
                });
        }
        return this._originalContentProvider;
    }
    /**
     * @override
     * @param {string} query
     * @param {boolean} caseSensitive
     * @param {boolean} isRegex
     * @return {!Promise<!Array<!TextUtils.ContentProvider.SearchMatch>>}
     */
    async searchInContent(query, caseSensitive, isRegex) {
        if (!this.scriptId) {
            return [];
        }
        const matches = await this.debuggerModel.target().debuggerAgent().invoke_searchInContent({ scriptId: this.scriptId, query, caseSensitive, isRegex });
        return (matches.result || [])
            .map(match => new ContentProvider.SearchMatch(match.lineNumber, match.lineContent));
    }
    /**
     * @param {string} source
     * @return {string}
     */
    _appendSourceURLCommentIfNeeded(source) {
        if (!this.hasSourceURL) {
            return source;
        }
        return source + '\n //# sourceURL=' + this.sourceURL;
    }
    /**
     * @param {string} newSource
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, !Protocol.Runtime.ExceptionDetails=, !Array.<!Protocol.Debugger.CallFrame>=, !Protocol.Runtime.StackTrace=, !Protocol.Runtime.StackTraceId=, boolean=):void} callback
     */
    async editSource(newSource, callback) {
        newSource = Script._trimSourceURLComment(newSource);
        // We append correct sourceURL to script for consistency only. It's not actually needed for things to work correctly.
        newSource = this._appendSourceURLCommentIfNeeded(newSource);
        if (!this.scriptId) {
            callback('Script failed to parse');
            return;
        }
        const { content: oldSource } = await this.requestContent();
        if (oldSource === newSource) {
            callback(null);
            return;
        }
        const response = await this.debuggerModel.target().debuggerAgent().invoke_setScriptSource({ scriptId: this.scriptId, scriptSource: newSource });
        if (!response.getError() && !response.exceptionDetails) {
            this._contentPromise = Promise.resolve({ content: newSource, isEncoded: false });
        }
        const needsStepIn = !!response.stackChanged;
        callback(response.getError() || null, response.exceptionDetails, response.callFrames, response.asyncStackTrace, response.asyncStackTraceId, needsStepIn);
    }
    /**
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?Location}
     */
    rawLocation(lineNumber, columnNumber) {
        if (this.containsLocation(lineNumber, columnNumber)) {
            return new Location(this.debuggerModel, this.scriptId, lineNumber, columnNumber);
        }
        return null;
    }
    /**
     *
     * @param {!Location} location
     * @return {!Array.<number>}
     */
    toRelativeLocation(location) {
        console.assert(location.scriptId === this.scriptId, '`toRelativeLocation` must be used with location of the same script');
        const relativeLineNumber = location.lineNumber - this.lineOffset;
        const relativeColumnNumber = (location.columnNumber || 0) - (relativeLineNumber === 0 ? this.columnOffset : 0);
        return [relativeLineNumber, relativeColumnNumber];
    }
    /**
     * @return {boolean}
     */
    isInlineScript() {
        const startsAtZero = !this.lineOffset && !this.columnOffset;
        return !this.isWasm() && !!this.sourceURL && !startsAtZero;
    }
    /**
     * @return {boolean}
     */
    isAnonymousScript() {
        return !this.sourceURL;
    }
    /**
     * @return {boolean}
     */
    isInlineScriptWithSourceURL() {
        return !!this.hasSourceURL && this.isInlineScript();
    }
    /**
     * @param {!Array<!Protocol.Debugger.ScriptPosition>} positions
     * @return {!Promise<boolean>}
     */
    async setBlackboxedRanges(positions) {
        const response = await this.debuggerModel.target().debuggerAgent().invoke_setBlackboxedRanges({ scriptId: this.scriptId, positions });
        return !response.getError();
    }
    /**
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {boolean}
     */
    containsLocation(lineNumber, columnNumber) {
        const afterStart = (lineNumber === this.lineOffset && columnNumber >= this.columnOffset) || lineNumber > this.lineOffset;
        const beforeEnd = lineNumber < this.endLine || (lineNumber === this.endLine && columnNumber <= this.endColumn);
        return afterStart && beforeEnd;
    }
    /**
     * @return {string}
     */
    get frameId() {
        if (typeof this[frameIdSymbol] !== 'string') {
            this[frameIdSymbol] = frameIdForScript(this);
        }
        return this[frameIdSymbol] || '';
    }
    /**
     * @returns {!PageResourceLoadInitiator}
     */
    createPageResourceLoadInitiator() {
        return { target: this.target(), frameId: this.frameId, initiatorUrl: this.embedderName() };
    }
}
const frameIdSymbol = Symbol('frameid');
/**
 * @param {!Script} script
 * @return {string}
 */
function frameIdForScript(script) {
    const executionContext = script.executionContext();
    if (executionContext) {
        return executionContext.frameId || '';
    }
    // This is to overcome compilation cache which doesn't get reset.
    const resourceTreeModel = script.debuggerModel.target().model(ResourceTreeModel);
    if (!resourceTreeModel || !resourceTreeModel.mainFrame) {
        return '';
    }
    return resourceTreeModel.mainFrame.id;
}
const sourceURLRegex = /^[\040\t]*\/\/[@#] sourceURL=\s*(\S*?)\s*$/;

var Script$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Script: Script,
    sourceURLRegex: sourceURLRegex
});

/*
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
*  @param {!Array<!LocationRange>} locationRanges
*  @return {!Array<!LocationRange>}
*/
function sortAndMergeRanges(locationRanges) {
    if (locationRanges.length === 0) {
        return [];
    }
    locationRanges.sort(LocationRange.comparator);
    let prev = locationRanges[0];
    const merged = [];
    for (let i = 1; i < locationRanges.length; ++i) {
        const current = locationRanges[i];
        if (prev.overlap(current)) {
            const largerEnd = prev.end.compareTo(current.end) > 0 ? prev.end : current.end;
            prev = new LocationRange(prev.scriptId, prev.start, largerEnd);
        }
        else {
            merged.push(prev);
            prev = current;
        }
    }
    merged.push(prev);
    return merged;
}
/**
 * TODO(bmeurer): Introduce a dedicated {DebuggerLocationRange} class or something!
 *
 * @param {!Location} location
 * @param {!{start:!Location, end:!Location}} range
 * @return {boolean}
 */
function contained(location, range) {
    const { start, end } = range;
    if (start.scriptId !== location.scriptId) {
        return false;
    }
    if (location.lineNumber < start.lineNumber || location.lineNumber > end.lineNumber) {
        return false;
    }
    if (location.lineNumber === start.lineNumber && location.columnNumber < start.columnNumber) {
        return false;
    }
    if (location.lineNumber === end.lineNumber && location.columnNumber >= end.columnNumber) {
        return false;
    }
    return true;
}
class DebuggerModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        target.registerDebuggerDispatcher(new DebuggerDispatcher(this));
        this._agent = target.debuggerAgent();
        this._runtimeModel = /** @type {!RuntimeModel} */ (target.model(RuntimeModel));
        /** @type {!SourceMapManager<!Script>} */
        this._sourceMapManager = new SourceMapManager(target);
        /** @type {!Map<string, !Script>} */
        this._sourceMapIdToScript = new Map();
        /** @type {?DebuggerPausedDetails} */
        this._debuggerPausedDetails = null;
        /** @type {!Map<string, !Script>} */
        this._scripts = new Map();
        /** @type {!Map.<string, !Array.<!Script>>} */
        this._scriptsBySourceURL = new Map();
        /** @type {!Array.<!Script>} */
        this._discardableScripts = [];
        /** @type {(function(!DebuggerPausedDetails):boolean)|null} */
        this._continueToLocationCallback = null;
        /** @type {?CallFrame} */
        this._selectedCallFrame = null;
        /** @type {boolean} */
        this._debuggerEnabled = false;
        /** @type {?string} */
        this._debuggerId = null;
        /** @type {number} */
        this._skipAllPausesTimeout = 0;
        /** @type {(function(!DebuggerPausedDetails):boolean)|null} */
        this._beforePausedCallback = null;
        /** @type {!Common.ObjectWrapper.ObjectWrapper} */
        this._breakpointResolvedEventTarget = new ObjectWrapper.ObjectWrapper();
        /** @type {boolean} */
        this._autoStepOver = false;
        this._isPausing = false;
        Settings.Settings.instance()
            .moduleSetting('pauseOnExceptionEnabled')
            .addChangeListener(this._pauseOnExceptionStateChanged, this);
        Settings.Settings.instance()
            .moduleSetting('pauseOnCaughtException')
            .addChangeListener(this._pauseOnExceptionStateChanged, this);
        Settings.Settings.instance()
            .moduleSetting('disableAsyncStackTraces')
            .addChangeListener(this._asyncStackTracesStateChanged, this);
        Settings.Settings.instance()
            .moduleSetting('breakpointsActive')
            .addChangeListener(this._breakpointsActiveChanged, this);
        if (!target.suspended()) {
            this._enableDebugger();
        }
        /** @type {!Map<string, string>} */
        this._stringMap = new Map();
        this._sourceMapManager.setEnabled(Settings.Settings.instance().moduleSetting('jsSourceMapsEnabled').get());
        Settings.Settings.instance()
            .moduleSetting('jsSourceMapsEnabled')
            .addChangeListener(event => this._sourceMapManager.setEnabled(/** @type {boolean} */ (event.data)));
    }
    /**
     * @param {!Protocol.Runtime.ExecutionContextId} executionContextId
     * @param {string} sourceURL
     * @param {string|undefined} sourceMapURL
     * @return {?string}
     */
    static _sourceMapId(executionContextId, sourceURL, sourceMapURL) {
        if (!sourceMapURL) {
            return null;
        }
        return executionContextId + ':' + sourceURL + ':' + sourceMapURL;
    }
    /**
     * @return {!SourceMapManager<!Script>}
     */
    sourceMapManager() {
        return this._sourceMapManager;
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    /**
     * @return {boolean}
     */
    debuggerEnabled() {
        return !!this._debuggerEnabled;
    }
    /**
     * @return {!Promise<void>}
     */
    async _enableDebugger() {
        if (this._debuggerEnabled) {
            return Promise.resolve();
        }
        this._debuggerEnabled = true;
        // Set a limit for the total size of collected script sources retained by debugger.
        // 10MB for remote frontends, 100MB for others.
        const isRemoteFrontend = Runtime.Runtime.queryParam('remoteFrontend') || Runtime.Runtime.queryParam('ws');
        const maxScriptsCacheSize = isRemoteFrontend ? 10e6 : 100e6;
        const enablePromise = this._agent.invoke_enable({ maxScriptsCacheSize });
        enablePromise.then(this._registerDebugger.bind(this));
        this._pauseOnExceptionStateChanged();
        this._asyncStackTracesStateChanged();
        if (!Settings.Settings.instance().moduleSetting('breakpointsActive').get()) {
            this._breakpointsActiveChanged();
        }
        if (_scheduledPauseOnAsyncCall) {
            this._pauseOnAsyncCall(_scheduledPauseOnAsyncCall);
        }
        this.dispatchEventToListeners(Events$b.DebuggerWasEnabled, this);
        await enablePromise;
    }
    /**
     * @param {!Protocol.Debugger.EnableResponse} response
     */
    _registerDebugger(response) {
        if (response.getError()) {
            return;
        }
        const { debuggerId } = response;
        _debuggerIdToModel.set(debuggerId, this);
        this._debuggerId = debuggerId;
        this.dispatchEventToListeners(Events$b.DebuggerIsReadyToPause, this);
    }
    /**
     * @return {boolean}
     */
    isReadyToPause() {
        return !!this._debuggerId;
    }
    /**
     * @param {string} debuggerId
     * @return {?DebuggerModel}
     */
    static modelForDebuggerId(debuggerId) {
        return _debuggerIdToModel.get(debuggerId) || null;
    }
    /**
     * @return {!Promise<void>}
     */
    async _disableDebugger() {
        if (!this._debuggerEnabled) {
            return Promise.resolve();
        }
        this._debuggerEnabled = false;
        await this._asyncStackTracesStateChanged();
        await this._agent.invoke_disable();
        this._isPausing = false;
        this.globalObjectCleared();
        this.dispatchEventToListeners(Events$b.DebuggerWasDisabled);
        if (typeof this._debuggerId === 'string') {
            _debuggerIdToModel.delete(this._debuggerId);
        }
    }
    /**
     * @param {boolean} skip
     */
    _skipAllPauses(skip) {
        if (this._skipAllPausesTimeout) {
            clearTimeout(this._skipAllPausesTimeout);
            this._skipAllPausesTimeout = 0;
        }
        this._agent.invoke_setSkipAllPauses({ skip });
    }
    /**
     * @param {number} timeout
     */
    skipAllPausesUntilReloadOrTimeout(timeout) {
        if (this._skipAllPausesTimeout) {
            clearTimeout(this._skipAllPausesTimeout);
        }
        this._agent.invoke_setSkipAllPauses({ skip: true });
        // If reload happens before the timeout, the flag will be already unset and the timeout callback won't change anything.
        this._skipAllPausesTimeout = setTimeout(this._skipAllPauses.bind(this, false), timeout);
    }
    _pauseOnExceptionStateChanged() {
        /** @type {!Protocol.Debugger.SetPauseOnExceptionsRequestState} */
        let state;
        if (!Settings.Settings.instance().moduleSetting('pauseOnExceptionEnabled').get()) {
            state = Protocol.Debugger.SetPauseOnExceptionsRequestState.None;
        }
        else if (Settings.Settings.instance().moduleSetting('pauseOnCaughtException').get()) {
            state = Protocol.Debugger.SetPauseOnExceptionsRequestState.All;
        }
        else {
            state = Protocol.Debugger.SetPauseOnExceptionsRequestState.Uncaught;
        }
        this._agent.invoke_setPauseOnExceptions({ state });
    }
    _asyncStackTracesStateChanged() {
        const maxAsyncStackChainDepth = 32;
        const enabled = !Settings.Settings.instance().moduleSetting('disableAsyncStackTraces').get() && this._debuggerEnabled;
        const maxDepth = enabled ? maxAsyncStackChainDepth : 0;
        return this._agent.invoke_setAsyncCallStackDepth({ maxDepth });
    }
    _breakpointsActiveChanged() {
        this._agent.invoke_setBreakpointsActive({ active: Settings.Settings.instance().moduleSetting('breakpointsActive').get() });
    }
    /**
     *  @return {!Promise<!Array<!LocationRange>>}
     */
    async _computeAutoStepSkipList() {
        // @ts-ignore
        const pluginManager = Bindings.DebuggerWorkspaceBinding.instance().getLanguagePluginManager(this);
        if (pluginManager) {
            // @ts-ignore
            const rawLocation = this._debuggerPausedDetails.callFrames[0].location();
            const uiLocation = await pluginManager.rawLocationToUILocation(rawLocation);
            if (uiLocation) {
                const ranges = await pluginManager.uiLocationToRawLocationRanges(uiLocation.uiSourceCode, uiLocation.lineNumber, uiLocation.columnNumber);
                if (ranges) {
                    // TODO(bmeurer): Remove the {rawLocation} from the {ranges}?
                    // @ts-ignore
                    const filtered = ranges.filter(range => contained(rawLocation, range));
                    const skipList = filtered.map(
                    // @ts-ignore
                    location => new LocationRange(location.start.scriptId, new ScriptPosition(location.start.lineNumber, location.start.columnNumber), new ScriptPosition(location.end.lineNumber, location.end.columnNumber)));
                    return sortAndMergeRanges(skipList);
                }
            }
        }
        return [];
    }
    async stepInto() {
        const skipList = await this._computeAutoStepSkipList();
        this._agent.invoke_stepInto({ breakOnAsyncCall: false, skipList: skipList.map(x => x.payload()) });
    }
    async stepOver() {
        // Mark that in case of auto-stepping, we should be doing
        // step-over instead of step-in.
        this._autoStepOver = true;
        const skipList = await this._computeAutoStepSkipList();
        this._agent.invoke_stepOver({ skipList: skipList.map(x => x.payload()) });
    }
    stepOut() {
        this._agent.invoke_stepOut();
    }
    scheduleStepIntoAsync() {
        this._agent.invoke_stepInto({ breakOnAsyncCall: true });
    }
    resume() {
        this._agent.invoke_resume({ terminateOnResume: false });
        this._isPausing = false;
    }
    pause() {
        this._isPausing = true;
        this._skipAllPauses(false);
        this._agent.invoke_pause();
    }
    /**
     * @param {!Protocol.Runtime.StackTraceId} parentStackTraceId
     * @return {!Promise<!Object>}
     */
    _pauseOnAsyncCall(parentStackTraceId) {
        return this._agent.invoke_pauseOnAsyncCall({ parentStackTraceId: parentStackTraceId });
    }
    /**
     * @param {string} url
     * @param {number} lineNumber
     * @param {number=} columnNumber
     * @param {string=} condition
     * @return {!Promise<!SetBreakpointResult>}
     */
    async setBreakpointByURL(url, lineNumber, columnNumber, condition) {
        // Convert file url to node-js path.
        let urlRegex;
        if (this.target().type() === Type.Node) {
            const platformPath = ParsedURL.ParsedURL.urlToPlatformPath(url, Platform.isWin());
            urlRegex = `${platformPath.escapeForRegExp()}|${url.escapeForRegExp()}`;
        }
        // Adjust column if needed.
        let minColumnNumber = 0;
        const scripts = this._scriptsBySourceURL.get(url) || [];
        for (let i = 0, l = scripts.length; i < l; ++i) {
            const script = scripts[i];
            if (lineNumber === script.lineOffset) {
                minColumnNumber = minColumnNumber ? Math.min(minColumnNumber, script.columnOffset) : script.columnOffset;
            }
        }
        columnNumber = Math.max(columnNumber || 0, minColumnNumber);
        const response = await this._agent.invoke_setBreakpointByUrl({
            lineNumber: lineNumber,
            url: urlRegex ? undefined : url,
            urlRegex: urlRegex,
            columnNumber: columnNumber,
            condition: condition
        });
        if (response.getError()) {
            return { locations: [], breakpointId: null };
        }
        /** @type {!Array<!Location>} */
        let locations = [];
        if (response.locations) {
            locations = response.locations.map(payload => Location.fromPayload(this, payload));
        }
        return { locations, breakpointId: response.breakpointId };
    }
    /**
     * @param {string} scriptId
     * @param {string} scriptHash
     * @param {number} lineNumber
     * @param {number=} columnNumber
     * @param {string=} condition
     * @return {!Promise<!SetBreakpointResult>}
     */
    async setBreakpointInAnonymousScript(scriptId, scriptHash, lineNumber, columnNumber, condition) {
        const response = await this._agent.invoke_setBreakpointByUrl({ lineNumber: lineNumber, scriptHash: scriptHash, columnNumber: columnNumber, condition: condition });
        const error = response.getError();
        if (error) {
            // Old V8 backend doesn't support scriptHash argument.
            if (error !== 'Either url or urlRegex must be specified.') {
                return { locations: [], breakpointId: null };
            }
            return this._setBreakpointBySourceId(scriptId, lineNumber, columnNumber, condition);
        }
        /** @type {!Array<!Location>} */
        let locations = [];
        if (response.locations) {
            locations = response.locations.map(payload => Location.fromPayload(this, payload));
        }
        return { locations, breakpointId: response.breakpointId };
    }
    /**
     * @param {string} scriptId
     * @param {number} lineNumber
     * @param {number=} columnNumber
     * @param {string=} condition
     * @return {!Promise<!SetBreakpointResult>}
     */
    async _setBreakpointBySourceId(scriptId, lineNumber, columnNumber, condition) {
        // This method is required for backward compatibility with V8 before 6.3.275.
        const response = await this._agent.invoke_setBreakpoint({ location: { scriptId: scriptId, lineNumber: lineNumber, columnNumber: columnNumber }, condition: condition });
        if (response.getError()) {
            return { breakpointId: null, locations: [] };
        }
        /** @type {!Array<!Location>} */
        let actualLocation = [];
        if (response.actualLocation) {
            actualLocation = [Location.fromPayload(this, response.actualLocation)];
        }
        return { locations: actualLocation, breakpointId: response.breakpointId };
    }
    /**
     * @param {!Protocol.Debugger.BreakpointId} breakpointId
     * @return {!Promise<void>}
     */
    async removeBreakpoint(breakpointId) {
        const response = await this._agent.invoke_removeBreakpoint({ breakpointId });
        if (response.getError()) {
            console.error('Failed to remove breakpoint: ' + response.getError());
        }
    }
    /**
     * @param {!Location} startLocation
     * @param {?Location} endLocation
     * @param {boolean} restrictToFunction
     * @return {!Promise<!Array<!BreakLocation>>}
     */
    async getPossibleBreakpoints(startLocation, endLocation, restrictToFunction) {
        const response = await this._agent.invoke_getPossibleBreakpoints({
            start: startLocation.payload(),
            end: endLocation ? endLocation.payload() : undefined,
            restrictToFunction: restrictToFunction
        });
        if (response.getError() || !response.locations) {
            return [];
        }
        return response.locations.map(location => BreakLocation.fromPayload(this, location));
    }
    /**
     * @param {!Protocol.Runtime.StackTraceId} stackId
     * @return {!Promise<?Protocol.Runtime.StackTrace>}
     */
    async fetchAsyncStackTrace(stackId) {
        const response = await this._agent.invoke_getStackTrace({ stackTraceId: stackId });
        return response.getError() ? null : response.stackTrace;
    }
    /**
     * @param {!Protocol.Debugger.BreakpointId} breakpointId
     * @param {!Protocol.Debugger.Location} location
     */
    _breakpointResolved(breakpointId, location) {
        this._breakpointResolvedEventTarget.dispatchEventToListeners(breakpointId, Location.fromPayload(this, location));
    }
    globalObjectCleared() {
        this._setDebuggerPausedDetails(null);
        this._reset();
        // TODO(dgozman): move clients to ExecutionContextDestroyed/ScriptCollected events.
        this.dispatchEventToListeners(Events$b.GlobalObjectCleared, this);
    }
    _reset() {
        for (const scriptWithSourceMap of this._sourceMapIdToScript.values()) {
            this._sourceMapManager.detachSourceMap(scriptWithSourceMap);
        }
        // @ts-ignore
        const pluginManager = Bindings.DebuggerWorkspaceBinding.instance().getLanguagePluginManager(this);
        if (pluginManager) {
            for (const script of this._scripts.values()) {
                pluginManager.removeScript(script);
            }
        }
        this._sourceMapIdToScript.clear();
        this._scripts.clear();
        this._scriptsBySourceURL.clear();
        this._stringMap.clear();
        this._discardableScripts = [];
        this._autoStepOver = false;
    }
    /**
     * @return {!Array<!Script>}
     */
    scripts() {
        return Array.from(this._scripts.values());
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @return {?Script}
     */
    scriptForId(scriptId) {
        return this._scripts.get(scriptId) || null;
    }
    /**
     * @param {?string} sourceURL
     * @return {!Array.<!Script>}
     */
    scriptsForSourceURL(sourceURL) {
        if (!sourceURL) {
            return [];
        }
        return this._scriptsBySourceURL.get(sourceURL) || [];
    }
    /**
     * @param {!ExecutionContext} executionContext
     * @return {!Array<!Script>}
     */
    scriptsForExecutionContext(executionContext) {
        const result = [];
        for (const script of this._scripts.values()) {
            if (script.executionContextId === executionContext.id) {
                result.push(script);
            }
        }
        return result;
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @param {string} newSource
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, !Protocol.Runtime.ExceptionDetails=):void} callback
     */
    setScriptSource(scriptId, newSource, callback) {
        const script = this._scripts.get(scriptId);
        if (script) {
            script.editSource(newSource, this._didEditScriptSource.bind(this, scriptId, newSource, callback));
        }
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @param {string} newSource
     * @param {function(?ProtocolClient.InspectorBackend.ProtocolError, !Protocol.Runtime.ExceptionDetails=):void} callback
     * @param {?ProtocolClient.InspectorBackend.ProtocolError} error
     * @param {!Protocol.Runtime.ExceptionDetails=} exceptionDetails
     * @param {!Array.<!Protocol.Debugger.CallFrame>=} callFrames
     * @param {!Protocol.Runtime.StackTrace=} asyncStackTrace
     * @param {!Protocol.Runtime.StackTraceId=} asyncStackTraceId
     * @param {boolean=} needsStepIn
     */
    _didEditScriptSource(scriptId, newSource, callback, error, exceptionDetails, callFrames, asyncStackTrace, asyncStackTraceId, needsStepIn) {
        callback(error, exceptionDetails);
        if (needsStepIn) {
            this.stepInto();
            return;
        }
        if (!error && callFrames && callFrames.length && this._debuggerPausedDetails) {
            this._pausedScript(callFrames, this._debuggerPausedDetails.reason, this._debuggerPausedDetails.auxData, this._debuggerPausedDetails.breakpointIds, asyncStackTrace, asyncStackTraceId);
        }
    }
    /**
     * @return {?Array.<!CallFrame>}
     */
    get callFrames() {
        return this._debuggerPausedDetails ? this._debuggerPausedDetails.callFrames : null;
    }
    /**
     * @return {?DebuggerPausedDetails}
     */
    debuggerPausedDetails() {
        return this._debuggerPausedDetails;
    }
    /**
     * @param {?DebuggerPausedDetails} debuggerPausedDetails
     * @return {!Promise<boolean>}
     */
    async _setDebuggerPausedDetails(debuggerPausedDetails) {
        this._isPausing = false;
        this._debuggerPausedDetails = debuggerPausedDetails;
        if (debuggerPausedDetails) {
            if (this._beforePausedCallback) {
                if (!this._beforePausedCallback.call(null, debuggerPausedDetails)) {
                    return false;
                }
            }
            // @ts-ignore
            const pluginManager = Bindings.DebuggerWorkspaceBinding.instance().getLanguagePluginManager(this);
            if (pluginManager) {
                for (const callFrame of debuggerPausedDetails.callFrames) {
                    // @ts-ignore
                    callFrame.sourceScopeChain = await pluginManager.resolveScopeChain(callFrame);
                }
            }
            // If we resolved a location in auto-stepping callback, reset the
            // step-over marker.
            this._autoStepOver = false;
            this.dispatchEventToListeners(Events$b.DebuggerPaused, this);
            this.setSelectedCallFrame(debuggerPausedDetails.callFrames[0]);
        }
        else {
            this.setSelectedCallFrame(null);
        }
        return true;
    }
    /**
     * @param {?function(!DebuggerPausedDetails):boolean} callback
     */
    setBeforePausedCallback(callback) {
        this._beforePausedCallback = callback;
    }
    /**
     * @param {!Array.<!Protocol.Debugger.CallFrame>} callFrames
     * @param {string} reason
     * @param {!Object|undefined} auxData
     * @param {!Array.<string>} breakpointIds
     * @param {!Protocol.Runtime.StackTrace=} asyncStackTrace
     * @param {!Protocol.Runtime.StackTraceId=} asyncStackTraceId
     * @param {!Protocol.Runtime.StackTraceId=} asyncCallStackTraceId
     */
    async _pausedScript(callFrames, reason, auxData, breakpointIds, asyncStackTrace, asyncStackTraceId, asyncCallStackTraceId) {
        if (asyncCallStackTraceId) {
            // Note: this is only to support old backends. Newer ones do not send asyncCallStackTraceId.
            _scheduledPauseOnAsyncCall = asyncCallStackTraceId;
            const promises = [];
            for (const model of _debuggerIdToModel.values()) {
                promises.push(model._pauseOnAsyncCall(asyncCallStackTraceId));
            }
            await Promise.all(promises);
            this.resume();
            return;
        }
        const pausedDetails = new DebuggerPausedDetails(this, callFrames, reason, auxData, breakpointIds, asyncStackTrace, asyncStackTraceId);
        if (this._continueToLocationCallback) {
            const callback = this._continueToLocationCallback;
            this._continueToLocationCallback = null;
            if (callback(pausedDetails)) {
                return;
            }
        }
        if (!await this._setDebuggerPausedDetails(pausedDetails)) {
            if (this._autoStepOver) {
                this._agent.invoke_stepOver({});
            }
            else {
                this._agent.invoke_stepInto({ breakOnAsyncCall: false });
            }
        }
        _scheduledPauseOnAsyncCall = null;
    }
    _resumedScript() {
        this._setDebuggerPausedDetails(null);
        this.dispatchEventToListeners(Events$b.DebuggerResumed, this);
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @param {string} sourceURL
     * @param {number} startLine
     * @param {number} startColumn
     * @param {number} endLine
     * @param {number} endColumn
     * @param {!Protocol.Runtime.ExecutionContextId} executionContextId
     * @param {string} hash
     * @param {*|undefined} executionContextAuxData
     * @param {boolean} isLiveEdit
     * @param {string|undefined} sourceMapURL
     * @param {boolean} hasSourceURLComment
     * @param {boolean} hasSyntaxError
     * @param {number} length
     * @param {?Protocol.Runtime.StackTrace} originStackTrace
     * @param {?number} codeOffset
     * @param {?string} scriptLanguage
     * @param {?Protocol.Debugger.DebugSymbols} debugSymbols
     * @param {?string} embedderName
     * @return {!Script}
     */
    _parsedScriptSource(scriptId, sourceURL, startLine, startColumn, endLine, endColumn, executionContextId, hash, executionContextAuxData, isLiveEdit, sourceMapURL, hasSourceURLComment, hasSyntaxError, length, originStackTrace, codeOffset, scriptLanguage, debugSymbols, embedderName) {
        const knownScript = this._scripts.get(scriptId);
        if (knownScript) {
            return knownScript;
        }
        let isContentScript = false;
        if (executionContextAuxData && ('isDefault' in executionContextAuxData)) {
            isContentScript = !executionContextAuxData['isDefault'];
        }
        sourceURL = this._internString(sourceURL);
        const script = new Script(this, scriptId, sourceURL, startLine, startColumn, endLine, endColumn, executionContextId, this._internString(hash), isContentScript, isLiveEdit, sourceMapURL, hasSourceURLComment, length, originStackTrace, codeOffset, scriptLanguage, debugSymbols, embedderName);
        this._registerScript(script);
        this.dispatchEventToListeners(Events$b.ParsedScriptSource, script);
        // @ts-ignore
        const pluginManager = Bindings.DebuggerWorkspaceBinding.instance().getLanguagePluginManager(this);
        if (!Runtime.experiments.isEnabled('wasmDWARFDebugging') || !pluginManager ||
            !pluginManager.hasPluginForScript(script)) {
            const sourceMapId = DebuggerModel._sourceMapId(script.executionContextId, script.sourceURL, script.sourceMapURL);
            if (sourceMapId && !hasSyntaxError) {
                // Consecutive script evaluations in the same execution context with the same sourceURL
                // and sourceMappingURL should result in source map reloading.
                const previousScript = this._sourceMapIdToScript.get(sourceMapId);
                if (previousScript) {
                    this._sourceMapManager.detachSourceMap(previousScript);
                }
                this._sourceMapIdToScript.set(sourceMapId, script);
                this._sourceMapManager.attachSourceMap(script, script.sourceURL, script.sourceMapURL);
            }
        }
        const isDiscardable = hasSyntaxError && script.isAnonymousScript();
        if (isDiscardable) {
            this._discardableScripts.push(script);
            this._collectDiscardedScripts();
        }
        return script;
    }
    /**
     * @param {!Script} script
     * @param {string} newSourceMapURL
     */
    setSourceMapURL(script, newSourceMapURL) {
        let sourceMapId = DebuggerModel._sourceMapId(script.executionContextId, script.sourceURL, script.sourceMapURL);
        if (sourceMapId && this._sourceMapIdToScript.get(sourceMapId) === script) {
            this._sourceMapIdToScript.delete(sourceMapId);
        }
        this._sourceMapManager.detachSourceMap(script);
        script.sourceMapURL = newSourceMapURL;
        sourceMapId = DebuggerModel._sourceMapId(script.executionContextId, script.sourceURL, script.sourceMapURL);
        if (!sourceMapId) {
            return;
        }
        this._sourceMapIdToScript.set(sourceMapId, script);
        this._sourceMapManager.attachSourceMap(script, script.sourceURL, script.sourceMapURL);
    }
    /**
     * @param {!ExecutionContext} executionContext
     */
    executionContextDestroyed(executionContext) {
        const sourceMapIds = Array.from(this._sourceMapIdToScript.keys());
        for (const sourceMapId of sourceMapIds) {
            const script = this._sourceMapIdToScript.get(sourceMapId);
            if (script && script.executionContextId === executionContext.id) {
                this._sourceMapIdToScript.delete(sourceMapId);
                this._sourceMapManager.detachSourceMap(script);
            }
        }
    }
    /**
     * @param {!Script} script
     */
    _registerScript(script) {
        this._scripts.set(script.scriptId, script);
        if (script.isAnonymousScript()) {
            return;
        }
        let scripts = this._scriptsBySourceURL.get(script.sourceURL);
        if (!scripts) {
            scripts = [];
            this._scriptsBySourceURL.set(script.sourceURL, scripts);
        }
        scripts.push(script);
    }
    /**
     * @param {!Script} script
     */
    _unregisterScript(script) {
        console.assert(script.isAnonymousScript());
        this._scripts.delete(script.scriptId);
    }
    _collectDiscardedScripts() {
        if (this._discardableScripts.length < 1000) {
            return;
        }
        const scriptsToDiscard = this._discardableScripts.splice(0, 100);
        for (const script of scriptsToDiscard) {
            this._unregisterScript(script);
            this.dispatchEventToListeners(Events$b.DiscardedAnonymousScriptSource, script);
        }
    }
    /**
     * @param {!Script} script
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {!Location}
     */
    createRawLocation(script, lineNumber, columnNumber) {
        return new Location(this, script.scriptId, lineNumber, columnNumber);
    }
    /**
     * @param {string} sourceURL
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?Location}
     */
    createRawLocationByURL(sourceURL, lineNumber, columnNumber) {
        let closestScript = null;
        const scripts = this._scriptsBySourceURL.get(sourceURL) || [];
        for (let i = 0, l = scripts.length; i < l; ++i) {
            const script = scripts[i];
            if (!closestScript) {
                closestScript = script;
            }
            if (script.lineOffset > lineNumber || (script.lineOffset === lineNumber && script.columnOffset > columnNumber)) {
                continue;
            }
            if (script.endLine < lineNumber || (script.endLine === lineNumber && script.endColumn <= columnNumber)) {
                continue;
            }
            closestScript = script;
            break;
        }
        return closestScript ? new Location(this, closestScript.scriptId, lineNumber, columnNumber) : null;
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @param {number} lineNumber
     * @param {number} columnNumber
     * @return {?Location}
     */
    createRawLocationByScriptId(scriptId, lineNumber, columnNumber) {
        const script = this.scriptForId(scriptId);
        return script ? this.createRawLocation(script, lineNumber, columnNumber) : null;
    }
    /**
     * @param {!Protocol.Runtime.StackTrace} stackTrace
     * @return {!Array<!Location>}
     */
    createRawLocationsByStackTrace(stackTrace) {
        const frames = [];
        /** @type {!Protocol.Runtime.StackTrace|undefined} */
        let current = stackTrace;
        while (current) {
            for (const frame of current.callFrames) {
                frames.push(frame);
            }
            current = current.parent;
        }
        const rawLocations = [];
        for (const frame of frames) {
            const rawLocation = this.createRawLocationByScriptId(frame.scriptId, frame.lineNumber, frame.columnNumber);
            if (rawLocation) {
                rawLocations.push(rawLocation);
            }
        }
        return rawLocations;
    }
    /**
     * @return {boolean}
     */
    isPaused() {
        return !!this.debuggerPausedDetails();
    }
    /**
     * @return {boolean}
     */
    isPausing() {
        return this._isPausing;
    }
    /**
     * @param {?CallFrame} callFrame
     */
    setSelectedCallFrame(callFrame) {
        if (this._selectedCallFrame === callFrame) {
            return;
        }
        this._selectedCallFrame = callFrame;
        this.dispatchEventToListeners(Events$b.CallFrameSelected, this);
    }
    /**
     * @return {?CallFrame}
     */
    selectedCallFrame() {
        return this._selectedCallFrame;
    }
    /**
     * @param {!EvaluationOptions} options
     * @return {!Promise<!EvaluationResult>}
     */
    async evaluateOnSelectedCallFrame(options) {
        const callFrame = this.selectedCallFrame();
        if (!callFrame) {
            throw new Error('No call frame selected');
        }
        return callFrame.evaluate(options);
    }
    /**
     * @param {string} callFrameId
     * @param {string} evaluator
     * @return {!Promise<!Protocol.Debugger.ExecuteWasmEvaluatorResponse>}
     */
    executeWasmEvaluator(callFrameId, evaluator) {
        return this._agent.invoke_executeWasmEvaluator({ callFrameId, evaluator });
    }
    /**
     * @param {!RemoteObject} remoteObject
     * @return {!Promise<?FunctionDetails>}
     */
    functionDetailsPromise(remoteObject) {
        return remoteObject.getAllProperties(false /* accessorPropertiesOnly */, false /* generatePreview */)
            .then(buildDetails.bind(this));
        /**
         * @param {!GetPropertiesResult} response
         * @return {?FunctionDetails}
         * @this {!DebuggerModel}
         */
        function buildDetails(response) {
            if (!response) {
                return null;
            }
            let location = null;
            if (response.internalProperties) {
                for (const prop of response.internalProperties) {
                    if (prop.name === '[[FunctionLocation]]') {
                        location = prop.value;
                    }
                }
            }
            let functionName = null;
            if (response.properties) {
                for (const prop of response.properties) {
                    if (prop.name === 'name' && prop.value && prop.value.type === 'string') {
                        functionName = prop.value;
                    }
                    if (prop.name === 'displayName' && prop.value && prop.value.type === 'string') {
                        functionName = prop.value;
                        break;
                    }
                }
            }
            let debuggerLocation = null;
            if (location) {
                debuggerLocation = this.createRawLocationByScriptId(location.value.scriptId, location.value.lineNumber, location.value.columnNumber);
            }
            return { location: debuggerLocation, functionName: functionName ? /** @type {string} */ (functionName.value) : '' };
        }
    }
    /**
     * @param {number} scopeNumber
     * @param {string} variableName
     * @param {!Protocol.Runtime.CallArgument} newValue
     * @param {string} callFrameId
     * @return {!Promise<string|undefined>}
     */
    async setVariableValue(scopeNumber, variableName, newValue, callFrameId) {
        const response = await this._agent.invoke_setVariableValue({ scopeNumber, variableName, newValue, callFrameId });
        const error = response.getError();
        if (error) {
            console.error(error);
        }
        return error;
    }
    /**
     * @param {!Protocol.Debugger.BreakpointId} breakpointId
     * @param {function(!Common.EventTarget.EventTargetEvent):void} listener
     * @param {!Object=} thisObject
     */
    addBreakpointListener(breakpointId, listener, thisObject) {
        this._breakpointResolvedEventTarget.addEventListener(breakpointId, listener, thisObject);
    }
    /**
     * @param {!Protocol.Debugger.BreakpointId} breakpointId
     * @param {function(!Common.EventTarget.EventTargetEvent):void} listener
     * @param {!Object=} thisObject
     */
    removeBreakpointListener(breakpointId, listener, thisObject) {
        this._breakpointResolvedEventTarget.removeEventListener(breakpointId, listener, thisObject);
    }
    /**
     * @param {!Array<string>} patterns
     * @return {!Promise<boolean>}
     */
    async setBlackboxPatterns(patterns) {
        const response = await this._agent.invoke_setBlackboxPatterns({ patterns });
        const error = response.getError();
        if (error) {
            console.error(error);
        }
        return !error;
    }
    /**
     * @override
     */
    dispose() {
        this._sourceMapManager.dispose();
        if (this._debuggerId) {
            _debuggerIdToModel.delete(this._debuggerId);
        }
        Settings.Settings.instance()
            .moduleSetting('pauseOnExceptionEnabled')
            .removeChangeListener(this._pauseOnExceptionStateChanged, this);
        Settings.Settings.instance()
            .moduleSetting('pauseOnCaughtException')
            .removeChangeListener(this._pauseOnExceptionStateChanged, this);
        Settings.Settings.instance()
            .moduleSetting('disableAsyncStackTraces')
            .removeChangeListener(this._asyncStackTracesStateChanged, this);
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    async suspendModel() {
        await this._disableDebugger();
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    async resumeModel() {
        await this._enableDebugger();
    }
    /**
     * @param {string} string
     * @return {string} string
     */
    _internString(string) {
        const internedString = this._stringMap.get(string);
        if (internedString === undefined) {
            this._stringMap.set(string, string);
            return string;
        }
        return internedString;
    }
}
/** @type {!Map<string, !DebuggerModel>} */
const _debuggerIdToModel = new Map();
/** @type {?Protocol.Runtime.StackTraceId} */
let _scheduledPauseOnAsyncCall = null;
/**
 * Keep these in sync with WebCore::V8Debugger
 *
 * @enum {string}
 */
const PauseOnExceptionsState = {
    DontPauseOnExceptions: 'none',
    PauseOnAllExceptions: 'all',
    PauseOnUncaughtExceptions: 'uncaught'
};
/** @enum {symbol} */
const Events$b = {
    DebuggerWasEnabled: Symbol('DebuggerWasEnabled'),
    DebuggerWasDisabled: Symbol('DebuggerWasDisabled'),
    DebuggerPaused: Symbol('DebuggerPaused'),
    DebuggerResumed: Symbol('DebuggerResumed'),
    ParsedScriptSource: Symbol('ParsedScriptSource'),
    FailedToParseScriptSource: Symbol('FailedToParseScriptSource'),
    DiscardedAnonymousScriptSource: Symbol('DiscardedAnonymousScriptSource'),
    GlobalObjectCleared: Symbol('GlobalObjectCleared'),
    CallFrameSelected: Symbol('CallFrameSelected'),
    ConsoleCommandEvaluatedInSelectedCallFrame: Symbol('ConsoleCommandEvaluatedInSelectedCallFrame'),
    DebuggerIsReadyToPause: Symbol('DebuggerIsReadyToPause'),
};
/** @enum {string} */
const BreakReason = {
    DOM: 'DOM',
    EventListener: 'EventListener',
    XHR: 'XHR',
    Exception: 'exception',
    PromiseRejection: 'promiseRejection',
    Assert: 'assert',
    DebugCommand: 'debugCommand',
    OOM: 'OOM',
    Other: 'other'
};
/**
 * @implements {ProtocolProxyApi.DebuggerDispatcher}
 */
class DebuggerDispatcher {
    /**
     * @param {!DebuggerModel} debuggerModel
     */
    constructor(debuggerModel) {
        this._debuggerModel = debuggerModel;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @override
     * @param {!Protocol.Debugger.PausedEvent} event
     */
    paused({ callFrames, reason, data, hitBreakpoints, asyncStackTrace, asyncStackTraceId, asyncCallStackTraceId }) {
        this._debuggerModel._pausedScript(callFrames, reason, data, hitBreakpoints || [], asyncStackTrace, asyncStackTraceId, asyncCallStackTraceId);
    }
    /**
     * @override
     */
    resumed() {
        this._debuggerModel._resumedScript();
    }
    /**
     * @override
     * @param {!Protocol.Debugger.ScriptParsedEvent} event
     */
    scriptParsed({ scriptId, url, startLine, startColumn, endLine, endColumn, executionContextId, hash, executionContextAuxData, isLiveEdit, sourceMapURL, hasSourceURL, isModule, length, stackTrace, codeOffset, scriptLanguage, debugSymbols, embedderName }) {
        this._debuggerModel._parsedScriptSource(scriptId, url, startLine, startColumn, endLine, endColumn, executionContextId, hash, executionContextAuxData, !!isLiveEdit, sourceMapURL, !!hasSourceURL, false, length || 0, stackTrace || null, codeOffset || null, scriptLanguage || null, debugSymbols || null, embedderName || null);
    }
    /**
     * @override
     * @param {!Protocol.Debugger.ScriptFailedToParseEvent} event
     */
    scriptFailedToParse({ scriptId, url, startLine, startColumn, endLine, endColumn, executionContextId, hash, executionContextAuxData, sourceMapURL, hasSourceURL, isModule, length, stackTrace, codeOffset, scriptLanguage, embedderName }) {
        this._debuggerModel._parsedScriptSource(scriptId, url, startLine, startColumn, endLine, endColumn, executionContextId, hash, executionContextAuxData, false, sourceMapURL, !!hasSourceURL, true, length || 0, stackTrace || null, codeOffset || null, scriptLanguage || null, null, embedderName || null);
    }
    /**
     * @override
     * @param {!Protocol.Debugger.BreakpointResolvedEvent} event
     */
    breakpointResolved({ breakpointId, location }) {
        this._debuggerModel._breakpointResolved(breakpointId, location);
    }
}
class Location {
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {string} scriptId
     * @param {number} lineNumber
     * @param {number=} columnNumber
     */
    constructor(debuggerModel, scriptId, lineNumber, columnNumber) {
        this.debuggerModel = debuggerModel;
        this.scriptId = scriptId;
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber || 0;
    }
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {!Protocol.Debugger.Location} payload
     * @return {!Location}
     */
    static fromPayload(debuggerModel, payload) {
        return new Location(debuggerModel, payload.scriptId, payload.lineNumber, payload.columnNumber);
    }
    /**
     * @return {!Protocol.Debugger.Location}
     */
    payload() {
        return { scriptId: this.scriptId, lineNumber: this.lineNumber, columnNumber: this.columnNumber };
    }
    /**
     * @return {?Script}
     */
    script() {
        return this.debuggerModel.scriptForId(this.scriptId);
    }
    /**
     * @param {function():void=} pausedCallback
     */
    continueToLocation(pausedCallback) {
        if (pausedCallback) {
            this.debuggerModel._continueToLocationCallback = this._paused.bind(this, pausedCallback);
        }
        this.debuggerModel._agent.invoke_continueToLocation({
            location: this.payload(),
            targetCallFrames: Protocol.Debugger.ContinueToLocationRequestTargetCallFrames.Current
        });
    }
    /**
     * @param {function():void|undefined} pausedCallback
     * @param {!DebuggerPausedDetails} debuggerPausedDetails
     * @return {boolean}
     */
    _paused(pausedCallback, debuggerPausedDetails) {
        const location = debuggerPausedDetails.callFrames[0].location();
        if (location.scriptId === this.scriptId && location.lineNumber === this.lineNumber &&
            location.columnNumber === this.columnNumber) {
            pausedCallback();
            return true;
        }
        return false;
    }
    /**
     * @return {string}
     */
    id() {
        return this.debuggerModel.target().id() + ':' + this.scriptId + ':' + this.lineNumber + ':' + this.columnNumber;
    }
}
class ScriptPosition {
    /**
     * @param {number} lineNumber
     * @param {number} columnNumber
     */
    constructor(lineNumber, columnNumber) {
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber;
    }
    /**
     * @return {!Protocol.Debugger.ScriptPosition}
     */
    payload() {
        return { lineNumber: this.lineNumber, columnNumber: this.columnNumber };
    }
    /**
    * @param {!ScriptPosition} other
    * @return {number}
    */
    compareTo(other) {
        if (this.lineNumber !== other.lineNumber) {
            return this.lineNumber - other.lineNumber;
        }
        return this.columnNumber - other.columnNumber;
    }
}
class LocationRange {
    /**
     * @param {string} scriptId
     * @param {!ScriptPosition} start
     * @param {!ScriptPosition} end
     */
    constructor(scriptId, start, end) {
        this.scriptId = scriptId;
        this.start = start;
        this.end = end;
    }
    /**
     * @return {!Protocol.Debugger.LocationRange}
     */
    payload() {
        return { scriptId: this.scriptId, start: this.start.payload(), end: this.end.payload() };
    }
    /**
     * @param {!LocationRange} location1
     * @param {!LocationRange} location2
     * @return {number}
     */
    static comparator(location1, location2) {
        return location1.compareTo(location2);
    }
    /**
     * @param {!LocationRange} other
     * @return {number}
     */
    compareTo(other) {
        if (this.scriptId !== other.scriptId) {
            return this.scriptId > other.scriptId ? 1 : -1;
        }
        const startCmp = this.start.compareTo(other.start);
        if (startCmp) {
            return startCmp;
        }
        return this.end.compareTo(other.end);
    }
    /**
     * @param {!LocationRange} other
     * @return boolean
     */
    overlap(other) {
        if (this.scriptId !== other.scriptId) {
            return false;
        }
        const startCmp = this.start.compareTo(other.start);
        if (startCmp < 0) {
            return this.end.compareTo(other.start) >= 0;
        }
        if (startCmp > 0) {
            return this.start.compareTo(other.end) <= 0;
        }
        return true;
    }
}
class BreakLocation extends Location {
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {string} scriptId
     * @param {number} lineNumber
     * @param {number=} columnNumber
     * @param {!Protocol.Debugger.BreakLocationType=} type
     */
    constructor(debuggerModel, scriptId, lineNumber, columnNumber, type) {
        super(debuggerModel, scriptId, lineNumber, columnNumber);
        if (type) {
            this.type = type;
        }
    }
    /**
     * @override
     * @param {!DebuggerModel} debuggerModel
     * @param {!Protocol.Debugger.BreakLocation} payload
     * @return {!BreakLocation}
     */
    static fromPayload(debuggerModel, payload) {
        return new BreakLocation(debuggerModel, payload.scriptId, payload.lineNumber, payload.columnNumber, payload.type);
    }
}
class CallFrame {
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {!Script} script
     * @param {!Protocol.Debugger.CallFrame} payload
     */
    constructor(debuggerModel, script, payload) {
        this.debuggerModel = debuggerModel;
        /** @type {?Array<!ScopeChainEntry>} */
        this.sourceScopeChain = null;
        this._script = script;
        this._payload = payload;
        this._location = Location.fromPayload(debuggerModel, payload.location);
        /** @type {!Array<!Scope>} */
        this._scopeChain = [];
        this._localScope = null;
        for (let i = 0; i < payload.scopeChain.length; ++i) {
            const scope = new Scope(this, i);
            this._scopeChain.push(scope);
            if (scope.type() === Protocol.Debugger.ScopeType.Local) {
                this._localScope = scope;
            }
        }
        if (payload.functionLocation) {
            this._functionLocation = Location.fromPayload(debuggerModel, payload.functionLocation);
        }
        this._returnValue =
            payload.returnValue ? this.debuggerModel._runtimeModel.createRemoteObject(payload.returnValue) : null;
    }
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {!Array.<!Protocol.Debugger.CallFrame>} callFrames
     * @return {!Array.<!CallFrame>}
     */
    static fromPayloadArray(debuggerModel, callFrames) {
        const result = [];
        for (let i = 0; i < callFrames.length; ++i) {
            const callFrame = callFrames[i];
            const script = debuggerModel.scriptForId(callFrame.location.scriptId);
            if (script) {
                result.push(new CallFrame(debuggerModel, script, callFrame));
            }
        }
        return result;
    }
    /**
     * @return {!Script}
     */
    get script() {
        return this._script;
    }
    /**
     * @return {string}
     */
    get id() {
        return this._payload.callFrameId;
    }
    /**
     * @return {!Array.<!Scope>}
     */
    scopeChain() {
        return this._scopeChain;
    }
    /**
     * @return {?Scope}
     */
    localScope() {
        return this._localScope;
    }
    /**
     * @return {?RemoteObject}
     */
    thisObject() {
        return this._payload.this ? this.debuggerModel._runtimeModel.createRemoteObject(this._payload.this) : null;
    }
    /**
     * @return {?RemoteObject}
     */
    returnValue() {
        return this._returnValue;
    }
    /**
     * @param {string} expression
     * @return {!Promise<?RemoteObject>}
     */
    async setReturnValue(expression) {
        if (!this._returnValue) {
            return null;
        }
        const evaluateResponse = await this.debuggerModel._agent.invoke_evaluateOnCallFrame({ callFrameId: this.id, expression: expression, silent: true, objectGroup: 'backtrace' });
        if (evaluateResponse.getError() || evaluateResponse.exceptionDetails) {
            return null;
        }
        const response = await this.debuggerModel._agent.invoke_setReturnValue({ newValue: evaluateResponse.result });
        if (response.getError()) {
            return null;
        }
        this._returnValue = this.debuggerModel._runtimeModel.createRemoteObject(evaluateResponse.result);
        return this._returnValue;
    }
    /**
     * @return {string}
     */
    get functionName() {
        return this._payload.functionName;
    }
    /**
     * @return {!Location}
     */
    location() {
        return this._location;
    }
    /**
     * @return {?Location}
     */
    functionLocation() {
        return this._functionLocation || null;
    }
    /**
     * @param {!EvaluationOptions} options
     * @return {!Promise<!EvaluationResult>}
     */
    async evaluate(options) {
        const runtimeModel = this.debuggerModel.runtimeModel();
        // Assume backends either support both throwOnSideEffect and timeout options or neither.
        const needsTerminationOptions = !!options.throwOnSideEffect || options.timeout !== undefined;
        if (needsTerminationOptions &&
            (runtimeModel.hasSideEffectSupport() === false ||
                (runtimeModel.hasSideEffectSupport() === null && !await runtimeModel.checkSideEffectSupport()))) {
            return { error: 'Side-effect checks not supported by backend.' };
        }
        const response = await this.debuggerModel._agent.invoke_evaluateOnCallFrame({
            callFrameId: this.id,
            expression: options.expression,
            objectGroup: options.objectGroup,
            includeCommandLineAPI: options.includeCommandLineAPI,
            silent: options.silent,
            returnByValue: options.returnByValue,
            generatePreview: options.generatePreview,
            throwOnSideEffect: options.throwOnSideEffect,
            timeout: options.timeout
        });
        const error = response.getError();
        if (error) {
            console.error(error);
            return { error: error };
        }
        return { object: runtimeModel.createRemoteObject(response.result), exceptionDetails: response.exceptionDetails };
    }
    async restart() {
        const response = await this.debuggerModel._agent.invoke_restartFrame({ callFrameId: this._payload.callFrameId });
        if (!response.getError()) {
            this.debuggerModel.stepInto();
        }
    }
}
/**
 * @interface
 */
class ScopeChainEntry {
    /**
     * @return {!CallFrame}
     */
    callFrame() {
        throw new Error('not implemented');
    }
    /**
     * @return {string}
     */
    type() {
        throw new Error('not implemented');
    }
    /**
     * @return {string}
     */
    typeName() {
        throw new Error('not implemented');
    }
    /**
     * @return {string|undefined}
     */
    name() {
        throw new Error('not implemented');
    }
    /**
     * @return {?Location}
     */
    startLocation() {
        throw new Error('not implemented');
    }
    /**
     * @return {?Location}
     */
    endLocation() {
        throw new Error('not implemented');
    }
    /**
     * @return {!RemoteObject}
     */
    object() {
        throw new Error('not implemented');
    }
    /**
     * @return {string}
     */
    description() {
        throw new Error('not implemented');
    }
}
/**
 * @implements {ScopeChainEntry}
 */
class Scope {
    /**
     * @param {!CallFrame} callFrame
     * @param {number} ordinal
     */
    constructor(callFrame, ordinal) {
        this._callFrame = callFrame;
        this._payload = callFrame._payload.scopeChain[ordinal];
        this._type = this._payload.type;
        this._name = this._payload.name;
        this._ordinal = ordinal;
        this._startLocation =
            this._payload.startLocation ? Location.fromPayload(callFrame.debuggerModel, this._payload.startLocation) : null;
        this._endLocation =
            this._payload.endLocation ? Location.fromPayload(callFrame.debuggerModel, this._payload.endLocation) : null;
        /** @type {?RemoteObject} */
        this._object = null;
    }
    /**
     * @override
     * @return {!CallFrame}
     */
    callFrame() {
        return this._callFrame;
    }
    /**
     * @override
     * @return {string}
     */
    type() {
        return this._type;
    }
    /**
     * @override
     * @return {string}
     */
    typeName() {
        switch (this._type) {
            case Protocol.Debugger.ScopeType.Local:
                return UIString.UIString('Local');
            case Protocol.Debugger.ScopeType.Closure:
                return UIString.UIString('Closure');
            case Protocol.Debugger.ScopeType.Catch:
                return UIString.UIString('Catch');
            case Protocol.Debugger.ScopeType.Block:
                return UIString.UIString('Block');
            case Protocol.Debugger.ScopeType.Script:
                return UIString.UIString('Script');
            case Protocol.Debugger.ScopeType.With:
                return UIString.UIString('With Block');
            case Protocol.Debugger.ScopeType.Global:
                return UIString.UIString('Global');
            case Protocol.Debugger.ScopeType.Module:
                return UIString.UIString('Module');
            case Protocol.Debugger.ScopeType.WasmExpressionStack:
                return UIString.UIString('Stack');
        }
        return '';
    }
    /**
     * @override
     * @return {string|undefined}
     */
    name() {
        return this._name;
    }
    /**
     * @override
     * @return {?Location}
     */
    startLocation() {
        return this._startLocation;
    }
    /**
     * @override
     * @return {?Location}
     */
    endLocation() {
        return this._endLocation;
    }
    /**
     * @override
     * @return {!RemoteObject}
     */
    object() {
        if (this._object) {
            return this._object;
        }
        const runtimeModel = this._callFrame.debuggerModel._runtimeModel;
        const declarativeScope = this._type !== Protocol.Debugger.ScopeType.With && this._type !== Protocol.Debugger.ScopeType.Global;
        if (declarativeScope) {
            this._object =
                runtimeModel.createScopeRemoteObject(this._payload.object, new ScopeRef(this._ordinal, this._callFrame.id));
        }
        else {
            this._object = runtimeModel.createRemoteObject(this._payload.object);
        }
        return this._object;
    }
    /**
     * @override
     * @return {string}
     */
    description() {
        const declarativeScope = this._type !== Protocol.Debugger.ScopeType.With && this._type !== Protocol.Debugger.ScopeType.Global;
        return declarativeScope ? '' : (this._payload.object.description || '');
    }
}
class DebuggerPausedDetails {
    /**
     * @param {!DebuggerModel} debuggerModel
     * @param {!Array.<!Protocol.Debugger.CallFrame>} callFrames
     * @param {string} reason
     * @param {!Object|undefined} auxData
     * @param {!Array.<string>} breakpointIds
     * @param {!Protocol.Runtime.StackTrace=} asyncStackTrace
     * @param {!Protocol.Runtime.StackTraceId=} asyncStackTraceId
     */
    constructor(debuggerModel, callFrames, reason, auxData, breakpointIds, asyncStackTrace, asyncStackTraceId) {
        this.debuggerModel = debuggerModel;
        this.callFrames = CallFrame.fromPayloadArray(debuggerModel, callFrames);
        this.reason = reason;
        this.auxData = auxData;
        this.breakpointIds = breakpointIds;
        if (asyncStackTrace) {
            this.asyncStackTrace = this._cleanRedundantFrames(asyncStackTrace);
        }
        this.asyncStackTraceId = asyncStackTraceId;
    }
    /**
     * @return {?RemoteObject}
     */
    exception() {
        if (this.reason !== BreakReason.Exception && this.reason !== BreakReason.PromiseRejection) {
            return null;
        }
        return this.debuggerModel._runtimeModel.createRemoteObject(
        /** @type {!Protocol.Runtime.RemoteObject} */ (this.auxData));
    }
    /**
     * @param {!Protocol.Runtime.StackTrace} asyncStackTrace
     * @return {!Protocol.Runtime.StackTrace}
     */
    _cleanRedundantFrames(asyncStackTrace) {
        /** @type {!Protocol.Runtime.StackTrace|undefined} */
        let stack = asyncStackTrace;
        let previous = null;
        while (stack) {
            if (stack.description === 'async function' && stack.callFrames.length) {
                stack.callFrames.shift();
            }
            if (previous && !stack.callFrames.length) {
                previous.parent = stack.parent;
            }
            else {
                previous = stack;
            }
            stack = stack.parent;
        }
        return asyncStackTrace;
    }
}
SDKModel.register(DebuggerModel, Capability.JS, true);
/** @typedef {{location: ?Location, functionName: string}} */
// @ts-ignore typedef
let FunctionDetails;
/** @typedef {{
 *    breakpointId: ?Protocol.Debugger.BreakpointId,
 *    locations: !Array<!Location>
 *  }}
 */
// @ts-ignore typedef
let SetBreakpointResult;

var DebuggerModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    sortAndMergeRanges: sortAndMergeRanges,
    DebuggerModel: DebuggerModel,
    _debuggerIdToModel: _debuggerIdToModel,
    get _scheduledPauseOnAsyncCall () { return _scheduledPauseOnAsyncCall; },
    PauseOnExceptionsState: PauseOnExceptionsState,
    Events: Events$b,
    BreakReason: BreakReason,
    Location: Location,
    ScriptPosition: ScriptPosition,
    LocationRange: LocationRange,
    BreakLocation: BreakLocation,
    CallFrame: CallFrame,
    ScopeChainEntry: ScopeChainEntry,
    Scope: Scope,
    DebuggerPausedDetails: DebuggerPausedDetails,
    FunctionDetails: FunctionDetails,
    SetBreakpointResult: SetBreakpointResult
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class HeapProfilerModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        target.registerHeapProfilerDispatcher(new HeapProfilerDispatcher(this));
        this._enabled = false;
        this._heapProfilerAgent = target.heapProfilerAgent();
        this._memoryAgent = target.memoryAgent();
        this._runtimeModel = /** @type {!RuntimeModel} */ (target.model(RuntimeModel));
        this._samplingProfilerDepth = 0;
    }
    /**
     * @return {!DebuggerModel}
     */
    debuggerModel() {
        return this._runtimeModel.debuggerModel();
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    async enable() {
        if (this._enabled) {
            return;
        }
        this._enabled = true;
        await this._heapProfilerAgent.invoke_enable();
    }
    /**
     * @param {number=} samplingRateInBytes
     * @returns {!Promise<boolean>}
     */
    async startSampling(samplingRateInBytes) {
        if (this._samplingProfilerDepth++) {
            return false;
        }
        const defaultSamplingIntervalInBytes = 16384;
        const response = await this._heapProfilerAgent.invoke_startSampling({ samplingInterval: samplingRateInBytes || defaultSamplingIntervalInBytes });
        return !!response.getError();
    }
    /**
     * @return {!Promise<?Protocol.HeapProfiler.SamplingHeapProfile>}
     */
    async stopSampling() {
        if (!this._samplingProfilerDepth) {
            throw new Error('Sampling profiler is not running.');
        }
        if (--this._samplingProfilerDepth) {
            return this.getSamplingProfile();
        }
        const response = await this._heapProfilerAgent.invoke_stopSampling();
        if (response.getError()) {
            return null;
        }
        return response.profile;
    }
    /**
     * @return {!Promise<?Protocol.HeapProfiler.SamplingHeapProfile>}
     */
    async getSamplingProfile() {
        const response = await this._heapProfilerAgent.invoke_getSamplingProfile();
        if (response.getError()) {
            return null;
        }
        return response.profile;
    }
    /**
     * @returns {!Promise<boolean>}
     */
    async startNativeSampling() {
        const samplingInterval = 65536;
        const response = await this._memoryAgent.invoke_startSampling({ samplingInterval });
        return !!response.getError();
    }
    /**
     * @return {!Promise<?NativeHeapProfile>}
     */
    async stopNativeSampling() {
        const response = await this._memoryAgent.invoke_getSamplingProfile();
        // Try to stop sampling independent of an error in `getSamplingProfile`.
        await this._memoryAgent.invoke_stopSampling();
        if (response.getError()) {
            return null;
        }
        return this._convertNativeProfile(response.profile);
    }
    /**
     * @return {!Promise<?NativeHeapProfile>}
     */
    async takeNativeSnapshot() {
        const response = await this._memoryAgent.invoke_getAllTimeSamplingProfile();
        if (response.getError()) {
            return null;
        }
        return this._convertNativeProfile(response.profile);
    }
    /**
     * @return {!Promise<?NativeHeapProfile>}
     */
    async takeNativeBrowserSnapshot() {
        const response = await this._memoryAgent.invoke_getBrowserSamplingProfile();
        if (response.getError()) {
            return null;
        }
        return this._convertNativeProfile(response.profile);
    }
    /**
     * @param {!Protocol.Memory.SamplingProfile} rawProfile
     * @return {!NativeHeapProfile}
     */
    _convertNativeProfile(rawProfile) {
        /** @type {!NodeForConstruction}} */
        const head = (({
            childMap: new Map(),
            selfSize: 0,
            callFrame: { functionName: '(root)', url: '', scriptId: '', lineNumber: -1, columnNumber: -1 }
        }));
        for (const sample of rawProfile.samples) {
            const node = sample.stack.reverse().reduce((node, name) => {
                let child = node.childMap.get(name);
                if (child) {
                    return child;
                }
                const namespace = /^([^:]*)::/.exec(name);
                // The native profile doesn't have scriptId and line/column numbers, so we need to pass undefined to
                // not have them displayed.
                const callFrame = /** @type {!NativeProfilerCallFrame} */ ({
                    functionName: name,
                    url: namespace && namespace[1] || '',
                    scriptId: undefined,
                    lineNumber: undefined,
                    columnNumber: undefined,
                });
                child = { childMap: new Map(), callFrame, selfSize: 0 };
                node.childMap.set(name, child);
                return child;
            }, head);
            node.selfSize += sample.total;
        }
        /**
         * @param {!NodeForConstruction} node
         * @return {!CommonHeapProfileNode}
         */
        function convertChildren(node) {
            const children = Array.from(node.childMap.values()).map(convertChildren);
            return { selfSize: node.selfSize, callFrame: node.callFrame, children, id: -1 };
        }
        return new NativeHeapProfile(convertChildren(head), rawProfile.modules);
    }
    /**
     * @return {!Promise<boolean>}
     */
    async collectGarbage() {
        const response = await this._heapProfilerAgent.invoke_collectGarbage();
        return !!response.getError();
    }
    /**
     * @param {string} objectId
     * @return {!Promise<?string>}
     */
    async snapshotObjectIdForObjectId(objectId) {
        const response = await this._heapProfilerAgent.invoke_getHeapObjectId({ objectId });
        if (response.getError()) {
            return null;
        }
        return response.heapSnapshotObjectId;
    }
    /**
     * @param {string} snapshotObjectId
     * @param {string} objectGroupName
     * @return {!Promise<?RemoteObject>}
     */
    async objectForSnapshotObjectId(snapshotObjectId, objectGroupName) {
        const result = await this._heapProfilerAgent.invoke_getObjectByHeapObjectId({ objectId: snapshotObjectId, objectGroup: objectGroupName });
        if (result.getError()) {
            return null;
        }
        return this._runtimeModel.createRemoteObject(result.result);
    }
    /**
     * @param {string} snapshotObjectId
     * @return {!Promise<boolean>}
     */
    async addInspectedHeapObject(snapshotObjectId) {
        const response = await this._heapProfilerAgent.invoke_addInspectedHeapObject({ heapObjectId: snapshotObjectId });
        return !!response.getError();
    }
    /**
     * @param {boolean} reportProgress
     * @param {boolean} treatGlobalObjectsAsRoots
     * @return {!Promise<void>}
     */
    async takeHeapSnapshot(reportProgress, treatGlobalObjectsAsRoots) {
        await this._heapProfilerAgent.invoke_takeHeapSnapshot({ reportProgress, treatGlobalObjectsAsRoots });
    }
    /**
     * @param {boolean} recordAllocationStacks
     * @return {!Promise<boolean>}
     */
    async startTrackingHeapObjects(recordAllocationStacks) {
        const response = await this._heapProfilerAgent.invoke_startTrackingHeapObjects({ trackAllocations: recordAllocationStacks });
        return !!response.getError();
    }
    /**
     * @param {boolean} reportProgress
     * @return {!Promise<boolean>}
     */
    async stopTrackingHeapObjects(reportProgress) {
        const response = await this._heapProfilerAgent.invoke_stopTrackingHeapObjects({ reportProgress });
        return !!response.getError();
    }
    /**
     * @param {!Array<number>} samples
     */
    heapStatsUpdate(samples) {
        this.dispatchEventToListeners(Events$c.HeapStatsUpdate, samples);
    }
    /**
     * @param {number} lastSeenObjectId
     * @param {number} timestamp
     */
    lastSeenObjectId(lastSeenObjectId, timestamp) {
        this.dispatchEventToListeners(Events$c.LastSeenObjectId, { lastSeenObjectId: lastSeenObjectId, timestamp: timestamp });
    }
    /**
     * @param {string} chunk
     */
    addHeapSnapshotChunk(chunk) {
        this.dispatchEventToListeners(Events$c.AddHeapSnapshotChunk, chunk);
    }
    /**
     * @param {number} done
     * @param {number} total
     * @param {boolean=} finished
     */
    reportHeapSnapshotProgress(done, total, finished) {
        this.dispatchEventToListeners(Events$c.ReportHeapSnapshotProgress, { done: done, total: total, finished: finished });
    }
    resetProfiles() {
        this.dispatchEventToListeners(Events$c.ResetProfiles, this);
    }
}
/** @enum {symbol} */
const Events$c = {
    HeapStatsUpdate: Symbol('HeapStatsUpdate'),
    LastSeenObjectId: Symbol('LastSeenObjectId'),
    AddHeapSnapshotChunk: Symbol('AddHeapSnapshotChunk'),
    ReportHeapSnapshotProgress: Symbol('ReportHeapSnapshotProgress'),
    ResetProfiles: Symbol('ResetProfiles')
};
/** @typedef {!{functionName: string, url: string, scriptId: (Protocol.Runtime.ScriptId|undefined), lineNumber: (number|undefined), columnNumber: (number|undefined)}} */
// @ts-ignore typedef
let NativeProfilerCallFrame; // eslint-disable-line no-unused-vars
/** @typedef {!{callFrame: !NativeProfilerCallFrame, selfSize: number, id: (number|undefined), children: Array<!CommonHeapProfileNode>}} */
// @ts-ignore typedef
let CommonHeapProfileNode; // eslint-disable-line no-unused-vars
/** @typedef {!{head:!CommonHeapProfileNode, modules:!Array<!Protocol.Memory.Module>}} */
// @ts-ignore typedef
let CommonHeapProfile; // eslint-disable-line no-unused-vars
/**
 * TODO(chromium:1011811): Change this to implements once we are TypeScript-only.
 * @extends {CommonHeapProfile}
 */
class NativeHeapProfile {
    /**
     * @param {!CommonHeapProfileNode} head
     * @param {!Array<!Protocol.Memory.Module>} modules
     */
    constructor(head, modules) {
        this.head = head;
        this.modules = modules;
    }
}
/**
 * @implements {ProtocolProxyApi.HeapProfilerDispatcher}
 */
class HeapProfilerDispatcher {
    /**
     * @param {!HeapProfilerModel} model
     */
    constructor(model) {
        this._heapProfilerModel = model;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @override
     * @param {!Protocol.HeapProfiler.HeapStatsUpdateEvent} event
     */
    heapStatsUpdate({ statsUpdate }) {
        this._heapProfilerModel.heapStatsUpdate(statsUpdate);
    }
    /**
     * @override
     * @param {!Protocol.HeapProfiler.LastSeenObjectIdEvent} event
     */
    lastSeenObjectId({ lastSeenObjectId, timestamp }) {
        this._heapProfilerModel.lastSeenObjectId(lastSeenObjectId, timestamp);
    }
    /**
     * @override
     * @param {!Protocol.HeapProfiler.AddHeapSnapshotChunkEvent} event
     */
    addHeapSnapshotChunk({ chunk }) {
        this._heapProfilerModel.addHeapSnapshotChunk(chunk);
    }
    /**
     * @override
     * @param {!Protocol.HeapProfiler.ReportHeapSnapshotProgressEvent} event
     */
    reportHeapSnapshotProgress({ done, total, finished }) {
        this._heapProfilerModel.reportHeapSnapshotProgress(done, total, finished);
    }
    /**
     * @override
     */
    resetProfiles() {
        this._heapProfilerModel.resetProfiles();
    }
}
/** @typedef {!{callFrame: !NativeProfilerCallFrame, selfSize: number, childMap: !Map<string, !NodeForConstruction>}} */
// @ts-ignore typedef
let NodeForConstruction; // eslint-disable-line no-unused-vars
SDKModel.register(HeapProfilerModel, Capability.JS, false);

var HeapProfilerModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    HeapProfilerModel: HeapProfilerModel,
    Events: Events$c,
    NativeProfilerCallFrame: NativeProfilerCallFrame,
    CommonHeapProfileNode: CommonHeapProfileNode,
    CommonHeapProfile: CommonHeapProfile
});

/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class RuntimeModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._agent = target.runtimeAgent();
        this.target().registerRuntimeDispatcher(new RuntimeDispatcher(this));
        this._agent.enable();
        /** @type {!Map<number, !ExecutionContext>} */
        this._executionContextById = new Map();
        /** @type {function(!ExecutionContext,!ExecutionContext):number} */
        this._executionContextComparator = ExecutionContext.comparator;
        /** @type {?boolean} */
        this._hasSideEffectSupport = null;
        if (Settings.Settings.instance().moduleSetting('customFormatters').get()) {
            this._agent.setCustomObjectFormatterEnabled(true);
        }
        Settings.Settings.instance()
            .moduleSetting('customFormatters')
            .addChangeListener(this._customFormattersStateChanged.bind(this));
    }
    /**
     * @param {!EvaluationResult} response
     * @return {boolean}
     */
    static isSideEffectFailure(response) {
        const exceptionDetails = !response[InspectorBackend.ProtocolError] && response.exceptionDetails;
        return !!(exceptionDetails && exceptionDetails.exception && exceptionDetails.exception.description &&
            exceptionDetails.exception.description.startsWith('EvalError: Possible side-effect in debug-evaluate'));
    }
    /**
     * @return {!DebuggerModel}
     */
    debuggerModel() {
        return /** @type {!DebuggerModel} */ (this.target().model(DebuggerModel));
    }
    /**
     * @return {!HeapProfilerModel}
     */
    heapProfilerModel() {
        return /** @type {!HeapProfilerModel} */ (this.target().model(HeapProfilerModel));
    }
    /**
     * @return {!Array.<!ExecutionContext>}
     */
    executionContexts() {
        return [...this._executionContextById.values()].sort(this.executionContextComparator());
    }
    /**
     * @param {function(!ExecutionContext,!ExecutionContext):number} comparator
     */
    setExecutionContextComparator(comparator) {
        this._executionContextComparator = comparator;
    }
    /**
     * @return {function(!ExecutionContext,!ExecutionContext):number} comparator
     */
    executionContextComparator() {
        return this._executionContextComparator;
    }
    /**
     * @return {?ExecutionContext}
     */
    defaultExecutionContext() {
        for (const context of this.executionContexts()) {
            if (context.isDefault) {
                return context;
            }
        }
        return null;
    }
    /**
     * @param {!Protocol.Runtime.ExecutionContextId} id
     * @return {?ExecutionContext}
     */
    executionContext(id) {
        return this._executionContextById.get(id) || null;
    }
    /**
     * @param {!Protocol.Runtime.ExecutionContextDescription} context
     */
    _executionContextCreated(context) {
        const data = context.auxData || { isDefault: true };
        const executionContext = new ExecutionContext(this, context.id, context.name, context.origin, data['isDefault'], data['frameId']);
        this._executionContextById.set(executionContext.id, executionContext);
        this.dispatchEventToListeners(Events$d.ExecutionContextCreated, executionContext);
    }
    /**
     * @param {number} executionContextId
     */
    _executionContextDestroyed(executionContextId) {
        const executionContext = this._executionContextById.get(executionContextId);
        if (!executionContext) {
            return;
        }
        this.debuggerModel().executionContextDestroyed(executionContext);
        this._executionContextById.delete(executionContextId);
        this.dispatchEventToListeners(Events$d.ExecutionContextDestroyed, executionContext);
    }
    fireExecutionContextOrderChanged() {
        this.dispatchEventToListeners(Events$d.ExecutionContextOrderChanged, this);
    }
    _executionContextsCleared() {
        this.debuggerModel().globalObjectCleared();
        const contexts = this.executionContexts();
        this._executionContextById.clear();
        for (let i = 0; i < contexts.length; ++i) {
            this.dispatchEventToListeners(Events$d.ExecutionContextDestroyed, contexts[i]);
        }
    }
    /**
     * @param {!Protocol.Runtime.RemoteObject} payload
     * @return {!RemoteObject}
     */
    createRemoteObject(payload) {
        console.assert(typeof payload === 'object', 'Remote object payload should only be an object');
        return new RemoteObjectImpl(this, payload.objectId, payload.type, payload.subtype, payload.value, payload.unserializableValue, payload.description, payload.preview, payload.customPreview, payload.className);
    }
    /**
     * @param {!Protocol.Runtime.RemoteObject} payload
     * @param {!ScopeRef} scopeRef
     * @return {!RemoteObject}
     */
    createScopeRemoteObject(payload, scopeRef) {
        return new ScopeRemoteObject(this, payload.objectId, scopeRef, payload.type, payload.subtype, payload.value, payload.unserializableValue, payload.description, payload.preview);
    }
    /**
     * @param {number|string|boolean|undefined|bigint} value
     * @return {!RemoteObject}
     */
    createRemoteObjectFromPrimitiveValue(value) {
        const type = typeof value;
        let unserializableValue = undefined;
        const unserializableDescription = RemoteObject.unserializableDescription(value);
        if (unserializableDescription !== null) {
            unserializableValue = /** @type {!Protocol.Runtime.UnserializableValue} */ (unserializableDescription);
        }
        if (typeof unserializableValue !== 'undefined') {
            value = undefined;
        }
        return new RemoteObjectImpl(this, undefined, type, undefined, value, unserializableValue);
    }
    /**
     * @param {string} name
     * @param {number|string|boolean} value
     * @return {!RemoteObjectProperty}
     */
    createRemotePropertyFromPrimitiveValue(name, value) {
        return new RemoteObjectProperty(name, this.createRemoteObjectFromPrimitiveValue(value));
    }
    discardConsoleEntries() {
        this._agent.discardConsoleEntries();
    }
    /**
     * @param {string} objectGroupName
     */
    releaseObjectGroup(objectGroupName) {
        this._agent.releaseObjectGroup(objectGroupName);
    }
    /**
     * @param {!EvaluationResult} result
     */
    releaseEvaluationResult(result) {
        if (result.object) {
            result.object.release();
        }
        if (result.exceptionDetails && result.exceptionDetails.exception) {
            const exception = result.exceptionDetails.exception;
            const exceptionObject = this.createRemoteObject({ type: exception.type, objectId: exception.objectId });
            exceptionObject.release();
        }
    }
    runIfWaitingForDebugger() {
        this._agent.runIfWaitingForDebugger();
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _customFormattersStateChanged(event) {
        const enabled = /** @type {boolean} */ (event.data);
        this._agent.setCustomObjectFormatterEnabled(enabled);
    }
    /**
     * @param {string} expression
     * @param {string} sourceURL
     * @param {boolean} persistScript
     * @param {number} executionContextId
     * @return {!Promise<?CompileScriptResult>}
     */
    async compileScript(expression, sourceURL, persistScript, executionContextId) {
        const response = await this._agent.invoke_compileScript({
            expression: expression,
            sourceURL: sourceURL,
            persistScript: persistScript,
            executionContextId: executionContextId,
        });
        if (response[InspectorBackend.ProtocolError]) {
            console.error(response[InspectorBackend.ProtocolError]);
            return null;
        }
        return { scriptId: response.scriptId, exceptionDetails: response.exceptionDetails };
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @param {number} executionContextId
     * @param {string=} objectGroup
     * @param {boolean=} silent
     * @param {boolean=} includeCommandLineAPI
     * @param {boolean=} returnByValue
     * @param {boolean=} generatePreview
     * @param {boolean=} awaitPromise
     * @return {!Promise<!EvaluationResult>}
     */
    async runScript(scriptId, executionContextId, objectGroup, silent, includeCommandLineAPI, returnByValue, generatePreview, awaitPromise) {
        const response = await this._agent.invoke_runScript({
            scriptId,
            executionContextId,
            objectGroup,
            silent,
            includeCommandLineAPI,
            returnByValue,
            generatePreview,
            awaitPromise,
        });
        const error = response[InspectorBackend.ProtocolError];
        if (error) {
            console.error(error);
            return { error: error };
        }
        return { object: this.createRemoteObject(response.result), exceptionDetails: response.exceptionDetails };
    }
    /**
     * @param {!RemoteObject} prototype
     * @return {!Promise<!QueryObjectResult>}
     */
    async queryObjects(prototype) {
        if (!prototype.objectId) {
            return { error: 'Prototype should be an Object.' };
        }
        const response = await this._agent.invoke_queryObjects({ prototypeObjectId: /** @type {string} */ (prototype.objectId), objectGroup: 'console' });
        const error = response[InspectorBackend.ProtocolError];
        if (error) {
            console.error(error);
            return { error: error };
        }
        return { objects: this.createRemoteObject(response.objects) };
    }
    /**
     * @return {!Promise<string>}
     */
    async isolateId() {
        return (await this._agent.getIsolateId()) || this.target().id();
    }
    /**
     * @return {!Promise<?{usedSize: number, totalSize: number}>}
     */
    async heapUsage() {
        const result = await this._agent.invoke_getHeapUsage({});
        return result[InspectorBackend.ProtocolError] ? null : result;
    }
    /**
     * @param {!Protocol.Runtime.RemoteObject} payload
     * @param {!Object=} hints
     */
    _inspectRequested(payload, hints) {
        const object = this.createRemoteObject(payload);
        if (hints.copyToClipboard) {
            this._copyRequested(object);
            return;
        }
        if (hints.queryObjects) {
            this._queryObjectsRequested(object);
            return;
        }
        if (object.isNode()) {
            Revealer.reveal(object).then(object.release.bind(object));
            return;
        }
        if (object.type === 'function') {
            RemoteFunction.objectAsFunction(object).targetFunctionDetails().then(didGetDetails);
            return;
        }
        /**
         * @param {?FunctionDetails} response
         */
        function didGetDetails(response) {
            object.release();
            if (!response || !response.location) {
                return;
            }
            Revealer.reveal(response.location);
        }
        object.release();
    }
    /**
     * @param {!RemoteObject} object
     */
    _copyRequested(object) {
        if (!object.objectId) {
            InspectorFrontendHost.InspectorFrontendHostInstance.copyText(object.unserializableValue() || /** @type {string} */ (object.value));
            return;
        }
        object.callFunctionJSON(toStringForClipboard, [{ value: object.subtype }])
            .then(InspectorFrontendHost.InspectorFrontendHostInstance.copyText.bind(InspectorFrontendHost.InspectorFrontendHostInstance));
        /**
         * @param {string} subtype
         * @this {Object}
         * @suppressReceiverCheck
         */
        function toStringForClipboard(subtype) {
            if (subtype === 'node') {
                return this.outerHTML;
            }
            if (subtype && typeof this === 'undefined') {
                return subtype + '';
            }
            try {
                return JSON.stringify(this, null, '  ');
            }
            catch (e) {
                return '' + this;
            }
        }
    }
    /**
     * @param {!RemoteObject} object
     */
    async _queryObjectsRequested(object) {
        const result = await this.queryObjects(object);
        object.release();
        if (result.error) {
            Console.Console.instance().error(result.error);
            return;
        }
        this.dispatchEventToListeners(Events$d.QueryObjectRequested, { objects: result.objects });
    }
    /**
     * @param {!Protocol.Runtime.ExceptionDetails} exceptionDetails
     * @return {string}
     */
    static simpleTextFromException(exceptionDetails) {
        let text = exceptionDetails.text;
        if (exceptionDetails.exception && exceptionDetails.exception.description) {
            let description = exceptionDetails.exception.description;
            if (description.indexOf('\n') !== -1) {
                description = description.substring(0, description.indexOf('\n'));
            }
            text += ' ' + description;
        }
        return text;
    }
    /**
     * @param {number} timestamp
     * @param {!Protocol.Runtime.ExceptionDetails} exceptionDetails
     */
    exceptionThrown(timestamp, exceptionDetails) {
        const exceptionWithTimestamp = { timestamp: timestamp, details: exceptionDetails };
        this.dispatchEventToListeners(Events$d.ExceptionThrown, exceptionWithTimestamp);
    }
    /**
     * @param {number} exceptionId
     */
    _exceptionRevoked(exceptionId) {
        this.dispatchEventToListeners(Events$d.ExceptionRevoked, exceptionId);
    }
    /**
     * @param {string} type
     * @param {!Array.<!Protocol.Runtime.RemoteObject>} args
     * @param {number} executionContextId
     * @param {number} timestamp
     * @param {!Protocol.Runtime.StackTrace=} stackTrace
     * @param {string=} context
     */
    _consoleAPICalled(type, args, executionContextId, timestamp, stackTrace, context) {
        const consoleAPICall = {
            type: type,
            args: args,
            executionContextId: executionContextId,
            timestamp: timestamp,
            stackTrace: stackTrace,
            context: context,
        };
        this.dispatchEventToListeners(Events$d.ConsoleAPICalled, consoleAPICall);
    }
    /**
     * @param {!Protocol.Runtime.ScriptId} scriptId
     * @return {number}
     */
    executionContextIdForScriptId(scriptId) {
        const script = this.debuggerModel().scriptForId(scriptId);
        return script ? script.executionContextId : 0;
    }
    /**
     * @param {!Protocol.Runtime.StackTrace} stackTrace
     * @return {number}
     */
    executionContextForStackTrace(stackTrace) {
        while (stackTrace && !stackTrace.callFrames.length) {
            stackTrace = stackTrace.parent;
        }
        if (!stackTrace || !stackTrace.callFrames.length) {
            return 0;
        }
        return this.executionContextIdForScriptId(stackTrace.callFrames[0].scriptId);
    }
    /**
     * @return {?boolean}
     */
    hasSideEffectSupport() {
        return this._hasSideEffectSupport;
    }
    /**
     * @return {!Promise<boolean>}
     */
    async checkSideEffectSupport() {
        const testContext = this.executionContexts().peekLast();
        if (!testContext) {
            return false;
        }
        // Check for a positive throwOnSideEffect response without triggering side effects.
        const response = await this._agent.invoke_evaluate({
            expression: _sideEffectTestExpression,
            contextId: testContext.id,
            throwOnSideEffect: true,
        });
        this._hasSideEffectSupport = RuntimeModel.isSideEffectFailure(response);
        return this._hasSideEffectSupport;
    }
    /**
     * @return {!Promise<*>}
     */
    terminateExecution() {
        return this._agent.invoke_terminateExecution({});
    }
}
/**
 * This expression:
 * - IMPORTANT: must not actually cause user-visible or JS-visible side-effects.
 * - Must throw when evaluated with `throwOnSideEffect: true`.
 * - Must be valid when run from any ExecutionContext that supports `throwOnSideEffect`.
 * @const
 * @type {string}
 */
const _sideEffectTestExpression = '(async function(){ await 1; })()';
/** @enum {symbol} */
const Events$d = {
    ExecutionContextCreated: Symbol('ExecutionContextCreated'),
    ExecutionContextDestroyed: Symbol('ExecutionContextDestroyed'),
    ExecutionContextChanged: Symbol('ExecutionContextChanged'),
    ExecutionContextOrderChanged: Symbol('ExecutionContextOrderChanged'),
    ExceptionThrown: Symbol('ExceptionThrown'),
    ExceptionRevoked: Symbol('ExceptionRevoked'),
    ConsoleAPICalled: Symbol('ConsoleAPICalled'),
    QueryObjectRequested: Symbol('QueryObjectRequested'),
};
/**
 * @extends {Protocol.RuntimeDispatcher}
 * @unrestricted
 */
class RuntimeDispatcher {
    /**
     * @param {!RuntimeModel} runtimeModel
     */
    constructor(runtimeModel) {
        this._runtimeModel = runtimeModel;
    }
    /**
     * @override
     * @param {!Protocol.Runtime.ExecutionContextDescription} context
     */
    executionContextCreated(context) {
        this._runtimeModel._executionContextCreated(context);
    }
    /**
     * @override
     * @param {!Protocol.Runtime.ExecutionContextId} executionContextId
     */
    executionContextDestroyed(executionContextId) {
        this._runtimeModel._executionContextDestroyed(executionContextId);
    }
    /**
     * @override
     */
    executionContextsCleared() {
        this._runtimeModel._executionContextsCleared();
    }
    /**
     * @override
     * @param {number} timestamp
     * @param {!Protocol.Runtime.ExceptionDetails} exceptionDetails
     */
    exceptionThrown(timestamp, exceptionDetails) {
        this._runtimeModel.exceptionThrown(timestamp, exceptionDetails);
    }
    /**
     * @override
     * @param {string} reason
     * @param {number} exceptionId
     */
    exceptionRevoked(reason, exceptionId) {
        this._runtimeModel._exceptionRevoked(exceptionId);
    }
    /**
     * @override
     * @param {string} type
     * @param {!Array.<!Protocol.Runtime.RemoteObject>} args
     * @param {number} executionContextId
     * @param {number} timestamp
     * @param {!Protocol.Runtime.StackTrace=} stackTrace
     * @param {string=} context
     */
    consoleAPICalled(type, args, executionContextId, timestamp, stackTrace, context) {
        this._runtimeModel._consoleAPICalled(type, args, executionContextId, timestamp, stackTrace, context);
    }
    /**
     * @override
     * @param {!Protocol.Runtime.RemoteObject} payload
     * @param {!Object=} hints
     */
    inspectRequested(payload, hints) {
        this._runtimeModel._inspectRequested(payload, hints);
    }
}
/**
 * @unrestricted
 */
class ExecutionContext {
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {number} id
     * @param {string} name
     * @param {string} origin
     * @param {boolean} isDefault
     * @param {string=} frameId
     */
    constructor(runtimeModel, id, name, origin, isDefault, frameId) {
        this.id = id;
        this.name = name;
        this.origin = origin;
        this.isDefault = isDefault;
        this.runtimeModel = runtimeModel;
        this.debuggerModel = runtimeModel.debuggerModel();
        this.frameId = frameId;
        this._setLabel('');
    }
    /**
     * @return {!Target}
     */
    target() {
        return this.runtimeModel.target();
    }
    /**
     * @param {!ExecutionContext} a
     * @param {!ExecutionContext} b
     * @return {number}
     */
    static comparator(a, b) {
        /**
         * @param {!Target} target
         * @return {number}
         */
        function targetWeight(target) {
            if (!target.parentTarget()) {
                return 5;
            }
            if (target.type() === Type.Frame) {
                return 4;
            }
            if (target.type() === Type.ServiceWorker) {
                return 3;
            }
            if (target.type() === Type.Worker) {
                return 2;
            }
            return 1;
        }
        /**
         * @param {!Target} target
         * @return {!Array<!Target>}
         */
        function targetPath(target) {
            let currentTarget = target;
            const parents = [];
            while (currentTarget) {
                parents.push(currentTarget);
                currentTarget = currentTarget.parentTarget();
            }
            return parents.reverse();
        }
        const tagetsA = targetPath(a.target());
        const targetsB = targetPath(b.target());
        let targetA;
        let targetB;
        for (let i = 0;; i++) {
            if (!tagetsA[i] || !targetsB[i] || (tagetsA[i] !== targetsB[i])) {
                targetA = tagetsA[i];
                targetB = targetsB[i];
                break;
            }
        }
        if (!targetA && targetB) {
            return -1;
        }
        if (!targetB && targetA) {
            return 1;
        }
        if (targetA && targetB) {
            const weightDiff = targetWeight(targetA) - targetWeight(targetB);
            if (weightDiff) {
                return -weightDiff;
            }
            return targetA.id().localeCompare(targetB.id());
        }
        // Main world context should always go first.
        if (a.isDefault) {
            return -1;
        }
        if (b.isDefault) {
            return +1;
        }
        return a.name.localeCompare(b.name);
    }
    /**
     * @param {!EvaluationOptions} options
     * @param {boolean} userGesture
     * @param {boolean} awaitPromise
     * @return {!Promise<!EvaluationResult>}
     */
    evaluate(options, userGesture, awaitPromise) {
        // FIXME: It will be moved to separate ExecutionContext.
        if (this.debuggerModel.selectedCallFrame()) {
            return this.debuggerModel.evaluateOnSelectedCallFrame(options);
        }
        // Assume backends either support both throwOnSideEffect and timeout options or neither.
        const needsTerminationOptions = !!options.throwOnSideEffect || options.timeout !== undefined;
        if (!needsTerminationOptions || this.runtimeModel.hasSideEffectSupport()) {
            return this._evaluateGlobal(options, userGesture, awaitPromise);
        }
        /** @type {!EvaluationResult} */
        const unsupportedError = { error: 'Side-effect checks not supported by backend.' };
        if (this.runtimeModel.hasSideEffectSupport() === false) {
            return Promise.resolve(unsupportedError);
        }
        return this.runtimeModel.checkSideEffectSupport().then(() => {
            if (this.runtimeModel.hasSideEffectSupport()) {
                return this._evaluateGlobal(options, userGesture, awaitPromise);
            }
            return Promise.resolve(unsupportedError);
        });
    }
    /**
     * @param {string} objectGroup
     * @param {boolean} generatePreview
     * @return {!Promise<!EvaluationResult>}
     */
    globalObject(objectGroup, generatePreview) {
        return this._evaluateGlobal({
            expression: 'this',
            objectGroup: objectGroup,
            includeCommandLineAPI: false,
            silent: true,
            returnByValue: false,
            generatePreview: generatePreview,
        }, 
        /* userGesture */ false, /* awaitPromise */ false);
    }
    /**
     * @param {!EvaluationOptions} options
     * @param {boolean} userGesture
     * @param {boolean} awaitPromise
     * @return {!Promise<!EvaluationResult>}
     */
    async _evaluateGlobal(options, userGesture, awaitPromise) {
        if (!options.expression) {
            // There is no expression, so the completion should happen against global properties.
            options.expression = 'this';
        }
        const response = await this.runtimeModel._agent.invoke_evaluate({
            expression: options.expression,
            objectGroup: options.objectGroup,
            includeCommandLineAPI: options.includeCommandLineAPI,
            silent: options.silent,
            contextId: this.id,
            returnByValue: options.returnByValue,
            generatePreview: options.generatePreview,
            userGesture: userGesture,
            awaitPromise: awaitPromise,
            throwOnSideEffect: options.throwOnSideEffect,
            timeout: options.timeout,
            disableBreaks: options.disableBreaks,
            replMode: options.replMode,
            allowUnsafeEvalBlockedByCSP: options.allowUnsafeEvalBlockedByCSP,
        });
        const error = response[InspectorBackend.ProtocolError];
        if (error) {
            console.error(error);
            return { error: error };
        }
        return { object: this.runtimeModel.createRemoteObject(response.result), exceptionDetails: response.exceptionDetails };
    }
    /**
     * @return {!Promise<?Array<string>>}
     */
    async globalLexicalScopeNames() {
        const response = await this.runtimeModel._agent.invoke_globalLexicalScopeNames({ executionContextId: this.id });
        return response[InspectorBackend.ProtocolError] ? [] : response.names;
    }
    /**
     * @return {string}
     */
    label() {
        return this._label;
    }
    /**
     * @param {string} label
     */
    setLabel(label) {
        this._setLabel(label);
        this.runtimeModel.dispatchEventToListeners(Events$d.ExecutionContextChanged, this);
    }
    /**
     * @param {string} label
     */
    _setLabel(label) {
        if (label) {
            this._label = label;
            return;
        }
        if (this.name) {
            this._label = this.name;
            return;
        }
        const parsedUrl = ParsedURL.ParsedURL.fromString(this.origin);
        this._label = parsedUrl ? parsedUrl.lastPathComponentWithFragment() : '';
    }
}
SDKModel.register(RuntimeModel, Capability.JS, true);
/** @typedef {{
 *    object: !RemoteObject,
 *    exceptionDetails: (!Protocol.Runtime.ExceptionDetails|undefined)}|{
 *    error: (!ProtocolClient.InspectorBackend.ProtocolError|undefined)}
 *  }}
 */
let EvaluationResult;
/** @typedef {{
 *    scriptId: (Protocol.Runtime.ScriptId|undefined),
 *    exceptionDetails: (!Protocol.Runtime.ExceptionDetails|undefined)
 *  }}
 */
let CompileScriptResult;
/** @typedef {{
 *    expression: string,
 *    objectGroup: (string|undefined),
 *    includeCommandLineAPI: (boolean|undefined),
 *    silent: (boolean|undefined),
 *    returnByValue: (boolean|undefined),
 *    generatePreview: (boolean|undefined),
 *    throwOnSideEffect: (boolean|undefined),
 *    timeout: (number|undefined),
 *    disableBreaks: (boolean|undefined),
 *    replMode: (boolean|undefined),
 *    allowUnsafeEvalBlockedByCSP: (boolean|undefined)
 *  }}
 */
let EvaluationOptions;
/** @typedef {{
 *    objects: (!RemoteObject|undefined),
 *    error: (!ProtocolClient.InspectorBackend.ProtocolError|undefined)}
 *  }}
 */
let QueryObjectResult;

var RuntimeModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    RuntimeModel: RuntimeModel,
    Events: Events$d,
    ExecutionContext: ExecutionContext,
    EvaluationResult: EvaluationResult,
    CompileScriptResult: CompileScriptResult,
    EvaluationOptions: EvaluationOptions,
    QueryObjectResult: QueryObjectResult
});

// Copyright (c) 2015 The Chromium Authors. All rights reserved.
/**
 * @implements {ProtocolClient.InspectorBackend.Connection}
 */
class MainConnection {
    constructor() {
        this._onMessage = null;
        this._onDisconnect = null;
        this._messageBuffer = '';
        this._messageSize = 0;
        this._eventListeners = [
            InspectorFrontendHost.InspectorFrontendHostInstance.events.addEventListener(InspectorFrontendHostAPI.Events.DispatchMessage, this._dispatchMessage, this),
            InspectorFrontendHost.InspectorFrontendHostInstance.events.addEventListener(InspectorFrontendHostAPI.Events.DispatchMessageChunk, this._dispatchMessageChunk, this),
        ];
    }
    /**
     * @override
     * @param {function((!Object|string)):void} onMessage
     */
    setOnMessage(onMessage) {
        this._onMessage = onMessage;
    }
    /**
     * @override
     * @param {function(string):void} onDisconnect
     */
    setOnDisconnect(onDisconnect) {
        this._onDisconnect = onDisconnect;
    }
    /**
     * @override
     * @param {string} message
     */
    sendRawMessage(message) {
        if (this._onMessage) {
            InspectorFrontendHost.InspectorFrontendHostInstance.sendMessageToBackend(message);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _dispatchMessage(event) {
        if (this._onMessage) {
            this._onMessage.call(null, /** @type {string} */ (event.data));
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _dispatchMessageChunk(event) {
        const messageChunk = /** @type {string} */ (event.data['messageChunk']);
        const messageSize = /** @type {number} */ (event.data['messageSize']);
        if (messageSize) {
            this._messageBuffer = '';
            this._messageSize = messageSize;
        }
        this._messageBuffer += messageChunk;
        if (this._messageBuffer.length === this._messageSize && this._onMessage) {
            this._onMessage.call(null, this._messageBuffer);
            this._messageBuffer = '';
            this._messageSize = 0;
        }
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    disconnect() {
        const onDisconnect = this._onDisconnect;
        EventTarget.EventTarget.removeEventListeners(this._eventListeners);
        this._onDisconnect = null;
        this._onMessage = null;
        if (onDisconnect) {
            onDisconnect.call(null, 'force disconnect');
        }
        return Promise.resolve();
    }
}
/**
 * @implements {ProtocolClient.InspectorBackend.Connection}
 */
class WebSocketConnection {
    /**
     * @param {string} url
     * @param {function():void} onWebSocketDisconnect
     */
    constructor(url, onWebSocketDisconnect) {
        /** @type {?WebSocket} */
        this._socket = new WebSocket(url);
        this._socket.onerror = this._onError.bind(this);
        this._socket.onopen = this._onOpen.bind(this);
        this._socket.onmessage = messageEvent => {
            if (this._onMessage) {
                this._onMessage.call(null, /** @type {string} */ (messageEvent.data));
            }
        };
        this._socket.onclose = this._onClose.bind(this);
        this._onMessage = null;
        this._onDisconnect = null;
        /** @type {?function():void} */
        this._onWebSocketDisconnect = onWebSocketDisconnect;
        this._connected = false;
        /** @type {!Array<string>} */
        this._messages = [];
    }
    /**
     * @override
     * @param {function((!Object|string)):void} onMessage
     */
    setOnMessage(onMessage) {
        this._onMessage = onMessage;
    }
    /**
     * @override
     * @param {function(string):void} onDisconnect
     */
    setOnDisconnect(onDisconnect) {
        this._onDisconnect = onDisconnect;
    }
    _onError() {
        if (this._onWebSocketDisconnect) {
            this._onWebSocketDisconnect.call(null);
        }
        if (this._onDisconnect) {
            // This is called if error occurred while connecting.
            this._onDisconnect.call(null, 'connection failed');
        }
        this._close();
    }
    _onOpen() {
        this._connected = true;
        if (this._socket) {
            this._socket.onerror = console.error;
            for (const message of this._messages) {
                this._socket.send(message);
            }
        }
        this._messages = [];
    }
    _onClose() {
        if (this._onWebSocketDisconnect) {
            this._onWebSocketDisconnect.call(null);
        }
        if (this._onDisconnect) {
            this._onDisconnect.call(null, 'websocket closed');
        }
        this._close();
    }
    /**
     * @param {function():void=} callback
     */
    _close(callback) {
        if (this._socket) {
            this._socket.onerror = null;
            this._socket.onopen = null;
            this._socket.onclose = callback || null;
            this._socket.onmessage = null;
            this._socket.close();
            this._socket = null;
        }
        this._onWebSocketDisconnect = null;
    }
    /**
     * @override
     * @param {string} message
     */
    sendRawMessage(message) {
        if (this._connected && this._socket) {
            this._socket.send(message);
        }
        else {
            this._messages.push(message);
        }
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    disconnect() {
        return new Promise(fulfill => {
            this._close(() => {
                if (this._onDisconnect) {
                    this._onDisconnect.call(null, 'force disconnect');
                }
                fulfill();
            });
        });
    }
}
/**
 * @implements {ProtocolClient.InspectorBackend.Connection}
 */
class StubConnection {
    constructor() {
        this._onMessage = null;
        this._onDisconnect = null;
    }
    /**
     * @override
     * @param {function((!Object|string)):void} onMessage
     */
    setOnMessage(onMessage) {
        this._onMessage = onMessage;
    }
    /**
     * @override
     * @param {function(string):void} onDisconnect
     */
    setOnDisconnect(onDisconnect) {
        this._onDisconnect = onDisconnect;
    }
    /**
     * @override
     * @param {string} message
     */
    sendRawMessage(message) {
        setTimeout(this._respondWithError.bind(this, message), 0);
    }
    /**
     * @param {string} message
     */
    _respondWithError(message) {
        const messageObject = JSON.parse(message);
        const error = {
            message: 'This is a stub connection, can\'t dispatch message.',
            code: InspectorBackend.DevToolsStubErrorCode,
            data: messageObject
        };
        if (this._onMessage) {
            this._onMessage.call(null, { id: messageObject.id, error: error });
        }
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    disconnect() {
        if (this._onDisconnect) {
            this._onDisconnect.call(null, 'force disconnect');
        }
        this._onDisconnect = null;
        this._onMessage = null;
        return Promise.resolve();
    }
}
/**
 * @implements {ProtocolClient.InspectorBackend.Connection}
 */
class ParallelConnection {
    /**
     * @param {!ProtocolClient.InspectorBackend.Connection} connection
     * @param {string} sessionId
     */
    constructor(connection, sessionId) {
        this._connection = connection;
        this._sessionId = sessionId;
        this._onMessage = null;
        this._onDisconnect = null;
    }
    /**
     * @override
     * @param {function(!Object):void} onMessage
     */
    setOnMessage(onMessage) {
        this._onMessage = onMessage;
    }
    /**
     * @override
     * @param {function(string):void} onDisconnect
     */
    setOnDisconnect(onDisconnect) {
        this._onDisconnect = onDisconnect;
    }
    /**
     * @override
     * @param {string} message
     */
    sendRawMessage(message) {
        const messageObject = JSON.parse(message);
        // If the message isn't for a specific session, it must be for the root session.
        if (!messageObject.sessionId) {
            messageObject.sessionId = this._sessionId;
        }
        this._connection.sendRawMessage(JSON.stringify(messageObject));
    }
    /**
     * @override
     * @return {!Promise<void>}
     */
    disconnect() {
        if (this._onDisconnect) {
            this._onDisconnect.call(null, 'force disconnect');
        }
        this._onDisconnect = null;
        this._onMessage = null;
        return Promise.resolve();
    }
}
/**
 * @param {function():!Promise<undefined>} createMainTarget
 * @param {function():void} websocketConnectionLost
 * @return {!Promise<void>}
 */
async function initMainConnection(createMainTarget, websocketConnectionLost) {
    InspectorBackend.Connection.setFactory(_createMainConnection.bind(null, websocketConnectionLost));
    await createMainTarget();
    InspectorFrontendHost.InspectorFrontendHostInstance.connectionReady();
    InspectorFrontendHost.InspectorFrontendHostInstance.events.addEventListener(InspectorFrontendHostAPI.Events.ReattachMainTarget, () => {
        const target = TargetManager.instance().mainTarget();
        if (target) {
            const router = target.router();
            if (router) {
                router.connection().disconnect();
            }
        }
        createMainTarget();
    });
    return Promise.resolve();
}
/**
 * @param {function():void} websocketConnectionLost
 * @return {!ProtocolClient.InspectorBackend.Connection}
 */
function _createMainConnection(websocketConnectionLost) {
    const wsParam = Runtime.Runtime.queryParam('ws');
    const wssParam = Runtime.Runtime.queryParam('wss');
    if (wsParam || wssParam) {
        const ws = wsParam ? `ws://${wsParam}` : `wss://${wssParam}`;
        return new WebSocketConnection(ws, websocketConnectionLost);
    }
    if (InspectorFrontendHost.InspectorFrontendHostInstance.isHostedMode()) {
        return new StubConnection();
    }
    return new MainConnection();
}

var Connections = /*#__PURE__*/Object.freeze({
    __proto__: null,
    MainConnection: MainConnection,
    WebSocketConnection: WebSocketConnection,
    StubConnection: StubConnection,
    ParallelConnection: ParallelConnection,
    initMainConnection: initMainConnection,
    _createMainConnection: _createMainConnection
});

// Copyright 2018 The Chromium Authors. All rights reserved.
let _lastAnonymousTargetId = 0;
/** @type {(function({target: !Target, waitingForDebugger: boolean}):!Promise<?>)|undefined} */
let _attachCallback;
/**
 * @implements {ProtocolProxyApi.TargetDispatcher}
 */
class ChildTargetManager extends SDKModel {
    /**
     * @param {!Target} parentTarget
     */
    constructor(parentTarget) {
        super(parentTarget);
        this._targetManager = parentTarget.targetManager();
        this._parentTarget = parentTarget;
        this._targetAgent = parentTarget.targetAgent();
        /** @type {!Map<string, !Protocol.Target.TargetInfo>} */
        this._targetInfos = new Map();
        /** @type {!Map<string, !Target>} */
        this._childTargets = new Map();
        /** @type {!Map<string, !ProtocolClient.InspectorBackend.Connection>} */
        this._parallelConnections = new Map();
        /** @type {string | null} */
        this._parentTargetId = null;
        parentTarget.registerTargetDispatcher(this);
        this._targetAgent.invoke_setAutoAttach({ autoAttach: true, waitForDebuggerOnStart: true, flatten: true });
        if (!parentTarget.parentTarget() && !InspectorFrontendHost.isUnderTest()) {
            this._targetAgent.invoke_setDiscoverTargets({ discover: true });
            this._targetAgent.invoke_setRemoteLocations({ locations: [{ host: 'localhost', port: 9229 }] });
        }
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @param {function({target: !Target, waitingForDebugger: boolean}):!Promise<?>=} attachCallback
     */
    static install(attachCallback) {
        _attachCallback = attachCallback;
        SDKModel.register(ChildTargetManager, Capability.Target, true);
    }
    /**
     * @override
     * @return {!Promise<?>}
     */
    suspendModel() {
        return this._targetAgent.invoke_setAutoAttach({ autoAttach: true, waitForDebuggerOnStart: false, flatten: true });
    }
    /**
     * @override
     * @return {!Promise<?>}
     */
    resumeModel() {
        return this._targetAgent.invoke_setAutoAttach({ autoAttach: true, waitForDebuggerOnStart: true, flatten: true });
    }
    /**
     * @override
     */
    dispose() {
        for (const sessionId of this._childTargets.keys()) {
            this.detachedFromTarget({ sessionId, targetId: undefined });
        }
    }
    /**
     * @override
     * @param {!Protocol.Target.TargetCreatedEvent} event
     */
    targetCreated({ targetInfo }) {
        this._targetInfos.set(targetInfo.targetId, targetInfo);
        this._fireAvailableTargetsChanged();
        this.dispatchEventToListeners(Events$e.TargetCreated, targetInfo);
    }
    /**
     * @override
     * @param {!Protocol.Target.TargetInfoChangedEvent} event
     */
    targetInfoChanged({ targetInfo }) {
        this._targetInfos.set(targetInfo.targetId, targetInfo);
        this._fireAvailableTargetsChanged();
        this.dispatchEventToListeners(Events$e.TargetInfoChanged, targetInfo);
    }
    /**
     * @override
     * @param {!Protocol.Target.TargetDestroyedEvent} event
     */
    targetDestroyed({ targetId }) {
        this._targetInfos.delete(targetId);
        this._fireAvailableTargetsChanged();
        this.dispatchEventToListeners(Events$e.TargetDestroyed, targetId);
    }
    /**
     * @override
     * @param {!Protocol.Target.TargetCrashedEvent} event
     */
    targetCrashed({ targetId, status, errorCode }) {
    }
    _fireAvailableTargetsChanged() {
        TargetManager.instance().dispatchEventToListeners(Events.AvailableTargetsChanged, [...this._targetInfos.values()]);
    }
    /**
     * @return {!Promise<string>}
     */
    async _getParentTargetId() {
        if (!this._parentTargetId) {
            this._parentTargetId = (await this._parentTarget.targetAgent().invoke_getTargetInfo({})).targetInfo.targetId;
        }
        return this._parentTargetId;
    }
    /**
     * @override
     * @param {!Protocol.Target.AttachedToTargetEvent} event
     */
    attachedToTarget({ sessionId, targetInfo, waitingForDebugger }) {
        if (this._parentTargetId === targetInfo.targetId) {
            return;
        }
        let targetName = '';
        if (targetInfo.type === 'worker' && targetInfo.title && targetInfo.title !== targetInfo.url) {
            targetName = targetInfo.title;
        }
        else if (targetInfo.type !== 'iframe') {
            const parsedURL = ParsedURL.ParsedURL.fromString(targetInfo.url);
            targetName = parsedURL ? parsedURL.lastPathComponentWithFragment() : '#' + (++_lastAnonymousTargetId);
        }
        let type = Type.Browser;
        if (targetInfo.type === 'iframe') {
            type = Type.Frame;
        }
        // TODO(lfg): ensure proper capabilities for child pages (e.g. portals).
        else if (targetInfo.type === 'page') {
            type = Type.Frame;
        }
        else if (targetInfo.type === 'worker') {
            type = Type.Worker;
        }
        else if (targetInfo.type === 'service_worker') {
            type = Type.ServiceWorker;
        }
        const target = this._targetManager.createTarget(targetInfo.targetId, targetName, type, this._parentTarget, sessionId);
        this._childTargets.set(sessionId, target);
        if (_attachCallback) {
            _attachCallback({ target, waitingForDebugger }).then(() => {
                target.runtimeAgent().invoke_runIfWaitingForDebugger();
            });
        }
        else {
            target.runtimeAgent().invoke_runIfWaitingForDebugger();
        }
    }
    /**
     * @override
     * @param {!Protocol.Target.DetachedFromTargetEvent} event
     */
    detachedFromTarget({ sessionId }) {
        if (this._parallelConnections.has(sessionId)) {
            this._parallelConnections.delete(sessionId);
        }
        else {
            const session = this._childTargets.get(sessionId);
            if (session) {
                session.dispose('target terminated');
                this._childTargets.delete(sessionId);
            }
        }
    }
    /**
     * @override
     * @param {!Protocol.Target.ReceivedMessageFromTargetEvent} event
     */
    receivedMessageFromTarget({}) {
        // We use flatten protocol.
    }
    /**
     * @param {function((!Object|string)):void} onMessage
     * @return {!Promise<!ProtocolClient.InspectorBackend.Connection>}
     */
    async createParallelConnection(onMessage) {
        // The main Target id is actually just `main`, instead of the real targetId.
        // Get the real id (requires an async operation) so that it can be used synchronously later.
        const targetId = await this._getParentTargetId();
        const { connection, sessionId } = await this._createParallelConnectionAndSessionForTarget(this._parentTarget, targetId);
        connection.setOnMessage(onMessage);
        this._parallelConnections.set(sessionId, connection);
        return connection;
    }
    /**
     * @param {!Target} target
     * @param {string} targetId
     * @return {!Promise<!{connection: !ProtocolClient.InspectorBackend.Connection, sessionId: string}>}
     */
    async _createParallelConnectionAndSessionForTarget(target, targetId) {
        const targetAgent = target.targetAgent();
        const targetRouter = /** @type {!ProtocolClient.InspectorBackend.SessionRouter} */ (target.router());
        const sessionId = (await targetAgent.invoke_attachToTarget({ targetId, flatten: true })).sessionId;
        const connection = new ParallelConnection(targetRouter.connection(), sessionId);
        targetRouter.registerSession(target, sessionId, connection);
        connection.setOnDisconnect(() => {
            targetRouter.unregisterSession(sessionId);
            targetAgent.invoke_detachFromTarget({ sessionId });
        });
        return { connection, sessionId };
    }
    /**
     * @return {!Array<!Protocol.Target.TargetInfo>}
     */
    targetInfos() {
        return Array.from(this._targetInfos.values());
    }
}
/** @enum {symbol} */
const Events$e = {
    TargetCreated: Symbol('TargetCreated'),
    TargetDestroyed: Symbol('TargetDestroyed'),
    TargetInfoChanged: Symbol('TargetInforChanged'),
};

var ChildTargetManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ChildTargetManager: ChildTargetManager,
    Events: Events$e
});

/*
 * Copyright (C) 2014 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @implements {ProtocolProxyApi.ProfilerDispatcher}
 */
class CPUProfilerModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._isRecording = false;
        this._nextAnonymousConsoleProfileNumber = 1;
        this._anonymousConsoleProfileIdToTitle = new Map();
        this._profilerAgent = target.profilerAgent();
        /** @type {?function(number, string, !Array<!Protocol.Profiler.ScriptCoverage>):void} */
        this._preciseCoverageDeltaUpdateCallback = null;
        target.registerProfilerDispatcher(this);
        this._profilerAgent.invoke_enable();
        this._debuggerModel = /** @type {!DebuggerModel} */ (target.model(DebuggerModel));
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._debuggerModel.runtimeModel();
    }
    /**
     * @return {!DebuggerModel}
     */
    debuggerModel() {
        return this._debuggerModel;
    }
    /**
     * @override
     * @param {!Protocol.Profiler.ConsoleProfileStartedEvent} event
     */
    consoleProfileStarted({ id, location, title }) {
        if (!title) {
            title = UIString.UIString('Profile %d', this._nextAnonymousConsoleProfileNumber++);
            this._anonymousConsoleProfileIdToTitle.set(id, title);
        }
        this._dispatchProfileEvent(Events$f.ConsoleProfileStarted, id, location, title);
    }
    /**
     * @override
     * @param {!Protocol.Profiler.ConsoleProfileFinishedEvent} event
     */
    consoleProfileFinished({ id, location, profile, title }) {
        if (!title) {
            title = this._anonymousConsoleProfileIdToTitle.get(id);
            this._anonymousConsoleProfileIdToTitle.delete(id);
        }
        // Make sure ProfilesPanel is initialized and CPUProfileType is created.
        Runtime.Runtime.instance().loadModulePromise('profiler').then(() => {
            this._dispatchProfileEvent(Events$f.ConsoleProfileFinished, id, location, title, profile);
        });
    }
    /**
     * @param {symbol} eventName
     * @param {string} id
     * @param {!Protocol.Debugger.Location} scriptLocation
     * @param {string=} title
     * @param {!Protocol.Profiler.Profile=} cpuProfile
     */
    _dispatchProfileEvent(eventName, id, scriptLocation, title, cpuProfile) {
        const debuggerLocation = Location.fromPayload(this._debuggerModel, scriptLocation);
        const globalId = this.target().id() + '.' + id;
        const data = /** @type {!EventData} */ ({ id: globalId, scriptLocation: debuggerLocation, cpuProfile: cpuProfile, title: title, cpuProfilerModel: this });
        this.dispatchEventToListeners(eventName, data);
    }
    /**
     * @return {boolean}
     */
    isRecordingProfile() {
        return this._isRecording;
    }
    /**
     * @return {!Promise<?>}
     */
    startRecording() {
        this._isRecording = true;
        const intervalUs = 100;
        this._profilerAgent.invoke_setSamplingInterval({ interval: intervalUs });
        return this._profilerAgent.invoke_start();
    }
    /**
     * @return {!Promise<?Protocol.Profiler.Profile>}
     */
    stopRecording() {
        this._isRecording = false;
        return this._profilerAgent.invoke_stop().then(response => response.profile || null);
    }
    /**
     * @param {boolean} jsCoveragePerBlock - Collect per Block coverage if `true`, per function coverage otherwise.
     * @param {?function(number, string, !Array<!Protocol.Profiler.ScriptCoverage>):void} preciseCoverageDeltaUpdateCallback - Callback for coverage updates initiated from the back-end
     * @return {!Promise<?>}
     */
    startPreciseCoverage(jsCoveragePerBlock, preciseCoverageDeltaUpdateCallback) {
        const callCount = false;
        this._preciseCoverageDeltaUpdateCallback = preciseCoverageDeltaUpdateCallback;
        const allowUpdatesTriggeredByBackend = true;
        return this._profilerAgent.invoke_startPreciseCoverage({ callCount, detailed: jsCoveragePerBlock, allowTriggeredUpdates: allowUpdatesTriggeredByBackend });
    }
    /**
     * @return {!Promise<{timestamp:number, coverage:!Array<!Protocol.Profiler.ScriptCoverage>}>}
     */
    async takePreciseCoverage() {
        const r = await this._profilerAgent.invoke_takePreciseCoverage();
        const timestamp = (r && r.timestamp) || 0;
        const coverage = (r && r.result) || [];
        return { timestamp, coverage };
    }
    /**
     * @return {!Promise<?>}
     */
    stopPreciseCoverage() {
        this._preciseCoverageDeltaUpdateCallback = null;
        return this._profilerAgent.invoke_stopPreciseCoverage();
    }
    /**
     * @override
     * @param {!Protocol.Profiler.PreciseCoverageDeltaUpdateEvent} event
     */
    preciseCoverageDeltaUpdate({ timestamp, occassion, result }) {
        if (this._preciseCoverageDeltaUpdateCallback) {
            this._preciseCoverageDeltaUpdateCallback(timestamp, occassion, result);
        }
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
}
/** @enum {symbol} */
const Events$f = {
    ConsoleProfileStarted: Symbol('ConsoleProfileStarted'),
    ConsoleProfileFinished: Symbol('ConsoleProfileFinished')
};
SDKModel.register(CPUProfilerModel, Capability.JS, true);
/** @typedef {!{id: string, scriptLocation: !Location, title: string, cpuProfile: (!Protocol.Profiler.Profile|undefined), cpuProfilerModel: !CPUProfilerModel}} */
// @ts-ignore typedef
let EventData;

var CPUProfilerModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CPUProfilerModel: CPUProfilerModel,
    Events: Events$f,
    EventData: EventData
});

// Copyright 2017 The Chromium Authors. All rights reserved.
/**
 * @implements {ProtocolProxyApi.LogDispatcher}
 */
class LogModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        target.registerLogDispatcher(this);
        this._logAgent = target.logAgent();
        this._logAgent.invoke_enable();
        if (!InspectorFrontendHost.isUnderTest()) {
            this._logAgent.invoke_startViolationsReport({
                config: [
                    { name: Protocol.Log.ViolationSettingName.LongTask, threshold: 200 },
                    { name: Protocol.Log.ViolationSettingName.LongLayout, threshold: 30 },
                    { name: Protocol.Log.ViolationSettingName.BlockedEvent, threshold: 100 },
                    { name: Protocol.Log.ViolationSettingName.BlockedParser, threshold: -1 },
                    { name: Protocol.Log.ViolationSettingName.Handler, threshold: 150 },
                    { name: Protocol.Log.ViolationSettingName.RecurringHandler, threshold: 50 },
                    { name: Protocol.Log.ViolationSettingName.DiscouragedAPIUse, threshold: -1 }
                ]
            });
        }
    }
    /**
     * @override
     * @param {!Protocol.Log.EntryAddedEvent} event
     */
    entryAdded({ entry }) {
        this.dispatchEventToListeners(Events$g.EntryAdded, { logModel: this, entry });
    }
    requestClear() {
        this._logAgent.invoke_clear();
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
}
/** @enum {symbol} */
const Events$g = {
    EntryAdded: Symbol('EntryAdded')
};
SDKModel.register(LogModel, Capability.Log, true);

var LogModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    LogModel: LogModel,
    Events: Events$g
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
const _events = Symbol('SDK.ConsoleModel.events');
/**
 * @type {!ConsoleModel}
 */
let settingsInstance;
/**
 * @implements {Observer}
 */
class ConsoleModel extends ObjectWrapper.ObjectWrapper {
    /**
     * @private
     */
    constructor() {
        super();
        /** @type {!Array.<!ConsoleMessage>} */
        this._messages = [];
        /** @type {!Map<!RuntimeModel, !Map<number, !ConsoleMessage>>} */
        this._messageByExceptionId = new Map();
        this._warnings = 0;
        this._errors = 0;
        this._violations = 0;
        this._pageLoadSequenceNumber = 0;
        TargetManager.instance().observeTargets(this);
    }
    /**
     * @param {{forceNew: ?boolean}} opts
     */
    static instance(opts = { forceNew: null }) {
        const { forceNew } = opts;
        if (!settingsInstance || forceNew) {
            settingsInstance = new ConsoleModel();
        }
        return settingsInstance;
    }
    /**
     * @override
     * @param {!Target} target
     */
    targetAdded(target) {
        const resourceTreeModel = target.model(ResourceTreeModel);
        if (!resourceTreeModel || resourceTreeModel.cachedResourcesLoaded()) {
            this._initTarget(target);
            return;
        }
        const eventListener = resourceTreeModel.addEventListener(Events$a.CachedResourcesLoaded, () => {
            EventTarget.EventTarget.removeEventListeners([eventListener]);
            this._initTarget(target);
        });
    }
    /**
     * @param {!Target} target
     */
    _initTarget(target) {
        const eventListeners = [];
        const cpuProfilerModel = target.model(CPUProfilerModel);
        if (cpuProfilerModel) {
            eventListeners.push(cpuProfilerModel.addEventListener(Events$f.ConsoleProfileStarted, this._consoleProfileStarted.bind(this, cpuProfilerModel)));
            eventListeners.push(cpuProfilerModel.addEventListener(Events$f.ConsoleProfileFinished, this._consoleProfileFinished.bind(this, cpuProfilerModel)));
        }
        const resourceTreeModel = target.model(ResourceTreeModel);
        if (resourceTreeModel && !target.parentTarget()) {
            eventListeners.push(resourceTreeModel.addEventListener(Events$a.MainFrameNavigated, this._mainFrameNavigated, this));
        }
        const runtimeModel = target.model(RuntimeModel);
        if (runtimeModel) {
            eventListeners.push(runtimeModel.addEventListener(Events$d.ExceptionThrown, this._exceptionThrown.bind(this, runtimeModel)));
            eventListeners.push(runtimeModel.addEventListener(Events$d.ExceptionRevoked, this._exceptionRevoked.bind(this, runtimeModel)));
            eventListeners.push(runtimeModel.addEventListener(Events$d.ConsoleAPICalled, this._consoleAPICalled.bind(this, runtimeModel)));
            if (!target.parentTarget()) {
                eventListeners.push(runtimeModel.debuggerModel().addEventListener(Events$b.GlobalObjectCleared, this._clearIfNecessary, this));
            }
            eventListeners.push(runtimeModel.addEventListener(Events$d.QueryObjectRequested, this._queryObjectRequested.bind(this, runtimeModel)));
        }
        target[_events] = eventListeners;
    }
    /**
     * @override
     * @param {!Target} target
     */
    targetRemoved(target) {
        const runtimeModel = target.model(RuntimeModel);
        if (runtimeModel) {
            this._messageByExceptionId.delete(runtimeModel);
        }
        EventTarget.EventTarget.removeEventListeners(target[_events] || []);
    }
    /**
     * @param {!ExecutionContext} executionContext
     * @param {!ConsoleMessage} originatingMessage
     * @param {string} expression
     * @param {boolean} useCommandLineAPI
     */
    async evaluateCommandInConsole(executionContext, originatingMessage, expression, useCommandLineAPI) {
        const result = await executionContext.evaluate({
            expression: expression,
            objectGroup: 'console',
            includeCommandLineAPI: useCommandLineAPI,
            silent: false,
            returnByValue: false,
            generatePreview: true,
            replMode: true,
            allowUnsafeEvalBlockedByCSP: false
        }, Settings.Settings.instance().moduleSetting('consoleUserActivationEval').get(), /* awaitPromise */ false);
        userMetrics.actionTaken(Host.UserMetrics.Action.ConsoleEvaluated);
        if (result.error) {
            return;
        }
        await Console.Console.instance().showPromise();
        this.dispatchEventToListeners(Events$h.CommandEvaluated, { result: result.object, commandMessage: originatingMessage, exceptionDetails: result.exceptionDetails });
    }
    /**
     * @param {!ExecutionContext} executionContext
     * @param {string} text
     * @return {!ConsoleMessage}
     */
    addCommandMessage(executionContext, text) {
        const commandMessage = new ConsoleMessage(executionContext.runtimeModel, MessageSource.JS, null, text, MessageType.Command);
        commandMessage.setExecutionContextId(executionContext.id);
        this.addMessage(commandMessage);
        return commandMessage;
    }
    /**
     * @param {!ConsoleMessage} msg
     */
    addMessage(msg) {
        msg._pageLoadSequenceNumber = this._pageLoadSequenceNumber;
        if (msg.source === MessageSource.ConsoleAPI && msg.type === MessageType.Clear) {
            this._clearIfNecessary();
        }
        this._messages.push(msg);
        const runtimeModel = msg.runtimeModel();
        if (msg._exceptionId && runtimeModel) {
            let modelMap = this._messageByExceptionId.get(runtimeModel);
            if (!modelMap) {
                modelMap = new Map();
                this._messageByExceptionId.set(runtimeModel, modelMap);
            }
            modelMap.set(msg._exceptionId, msg);
        }
        this._incrementErrorWarningCount(msg);
        this.dispatchEventToListeners(Events$h.MessageAdded, msg);
    }
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _exceptionThrown(runtimeModel, event) {
        const exceptionWithTimestamp = /** @type {!ExceptionWithTimestamp} */ (event.data);
        const consoleMessage = ConsoleMessage.fromException(runtimeModel, exceptionWithTimestamp.details, undefined, exceptionWithTimestamp.timestamp, undefined);
        consoleMessage.setExceptionId(exceptionWithTimestamp.details.exceptionId);
        this.addMessage(consoleMessage);
    }
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _exceptionRevoked(runtimeModel, event) {
        const exceptionId = /** @type {number} */ (event.data);
        const modelMap = this._messageByExceptionId.get(runtimeModel);
        const exceptionMessage = modelMap ? modelMap.get(exceptionId) : null;
        if (!exceptionMessage) {
            return;
        }
        this._errors--;
        exceptionMessage.level = MessageLevel.Verbose;
        this.dispatchEventToListeners(Events$h.MessageUpdated, exceptionMessage);
    }
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _consoleAPICalled(runtimeModel, event) {
        const call = /** @type {!ConsoleAPICall} */ (event.data);
        let level = MessageLevel.Info;
        if (call.type === MessageType.Debug) {
            level = MessageLevel.Verbose;
        }
        else if (call.type === MessageType.Error || call.type === MessageType.Assert) {
            level = MessageLevel.Error;
        }
        else if (call.type === MessageType.Warning) {
            level = MessageLevel.Warning;
        }
        else if (call.type === MessageType.Info || call.type === MessageType.Log) {
            level = MessageLevel.Info;
        }
        let message = '';
        if (call.args.length && call.args[0].unserializableValue) {
            message = call.args[0].unserializableValue;
        }
        else if (call.args.length && (typeof call.args[0].value !== 'object' || call.args[0].value === null)) {
            message = call.args[0].value + '';
        }
        else if (call.args.length && call.args[0].description) {
            message = call.args[0].description;
        }
        const callFrame = call.stackTrace && call.stackTrace.callFrames.length ? call.stackTrace.callFrames[0] : null;
        const consoleMessage = new ConsoleMessage(runtimeModel, MessageSource.ConsoleAPI, level, 
        /** @type {string} */ (message), call.type, callFrame ? callFrame.url : undefined, callFrame ? callFrame.lineNumber : undefined, callFrame ? callFrame.columnNumber : undefined, call.args, call.stackTrace, call.timestamp, call.executionContextId, undefined, undefined, call.context);
        this.addMessage(consoleMessage);
    }
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _queryObjectRequested(runtimeModel, event) {
        const consoleMessage = new ConsoleMessage(runtimeModel, MessageSource.ConsoleAPI, MessageLevel.Info, '', MessageType.QueryObjectResult, undefined, undefined, undefined, [event.data.objects]);
        this.addMessage(consoleMessage);
    }
    _clearIfNecessary() {
        if (!Settings.Settings.instance().moduleSetting('preserveConsoleLog').get()) {
            this._clear();
        }
        ++this._pageLoadSequenceNumber;
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _mainFrameNavigated(event) {
        if (Settings.Settings.instance().moduleSetting('preserveConsoleLog').get()) {
            Console.Console.instance().log(UIString.UIString('Navigated to %s', event.data.url));
        }
    }
    /**
     * @param {!CPUProfilerModel} cpuProfilerModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _consoleProfileStarted(cpuProfilerModel, event) {
        const data = /** @type {!EventData} */ (event.data);
        this._addConsoleProfileMessage(cpuProfilerModel, MessageType.Profile, data.scriptLocation, UIString.UIString('Profile \'%s\' started.', data.title));
    }
    /**
     * @param {!CPUProfilerModel} cpuProfilerModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _consoleProfileFinished(cpuProfilerModel, event) {
        const data = /** @type {!EventData} */ (event.data);
        this._addConsoleProfileMessage(cpuProfilerModel, MessageType.ProfileEnd, data.scriptLocation, UIString.UIString('Profile \'%s\' finished.', data.title));
    }
    /**
     * @param {!CPUProfilerModel} cpuProfilerModel
     * @param {string} type
     * @param {!Location} scriptLocation
     * @param {string} messageText
     */
    _addConsoleProfileMessage(cpuProfilerModel, type, scriptLocation, messageText) {
        const stackTrace = [{
                functionName: '',
                scriptId: scriptLocation.scriptId,
                url: scriptLocation.script() ? scriptLocation.script().contentURL() : '',
                lineNumber: scriptLocation.lineNumber,
                columnNumber: scriptLocation.columnNumber || 0
            }];
        this.addMessage(new ConsoleMessage(cpuProfilerModel.runtimeModel(), MessageSource.ConsoleAPI, MessageLevel.Info, messageText, type, undefined, undefined, undefined, stackTrace));
    }
    /**
     * @param {!ConsoleMessage} msg
     */
    _incrementErrorWarningCount(msg) {
        if (msg.source === MessageSource.Violation) {
            this._violations++;
            return;
        }
        switch (msg.level) {
            case MessageLevel.Warning:
                this._warnings++;
                break;
            case MessageLevel.Error:
                this._errors++;
                break;
        }
    }
    /**
     * @return {!Array.<!ConsoleMessage>}
     */
    messages() {
        return this._messages;
    }
    requestClearMessages() {
        for (const logModel of TargetManager.instance().models(LogModel)) {
            logModel.requestClear();
        }
        for (const runtimeModel of TargetManager.instance().models(RuntimeModel)) {
            runtimeModel.discardConsoleEntries();
        }
        this._clear();
    }
    _clear() {
        this._messages = [];
        this._messageByExceptionId.clear();
        this._errors = 0;
        this._warnings = 0;
        this._violations = 0;
        this.dispatchEventToListeners(Events$h.ConsoleCleared);
    }
    /**
     * @return {number}
     */
    errors() {
        return this._errors;
    }
    /**
     * @return {number}
     */
    warnings() {
        return this._warnings;
    }
    /**
     * @return {number}
     */
    violations() {
        return this._violations;
    }
    /**
     * @param {?ExecutionContext} currentExecutionContext
     * @param {?RemoteObject} remoteObject
     */
    async saveToTempVariable(currentExecutionContext, remoteObject) {
        if (!remoteObject || !currentExecutionContext) {
            failedToSave(null);
            return;
        }
        const executionContext = /** @type {!ExecutionContext} */ (currentExecutionContext);
        const result = await executionContext.globalObject(/* objectGroup */ '', /* generatePreview */ false);
        if (!!result.exceptionDetails || !result.object) {
            failedToSave(result.object || null);
            return;
        }
        const globalObject = result.object;
        const callFunctionResult = await globalObject.callFunction(saveVariable, [RemoteObject.toCallArgument(remoteObject)]);
        globalObject.release();
        if (callFunctionResult.wasThrown || !callFunctionResult.object || callFunctionResult.object.type !== 'string') {
            failedToSave(callFunctionResult.object || null);
        }
        else {
            const text = /** @type {string} */ (callFunctionResult.object.value);
            const message = this.addCommandMessage(executionContext, text);
            this.evaluateCommandInConsole(executionContext, message, text, /* useCommandLineAPI */ false);
        }
        if (callFunctionResult.object) {
            callFunctionResult.object.release();
        }
        /**
         * @suppressReceiverCheck
         * @this {Window}
         */
        function saveVariable(value) {
            const prefix = 'temp';
            let index = 1;
            while ((prefix + index) in this) {
                ++index;
            }
            const name = prefix + index;
            this[name] = value;
            return name;
        }
        /**
         * @param {?RemoteObject} result
         */
        function failedToSave(result) {
            let message = UIString.UIString('Failed to save to temp variable.');
            if (result) {
                message += ' ' + result.description;
            }
            Console.Console.instance().error(message);
        }
    }
}
/** @enum {symbol} */
const Events$h = {
    ConsoleCleared: Symbol('ConsoleCleared'),
    MessageAdded: Symbol('MessageAdded'),
    MessageUpdated: Symbol('MessageUpdated'),
    CommandEvaluated: Symbol('CommandEvaluated')
};
/**
 * @unrestricted
 */
class ConsoleMessage {
    /**
     * @param {?RuntimeModel} runtimeModel
     * @param {string} source
     * @param {?string} level
     * @param {string} messageText
     * @param {string=} type
     * @param {?string=} url
     * @param {number=} line
     * @param {number=} column
     * @param {!Array.<!Protocol.Runtime.RemoteObject|string>=} parameters
     * @param {!Protocol.Runtime.StackTrace=} stackTrace
     * @param {number=} timestamp
     * @param {!Protocol.Runtime.ExecutionContextId=} executionContextId
     * @param {?string=} scriptId
     * @param {?string=} workerId
     * @param {string=} context
     */
    constructor(runtimeModel, source, level, messageText, type, url, line, column, parameters, stackTrace, timestamp, executionContextId, scriptId, workerId, context) {
        this._runtimeModel = runtimeModel;
        this.source = source;
        this.level = /** @type {?MessageLevel} */ (level);
        this.messageText = messageText;
        this.type = type || MessageType.Log;
        /** @type {string|undefined} */
        this.url = url || undefined;
        /** @type {number} */
        this.line = line || 0;
        /** @type {number} */
        this.column = column || 0;
        this.parameters = parameters;
        /** @type {!Protocol.Runtime.StackTrace|undefined} */
        this.stackTrace = stackTrace;
        this.timestamp = timestamp || Date.now();
        this.executionContextId = executionContextId || 0;
        this.scriptId = scriptId || null;
        this.workerId = workerId || null;
        this.frameId = null;
        if (!this.executionContextId && this._runtimeModel) {
            if (this.scriptId) {
                this.executionContextId = this._runtimeModel.executionContextIdForScriptId(this.scriptId);
            }
            else if (this.stackTrace) {
                this.executionContextId = this._runtimeModel.executionContextForStackTrace(this.stackTrace);
            }
        }
        if (this.executionContextId && this._runtimeModel) {
            const executionContext = this._runtimeModel.executionContext(this.executionContextId);
            this.frameId = executionContext ? executionContext.frameId : null;
        }
        if (context) {
            this.context = context.match(/[^#]*/)[0];
        }
    }
    /**
     * @param {!RuntimeModel} runtimeModel
     * @param {!Protocol.Runtime.ExceptionDetails} exceptionDetails
     * @param {string=} messageType
     * @param {number=} timestamp
     * @param {string=} forceUrl
     * @return {!ConsoleMessage}
     */
    static fromException(runtimeModel, exceptionDetails, messageType, timestamp, forceUrl) {
        return new ConsoleMessage(runtimeModel, MessageSource.JS, MessageLevel.Error, RuntimeModel.simpleTextFromException(exceptionDetails), messageType, forceUrl || exceptionDetails.url, exceptionDetails.lineNumber, exceptionDetails.columnNumber, exceptionDetails.exception ? [RemoteObject.fromLocalObject(exceptionDetails.text), exceptionDetails.exception] :
            undefined, exceptionDetails.stackTrace, timestamp, exceptionDetails.executionContextId, exceptionDetails.scriptId);
    }
    /**
     * @return {?RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    /**
     * @return {?Target}
     */
    target() {
        return this._runtimeModel ? this._runtimeModel.target() : null;
    }
    /**
     * @param {!ConsoleMessage} originatingMessage
     */
    setOriginatingMessage(originatingMessage) {
        this._originatingConsoleMessage = originatingMessage;
        this.executionContextId = originatingMessage.executionContextId;
    }
    /**
     * @param {!Protocol.Runtime.ExecutionContextId} executionContextId
     */
    setExecutionContextId(executionContextId) {
        this.executionContextId = executionContextId;
    }
    /**
     * @param {number} exceptionId
     */
    setExceptionId(exceptionId) {
        this._exceptionId = exceptionId;
    }
    /**
     * @return {?ConsoleMessage}
     */
    originatingMessage() {
        return this._originatingConsoleMessage;
    }
    /**
     * @return {boolean}
     */
    isGroupMessage() {
        return this.type === MessageType.StartGroup || this.type === MessageType.StartGroupCollapsed ||
            this.type === MessageType.EndGroup;
    }
    /**
     * @return {boolean}
     */
    isGroupStartMessage() {
        return this.type === MessageType.StartGroup || this.type === MessageType.StartGroupCollapsed;
    }
    /**
     * @return {boolean}
     */
    isErrorOrWarning() {
        return (this.level === MessageLevel.Warning || this.level === MessageLevel.Error);
    }
    /**
     * @return {boolean}
     */
    isGroupable() {
        const isUngroupableError = this.level === MessageLevel.Error &&
            (this.source === MessageSource.JS || this.source === MessageSource.Network);
        return (this.source !== MessageSource.ConsoleAPI && this.type !== MessageType.Command &&
            this.type !== MessageType.Result && this.type !== MessageType.System && !isUngroupableError);
    }
    /**
     * @return {string}
     */
    groupCategoryKey() {
        return [this.source, this.level, this.type, this._pageLoadSequenceNumber].join(':');
    }
    /**
     * @param {?ConsoleMessage} msg
     * @return {boolean}
     */
    isEqual(msg) {
        if (!msg) {
            return false;
        }
        if (!this._isEqualStackTraces(this.stackTrace, msg.stackTrace)) {
            return false;
        }
        if (this.parameters) {
            if (!msg.parameters || this.parameters.length !== msg.parameters.length) {
                return false;
            }
            for (let i = 0; i < msg.parameters.length; ++i) {
                // Never treat objects as equal - their properties might change over time. Errors can be treated as equal
                // since they are always formatted as strings.
                if (msg.parameters[i].type === 'object' && msg.parameters[i].subtype !== 'error') {
                    return false;
                }
                if (this.parameters[i].type !== msg.parameters[i].type ||
                    this.parameters[i].value !== msg.parameters[i].value ||
                    this.parameters[i].description !== msg.parameters[i].description) {
                    return false;
                }
            }
        }
        return (this.runtimeModel() === msg.runtimeModel()) && (this.source === msg.source) && (this.type === msg.type) &&
            (this.level === msg.level) && (this.line === msg.line) && (this.url === msg.url) &&
            (this.messageText === msg.messageText) && (this.executionContextId === msg.executionContextId);
    }
    /**
     * @param {!Protocol.Runtime.StackTrace|undefined} stackTrace1
     * @param {!Protocol.Runtime.StackTrace|undefined} stackTrace2
     * @return {boolean}
     */
    _isEqualStackTraces(stackTrace1, stackTrace2) {
        if (!stackTrace1 !== !stackTrace2) {
            return false;
        }
        if (!stackTrace1) {
            return true;
        }
        const callFrames1 = stackTrace1.callFrames;
        const callFrames2 = stackTrace2.callFrames;
        if (callFrames1.length !== callFrames2.length) {
            return false;
        }
        for (let i = 0, n = callFrames1.length; i < n; ++i) {
            if (callFrames1[i].url !== callFrames2[i].url || callFrames1[i].functionName !== callFrames2[i].functionName ||
                callFrames1[i].lineNumber !== callFrames2[i].lineNumber ||
                callFrames1[i].columnNumber !== callFrames2[i].columnNumber) {
                return false;
            }
        }
        return this._isEqualStackTraces(stackTrace1.parent, stackTrace2.parent);
    }
}
// Note: Keep these constants in sync with the ones in ConsoleTypes.h
/**
 * @enum {string}
 */
const MessageSource = {
    XML: 'xml',
    JS: 'javascript',
    Network: 'network',
    ConsoleAPI: 'console-api',
    Storage: 'storage',
    AppCache: 'appcache',
    Rendering: 'rendering',
    CSS: 'css',
    Security: 'security',
    Deprecation: 'deprecation',
    Worker: 'worker',
    Violation: 'violation',
    Intervention: 'intervention',
    Recommendation: 'recommendation',
    Other: 'other'
};
/**
 * @enum {string}
 */
const MessageType = {
    Log: 'log',
    Debug: 'debug',
    Info: 'info',
    Error: 'error',
    Warning: 'warning',
    Dir: 'dir',
    DirXML: 'dirxml',
    Table: 'table',
    Trace: 'trace',
    Clear: 'clear',
    StartGroup: 'startGroup',
    StartGroupCollapsed: 'startGroupCollapsed',
    EndGroup: 'endGroup',
    Assert: 'assert',
    Result: 'result',
    Profile: 'profile',
    ProfileEnd: 'profileEnd',
    Command: 'command',
    System: 'system',
    QueryObjectResult: 'queryObjectResult'
};
/**
 * @enum {string}
 */
const MessageLevel = {
    Verbose: 'verbose',
    Info: 'info',
    Warning: 'warning',
    Error: 'error'
};
/** @type {!Map<!MessageSource, string>} */
const MessageSourceDisplayName = new Map([
    [MessageSource.XML, 'xml'], [MessageSource.JS, 'javascript'], [MessageSource.Network, 'network'],
    [MessageSource.ConsoleAPI, 'console-api'], [MessageSource.Storage, 'storage'], [MessageSource.AppCache, 'appcache'],
    [MessageSource.Rendering, 'rendering'], [MessageSource.CSS, 'css'], [MessageSource.Security, 'security'],
    [MessageSource.Deprecation, 'deprecation'], [MessageSource.Worker, 'worker'], [MessageSource.Violation, 'violation'],
    [MessageSource.Intervention, 'intervention'], [MessageSource.Recommendation, 'recommendation'],
    [MessageSource.Other, 'other']
]);
/**
 * @typedef {{
  *    type: string,
  *    args: !Array<!Protocol.Runtime.RemoteObject>,
  *    executionContextId: number,
  *    timestamp: number,
  *    stackTrace: (!Protocol.Runtime.StackTrace|undefined)
  * }}
  */
let ConsoleAPICall;
/** @typedef {{timestamp: number, details: !Protocol.Runtime.ExceptionDetails}} */
let ExceptionWithTimestamp;

var ConsoleModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ConsoleModel: ConsoleModel,
    Events: Events$h,
    ConsoleMessage: ConsoleMessage,
    MessageSource: MessageSource,
    MessageType: MessageType,
    MessageLevel: MessageLevel,
    MessageSourceDisplayName: MessageSourceDisplayName,
    ConsoleAPICall: ConsoleAPICall,
    ExceptionWithTimestamp: ExceptionWithTimestamp
});

// Copyright 2020 The Chromium Authors. All rights reserved.
/** @enum {symbol} */
const IssueCategory = {
    CrossOriginEmbedderPolicy: Symbol('CrossOriginEmbedderPolicy'),
    MixedContent: Symbol('MixedContent'),
    SameSiteCookie: Symbol('SameSiteCookie'),
    HeavyAd: Symbol('HeavyAd'),
    ContentSecurityPolicy: Symbol('ContentSecurityPolicy'),
    Other: Symbol('Other')
};
/** @enum {symbol} */
const IssueKind = {
    BreakingChange: Symbol('BreakingChange'),
};
/** @return {!Common.Settings.Setting<boolean>} */
function getShowThirdPartyIssuesSetting() {
    return Settings.Settings.instance().createSetting('showThirdPartyIssues', false);
}
/**
 * @typedef {{
  *            title:string,
  *            message: (function():!Element),
  *            issueKind: !IssueKind,
  *            links: !Array<!{link: string, linkTitle: string}>
  *          }}
  */
// @ts-ignore typedef
let IssueDescription; // eslint-disable-line no-unused-vars
/**
 * @typedef {{
  *             file: string,
  *             issueKind: !IssueKind,
  *             links: !Array<!{link: string, linkTitle: string}>
  *          }}
  */
// @ts-ignore typedef
let MarkdownIssueDescription; // eslint-disable-line no-unused-vars
/**
 * @typedef {{
  *            backendNodeId: number,
  *            nodeName: string
  *          }}
  */
// @ts-ignore typedef
let AffectedElement; // eslint-disable-line no-unused-vars
/**
 * @typedef {{
 *            columnNumber: (number|undefined),
 *            lineNumber: number,
 *            url: string
 *          }}
 */
// @ts-ignore typedef
let AffectedSource; // eslint-disable-line no-unused-vars
/**
 * @abstract
 */
class Issue extends ObjectWrapper.ObjectWrapper {
    /**
     * @param {string} code
     */
    constructor(code) {
        super();
        /** @type {string} */
        this._code = code;
    }
    /**
     * @return {string}
     */
    code() {
        return this._code;
    }
    /**
     * @return {string}
     */
    primaryKey() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Iterable<!Protocol.Audits.BlockedByResponseIssueDetails>}
     */
    blockedByResponseDetails() {
        return [];
    }
    /**
     * @return {!Iterable<!Protocol.Audits.AffectedCookie>}
     */
    cookies() {
        return [];
    }
    /**
     * @return {!Iterable<!AffectedElement>}
     */
    elements() {
        return [];
    }
    /**
     * @returns {!Iterable<!Protocol.Audits.HeavyAdIssueDetails>}
     */
    heavyAds() {
        return [];
    }
    /**
     * @returns {!Iterable<!Protocol.Audits.MixedContentIssueDetails>}
     */
    mixedContents() {
        return [];
    }
    /**
     * @return {!Iterable<!Protocol.Audits.AffectedRequest>}
     */
    requests() {
        return [];
    }
    /**
     * @returns {!Iterable<!AffectedSource>}
     */
    sources() {
        return [];
    }
    /**
     * @param {string} requestId
     * @return {boolean}
     */
    isAssociatedWithRequestId(requestId) {
        for (const request of this.requests()) {
            if (request.requestId === requestId) {
                return true;
            }
        }
        return false;
    }
    /**
     * @return {?(!IssueDescription|!MarkdownIssueDescription)}
     */
    getDescription() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!IssueCategory}
     */
    getCategory() {
        throw new Error('Not implemented');
    }
    /**
     * @return {boolean}
     */
    isCausedByThirdParty() {
        return false;
    }
}

var Issue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IssueCategory: IssueCategory,
    IssueKind: IssueKind,
    getShowThirdPartyIssuesSetting: getShowThirdPartyIssuesSetting,
    IssueDescription: IssueDescription,
    MarkdownIssueDescription: MarkdownIssueDescription,
    AffectedElement: AffectedElement,
    AffectedSource: AffectedSource,
    Issue: Issue
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class ContentSecurityPolicyIssue extends Issue {
    /**
     * @param {!Protocol.Audits.ContentSecurityPolicyIssueDetails} issueDetails
     * @param {!IssuesModel} issuesModel
     */
    constructor(issueDetails, issuesModel) {
        const issue_code = [
            Protocol.Audits.InspectorIssueCode.ContentSecurityPolicyIssue, issueDetails.contentSecurityPolicyViolationType
        ].join('::');
        super(issue_code);
        this._issueDetails = issueDetails;
        this._issuesModel = issuesModel;
    }
    /**
     * @override
     * @return {!IssueCategory}
     */
    getCategory() {
        return IssueCategory.ContentSecurityPolicy;
    }
    /**
     * @override
     * @return {string}
     */
    primaryKey() {
        return JSON.stringify(this._issueDetails, [
            'blockedURL', 'contentSecurityPolicyViolationType', 'violatedDirective', 'sourceCodeLocation', 'url',
            'lineNumber', 'columnNumber', 'violatingNodeId'
        ]);
    }
    /**
     * @override
     * @returns {?IssueDescription}
     */
    getDescription() {
        const description = issueDescriptions.get(this._issueDetails.contentSecurityPolicyViolationType);
        if (description) {
            return description;
        }
        return null;
    }
    /**
     * @returns {!IssuesModel}
     */
    model() {
        return this._issuesModel;
    }
    /**
     * @returns {!Protocol.Audits.ContentSecurityPolicyIssueDetails}
     */
    details() {
        return this._issueDetails;
    }
}
/**
 * @param {!Array<string>} paragraphs
 * @param {!Array<string>} fixesList
 * @return {!Element}
 */
function paragraphedMessage(paragraphs, fixesList) {
    /**
     * Inserts <code> tags for substrings of `message` that are enclosed
     * by |, i.e. "Hello |code|" causes code get enclosed in a <code> tag.
     * This is not an injection risk, as it only use `textContent` and only
     * programmatically creates <span> and <code> elements.
     * @param {!Element} element
     * @param {string} message
     */
    const appendStyled = (element, message) => {
        let lastIndex = 0;
        // Closure doesn't know String.p.matchAll exists.
        /** @suppress {missingProperties} */
        const matches = message.matchAll(/\|(.*?)\|/g);
        for (const match of matches) { //
            if (match.index !== undefined) {
                const span = document.createElement('span');
                span.textContent = message.substring(lastIndex, match.index);
                element.appendChild(span);
                const code = document.createElement('code');
                code.textContent = match[1];
                lastIndex = match.index + match[0].length;
                element.appendChild(code);
            }
        }
        if (lastIndex < message.length) {
            const span = document.createElement('span');
            span.textContent = message.substring(lastIndex, message.length);
            element.appendChild(span);
        }
    };
    const message = document.createElement('div');
    message.classList.add('message');
    for (const paragraph of paragraphs) {
        const paragraphElement = document.createElement('p');
        appendStyled(paragraphElement, paragraph);
        message.appendChild(paragraphElement);
    }
    if (fixesList.length > 0) {
        const inlineFixesList = document.createElement('ul');
        inlineFixesList.classList.add('resolutions-list');
        message.append(inlineFixesList);
        for (const fixSuggestion of fixesList) {
            const listItem = document.createElement('li');
            appendStyled(listItem, fixSuggestion);
            inlineFixesList.append(listItem);
        }
    }
    return message;
}
const cspURLViolation = {
    title: ls$2 `Content Security Policy of your site blocks some resources because their origin is not included in the content security policy header`,
    message: () => paragraphedMessage([
        ls$2 `The Content Security Policy (CSP) improves the security of your site by defining a list of trusted sources and
     instructs the browser to only execute or render resources from this list. Some resources on your site can't be accessed
     because their origin is not listed in the CSP.`,
        ls$2 `To solve this, carefully check that all of the blocked resources listed below are trustworthy; if they are,
     include their sources in the content security policy of your site. You can set a policy as a HTTP header (recommended),
     or via an HTML |<meta>| tag.`,
        ls$2 `⚠️ Never add a source you don't trust to your site's Content Security Policy. If you don't trust the source, consider
     hosting resources on your own site instead.`
    ], []),
    issueKind: IssueKind.BreakingChange,
    links: [{
            link: 'https://developers.google.com/web/fundamentals/security/csp#source_whitelists',
            linkTitle: ls$2 `Content Security Policy - Source Allowlists`
        }],
};
const cspInlineViolation = {
    title: ls$2 `Content Security Policy blocks inline execution of scripts and stylesheets`,
    message: () => paragraphedMessage([
        ls$2 `The Content Security Policy (CSP) prevents cross-site scripting attacks by blocking inline execution of scripts
     and style sheets.`,
        ls$2 `To solve this, move all inline scripts (e.g. |onclick=[JS code]|) and styles into external files.`,
        ls$2 `⚠️ Allowing inline execution comes at the risk of script injection via injection of HTML script elements.
     If you absolutely must, you can allow inline script and styles by:`
    ], [
        ls$2 `adding |unsafe-inline| as a source to the CSP header`,
        ls$2 `adding the hash or nonce of the inline script to your CSP header.`
    ]),
    issueKind: IssueKind.BreakingChange,
    links: [{
            link: 'https://developers.google.com/web/fundamentals/security/csp#inline_code_is_considered_harmful',
            linkTitle: ls$2 `Content Security Policy - Inline Code`
        }],
};
const cspEvalViolation = {
    title: ls$2 `Content Security Policy of your site blocks the use of 'eval' in JavaScript`,
    message: () => paragraphedMessage([
        ls$2 `The Content Security Policy (CSP) prevents the evaluation of arbitrary strings as JavaScript to make it
         more difficult for an attacker to inject unathorized code on your site.`,
        ls$2 `To solve this issue, avoid using |eval()|, |new Function()|, |setTimeout([string], ...)| and
         |setInterval([string], ...)| for evaluating strings.`,
        ls$2 `If you absolutely must: you can enable string evaluation by adding |unsafe-eval| as an allowed source
         in a |script-src| directive.`,
        ls$2 `⚠️ Allowing string evaluation comes at the risk of inline script injection.`
    ], []),
    issueKind: IssueKind.BreakingChange,
    links: [{
            link: 'https://developers.google.com/web/fundamentals/security/csp#eval_too',
            linkTitle: ls$2 `Content Security Policy - Eval`
        }],
};
/** @type {string} */
const urlViolationCode = [
    Protocol.Audits.InspectorIssueCode.ContentSecurityPolicyIssue,
    Protocol.Audits.ContentSecurityPolicyViolationType.KURLViolation
].join('::');
/** @type {string} */
const inlineViolationCode = [
    Protocol.Audits.InspectorIssueCode.ContentSecurityPolicyIssue,
    Protocol.Audits.ContentSecurityPolicyViolationType.KInlineViolation
].join('::');
/** @type {string} */
const evalViolationCode = [
    Protocol.Audits.InspectorIssueCode.ContentSecurityPolicyIssue,
    Protocol.Audits.ContentSecurityPolicyViolationType.KEvalViolation
].join('::');
// TODO(crbug.com/1082628): Add handling of other CSP violation types later as they'll need more work.
/** @type {!Map<!Protocol.Audits.ContentSecurityPolicyViolationType, !IssueDescription>} */
const issueDescriptions = new Map([
    [Protocol.Audits.ContentSecurityPolicyViolationType.KURLViolation, cspURLViolation],
    [Protocol.Audits.ContentSecurityPolicyViolationType.KInlineViolation, cspInlineViolation],
    [Protocol.Audits.ContentSecurityPolicyViolationType.KEvalViolation, cspEvalViolation],
]);

var ContentSecurityPolicyIssue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ContentSecurityPolicyIssue: ContentSecurityPolicyIssue,
    urlViolationCode: urlViolationCode,
    inlineViolationCode: inlineViolationCode,
    evalViolationCode: evalViolationCode
});

// Copyright 2017 The Chromium Authors. All rights reserved.
class CookieModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        /** Array<!Cookie> */
        this._blockedCookies = new Map();
        this._cookieToBlockedReasons = new Map();
    }
    /**
     * @param {!Cookie} cookie
     * @param {?Array<!BlockedReason>} blockedReasons
     */
    addBlockedCookie(cookie, blockedReasons) {
        const key = cookie.key();
        const previousCookie = this._blockedCookies.get(key);
        this._blockedCookies.set(key, cookie);
        this._cookieToBlockedReasons.set(cookie, blockedReasons);
        if (previousCookie) {
            this._cookieToBlockedReasons.delete(key);
        }
    }
    getCookieToBlockedReasonsMap() {
        return this._cookieToBlockedReasons;
    }
    /**
     * @param {!Array<string>} urls
     * @return {!Promise<!Array<!Cookie>>}
     */
    async getCookies(urls) {
        const response = await this.target().networkAgent().invoke_getCookies({ urls });
        if (response.getError()) {
            return [];
        }
        const normalCookies = response.cookies.map(Cookie.fromProtocolCookie);
        return normalCookies.concat(Array.from(this._blockedCookies.values()));
    }
    /**
     * @param {!Cookie} cookie
     * @return {!Promise<void>}
     */
    async deleteCookie(cookie) {
        await this._deleteAll([cookie]);
    }
    /**
     * @param {string=} domain
     * @return {!Promise<void>}
     */
    async clear(domain) {
        const cookies = await this.getCookiesForDomain(domain || null);
        await this._deleteAll(cookies);
    }
    /**
     * @param {!Cookie} cookie
     * @return {!Promise<boolean>}
     */
    async saveCookie(cookie) {
        let domain = cookie.domain();
        if (!domain.startsWith('.')) {
            domain = '';
        }
        let expires = undefined;
        if (cookie.expires()) {
            expires = Math.floor(Date.parse(`${cookie.expires()}`) / 1000);
        }
        const protocolCookie = {
            name: cookie.name(),
            value: cookie.value(),
            url: cookie.url() || undefined,
            domain,
            path: cookie.path(),
            secure: cookie.secure(),
            httpOnly: cookie.httpOnly(),
            sameSite: cookie.sameSite(),
            expires,
            priority: cookie.priority()
        };
        const response = await this.target().networkAgent().invoke_setCookie(protocolCookie);
        if (response.getError()) {
            return false;
        }
        return response.success;
    }
    /**
     * Returns cookies needed by current page's frames whose security origins are |domain|.
     * @param {?string} domain
     * @return {!Promise<!Array<!Cookie>>}
     */
    getCookiesForDomain(domain) {
        const resourceURLs = [];
        /**
         * @param {!Resource} resource
         */
        function populateResourceURLs(resource) {
            const documentURL = ParsedURL.ParsedURL.fromString(resource.documentURL);
            if (documentURL && (!domain || documentURL.securityOrigin() === domain)) {
                resourceURLs.push(resource.url);
            }
        }
        const resourceTreeModel = this.target().model(ResourceTreeModel);
        if (resourceTreeModel) {
            // In case the current frame was unreachable, add it's cookies
            // because they might help to debug why the frame was unreachable.
            if (resourceTreeModel.mainFrame.unreachableUrl()) {
                resourceURLs.push(resourceTreeModel.mainFrame.unreachableUrl());
            }
            resourceTreeModel.forAllResources(populateResourceURLs);
        }
        return this.getCookies(resourceURLs);
    }
    /**
     * @param {!Array<!Cookie>} cookies
     * @return {!Promise<void>}
     */
    async _deleteAll(cookies) {
        const networkAgent = this.target().networkAgent();
        this._blockedCookies.clear();
        this._cookieToBlockedReasons.clear();
        await Promise.all(cookies.map(cookie => networkAgent.invoke_deleteCookies({ name: cookie.name(), url: undefined, domain: cookie.domain(), path: cookie.path() })));
    }
}
SDKModel.register(CookieModel, Capability.Network, false);
/** @typedef {!{uiString: string, attribute: ?Attributes}} */
// @ts-ignore typedef
let BlockedReason;

var CookieModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CookieModel: CookieModel,
    BlockedReason: BlockedReason
});

// Copyright 2014 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class CPUProfileNode extends ProfileNode {
    /**
     * @param {!Protocol.Profiler.ProfileNode} node
     * @param {number} sampleTime
     */
    constructor(node, sampleTime) {
        const callFrame = node.callFrame || /** @type {!Protocol.Runtime.CallFrame} */ ({
            // Backward compatibility for old SamplingHeapProfileNode format.
            functionName: node['functionName'],
            scriptId: node['scriptId'],
            url: node['url'],
            lineNumber: node['lineNumber'] - 1,
            columnNumber: node['columnNumber'] - 1
        });
        super(callFrame);
        this.id = node.id;
        this.self = node.hitCount * sampleTime;
        this.positionTicks = node.positionTicks;
        // Compatibility: legacy backends could provide "no reason" for optimized functions.
        this.deoptReason = node.deoptReason && node.deoptReason !== 'no reason' ? node.deoptReason : null;
    }
}
/**
 * @unrestricted
 */
class CPUProfileDataModel extends ProfileTreeModel {
    /**
     * @param {!Protocol.Profiler.Profile} profile
     * @param {?Target} target
     */
    constructor(profile, target) {
        super(target);
        const isLegacyFormat = !!profile['head'];
        if (isLegacyFormat) {
            // Legacy format contains raw timestamps and start/stop times are in seconds.
            this.profileStartTime = profile.startTime * 1000;
            this.profileEndTime = profile.endTime * 1000;
            this.timestamps = profile.timestamps;
            this._compatibilityConversionHeadToNodes(profile);
        }
        else {
            // Current format encodes timestamps as deltas. Start/stop times are in microseconds.
            this.profileStartTime = profile.startTime / 1000;
            this.profileEndTime = profile.endTime / 1000;
            this.timestamps = this._convertTimeDeltas(profile);
        }
        this.samples = profile.samples;
        this.lines = profile.lines;
        this.totalHitCount = 0;
        this.profileHead = this._translateProfileTree(profile.nodes);
        this.initialize(this.profileHead);
        this._extractMetaNodes();
        if (this.samples) {
            this._buildIdToNodeMap();
            this._sortSamples();
            this._normalizeTimestamps();
            this._fixMissingSamples();
        }
    }
    /**
     * @param {!Protocol.Profiler.Profile} profile
     */
    _compatibilityConversionHeadToNodes(profile) {
        if (!profile.head || profile.nodes) {
            return;
        }
        /** @type {!Array<!Protocol.Profiler.ProfileNode>} */
        const nodes = [];
        convertNodesTree(profile.head);
        profile.nodes = nodes;
        delete profile.head;
        /**
         * @param {!Protocol.Profiler.ProfileNode} node
         * @return {number}
         */
        function convertNodesTree(node) {
            nodes.push(node);
            node.children = ( /** @type {!Array<!Protocol.Profiler.ProfileNode>} */(node.children)).map(convertNodesTree);
            return node.id;
        }
    }
    /**
     * @param {!Protocol.Profiler.Profile} profile
     * @return {?Array<number>}
     */
    _convertTimeDeltas(profile) {
        if (!profile.timeDeltas) {
            return null;
        }
        let lastTimeUsec = profile.startTime;
        const timestamps = new Array(profile.timeDeltas.length);
        for (let i = 0; i < profile.timeDeltas.length; ++i) {
            lastTimeUsec += profile.timeDeltas[i];
            timestamps[i] = lastTimeUsec;
        }
        return timestamps;
    }
    /**
     * @param {!Array<!Protocol.Profiler.ProfileNode>} nodes
     * @return {!CPUProfileNode}
     */
    _translateProfileTree(nodes) {
        /**
         * @param {!Protocol.Profiler.ProfileNode} node
         * @return {boolean}
         */
        function isNativeNode(node) {
            if (node.callFrame) {
                return !!node.callFrame.url && node.callFrame.url.startsWith('native ');
            }
            return !!node['url'] && node['url'].startsWith('native ');
        }
        /**
         * @param {!Array<!Protocol.Profiler.ProfileNode>} nodes
         */
        function buildChildrenFromParents(nodes) {
            if (nodes[0].children) {
                return;
            }
            nodes[0].children = [];
            for (let i = 1; i < nodes.length; ++i) {
                const node = nodes[i];
                const parentNode = nodeByIdMap.get(node.parent);
                if (parentNode.children) {
                    parentNode.children.push(node.id);
                }
                else {
                    parentNode.children = [node.id];
                }
            }
        }
        /**
         * @param {!Array<!Protocol.Profiler.ProfileNode>} nodes
         * @param {!Array<number>|undefined} samples
         */
        function buildHitCountFromSamples(nodes, samples) {
            if (typeof (nodes[0].hitCount) === 'number') {
                return;
            }
            console.assert(samples, 'Error: Neither hitCount nor samples are present in profile.');
            for (let i = 0; i < nodes.length; ++i) {
                nodes[i].hitCount = 0;
            }
            for (let i = 0; i < samples.length; ++i) {
                ++nodeByIdMap.get(samples[i]).hitCount;
            }
        }
        /** @type {!Map<number, !Protocol.Profiler.ProfileNode>} */
        const nodeByIdMap = new Map();
        for (let i = 0; i < nodes.length; ++i) {
            const node = nodes[i];
            nodeByIdMap.set(node.id, node);
        }
        buildHitCountFromSamples(nodes, this.samples);
        buildChildrenFromParents(nodes);
        this.totalHitCount = nodes.reduce((acc, node) => acc + node.hitCount, 0);
        const sampleTime = (this.profileEndTime - this.profileStartTime) / this.totalHitCount;
        const keepNatives = !!Settings.Settings.instance().moduleSetting('showNativeFunctionsInJSProfile').get();
        const root = nodes[0];
        /** @type {!Map<number, number>} */
        const idMap = new Map([[root.id, root.id]]);
        const resultRoot = new CPUProfileNode(root, sampleTime);
        const parentNodeStack = root.children.map(() => resultRoot);
        const sourceNodeStack = root.children.map(id => nodeByIdMap.get(id));
        while (sourceNodeStack.length) {
            let parentNode = parentNodeStack.pop();
            const sourceNode = sourceNodeStack.pop();
            if (!sourceNode.children) {
                sourceNode.children = [];
            }
            const targetNode = new CPUProfileNode(sourceNode, sampleTime);
            if (keepNatives || !isNativeNode(sourceNode)) {
                parentNode.children.push(targetNode);
                parentNode = targetNode;
            }
            else {
                parentNode.self += targetNode.self;
            }
            idMap.set(sourceNode.id, parentNode.id);
            parentNodeStack.push.apply(parentNodeStack, sourceNode.children.map(() => parentNode));
            sourceNodeStack.push.apply(sourceNodeStack, sourceNode.children.map(id => nodeByIdMap.get(id)));
        }
        if (this.samples) {
            this.samples = this.samples.map(id => idMap.get(id));
        }
        return resultRoot;
    }
    _sortSamples() {
        const timestamps = this.timestamps;
        if (!timestamps) {
            return;
        }
        const samples = this.samples;
        const indices = timestamps.map((x, index) => index);
        indices.sort((a, b) => timestamps[a] - timestamps[b]);
        for (let i = 0; i < timestamps.length; ++i) {
            let index = indices[i];
            if (index === i) {
                continue;
            }
            // Move items in a cycle.
            const savedTimestamp = timestamps[i];
            const savedSample = samples[i];
            let currentIndex = i;
            while (index !== i) {
                samples[currentIndex] = samples[index];
                timestamps[currentIndex] = timestamps[index];
                currentIndex = index;
                index = indices[index];
                indices[currentIndex] = currentIndex;
            }
            samples[currentIndex] = savedSample;
            timestamps[currentIndex] = savedTimestamp;
        }
    }
    _normalizeTimestamps() {
        let timestamps = this.timestamps;
        if (!timestamps) {
            // Support loading old CPU profiles that are missing timestamps.
            // Derive timestamps from profile start and stop times.
            const profileStartTime = this.profileStartTime;
            const interval = (this.profileEndTime - profileStartTime) / this.samples.length;
            timestamps = new Float64Array(this.samples.length + 1);
            for (let i = 0; i < timestamps.length; ++i) {
                timestamps[i] = profileStartTime + i * interval;
            }
            this.timestamps = timestamps;
            return;
        }
        // Convert samples from usec to msec
        for (let i = 0; i < timestamps.length; ++i) {
            timestamps[i] /= 1000;
        }
        if (this.samples.length === timestamps.length) {
            // Support for a legacy format where were no timeDeltas.
            // Add an extra timestamp used to calculate the last sample duration.
            const averageSample = (timestamps.peekLast() - timestamps[0]) / (timestamps.length - 1);
            this.timestamps.push(timestamps.peekLast() + averageSample);
        }
        this.profileStartTime = timestamps[0];
        this.profileEndTime = timestamps.peekLast();
    }
    _buildIdToNodeMap() {
        /** @type {!Map<number, !CPUProfileNode>} */
        this._idToNode = new Map();
        const idToNode = this._idToNode;
        const stack = [this.profileHead];
        while (stack.length) {
            const node = stack.pop();
            idToNode.set(node.id, node);
            stack.push.apply(stack, node.children);
        }
    }
    _extractMetaNodes() {
        const topLevelNodes = this.profileHead.children;
        for (let i = 0; i < topLevelNodes.length && !(this.gcNode && this.programNode && this.idleNode); i++) {
            const node = topLevelNodes[i];
            if (node.functionName === '(garbage collector)') {
                this.gcNode = node;
            }
            else if (node.functionName === '(program)') {
                this.programNode = node;
            }
            else if (node.functionName === '(idle)') {
                this.idleNode = node;
            }
        }
    }
    _fixMissingSamples() {
        // Sometimes sampler is not able to parse the JS stack and returns
        // a (program) sample instead. The issue leads to call frames belong
        // to the same function invocation being split apart.
        // Here's a workaround for that. When there's a single (program) sample
        // between two call stacks sharing the same bottom node, it is replaced
        // with the preceeding sample.
        const samples = this.samples;
        const samplesCount = samples.length;
        if (!this.programNode || samplesCount < 3) {
            return;
        }
        const idToNode = this._idToNode;
        const programNodeId = this.programNode.id;
        const gcNodeId = this.gcNode ? this.gcNode.id : -1;
        const idleNodeId = this.idleNode ? this.idleNode.id : -1;
        let prevNodeId = samples[0];
        let nodeId = samples[1];
        let count = 0;
        for (let sampleIndex = 1; sampleIndex < samplesCount - 1; sampleIndex++) {
            const nextNodeId = samples[sampleIndex + 1];
            if (nodeId === programNodeId && !isSystemNode(prevNodeId) && !isSystemNode(nextNodeId) &&
                bottomNode(idToNode.get(prevNodeId)) === bottomNode(idToNode.get(nextNodeId))) {
                ++count;
                samples[sampleIndex] = prevNodeId;
            }
            prevNodeId = nodeId;
            nodeId = nextNodeId;
        }
        if (count) {
            Console.Console.instance().warn(ls `DevTools: CPU profile parser is fixing ${count} missing samples.`);
        }
        /**
         * @param {!ProfileNode} node
         * @return {!ProfileNode}
         */
        function bottomNode(node) {
            while (node.parent && node.parent.parent) {
                node = node.parent;
            }
            return node;
        }
        /**
         * @param {number} nodeId
         * @return {boolean}
         */
        function isSystemNode(nodeId) {
            return nodeId === programNodeId || nodeId === gcNodeId || nodeId === idleNodeId;
        }
    }
    /**
     * @param {function(number, !CPUProfileNode, number):void} openFrameCallback
     * @param {function(number, !CPUProfileNode, number, number, number):void} closeFrameCallback
     * @param {number=} startTime
     * @param {number=} stopTime
     */
    forEachFrame(openFrameCallback, closeFrameCallback, startTime, stopTime) {
        if (!this.profileHead || !this.samples) {
            return;
        }
        startTime = startTime || 0;
        stopTime = stopTime || Infinity;
        const samples = this.samples;
        const timestamps = this.timestamps;
        const idToNode = this._idToNode;
        const gcNode = this.gcNode;
        const samplesCount = samples.length;
        const startIndex = timestamps.lowerBound(startTime);
        let stackTop = 0;
        const stackNodes = [];
        let prevId = this.profileHead.id;
        let sampleTime;
        let gcParentNode = null;
        // Extra slots for gc being put on top,
        // and one at the bottom to allow safe stackTop-1 access.
        const stackDepth = this.maxDepth + 3;
        if (!this._stackStartTimes) {
            this._stackStartTimes = new Float64Array(stackDepth);
        }
        const stackStartTimes = this._stackStartTimes;
        if (!this._stackChildrenDuration) {
            this._stackChildrenDuration = new Float64Array(stackDepth);
        }
        const stackChildrenDuration = this._stackChildrenDuration;
        let node;
        let sampleIndex;
        for (sampleIndex = startIndex; sampleIndex < samplesCount; sampleIndex++) {
            sampleTime = timestamps[sampleIndex];
            if (sampleTime >= stopTime) {
                break;
            }
            const id = samples[sampleIndex];
            if (id === prevId) {
                continue;
            }
            node = idToNode.get(id);
            let prevNode = idToNode.get(prevId);
            if (node === gcNode) {
                // GC samples have no stack, so we just put GC node on top of the last recorded sample.
                gcParentNode = prevNode;
                openFrameCallback(gcParentNode.depth + 1, gcNode, sampleTime);
                stackStartTimes[++stackTop] = sampleTime;
                stackChildrenDuration[stackTop] = 0;
                prevId = id;
                continue;
            }
            if (prevNode === gcNode) {
                // end of GC frame
                const start = stackStartTimes[stackTop];
                const duration = sampleTime - start;
                stackChildrenDuration[stackTop - 1] += duration;
                closeFrameCallback(gcParentNode.depth + 1, gcNode, start, duration, duration - stackChildrenDuration[stackTop]);
                --stackTop;
                prevNode = gcParentNode;
                prevId = prevNode.id;
                gcParentNode = null;
            }
            while (node.depth > prevNode.depth) {
                stackNodes.push(node);
                node = node.parent;
            }
            // Go down to the LCA and close current intervals.
            while (prevNode !== node) {
                const start = stackStartTimes[stackTop];
                const duration = sampleTime - start;
                stackChildrenDuration[stackTop - 1] += duration;
                closeFrameCallback(prevNode.depth, /** @type {!CPUProfileNode} */ (prevNode), start, duration, duration - stackChildrenDuration[stackTop]);
                --stackTop;
                if (node.depth === prevNode.depth) {
                    stackNodes.push(node);
                    node = node.parent;
                }
                prevNode = prevNode.parent;
            }
            // Go up the nodes stack and open new intervals.
            while (stackNodes.length) {
                node = stackNodes.pop();
                openFrameCallback(node.depth, node, sampleTime);
                stackStartTimes[++stackTop] = sampleTime;
                stackChildrenDuration[stackTop] = 0;
            }
            prevId = id;
        }
        sampleTime = timestamps[sampleIndex] || this.profileEndTime;
        if (idToNode.get(prevId) === gcNode) {
            const start = stackStartTimes[stackTop];
            const duration = sampleTime - start;
            stackChildrenDuration[stackTop - 1] += duration;
            closeFrameCallback(gcParentNode.depth + 1, node, start, duration, duration - stackChildrenDuration[stackTop]);
            --stackTop;
            prevId = gcParentNode.id;
        }
        for (let node = idToNode.get(prevId); node.parent; node = node.parent) {
            const start = stackStartTimes[stackTop];
            const duration = sampleTime - start;
            stackChildrenDuration[stackTop - 1] += duration;
            closeFrameCallback(node.depth, /** @type {!CPUProfileNode} */ (node), start, duration, duration - stackChildrenDuration[stackTop]);
            --stackTop;
        }
    }
    /**
     * @param {number} index
     * @return {?CPUProfileNode}
     */
    nodeByIndex(index) {
        return this._idToNode.get(this.samples[index]) || null;
    }
}

var CPUProfileDataModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CPUProfileNode: CPUProfileNode,
    CPUProfileDataModel: CPUProfileDataModel
});

// Copyright 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
const globalValues = new Set(['inherit', 'initial', 'unset']);
const tagRegexp = /[\x20-\x7E]{4}/;
const numRegexp = /[+-]?(?:\d*\.)?\d+(?:[eE]\d+)?/;
const fontVariationSettingsRegexp = new RegExp(`(?:'(${tagRegexp.source})')|(?:"(${tagRegexp.source})")\\s+(${numRegexp.source})`);
/**
 * Extracts information about font variation settings assuming
 * value is valid according to the spec: https://drafts.csswg.org/css-fonts-4/#font-variation-settings-def
 *
 * @param {string} value A valid CSS font-variation-settings property value
 * @return {!Array<!{tag:string, value:number}>}
 */
function parseFontVariationSettings(value) {
    if (globalValues.has(value.trim()) || value.trim() === 'normal') {
        return [];
    }
    const results = [];
    for (const setting of splitByComma(stripComments(value))) {
        const match = setting.match(fontVariationSettingsRegexp);
        if (match) {
            results.push({
                tag: match[1] || match[2],
                value: parseFloat(match[3]),
            });
        }
    }
    return results;
}
// "str" or 'str'
const fontFamilyRegexp = /^"(.+)"|'(.+)'$/;
/**
 * Extracts font families assuming the value is valid according to
 * the spec: https://drafts.csswg.org/css-fonts-4/#font-family-prop
 *
 * @param {string} value A valid CSS font-family property value
 * @return {!Array<string>}
 */
function parseFontFamily(value) {
    if (globalValues.has(value.trim())) {
        return [];
    }
    const results = [];
    for (const family of splitByComma(stripComments(value))) {
        const match = family.match(fontFamilyRegexp);
        if (match) {
            // Either the 1st or 2nd group matches if the value is in quotes
            results.push(match[1] || match[2]);
        }
        else {
            // Value without without quotes.
            results.push(family);
        }
    }
    return results;
}
/**
 * Splits a list of values by comma and trims parts
 * @param {string} value
 * @return {!Array<string>}
 */
function splitByComma(value) {
    return value.split(',').map(part => part.trim());
}
/**
 * @param {string} value
 * @return {string}
 * @suppress {missingProperties} Closure doesn't know String.p.replaceAll exists.
 */
function stripComments(value) {
    return value.replaceAll(/(\/\*(?:.|\s)*?\*\/)/g, '');
}

var CSSPropertyParser = /*#__PURE__*/Object.freeze({
    __proto__: null,
    parseFontVariationSettings: parseFontVariationSettings,
    parseFontFamily: parseFontFamily,
    splitByComma: splitByComma,
    stripComments: stripComments
});

// Copyright 2017 The Chromium Authors. All rights reserved.
class DOMDebuggerModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._agent = target.domdebuggerAgent();
        this._runtimeModel = /** @type {!RuntimeModel} */ (target.model(RuntimeModel));
        this._domModel = /** @type {!DOMModel} */ (target.model(DOMModel));
        this._domModel.addEventListener(Events$8.DocumentUpdated, this._documentUpdated, this);
        this._domModel.addEventListener(Events$8.NodeRemoved, this._nodeRemoved, this);
        /** @type {!Array<!DOMBreakpoint>} */
        this._domBreakpoints = [];
        this._domBreakpointsSetting = Settings.Settings.instance().createLocalSetting('domBreakpoints', []);
        if (this._domModel.existingDocument()) {
            this._documentUpdated();
        }
    }
    /**
     * @return {!RuntimeModel}
     */
    runtimeModel() {
        return this._runtimeModel;
    }
    /**
     * @param {!RemoteObject} remoteObject
     * @return {!Promise<!Array<!EventListener>>}
     */
    async eventListeners(remoteObject) {
        console.assert(remoteObject.runtimeModel() === this._runtimeModel);
        if (!remoteObject.objectId) {
            return [];
        }
        const listeners = await this._agent.invoke_getEventListeners({ objectId: /** @type {string} */ (remoteObject.objectId) });
        const eventListeners = [];
        for (const payload of listeners.listeners || []) {
            const location = this._runtimeModel.debuggerModel().createRawLocationByScriptId(payload.scriptId, payload.lineNumber, payload.columnNumber);
            if (!location) {
                continue;
            }
            eventListeners.push(new EventListener(this, remoteObject, payload.type, payload.useCapture, payload.passive, payload.once, payload.handler ? this._runtimeModel.createRemoteObject(payload.handler) : null, payload.originalHandler ? this._runtimeModel.createRemoteObject(payload.originalHandler) : null, location, null));
        }
        return eventListeners;
    }
    retrieveDOMBreakpoints() {
        this._domModel.requestDocument();
    }
    /**
     * @return {!Array<!DOMBreakpoint>}
     */
    domBreakpoints() {
        return this._domBreakpoints.slice();
    }
    /**
     * @param {!DOMNode} node
     * @param {!Protocol.DOMDebugger.DOMBreakpointType} type
     * @return {boolean}
     */
    hasDOMBreakpoint(node, type) {
        return this._domBreakpoints.some(breakpoint => (breakpoint.node === node && breakpoint.type === type));
    }
    /**
     * @param {!DOMNode} node
     * @param {!Protocol.DOMDebugger.DOMBreakpointType} type
     * @return {!DOMBreakpoint}
     */
    setDOMBreakpoint(node, type) {
        for (const breakpoint of this._domBreakpoints) {
            if (breakpoint.node === node && breakpoint.type === type) {
                this.toggleDOMBreakpoint(breakpoint, true);
                return breakpoint;
            }
        }
        const breakpoint = new DOMBreakpoint(this, node, type, true);
        this._domBreakpoints.push(breakpoint);
        this._saveDOMBreakpoints();
        this._enableDOMBreakpoint(breakpoint);
        this.dispatchEventToListeners(Events$i.DOMBreakpointAdded, breakpoint);
        return breakpoint;
    }
    /**
     * @param {!DOMNode} node
     * @param {!Protocol.DOMDebugger.DOMBreakpointType} type
     */
    removeDOMBreakpoint(node, type) {
        this._removeDOMBreakpoints(breakpoint => breakpoint.node === node && breakpoint.type === type);
    }
    removeAllDOMBreakpoints() {
        this._removeDOMBreakpoints(breakpoint => true);
    }
    /**
     * @param {!DOMBreakpoint} breakpoint
     * @param {boolean} enabled
     */
    toggleDOMBreakpoint(breakpoint, enabled) {
        if (enabled === breakpoint.enabled) {
            return;
        }
        breakpoint.enabled = enabled;
        if (enabled) {
            this._enableDOMBreakpoint(breakpoint);
        }
        else {
            this._disableDOMBreakpoint(breakpoint);
        }
        this.dispatchEventToListeners(Events$i.DOMBreakpointToggled, breakpoint);
    }
    /**
     * @param {!DOMBreakpoint} breakpoint
     */
    _enableDOMBreakpoint(breakpoint) {
        if (breakpoint.node.id) {
            this._agent.invoke_setDOMBreakpoint({ nodeId: breakpoint.node.id, type: breakpoint.type });
            breakpoint.node.setMarker(Marker, true);
        }
    }
    /**
     * @param {!DOMBreakpoint} breakpoint
     */
    _disableDOMBreakpoint(breakpoint) {
        if (breakpoint.node.id) {
            this._agent.invoke_removeDOMBreakpoint({ nodeId: breakpoint.node.id, type: breakpoint.type });
            breakpoint.node.setMarker(Marker, this._nodeHasBreakpoints(breakpoint.node) ? true : null);
        }
    }
    /**
     * @param {!DOMNode} node
     * @return {boolean}
     */
    _nodeHasBreakpoints(node) {
        for (const breakpoint of this._domBreakpoints) {
            if (breakpoint.node === node && breakpoint.enabled) {
                return true;
            }
        }
        return false;
    }
    /**
     * @param {{type: !Protocol.DOMDebugger.DOMBreakpointType, nodeId: !Protocol.DOM.NodeId, targetNodeId: !Protocol.DOM.NodeId, insertion: boolean}} auxData
     * @return {?{type: !Protocol.DOMDebugger.DOMBreakpointType, node: !DOMNode, targetNode: ?DOMNode, insertion: boolean}}
     */
    resolveDOMBreakpointData(auxData) {
        const type = auxData['type'];
        const node = this._domModel.nodeForId(auxData['nodeId']);
        if (!type || !node) {
            return null;
        }
        let targetNode = null;
        let insertion = false;
        if (type === Protocol.DOMDebugger.DOMBreakpointType.SubtreeModified) {
            insertion = auxData['insertion'] || false;
            targetNode = this._domModel.nodeForId(auxData['targetNodeId']);
        }
        return { type: type, node: node, targetNode: targetNode, insertion: insertion };
    }
    /**
     * @return {string}
     */
    _currentURL() {
        const domDocument = this._domModel.existingDocument();
        return domDocument ? domDocument.documentURL : '';
    }
    _documentUpdated() {
        const removed = this._domBreakpoints;
        this._domBreakpoints = [];
        this.dispatchEventToListeners(Events$i.DOMBreakpointsRemoved, removed);
        const currentURL = this._currentURL();
        for (const breakpoint of this._domBreakpointsSetting.get()) {
            if (breakpoint.url === currentURL) {
                this._domModel.pushNodeByPathToFrontend(breakpoint.path).then(appendBreakpoint.bind(this, breakpoint));
            }
        }
        /**
         * @param {!{type: !Protocol.DOMDebugger.DOMBreakpointType, enabled: boolean}} breakpoint
         * @param {?number} nodeId
         * @this {DOMDebuggerModel}
         */
        function appendBreakpoint(breakpoint, nodeId) {
            const node = nodeId ? this._domModel.nodeForId(nodeId) : null;
            if (!node) {
                return;
            }
            const domBreakpoint = new DOMBreakpoint(this, node, breakpoint.type, breakpoint.enabled);
            this._domBreakpoints.push(domBreakpoint);
            if (breakpoint.enabled) {
                this._enableDOMBreakpoint(domBreakpoint);
            }
            this.dispatchEventToListeners(Events$i.DOMBreakpointAdded, domBreakpoint);
        }
    }
    /**
     * @param {function(!DOMBreakpoint):boolean} filter
     */
    _removeDOMBreakpoints(filter) {
        const removed = [];
        const left = [];
        for (const breakpoint of this._domBreakpoints) {
            if (filter(breakpoint)) {
                removed.push(breakpoint);
                if (breakpoint.enabled) {
                    breakpoint.enabled = false;
                    this._disableDOMBreakpoint(breakpoint);
                }
            }
            else {
                left.push(breakpoint);
            }
        }
        if (!removed.length) {
            return;
        }
        this._domBreakpoints = left;
        this._saveDOMBreakpoints();
        this.dispatchEventToListeners(Events$i.DOMBreakpointsRemoved, removed);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _nodeRemoved(event) {
        const node = /** @type {!DOMNode} */ (event.data.node);
        const children = node.children() || [];
        this._removeDOMBreakpoints(breakpoint => breakpoint.node === node || children.indexOf(breakpoint.node) !== -1);
    }
    _saveDOMBreakpoints() {
        const currentURL = this._currentURL();
        const breakpoints = this._domBreakpointsSetting.get().filter(
        /** @param {{url: string}} breakpoint */ breakpoint => breakpoint.url !== currentURL);
        for (const breakpoint of this._domBreakpoints) {
            breakpoints.push({ url: currentURL, path: breakpoint.node.path(), type: breakpoint.type, enabled: breakpoint.enabled });
        }
        this._domBreakpointsSetting.set(breakpoints);
    }
}
/** @enum {symbol} */
const Events$i = {
    DOMBreakpointAdded: Symbol('DOMBreakpointAdded'),
    DOMBreakpointToggled: Symbol('DOMBreakpointToggled'),
    DOMBreakpointsRemoved: Symbol('DOMBreakpointsRemoved'),
};
const Marker = 'breakpoint-marker';
class DOMBreakpoint {
    /**
     * @param {!DOMDebuggerModel} domDebuggerModel
     * @param {!DOMNode} node
     * @param {!Protocol.DOMDebugger.DOMBreakpointType} type
     * @param {boolean} enabled
     */
    constructor(domDebuggerModel, node, type, enabled) {
        this.domDebuggerModel = domDebuggerModel;
        this.node = node;
        this.type = type;
        this.enabled = enabled;
    }
}
class EventListener {
    /**
     * @param {!DOMDebuggerModel} domDebuggerModel
     * @param {!RemoteObject} eventTarget
     * @param {string} type
     * @param {boolean} useCapture
     * @param {boolean} passive
     * @param {boolean} once
     * @param {?RemoteObject} handler
     * @param {?RemoteObject} originalHandler
     * @param {!Location} location
     * @param {?RemoteObject} customRemoveFunction
     * @param {!EventListener.Origin=} origin
     */
    constructor(domDebuggerModel, eventTarget, type, useCapture, passive, once, handler, originalHandler, location, customRemoveFunction, origin) {
        this._domDebuggerModel = domDebuggerModel;
        this._eventTarget = eventTarget;
        this._type = type;
        this._useCapture = useCapture;
        this._passive = passive;
        this._once = once;
        this._handler = handler;
        this._originalHandler = originalHandler || handler;
        this._location = location;
        const script = location.script();
        this._sourceURL = script ? script.contentURL() : '';
        this._customRemoveFunction = customRemoveFunction;
        this._origin = origin || EventListener.Origin.Raw;
    }
    /**
     * @return {!DOMDebuggerModel}
     */
    domDebuggerModel() {
        return this._domDebuggerModel;
    }
    /**
     * @return {string}
     */
    type() {
        return this._type;
    }
    /**
     * @return {boolean}
     */
    useCapture() {
        return this._useCapture;
    }
    /**
     * @return {boolean}
     */
    passive() {
        return this._passive;
    }
    /**
     * @return {boolean}
     */
    once() {
        return this._once;
    }
    /**
     * @return {?RemoteObject}
     */
    handler() {
        return this._handler;
    }
    /**
     * @return {!Location}
     */
    location() {
        return this._location;
    }
    /**
     * @return {string}
     */
    sourceURL() {
        return this._sourceURL;
    }
    /**
     * @return {?RemoteObject}
     */
    originalHandler() {
        return this._originalHandler;
    }
    /**
     * @return {boolean}
     */
    canRemove() {
        return !!this._customRemoveFunction || this._origin !== EventListener.Origin.FrameworkUser;
    }
    /**
     * @return {!Promise<undefined>}
     */
    remove() {
        if (!this.canRemove()) {
            return Promise.resolve(undefined);
        }
        if (this._origin !== EventListener.Origin.FrameworkUser) {
            /**
             * @param {string} type
             * @param {function():void} listener
             * @param {boolean} useCapture
             * @this {{removeEventListener: function(string, function():void, boolean):void}}
             * @suppressReceiverCheck
             */
            function removeListener(type, listener, useCapture) {
                this.removeEventListener(type, listener, useCapture);
                // @ts-ignore:
                if (this['on' + type]) {
                    // @ts-ignore:
                    this['on' + type] = undefined;
                }
            }
            return /** @type {!Promise<undefined>} */ (this._eventTarget
                .callFunction(removeListener, [
                RemoteObject.toCallArgument(this._type), RemoteObject.toCallArgument(this._originalHandler),
                RemoteObject.toCallArgument(this._useCapture)
            ])
                .then(() => undefined));
        }
        if (this._customRemoveFunction) {
            /**
             * @param {string} type
             * @param {function():void} listener
             * @param {boolean} useCapture
             * @param {boolean} passive
             * @this {function(string, function():void, boolean, boolean):void}
             * @suppressReceiverCheck
             */
            function callCustomRemove(type, listener, useCapture, passive) {
                this.call(null, type, listener, useCapture, passive);
            }
            return this._customRemoveFunction
                .callFunction(callCustomRemove, [
                RemoteObject.toCallArgument(this._type),
                RemoteObject.toCallArgument(this._originalHandler),
                RemoteObject.toCallArgument(this._useCapture),
                RemoteObject.toCallArgument(this._passive),
            ])
                .then(() => undefined);
        }
        return Promise.resolve(undefined);
    }
    /**
     * @return {boolean}
     */
    canTogglePassive() {
        return this._origin !== EventListener.Origin.FrameworkUser;
    }
    /**
     * @return {!Promise<undefined>}
     */
    togglePassive() {
        return this._eventTarget
            .callFunction(callTogglePassive, [
            RemoteObject.toCallArgument(this._type),
            RemoteObject.toCallArgument(this._originalHandler),
            RemoteObject.toCallArgument(this._useCapture),
            RemoteObject.toCallArgument(this._passive),
        ])
            .then(() => undefined);
        /**
         * @param {string} type
         * @param {function():void} listener
         * @param {boolean} useCapture
         * @param {boolean} passive
         * @this {{
             addEventListener:function(string, function():void, {capture: boolean, passive: boolean}):void,
             removeEventListener: function(string, function():void, {capture: boolean}):void,
           }}
         * @suppressReceiverCheck
         */
        function callTogglePassive(type, listener, useCapture, passive) {
            this.removeEventListener(type, listener, { capture: useCapture });
            this.addEventListener(type, listener, { capture: useCapture, passive: !passive });
        }
    }
    /**
     * @return {!EventListener.Origin}
     */
    origin() {
        return this._origin;
    }
    markAsFramework() {
        this._origin = EventListener.Origin.Framework;
    }
    /**
     * @return {boolean}
     */
    isScrollBlockingType() {
        return this._type === 'touchstart' || this._type === 'touchmove' || this._type === 'mousewheel' ||
            this._type === 'wheel';
    }
}
/** @enum {string} */
EventListener.Origin = {
    Raw: 'Raw',
    Framework: 'Framework',
    FrameworkUser: 'FrameworkUser'
};
class EventListenerBreakpoint {
    /**
     * @param {string} instrumentationName
     * @param {string} eventName
     * @param {!Array<string>} eventTargetNames
     * @param {string} category
     * @param {string} title
     */
    constructor(instrumentationName, eventName, eventTargetNames, category, title) {
        this._instrumentationName = instrumentationName;
        this._eventName = eventName;
        this._eventTargetNames = eventTargetNames;
        this._category = category;
        this._title = title;
        /** @type {boolean} */
        this._enabled = false;
    }
    /**
     * @return {string}
     */
    category() {
        return this._category;
    }
    /**
     * @return {boolean}
     */
    enabled() {
        return this._enabled;
    }
    /**
     * @param {boolean} enabled
     */
    setEnabled(enabled) {
        if (this._enabled === enabled) {
            return;
        }
        this._enabled = enabled;
        for (const model of TargetManager.instance().models(DOMDebuggerModel)) {
            this._updateOnModel(model);
        }
    }
    /**
     * @param {!DOMDebuggerModel} model
     */
    _updateOnModel(model) {
        if (this._instrumentationName) {
            if (this._enabled) {
                model._agent.invoke_setInstrumentationBreakpoint({ eventName: this._instrumentationName });
            }
            else {
                model._agent.invoke_removeInstrumentationBreakpoint({ eventName: this._instrumentationName });
            }
        }
        else {
            for (const eventTargetName of this._eventTargetNames) {
                if (this._enabled) {
                    model._agent.invoke_setEventListenerBreakpoint({ eventName: this._eventName, targetName: eventTargetName });
                }
                else {
                    model._agent.invoke_removeEventListenerBreakpoint({ eventName: this._eventName, targetName: eventTargetName });
                }
            }
        }
    }
    /**
     * @return {string}
     */
    title() {
        return this._title;
    }
}
EventListenerBreakpoint._listener = 'listener:';
EventListenerBreakpoint._instrumentation = 'instrumentation:';
/** @type {!DOMDebuggerManager} */
let domDebuggerManagerInstance;
/**
 * @implements {SDKModelObserver<!DOMDebuggerModel>}
 */
class DOMDebuggerManager {
    constructor() {
        this._xhrBreakpointsSetting = Settings.Settings.instance().createLocalSetting('xhrBreakpoints', []);
        /** @type {!Map<string, boolean>} */
        this._xhrBreakpoints = new Map();
        for (const breakpoint of this._xhrBreakpointsSetting.get()) {
            this._xhrBreakpoints.set(breakpoint.url, breakpoint.enabled);
        }
        /** @type {!Array<!EventListenerBreakpoint>} */
        this._eventListenerBreakpoints = [];
        this._createInstrumentationBreakpoints(UIString.UIString('Animation'), ['requestAnimationFrame', 'cancelAnimationFrame', 'requestAnimationFrame.callback']);
        this._createInstrumentationBreakpoints(UIString.UIString('Canvas'), ['canvasContextCreated', 'webglErrorFired', 'webglWarningFired']);
        this._createInstrumentationBreakpoints(UIString.UIString('Geolocation'), ['Geolocation.getCurrentPosition', 'Geolocation.watchPosition']);
        this._createInstrumentationBreakpoints(UIString.UIString('Notification'), ['Notification.requestPermission']);
        this._createInstrumentationBreakpoints(UIString.UIString('Parse'), ['Element.setInnerHTML', 'Document.write']);
        this._createInstrumentationBreakpoints(UIString.UIString('Script'), ['scriptFirstStatement', 'scriptBlockedByCSP']);
        this._createInstrumentationBreakpoints(UIString.UIString('Timer'), ['setTimeout', 'clearTimeout', 'setInterval', 'clearInterval', 'setTimeout.callback', 'setInterval.callback']);
        this._createInstrumentationBreakpoints(UIString.UIString('Window'), ['DOMWindow.close']);
        this._createInstrumentationBreakpoints(UIString.UIString('WebAudio'), ['audioContextCreated', 'audioContextClosed', 'audioContextResumed', 'audioContextSuspended']);
        this._createEventListenerBreakpoints(UIString.UIString('Media'), [
            'play', 'pause', 'playing', 'canplay', 'canplaythrough', 'seeking',
            'seeked', 'timeupdate', 'ended', 'ratechange', 'durationchange', 'volumechange',
            'loadstart', 'progress', 'suspend', 'abort', 'error', 'emptied',
            'stalled', 'loadedmetadata', 'loadeddata', 'waiting'
        ], ['audio', 'video']);
        this._createEventListenerBreakpoints(UIString.UIString('Picture-in-Picture'), ['enterpictureinpicture', 'leavepictureinpicture'], ['video']);
        this._createEventListenerBreakpoints(UIString.UIString('Picture-in-Picture'), ['resize'], ['PictureInPictureWindow']);
        this._createEventListenerBreakpoints(UIString.UIString('Clipboard'), ['copy', 'cut', 'paste', 'beforecopy', 'beforecut', 'beforepaste'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Control'), ['resize', 'scroll', 'zoom', 'focus', 'blur', 'select', 'change', 'submit', 'reset'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Device'), ['deviceorientation', 'devicemotion'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('DOM Mutation'), [
            'DOMActivate', 'DOMFocusIn', 'DOMFocusOut', 'DOMAttrModified', 'DOMCharacterDataModified', 'DOMNodeInserted',
            'DOMNodeInsertedIntoDocument', 'DOMNodeRemoved', 'DOMNodeRemovedFromDocument', 'DOMSubtreeModified',
            'DOMContentLoaded'
        ], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Drag / drop'), ['drag', 'dragstart', 'dragend', 'dragenter', 'dragover', 'dragleave', 'drop'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Keyboard'), ['keydown', 'keyup', 'keypress', 'input'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Load'), ['load', 'beforeunload', 'unload', 'abort', 'error', 'hashchange', 'popstate'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Mouse'), [
            'auxclick', 'click', 'dblclick', 'mousedown', 'mouseup', 'mouseover', 'mousemove', 'mouseout', 'mouseenter',
            'mouseleave', 'mousewheel', 'wheel', 'contextmenu'
        ], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Pointer'), [
            'pointerover', 'pointerout', 'pointerenter', 'pointerleave', 'pointerdown', 'pointerup', 'pointermove',
            'pointercancel', 'gotpointercapture', 'lostpointercapture', 'pointerrawupdate'
        ], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Touch'), ['touchstart', 'touchmove', 'touchend', 'touchcancel'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('Worker'), ['message', 'messageerror'], ['*']);
        this._createEventListenerBreakpoints(UIString.UIString('XHR'), ['readystatechange', 'load', 'loadstart', 'loadend', 'abort', 'error', 'progress', 'timeout'], ['xmlhttprequest', 'xmlhttprequestupload']);
        let breakpoint;
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:setTimeout.callback');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('setTimeout fired');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:setInterval.callback');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('setInterval fired');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:scriptFirstStatement');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Script First Statement');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:scriptBlockedByCSP');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Script Blocked by Content Security Policy');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:requestAnimationFrame');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Request Animation Frame');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:cancelAnimationFrame');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Cancel Animation Frame');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:requestAnimationFrame.callback');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Animation Frame Fired');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:webglErrorFired');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('WebGL Error Fired');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:webglWarningFired');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('WebGL Warning Fired');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:Element.setInnerHTML');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Set innerHTML');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:canvasContextCreated');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Create canvas context');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:Geolocation.getCurrentPosition');
        if (breakpoint) {
            breakpoint._title = 'getCurrentPosition';
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:Geolocation.watchPosition');
        if (breakpoint) {
            breakpoint._title = 'watchPosition';
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:Notification.requestPermission');
        if (breakpoint) {
            breakpoint._title = 'requestPermission';
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:DOMWindow.close');
        if (breakpoint) {
            breakpoint._title = 'window.close';
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:Document.write');
        if (breakpoint) {
            breakpoint._title = 'document.write';
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:audioContextCreated');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Create AudioContext');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:audioContextClosed');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Close AudioContext');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:audioContextResumed');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Resume AudioContext');
        }
        breakpoint = this._resolveEventListenerBreakpoint('instrumentation:audioContextSuspended');
        if (breakpoint) {
            breakpoint._title = UIString.UIString('Suspend AudioContext');
        }
        TargetManager.instance().observeModels(DOMDebuggerModel, this);
    }
    /**
     * @param {{forceNew: ?boolean}} opts
     * @return {!DOMDebuggerManager}
     */
    static instance(opts = { forceNew: null }) {
        const { forceNew } = opts;
        if (!domDebuggerManagerInstance || forceNew) {
            domDebuggerManagerInstance = new DOMDebuggerManager();
        }
        return domDebuggerManagerInstance;
    }
    /**
     * @param {string} category
     * @param {!Array<string>} instrumentationNames
     */
    _createInstrumentationBreakpoints(category, instrumentationNames) {
        for (const instrumentationName of instrumentationNames) {
            this._eventListenerBreakpoints.push(new EventListenerBreakpoint(instrumentationName, '', [], category, instrumentationName));
        }
    }
    /**
     * @param {string} category
     * @param {!Array<string>} eventNames
     * @param {!Array<string>} eventTargetNames
     */
    _createEventListenerBreakpoints(category, eventNames, eventTargetNames) {
        for (const eventName of eventNames) {
            this._eventListenerBreakpoints.push(new EventListenerBreakpoint('', eventName, eventTargetNames, category, eventName));
        }
    }
    /**
     * @param {string} eventName
     * @param {string=} eventTargetName
     * @return {?EventListenerBreakpoint}
     */
    _resolveEventListenerBreakpoint(eventName, eventTargetName) {
        const instrumentationPrefix = 'instrumentation:';
        const listenerPrefix = 'listener:';
        let instrumentationName = '';
        if (eventName.startsWith(instrumentationPrefix)) {
            instrumentationName = eventName.substring(instrumentationPrefix.length);
            eventName = '';
        }
        else if (eventName.startsWith(listenerPrefix)) {
            eventName = eventName.substring(listenerPrefix.length);
        }
        else {
            return null;
        }
        eventTargetName = (eventTargetName || '*').toLowerCase();
        let result = null;
        for (const breakpoint of this._eventListenerBreakpoints) {
            if (instrumentationName && breakpoint._instrumentationName === instrumentationName) {
                result = breakpoint;
            }
            if (eventName && breakpoint._eventName === eventName &&
                breakpoint._eventTargetNames.indexOf(eventTargetName) !== -1) {
                result = breakpoint;
            }
            if (!result && eventName && breakpoint._eventName === eventName &&
                breakpoint._eventTargetNames.indexOf('*') !== -1) {
                result = breakpoint;
            }
        }
        return result;
    }
    /**
     * @return {!Array<!EventListenerBreakpoint>}
     */
    eventListenerBreakpoints() {
        return this._eventListenerBreakpoints.slice();
    }
    /**
     * @param {!{eventName: string, webglErrorName: string, directiveText: string, targetName: string}} auxData
     * @return {string}
     */
    resolveEventListenerBreakpointTitle(auxData) {
        const id = auxData['eventName'];
        if (id === 'instrumentation:webglErrorFired' && auxData['webglErrorName']) {
            let errorName = auxData['webglErrorName'];
            // If there is a hex code of the error, display only this.
            errorName = errorName.replace(/^.*(0x[0-9a-f]+).*$/i, '$1');
            return UIString.UIString('WebGL Error Fired (%s)', errorName);
        }
        if (id === 'instrumentation:scriptBlockedByCSP' && auxData['directiveText']) {
            return UIString.UIString('Script blocked due to Content Security Policy directive: %s', auxData['directiveText']);
        }
        const breakpoint = this._resolveEventListenerBreakpoint(id, auxData['targetName']);
        if (!breakpoint) {
            return '';
        }
        if (auxData['targetName']) {
            return auxData['targetName'] + '.' + breakpoint._title;
        }
        return breakpoint._title;
    }
    /**
     * @param {!{eventName: string, targetName: string}} auxData
     * @return {?EventListenerBreakpoint}
     */
    resolveEventListenerBreakpoint(auxData) {
        return this._resolveEventListenerBreakpoint(auxData['eventName'], auxData['targetName']);
    }
    /**
     * @return {!Map<string, boolean>}
     */
    xhrBreakpoints() {
        return this._xhrBreakpoints;
    }
    _saveXHRBreakpoints() {
        const breakpoints = [];
        for (const url of this._xhrBreakpoints.keys()) {
            breakpoints.push({ url: url, enabled: this._xhrBreakpoints.get(url) });
        }
        this._xhrBreakpointsSetting.set(breakpoints);
    }
    /**
     * @param {string} url
     * @param {boolean} enabled
     */
    addXHRBreakpoint(url, enabled) {
        this._xhrBreakpoints.set(url, enabled);
        if (enabled) {
            for (const model of TargetManager.instance().models(DOMDebuggerModel)) {
                model._agent.setXHRBreakpoint(url);
            }
        }
        this._saveXHRBreakpoints();
    }
    /**
     * @param {string} url
     */
    removeXHRBreakpoint(url) {
        const enabled = this._xhrBreakpoints.get(url);
        this._xhrBreakpoints.delete(url);
        if (enabled) {
            for (const model of TargetManager.instance().models(DOMDebuggerModel)) {
                model._agent.removeXHRBreakpoint(url);
            }
        }
        this._saveXHRBreakpoints();
    }
    /**
     * @param {string} url
     * @param {boolean} enabled
     */
    toggleXHRBreakpoint(url, enabled) {
        this._xhrBreakpoints.set(url, enabled);
        for (const model of TargetManager.instance().models(DOMDebuggerModel)) {
            if (enabled) {
                model._agent.setXHRBreakpoint(url);
            }
            else {
                model._agent.removeXHRBreakpoint(url);
            }
        }
        this._saveXHRBreakpoints();
    }
    /**
     * @override
     * @param {!DOMDebuggerModel} domDebuggerModel
     */
    modelAdded(domDebuggerModel) {
        for (const url of this._xhrBreakpoints.keys()) {
            if (this._xhrBreakpoints.get(url)) {
                domDebuggerModel._agent.invoke_setXHRBreakpoint({ url: url });
            }
        }
        for (const breakpoint of this._eventListenerBreakpoints) {
            if (breakpoint._enabled) {
                breakpoint._updateOnModel(domDebuggerModel);
            }
        }
    }
    /**
     * @override
     * @param {!DOMDebuggerModel} domDebuggerModel
     */
    modelRemoved(domDebuggerModel) {
    }
}
SDKModel.register(DOMDebuggerModel, Capability.DOM, false);

var DOMDebuggerModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    DOMDebuggerModel: DOMDebuggerModel,
    Events: Events$i,
    DOMBreakpoint: DOMBreakpoint,
    EventListener: EventListener,
    EventListenerBreakpoint: EventListenerBreakpoint,
    DOMDebuggerManager: DOMDebuggerManager
});

// Copyright 2017 The Chromium Authors. All rights reserved.
class EmulationModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._emulationAgent = target.emulationAgent();
        this._pageAgent = target.pageAgent();
        this._deviceOrientationAgent = target.deviceOrientationAgent();
        this._cssModel = target.model(CSSModel);
        this._overlayModel = target.model(OverlayModel);
        if (this._overlayModel) {
            this._overlayModel.addEventListener(Events$7.InspectModeWillBeToggled, this._updateTouch, this);
        }
        const disableJavascriptSetting = Settings.Settings.instance().moduleSetting('javaScriptDisabled');
        disableJavascriptSetting.addChangeListener(() => this._emulationAgent.setScriptExecutionDisabled(disableJavascriptSetting.get()));
        if (disableJavascriptSetting.get()) {
            this._emulationAgent.setScriptExecutionDisabled(true);
        }
        const touchSetting = Settings.Settings.instance().moduleSetting('emulation.touch');
        touchSetting.addChangeListener(() => {
            const settingValue = touchSetting.get();
            this.overrideEmulateTouch(settingValue === 'force');
        });
        const idleDetectionSetting = Settings.Settings.instance().moduleSetting('emulation.idleDetection');
        idleDetectionSetting.addChangeListener(async () => {
            const settingValue = idleDetectionSetting.get();
            if (settingValue === 'none') {
                await this.clearIdleOverride();
                return;
            }
            const emulationParams = 
            /** @type {{isUserActive: boolean, isScreenUnlocked: boolean}} */ (JSON.parse(settingValue));
            await this.setIdleOverride(emulationParams);
        });
        const mediaTypeSetting = Settings.Settings.instance().moduleSetting('emulatedCSSMedia');
        const mediaFeaturePrefersColorSchemeSetting = Settings.Settings.instance().moduleSetting('emulatedCSSMediaFeaturePrefersColorScheme');
        const mediaFeaturePrefersReducedMotionSetting = Settings.Settings.instance().moduleSetting('emulatedCSSMediaFeaturePrefersReducedMotion');
        const mediaFeaturePrefersReducedDataSetting = Settings.Settings.instance().moduleSetting('emulatedCSSMediaFeaturePrefersReducedData');
        // Note: this uses a different format than what the CDP API expects,
        // because we want to update these values per media type/feature
        // without having to search the `features` array (inefficient) or
        // hardcoding the indices (not readable/maintainable).
        this._mediaConfiguration = new Map([
            ['type', mediaTypeSetting.get()],
            ['prefers-color-scheme', mediaFeaturePrefersColorSchemeSetting.get()],
            ['prefers-reduced-motion', mediaFeaturePrefersReducedMotionSetting.get()],
            ['prefers-reduced-data', mediaFeaturePrefersReducedDataSetting.get()],
        ]);
        mediaTypeSetting.addChangeListener(() => {
            this._mediaConfiguration.set('type', mediaTypeSetting.get());
            this._updateCssMedia();
        });
        mediaFeaturePrefersColorSchemeSetting.addChangeListener(() => {
            this._mediaConfiguration.set('prefers-color-scheme', mediaFeaturePrefersColorSchemeSetting.get());
            this._updateCssMedia();
        });
        mediaFeaturePrefersReducedMotionSetting.addChangeListener(() => {
            this._mediaConfiguration.set('prefers-reduced-motion', mediaFeaturePrefersReducedMotionSetting.get());
            this._updateCssMedia();
        });
        mediaFeaturePrefersReducedDataSetting.addChangeListener(() => {
            this._mediaConfiguration.set('prefers-reduced-data', mediaFeaturePrefersReducedDataSetting.get());
            this._updateCssMedia();
        });
        this._updateCssMedia();
        const visionDeficiencySetting = Settings.Settings.instance().moduleSetting('emulatedVisionDeficiency');
        visionDeficiencySetting.addChangeListener(() => this._emulateVisionDeficiency(visionDeficiencySetting.get()));
        if (visionDeficiencySetting.get()) {
            this._emulateVisionDeficiency(visionDeficiencySetting.get());
        }
        const localFontsDisabledSetting = Settings.Settings.instance().moduleSetting('localFontsDisabled');
        localFontsDisabledSetting.addChangeListener(() => this._setLocalFontsDisabled(localFontsDisabledSetting.get()));
        if (localFontsDisabledSetting.get()) {
            this._setLocalFontsDisabled(localFontsDisabledSetting.get());
        }
        this._touchEnabled = false;
        this._touchMobile = false;
        this._customTouchEnabled = false;
        this._touchConfiguration = {
            enabled: false,
            configuration: Protocol.Emulation.SetEmitTouchEventsForMouseRequestConfiguration.Mobile,
            scriptId: ''
        };
    }
    /**
     * @return {boolean}
     */
    supportsDeviceEmulation() {
        return this.target().hasAllCapabilities(Capability.DeviceEmulation);
    }
    /**
     * @return {!Promise<?>}
     */
    resetPageScaleFactor() {
        return this._emulationAgent.resetPageScaleFactor();
    }
    /**
     * @param {?Protocol.Page.SetDeviceMetricsOverrideRequest} metrics
     * @return {!Promise<?>}
     */
    emulateDevice(metrics) {
        if (metrics) {
            return this._emulationAgent.invoke_setDeviceMetricsOverride(metrics);
        }
        return this._emulationAgent.clearDeviceMetricsOverride();
    }
    /**
     * @return {?OverlayModel}
     */
    overlayModel() {
        return this._overlayModel;
    }
    /**
     * @param {?Location} location
     */
    async emulateLocation(location) {
        if (!location) {
            this._emulationAgent.clearGeolocationOverride();
            this._emulationAgent.setTimezoneOverride('');
            this._emulationAgent.setLocaleOverride('');
            this._emulationAgent.setUserAgentOverride(SDK.multitargetNetworkManager.currentUserAgent());
        }
        if (location.error) {
            this._emulationAgent.setGeolocationOverride();
            this._emulationAgent.setTimezoneOverride('');
            this._emulationAgent.setLocaleOverride('');
            this._emulationAgent.setUserAgentOverride(SDK.multitargetNetworkManager.currentUserAgent());
        }
        else {
            const processEmulationResult = (errorType, result) => {
                const errorMessage = result.getError();
                if (errorMessage) {
                    return Promise.reject({
                        type: errorType,
                        message: errorMessage,
                    });
                }
                return Promise.resolve(result);
            };
            return Promise.all([
                this._emulationAgent
                    .invoke_setGeolocationOverride({
                    latitude: location.latitude,
                    longitude: location.longitude,
                    accuracy: Location$1.DefaultGeoMockAccuracy,
                })
                    .then(result => processEmulationResult('emulation-set-location', result)),
                this._emulationAgent
                    .invoke_setTimezoneOverride({
                    timezoneId: location.timezoneId,
                })
                    .then(result => processEmulationResult('emulation-set-timezone', result)),
                this._emulationAgent
                    .invoke_setLocaleOverride({
                    locale: location.locale,
                })
                    .then(result => processEmulationResult('emulation-set-locale', result)),
                this._emulationAgent
                    .invoke_setUserAgentOverride({
                    userAgent: SDK.multitargetNetworkManager.currentUserAgent(),
                    acceptLanguage: location.locale,
                })
                    .then(result => processEmulationResult('emulation-set-user-agent', result)),
            ]);
        }
    }
    /**
     * @param {?DeviceOrientation} deviceOrientation
     */
    emulateDeviceOrientation(deviceOrientation) {
        if (deviceOrientation) {
            this._deviceOrientationAgent.setDeviceOrientationOverride(deviceOrientation.alpha, deviceOrientation.beta, deviceOrientation.gamma);
        }
        else {
            this._deviceOrientationAgent.clearDeviceOrientationOverride();
        }
    }
    /**
     * @param {{isUserActive: boolean, isScreenUnlocked: boolean}} emulationParams
     */
    async setIdleOverride(emulationParams) {
        await this._emulationAgent.invoke_setIdleOverride(emulationParams);
    }
    async clearIdleOverride() {
        await this._emulationAgent.invoke_clearIdleOverride();
    }
    /**
     * @param {string} type
     * @param {!Array<{name: string, value: string}>} features
     */
    _emulateCSSMedia(type, features) {
        this._emulationAgent.setEmulatedMedia(type, features);
        if (this._cssModel) {
            this._cssModel.mediaQueryResultChanged();
        }
    }
    /**
     * @param {!Protocol.Emulation.SetEmulatedVisionDeficiencyRequestType} type
     */
    _emulateVisionDeficiency(type) {
        this._emulationAgent.setEmulatedVisionDeficiency(type);
    }
    _setLocalFontsDisabled(disabled) {
        this._cssModel.setLocalFontsEnabled(!disabled);
    }
    /**
     * @param {number} rate
     */
    setCPUThrottlingRate(rate) {
        this._emulationAgent.setCPUThrottlingRate(rate);
    }
    /**
     * @param {boolean} enabled
     * @param {boolean} mobile
     */
    emulateTouch(enabled, mobile) {
        this._touchEnabled = enabled;
        this._touchMobile = mobile;
        this._updateTouch();
    }
    /**
     * @param {boolean} enabled
     */
    overrideEmulateTouch(enabled) {
        this._customTouchEnabled = enabled;
        this._updateTouch();
    }
    _updateTouch() {
        let configuration = {
            enabled: this._touchEnabled,
            configuration: this._touchMobile ? Protocol.Emulation.SetEmitTouchEventsForMouseRequestConfiguration.Mobile :
                Protocol.Emulation.SetEmitTouchEventsForMouseRequestConfiguration.Desktop,
        };
        if (this._customTouchEnabled) {
            configuration = {
                enabled: true,
                configuration: Protocol.Emulation.SetEmitTouchEventsForMouseRequestConfiguration.Mobile
            };
        }
        if (this._overlayModel && this._overlayModel.inspectModeEnabled()) {
            configuration = {
                enabled: false,
                configuration: Protocol.Emulation.SetEmitTouchEventsForMouseRequestConfiguration.Mobile
            };
        }
        if (!this._touchConfiguration.enabled && !configuration.enabled) {
            return;
        }
        if (this._touchConfiguration.enabled && configuration.enabled &&
            this._touchConfiguration.configuration === configuration.configuration) {
            return;
        }
        this._touchConfiguration = configuration;
        this._emulationAgent.setTouchEmulationEnabled(configuration.enabled, 1);
        this._emulationAgent.setEmitTouchEventsForMouse(configuration.enabled, configuration.configuration);
    }
    _updateCssMedia() {
        // See the note above, where this._mediaConfiguration is defined.
        const type = this._mediaConfiguration.get('type');
        const features = [
            {
                name: 'prefers-color-scheme',
                value: this._mediaConfiguration.get('prefers-color-scheme'),
            },
            {
                name: 'prefers-reduced-motion',
                value: this._mediaConfiguration.get('prefers-reduced-motion'),
            },
            {
                name: 'prefers-reduced-data',
                value: this._mediaConfiguration.get('prefers-reduced-data'),
            },
        ];
        this._emulateCSSMedia(type, features);
    }
}
class Location$1 {
    /**
     * @param {number} latitude
     * @param {number} longitude
     * @param {string} timezoneId
     * @param {string} locale
     * @param {boolean} error
     */
    constructor(latitude, longitude, timezoneId, locale, error) {
        this.latitude = latitude;
        this.longitude = longitude;
        this.timezoneId = timezoneId;
        this.locale = locale;
        this.error = error;
    }
    /**
     * @return {!Location}
     */
    static parseSetting(value) {
        if (value) {
            const [position, timezoneId, locale, error] = value.split(':');
            const [latitude, longitude] = position.split('@');
            return new Location$1(parseFloat(latitude), parseFloat(longitude), timezoneId, locale, Boolean(error));
        }
        return new Location$1(0, 0, '', '', false);
    }
    /**
     * @param {string} latitudeString
     * @param {string} longitudeString
     * @param {string} timezoneId
     * @return {?Location}
     */
    static parseUserInput(latitudeString, longitudeString, timezoneId, locale) {
        if (!latitudeString && !longitudeString) {
            return null;
        }
        const { valid: isLatitudeValid } = Location$1.latitudeValidator(latitudeString);
        const { valid: isLongitudeValid } = Location$1.longitudeValidator(longitudeString);
        if (!isLatitudeValid && !isLongitudeValid) {
            return null;
        }
        const latitude = isLatitudeValid ? parseFloat(latitudeString) : -1;
        const longitude = isLongitudeValid ? parseFloat(longitudeString) : -1;
        return new Location$1(latitude, longitude, timezoneId, locale, false);
    }
    /**
     * @param {string} value
     * @return {{valid: boolean, errorMessage: (string|undefined)}}
     */
    static latitudeValidator(value) {
        const numValue = parseFloat(value);
        const valid = /^([+-]?[\d]+(\.\d+)?|[+-]?\.\d+)$/.test(value) && numValue >= -90 && numValue <= 90;
        return { valid };
    }
    /**
     * @param {string} value
     * @return {{valid: boolean, errorMessage: (string|undefined)}}
     */
    static longitudeValidator(value) {
        const numValue = parseFloat(value);
        const valid = /^([+-]?[\d]+(\.\d+)?|[+-]?\.\d+)$/.test(value) && numValue >= -180 && numValue <= 180;
        return { valid };
    }
    /**
     * @param {string} value
     * @return {{valid: boolean, errorMessage: (string|undefined)}}
     */
    static timezoneIdValidator(value) {
        // Chromium uses ICU's timezone implementation, which is very
        // liberal in what it accepts. ICU does not simply use an allowlist
        // but instead tries to make sense of the input, even for
        // weird-looking timezone IDs. There's not much point in validating
        // the input other than checking if it contains at least one alphabet.
        // The empty string resets the override, and is accepted as well.
        const valid = value === '' || /[a-zA-Z]/.test(value);
        return { valid };
    }
    /**
     * @param {string} value
     * @return {{valid: boolean, errorMessage: (string|undefined)}}
     */
    static localeValidator(value) {
        // Similarly to timezone IDs, there's not much point in validating
        // input locales other than checking if it contains at least two
        // alphabetic characters.
        // https://unicode.org/reports/tr35/#Unicode_language_identifier
        // The empty string resets the override, and is accepted as
        // well.
        const valid = value === '' || /[a-zA-Z]{2}/.test(value);
        return { valid };
    }
    /**
     * @return {string}
     */
    toSetting() {
        return `${this.latitude}@${this.longitude}:${this.timezoneId}:${this.locale}:${this.error || ''}`;
    }
}
Location$1.DefaultGeoMockAccuracy = 150;
class DeviceOrientation {
    /**
     * @param {number} alpha
     * @param {number} beta
     * @param {number} gamma
     */
    constructor(alpha, beta, gamma) {
        this.alpha = alpha;
        this.beta = beta;
        this.gamma = gamma;
    }
    /**
     * @return {!DeviceOrientation}
     */
    static parseSetting(value) {
        if (value) {
            const jsonObject = JSON.parse(value);
            return new DeviceOrientation(jsonObject.alpha, jsonObject.beta, jsonObject.gamma);
        }
        return new DeviceOrientation(0, 0, 0);
    }
    /**
     * @return {?DeviceOrientation}
     */
    static parseUserInput(alphaString, betaString, gammaString) {
        if (!alphaString && !betaString && !gammaString) {
            return null;
        }
        const { valid: isAlphaValid } = DeviceOrientation.validator(alphaString);
        const { valid: isBetaValid } = DeviceOrientation.validator(betaString);
        const { valid: isGammaValid } = DeviceOrientation.validator(gammaString);
        if (!isAlphaValid && !isBetaValid && !isGammaValid) {
            return null;
        }
        const alpha = isAlphaValid ? parseFloat(alphaString) : -1;
        const beta = isBetaValid ? parseFloat(betaString) : -1;
        const gamma = isGammaValid ? parseFloat(gammaString) : -1;
        return new DeviceOrientation(alpha, beta, gamma);
    }
    /**
     * @param {string} value
     * @return {{valid: boolean, errorMessage: (string|undefined)}}
     */
    static validator(value) {
        const valid = /^([+-]?[\d]+(\.\d+)?|[+-]?\.\d+)$/.test(value);
        return { valid };
    }
    /**
     * @return {string}
     */
    toSetting() {
        return JSON.stringify(this);
    }
}
SDKModel.register(EmulationModel, Capability.Emulation, true);

var EmulationModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    EmulationModel: EmulationModel,
    Location: Location$1,
    DeviceOrientation: DeviceOrientation
});

// Copyright 2014 The Chromium Authors. All rights reserved.
class TracingModel {
    /**
     * @param {!BackingStorage} backingStorage
     */
    constructor(backingStorage) {
        this._backingStorage = backingStorage;
        // Avoid extra reset of the storage as it's expensive.
        this._firstWritePending = true;
        /** @type {!Map<(number|string), !Process>} */
        this._processById = new Map();
        this._processByName = new Map();
        this._minimumRecordTime = 0;
        /** @type {number} */
        this._maximumRecordTime = 0;
        /** @type {!Array<!Event>} */
        this._devToolsMetadataEvents = [];
        /** @type {!Array<!AsyncEvent>} */
        this._asyncEvents = [];
        /** @type {!Map<string, !AsyncEvent>} */
        this._openAsyncEvents = new Map();
        /** @type {!Map<string, !Array<!AsyncEvent>>} */
        this._openNestableAsyncEvents = new Map();
        /** @type {!Map<string, !ProfileEventsGroup>} */
        this._profileGroups = new Map();
        /** @type {!Map<string, !Set<string>>} */
        this._parsedCategories = new Map();
        /** @type {!Map<string, !Event>} */
        this._mainFrameNavStartTimes = new Map();
    }
    /**
     * @param {string} phase
     * @return {boolean}
     */
    static isNestableAsyncPhase(phase) {
        return phase === 'b' || phase === 'e' || phase === 'n';
    }
    /**
     * @param {string} phase
     * @return {boolean}
     */
    static isAsyncBeginPhase(phase) {
        return phase === 'S' || phase === 'b';
    }
    /**
     * @param {string} phase
     * @return {boolean}
     */
    static isAsyncPhase(phase) {
        return TracingModel.isNestableAsyncPhase(phase) || phase === 'S' || phase === 'T' || phase === 'F' || phase === 'p';
    }
    /**
     * @param {string} phase
     * @return {boolean}
     */
    static isFlowPhase(phase) {
        return phase === 's' || phase === 't' || phase === 'f';
    }
    /**
     * @param {!Event} event
     * @return {boolean}
     */
    static isTopLevelEvent(event) {
        return event.hasCategory(DevToolsTimelineEventCategory) && event.name === 'RunTask' ||
            event.hasCategory(LegacyTopLevelEventCategory) ||
            event.hasCategory(DevToolsMetadataEventCategory) &&
                event.name === 'Program'; // Older timelines may have this instead of toplevel.
    }
    /**
     * @param {!EventPayload} payload
     * @return {string|undefined}
     */
    static _extractId(payload) {
        const scope = payload.scope || '';
        if (typeof payload.id2 === 'undefined') {
            return scope && payload.id ? `${scope}@${payload.id}` : payload.id;
        }
        const id2 = payload.id2;
        if (typeof id2 === 'object' && ('global' in id2) !== ('local' in id2)) {
            return typeof id2['global'] !== 'undefined' ? `:${scope}:${id2['global']}` :
                `:${scope}:${payload.pid}:${id2['local']}`;
        }
        console.error(`Unexpected id2 field at ${payload.ts / 1000}, one and only one of 'local' and 'global' should be present.`);
        return undefined;
    }
    /**
     * @param {!TracingModel} tracingModel
     * @return {?Thread}
     *
     * TODO: Move this to a better place. This is here just for convenience o
     * re-use between modules. This really belongs to a higher level, since it
     * is specific to chrome's usage of tracing.
     */
    static browserMainThread(tracingModel) {
        const processes = tracingModel.sortedProcesses();
        // Avoid warning for an empty model.
        if (!processes.length) {
            return null;
        }
        const browserMainThreadName = 'CrBrowserMain';
        const browserProcesses = [];
        const browserMainThreads = [];
        for (const process of processes) {
            if (process.name().toLowerCase().endsWith('browser')) {
                browserProcesses.push(process);
            }
            browserMainThreads.push(...process.sortedThreads().filter(t => t.name() === browserMainThreadName));
        }
        if (browserMainThreads.length === 1) {
            return browserMainThreads[0];
        }
        if (browserProcesses.length === 1) {
            return browserProcesses[0].threadByName(browserMainThreadName);
        }
        const tracingStartedInBrowser = tracingModel.devToolsMetadataEvents().filter(e => e.name === 'TracingStartedInBrowser');
        if (tracingStartedInBrowser.length === 1) {
            return tracingStartedInBrowser[0].thread;
        }
        Console.Console.instance().error('Failed to find browser main thread in trace, some timeline features may be unavailable');
        return null;
    }
    /**
     * @return {!Array<!Event>}
     */
    devToolsMetadataEvents() {
        return this._devToolsMetadataEvents;
    }
    /**
     * @param {!Array<!EventPayload>} events
     */
    addEvents(events) {
        for (let i = 0; i < events.length; ++i) {
            this._addEvent(events[i]);
        }
    }
    tracingComplete() {
        this._processPendingAsyncEvents();
        this._backingStorage.appendString(this._firstWritePending ? '[]' : ']');
        this._backingStorage.finishWriting();
        this._firstWritePending = false;
        for (const process of this._processById.values()) {
            for (const thread of process._threads.values()) {
                thread.tracingComplete();
            }
        }
    }
    dispose() {
        if (!this._firstWritePending) {
            this._backingStorage.reset();
        }
    }
    /**
     * @param {number} offset
     */
    adjustTime(offset) {
        this._minimumRecordTime += offset;
        this._maximumRecordTime += offset;
        for (const process of this._processById.values()) {
            for (const thread of process._threads.values()) {
                for (const event of thread.events()) {
                    event.startTime += offset;
                    if (typeof event.endTime === 'number') {
                        event.endTime += offset;
                    }
                }
                for (const event of thread.asyncEvents()) {
                    event.startTime += offset;
                    if (typeof event.endTime === 'number') {
                        event.endTime += offset;
                    }
                }
            }
        }
    }
    /**
     * @param {!EventPayload} payload
     */
    _addEvent(payload) {
        let process = this._processById.get(payload.pid);
        if (!process) {
            process = new Process(this, payload.pid);
            this._processById.set(payload.pid, process);
        }
        const phase = Phase;
        const eventsDelimiter = ',\n';
        this._backingStorage.appendString(this._firstWritePending ? '[' : eventsDelimiter);
        this._firstWritePending = false;
        const stringPayload = JSON.stringify(payload);
        const isAccessible = payload.ph === phase.SnapshotObject;
        let backingStorage = null;
        const keepStringsLessThan = 10000;
        if (isAccessible && stringPayload.length > keepStringsLessThan) {
            backingStorage = this._backingStorage.appendAccessibleString(stringPayload);
        }
        else {
            this._backingStorage.appendString(stringPayload);
        }
        const timestamp = payload.ts / 1000;
        // We do allow records for unrelated threads to arrive out-of-order,
        // so there's a chance we're getting records from the past.
        if (timestamp && (!this._minimumRecordTime || timestamp < this._minimumRecordTime) &&
            (payload.ph === phase.Begin || payload.ph === phase.Complete || payload.ph === phase.Instant)) {
            this._minimumRecordTime = timestamp;
        }
        // Track only main thread navigation start items. This is done by tracking isLoadingMainFrame,
        // and whether documentLoaderURL is set.
        if (payload.name === 'navigationStart') {
            const data = /** @type {?{isLoadingMainFrame: boolean, documentLoaderURL: string, navigationId: string}} */ (payload.args.data);
            if (data) {
                const { isLoadingMainFrame, documentLoaderURL, navigationId } = data;
                if (isLoadingMainFrame && documentLoaderURL !== '') {
                    const thread = process.threadById(payload.tid);
                    const navStartEvent = Event.fromPayload(payload, thread);
                    this._mainFrameNavStartTimes.set(navigationId, navStartEvent);
                }
            }
        }
        const endTimeStamp = (payload.ts + (payload.dur || 0)) / 1000;
        this._maximumRecordTime = Math.max(this._maximumRecordTime, endTimeStamp);
        const event = process._addEvent(payload);
        if (!event) {
            return;
        }
        if (payload.ph === phase.Sample) {
            this._addSampleEvent(event);
            return;
        }
        // Build async event when we've got events from all threads & processes, so we can sort them and process in the
        // chronological order. However, also add individual async events to the thread flow (above), so we can easily
        // display them on the same chart as other events, should we choose so.
        if (TracingModel.isAsyncPhase(payload.ph)) {
            this._asyncEvents.push(/** @type {!AsyncEvent} */ (event));
        }
        event._setBackingStorage(backingStorage);
        if (event.hasCategory(DevToolsMetadataEventCategory)) {
            this._devToolsMetadataEvents.push(event);
        }
        if (payload.ph !== phase.Metadata) {
            return;
        }
        switch (payload.name) {
            case MetadataEvent.ProcessSortIndex: {
                process._setSortIndex(payload.args['sort_index']);
                break;
            }
            case MetadataEvent.ProcessName: {
                const processName = payload.args['name'];
                process._setName(processName);
                this._processByName.set(processName, process);
                break;
            }
            case MetadataEvent.ThreadSortIndex: {
                process.threadById(payload.tid)._setSortIndex(payload.args['sort_index']);
                break;
            }
            case MetadataEvent.ThreadName: {
                process.threadById(payload.tid)._setName(payload.args['name']);
                break;
            }
        }
    }
    /**
     * @param {!Event} event
     */
    _addSampleEvent(event) {
        const id = `${event.thread.process().id()}:${event.id}`;
        const group = this._profileGroups.get(id);
        if (group) {
            group._addChild(event);
        }
        else {
            this._profileGroups.set(id, new ProfileEventsGroup(event));
        }
    }
    /**
     * @param {!Event} event
     * @return {?ProfileEventsGroup}
     */
    profileGroup(event) {
        return this._profileGroups.get(`${event.thread.process().id()}:${event.id}`) || null;
    }
    /**
     * @return {number}
     */
    minimumRecordTime() {
        return this._minimumRecordTime;
    }
    /**
     * @return {number}
     */
    maximumRecordTime() {
        return this._maximumRecordTime;
    }
    /**
     * @return {!Map<string, !Event>}
     */
    navStartTimes() {
        return this._mainFrameNavStartTimes;
    }
    /**
     * @return {!Array<!Process>}
     */
    sortedProcesses() {
        return Sorter.sort([...this._processById.values()]);
    }
    /**
     * @param {string} name
     * @return {?Process}
     */
    processByName(name) {
        return this._processByName.get(name);
    }
    /**
     * @param {number} pid
     * @return {?Process}
     */
    processById(pid) {
        return this._processById.get(pid) || null;
    }
    /**
     * @param {string} processName
     * @param {string} threadName
     * @return {?Thread}
     */
    threadByName(processName, threadName) {
        const process = this.processByName(processName);
        return process && process.threadByName(threadName);
    }
    /**
     * @param {string} processName
     * @param {string} threadName
     * @param {string} eventName
     * @return {!Array<!Event>}
     */
    extractEventsFromThreadByName(processName, threadName, eventName) {
        const thread = this.threadByName(processName, threadName);
        if (!thread) {
            return [];
        }
        return thread.removeEventsByName(eventName);
    }
    _processPendingAsyncEvents() {
        this._asyncEvents.sort(Event.compareStartTime);
        for (let i = 0; i < this._asyncEvents.length; ++i) {
            const event = this._asyncEvents[i];
            if (TracingModel.isNestableAsyncPhase(event.phase)) {
                this._addNestableAsyncEvent(event);
            }
            else {
                this._addAsyncEvent(event);
            }
        }
        this._asyncEvents = [];
        this._closeOpenAsyncEvents();
    }
    _closeOpenAsyncEvents() {
        for (const event of this._openAsyncEvents.values()) {
            event.setEndTime(this._maximumRecordTime);
            // FIXME: remove this once we figure a better way to convert async console
            // events to sync [waterfall] timeline records.
            event.steps[0].setEndTime(this._maximumRecordTime);
        }
        this._openAsyncEvents.clear();
        for (const eventStack of this._openNestableAsyncEvents.values()) {
            while (eventStack.length) {
                const event = eventStack.pop();
                if (!event) {
                    continue;
                }
                event.setEndTime(this._maximumRecordTime);
            }
        }
        this._openNestableAsyncEvents.clear();
    }
    /**
     * @param {!Event} event
     */
    _addNestableAsyncEvent(event) {
        const phase = Phase;
        const key = event.categoriesString + '.' + event.id;
        let openEventsStack = this._openNestableAsyncEvents.get(key);
        switch (event.phase) {
            case phase.NestableAsyncBegin: {
                if (!openEventsStack) {
                    openEventsStack = [];
                    this._openNestableAsyncEvents.set(key, openEventsStack);
                }
                const asyncEvent = new AsyncEvent(event);
                openEventsStack.push(asyncEvent);
                event.thread._addAsyncEvent(asyncEvent);
                break;
            }
            case phase.NestableAsyncInstant: {
                if (openEventsStack && openEventsStack.length) {
                    const event = openEventsStack.peekLast();
                    if (event) {
                        event._addStep(event);
                    }
                }
                break;
            }
            case phase.NestableAsyncEnd: {
                if (!openEventsStack || !openEventsStack.length) {
                    break;
                }
                const top = openEventsStack.pop();
                if (!top) {
                    break;
                }
                if (top.name !== event.name) {
                    console.error(`Begin/end event mismatch for nestable async event, ${top.name} vs. ${event.name}, key: ${key}`);
                    break;
                }
                top._addStep(event);
            }
        }
    }
    /**
     * @param {!Event} event
     */
    _addAsyncEvent(event) {
        const phase = Phase;
        const key = event.categoriesString + '.' + event.name + '.' + event.id;
        let asyncEvent = this._openAsyncEvents.get(key);
        if (event.phase === phase.AsyncBegin) {
            if (asyncEvent) {
                console.error(`Event ${event.name} has already been started`);
                return;
            }
            asyncEvent = new AsyncEvent(event);
            this._openAsyncEvents.set(key, asyncEvent);
            event.thread._addAsyncEvent(asyncEvent);
            return;
        }
        if (!asyncEvent) {
            // Quietly ignore stray async events, we're probably too late for the start.
            return;
        }
        if (event.phase === phase.AsyncEnd) {
            asyncEvent._addStep(event);
            this._openAsyncEvents.delete(key);
            return;
        }
        if (event.phase === phase.AsyncStepInto || event.phase === phase.AsyncStepPast) {
            const lastStep = asyncEvent.steps.peekLast();
            if (lastStep && lastStep.phase !== phase.AsyncBegin && lastStep.phase !== event.phase) {
                console.assert(false, 'Async event step phase mismatch: ' + lastStep.phase + ' at ' + lastStep.startTime + ' vs. ' +
                    event.phase + ' at ' + event.startTime);
                return;
            }
            asyncEvent._addStep(event);
            return;
        }
        console.assert(false, 'Invalid async event phase');
    }
    /**
     * @return {!BackingStorage}
     */
    backingStorage() {
        return this._backingStorage;
    }
    /**
     * @param {string} str
     * @return {!Set<string>}
     */
    _parsedCategoriesForString(str) {
        let parsedCategories = this._parsedCategories.get(str);
        if (!parsedCategories) {
            parsedCategories = new Set(str ? str.split(',') : []);
            this._parsedCategories.set(str, parsedCategories);
        }
        return parsedCategories;
    }
}
/**
 * @enum {string}
 */
const Phase = {
    Begin: 'B',
    End: 'E',
    Complete: 'X',
    Instant: 'I',
    AsyncBegin: 'S',
    AsyncStepInto: 'T',
    AsyncStepPast: 'p',
    AsyncEnd: 'F',
    NestableAsyncBegin: 'b',
    NestableAsyncEnd: 'e',
    NestableAsyncInstant: 'n',
    FlowBegin: 's',
    FlowStep: 't',
    FlowEnd: 'f',
    Metadata: 'M',
    Counter: 'C',
    Sample: 'P',
    CreateObject: 'N',
    SnapshotObject: 'O',
    DeleteObject: 'D'
};
const MetadataEvent = {
    ProcessSortIndex: 'process_sort_index',
    ProcessName: 'process_name',
    ThreadSortIndex: 'thread_sort_index',
    ThreadName: 'thread_name'
};
// TODO(alph): LegacyTopLevelEventCategory is not recorded since M74 and used for loading
// legacy profiles. Drop at some point.
const LegacyTopLevelEventCategory = 'toplevel';
const DevToolsMetadataEventCategory = 'disabled-by-default-devtools.timeline';
const DevToolsTimelineEventCategory = 'disabled-by-default-devtools.timeline';
/**
 * @interface
 */
class BackingStorage {
    /**
     * @param {string} string
     */
    appendString(string) {
    }
    /**
     * @param {string} string
     * @return {function():!Promise<?string>}
     */
    appendAccessibleString(string) {
        throw new Error('Not implemented yet');
    }
    finishWriting() {
    }
    reset() { }
}
/** @typedef {*} */
let Args; // eslint-disable-line no-unused-vars
/**
 * @unrestricted
 */
class Event {
    /**
     * @param {string|undefined} categories
     * @param {string} name
     * @param {!Phase} phase
     * @param {number} startTime
     * @param {!Thread} thread
     */
    constructor(categories, name, phase, startTime, thread) {
        /** @type {string} */
        this.categoriesString = categories || '';
        /** @type {!Set<string>} */
        this._parsedCategories = thread._model._parsedCategoriesForString(this.categoriesString);
        /** @type {string} */
        this.name = name;
        /** @type {!Phase} */
        this.phase = phase;
        /** @type {number} */
        this.startTime = startTime;
        /** @type {!Thread} */
        this.thread = thread;
        /** @type {!Args} */
        this.args = {};
        /** @type {?string} */
        this.id;
        /** @type {?string} */
        this.bind_id;
        /** @type {number} */
        this.ordinal = 0;
        /** @type {number} */
        this.selfTime = 0;
    }
    /**
     * @this {null}
     * @param {!EventPayload} payload
     * @param {!Thread} thread
     * @return {!Event}
     */
    static fromPayload(payload, thread) {
        const event = new Event(payload.cat, payload.name, /** @type {!Phase} */ (payload.ph), payload.ts / 1000, thread);
        if (payload.args) {
            event.addArgs(payload.args);
        }
        if (typeof payload.dur === 'number') {
            event.setEndTime((payload.ts + payload.dur) / 1000);
        }
        const id = TracingModel._extractId(payload);
        if (typeof id !== 'undefined') {
            event.id = id;
        }
        if (payload.bind_id) {
            event.bind_id = payload.bind_id;
        }
        return event;
    }
    /**
     * @param {?Event} a
     * @param {?Event} b
     * @return {number}
     */
    static compareStartTime(a, b) {
        if (!a || !b) {
            return 0;
        }
        return a.startTime - b.startTime;
    }
    /**
     * @param {!Event} a
     * @param {!Event} b
     * @return {number}
     */
    static orderedCompareStartTime(a, b) {
        // Array.mergeOrdered coalesces objects if comparator returns 0.
        // To change this behavior this comparator return -1 in the case events
        // startTime's are equal, so both events got placed into the result array.
        return a.startTime - b.startTime || a.ordinal - b.ordinal || -1;
    }
    /**
     * @param {string} categoryName
     * @return {boolean}
     */
    hasCategory(categoryName) {
        return this._parsedCategories.has(categoryName);
    }
    /**
     * @param {number} endTime
     */
    setEndTime(endTime) {
        if (endTime < this.startTime) {
            console.assert(false, 'Event out of order: ' + this.name);
            return;
        }
        this.endTime = endTime;
        this.duration = endTime - this.startTime;
    }
    /**
     * @param {!Args} args
     */
    addArgs(args) {
        // Shallow copy args to avoid modifying original payload which may be saved to file.
        for (const name in args) {
            if (name in this.args) {
                console.error('Same argument name (' + name + ') is used for begin and end phases of ' + this.name);
            }
            /** @typedef {Object<string, string|number|ObjectSnapshot>} */
            let BlackboxArgs; // eslint-disable-line no-unused-vars
            /** @type {!BlackboxArgs} */ (this.args)[name] = /** @type {!BlackboxArgs} */ (args)[name];
        }
    }
    /**
     * @param {!Event} endEvent
     */
    _complete(endEvent) {
        if (endEvent.args) {
            this.addArgs(endEvent.args);
        }
        else {
            console.error('Missing mandatory event argument \'args\' at ' + endEvent.startTime);
        }
        this.setEndTime(endEvent.startTime);
    }
    /**
     * @param {?function():!Promise.<?string>} backingStorage
     */
    _setBackingStorage(backingStorage) {
    }
}
class ObjectSnapshot extends Event {
    /**
     * @param {string|undefined} category
     * @param {string} name
     * @param {number} startTime
     * @param {!Thread} thread
     */
    constructor(category, name, startTime, thread) {
        super(category, name, Phase.SnapshotObject, startTime, thread);
        /** @type {?function():!Promise<?string>} */
        this._backingStorage = null;
        /** @type {string} */
        this.id;
        /** @type {?Promise<?>} */
        this._objectPromise = null;
    }
    /**
     * @override
     * @this {null}
     * @param {!EventPayload} payload
     * @param {!Thread} thread
     * @return {!ObjectSnapshot}
     */
    static fromPayload(payload, thread) {
        const snapshot = new ObjectSnapshot(payload.cat, payload.name, payload.ts / 1000, thread);
        const id = TracingModel._extractId(payload);
        if (typeof id !== 'undefined') {
            snapshot.id = id;
        }
        if (!payload.args || !payload.args['snapshot']) {
            console.error('Missing mandatory \'snapshot\' argument at ' + payload.ts / 1000);
            return snapshot;
        }
        if (payload.args) {
            snapshot.addArgs(payload.args);
        }
        return snapshot;
    }
    /**
     * @param {function(?ObjectSnapshot):void} callback
     */
    requestObject(callback) {
        const snapshot = this.args['snapshot'];
        if (snapshot) {
            callback(/** @type {!ObjectSnapshot} */ (snapshot));
            return;
        }
        const storage = this._backingStorage;
        if (storage) {
            storage().then(onRead, callback.bind(null, null));
        }
        /**
         * @param {?string} result
         */
        function onRead(result) {
            if (!result) {
                callback(null);
                return;
            }
            try {
                const payload = JSON.parse(result);
                callback(payload['args']['snapshot']);
            }
            catch (e) {
                Console.Console.instance().error('Malformed event data in backing storage');
                callback(null);
            }
        }
    }
    /**
     * @return {!Promise<*>}
     */
    objectPromise() {
        if (!this._objectPromise) {
            this._objectPromise = new Promise(this.requestObject.bind(this));
        }
        return this._objectPromise;
    }
    /**
     * @override
     * @param {?function():!Promise.<?>} backingStorage
     */
    _setBackingStorage(backingStorage) {
        if (!backingStorage) {
            return;
        }
        this._backingStorage = backingStorage;
        this.args = {};
    }
}
/**
 * @unrestricted
 */
class AsyncEvent extends Event {
    /**
     * @param {!Event} startEvent
     */
    constructor(startEvent) {
        super(startEvent.categoriesString, startEvent.name, startEvent.phase, startEvent.startTime, startEvent.thread);
        this.addArgs(startEvent.args);
        this.steps = [startEvent];
    }
    /**
     * @param {!Event} event
     */
    _addStep(event) {
        this.steps.push(event);
        if (event.phase === Phase.AsyncEnd || event.phase === Phase.NestableAsyncEnd) {
            this.setEndTime(event.startTime);
            // FIXME: ideally, we shouldn't do this, but this makes the logic of converting
            // async console events to sync ones much simpler.
            this.steps[0].setEndTime(event.startTime);
        }
    }
}
class ProfileEventsGroup {
    /**
     * @param {!Event} event
     */
    constructor(event) {
        /** @type {!Array<!Event>} */
        this.children = [event];
    }
    /**
     * @param {!Event} event
     */
    _addChild(event) {
        this.children.push(event);
    }
}
/**
  @typedef {{
    _sortIndex: number,
    name: function():string,
  }}
 */
let Sortable; // eslint-disable-line no-unused-vars
class Sorter {
    /**
     * @template Item
     * @param {!Array<!Item>} array
     * @return {!Array<!Item>}
     */
    static sort(array) {
        /**
         * @param {!Sortable} a
         * @param {!Sortable} b
         */
        function comparator(a, b) {
            return a._sortIndex !== b._sortIndex ? a._sortIndex - b._sortIndex : a.name().localeCompare(b.name());
        }
        // TODO(crbug.com/1082064): In TS the definition would be `param {!Array<Item & Sortable>} array`.
        // But Closure compiler does not support that, therefore, a cast is needed.
        return ( /** @type {?} */(array)).sort(comparator);
    }
}
class NamedObject {
    /**
     * @param {!TracingModel} model
     * @param {number} id
     */
    constructor(model, id) {
        this._model = model;
        this._id = id;
        this._name = '';
        this._sortIndex = 0;
    }
    /**
     * @param {string} name
     */
    _setName(name) {
        this._name = name;
    }
    /**
     * @return {string}
     */
    name() {
        return this._name;
    }
    /**
     * @param {number} sortIndex
     */
    _setSortIndex(sortIndex) {
        this._sortIndex = sortIndex;
    }
}
class Process extends NamedObject {
    /**
     * @param {!TracingModel} model
     * @param {number} id
     */
    constructor(model, id) {
        super(model, id);
        /** @type {!Map<number, !Thread>} */
        this._threads = new Map();
        this._threadByName = new Map();
    }
    /**
     * @return {number}
     */
    id() {
        return this._id;
    }
    /**
     * @param {number} id
     * @return {!Thread}
     */
    threadById(id) {
        let thread = this._threads.get(id);
        if (!thread) {
            thread = new Thread(this, id);
            this._threads.set(id, thread);
        }
        return thread;
    }
    /**
     * @param {string} name
     * @return {?Thread}
     */
    threadByName(name) {
        return this._threadByName.get(name) || null;
    }
    /**
     * @param {string} name
     * @param {!Thread} thread
     */
    _setThreadByName(name, thread) {
        this._threadByName.set(name, thread);
    }
    /**
     * @param {!EventPayload} payload
     * @return {?Event} event
     */
    _addEvent(payload) {
        return this.threadById(payload.tid)._addEvent(payload);
    }
    /**
     * @return {!Array<!Thread>}
     */
    sortedThreads() {
        return Sorter.sort([...this._threads.values()]);
    }
}
class Thread extends NamedObject {
    /**
     * @param {!Process} process
     * @param {number} id
     */
    constructor(process, id) {
        super(process._model, id);
        this._process = process;
        /**
         * @type {!Array<!Event>};
         */
        this._events = [];
        /**
         * @type {!Array<!AsyncEvent>};
         */
        this._asyncEvents = [];
        this._lastTopLevelEvent = null;
    }
    tracingComplete() {
        this._asyncEvents.sort(Event.compareStartTime);
        this._events.sort(Event.compareStartTime);
        const phases = Phase;
        const stack = [];
        /** @type {!Set<number>} */
        const toDelete = new Set();
        for (let i = 0; i < this._events.length; ++i) {
            const e = this._events[i];
            e.ordinal = i;
            switch (e.phase) {
                case phases.End: {
                    toDelete.add(i); // Mark for removal.
                    // Quietly ignore unbalanced close events, they're legit (we could have missed start one).
                    if (!stack.length) {
                        continue;
                    }
                    const top = stack.pop();
                    if (!top) {
                        continue;
                    }
                    if (top.name !== e.name || top.categoriesString !== e.categoriesString) {
                        console.error('B/E events mismatch at ' + top.startTime + ' (' + top.name + ') vs. ' + e.startTime + ' (' + e.name +
                            ')');
                    }
                    else {
                        top._complete(e);
                    }
                    break;
                }
                case phases.Begin: {
                    stack.push(e);
                    break;
                }
            }
        }
        while (stack.length) {
            const event = stack.pop();
            if (event) {
                event.setEndTime(this._model.maximumRecordTime());
            }
        }
        this._events = this._events.filter((_, idx) => !toDelete.has(idx));
    }
    /**
     * @param {!EventPayload} payload
     * @return {?Event} event
     */
    _addEvent(payload) {
        const event = payload.ph === Phase.SnapshotObject ? ObjectSnapshot.fromPayload(payload, this) :
            Event.fromPayload(payload, this);
        if (TracingModel.isTopLevelEvent(event)) {
            // Discard nested "top-level" events.
            const lastTopLevelEvent = this._lastTopLevelEvent;
            if (lastTopLevelEvent && (lastTopLevelEvent.endTime || 0) > event.startTime) {
                return null;
            }
            this._lastTopLevelEvent = event;
        }
        this._events.push(event);
        return event;
    }
    /**
     * @param {!AsyncEvent} asyncEvent
     */
    _addAsyncEvent(asyncEvent) {
        this._asyncEvents.push(asyncEvent);
    }
    /**
     * @override
     * @param {string} name
     */
    _setName(name) {
        super._setName(name);
        this._process._setThreadByName(name, this);
    }
    /**
     * @return {number}
     */
    id() {
        return this._id;
    }
    /**
     * @return {!Process}
     */
    process() {
        return this._process;
    }
    /**
     * @return {!Array<!Event>}
     */
    events() {
        return this._events;
    }
    /**
     * @return {!Array<!AsyncEvent>}
     */
    asyncEvents() {
        return this._asyncEvents;
    }
    /**
     * @param {string} name
     */
    removeEventsByName(name) {
        /**
         * @type {!Array<!Event>}
         */
        const extracted = [];
        this._events = this._events.filter(e => {
            if (!e) {
                return false;
            }
            if (e.name !== name) {
                return true;
            }
            extracted.push(e);
            return false;
        });
        return extracted;
    }
}

var TracingModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TracingModel: TracingModel,
    Phase: Phase,
    MetadataEvent: MetadataEvent,
    LegacyTopLevelEventCategory: LegacyTopLevelEventCategory,
    DevToolsMetadataEventCategory: DevToolsMetadataEventCategory,
    DevToolsTimelineEventCategory: DevToolsTimelineEventCategory,
    BackingStorage: BackingStorage,
    Event: Event,
    ObjectSnapshot: ObjectSnapshot,
    AsyncEvent: AsyncEvent,
    Process: Process,
    Thread: Thread
});

// Copyright 2015 The Chromium Authors. All rights reserved.
class FilmStripModel {
    /**
     * @param {!TracingModel} tracingModel
     * @param {number=} zeroTime
     */
    constructor(tracingModel, zeroTime) {
        /** @type {!Array<!Frame>} */
        this._frames = [];
        /** @type {number} */
        this._zeroTime = 0;
        /** @type {number} */
        this._spanTime = 0;
        this.reset(tracingModel, zeroTime);
    }
    /**
     * @param {!TracingModel} tracingModel
     * @param {number=} zeroTime
     */
    reset(tracingModel, zeroTime) {
        this._zeroTime = zeroTime || tracingModel.minimumRecordTime();
        this._spanTime = tracingModel.maximumRecordTime() - this._zeroTime;
        /** @type {!Array<!Frame>} */
        this._frames = [];
        const browserMain = TracingModel.browserMainThread(tracingModel);
        if (!browserMain) {
            return;
        }
        const events = browserMain.events();
        for (let i = 0; i < events.length; ++i) {
            const event = events[i];
            if (event.startTime < this._zeroTime) {
                continue;
            }
            if (!event.hasCategory(_category)) {
                continue;
            }
            if (event.name === TraceEvents.CaptureFrame) {
                const data = event.args['data'];
                if (data) {
                    this._frames.push(Frame._fromEvent(this, event, this._frames.length));
                }
            }
            else if (event.name === TraceEvents.Screenshot) {
                this._frames.push(Frame._fromSnapshot(this, /** @type {!ObjectSnapshot} */ (event), this._frames.length));
            }
        }
    }
    /**
     * @return {!Array<!Frame>}
     */
    frames() {
        return this._frames;
    }
    /**
     * @return {number}
     */
    zeroTime() {
        return this._zeroTime;
    }
    /**
     * @return {number}
     */
    spanTime() {
        return this._spanTime;
    }
    /**
     * @param {number} timestamp
     * @return {?Frame}
     */
    frameByTimestamp(timestamp) {
        const index = this._frames.upperBound(timestamp, (timestamp, frame) => timestamp - frame.timestamp) - 1;
        return index >= 0 ? this._frames[index] : null;
    }
}
const _category = 'disabled-by-default-devtools.screenshot';
const TraceEvents = {
    CaptureFrame: 'CaptureFrame',
    Screenshot: 'Screenshot'
};
/**
 * @unrestricted
 */
class Frame {
    /**
     * @param {!FilmStripModel} model
     * @param {number} timestamp
     * @param {number} index
     */
    constructor(model, timestamp, index) {
        this._model = model;
        this.timestamp = timestamp;
        this.index = index;
        /** @type {?string} */
        this._imageData = null;
        /** @type {?ObjectSnapshot} */
        this._snapshot = null;
    }
    /**
     * @param {!FilmStripModel} model
     * @param {!Event} event
     * @param {number} index
     * @return {!Frame}
     */
    static _fromEvent(model, event, index) {
        const frame = new Frame(model, event.startTime, index);
        frame._imageData = event.args['data'];
        return frame;
    }
    /**
     * @param {!FilmStripModel} model
     * @param {!ObjectSnapshot} snapshot
     * @param {number} index
     * @return {!Frame}
     */
    static _fromSnapshot(model, snapshot, index) {
        const frame = new Frame(model, snapshot.startTime, index);
        frame._snapshot = snapshot;
        return frame;
    }
    /**
     * @return {!FilmStripModel}
     */
    model() {
        return this._model;
    }
    /**
     * @return {!Promise<?string>}
     */
    imageDataPromise() {
        if (this._imageData || !this._snapshot) {
            return Promise.resolve(this._imageData);
        }
        return /** @type {!Promise<?string>} */ (this._snapshot.objectPromise());
    }
}

var FilmStripModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    FilmStripModel: FilmStripModel,
    Frame: Frame
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/** @type {!NetworkLog} */
let _instance$1;
/**
 * @implements {SDKModelObserver<!NetworkManager>}
 */
class NetworkLog extends ObjectWrapper.ObjectWrapper {
    constructor() {
        super();
        /** @type {!Array<!NetworkRequest>} */
        this._requests = [];
        /** @type {!Array<!Protocol.Network.Request>} */
        this._sentNetworkRequests = [];
        /** @type {!Array<!Protocol.Network.Response>} */
        this._receivedNetworkResponses = [];
        /** @type {!Set<!NetworkRequest>} */
        this._requestsSet = new Set();
        /** @type {!Map<string, !Array<!NetworkRequest>>} */
        this._requestsMap = new Map();
        /** @type {!Map<!NetworkManager, !PageLoad>} */
        this._pageLoadForManager = new Map();
        /** @type {boolean} */
        this._isRecording = true;
        /** @type {!WeakMap<!NetworkManager, !Array<!Common.EventTarget.EventDescriptor>>} */
        this._modelListeners = new WeakMap();
        /** @type {!WeakMap<!NetworkRequest, !InitiatorData>} */
        this._initiatorData = new WeakMap();
        TargetManager.instance().observeModels(NetworkManager, this);
    }
    /**
     * @return {!NetworkLog}
     */
    static instance() {
        if (!_instance$1) {
            _instance$1 = new NetworkLog();
        }
        return _instance$1;
    }
    /**
     * @override
     * @param {!NetworkManager} networkManager
     */
    modelAdded(networkManager) {
        const eventListeners = [];
        eventListeners.push(networkManager.addEventListener(Events$1.RequestStarted, this._onRequestStarted, this));
        eventListeners.push(networkManager.addEventListener(Events$1.RequestUpdated, this._onRequestUpdated, this));
        eventListeners.push(networkManager.addEventListener(Events$1.RequestRedirected, this._onRequestRedirect, this));
        eventListeners.push(networkManager.addEventListener(Events$1.RequestFinished, this._onRequestUpdated, this));
        eventListeners.push(networkManager.addEventListener(Events$1.MessageGenerated, this._networkMessageGenerated.bind(this, networkManager)));
        eventListeners.push(networkManager.addEventListener(Events$1.ResponseReceived, this._onResponseReceived, this));
        const resourceTreeModel = networkManager.target().model(ResourceTreeModel);
        if (resourceTreeModel) {
            eventListeners.push(resourceTreeModel.addEventListener(Events$a.WillReloadPage, this._willReloadPage, this));
            eventListeners.push(resourceTreeModel.addEventListener(Events$a.MainFrameNavigated, this._onMainFrameNavigated, this));
            eventListeners.push(resourceTreeModel.addEventListener(Events$a.Load, this._onLoad, this));
            eventListeners.push(resourceTreeModel.addEventListener(Events$a.DOMContentLoaded, this._onDOMContentLoaded.bind(this, resourceTreeModel)));
        }
        this._modelListeners.set(networkManager, eventListeners);
    }
    /**
     * @override
     * @param {!NetworkManager} networkManager
     */
    modelRemoved(networkManager) {
        this._removeNetworkManagerListeners(networkManager);
    }
    /**
     * @param {!NetworkManager} networkManager
     */
    _removeNetworkManagerListeners(networkManager) {
        EventTarget.EventTarget.removeEventListeners(this._modelListeners.get(networkManager) || []);
    }
    /**
     * @param {boolean} enabled
     */
    setIsRecording(enabled) {
        if (this._isRecording === enabled) {
            return;
        }
        this._isRecording = enabled;
        if (enabled) {
            TargetManager.instance().observeModels(NetworkManager, this);
        }
        else {
            TargetManager.instance().unobserveModels(NetworkManager, this);
            TargetManager.instance().models(NetworkManager).forEach(this._removeNetworkManagerListeners.bind(this));
        }
    }
    /**
     * @param {string} url
     * @return {?NetworkRequest}
     */
    requestForURL(url) {
        return this._requests.find(request => request.url() === url) || null;
    }
    /**
     * @param {string} url
     * @return {?Protocol.Network.Request}
     */
    originalRequestForURL(url) {
        return this._sentNetworkRequests.find(request => request.url === url) || null;
    }
    /**
     * @param {string} url
     * @return {?Protocol.Network.Response}
     */
    originalResponseForURL(url) {
        return this._receivedNetworkResponses.find(response => response.url === url) || null;
    }
    /**
     * @return {!Array<!NetworkRequest>}
     */
    requests() {
        return this._requests;
    }
    /**
     * @param {!NetworkManager} networkManager
     * @param {!Protocol.Network.RequestId} requestId
     * @return {?NetworkRequest}
     */
    requestByManagerAndId(networkManager, requestId) {
        // We itterate backwards because the last item will likely be the one needed for console network request lookups.
        for (let i = this._requests.length - 1; i >= 0; i--) {
            const request = this._requests[i];
            if (requestId === request.requestId() && networkManager === NetworkManager.forRequest(request)) {
                return request;
            }
        }
        return null;
    }
    /**
     * @param {!NetworkManager} networkManager
     * @param {string} url
     * @return {?NetworkRequest}
     */
    _requestByManagerAndURL(networkManager, url) {
        for (const request of this._requests) {
            if (url === request.url() && networkManager === NetworkManager.forRequest(request)) {
                return request;
            }
        }
        return null;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!InitiatorData}
     */
    _initializeInitiatorSymbolIfNeeded(request) {
        let initiatorInfo = this._initiatorData.get(request);
        if (initiatorInfo) {
            return initiatorInfo;
        }
        initiatorInfo = {
            info: null,
            chain: null,
            request: undefined,
        };
        this._initiatorData.set(request, initiatorInfo);
        return initiatorInfo;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!_InitiatorInfo}
     */
    initiatorInfoForRequest(request) {
        const initiatorInfo = this._initializeInitiatorSymbolIfNeeded(request);
        if (initiatorInfo.info) {
            return initiatorInfo.info;
        }
        let type = InitiatorType.Other;
        let url = '';
        let lineNumber = -Infinity;
        let columnNumber = -Infinity;
        let scriptId = null;
        let initiatorStack = null;
        const initiator = request.initiator();
        const redirectSource = request.redirectSource();
        if (redirectSource) {
            type = InitiatorType.Redirect;
            url = redirectSource.url();
        }
        else if (initiator) {
            if (initiator.type === Protocol.Network.InitiatorType.Parser) {
                type = InitiatorType.Parser;
                url = initiator.url ? initiator.url : url;
                lineNumber = initiator.lineNumber ? initiator.lineNumber : lineNumber;
            }
            else if (initiator.type === Protocol.Network.InitiatorType.Script) {
                for (let stack = initiator.stack; stack;) {
                    const topFrame = stack.callFrames.length ? stack.callFrames[0] : null;
                    if (!topFrame) {
                        stack = stack.parent;
                        continue;
                    }
                    type = InitiatorType.Script;
                    url = topFrame.url || UIString.UIString('<anonymous>');
                    lineNumber = topFrame.lineNumber;
                    columnNumber = topFrame.columnNumber;
                    scriptId = topFrame.scriptId;
                    break;
                }
                if (!initiator.stack && initiator.url) {
                    type = InitiatorType.Script;
                    url = initiator.url;
                    lineNumber = initiator.lineNumber || 0;
                }
                if (initiator.stack && initiator.stack.callFrames && initiator.stack.callFrames.length) {
                    initiatorStack = initiator.stack || null;
                }
            }
            else if (initiator.type === Protocol.Network.InitiatorType.Preload) {
                type = InitiatorType.Preload;
            }
            else if (initiator.type === Protocol.Network.InitiatorType.SignedExchange) {
                type = InitiatorType.SignedExchange;
                url = initiator.url || '';
            }
        }
        initiatorInfo.info = {
            type: type,
            url: url,
            lineNumber: lineNumber,
            columnNumber: columnNumber,
            scriptId: scriptId,
            stack: initiatorStack
        };
        return initiatorInfo.info;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!InitiatorGraph}
     */
    initiatorGraphForRequest(request) {
        /** @type {!Map<!NetworkRequest, !NetworkRequest>} */
        const initiated = new Map();
        const networkManager = NetworkManager.forRequest(request);
        for (const otherRequest of this._requests) {
            const otherRequestManager = NetworkManager.forRequest(otherRequest);
            if (networkManager === otherRequestManager && this._initiatorChain(otherRequest).has(request)) {
                // save parent request of otherRequst in order to build the initiator chain table later
                const initiatorRequest = this._initiatorRequest(otherRequest);
                if (initiatorRequest) {
                    initiated.set(otherRequest, initiatorRequest);
                }
            }
        }
        return { initiators: this._initiatorChain(request), initiated: initiated };
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!Set<!NetworkRequest>}
     */
    _initiatorChain(request) {
        const initiatorDataForRequest = this._initializeInitiatorSymbolIfNeeded(request);
        let initiatorChainCache = initiatorDataForRequest.chain;
        if (initiatorChainCache) {
            return initiatorChainCache;
        }
        initiatorChainCache = new Set();
        /** @type {?NetworkRequest} */
        let checkRequest = request;
        while (checkRequest) {
            const initiatorData = this._initializeInitiatorSymbolIfNeeded(checkRequest);
            if (initiatorData.chain) {
                SetUtilities.addAll(initiatorChainCache, initiatorData.chain);
                break;
            }
            if (initiatorChainCache.has(checkRequest)) {
                break;
            }
            initiatorChainCache.add(checkRequest);
            checkRequest = this._initiatorRequest(checkRequest);
        }
        initiatorDataForRequest.chain = initiatorChainCache;
        return initiatorChainCache;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {?NetworkRequest}
     */
    _initiatorRequest(request) {
        const initiatorData = this._initializeInitiatorSymbolIfNeeded(request);
        if (initiatorData.request !== undefined) {
            return initiatorData.request;
        }
        const url = this.initiatorInfoForRequest(request).url;
        const networkManager = NetworkManager.forRequest(request);
        initiatorData.request = networkManager ? this._requestByManagerAndURL(networkManager, url) : null;
        return initiatorData.request;
    }
    _willReloadPage() {
        if (!Settings.Settings.instance().moduleSetting('network_log.preserve-log').get()) {
            this.reset();
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onMainFrameNavigated(event) {
        const mainFrame = /** @type {!ResourceTreeFrame} */ (event.data);
        const manager = mainFrame.resourceTreeModel().target().model(NetworkManager);
        if (!manager || mainFrame.resourceTreeModel().target().parentTarget()) {
            return;
        }
        // If a page resulted in an error, the browser will navigate to an internal error page
        // hosted at 'chrome-error://...'. In this case, skip the frame navigated event to preserve
        // the network log.
        if (mainFrame.url !== mainFrame.unreachableUrl() && mainFrame.url.startsWith('chrome-error://')) {
            return;
        }
        const oldRequests = this._requests;
        const oldManagerRequests = this._requests.filter(request => NetworkManager.forRequest(request) === manager);
        const oldRequestsSet = this._requestsSet;
        this._requests = [];
        this._sentNetworkRequests = [];
        this._receivedNetworkResponses = [];
        this._requestsSet = new Set();
        this._requestsMap.clear();
        this.dispatchEventToListeners(Events$j.Reset);
        // Preserve requests from the new session.
        let currentPageLoad = null;
        const requestsToAdd = [];
        for (const request of oldManagerRequests) {
            if (request.loaderId !== mainFrame.loaderId) {
                continue;
            }
            if (!currentPageLoad) {
                currentPageLoad = new PageLoad(request);
                let redirectSource = request.redirectSource();
                while (redirectSource) {
                    requestsToAdd.push(redirectSource);
                    redirectSource = redirectSource.redirectSource();
                }
            }
            requestsToAdd.push(request);
        }
        // Preserve service worker requests from the new session.
        const serviceWorkerRequestsToAdd = [];
        for (const swRequest of oldRequests) {
            if (!swRequest.initiatedByServiceWorker()) {
                continue;
            }
            // If there is a matching request that came before this one, keep it.
            const keepRequest = requestsToAdd.some(request => request.url() === swRequest.url() && request.issueTime() <= swRequest.issueTime());
            if (keepRequest) {
                serviceWorkerRequestsToAdd.push(swRequest);
            }
        }
        requestsToAdd.push(...serviceWorkerRequestsToAdd);
        for (const request of requestsToAdd) {
            // TODO: Use optional chaining here once closure is gone.
            if (currentPageLoad) {
                currentPageLoad.bindRequest(request);
            }
            oldRequestsSet.delete(request);
            this._addRequest(request);
        }
        if (Settings.Settings.instance().moduleSetting('network_log.preserve-log').get()) {
            for (const request of oldRequestsSet) {
                this._addRequest(request);
            }
        }
        if (currentPageLoad) {
            this._pageLoadForManager.set(manager, currentPageLoad);
        }
    }
    /**
     * @param {!NetworkRequest} request
     */
    _addRequest(request) {
        this._requests.push(request);
        this._requestsSet.add(request);
        const requestList = this._requestsMap.get(request.requestId());
        if (!requestList) {
            this._requestsMap.set(request.requestId(), [request]);
        }
        else {
            requestList.push(request);
        }
        this.dispatchEventToListeners(Events$j.RequestAdded, request);
    }
    /**
     * @param {!Array<!NetworkRequest>} requests
     */
    importRequests(requests) {
        this.reset();
        this._requests = [];
        this._sentNetworkRequests = [];
        this._receivedNetworkResponses = [];
        this._requestsSet.clear();
        this._requestsMap.clear();
        for (const request of requests) {
            this._addRequest(request);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onRequestStarted(event) {
        const request = /** @type {!NetworkRequest} */ (event.data.request);
        if (event.data.originalRequest) {
            this._sentNetworkRequests.push(event.data.originalRequest);
        }
        this._requestsSet.add(request);
        const manager = NetworkManager.forRequest(request);
        const pageLoad = manager ? this._pageLoadForManager.get(manager) : null;
        if (pageLoad) {
            pageLoad.bindRequest(request);
        }
        this._addRequest(request);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onResponseReceived(event) {
        const response = /** @type {!Protocol.Network.Response} */ (event.data.response);
        this._receivedNetworkResponses.push(response);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onRequestUpdated(event) {
        const request = /** @type {!NetworkRequest} */ (event.data);
        if (!this._requestsSet.has(request)) {
            return;
        }
        this.dispatchEventToListeners(Events$j.RequestUpdated, request);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onRequestRedirect(event) {
        const request = /** @type {!NetworkRequest} */ (event.data);
        this._initiatorData.delete(request);
    }
    /**
     * @param {!ResourceTreeModel} resourceTreeModel
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onDOMContentLoaded(resourceTreeModel, event) {
        const networkManager = resourceTreeModel.target().model(NetworkManager);
        const pageLoad = networkManager ? this._pageLoadForManager.get(networkManager) : null;
        if (pageLoad) {
            pageLoad.contentLoadTime = /** @type {number} */ (event.data);
        }
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _onLoad(event) {
        const networkManager = event.data.resourceTreeModel.target().model(NetworkManager);
        const pageLoad = networkManager ? this._pageLoadForManager.get(networkManager) : null;
        if (pageLoad) {
            pageLoad.loadTime = /** @type {number} */ (event.data.loadTime);
        }
    }
    reset() {
        this._requests = [];
        this._sentNetworkRequests = [];
        this._receivedNetworkResponses = [];
        this._requestsSet.clear();
        this._requestsMap.clear();
        const managers = new Set(TargetManager.instance().models(NetworkManager));
        for (const manager of this._pageLoadForManager.keys()) {
            if (!managers.has(manager)) {
                this._pageLoadForManager.delete(manager);
            }
        }
        this.dispatchEventToListeners(Events$j.Reset);
    }
    /**
     * @param {!NetworkManager} networkManager
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _networkMessageGenerated(networkManager, event) {
        const message = /** @type {!Message} */ (event.data);
        const consoleMessage = new ConsoleMessage(networkManager.target().model(RuntimeModel), MessageSource.Network, message.warning ? MessageLevel.Warning : MessageLevel.Info, message.message);
        this.associateConsoleMessageWithRequest(consoleMessage, message.requestId);
        ConsoleModel.instance().addMessage(consoleMessage);
    }
    /**
     * @param {!ConsoleMessage} consoleMessage
     * @param {!Protocol.Network.RequestId} requestId
     */
    associateConsoleMessageWithRequest(consoleMessage, requestId) {
        const target = consoleMessage.target();
        const networkManager = target ? target.model(NetworkManager) : null;
        if (!networkManager) {
            return;
        }
        const request = this.requestByManagerAndId(networkManager, requestId);
        if (!request) {
            return;
        }
        consoleMessageToRequest.set(consoleMessage, request);
        const initiator = request.initiator();
        if (initiator) {
            consoleMessage.stackTrace = initiator.stack || undefined;
            if (initiator.url) {
                consoleMessage.url = initiator.url;
                consoleMessage.line = initiator.lineNumber || 0;
            }
        }
    }
    /**
     * @param {!ConsoleMessage} consoleMessage
     * @return {?NetworkRequest}
     */
    static requestForConsoleMessage(consoleMessage) {
        return consoleMessageToRequest.get(consoleMessage) || null;
    }
    /**
     * @param {string} requestId
     * @return {!Array<!NetworkRequest>}
     */
    requestsForId(requestId) {
        return this._requestsMap.get(requestId) || [];
    }
}
/** @type {!WeakMap<!ConsoleMessage, !NetworkRequest>} */
const consoleMessageToRequest = new WeakMap();
class PageLoad {
    /**
     * @param {!NetworkRequest} mainRequest
     */
    constructor(mainRequest) {
        this.id = ++PageLoad._lastIdentifier;
        this.url = mainRequest.url();
        this.startTime = mainRequest.startTime;
        /** @type {number} */
        this.loadTime;
        /** @type {number} */
        this.contentLoadTime;
        this.mainRequest = mainRequest;
        this._showDataSaverWarningIfNeeded();
    }
    async _showDataSaverWarningIfNeeded() {
        const manager = NetworkManager.forRequest(this.mainRequest);
        if (!manager) {
            return;
        }
        if (!this.mainRequest.finished) {
            await this.mainRequest.once(Events$2.FinishedLoading);
        }
        const saveDataHeader = this.mainRequest.requestHeaderValue('Save-Data');
        if (!PageLoad._dataSaverMessageWasShown && saveDataHeader && saveDataHeader === 'on') {
            const message = UIString.UIString('Consider disabling %s while debugging. For more info see: %s', UIString.UIString('Chrome Data Saver'), 'https://support.google.com/chrome/?p=datasaver');
            manager.dispatchEventToListeners(Events$1.MessageGenerated, { message: message, requestId: this.mainRequest.requestId(), warning: true });
            PageLoad._dataSaverMessageWasShown = true;
        }
    }
    /**
     * @param {!NetworkRequest} request
     * @return {?PageLoad}
     */
    static forRequest(request) {
        return pageLoadForRequest.get(request) || null;
    }
    /**
     * @param {!NetworkRequest} request
     */
    bindRequest(request) {
        pageLoadForRequest.set(request, this);
    }
}
PageLoad._lastIdentifier = 0;
/** @type {!WeakMap<!NetworkRequest, !PageLoad>} */
const pageLoadForRequest = new WeakMap();
PageLoad._dataSaverMessageWasShown = false;
const Events$j = {
    Reset: Symbol('Reset'),
    RequestAdded: Symbol('RequestAdded'),
    RequestUpdated: Symbol('RequestUpdated')
};
/**
 * @typedef {!{info: ?_InitiatorInfo, chain: ?Set<!NetworkRequest>, request: (?NetworkRequest|undefined)}}
 */
// @ts-ignore typedef
let InitiatorData; // eslint-disable-line no-unused-vars
/** @typedef {!{initiators: !Set<!NetworkRequest>, initiated: !Map<!NetworkRequest, !NetworkRequest>}} */
// @ts-ignore typedef
let InitiatorGraph;
/** @typedef {!{type: !InitiatorType, url: string, lineNumber: number, columnNumber: number, scriptId: ?string, stack: ?Protocol.Runtime.StackTrace}} */
// @ts-ignore typedef
let _InitiatorInfo;

var NetworkLog$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    NetworkLog: NetworkLog,
    PageLoad: PageLoad,
    Events: Events$j,
    InitiatorGraph: InitiatorGraph,
    _InitiatorInfo: _InitiatorInfo
});

/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class HARLog {
    /**
     * @param {!NetworkRequest} request
     * @param {number} monotonicTime
     * @return {!Date}
     */
    static pseudoWallTime(request, monotonicTime) {
        return new Date(request.pseudoWallTime(monotonicTime) * 1000);
    }
    /**
     * @param {!Array<!NetworkRequest>} requests
     * @return {!Promise<!HARLogDTO>}
     */
    static async build(requests) {
        const log = new HARLog();
        const entryPromises = [];
        for (const request of requests) {
            entryPromises.push(Entry.build(request));
        }
        const entries = await Promise.all(entryPromises);
        return { version: '1.2', creator: log._creator(), pages: log._buildPages(requests), entries: entries };
    }
    /**
     * @return {!Creator}
     */
    _creator() {
        const webKitVersion = /AppleWebKit\/([^ ]+)/.exec(window.navigator.userAgent);
        return { name: 'WebInspector', version: webKitVersion ? webKitVersion[1] : 'n/a' };
    }
    /**
     * @param {!Array<!NetworkRequest>} requests
     * @return {!Array<!Page>}
     */
    _buildPages(requests) {
        /** @type {!Map<number, boolean>} */
        const seenIdentifiers = new Map();
        const pages = [];
        for (let i = 0; i < requests.length; ++i) {
            const request = requests[i];
            const page = PageLoad.forRequest(request);
            if (!page || seenIdentifiers.get(page.id)) {
                continue;
            }
            seenIdentifiers.set(page.id, true);
            pages.push(this._convertPage(page, request));
        }
        return pages;
    }
    /**
     * @param {!PageLoad} page
     * @param {!NetworkRequest} request
     * @return {!Page}
     */
    _convertPage(page, request) {
        return {
            startedDateTime: HARLog.pseudoWallTime(request, page.startTime).toJSON(),
            id: 'page_' + page.id,
            title: page.url,
            pageTimings: {
                onContentLoad: this._pageEventTime(page, page.contentLoadTime),
                onLoad: this._pageEventTime(page, page.loadTime)
            }
        };
    }
    /**
     * @param {!PageLoad} page
     * @param {number} time
     * @return {number}
     */
    _pageEventTime(page, time) {
        const startTime = page.startTime;
        if (time === -1 || startTime === -1) {
            return -1;
        }
        return Entry._toMilliseconds(time - startTime);
    }
}
class Entry {
    /**
     * @param {!NetworkRequest} request
     */
    constructor(request) {
        this._request = request;
    }
    /**
     * @param {number} time
     * @return {number}
     */
    static _toMilliseconds(time) {
        return time === -1 ? -1 : time * 1000;
    }
    /**
     * @param {!NetworkRequest} request
     * @return {!Promise<!EntryDTO>}
     */
    static async build(request) {
        const harEntry = new Entry(request);
        let ipAddress = harEntry._request.remoteAddress();
        const portPositionInString = ipAddress.lastIndexOf(':');
        if (portPositionInString !== -1) {
            ipAddress = ipAddress.substr(0, portPositionInString);
        }
        const timings = harEntry._buildTimings();
        let time = 0;
        // "ssl" is included in the connect field, so do not double count it.
        for (const t of [timings.blocked, timings.dns, timings.connect, timings.send, timings.wait, timings.receive]) {
            time += Math.max(t, 0);
        }
        const initiator = harEntry._request.initiator();
        /** @type {?Protocol.Network.Initiator} */
        let exportedInitiator = null;
        if (initiator) {
            exportedInitiator = {
                type: initiator.type,
            };
            if (initiator.url !== undefined) {
                exportedInitiator.url = initiator.url;
            }
            if (initiator.lineNumber !== undefined) {
                exportedInitiator.lineNumber = initiator.lineNumber;
            }
            if (initiator.stack) {
                exportedInitiator.stack = initiator.stack;
            }
        }
        /**
         * @type {!EntryDTO}
         */
        const entry = {
            _fromCache: undefined,
            _initiator: exportedInitiator,
            _priority: harEntry._request.priority(),
            _resourceType: harEntry._request.resourceType().name(),
            _webSocketMessages: undefined,
            cache: {},
            connection: undefined,
            pageref: undefined,
            request: await harEntry._buildRequest(),
            response: harEntry._buildResponse(),
            // IPv6 address should not have square brackets per (https://tools.ietf.org/html/rfc2373#section-2.2).
            serverIPAddress: ipAddress.replace(/\[\]/g, ''),
            startedDateTime: HARLog.pseudoWallTime(harEntry._request, harEntry._request.issueTime()).toJSON(),
            time: time,
            timings: timings,
        };
        // Chrome specific.
        if (harEntry._request.cached()) {
            entry._fromCache = harEntry._request.cachedInMemory() ? 'memory' : 'disk';
        }
        else {
            delete entry._fromCache;
        }
        if (harEntry._request.connectionId !== '0') {
            entry.connection = harEntry._request.connectionId;
        }
        else {
            delete entry.connection;
        }
        const page = PageLoad.forRequest(harEntry._request);
        if (page) {
            entry.pageref = 'page_' + page.id;
        }
        else {
            delete entry.pageref;
        }
        if (harEntry._request.resourceType() === ResourceType.resourceTypes.WebSocket) {
            const messages = [];
            for (const message of harEntry._request.frames()) {
                messages.push({ type: message.type, time: message.time, opcode: message.opCode, data: message.text });
            }
            entry._webSocketMessages = messages;
        }
        else {
            delete entry._webSocketMessages;
        }
        return entry;
    }
    /**
     * @return {!Promise<!Request>}
     */
    async _buildRequest() {
        const headersText = this._request.requestHeadersText();
        /** @type {!Request} */
        const res = {
            method: this._request.requestMethod,
            url: this._buildRequestURL(this._request.url()),
            httpVersion: this._request.requestHttpVersion(),
            headers: this._request.requestHeaders(),
            queryString: this._buildParameters(this._request.queryParameters || []),
            cookies: this._buildCookies(this._request.requestCookies),
            headersSize: headersText ? headersText.length : -1,
            bodySize: await this._requestBodySize(),
            postData: undefined,
        };
        const postData = await this._buildPostData();
        if (postData) {
            res.postData = postData;
        }
        else {
            delete res.postData;
        }
        return res;
    }
    /**
     * @return {!Response}
     */
    _buildResponse() {
        const headersText = this._request.responseHeadersText;
        return {
            status: this._request.statusCode,
            statusText: this._request.statusText,
            httpVersion: this._request.responseHttpVersion(),
            headers: this._request.responseHeaders,
            cookies: this._buildCookies(this._request.responseCookies),
            content: this._buildContent(),
            redirectURL: this._request.responseHeaderValue('Location') || '',
            headersSize: headersText ? headersText.length : -1,
            bodySize: this.responseBodySize,
            _transferSize: this._request.transferSize,
            _error: this._request.localizedFailDescription
        };
    }
    /**
     * @return {!Content}
     */
    _buildContent() {
        /** @type {!Content} */
        const content = {
            size: this._request.resourceSize,
            mimeType: this._request.mimeType || 'x-unknown',
            compression: undefined,
        };
        const compression = this.responseCompression;
        if (typeof compression === 'number') {
            content.compression = compression;
        }
        else {
            delete content.compression;
        }
        return content;
    }
    /**
     * @return {!Timing}
     */
    _buildTimings() {
        // Order of events: request_start = 0, [proxy], [dns], [connect [ssl]], [send], duration
        const timing = this._request.timing;
        const issueTime = this._request.issueTime();
        const startTime = this._request.startTime;
        /** @type {!Timing} */
        const result = {
            blocked: -1,
            dns: -1,
            ssl: -1,
            connect: -1,
            send: 0,
            wait: 0,
            receive: 0,
            _blocked_queueing: -1,
            _blocked_proxy: undefined
        };
        const queuedTime = (issueTime < startTime) ? startTime - issueTime : -1;
        result.blocked = Entry._toMilliseconds(queuedTime);
        result._blocked_queueing = Entry._toMilliseconds(queuedTime);
        let highestTime = 0;
        if (timing) {
            // "blocked" here represents both queued + blocked/stalled + proxy (ie: anything before request was started).
            // We pick the better of when the network request start was reported and pref timing.
            const blockedStart = leastNonNegative([timing.dnsStart, timing.connectStart, timing.sendStart]);
            if (blockedStart !== Infinity) {
                result.blocked += blockedStart;
            }
            // Proxy is part of blocked but sometimes (like quic) blocked is -1 but has proxy timings.
            if (timing.proxyEnd !== -1) {
                result._blocked_proxy = timing.proxyEnd - timing.proxyStart;
            }
            if (result._blocked_proxy && result._blocked_proxy > result.blocked) {
                result.blocked = result._blocked_proxy;
            }
            const dnsStart = timing.dnsEnd >= 0 ? blockedStart : 0;
            const dnsEnd = timing.dnsEnd >= 0 ? timing.dnsEnd : -1;
            result.dns = dnsEnd - dnsStart;
            // SSL timing is included in connection timing.
            const sslStart = timing.sslEnd > 0 ? timing.sslStart : 0;
            const sslEnd = timing.sslEnd > 0 ? timing.sslEnd : -1;
            result.ssl = sslEnd - sslStart;
            const connectStart = timing.connectEnd >= 0 ? leastNonNegative([dnsEnd, blockedStart]) : 0;
            const connectEnd = timing.connectEnd >= 0 ? timing.connectEnd : -1;
            result.connect = connectEnd - connectStart;
            // Send should not be -1 for legacy reasons even if it is served from cache.
            const sendStart = timing.sendEnd >= 0 ? Math.max(connectEnd, dnsEnd, blockedStart) : 0;
            const sendEnd = timing.sendEnd >= 0 ? timing.sendEnd : 0;
            result.send = sendEnd - sendStart;
            // Quic sometimes says that sendStart is before connectionEnd (see: crbug.com/740792)
            if (result.send < 0) {
                result.send = 0;
            }
            highestTime = Math.max(sendEnd, connectEnd, sslEnd, dnsEnd, blockedStart, 0);
        }
        else if (this._request.responseReceivedTime === -1) {
            // Means that we don't have any more details after blocked, so attribute all to blocked.
            result.blocked = this._request.endTime - issueTime;
            return result;
        }
        const requestTime = timing ? timing.requestTime : startTime;
        const waitStart = highestTime;
        const waitEnd = Entry._toMilliseconds(this._request.responseReceivedTime - requestTime);
        result.wait = waitEnd - waitStart;
        const receiveStart = waitEnd;
        const receiveEnd = Entry._toMilliseconds(this._request.endTime - requestTime);
        result.receive = Math.max(receiveEnd - receiveStart, 0);
        return result;
        /**
         * @param {!Array<number>} values
         * @return {number}
         */
        function leastNonNegative(values) {
            return values.reduce((best, value) => (value >= 0 && value < best) ? value : best, Infinity);
        }
    }
    /**
     * @return {!Promise<?PostData>}
     */
    async _buildPostData() {
        const postData = await this._request.requestFormData();
        if (!postData) {
            return null;
        }
        /** @type {!PostData} */
        const res = { mimeType: this._request.requestContentType() || '', text: postData, params: undefined };
        const formParameters = await this._request.formParameters();
        if (formParameters) {
            res.params = this._buildParameters(formParameters);
        }
        else {
            delete res.params;
        }
        return res;
    }
    /**
     * @param {!Array<!Parameter>} parameters
     * @return {!Array<!Parameter>}
     */
    _buildParameters(parameters) {
        return parameters.slice();
    }
    /**
     * @param {string} url
     * @return {string}
     */
    _buildRequestURL(url) {
        return url.split('#', 2)[0];
    }
    /**
     * @param {!Array<!Cookie>} cookies
     * @return {!Array<!CookieDTO>}
     */
    _buildCookies(cookies) {
        return cookies.map(this._buildCookie.bind(this));
    }
    /**
     * @param {!Cookie} cookie
     * @return {!CookieDTO}
     */
    _buildCookie(cookie) {
        /** @type {!CookieDTO} */
        const c = {
            name: cookie.name(),
            value: cookie.value(),
            path: cookie.path(),
            domain: cookie.domain(),
            expires: cookie.expiresDate(HARLog.pseudoWallTime(this._request, this._request.startTime)),
            httpOnly: cookie.httpOnly(),
            secure: cookie.secure(),
            sameSite: undefined
        };
        if (cookie.sameSite()) {
            c.sameSite = cookie.sameSite();
        }
        else {
            delete c.sameSite;
        }
        return c;
    }
    /**
     * @return {!Promise<number>}
     */
    async _requestBodySize() {
        const postData = await this._request.requestFormData();
        if (!postData) {
            return 0;
        }
        // As per the har spec, returns the length in bytes of the posted data.
        // TODO(jarhar): This will be wrong if the underlying encoding is not UTF-8. NetworkRequest.requestFormData is
        //   assumed to be UTF-8 because the backend decodes post data to a UTF-8 string regardless of the provided
        //   content-type/charset in InspectorNetworkAgent::FormDataToString
        return new TextEncoder().encode(postData).length;
    }
    /**
     * @return {number}
     */
    get responseBodySize() {
        if (this._request.cached() || this._request.statusCode === 304) {
            return 0;
        }
        if (!this._request.responseHeadersText) {
            return -1;
        }
        return this._request.transferSize - this._request.responseHeadersText.length;
    }
    /**
     * @return {number|undefined}
     */
    get responseCompression() {
        if (this._request.cached() || this._request.statusCode === 304 || this._request.statusCode === 206) {
            return;
        }
        if (!this._request.responseHeadersText) {
            return;
        }
        return this._request.resourceSize - this.responseBodySize;
    }
}
/** @typedef {!{
  blocked: number,
  dns: number,
  ssl: number,
  connect: number,
  send: number,
  wait: number,
  receive: number,
  _blocked_queueing: number,
  _blocked_proxy: (number|undefined)
}} */
// @ts-ignore typedef
let Timing;
/** @typedef {{
  name: string,
  value: string,
}} */
// @ts-ignore typedef
let Parameter;
/** @typedef {{
  size: number,
  mimeType: string,
  compression: (number|undefined),
}} */
// @ts-ignore typedef
let Content;
/** @typedef {!{
  method: string,
  url: string,
  httpVersion: string,
  headers: !Object,
  queryString: !Array<!Parameter>,
  cookies: !Array<!CookieDTO>,
  headersSize: number,
  bodySize: number,
  postData: (PostData|undefined),
}} */
// @ts-ignore typedef
let Request$1;
/** @typedef {!{
  status: number,
  statusText: string,
  httpVersion: string,
  headers: !Object,
  cookies: !Array<!CookieDTO>,
  content: !Content,
  redirectURL: string,
  headersSize: number,
  bodySize: number,
  _transferSize: number,
  _error: ?string,
}} */
// @ts-ignore typedef
let Response$1;
/** @typedef {!{
  _fromCache: (string|undefined),
  _initiator: ?Protocol.Network.Initiator,
  _priority: ?Protocol.Network.ResourcePriority,
  _resourceType: string,
  _webSocketMessages: (!Array<!Object>|undefined),
  cache: !Object,
  connection: (string|undefined),
  pageref: (string|undefined),
  request: !Request,
  response: !Response,
  serverIPAddress: string,
  startedDateTime: (!Object|string),
  time: number,
  timings: !Timing,
}} */
// @ts-ignore typedef
let EntryDTO;
/** @typedef {!{
  mimeType: string,
  params: (!Array<!Parameter>|undefined),
  text: string,
}} */
// @ts-ignore typedef
let PostData;
/** @typedef {!{
  name: string,
  value: string,
  path: string,
  domain: string,
  expires: ?Date,
  httpOnly: boolean,
  secure: boolean,
  sameSite: (Protocol.Network.CookieSameSite|undefined),
}} */
// @ts-ignore typedef
let CookieDTO;
/** @typedef {!{
  startedDateTime: (!Object|string),
  id: string,
  title: string,
  pageTimings: !{
  onContentLoad: number,
  onLoad: number,
  }
}} */
// @ts-ignore typedef
let Page;
/** @typedef {!{
  version: string,
  name: string,
}} */
// @ts-ignore typedef
let Creator;
/** @typedef {!{
  version: string,
  creator: !Creator,
  pages: !Array<!Page>,
  entries: !Array<!EntryDTO>,
}} */
// @ts-ignore typedef
let HARLogDTO;

var HARLog$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    HARLog: HARLog,
    Entry: Entry,
    Timing: Timing,
    Parameter: Parameter,
    Content: Content,
    Request: Request$1,
    Response: Response$1,
    EntryDTO: EntryDTO,
    PostData: PostData,
    CookieDTO: CookieDTO,
    Page: Page,
    Creator: Creator,
    HARLogDTO: HARLogDTO
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class HeavyAdIssue extends Issue {
    /**
     * @param {!Protocol.Audits.HeavyAdIssueDetails} issueDetails
     */
    constructor(issueDetails) {
        super(Protocol.Audits.InspectorIssueCode.HeavyAdIssue);
        this._issueDetails = issueDetails;
    }
    /**
     * @override
     * @returns {!Iterable<!Protocol.Audits.HeavyAdIssueDetails>}
     */
    heavyAds() {
        return [this._issueDetails];
    }
    /**
     * @override
     * @returns {string}
     */
    primaryKey() {
        return `${Protocol.Audits.InspectorIssueCode.HeavyAdIssue}-${JSON.stringify(this._issueDetails)}`;
    }
    /**
     * @override
     * @return {?IssueDescription}
     */
    getDescription() {
        return {
            title: ls$1 `An ad on your site has exceeded resource limits`,
            message: mkHeavyAdDescription,
            issueKind: IssueKind.BreakingChange,
            links: [
                {
                    link: 'https://developers.google.com/web/updates/2020/05/heavy-ad-interventions',
                    linkTitle: ls$1 `Handling Heavy Ad Interventions`
                },
            ],
        };
    }
    /**
     * @override
     * @return {!IssueCategory}
     */
    getCategory() {
        return IssueCategory.HeavyAd;
    }
}
/**
 * @return {!Element}
 */
function mkHeavyAdDescription() {
    const message = document.createElement('div');
    message.classList.add('message');
    const contextParagraph = document.createElement('p');
    contextParagraph.textContent = ls$1 `
  Chrome identifies heavy ads on your site that use too many resources without a user gesture. Heavy ads have an impact on performance and harm the user’s browsing experience. They increase battery drain, consume mobile data, and make your site slow. To improve the user experience, Chrome warns about or removes heavy ads.`;
    message.append(contextParagraph);
    const resourceLimitHeader = ls$1 `An ad is considered heavy if the user has not interacted with it (for example, has not tapped or clicked it) and it meets any of the following criteria:`;
    const resourceLimits = [
        ls$1 `Uses the main thread for more than 60 seconds in total (CPU total limit)`,
        ls$1 `Uses the main thread for more than 15 seconds in any 30 second window (CPU peak limit)`,
        ls$1 `Uses more than 4 megabytes of network bandwidth (Network limit)`
    ];
    const resourceLimitsParagraph = document.createElement('p');
    resourceLimitsParagraph.appendChild(document.createTextNode(resourceLimitHeader));
    const resourceLimitsList = document.createElement('ul');
    for (const resourceLimit of resourceLimits) {
        const listItem = document.createElement('li');
        listItem.classList.add('plain-enum');
        listItem.textContent = resourceLimit;
        resourceLimitsList.append(listItem);
    }
    resourceLimitsParagraph.appendChild(resourceLimitsList);
    message.append(resourceLimitsParagraph);
    const resolutionParagraph = document.createElement('p');
    resolutionParagraph.textContent = ls$1 `Stop this from happening by only showing ads that stay within resource limits.`;
    message.append(resolutionParagraph);
    return message;
}

var HeavyAdIssue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    HeavyAdIssue: HeavyAdIssue
});

// Copyright 2019 The Chromium Authors. All rights reserved.
/** @type {!IsolateManager} */
let isolateManagerInstance;
/**
 * @implements {SDKModelObserver<!RuntimeModel>}
 */
class IsolateManager extends ObjectWrapper.ObjectWrapper {
    constructor() {
        super();
        /** @type {!Map<string, !Isolate>} */
        this._isolates = new Map();
        // _isolateIdByModel contains null while the isolateId is being retrieved.
        /** @type {!Map<!RuntimeModel, ?string>} */
        this._isolateIdByModel = new Map();
        /** @type {!Set<!Observer>} */
        this._observers = new Set();
        TargetManager.instance().observeModels(RuntimeModel, this);
        this._pollId = 0;
    }
    /**
     * @param {{forceNew: boolean}} opts
     */
    static instance({ forceNew } = { forceNew: false }) {
        if (!isolateManagerInstance || forceNew) {
            isolateManagerInstance = new IsolateManager();
        }
        return isolateManagerInstance;
    }
    /**
     * @param {!Observer} observer
     */
    observeIsolates(observer) {
        if (this._observers.has(observer)) {
            throw new Error('Observer can only be registered once');
        }
        if (!this._observers.size) {
            this._poll();
        }
        this._observers.add(observer);
        for (const isolate of this._isolates.values()) {
            observer.isolateAdded(isolate);
        }
    }
    /**
     * @param {!Observer} observer
     */
    unobserveIsolates(observer) {
        this._observers.delete(observer);
        if (!this._observers.size) {
            ++this._pollId;
        } // Stops the current polling loop.
    }
    /**
     * @override
     * @param {!RuntimeModel} model
     */
    modelAdded(model) {
        this._modelAdded(model);
    }
    /**
     * @param {!RuntimeModel} model
     */
    async _modelAdded(model) {
        this._isolateIdByModel.set(model, null);
        const isolateId = await model.isolateId();
        if (!this._isolateIdByModel.has(model)) {
            // The model has been removed during await.
            return;
        }
        if (!isolateId) {
            this._isolateIdByModel.delete(model);
            return;
        }
        this._isolateIdByModel.set(model, isolateId);
        let isolate = this._isolates.get(isolateId);
        if (!isolate) {
            isolate = new Isolate(isolateId);
            this._isolates.set(isolateId, isolate);
        }
        isolate._models.add(model);
        if (isolate._models.size === 1) {
            for (const observer of this._observers) {
                observer.isolateAdded(isolate);
            }
        }
        else {
            for (const observer of this._observers) {
                observer.isolateChanged(isolate);
            }
        }
    }
    /**
     * @override
     * @param {!RuntimeModel} model
     */
    modelRemoved(model) {
        const isolateId = this._isolateIdByModel.get(model);
        this._isolateIdByModel.delete(model);
        if (!isolateId) {
            return;
        }
        const isolate = this._isolates.get(isolateId);
        if (!isolate) {
            return;
        }
        isolate._models.delete(model);
        if (isolate._models.size) {
            for (const observer of this._observers) {
                observer.isolateChanged(isolate);
            }
            return;
        }
        for (const observer of this._observers) {
            observer.isolateRemoved(isolate);
        }
        this._isolates.delete(isolateId);
    }
    /**
     * @param {!RuntimeModel} model
     * @return {?Isolate}
     */
    isolateByModel(model) {
        return this._isolates.get(this._isolateIdByModel.get(model) || '') || null;
    }
    /**
     * @return {!Iterable<!Isolate>}
     */
    isolates() {
        return this._isolates.values();
    }
    async _poll() {
        const pollId = this._pollId;
        while (pollId === this._pollId) {
            await Promise.all(Array.from(this.isolates(), isolate => isolate._update()));
            await new Promise(r => setTimeout(r, PollIntervalMs));
        }
    }
}
/**
 * @interface
 */
class Observer$1 {
    /**
     * @param {!Isolate} isolate
     */
    isolateAdded(isolate) {
    }
    /**
     * @param {!Isolate} isolate
     */
    isolateRemoved(isolate) {
    }
    /**
     * @param {!Isolate} isolate
     */
    isolateChanged(isolate) {
    }
}
/** @enum {symbol} */
const Events$k = {
    MemoryChanged: Symbol('MemoryChanged')
};
const MemoryTrendWindowMs = 120e3;
const PollIntervalMs = 2e3;
class Isolate {
    /**
     * @param {string} id
     */
    constructor(id) {
        this._id = id;
        /** @type {!Set<!RuntimeModel>} */
        this._models = new Set();
        this._usedHeapSize = 0;
        const count = MemoryTrendWindowMs / PollIntervalMs;
        this._memoryTrend = new MemoryTrend(count);
    }
    /**
     * @return {string}
     */
    id() {
        return this._id;
    }
    /**
     * @return {!Set<!RuntimeModel>}
     */
    models() {
        return this._models;
    }
    /**
     * @return {?RuntimeModel}
     */
    runtimeModel() {
        return this._models.values().next().value || null;
    }
    /**
     * @return {?HeapProfilerModel}
     */
    heapProfilerModel() {
        const runtimeModel = this.runtimeModel();
        return runtimeModel && runtimeModel.heapProfilerModel();
    }
    async _update() {
        const model = this.runtimeModel();
        const usage = model && await model.heapUsage();
        if (!usage) {
            return;
        }
        this._usedHeapSize = usage.usedSize;
        this._memoryTrend.add(this._usedHeapSize);
        IsolateManager.instance().dispatchEventToListeners(Events$k.MemoryChanged, this);
    }
    /**
     * @return {number}
     */
    samplesCount() {
        return this._memoryTrend.count();
    }
    /**
     * @return {number}
     */
    usedHeapSize() {
        return this._usedHeapSize;
    }
    /**
     * @return {number} bytes per millisecond
     */
    usedHeapSizeGrowRate() {
        return this._memoryTrend.fitSlope();
    }
    /**
     * @return {boolean}
     */
    isMainThread() {
        const model = this.runtimeModel();
        return model ? model.target().id() === 'main' : false;
    }
}
/**
 * @unrestricted
 */
class MemoryTrend {
    /**
     * @param {number} maxCount
     */
    constructor(maxCount) {
        this._maxCount = maxCount | 0;
        /** @type {number} */
        this._base;
        /** @type {number} */
        this._index;
        /** @type {!Array<number>} */
        this._x;
        /** @type {!Array<number>} */
        this._y;
        /** @type {number} */
        this._sx;
        /** @type {number} */
        this._sy;
        /** @type {number} */
        this._sxx;
        /** @type {number} */
        this._sxy;
        this.reset();
    }
    reset() {
        this._base = Date.now();
        this._index = 0;
        this._x = [];
        this._y = [];
        this._sx = 0;
        this._sy = 0;
        this._sxx = 0;
        this._sxy = 0;
    }
    /**
     * @return {number}
     */
    count() {
        return this._x.length;
    }
    /**
     * @param {number} heapSize
     * @param {number=} timestamp
     */
    add(heapSize, timestamp) {
        const x = typeof timestamp === 'number' ? timestamp : Date.now() - this._base;
        const y = heapSize;
        if (this._x.length === this._maxCount) {
            // Turns into a cyclic buffer once it reaches the |_maxCount|.
            const x0 = this._x[this._index];
            const y0 = this._y[this._index];
            this._sx -= x0;
            this._sy -= y0;
            this._sxx -= x0 * x0;
            this._sxy -= x0 * y0;
        }
        this._sx += x;
        this._sy += y;
        this._sxx += x * x;
        this._sxy += x * y;
        this._x[this._index] = x;
        this._y[this._index] = y;
        this._index = (this._index + 1) % this._maxCount;
    }
    /**
     * @return {number}
     */
    fitSlope() {
        // We use the linear regression model to find the slope.
        const n = this.count();
        return n < 2 ? 0 : (this._sxy - this._sx * this._sy / n) / (this._sxx - this._sx * this._sx / n);
    }
}

var IsolateManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IsolateManager: IsolateManager,
    Observer: Observer$1,
    Events: Events$k,
    MemoryTrendWindowMs: MemoryTrendWindowMs,
    Isolate: Isolate,
    MemoryTrend: MemoryTrend
});

// Copyright 2020 The Chromium Authors. All rights reserved.
/**
 * @param {!Protocol.Audits.BlockedByResponseReason} reason
 * @return {boolean}
 */
function isCrossOriginEmbedderPolicyIssue(reason) {
    switch (reason) {
        case Protocol.Audits.BlockedByResponseReason.CoepFrameResourceNeedsCoepHeader:
            return true;
        case Protocol.Audits.BlockedByResponseReason.CoopSandboxedIFrameCannotNavigateToCoopPage:
            return true;
        case Protocol.Audits.BlockedByResponseReason.CorpNotSameOrigin:
            return true;
        case Protocol.Audits.BlockedByResponseReason.CorpNotSameOriginAfterDefaultedToSameOriginByCoep:
            return true;
        case Protocol.Audits.BlockedByResponseReason.CorpNotSameSite:
            return true;
    }
    return false;
}
class CrossOriginEmbedderPolicyIssue extends Issue {
    /**
     * @param {!Protocol.Audits.BlockedByResponseIssueDetails} issueDetails
     */
    constructor(issueDetails) {
        super(`CrossOriginEmbedderPolicy::${issueDetails.reason}`);
        /** @type {!Protocol.Audits.BlockedByResponseIssueDetails} */
        this._details = issueDetails;
    }
    /**
     * @override
     */
    primaryKey() {
        return `${this.code()}-(${this._details.request.requestId})`;
    }
    /**
     * @override
     * @returns {!Iterable<Protocol.Audits.BlockedByResponseIssueDetails>}
     */
    blockedByResponseDetails() {
        return [this._details];
    }
    /**
     * @override
     * @returns {!Iterable<Protocol.Audits.AffectedRequest>}
     */
    requests() {
        return [this._details.request];
    }
    /**
     * @override
     * @return {!IssueCategory}
     */
    getCategory() {
        return IssueCategory.CrossOriginEmbedderPolicy;
    }
    /**
     * @override
     * @returns {?MarkdownIssueDescription}
     */
    getDescription() {
        const description = issueDescriptions$1.get(this.code());
        if (!description) {
            return null;
        }
        return description;
    }
}
/** @type {!Map<string, !MarkdownIssueDescription>} */
const issueDescriptions$1 = new Map([
    [
        'CrossOriginEmbedderPolicy::CorpNotSameOriginAfterDefaultedToSameOriginByCoep', {
            file: 'issues/descriptions/CoepCorpNotSameOriginAfterDefaultedToSameOriginByCoep.md',
            issueKind: IssueKind.BreakingChange,
            links: [
                { link: 'https://web.dev/coop-coep/', linkTitle: ls$1 `COOP and COEP` },
                { link: 'https://web.dev/same-site-same-origin/', linkTitle: ls$1 `Same-Site and Same-Origin` },
            ],
        }
    ],
    [
        'CrossOriginEmbedderPolicy::CoepFrameResourceNeedsCoepHeader', {
            file: 'issues/descriptions/CoepFrameResourceNeedsCoepHeader.md',
            issueKind: IssueKind.BreakingChange,
            links: [
                { link: 'https://web.dev/coop-coep/', linkTitle: ls$1 `COOP and COEP` },
            ],
        }
    ],
    [
        'CrossOriginEmbedderPolicy::CoopSandboxedIframeCannotNavigateToCoopPage', {
            file: 'issues/descriptions/CoepCoopSandboxedIframeCannotNavigateToCoopPage.md',
            issueKind: IssueKind.BreakingChange,
            links: [
                { link: 'https://web.dev/coop-coep/', linkTitle: ls$1 `COOP and COEP` },
            ],
        }
    ],
    [
        'CrossOriginEmbedderPolicy::CorpNotSameSite', {
            file: 'issues/descriptions/CoepCorpNotSameSite.md',
            issueKind: IssueKind.BreakingChange,
            links: [
                { link: 'https://web.dev/coop-coep/', linkTitle: ls$1 `COOP and COEP` },
                { link: 'https://web.dev/same-site-same-origin/', linkTitle: ls$1 `Same-Site and Same-Origin` },
            ],
        }
    ],
    [
        'CrossOriginEmbedderPolicy::CorpNotSameOrigin', {
            file: 'issues/descriptions/CoepCorpNotSameOrigin.md',
            issueKind: IssueKind.BreakingChange,
            links: [
                { link: 'https://web.dev/coop-coep/', linkTitle: ls$1 `COOP and COEP` },
                { link: 'https://web.dev/same-site-same-origin/', linkTitle: ls$1 `Same-Site and Same-Origin` },
            ],
        }
    ],
]);

// Copyright 2020 The Chromium Authors. All rights reserved.
class MixedContentIssue extends Issue {
    /**
     * @param {!Protocol.Audits.MixedContentIssueDetails} issueDetails
     */
    constructor(issueDetails) {
        super(Protocol.Audits.InspectorIssueCode.MixedContentIssue);
        this._issueDetails = issueDetails;
    }
    /**
     * @param {string} resolutionStatus
     * @returns {!string}
     */
    static translateStatus(resolutionStatus) {
        return mixedContentStatus.get(resolutionStatus) || resolutionStatus;
    }
    /**
     * @override
     * @returns {!Iterable<Protocol.Audits.AffectedRequest>}
     */
    requests() {
        if (this._issueDetails.request) {
            return [this._issueDetails.request];
        }
        return [];
    }
    /**
     * @override
     * @returns {!Iterable<!Protocol.Audits.MixedContentIssueDetails>}
     */
    mixedContents() {
        return [this._issueDetails];
    }
    /**
     * @override
     * @return {!IssueCategory}
     */
    getCategory() {
        return IssueCategory.MixedContent;
    }
    /**
     * @override
     * @returns {!IssueDescription}
     */
    getDescription() {
        return {
            title: ls$1 `Mixed content: load all resources via HTTPS to improve the security of your site`,
            message: () => paragraphedMessage$1([
                ls$1 `Even though the initial HTML page is loaded over a secure HTTPS connection, some resources like images, stylesheets or scripts are being accessed over an insecure HTTP connection. Usage of insecure resources is restricted to strengthen the security of your entire site.`,
                ls$1 `To resolve this issue load all resources over a secure HTTPS connection.`
            ]),
            issueKind: IssueKind.BreakingChange,
            links: [{
                    link: 'https://developers.google.com/web/fundamentals/security/prevent-mixed-content/fixing-mixed-content',
                    linkTitle: ls$1 `Preventing mixed content`
                }],
        };
    }
    /**
    * @override
    */
    primaryKey() {
        return JSON.stringify(this._issueDetails);
    }
}
/**
 * @param {!Array<string>} paragraphs
 * @return {!Element}
 */
function paragraphedMessage$1(paragraphs) {
    const message = document.createElement('div');
    message.classList.add('message');
    for (const paragraph of paragraphs) {
        const paragraphElement = document.createElement('p');
        paragraphElement.textContent = paragraph;
        message.appendChild(paragraphElement);
    }
    return message;
}
/** @type {!Map<string, string>} */
const mixedContentStatus = new Map([
    ['MixedContentBlocked', ls$1 `blocked`],
    ['MixedContentAutomaticallyUpgraded', ls$1 `automatically upgraded`],
    ['MixedContentWarning', ls$1 `warned`],
]);

var MixedContentIssue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    MixedContentIssue: MixedContentIssue
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class SameSiteCookieIssue extends Issue {
    /**
     * @param {string} code
     * @param {!Protocol.Audits.SameSiteCookieIssueDetails} issueDetails
     */
    constructor(code, issueDetails) {
        super(code);
        this._issueDetails = issueDetails;
    }
    /**
     * @override
     */
    primaryKey() {
        const { domain, path, name } = this._issueDetails.cookie;
        const cookieId = `${domain};${path};${name}`;
        const requestId = this._issueDetails.request ? this._issueDetails.request.requestId : 'no-request';
        return `${this.code()}-(${cookieId})-(${requestId})`;
    }
    /**
     * Returns an array of issues from a given SameSiteCookieIssueDetails.
     *
     * @param {!Protocol.Audits.SameSiteCookieIssueDetails} sameSiteDetails
     * @return {!Array<!Issue>}
     */
    static createIssuesFromSameSiteDetails(sameSiteDetails) {
        /** @type {!Array<!Issue>} */
        const issues = [];
        // Exclusion reasons have priority. It means a cookie was blocked. Create an issue
        // for every exclusion reason but ignore warning reasons if the cookie was blocked.
        // Some exclusion reasons are dependent on warning reasons existing in order to produce an issue.
        if (sameSiteDetails.cookieExclusionReasons && sameSiteDetails.cookieExclusionReasons.length > 0) {
            for (const exclusionReason of sameSiteDetails.cookieExclusionReasons) {
                const code = SameSiteCookieIssue.codeForSameSiteDetails(exclusionReason, sameSiteDetails.cookieWarningReasons, sameSiteDetails.operation, sameSiteDetails.cookieUrl);
                if (code) {
                    issues.push(new SameSiteCookieIssue(code, sameSiteDetails));
                }
            }
            return issues;
        }
        if (sameSiteDetails.cookieWarningReasons) {
            for (const warningReason of sameSiteDetails.cookieWarningReasons) {
                // warningReasons should be an empty array here.
                const code = SameSiteCookieIssue.codeForSameSiteDetails(warningReason, [], sameSiteDetails.operation, sameSiteDetails.cookieUrl);
                if (code) {
                    issues.push(new SameSiteCookieIssue(code, sameSiteDetails));
                }
            }
        }
        return issues;
    }
    /**
     * Calculates an issue code from a reason, an operation, and an array of warningReasons. All these together
     * can uniquely identify a specific SameSite cookie issue.
     * warningReasons is only needed for some SameSiteCookieExclusionReason in order to determine if an issue should be raised.
     * It is not required if reason is a SameSiteCookieWarningReason.
     *
     * @param {!Protocol.Audits.SameSiteCookieExclusionReason|!Protocol.Audits.SameSiteCookieWarningReason} reason
     * @param {!Array<!Protocol.Audits.SameSiteCookieWarningReason>} warningReasons
     * @param {!Protocol.Audits.SameSiteCookieOperation} operation
     * @param {string|undefined} cookieUrl
     * @returns {?string}
     */
    static codeForSameSiteDetails(reason, warningReasons, operation, cookieUrl) {
        const isURLSecure = cookieUrl && (cookieUrl.startsWith('https://') || cookieUrl.startsWith('wss://'));
        const secure = isURLSecure ? 'Secure' : 'Insecure';
        if (reason === Protocol.Audits.SameSiteCookieExclusionReason.ExcludeSameSiteStrict ||
            reason === Protocol.Audits.SameSiteCookieExclusionReason.ExcludeSameSiteLax ||
            reason === Protocol.Audits.SameSiteCookieExclusionReason.ExcludeSameSiteUnspecifiedTreatedAsLax) {
            if (warningReasons && warningReasons.length > 0) {
                if (warningReasons.includes(Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictLaxDowngradeStrict)) {
                    return [
                        Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, 'ExcludeNavigationContextDowngrade', secure
                    ].join('::');
                }
                if (warningReasons.includes(Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictCrossDowngradeStrict) ||
                    warningReasons.includes(Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictCrossDowngradeLax) ||
                    warningReasons.includes(Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteLaxCrossDowngradeStrict) ||
                    warningReasons.includes(Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteLaxCrossDowngradeLax)) {
                    return [
                        Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, 'ExcludeContextDowngrade', operation, secure
                    ].join('::');
                }
            }
            // If we have ExcludeSameSiteUnspecifiedTreatedAsLax but no corresponding warnings, then add just
            // the Issue code for ExcludeSameSiteUnspecifiedTreatedAsLax.
            if (reason === Protocol.Audits.SameSiteCookieExclusionReason.ExcludeSameSiteUnspecifiedTreatedAsLax) {
                return [Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, reason, operation].join('::');
            }
            // ExcludeSameSiteStrict and ExcludeSameSiteLax require being paired with an appropriate warning. We didn't
            // find one of those warnings so return null to indicate there shouldn't be an issue created.
            return null;
        }
        if (reason === Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictLaxDowngradeStrict) {
            return [Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, reason, secure].join('::');
        }
        // These have the same message.
        if (reason === Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictCrossDowngradeStrict ||
            reason === Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteStrictCrossDowngradeLax ||
            reason === Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteLaxCrossDowngradeLax ||
            reason === Protocol.Audits.SameSiteCookieWarningReason.WarnSameSiteLaxCrossDowngradeStrict) {
            return [Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, 'WarnCrossDowngrade', operation, secure].join('::');
        }
        return [Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, reason, operation].join('::');
    }
    /**
     * @override
     * @returns {!Iterable<!Protocol.Audits.AffectedCookie>}
     */
    cookies() {
        return [this._issueDetails.cookie];
    }
    /**
     * @override
     * @returns {!Iterable<!Protocol.Audits.AffectedRequest>}
     */
    requests() {
        if (this._issueDetails.request) {
            return [this._issueDetails.request];
        }
        return [];
    }
    /**
     * @override
     * @return {!IssueCategory}
     */
    getCategory() {
        return IssueCategory.SameSiteCookie;
    }
    /**
     * @override
     * @returns {?IssueDescription}
     */
    getDescription() {
        const description = issueDescriptions$2.get(this.code());
        if (!description) {
            return null;
        }
        return description;
    }
    /**
     * @override
     * @return {boolean}
     */
    isCausedByThirdParty() {
        const topFrame = FrameManager.instance().getTopFrame();
        return isCausedByThirdParty(topFrame, this._issueDetails.cookieUrl);
    }
}
/**
 * @param {?ResourceTreeFrame} topFrame
 * @param {(string|undefined)} cookieUrl
 * @return {boolean}
 *
 * Exported for unit test.
 */
function isCausedByThirdParty(topFrame, cookieUrl) {
    if (!topFrame) {
        // The top frame is not yet available. Consider this issue as a third-party issue
        // until the top frame is available. This will prevent the issue from being visible
        // for only just a split second.
        return true;
    }
    // In the case of no domain and registry, we assume its an IP address or localhost
    // during development, in this case we classify the issue as first-party.
    if (!cookieUrl || topFrame.domainAndRegistry() === '') {
        return false;
    }
    const parsedCookieUrl = ParsedURL.ParsedURL.fromString(cookieUrl);
    if (!parsedCookieUrl) {
        return false;
    }
    // For both operation types we compare the cookieUrl's domain  with the top frames
    // registered domain to determine first-party vs third-party. If they don't match
    // then we consider this issue a third-party issue.
    //
    // For a Set operation: The Set-Cookie response is part of a request to a third-party.
    //
    // For a Read operation: The cookie was included in a request to a third-party
    //     site. Only cookies that have their domain also set to this third-party
    //     are included in the request. We assume that the cookie was set by the same
    //     third-party at some point, so we treat this as a third-party issue.
    //
    // TODO(crbug.com/1080589): Use "First-Party sets" instead of the sites registered domain.
    return !IsSubdomainOf(parsedCookieUrl.domain(), topFrame.domainAndRegistry());
}
/**
 * @param {string} subdomain
 * @param {string} superdomain
 * @return {boolean}
 */
function IsSubdomainOf(subdomain, superdomain) {
    // Subdomain must be identical or have strictly more labels than the
    // superdomain.
    if (subdomain.length <= superdomain.length) {
        return subdomain === superdomain;
    }
    // Superdomain must be suffix of subdomain, and the last character not
    // included in the matching substring must be a dot.
    if (!subdomain.endsWith(superdomain)) {
        return false;
    }
    const subdomainWithoutSuperdomian = subdomain.substr(0, subdomain.length - superdomain.length);
    return subdomainWithoutSuperdomian.endsWith('.');
}
/**
 * @param {string} text
 * @param {string} resolveMessage
 * @param {!Array<string>} resolutions
 * @return {!Element}
 */
function textMessageWithResolutions(text, resolveMessage, resolutions) {
    /**
     * Inserts <code> tags for substrings of `message` that are enclosed
     * by |, i.e. "Hello |code|" causes code get enclosed in a <code> tag.
     * This is not an injection risk, as it only use `textContent` and only
     * programmatically creates <span> and <code> elements.
     * @param {!Element} element
     * @param {string} message
     */
    const appendStyled = (element, message) => {
        let lastIndex = 0;
        // Closure doesn't know String.p.matchAll exists.
        /** @suppress {missingProperties} */
        const matches = message.matchAll(/\|(.*?)\|/g);
        for (const match of matches) { //
            if (match.index !== undefined) {
                const span = document.createElement('span');
                span.textContent = message.substring(lastIndex, match.index);
                element.appendChild(span);
                const code = document.createElement('code');
                code.textContent = match[1];
                lastIndex = match.index + match[0].length;
                element.appendChild(code);
            }
        }
        if (lastIndex < message.length) {
            const span = document.createElement('span');
            span.textContent = message.substring(lastIndex, message.length);
            element.appendChild(span);
        }
    };
    const message = document.createElement('div');
    message.classList.add('message');
    const messageContent = document.createElement('p');
    appendStyled(messageContent, text);
    message.append(messageContent);
    const resolutionParagraph = document.createElement('p');
    message.append(resolutionParagraph);
    const resolutionParagraphTextContent = document.createElement('span');
    appendStyled(resolutionParagraphTextContent, resolveMessage);
    resolutionParagraph.append(resolutionParagraphTextContent);
    if (resolutions.length > 0) {
        const resolutionList = document.createElement('ul');
        resolutionList.classList.add('resolutions-list');
        resolutionParagraph.append(resolutionList);
        for (const resolution of resolutions) {
            const listItem = document.createElement('li');
            appendStyled(listItem, resolution);
            resolutionList.append(listItem);
        }
    }
    return message;
}
const resolutionsRead = [
    ls$2 `Specify |SameSite=None| and |Secure| if the cookie should be sent in cross-site requests. This enables third-party use.`,
    ls$2 `Specify |SameSite=Strict| or |SameSite=Lax| if the cookie should not be sent in cross-site requests`,
];
const resolutionsSet = [
    ls$2 `Specify |SameSite=None| and |Secure| if the cookie is intended to be set in cross-site contexts. Note that only cookies sent over HTTPS may use the |Secure| attribute.`,
    ls$2 `Specify |SameSite=Strict| or |SameSite=Lax| if the cookie should not be set by cross-site requests`,
];
const resolveBySentence = ls$2 `Resolve this issue by updating the attributes of the cookie:`;
const sameSiteUnspecifiedErrorRead = {
    title: ls$2 `Indicate whether to send a cookie in a cross-site request by specifying its SameSite attribute`,
    message: () => textMessageWithResolutions(ls$2 `Because a cookie's |SameSite| attribute was not set or is invalid, it defaults to |SameSite=Lax|, which prevents the cookie from being sent in a cross-site request.
       This behavior protects user data from accidentally leaking to third parties and cross-site request forgery.`, resolveBySentence, resolutionsRead),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteUnspecifiedErrorSet = {
    title: ls$2 `Indicate whether a cookie is intended to be set in a cross-site context by specifying its SameSite attribute`,
    message: () => textMessageWithResolutions(ls$2 `Because a cookie's |SameSite| attribute  was not set or is invalid, it defaults to |SameSite=Lax|, which prevents the cookie from being set in a cross-site context.
       This behavior protects user data from accidentally leaking to third parties and cross-site request forgery.`, resolveBySentence, resolutionsSet),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteUnspecifiedWarnRead = {
    title: ls$2 `Indicate whether to send a cookie in a cross-site request by specifying its SameSite attribute`,
    message: () => textMessageWithResolutions(ls$2 `Because a cookie's |SameSite| attribute was not set or is invalid, it defaults to |SameSite=Lax|, which will prevent the cookie from being sent in a cross-site request in a future version of the browser.
       This behavior protects user data from accidentally leaking to third parties and cross-site request forgery.`, resolveBySentence, resolutionsRead),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteUnspecifiedWarnSet = {
    title: ls$2 `Indicate whether a cookie is intended to be set in cross-site context by specifying its SameSite attribute`,
    message: () => textMessageWithResolutions(ls$2 `Because a cookie's |SameSite| attribute was not set or is invalid, it defaults to |SameSite=Lax|, which will prevents the cookie from being set in a cross-site context in a future version of the browser.
       This behavior protects user data from accidentally leaking to third parties and cross-site request forgery.`, resolveBySentence, resolutionsSet),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteNoneInsecureErrorRead = {
    title: ls$2 `Mark cross-site cookies as Secure to allow them to be sent in cross-site requests`,
    message: () => textMessageWithResolutions(ls$2 `Cookies marked with |SameSite=None| must also be marked with |Secure| to get sent in cross-site requests.
       This behavior protects user data from being sent over an insecure connection.`, resolveBySentence, resolutionsRead),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteNoneInsecureErrorSet = {
    title: ls$2 `Mark cross-site cookies as Secure to allow setting them in cross-site contexts`,
    message: () => textMessageWithResolutions(ls$2 `Cookies marked with |SameSite=None| must also be marked with |Secure| to allow setting them in a cross-site context.
       This behavior protects user data from being sent over an insecure connection.`, resolveBySentence, resolutionsSet),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteNoneInsecureWarnRead = {
    title: ls$2 `Mark cross-site cookies as Secure to allow them to be sent in cross-site requests`,
    message: () => textMessageWithResolutions(ls$2 `In a future version of the browser, cookies marked with |SameSite=None| must also be marked with |Secure| to get sent in cross-site requests.
       This behavior protects user data from being sent over an insecure connection.`, resolveBySentence, resolutionsRead),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
const sameSiteNoneInsecureWarnSet = {
    title: ls$2 `Mark cross-site cookies as Secure to allow setting them in cross-site contexts`,
    message: () => textMessageWithResolutions(ls$2 `In a future version of the browser, cookies marked with |SameSite=None| must also be marked with |Secure| to allow setting them in a cross-site context.
       This behavior protects user data from being sent over an insecure connection.`, resolveBySentence, resolutionsSet),
    issueKind: IssueKind.BreakingChange,
    links: [{ link: 'https://web.dev/samesite-cookies-explained/', linkTitle: ls$2 `SameSite cookies explained` }],
};
/**
 * @type {!Array<!{link: string, linkTitle: string}>}
 */
const schemefulSameSiteArticles = [{
        link: 'https://www.chromium.org/updates/schemeful-same-site/schemeful-same-site-devtools-issues',
        linkTitle: ls$2 `How Schemeful Same-Site Works`
    }];
const resolveBySentenceForDowngrade = ls$2 `Resolve this issue by migrating your site (as defined by the eTLD+1) entirely to HTTPS. It is also recommended to mark the cookie with the |Secure| attribute if that is not already the case.`;
/**
 * @param {boolean} isSecure
 */
function sameSiteWarnStrictLaxDowngradeStrict(isSecure) {
    const destination = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const origin = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to continue having cookies sent on same-site requests`,
        message: () => textMessageWithResolutions(ls$2 `A cookie is being sent to ${destination} origin from ${origin} context on a navigation.
        Because this cookie is being sent across schemes on the same site, it will not be sent in a future version of Chrome.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/**
 * @param {boolean} isSecure
 */
function sameSiteExcludeNavigationContextDowngrade(isSecure) {
    const destination = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const origin = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to have cookies sent on same-site requests`,
        message: () => textMessageWithResolutions(ls$2 `A cookie was not sent to ${destination} origin from ${origin} context on a navigation.
        Because this cookie would have been sent across schemes on the same site, it was not sent.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/**
 * @param {boolean} isSecure
 */
function sameSiteWarnCrossDowngradeRead(isSecure) {
    const destination = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const origin = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to continue having cookies sent to same-site subresources`,
        message: () => textMessageWithResolutions(ls$2 `A cookie is being sent to ${destination} origin from ${origin} context.
        Because this cookie is being sent across schemes on the same site, it will not be sent in a future version of Chrome.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/**
 * @param {boolean} isSecure
 */
function sameSiteExcludeContextDowngradeRead(isSecure) {
    const destination = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const origin = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to have cookies sent to same-site subresources`,
        message: () => textMessageWithResolutions(ls$2 `A cookie was not sent to ${destination} origin from ${origin} context.
        Because this cookie would have been sent across schemes on the same site, it was not sent.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/**
 * @param {boolean} isSecure
 */
function sameSiteWarnCrossDowngradeSet(isSecure) {
    const origin = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const destination = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to continue allowing cookies to be set by same-site subresources`,
        message: () => textMessageWithResolutions(ls$2 `A cookie is being set by ${origin} origin in ${destination} context.
        Because this cookie is being set across schemes on the same site, it will be blocked in a future version of Chrome.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/**
 * @param {boolean} isSecure
 */
function sameSiteExcludeContextDowngradeSet(isSecure) {
    const destination = isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    const origin = !isSecure ? ls$2 `a secure` : ls$2 `an insecure`;
    return {
        title: ls$2 `Migrate entirely to HTTPS to allow cookies to be set by same-site subresources`,
        message: () => textMessageWithResolutions(ls$2 `A cookie was not set by ${origin} origin in ${destination} context.
        Because this cookie would have been set across schemes on the same site, it was blocked.
        This behavior enhances the |SameSite| attribute’s protection of user data from request forgery by network attackers.`, resolveBySentenceForDowngrade, []),
        issueKind: IssueKind.BreakingChange,
        links: schemefulSameSiteArticles,
    };
}
/** @type {!Map<string, !IssueDescription>} */
const issueDescriptions$2 = new Map([
    ['SameSiteCookieIssue::ExcludeSameSiteUnspecifiedTreatedAsLax::ReadCookie', sameSiteUnspecifiedErrorRead],
    ['SameSiteCookieIssue::ExcludeSameSiteUnspecifiedTreatedAsLax::SetCookie', sameSiteUnspecifiedErrorSet],
    // These two don't have a deprecation date yet, but they need to be fixed eventually.
    ['SameSiteCookieIssue::WarnSameSiteUnspecifiedLaxAllowUnsafe::ReadCookie', sameSiteUnspecifiedWarnRead],
    ['SameSiteCookieIssue::WarnSameSiteUnspecifiedLaxAllowUnsafe::SetCookie', sameSiteUnspecifiedWarnSet],
    ['SameSiteCookieIssue::ExcludeSameSiteNoneInsecure::ReadCookie', sameSiteNoneInsecureErrorRead],
    ['SameSiteCookieIssue::ExcludeSameSiteNoneInsecure::SetCookie', sameSiteNoneInsecureErrorSet],
    ['SameSiteCookieIssue::WarnSameSiteNoneInsecure::ReadCookie', sameSiteNoneInsecureWarnRead],
    ['SameSiteCookieIssue::WarnSameSiteNoneInsecure::SetCookie', sameSiteNoneInsecureWarnSet],
    ['SameSiteCookieIssue::WarnSameSiteUnspecifiedCrossSiteContext::ReadCookie', sameSiteUnspecifiedWarnRead],
    ['SameSiteCookieIssue::WarnSameSiteUnspecifiedCrossSiteContext::SetCookie', sameSiteUnspecifiedWarnSet],
    ['SameSiteCookieIssue::WarnSameSiteStrictLaxDowngradeStrict::Secure', sameSiteWarnStrictLaxDowngradeStrict(true)],
    ['SameSiteCookieIssue::WarnSameSiteStrictLaxDowngradeStrict::Insecure', sameSiteWarnStrictLaxDowngradeStrict(false)],
    ['SameSiteCookieIssue::WarnCrossDowngrade::ReadCookie::Secure', sameSiteWarnCrossDowngradeRead(true)],
    ['SameSiteCookieIssue::WarnCrossDowngrade::ReadCookie::Insecure', sameSiteWarnCrossDowngradeRead(false)],
    ['SameSiteCookieIssue::WarnCrossDowngrade::SetCookie::Secure', sameSiteWarnCrossDowngradeSet(true)],
    ['SameSiteCookieIssue::WarnCrossDowngrade::SetCookie::Insecure', sameSiteWarnCrossDowngradeSet(false)],
    ['SameSiteCookieIssue::ExcludeNavigationContextDowngrade::Secure', sameSiteExcludeNavigationContextDowngrade(true)],
    [
        'SameSiteCookieIssue::ExcludeNavigationContextDowngrade::Insecure', sameSiteExcludeNavigationContextDowngrade(false)
    ],
    ['SameSiteCookieIssue::ExcludeContextDowngrade::ReadCookie::Secure', sameSiteExcludeContextDowngradeRead(true)],
    ['SameSiteCookieIssue::ExcludeContextDowngrade::ReadCookie::Insecure', sameSiteExcludeContextDowngradeRead(false)],
    ['SameSiteCookieIssue::ExcludeContextDowngrade::SetCookie::Secure', sameSiteExcludeContextDowngradeSet(true)],
    ['SameSiteCookieIssue::ExcludeContextDowngrade::SetCookie::Insecure', sameSiteExcludeContextDowngradeSet(false)]
]);

var SameSiteCookieIssue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    SameSiteCookieIssue: SameSiteCookieIssue,
    isCausedByThirdParty: isCausedByThirdParty
});

// Copyright 2019 The Chromium Authors. All rights reserved.
/**
 * The `IssuesModel` is a thin dispatch that does not store issues, but only creates the representation
 * class (usually derived from `Issue`) and passes the instances on via a dispatched event.
 * We chose this approach here because the lifetime of the Model is tied to the target, but DevTools
 * wants to preserve issues for targets (e.g. iframes) that are already gone as well.
 * @implements {ProtocolProxyApi.AuditsDispatcher}
 */
class IssuesModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._enabled = false;
        /** @type {*} */
        this._auditsAgent = null;
        this.ensureEnabled();
        this._disposed = false;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    ensureEnabled() {
        if (this._enabled) {
            return;
        }
        this._enabled = true;
        this.target().registerAuditsDispatcher(this);
        this._auditsAgent = this.target().auditsAgent();
        this._auditsAgent.invoke_enable();
    }
    /**
     * @override
     * @param {!Protocol.Audits.IssueAddedEvent} issueAddedEvent
     */
    issueAdded(issueAddedEvent) {
        const issues = this._createIssuesFromProtocolIssue(issueAddedEvent.issue);
        for (const issue of issues) {
            this.addIssue(issue);
        }
    }
    /**
     * @param {!Issue} issue
     */
    addIssue(issue) {
        this.dispatchEventToListeners(Events$l.IssueAdded, { issuesModel: this, issue });
    }
    /**
     * Each issue reported by the backend can result in multiple {!Issue} instances.
     * Handlers are simple functions hard-coded into a map.
     * @param {!Protocol.Audits.InspectorIssue} inspectorIssue} inspectorIssue
     * @return {!Array<!Issue>}
     */
    _createIssuesFromProtocolIssue(inspectorIssue) {
        const handler = issueCodeHandlers.get(inspectorIssue.code);
        if (handler) {
            return handler(this, inspectorIssue.details);
        }
        console.warn(`No handler registered for issue code ${inspectorIssue.code}`);
        return [];
    }
    /**
     * @override
     */
    dispose() {
        super.dispose();
        this._disposed = true;
    }
    /**
     * @returns {?Target}
     */
    getTargetIfNotDisposed() {
        if (!this._disposed) {
            return this.target();
        }
        return null;
    }
}
/**
 * @param {!IssuesModel} issuesModel
 * @param {!Protocol.Audits.InspectorIssueDetails} inspectorDetails
 * @return {!Array<!Issue>}
 */
function createIssuesForSameSiteCookieIssue(issuesModel, inspectorDetails) {
    const sameSiteDetails = inspectorDetails.sameSiteCookieIssueDetails;
    if (!sameSiteDetails) {
        console.warn('SameSite issue without details received.');
        return [];
    }
    return SameSiteCookieIssue.createIssuesFromSameSiteDetails(sameSiteDetails);
}
/**
 * @param {!IssuesModel} issuesModel
 * @param {!Protocol.Audits.InspectorIssueDetails} inspectorDetails
 * @return {!Array<!Issue>}
 */
function createIssuesForMixedContentIssue(issuesModel, inspectorDetails) {
    const mixedContentDetails = inspectorDetails.mixedContentIssueDetails;
    if (!mixedContentDetails) {
        console.warn('Mixed content issue without details received.');
        return [];
    }
    return [new MixedContentIssue(mixedContentDetails)];
}
/**
 * @param {!IssuesModel} issuesModel
 * @param {!Protocol.Audits.InspectorIssueDetails} inspectorDetails
 * @return {!Array<!Issue>}
 */
function createIssuesForContentSecurityPolicyIssue(issuesModel, inspectorDetails) {
    const cspDetails = inspectorDetails.contentSecurityPolicyIssueDetails;
    if (!cspDetails) {
        console.warn('Content security policy issue without details received.');
        return [];
    }
    return [new ContentSecurityPolicyIssue(cspDetails, issuesModel)];
}
/**
 * @param {!IssuesModel} issuesModel
 * @param {!Protocol.Audits.InspectorIssueDetails} inspectorDetails
 * @return {!Array<!Issue>}
 */
function createIssuesForHeavyAdIssue(issuesModel, inspectorDetails) {
    const heavyAdIssueDetails = inspectorDetails.heavyAdIssueDetails;
    if (!heavyAdIssueDetails) {
        console.warn('Heavy Ad issue without details received.');
        return [];
    }
    return [new HeavyAdIssue(heavyAdIssueDetails)];
}
/**
 * @param {!IssuesModel} issuesModel
 * @param {!Protocol.Audits.InspectorIssueDetails} inspectorDetails
 * @return {!Array<!Issue>}
 */
function createIssuesForBlockedByResponseIssue(issuesModel, inspectorDetails) {
    const blockedByResponseIssueDetails = inspectorDetails.blockedByResponseIssueDetails;
    if (!blockedByResponseIssueDetails) {
        console.warn('BlockedByResponse issue without details received.');
        return [];
    }
    if (isCrossOriginEmbedderPolicyIssue(blockedByResponseIssueDetails.reason)) {
        return [new CrossOriginEmbedderPolicyIssue(blockedByResponseIssueDetails)];
    }
    return [];
}
/**
 * @type {!Map<!Protocol.Audits.InspectorIssueCode, function(!IssuesModel, !Protocol.Audits.InspectorIssueDetails):!Array<!Issue>>}
 */
const issueCodeHandlers = new Map([
    [Protocol.Audits.InspectorIssueCode.SameSiteCookieIssue, createIssuesForSameSiteCookieIssue],
    [Protocol.Audits.InspectorIssueCode.MixedContentIssue, createIssuesForMixedContentIssue],
    [Protocol.Audits.InspectorIssueCode.HeavyAdIssue, createIssuesForHeavyAdIssue],
    [Protocol.Audits.InspectorIssueCode.ContentSecurityPolicyIssue, createIssuesForContentSecurityPolicyIssue],
    [Protocol.Audits.InspectorIssueCode.BlockedByResponseIssue, createIssuesForBlockedByResponseIssue],
]);
/** @enum {symbol} */
const Events$l = {
    IssueAdded: Symbol('IssueAdded'),
};
SDKModel.register(IssuesModel, Capability.Audits, true);

var IssuesModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    IssuesModel: IssuesModel,
    Events: Events$l
});

// Copyright 2016 The Chromium Authors. All rights reserved.
/**
 * @interface
 */
class Layer {
    /**
     * @return {string}
     */
    id() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?string}
     */
    parentId() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?Layer}
     */
    parent() {
        throw new Error('Not implemented');
    }
    /**
     * @return {boolean}
     */
    isRoot() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array.<!Layer>}
     */
    children() {
        throw new Error('Not implemented');
    }
    /**
     * @param {!Layer} child
     */
    addChild(child) {
    }
    /**
     * @return {?DOMNode}
     */
    node() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?DOMNode}
     */
    nodeForSelfOrAncestor() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    offsetX() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    offsetY() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    width() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    height() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?Array.<number>}
     */
    transform() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array.<number>}
     */
    quad() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array.<number>}
     */
    anchorPoint() {
        throw new Error('Not implemented');
    }
    /**
     * @return {boolean}
     */
    invisible() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    paintCount() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?Protocol.DOM.Rect}
     */
    lastPaintRect() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array.<!Protocol.LayerTree.ScrollRect>}
     */
    scrollRects() {
        throw new Error('Not implemented');
    }
    /**
     * @return {?StickyPositionConstraint}
     */
    stickyPositionConstraint() {
        throw new Error('Not implemented');
    }
    /**
     * @return {number}
     */
    gpuMemoryUsage() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Promise<!Array<string>>}
     */
    requestCompositingReasonIds() {
        throw new Error('Not implemented');
    }
    /**
     * @return {boolean}
     */
    drawsContent() {
        throw new Error('Not implemented');
    }
    /**
     * @return {!Array<!Promise<?SnapshotWithRect>>}
     */
    snapshots() {
        throw new Error('Not implemented');
    }
}
Layer.ScrollRectType = {
    NonFastScrollable: 'NonFastScrollable',
    TouchEventHandler: 'TouchEventHandler',
    WheelEventHandler: 'WheelEventHandler',
    RepaintsOnScroll: 'RepaintsOnScroll',
    MainThreadScrollingReason: 'MainThreadScrollingReason'
};
class StickyPositionConstraint {
    /**
     * @param {?LayerTreeBase} layerTree
     * @param {!Protocol.LayerTree.StickyPositionConstraint} constraint
     * @struct
     */
    constructor(layerTree, constraint) {
        /** @type {!Protocol.DOM.Rect} */
        this._stickyBoxRect = constraint.stickyBoxRect;
        /** @type {!Protocol.DOM.Rect} */
        this._containingBlockRect = constraint.containingBlockRect;
        /** @type {?Layer} */
        this._nearestLayerShiftingStickyBox = null;
        if (layerTree && constraint.nearestLayerShiftingStickyBox) {
            this._nearestLayerShiftingStickyBox = layerTree.layerById(constraint.nearestLayerShiftingStickyBox);
        }
        /** @type {?Layer} */
        this._nearestLayerShiftingContainingBlock = null;
        if (layerTree && constraint.nearestLayerShiftingContainingBlock) {
            this._nearestLayerShiftingContainingBlock = layerTree.layerById(constraint.nearestLayerShiftingContainingBlock);
        }
    }
    /**
     * @return {!Protocol.DOM.Rect}
     */
    stickyBoxRect() {
        return this._stickyBoxRect;
    }
    /**
     * @return {!Protocol.DOM.Rect}
     */
    containingBlockRect() {
        return this._containingBlockRect;
    }
    /**
     * @return {?Layer}
     */
    nearestLayerShiftingStickyBox() {
        return this._nearestLayerShiftingStickyBox;
    }
    /**
     * @return {?Layer}
     */
    nearestLayerShiftingContainingBlock() {
        return this._nearestLayerShiftingContainingBlock;
    }
}
/**
 * @unrestricted
 */
class LayerTreeBase {
    /**
     * @param {?Target} target
     */
    constructor(target) {
        this._target = target;
        this._domModel = target ? target.model(DOMModel) : null;
        /** @type {!Map<(string|number), !Layer>} */
        this.layersById = new Map();
        this._root = null;
        this._contentRoot = null;
        /** @type {!Map<number, ?DOMNode>} */
        this._backendNodeIdToNode = new Map();
    }
    /**
     * @return {?Target}
     */
    target() {
        return this._target;
    }
    /**
     * @return {?Layer}
     */
    root() {
        return this._root;
    }
    /**
     * @param {?Layer} root
     * @protected
     */
    setRoot(root) {
        this._root = root;
    }
    /**
     * @return {?Layer}
     */
    contentRoot() {
        return this._contentRoot;
    }
    /**
     * @param {?Layer} contentRoot
     * @protected
     */
    setContentRoot(contentRoot) {
        this._contentRoot = contentRoot;
    }
    /**
     * @param {function(!Layer):*} callback
     * @param {?Layer=} root
     * @return {*}
     */
    forEachLayer(callback, root) {
        if (!root) {
            root = this.root();
            if (!root) {
                return false;
            }
        }
        return callback(root) || root.children().some(this.forEachLayer.bind(this, callback));
    }
    /**
     * @param {string} id
     * @return {?Layer}
     */
    layerById(id) {
        return this.layersById.get(id) || null;
    }
    /**
     * @param {!Set<number>} requestedNodeIds
     * @return {!Promise<void>}
     */
    async resolveBackendNodeIds(requestedNodeIds) {
        if (!requestedNodeIds.size || !this._domModel) {
            return;
        }
        const nodesMap = await this._domModel.pushNodesByBackendIdsToFrontend(requestedNodeIds);
        if (!nodesMap) {
            return;
        }
        for (const nodeId of nodesMap.keys()) {
            this._backendNodeIdToNode.set(nodeId, nodesMap.get(nodeId) || null);
        }
    }
    /**
     * @return {!Map<number, ?DOMNode>}
     */
    backendNodeIdToNode() {
        return this._backendNodeIdToNode;
    }
    /**
     * @param {!{width: number, height: number}} viewportSize
     */
    setViewportSize(viewportSize) {
        this._viewportSize = viewportSize;
    }
    /**
     * @return {!{width: number, height: number}|undefined}
     */
    viewportSize() {
        return this._viewportSize;
    }
    /**
     * @param {number} id
     * @return {?DOMNode}
     */
    _nodeForId(id) {
        return this._domModel ? this._domModel.nodeForId(id) : null;
    }
}

var LayerTreeBase$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Layer: Layer,
    StickyPositionConstraint: StickyPositionConstraint,
    LayerTreeBase: LayerTreeBase
});

/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class PaintProfilerModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._layerTreeAgent = target.layerTreeAgent();
    }
    /**
     * @param {!Array.<!Protocol.LayerTree.PictureTile>} tiles
     * @return {!Promise<?PaintProfilerSnapshot>}
     */
    async loadSnapshotFromFragments(tiles) {
        const { snapshotId } = await this._layerTreeAgent.invoke_loadSnapshot({ tiles });
        return snapshotId ? new PaintProfilerSnapshot(this, snapshotId) : null;
    }
    /**
     * @param {!Protocol.binary} encodedPicture
     * @return {!Promise<?PaintProfilerSnapshot>}
     */
    loadSnapshot(encodedPicture) {
        const fragment = { x: 0, y: 0, picture: encodedPicture };
        return this.loadSnapshotFromFragments([fragment]);
    }
    /**
     * @param {!Protocol.LayerTree.LayerId} layerId
     * @return {!Promise<?PaintProfilerSnapshot>}
     */
    async makeSnapshot(layerId) {
        const { snapshotId } = await this._layerTreeAgent.invoke_makeSnapshot({ layerId });
        return snapshotId ? new PaintProfilerSnapshot(this, snapshotId) : null;
    }
}
class PaintProfilerSnapshot {
    /**
     * @param {!PaintProfilerModel} paintProfilerModel
     * @param {!Protocol.LayerTree.SnapshotId} snapshotId
     */
    constructor(paintProfilerModel, snapshotId) {
        this._paintProfilerModel = paintProfilerModel;
        this._id = snapshotId;
        this._refCount = 1;
    }
    release() {
        console.assert(this._refCount > 0, 'release is already called on the object');
        if (!--this._refCount) {
            this._paintProfilerModel._layerTreeAgent.invoke_releaseSnapshot({ snapshotId: this._id });
        }
    }
    addReference() {
        ++this._refCount;
        console.assert(this._refCount > 0, 'Referencing a dead object');
    }
    /**
     * @param {number=} scale
     * @param {number=} fromStep
     * @param {number=} toStep
     * @return {!Promise<?string>}
     */
    async replay(scale, fromStep, toStep) {
        const response = await this._paintProfilerModel._layerTreeAgent.invoke_replaySnapshot({ snapshotId: this._id, fromStep, toStep, scale: scale || 1.0 });
        return response.dataURL;
    }
    /**
     * @param {?Protocol.DOM.Rect} clipRect
     * @return {!Promise<?Array<!Protocol.LayerTree.PaintProfile>>}
     */
    async profile(clipRect) {
        const response = await this._paintProfilerModel._layerTreeAgent.invoke_profileSnapshot({ snapshotId: this._id, minRepeatCount: 5, minDuration: 1, clipRect: clipRect || undefined });
        return response.timings;
    }
    /**
     * @return {!Promise<?Array<!PaintProfilerLogItem>>}
     */
    async commandLog() {
        const response = await this._paintProfilerModel._layerTreeAgent.invoke_snapshotCommandLog({ snapshotId: this._id });
        return response.commandLog ?
            response.commandLog.map((entry, index) => new PaintProfilerLogItem(/** @type {!RawPaintProfilerLogItem} */ (entry), index)) :
            null;
    }
}
class PaintProfilerLogItem {
    /**
     * @param {!RawPaintProfilerLogItem} rawEntry
     * @param {number} commandIndex
     */
    constructor(rawEntry, commandIndex) {
        this.method = rawEntry.method;
        this.params = rawEntry.params;
        this.commandIndex = commandIndex;
    }
}
SDKModel.register(PaintProfilerModel, Capability.DOM, false);
/** @typedef {!{
        rect: !Protocol.DOM.Rect,
        snapshot: !PaintProfilerSnapshot
    }}
*/
// @ts-ignore typedef
let SnapshotWithRect;
/**
 * @typedef {!{x: number, y: number, picture: string}}
 */
// @ts-ignore typedef
let PictureFragment;
/**
 * @typedef {!{method: string, params: ?Object<string, *>}}
 */
// @ts-ignore typedef
let RawPaintProfilerLogItem;

var PaintProfiler = /*#__PURE__*/Object.freeze({
    __proto__: null,
    PaintProfilerModel: PaintProfilerModel,
    PaintProfilerSnapshot: PaintProfilerSnapshot,
    PaintProfilerLogItem: PaintProfilerLogItem,
    SnapshotWithRect: SnapshotWithRect,
    PictureFragment: PictureFragment,
    RawPaintProfilerLogItem: RawPaintProfilerLogItem
});

// Copyright 2017 The Chromium Authors. All rights reserved.
class PerformanceMetricsModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._agent = target.performanceAgent();
        const mode = MetricMode;
        /** @type {!Map<string, !MetricMode>} */
        this._metricModes = new Map([
            ['TaskDuration', mode.CumulativeTime], ['ScriptDuration', mode.CumulativeTime],
            ['LayoutDuration', mode.CumulativeTime], ['RecalcStyleDuration', mode.CumulativeTime],
            ['LayoutCount', mode.CumulativeCount], ['RecalcStyleCount', mode.CumulativeCount]
        ]);
        /** @type {!Map<string, !{lastValue: (number|undefined), lastTimestamp: (number|undefined)}>} */
        this._metricData = new Map();
    }
    /**
     * @return {!Promise<!Object>}
     */
    enable() {
        return this._agent.invoke_enable({});
    }
    /**
     * @return {!Promise<!Object>}
     */
    disable() {
        return this._agent.invoke_disable();
    }
    /**
     * @return {!Promise<!{metrics: !Map<string, number>, timestamp: number}>}
     */
    async requestMetrics() {
        const rawMetrics = await this._agent.invoke_getMetrics() || [];
        const metrics = new Map();
        const timestamp = performance.now();
        for (const metric of rawMetrics.metrics) {
            let data = this._metricData.get(metric.name);
            if (!data) {
                data = { lastValue: undefined, lastTimestamp: undefined };
                this._metricData.set(metric.name, data);
            }
            let value;
            switch (this._metricModes.get(metric.name)) {
                case MetricMode.CumulativeTime:
                    value = (data.lastTimestamp && data.lastValue) ?
                        NumberUtilities.clamp((metric.value - data.lastValue) * 1000 / (timestamp - data.lastTimestamp), 0, 1) :
                        0;
                    data.lastValue = metric.value;
                    data.lastTimestamp = timestamp;
                    break;
                case MetricMode.CumulativeCount:
                    value = (data.lastTimestamp && data.lastValue) ?
                        Math.max(0, (metric.value - data.lastValue) * 1000 / (timestamp - data.lastTimestamp)) :
                        0;
                    data.lastValue = metric.value;
                    data.lastTimestamp = timestamp;
                    break;
                default:
                    value = metric.value;
                    break;
            }
            metrics.set(metric.name, value);
        }
        return { metrics: metrics, timestamp: timestamp };
    }
}
/** @enum {symbol} */
const MetricMode = {
    CumulativeTime: Symbol('CumulativeTime'),
    CumulativeCount: Symbol('CumulativeCount'),
};
SDKModel.register(PerformanceMetricsModel, Capability.DOM, false);

var PerformanceMetricsModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    PerformanceMetricsModel: PerformanceMetricsModel
});

// Copyright 2017 The Chromium Authors. All rights reserved.
/**
 * @implements {ProtocolProxyApi.PageDispatcher}
 */
class ScreenCaptureModel extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._agent = target.pageAgent();
        /** @type {?function(!Protocol.binary, !Protocol.Page.ScreencastFrameMetadata):void} */
        this._onScreencastFrame = null;
        /** @type {?function(boolean):void} */
        this._onScreencastVisibilityChanged = null;
        target.registerPageDispatcher(this);
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    /**
     * @param {!Protocol.Page.StartScreencastRequestFormat} format
     * @param {number} quality
     * @param {number|undefined} maxWidth
     * @param {number|undefined} maxHeight
     * @param {number|undefined} everyNthFrame
     * @param {function(!Protocol.binary, !Protocol.Page.ScreencastFrameMetadata): void} onFrame
     * @param {function(boolean): void} onVisibilityChanged
     */
    startScreencast(format, quality, maxWidth, maxHeight, everyNthFrame, onFrame, onVisibilityChanged) {
        this._onScreencastFrame = onFrame;
        this._onScreencastVisibilityChanged = onVisibilityChanged;
        this._agent.invoke_startScreencast({ format, quality, maxWidth, maxHeight, everyNthFrame });
    }
    stopScreencast() {
        this._onScreencastFrame = null;
        this._onScreencastVisibilityChanged = null;
        this._agent.invoke_stopScreencast();
    }
    /**
     * @param {!Protocol.Page.CaptureScreenshotRequestFormat} format
     * @param {number} quality
     * @param {!Protocol.Page.Viewport=} clip
     * @return {!Promise<?Protocol.binary>}
     */
    async captureScreenshot(format, quality, clip) {
        await OverlayModel.muteHighlight();
        const result = await this._agent.invoke_captureScreenshot({ format, quality, clip, fromSurface: true });
        await OverlayModel.unmuteHighlight();
        return result.data;
    }
    /**
     * @return {!Promise<?{viewportX: number, viewportY: number, viewportScale: number, contentWidth: number, contentHeight: number}>}
     */
    async fetchLayoutMetrics() {
        const response = await this._agent.invoke_getLayoutMetrics();
        if (response.getError()) {
            return null;
        }
        return {
            viewportX: response.visualViewport.pageX,
            viewportY: response.visualViewport.pageY,
            viewportScale: response.visualViewport.scale,
            contentWidth: response.contentSize.width,
            contentHeight: response.contentSize.height
        };
    }
    /**
     * @override
     * @param {!Protocol.Page.ScreencastFrameEvent} _
     */
    screencastFrame({ data, metadata, sessionId }) {
        this._agent.invoke_screencastFrameAck({ sessionId });
        if (this._onScreencastFrame) {
            this._onScreencastFrame.call(null, data, metadata);
        }
    }
    /**
     * @override
     * @param {!Protocol.Page.ScreencastVisibilityChangedEvent} _
     */
    screencastVisibilityChanged({ visible }) {
        if (this._onScreencastVisibilityChanged) {
            this._onScreencastVisibilityChanged.call(null, visible);
        }
    }
    /**
     * @override
     * @param {!Protocol.Page.DomContentEventFiredEvent} params
     */
    domContentEventFired(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.LoadEventFiredEvent} params
     */
    loadEventFired(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.LifecycleEventEvent} params
     */
    lifecycleEvent(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.NavigatedWithinDocumentEvent} params
     */
    navigatedWithinDocument(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameAttachedEvent} params
     */
    frameAttached(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameNavigatedEvent} params
     */
    frameNavigated(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameDetachedEvent} params
     */
    frameDetached(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameStartedLoadingEvent} params
     */
    frameStartedLoading(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameStoppedLoadingEvent} params
     */
    frameStoppedLoading(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameRequestedNavigationEvent} params
     */
    frameRequestedNavigation(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameScheduledNavigationEvent} params
     */
    frameScheduledNavigation(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FrameClearedScheduledNavigationEvent} params
     */
    frameClearedScheduledNavigation(params) {
    }
    /**
     * @override
     */
    frameResized() {
    }
    /**
     * @override
     * @param {!Protocol.Page.JavascriptDialogOpeningEvent} params
     */
    javascriptDialogOpening(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.JavascriptDialogClosedEvent} params
     */
    javascriptDialogClosed(params) {
    }
    /**
     * @override
     */
    interstitialShown() {
    }
    /**
     * @override
     */
    interstitialHidden() {
    }
    /**
     * @override
     * @param {!Protocol.Page.WindowOpenEvent} params
     */
    windowOpen(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.FileChooserOpenedEvent} params
     */
    fileChooserOpened(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.CompilationCacheProducedEvent} params
     */
    compilationCacheProduced(params) {
    }
    /**
     * @override
     * @param {!Protocol.Page.DownloadWillBeginEvent} params
     */
    downloadWillBegin(params) {
    }
    /**
     * @override
     */
    downloadProgress() {
    }
}
SDKModel.register(ScreenCaptureModel, Capability.ScreenCapture, false);

var ScreenCaptureModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ScreenCaptureModel: ScreenCaptureModel
});

// Copyright 2014 The Chromium Authors. All rights reserved.
/**
 * @implements {ProtocolProxyApi.StorageDispatcher}
 */
class ServiceWorkerCacheModel extends SDKModel {
    /**
     * Invariant: This model can only be constructed on a ServiceWorker target.
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        target.registerStorageDispatcher(this);
        /** @type {!Map<string, !Cache>} */
        this._caches = new Map();
        this._cacheAgent = target.cacheStorageAgent();
        this._storageAgent = target.storageAgent();
        this._securityOriginManager = target.model(SecurityOriginManager);
        this._originsUpdated = new Set();
        this._throttler = new Throttler.Throttler(2000);
        /** @type {boolean} */
        this._enabled = false;
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
    enable() {
        if (this._enabled) {
            return;
        }
        this._securityOriginManager.addEventListener(Events$9.SecurityOriginAdded, this._securityOriginAdded, this);
        this._securityOriginManager.addEventListener(Events$9.SecurityOriginRemoved, this._securityOriginRemoved, this);
        for (const securityOrigin of this._securityOriginManager.securityOrigins()) {
            this._addOrigin(securityOrigin);
        }
        this._enabled = true;
    }
    /**
     * @param {string} origin
     */
    clearForOrigin(origin) {
        this._removeOrigin(origin);
        this._addOrigin(origin);
    }
    refreshCacheNames() {
        for (const cache of this._caches.values()) {
            this._cacheRemoved(cache);
        }
        this._caches.clear();
        const securityOrigins = this._securityOriginManager.securityOrigins();
        for (const securityOrigin of securityOrigins) {
            this._loadCacheNames(securityOrigin);
        }
    }
    /**
     * @param {!Cache} cache
     */
    async deleteCache(cache) {
        const response = await this._cacheAgent.invoke_deleteCache({ cacheId: cache.cacheId });
        if (response.getError()) {
            console.error(`ServiceWorkerCacheAgent error deleting cache ${cache.toString()}: ${response.getError()}`);
            return;
        }
        this._caches.delete(cache.cacheId);
        this._cacheRemoved(cache);
    }
    /**
     * @param {!Cache} cache
     * @param {string} request
     * @return {!Promise<void>}
     */
    async deleteCacheEntry(cache, request) {
        const response = await this._cacheAgent.invoke_deleteEntry({ cacheId: cache.cacheId, request });
        if (response.getError()) {
            Console.Console.instance().error(UIString.UIString('ServiceWorkerCacheAgent error deleting cache entry %s in cache: %s', cache.toString(), response.getError()));
            return;
        }
    }
    /**
     * @param {!Cache} cache
     * @param {number} skipCount
     * @param {number} pageSize
     * @param {string} pathFilter
     * @param {function(!Array.<!Protocol.CacheStorage.DataEntry>, number):void} callback
     */
    loadCacheData(cache, skipCount, pageSize, pathFilter, callback) {
        this._requestEntries(cache, skipCount, pageSize, pathFilter, callback);
    }
    /**
     * @param {!Cache} cache
     * @param {string} pathFilter
     * @param {function(!Array.<!Protocol.CacheStorage.DataEntry>, number):void} callback
     */
    loadAllCacheData(cache, pathFilter, callback) {
        this._requestAllEntries(cache, pathFilter, callback);
    }
    /**
     * @return {!Array.<!Cache>}
     */
    caches() {
        const caches = new Array();
        for (const cache of this._caches.values()) {
            caches.push(cache);
        }
        return caches;
    }
    /**
     * @override
     */
    dispose() {
        for (const cache of this._caches.values()) {
            this._cacheRemoved(cache);
        }
        this._caches.clear();
        if (this._enabled) {
            this._securityOriginManager.removeEventListener(Events$9.SecurityOriginAdded, this._securityOriginAdded, this);
            this._securityOriginManager.removeEventListener(Events$9.SecurityOriginRemoved, this._securityOriginRemoved, this);
        }
    }
    /**
     * @param {string} securityOrigin
     */
    _addOrigin(securityOrigin) {
        this._loadCacheNames(securityOrigin);
        if (this._isValidSecurityOrigin(securityOrigin)) {
            this._storageAgent.invoke_trackCacheStorageForOrigin({ origin: securityOrigin });
        }
    }
    /**
     * @param {string} securityOrigin
     */
    _removeOrigin(securityOrigin) {
        for (const [opaqueId, cache] of this._caches.entries()) {
            if (cache.securityOrigin === securityOrigin) {
                // TODO: Remove the closure-style casts once we are typescript-only.
                this._caches.delete(/** @type {string} */ (opaqueId));
                this._cacheRemoved(/** @type {!Cache} */ (cache));
            }
        }
        if (this._isValidSecurityOrigin(securityOrigin)) {
            this._storageAgent.invoke_untrackCacheStorageForOrigin({ origin: securityOrigin });
        }
    }
    /**
     * @param {string} securityOrigin
     * @return {boolean}
     */
    _isValidSecurityOrigin(securityOrigin) {
        const parsedURL = ParsedURL.ParsedURL.fromString(securityOrigin);
        return !!parsedURL && parsedURL.scheme.startsWith('http');
    }
    /**
     * @param {string} securityOrigin
     */
    async _loadCacheNames(securityOrigin) {
        const response = await this._cacheAgent.invoke_requestCacheNames({ securityOrigin: securityOrigin });
        if (response.getError()) {
            return;
        }
        this._updateCacheNames(securityOrigin, response.caches);
    }
    /**
     * @param {string} securityOrigin
     * @param {!Array<!Protocol.CacheStorage.Cache>} cachesJson
     */
    _updateCacheNames(securityOrigin, cachesJson) {
        /**
         * @param {!Cache} cache
         * @this {ServiceWorkerCacheModel}
         */
        function deleteAndSaveOldCaches(cache) {
            if (cache.securityOrigin === securityOrigin && !updatingCachesIds.has(cache.cacheId)) {
                oldCaches.set(cache.cacheId, cache);
                this._caches.delete(cache.cacheId);
            }
        }
        /** @type {!Set<string>} */
        const updatingCachesIds = new Set();
        /** @type {!Map<string, !Cache>} */
        const newCaches = new Map();
        /** @type {!Map<string, !Cache>} */
        const oldCaches = new Map();
        for (const cacheJson of cachesJson) {
            const cache = new Cache(this, cacheJson.securityOrigin, cacheJson.cacheName, cacheJson.cacheId);
            updatingCachesIds.add(cache.cacheId);
            if (this._caches.has(cache.cacheId)) {
                continue;
            }
            newCaches.set(cache.cacheId, cache);
            this._caches.set(cache.cacheId, cache);
        }
        this._caches.forEach(deleteAndSaveOldCaches, this);
        newCaches.forEach(this._cacheAdded, this);
        oldCaches.forEach(this._cacheRemoved, this);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _securityOriginAdded(event) {
        const securityOrigin = /** @type {string} */ (event.data);
        this._addOrigin(securityOrigin);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _securityOriginRemoved(event) {
        const securityOrigin = /** @type {string} */ (event.data);
        this._removeOrigin(securityOrigin);
    }
    /**
     * @param {!Cache} cache
     */
    _cacheAdded(cache) {
        this.dispatchEventToListeners(Events$m.CacheAdded, { model: this, cache: cache });
    }
    /**
     * @param {!Cache} cache
     */
    _cacheRemoved(cache) {
        this.dispatchEventToListeners(Events$m.CacheRemoved, { model: this, cache: cache });
    }
    /**
     * @param {!Cache} cache
     * @param {number} skipCount
     * @param {number} pageSize
     * @param {string} pathFilter
     * @param {function(!Array<!Protocol.CacheStorage.DataEntry>, number):void} callback
     */
    async _requestEntries(cache, skipCount, pageSize, pathFilter, callback) {
        const response = await this._cacheAgent.invoke_requestEntries({ cacheId: cache.cacheId, skipCount, pageSize, pathFilter });
        if (response.getError()) {
            console.error('ServiceWorkerCacheAgent error while requesting entries: ', response.getError());
            return;
        }
        callback(response.cacheDataEntries, response.returnCount);
    }
    /**
     * @param {!Cache} cache
     * @param {string} pathFilter
     * @param {function(!Array<!Protocol.CacheStorage.DataEntry>, number):void} callback
     */
    async _requestAllEntries(cache, pathFilter, callback) {
        const response = await this._cacheAgent.invoke_requestEntries({ cacheId: cache.cacheId, pathFilter });
        if (response.getError()) {
            console.error('ServiceWorkerCacheAgent error while requesting entries: ', response.getError());
            return;
        }
        callback(response.cacheDataEntries, response.returnCount);
    }
    /**
     * @param {!Protocol.Storage.CacheStorageListUpdatedEvent} event
     * @override
     */
    cacheStorageListUpdated({ origin }) {
        this._originsUpdated.add(origin);
        this._throttler.schedule(() => {
            const promises = Array.from(this._originsUpdated, origin => this._loadCacheNames(origin));
            this._originsUpdated.clear();
            return Promise.all(promises);
        });
    }
    /**
     * @param {!Protocol.Storage.CacheStorageContentUpdatedEvent} event
     * @override
     */
    cacheStorageContentUpdated({ origin, cacheName }) {
        this.dispatchEventToListeners(Events$m.CacheStorageContentUpdated, { origin, cacheName });
    }
    /**
     * @param {!Protocol.Storage.IndexedDBListUpdatedEvent} event
     * @override
     */
    indexedDBListUpdated(event) {
    }
    /**
     * @param {!Protocol.Storage.IndexedDBContentUpdatedEvent} event
     * @override
     */
    indexedDBContentUpdated(event) {
    }
}
/** @enum {symbol} */
const Events$m = {
    CacheAdded: Symbol('CacheAdded'),
    CacheRemoved: Symbol('CacheRemoved'),
    CacheStorageContentUpdated: Symbol('CacheStorageContentUpdated')
};
class Cache {
    /**
     * @param {!ServiceWorkerCacheModel} model
     * @param {string} securityOrigin
     * @param {string} cacheName
     * @param {string} cacheId
     */
    constructor(model, securityOrigin, cacheName, cacheId) {
        this._model = model;
        this.securityOrigin = securityOrigin;
        this.cacheName = cacheName;
        this.cacheId = cacheId;
    }
    /**
     * @param {!Cache} cache
     * @return {boolean}
     */
    equals(cache) {
        return this.cacheId === cache.cacheId;
    }
    /**
     * @override
     * @return {string}
     */
    toString() {
        return this.securityOrigin + this.cacheName;
    }
    /**
     * @param {string} url
     * @param {!Array.<!NameValue>} requestHeaders
     * @return {!Promise<?Protocol.CacheStorage.CachedResponse>}
     */
    async requestCachedResponse(url, requestHeaders) {
        const response = await this._model._cacheAgent.invoke_requestCachedResponse({ cacheId: this.cacheId, requestURL: url, requestHeaders });
        if (response.getError()) {
            return null;
        }
        return response.response;
    }
}
SDKModel.register(ServiceWorkerCacheModel, Capability.Storage, false);

var ServiceWorkerCacheModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ServiceWorkerCacheModel: ServiceWorkerCacheModel,
    Events: Events$m,
    Cache: Cache
});

/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class ServiceWorkerManager extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        target.registerServiceWorkerDispatcher(new ServiceWorkerDispatcher(this));
        this._lastAnonymousTargetId = 0;
        this._agent = target.serviceWorkerAgent();
        /** @type {!Map.<string, !ServiceWorkerRegistration>} */
        this._registrations = new Map();
        /** @type {boolean} */
        this._enabled = false;
        this.enable();
        /** @type {!Common.Settings.Setting<boolean>} */
        this._forceUpdateSetting = Settings.Settings.instance().createSetting('serviceWorkerUpdateOnReload', false);
        if (this._forceUpdateSetting.get()) {
            this._forceUpdateSettingChanged();
        }
        this._forceUpdateSetting.addChangeListener(this._forceUpdateSettingChanged, this);
        new ServiceWorkerContextNamer(target, this);
    }
    async enable() {
        if (this._enabled) {
            return;
        }
        this._enabled = true;
        await this._agent.invoke_enable();
    }
    async disable() {
        if (!this._enabled) {
            return;
        }
        this._enabled = false;
        this._registrations.clear();
        await this._agent.invoke_enable();
    }
    /**
     * @return {!Map.<string, !ServiceWorkerRegistration>}
     */
    registrations() {
        return this._registrations;
    }
    /**
     * @param {!Array<string>} urls
     * @return {boolean}
     */
    hasRegistrationForURLs(urls) {
        for (const registration of this._registrations.values()) {
            if (urls.filter(url => url && url.startsWith(registration.scopeURL)).length === urls.length) {
                return true;
            }
        }
        return false;
    }
    /**
     * @param {string} versionId
     * @return {?ServiceWorkerVersion}
     */
    findVersion(versionId) {
        for (const registration of this.registrations().values()) {
            const version = registration.versions.get(versionId);
            if (version) {
                return version;
            }
        }
        return null;
    }
    /**
     * @param {string} registrationId
     */
    deleteRegistration(registrationId) {
        const registration = this._registrations.get(registrationId);
        if (!registration) {
            return;
        }
        if (registration._isRedundant()) {
            this._registrations.delete(registrationId);
            this.dispatchEventToListeners(Events$n.RegistrationDeleted, registration);
            return;
        }
        registration._deleting = true;
        for (const version of registration.versions.values()) {
            this.stopWorker(version.id);
        }
        this._unregister(registration.scopeURL);
    }
    /**
     * @param {string} registrationId
     */
    async updateRegistration(registrationId) {
        const registration = this._registrations.get(registrationId);
        if (!registration) {
            return;
        }
        await this._agent.invoke_updateRegistration({ scopeURL: registration.scopeURL });
    }
    /**
     * @param {string} registrationId
     * @param {string} data
     */
    async deliverPushMessage(registrationId, data) {
        const registration = this._registrations.get(registrationId);
        if (!registration) {
            return;
        }
        const origin = ParsedURL.ParsedURL.extractOrigin(registration.scopeURL);
        await this._agent.invoke_deliverPushMessage({ origin, registrationId, data });
    }
    /**
     * @param {string} registrationId
     * @param {string} tag
     * @param {boolean} lastChance
     */
    async dispatchSyncEvent(registrationId, tag, lastChance) {
        const registration = this._registrations.get(registrationId);
        if (!registration) {
            return;
        }
        const origin = ParsedURL.ParsedURL.extractOrigin(registration.scopeURL);
        await this._agent.invoke_dispatchSyncEvent({ origin, registrationId, tag, lastChance });
    }
    /**
     * @param {string} registrationId
     * @param {string} tag
     */
    async dispatchPeriodicSyncEvent(registrationId, tag) {
        const registration = this._registrations.get(registrationId);
        if (!registration) {
            return;
        }
        const origin = ParsedURL.ParsedURL.extractOrigin(registration.scopeURL);
        await this._agent.invoke_dispatchPeriodicSyncEvent({ origin, registrationId, tag });
    }
    /**
     * @param {string} scopeURL
     */
    async _unregister(scopeURL) {
        await this._agent.invoke_unregister({ scopeURL });
    }
    /**
     * @param {string} scopeURL
     */
    async startWorker(scopeURL) {
        await this._agent.invoke_startWorker({ scopeURL });
    }
    /**
     * @param {string} scopeURL
     */
    async skipWaiting(scopeURL) {
        await this._agent.invoke_skipWaiting({ scopeURL });
    }
    /**
     * @param {string} versionId
     */
    async stopWorker(versionId) {
        await this._agent.invoke_stopWorker({ versionId });
    }
    /**
     * @param {string} versionId
     */
    async inspectWorker(versionId) {
        await this._agent.invoke_inspectWorker({ versionId });
    }
    /**
     * @param {!Array.<!Protocol.ServiceWorker.ServiceWorkerRegistration>} registrations
     */
    _workerRegistrationUpdated(registrations) {
        for (const payload of registrations) {
            let registration = this._registrations.get(payload.registrationId);
            if (!registration) {
                registration = new ServiceWorkerRegistration(payload);
                this._registrations.set(payload.registrationId, registration);
                this.dispatchEventToListeners(Events$n.RegistrationUpdated, registration);
                continue;
            }
            registration._update(payload);
            if (registration._shouldBeRemoved()) {
                this._registrations.delete(registration.id);
                this.dispatchEventToListeners(Events$n.RegistrationDeleted, registration);
            }
            else {
                this.dispatchEventToListeners(Events$n.RegistrationUpdated, registration);
            }
        }
    }
    /**
     * @param {!Array.<!Protocol.ServiceWorker.ServiceWorkerVersion>} versions
     */
    _workerVersionUpdated(versions) {
        /** @type {!Set.<!ServiceWorkerRegistration>} */
        const registrations = new Set();
        for (const payload of versions) {
            const registration = this._registrations.get(payload.registrationId);
            if (!registration) {
                continue;
            }
            registration._updateVersion(payload);
            registrations.add(registration);
        }
        for (const registration of registrations) {
            if (registration._shouldBeRemoved()) {
                this._registrations.delete(registration.id);
                this.dispatchEventToListeners(Events$n.RegistrationDeleted, registration);
            }
            else {
                this.dispatchEventToListeners(Events$n.RegistrationUpdated, registration);
            }
        }
    }
    /**
     * @param {!Protocol.ServiceWorker.ServiceWorkerErrorMessage} payload
     */
    _workerErrorReported(payload) {
        const registration = this._registrations.get(payload.registrationId);
        if (!registration) {
            return;
        }
        registration.errors.push(payload);
        this.dispatchEventToListeners(Events$n.RegistrationErrorAdded, { registration: registration, error: payload });
    }
    /**
     * @return {!Common.Settings.Setting<boolean>}
     */
    forceUpdateOnReloadSetting() {
        return this._forceUpdateSetting;
    }
    _forceUpdateSettingChanged() {
        const forceUpdateOnPageLoad = this._forceUpdateSetting.get();
        this._agent.invoke_setForceUpdateOnPageLoad({ forceUpdateOnPageLoad });
    }
}
/** @enum {symbol} */
const Events$n = {
    RegistrationUpdated: Symbol('RegistrationUpdated'),
    RegistrationErrorAdded: Symbol('RegistrationErrorAdded'),
    RegistrationDeleted: Symbol('RegistrationDeleted')
};
/**
 * @implements {ProtocolProxyApi.ServiceWorkerDispatcher}
 */
class ServiceWorkerDispatcher {
    /**
     * @param {!ServiceWorkerManager} manager
     */
    constructor(manager) {
        this._manager = manager;
    }
    /** @return {!Protocol.UsesObjectNotation} */
    usesObjectNotation() {
        return true;
    }
    /**
     * @override
     * @param {!Protocol.ServiceWorker.WorkerRegistrationUpdatedEvent} event
     */
    workerRegistrationUpdated({ registrations }) {
        this._manager._workerRegistrationUpdated(registrations);
    }
    /**
     * @override
     * @param {!Protocol.ServiceWorker.WorkerVersionUpdatedEvent} event
     */
    workerVersionUpdated({ versions }) {
        this._manager._workerVersionUpdated(versions);
    }
    /**
     * @override
     * @param {!Protocol.ServiceWorker.WorkerErrorReportedEvent} event
     */
    workerErrorReported({ errorMessage }) {
        this._manager._workerErrorReported(errorMessage);
    }
}
class ServiceWorkerVersion {
    /**
     * @param {!ServiceWorkerRegistration} registration
     * @param {!Protocol.ServiceWorker.ServiceWorkerVersion} payload
     */
    constructor(registration, payload) {
        /** @type {string} */ this.id;
        /** @type {string} */ this.scriptURL;
        /** @type {!Common.ParsedURL.ParsedURL} */ this.parsedURL;
        /** @type {string} */ this.securityOrigin;
        /** @type {!Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus} */ this.runningStatus;
        /** @type {!Protocol.ServiceWorker.ServiceWorkerVersionStatus} */ this.status;
        /** @type {number|undefined} */ this.scriptLastModified;
        /** @type {number|undefined} */ this.scriptResponseTime;
        /** @type {!Array<!Protocol.Target.TargetID>} */ this.controlledClients;
        /** @type {?Protocol.Target.TargetID} */ this.targetId;
        this.registration = registration;
        this._update(payload);
    }
    /**
     * @param {!Protocol.ServiceWorker.ServiceWorkerVersion} payload
     */
    _update(payload) {
        this.id = payload.versionId;
        this.scriptURL = payload.scriptURL;
        const parsedURL = new ParsedURL.ParsedURL(payload.scriptURL);
        this.securityOrigin = parsedURL.securityOrigin();
        this.runningStatus = payload.runningStatus;
        this.status = payload.status;
        this.scriptLastModified = payload.scriptLastModified;
        this.scriptResponseTime = payload.scriptResponseTime;
        if (payload.controlledClients) {
            this.controlledClients = payload.controlledClients.slice();
        }
        else {
            this.controlledClients = [];
        }
        this.targetId = payload.targetId || null;
    }
    /**
     * @return {boolean}
     */
    isStartable() {
        return !this.registration.isDeleted && this.isActivated() && this.isStopped();
    }
    /**
     * @return {boolean}
     */
    isStoppedAndRedundant() {
        return this.runningStatus === Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Stopped &&
            this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Redundant;
    }
    /**
     * @return {boolean}
     */
    isStopped() {
        return this.runningStatus === Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Stopped;
    }
    /**
     * @return {boolean}
     */
    isStarting() {
        return this.runningStatus === Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Starting;
    }
    /**
     * @return {boolean}
     */
    isRunning() {
        return this.runningStatus === Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Running;
    }
    /**
     * @return {boolean}
     */
    isStopping() {
        return this.runningStatus === Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Stopping;
    }
    /**
     * @return {boolean}
     */
    isNew() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.New;
    }
    /**
     * @return {boolean}
     */
    isInstalling() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Installing;
    }
    /**
     * @return {boolean}
     */
    isInstalled() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Installed;
    }
    /**
     * @return {boolean}
     */
    isActivating() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Activating;
    }
    /**
     * @return {boolean}
     */
    isActivated() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Activated;
    }
    /**
     * @return {boolean}
     */
    isRedundant() {
        return this.status === Protocol.ServiceWorker.ServiceWorkerVersionStatus.Redundant;
    }
    /**
     * @return {string}
     */
    mode() {
        if (this.isNew() || this.isInstalling()) {
            return ServiceWorkerVersion.Modes.Installing;
        }
        if (this.isInstalled()) {
            return ServiceWorkerVersion.Modes.Waiting;
        }
        if (this.isActivating() || this.isActivated()) {
            return ServiceWorkerVersion.Modes.Active;
        }
        return ServiceWorkerVersion.Modes.Redundant;
    }
}
/**
 * @type {!Object<string, string>}
 */
ServiceWorkerVersion.RunningStatus = {
    [Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Running]: ls$2 `running`,
    [Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Starting]: ls$2 `starting`,
    [Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Stopped]: ls$2 `stopped`,
    [Protocol.ServiceWorker.ServiceWorkerVersionRunningStatus.Stopping]: ls$2 `stopping`,
};
/**
 * @type {!Object<string, string>}
 */
ServiceWorkerVersion.Status = {
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.Activated]: ls$2 `activated`,
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.Activating]: ls$2 `activating`,
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.Installed]: ls$2 `installed`,
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.Installing]: ls$2 `installing`,
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.New]: ls$2 `new`,
    [Protocol.ServiceWorker.ServiceWorkerVersionStatus.Redundant]: ls$2 `redundant`,
};
/**
 * @enum {string}
 */
ServiceWorkerVersion.Modes = {
    Installing: 'installing',
    Waiting: 'waiting',
    Active: 'active',
    Redundant: 'redundant'
};
class ServiceWorkerRegistration {
    /**
     * @param {!Protocol.ServiceWorker.ServiceWorkerRegistration} payload
     */
    constructor(payload) {
        /** @type {symbol} */ this._fingerprint;
        /** @type {string} */ this.id;
        /** @type {string} */ this.scopeURL;
        /** @type {string} */ this.securityOrigin;
        /** @type {boolean} */ this.isDeleted;
        this._update(payload);
        /** @type {!Map.<string, !ServiceWorkerVersion>} */
        this.versions = new Map();
        this._deleting = false;
        /** @type {!Array<!Protocol.ServiceWorker.ServiceWorkerErrorMessage>} */
        this.errors = [];
    }
    /**
     * @param {!Protocol.ServiceWorker.ServiceWorkerRegistration} payload
     */
    _update(payload) {
        this._fingerprint = Symbol('fingerprint');
        this.id = payload.registrationId;
        this.scopeURL = payload.scopeURL;
        const parsedURL = new ParsedURL.ParsedURL(payload.scopeURL);
        this.securityOrigin = parsedURL.securityOrigin();
        this.isDeleted = payload.isDeleted;
    }
    /**
     * @return {symbol}
     */
    fingerprint() {
        return this._fingerprint;
    }
    /**
     * @return {!Map<string, !ServiceWorkerVersion>}
     */
    versionsByMode() {
        /** @type {!Map<string, !ServiceWorkerVersion>} */
        const result = new Map();
        for (const version of this.versions.values()) {
            result.set(version.mode(), version);
        }
        return result;
    }
    /**
     * @param {!Protocol.ServiceWorker.ServiceWorkerVersion} payload
     * @return {!ServiceWorkerVersion}
     */
    _updateVersion(payload) {
        this._fingerprint = Symbol('fingerprint');
        let version = this.versions.get(payload.versionId);
        if (!version) {
            version = new ServiceWorkerVersion(this, payload);
            this.versions.set(payload.versionId, version);
            return version;
        }
        version._update(payload);
        return version;
    }
    /**
     * @return {boolean}
     */
    _isRedundant() {
        for (const version of this.versions.values()) {
            if (!version.isStoppedAndRedundant()) {
                return false;
            }
        }
        return true;
    }
    /**
     * @return {boolean}
     */
    _shouldBeRemoved() {
        return this._isRedundant() && (!this.errors.length || this._deleting);
    }
    /**
     * @return {boolean}
     */
    canBeRemoved() {
        return this.isDeleted || this._deleting;
    }
    clearErrors() {
        this._fingerprint = Symbol('fingerprint');
        this.errors = [];
    }
}
class ServiceWorkerContextNamer {
    /**
     * @param {!Target} target
     * @param {!ServiceWorkerManager} serviceWorkerManager
     */
    constructor(target, serviceWorkerManager) {
        this._target = target;
        this._serviceWorkerManager = serviceWorkerManager;
        /** @type {!Map<string, !ServiceWorkerVersion>} */
        this._versionByTargetId = new Map();
        serviceWorkerManager.addEventListener(Events$n.RegistrationUpdated, this._registrationsUpdated, this);
        serviceWorkerManager.addEventListener(Events$n.RegistrationDeleted, this._registrationsUpdated, this);
        TargetManager.instance().addModelListener(RuntimeModel, Events$d.ExecutionContextCreated, this._executionContextCreated, this);
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _registrationsUpdated(event) {
        this._versionByTargetId.clear();
        const registrations = this._serviceWorkerManager.registrations().values();
        for (const registration of registrations) {
            for (const version of registration.versions.values()) {
                if (version.targetId) {
                    this._versionByTargetId.set(version.targetId, version);
                }
            }
        }
        this._updateAllContextLabels();
    }
    /**
     * @param {!Common.EventTarget.EventTargetEvent} event
     */
    _executionContextCreated(event) {
        const executionContext = /** @type {!ExecutionContext} */ (event.data);
        const serviceWorkerTargetId = this._serviceWorkerTargetId(executionContext.target());
        if (!serviceWorkerTargetId) {
            return;
        }
        this._updateContextLabel(executionContext, this._versionByTargetId.get(serviceWorkerTargetId) || null);
    }
    /**
     * @param {!Target} target
     * @return {?string}
     */
    _serviceWorkerTargetId(target) {
        if (target.parentTarget() !== this._target || target.type() !== Type.ServiceWorker) {
            return null;
        }
        return target.id();
    }
    _updateAllContextLabels() {
        for (const target of TargetManager.instance().targets()) {
            const serviceWorkerTargetId = this._serviceWorkerTargetId(target);
            if (!serviceWorkerTargetId) {
                continue;
            }
            const version = this._versionByTargetId.get(serviceWorkerTargetId) || null;
            const runtimeModel = target.model(RuntimeModel);
            const executionContexts = runtimeModel ? runtimeModel.executionContexts() : [];
            for (const context of executionContexts) {
                this._updateContextLabel(context, version);
            }
        }
    }
    /**
     * @param {!ExecutionContext} context
     * @param {?ServiceWorkerVersion} version
     */
    _updateContextLabel(context, version) {
        if (!version) {
            context.setLabel('');
            return;
        }
        const parsedUrl = ParsedURL.ParsedURL.fromString(context.origin);
        const label = parsedUrl ? parsedUrl.lastPathComponentWithFragment() : context.name;
        const localizedStatus = ServiceWorkerVersion.Status[version.status];
        context.setLabel(ls$2 `${label} #${version.id} (${localizedStatus})`);
    }
}
SDKModel.register(ServiceWorkerManager, Capability.ServiceWorker, true);

var ServiceWorkerManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    ServiceWorkerManager: ServiceWorkerManager,
    Events: Events$n,
    ServiceWorkerVersion: ServiceWorkerVersion,
    ServiceWorkerRegistration: ServiceWorkerRegistration
});

// Copyright 2014 The Chromium Authors. All rights reserved.
/**
 * @unrestricted
 */
class TracingManager extends SDKModel {
    /**
     * @param {!Target} target
     */
    constructor(target) {
        super(target);
        this._tracingAgent = target.tracingAgent();
        target.registerTracingDispatcher(new TracingDispatcher(this));
        /** @type {?TracingManagerClient} */
        this._activeClient = null;
        /** @type {?number} */
        this._eventBufferSize = 0;
        this._eventsRetrieved = 0;
    }
    /**
     * @param {number=} usage
     * @param {number=} eventCount
     * @param {number=} percentFull
     */
    _bufferUsage(usage, eventCount, percentFull) {
        this._eventBufferSize = eventCount === undefined ? null : eventCount;
        if (this._activeClient) {
            this._activeClient.tracingBufferUsage(usage || percentFull || 0);
        }
    }
    /**
     * @param {!Array.<!EventPayload>} events
     */
    _eventsCollected(events) {
        if (!this._activeClient) {
            return;
        }
        this._activeClient.traceEventsCollected(events);
        this._eventsRetrieved += events.length;
        if (!this._eventBufferSize) {
            this._activeClient.eventsRetrievalProgress(0);
            return;
        }
        if (this._eventsRetrieved > this._eventBufferSize) {
            this._eventsRetrieved = this._eventBufferSize;
        }
        this._activeClient.eventsRetrievalProgress(this._eventsRetrieved / this._eventBufferSize);
    }
    _tracingComplete() {
        this._eventBufferSize = 0;
        this._eventsRetrieved = 0;
        if (this._activeClient) {
            this._activeClient.tracingComplete();
            this._activeClient = null;
        }
        this._finishing = false;
    }
    // TODO(petermarshall): Use the traceConfig argument instead of deprecated
    // categories + options.
    /**
     * @param {!TracingManagerClient} client
     * @param {string} categoryFilter
     * @param {string} options
     * @return {!Promise<!Object>}
     */
    async start(client, categoryFilter, options) {
        if (this._activeClient) {
            throw new Error('Tracing is already started');
        }
        const bufferUsageReportingIntervalMs = 500;
        this._activeClient = client;
        const args = {
            bufferUsageReportingInterval: bufferUsageReportingIntervalMs,
            categories: categoryFilter,
            options: options,
            transferMode: Protocol.Tracing.StartRequestTransferMode.ReportEvents,
        };
        const response = await this._tracingAgent.invoke_start(args);
        if (response.getError()) {
            this._activeClient = null;
        }
        return response;
    }
    stop() {
        if (!this._activeClient) {
            throw new Error('Tracing is not started');
        }
        if (this._finishing) {
            throw new Error('Tracing is already being stopped');
        }
        this._finishing = true;
        this._tracingAgent.invoke_end();
    }
}
/**
 * @interface
 */
class TracingManagerClient {
    /**
     * @param {!Array.<!EventPayload>} events
     */
    traceEventsCollected(events) {
    }
    tracingComplete() {
    }
    /**
     * @param {number} usage
     */
    tracingBufferUsage(usage) {
    }
    /**
     * @param {number} progress
     */
    eventsRetrievalProgress(progress) {
    }
}
/**
 * @implements {ProtocolProxyApi.TracingDispatcher}
 * @unrestricted
 */
class TracingDispatcher {
    /**
     * @param {!TracingManager} tracingManager
     */
    constructor(tracingManager) {
        this._tracingManager = tracingManager;
    }
    /**
     * @override
     * @param {!Protocol.Tracing.BufferUsageEvent} event
     */
    bufferUsage({ value, eventCount, percentFull }) {
        this._tracingManager._bufferUsage(value, eventCount, percentFull);
    }
    /**
     * @override
     * @param {!Protocol.Tracing.DataCollectedEvent} event
     */
    dataCollected({ value }) {
        this._tracingManager._eventsCollected(value);
    }
    /**
     * @override
     */
    tracingComplete() {
        this._tracingManager._tracingComplete();
    }
    /**
     * @return {!Protocol.UsesObjectNotation}
     */
    usesObjectNotation() {
        return true;
    }
}
SDKModel.register(TracingManager, Capability.Tracing, false);
/** @typedef {!{
        cat: (string|undefined),
        pid: number,
        tid: number,
        ts: number,
        ph: string,
        name: string,
        args: !{
          sort_index: number,
          name: string,
          snapshot: ObjectSnapshot,
          data: ?Object
        },
        dur: number,
        id: string,
        id2: (!{global: (string|undefined), local: (string|undefined)}|undefined),
        scope: string,
        bind_id: string,
        s: string
    }}
 */
// @ts-ignore typedef
let EventPayload;

var TracingManager$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    TracingManager: TracingManager,
    TracingManagerClient: TracingManagerClient,
    EventPayload: EventPayload
});

// Copyright 2020 The Chromium Authors. All rights reserved.
class WebAuthnModel extends SDKModel {
    /**
    * @param {!Target} target
    */
    constructor(target) {
        super(target);
        this._agent = target.webAuthnAgent();
    }
    /**
     * @param {boolean} enable
     * @return {!Promise<!Object>}
     */
    setVirtualAuthEnvEnabled(enable) {
        if (enable) {
            return this._agent.invoke_enable();
        }
        return this._agent.invoke_disable();
    }
    /**
     * @param {!Protocol.WebAuthn.VirtualAuthenticatorOptions} options
     * @return {!Promise<!Protocol.WebAuthn.AuthenticatorId>}
     */
    async addAuthenticator(options) {
        const response = await this._agent.invoke_addVirtualAuthenticator({ options });
        return response.authenticatorId;
    }
    /**
     * @param {!Protocol.WebAuthn.AuthenticatorId} authenticatorId
     */
    async removeAuthenticator(authenticatorId) {
        await this._agent.invoke_removeVirtualAuthenticator({ authenticatorId });
    }
    /**
     * @param {!Protocol.WebAuthn.AuthenticatorId} authenticatorId
     * @param {boolean} enabled
     */
    async setAutomaticPresenceSimulation(authenticatorId, enabled) {
        await this._agent.invoke_setAutomaticPresenceSimulation({ authenticatorId, enabled });
    }
    /**
     * @param {!Protocol.WebAuthn.AuthenticatorId} authenticatorId
     * @return {!Promise<!Array<!Protocol.WebAuthn.Credential>>}
     */
    async getCredentials(authenticatorId) {
        const response = await this._agent.invoke_getCredentials({ authenticatorId });
        return response.credentials;
    }
    /**
     * @param {!Protocol.WebAuthn.AuthenticatorId} authenticatorId
     * @param {string} credentialId
     */
    async removeCredential(authenticatorId, credentialId) {
        await this._agent.invoke_removeCredential({ authenticatorId, credentialId });
    }
}
SDKModel.register(WebAuthnModel, Capability.WebAuthn, false);

var WebAuthnModel$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    WebAuthnModel: WebAuthnModel
});

// Copyright 2019 The Chromium Authors. All rights reserved.
// We need to force creation of the FrameManager early to make sure no issues are missed.
FrameManager.instance();
// We need to force creation of the NetworkLog early to make sure no requests are missed.
NetworkLog.instance();

export { CPUProfileDataModel$1 as CPUProfileDataModel, CPUProfilerModel$1 as CPUProfilerModel, CSSFontFace$1 as CSSFontFace, CSSMatchedStyles$1 as CSSMatchedStyles, CSSMedia$1 as CSSMedia, CSSMetadata$1 as CSSMetadata, CSSModel$1 as CSSModel, CSSProperty$1 as CSSProperty, CSSPropertyParser, CSSRule$1 as CSSRule, CSSStyleDeclaration$1 as CSSStyleDeclaration, CSSStyleSheetHeader$1 as CSSStyleSheetHeader, ChildTargetManager$1 as ChildTargetManager, CompilerSourceMappingContentProvider$1 as CompilerSourceMappingContentProvider, Connections, ConsoleModel$1 as ConsoleModel, ContentSecurityPolicyIssue$1 as ContentSecurityPolicyIssue, Cookie$1 as Cookie, CookieModel$1 as CookieModel, CookieParser$1 as CookieParser, DOMDebuggerModel$1 as DOMDebuggerModel, DOMModel$1 as DOMModel, DebuggerModel$1 as DebuggerModel, EmulationModel$1 as EmulationModel, FilmStripModel$1 as FilmStripModel, FrameManager$1 as FrameManager, HARLog$1 as HARLog, HeapProfilerModel$1 as HeapProfilerModel, HeavyAdIssue$1 as HeavyAdIssue, IOModel$1 as IOModel, IsolateManager$1 as IsolateManager, Issue$1 as Issue, IssuesModel$1 as IssuesModel, LayerTreeBase$1 as LayerTreeBase, LogModel$1 as LogModel, MixedContentIssue$1 as MixedContentIssue, NetworkLog$1 as NetworkLog, NetworkManager$1 as NetworkManager, NetworkRequest$1 as NetworkRequest, OverlayModel$1 as OverlayModel, PageResourceLoader$1 as PageResourceLoader, PaintProfiler, PerformanceMetricsModel$1 as PerformanceMetricsModel, ProfileTreeModel$1 as ProfileTreeModel, RemoteObject$1 as RemoteObject, Resource$1 as Resource, ResourceTreeModel$1 as ResourceTreeModel, RuntimeModel$1 as RuntimeModel, SDKModel$1 as SDKModel, SameSiteCookieIssue$1 as SameSiteCookieIssue, ScreenCaptureModel$1 as ScreenCaptureModel, Script$1 as Script, SecurityOriginManager$1 as SecurityOriginManager, ServerTiming$1 as ServerTiming, ServiceWorkerCacheModel$1 as ServiceWorkerCacheModel, ServiceWorkerManager$1 as ServiceWorkerManager, SourceMap$1 as SourceMap, SourceMapManager$1 as SourceMapManager, TracingManager$1 as TracingManager, TracingModel$1 as TracingModel, WebAuthnModel$1 as WebAuthnModel };
